/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import java.text.DecimalFormat;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IActionDelegate2;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Action zur Anzeige des Tooltip fr Anzeigen der Spalte Trichter des Sonderprogrammdialoges
 * 
 * @author Pittner
 * 
 */
public class ActionAnzeigeKontextTrichterToolTip implements IWorkbenchWindowActionDelegate, IActionDelegate2
{

  private IWorkbenchPart _workbenchPart;

  ClientDavInterface _connection = DaVVerbindung.getInstanz().getDaVVerbindung();

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    _workbenchPart = window.getActivePage().getActivePart();
  }

  @Override
  public void run(IAction action)
  {
  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  /**
   * @return
   */
  protected IWorkbenchPart getWorkbenchPart()
  {
    return _workbenchPart;
  }

  /**
   * Gibt den {@link GraphicalViewer} des Workbench Parts zurck.
   * 
   * @return der Viewer oder <code>null</code>, wenn es keinen gibt.
   */
  protected GraphicalViewer getViewer()
  {
    return (GraphicalViewer)getWorkbenchPart().getAdapter(GraphicalViewer.class);
  }

  /**
   * 
   * @param workbenchPart
   */
  protected void setWorkbenchPart(IWorkbenchPart workbenchPart)
  {
    _workbenchPart = workbenchPart;
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = getViewer().getEditDomain().getCommandStack();
    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  @Override
  public void init(IAction action)
  {
    // TODO Auto-generated method stub

  }

  @Override
  public void runWithEvent(IAction action, Event event)
  {
    GraphicalViewer viewer = getViewer();
    Object obj = event.data;
    if (viewer != null && obj != null)
    {
      AnzeigeModelTypAbstract doModel = null;
      if (obj instanceof AnzeigeModelTypAbstract)
        ;
      {
        doModel = (AnzeigeModelTypAbstract)obj;

        // if (!(doModel instanceof AnzeigeModelTypD || doModel instanceof AnzeigeModelTypD3))
        {
          SystemObject so = doModel.getSystemObject();

          if (so != null)
          {
            if (_connection != null)
            {
              DataModel dataModel = _connection.getDataModel();
//              System.out.println("ActionAnzeigeKontextTrichterToolTip.runWithEvent(): " + so.getName());

              IPreferenceStore store = Activator.getDefault().getPreferenceStore();
              String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
              if (pidSba != null && pidSba.length() > 0)
              {
                SystemObject object = dataModel.getObject(pidSba);
                if (object != null)
                {
                  AttributeGroup atg = dataModel.getAttributeGroup("atg.sswSbaBetriebsKilometer");

                  AnzeigeQuerschnittModelTypAbstract aq = DaVHelper.holeAnzeigeQuerschnittVonAnzeige(_connection,
                      viewer, doModel);

                  SystemObject soSbaAq = DaVHelper.holeSystemObjectSswSbaAqVonAq(_connection, aq.getSystemObject());

                  String km = "";                   
                  if (soSbaAq != null)
                  {
                    Data cd = soSbaAq.getConfigurationData(atg);

//                    System.out.println("cd: " + cd);

                    Double d = Double.MIN_VALUE;

                    String s0 = "";                   
                    if (cd != null)
                    {
                      km = cd.getItem("Wert").asTextValue().getValueText();
                      if (km != null && km.length() > 0)
                      {
                        d = new Double(km);
                        if (d > 0)
                        {
                          d /= 1000;
                          DecimalFormat f = new DecimalFormat("#0.00"); 
                          s0 = f.format(d); 
                        }
                      }
                    }
                    
                    

                    String prio = doModel.getPrioritaet();

                    boolean b1 = doModel.getNotwendig() > 0 ? true : false;
                    boolean b2 = doModel.getAbsolutNotwendig() > 0 ? true : false;
                    String s2 = doModel.getBlinken() > 0 ? "Ja" : "Nein"; 
                    String s3 = doModel.getBlinktakt();

                    String s1 = "";
                    if (b1 && b2)
                    {
                      s1 = "absolut notwendig";
                    }
                    else if (b1 && !b2)
                    {
                      s1 = "notwendig";
                    }
                    else
                    {
                      s1 = "nicht notwendig";
                    }
                    
                    String s = "Anzeige: " + doModel.getSystemObject().getName();
                    s += "\nPrioritt: " + prio;
                    s += "\nKilometer: " + s0;
                    s += "\nNotwendig: " + s1;
                    s += "\nBlinken: " + s2;
                    s += "\nBlinktakt: " + s3;

                    EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE;
                    final SetCommand cmd = new SetCommand(doModel, feature, s);
                    getCommandStack().execute(cmd);
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}
