/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.AnzeigeTypC2EditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.AnzeigeTypD3EditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.AnzeigeTypDEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.DoMitVerbLinieEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeInhaltAuswahlDialog;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeInhaltFreiTextDialog;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.DynamischerWechselText.EnumDynamischerWechseltextTextAusrichtung;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Action zur Anzeige der Auswahl fr Anzeigen der Sonderprogramme. 
 * Es sollen nur Daten im Kontext Trichter verndert werden.
 * @author Pittner
 *
 */
public class ActionKontextTrichterAnzeigeABC2 implements IWorkbenchWindowActionDelegate
{
  private IWorkbenchPart _workbenchPart;
  private ClientDavInterface _connection;

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    _workbenchPart = window.getActivePage().getActivePart();
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
  }

  @Override
  public void run(IAction action)
  {

    GraphicalViewer viewer = getViewer();
    if (viewer != null)
    {
      ISelection selection = viewer.getSelection();

      StructuredSelection ss = (StructuredSelection)selection;

      List<DoMitVerbLinieEditPart> list = new ArrayList<DoMitVerbLinieEditPart>();

      Iterator it = ss.iterator();
      while (it.hasNext())
      {
        Object obj = it.next();

        DoMitVerbLinieEditPart domep = null;
        AnzeigeModelTypAbstract doModel = null;
        AnzeigeElementFigure figure = null;

        if (obj instanceof DoMitVerbLinieEditPart)
        {
          domep = (DoMitVerbLinieEditPart)obj;
          DoModelMitVerbLinie model = domep.getModel();
          if (model instanceof AnzeigeModelTypAbstract)
          {
            doModel = (AnzeigeModelTypAbstract)model;
            list.add(domep);
          }
        }

        boolean unterschiedlich = false;

        List<DoMitVerbLinieEditPart> copyList = new ArrayList<DoMitVerbLinieEditPart>(list);

        for (DoMitVerbLinieEditPart epart : list)
        {
          for (DoMitVerbLinieEditPart copy : copyList)
          {
            if (!epart.getClass().equals(copy.getClass()))
            {
              unterschiedlich = true;
              break;
            }
          }
          if (unterschiedlich)
            break;
        }

        if (unterschiedlich)
        {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_ERROR | SWT.OK);
          mb.setText("Fehler Selektion");
          mb.setMessage("Es knnnen nur Inhalte eines Anzeigetyps gesetzt werden!");
          mb.open();
          return;
        }

      }

      if (list.size() > 0)
      {
        for (DoMitVerbLinieEditPart domep : list)
        {
//          DoMitVerbLinieEditPart domep = list.get(0);
          AnzeigeModelTypAbstract doModel = (AnzeigeModelTypAbstract)domep.getModel();
          AnzeigeElementFigure figure = (AnzeigeElementFigure)domep.getFigure();
          
          SystemObject[] fonts = DaVHelper.holeFontsUeberAnzeige(_connection, doModel.getSystemObject());
          
          if (!(domep instanceof AnzeigeTypDEditPart || domep instanceof AnzeigeTypD3EditPart))
          {
            AnzeigeInhaltAuswahlDialog anzeigeInhaltAuswahlDialog = new AnzeigeInhaltAuswahlDialog(PlatformUI
                .getWorkbench().getDisplay().getActiveShell(), true);
            if (doModel != null && figure != null)
            {
              EList<String> inhalte1 = (EList<String>)doModel.getInhalte1();
              if (inhalte1 != null)
              {
                inhalte1.add(AnzeigeInhaltTyp.BELIEBIG_BI.getType());
                
                if (domep instanceof AnzeigeTypC2EditPart)
                  inhalte1.add(AnzeigeInhaltTyp.FREITEXT_BI.getType());
                
                
                Object value = doModel.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
                AnzeigeInhaltTyp inhalt = anzeigeInhaltAuswahlDialog.oeffne(figure.getTyp(), inhalte1);
                
//              System.out.println(value + " : " + inhalt);
                
                if (value instanceof AnzeigeInhaltTyp)
                {
                  AnzeigeInhaltTyp inhaltAlt = (AnzeigeInhaltTyp)value;
                  if (!inhaltAlt.equals(inhalt)){
                    doModel.setAbsolutNotwendig(0);
                    doModel.setNotwendig(0);
                    doModel.setBlinken(0);
                    doModel.setBlinktakt("Undefiniert");
                    doModel.setPrioritaet("undefiniert");
                  }
                }
                
                if (inhalt != null)
                {
                  
                  if (!inhalt.equals(AnzeigeInhaltTyp.FREITEXT_BI))
                  {
                    
                    EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                    for (DoMitVerbLinieEditPart epart : list)
                    {
                      DoModelMitVerbLinie model = epart.getModel();
                      final SetCommand cmd = new SetCommand(model, feature, inhalt);
                      
//                    System.out.println("setze1: " + model.getSystemObject() + " -> " + inhalt.getType());
                      
                      getCommandStack().execute(cmd);
                    }
                    
                  }
                  else
                  {
                    AnzeigeInhaltFreiTextDialog freiTextAuswahlDialog = new AnzeigeInhaltFreiTextDialog(PlatformUI
                        .getWorkbench().getDisplay().getActiveShell());
                    
                    if (doModel != null && figure != null)
                    {
                      
//                    Object value = doModel
//                        .eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
                      
                      
                      System.out.println("Value: " + value);
                      
                      VisuAnzeigeKapsel kapsel = null;
                      if (value instanceof VisuAnzeigeKapsel)
                      {
                        kapsel = (VisuAnzeigeKapsel)value;
                        kapsel.setAbstand(0);
                        kapsel.setAusrichtung(EnumDynamischerWechseltextTextAusrichtung.Zentriert.toString());
                        kapsel.setSchriftart(fonts[0].getPid());
                      }
//                    else if (value instanceof ResultData)
//                    {
//                      ResultData result = (ResultData)value;
//                      kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(result);
//                      
//                    }
                      else
                      {
                        kapsel = new VisuAnzeigeKapsel();
                        kapsel.setAbstand(0);
                        kapsel.setAusrichtung(EnumDynamischerWechseltextTextAusrichtung.Zentriert.toString());
                        kapsel.setSchriftart(fonts[0].getPid());
                        kapsel.setPidsZeichen(new String[0]);
                      }
                      
                      kapsel = freiTextAuswahlDialog.oeffne(kapsel, doModel.getSystemObject());
                      if (kapsel != null)
                      {
                        EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                        
                        for (DoMitVerbLinieEditPart epart : list)
                        {
                          DoModelMitVerbLinie model = epart.getModel();
                          SetCommand cmd = new SetCommand(model, feature, kapsel);
                          if (kapsel.isInaktiv())
                          {
                            cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                          }
                          
//                        System.out.println("setze2: " + model.getSystemObject() + " -> " + inhalt.getType());
                          
                          getCommandStack().execute(cmd);
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          else
          {
            AnzeigeInhaltFreiTextDialog anzeigeInhaltAuswahlDialog = new AnzeigeInhaltFreiTextDialog(PlatformUI
                .getWorkbench().getDisplay().getActiveShell());
            
            if (doModel != null && figure != null)
            {
              Object value = doModel.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
              VisuAnzeigeKapsel kapsel = null;
              if (value instanceof VisuAnzeigeKapsel)
              {
                kapsel = (VisuAnzeigeKapsel)value;
              }
//            else if (value instanceof ResultData)
//            {
//              ResultData result = (ResultData)value;
//              kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(result);
//              
//            }
              else
              {
                kapsel = new VisuAnzeigeKapsel(new String[] {}, fonts[0].getPid(),
                    EnumDynamischerWechseltextTextAusrichtung.Linksbndig.toString(), 4);
              }
              VisuAnzeigeKapsel kapselNeu = anzeigeInhaltAuswahlDialog.oeffne(kapsel, doModel.getSystemObject());
              if (kapselNeu != null)
              {
                EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                
                for (DoMitVerbLinieEditPart epart : list)
                {
                  DoModelMitVerbLinie model = epart.getModel();
                  SetCommand cmd = new SetCommand(model, feature, kapselNeu);
                  if (kapsel.isInaktiv())
                  {
                    cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                  }
                  
//                System.out.println("setze3: " + model.getSystemObject() + " -> " + kapselNeu.getPidsZeichen().length);
                  
                  getCommandStack().execute(cmd);
                }
              }
            }
          }
          
        }
      }

    }
    else
    {
      System.out.println("Viewer ist null");
    }
  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  /**
   * @return
   */
  protected IWorkbenchPart getWorkbenchPart()
  {
    return _workbenchPart;
  }

  /**
   * Gibt den {@link GraphicalViewer} des Workbench Parts zurck.
   * 
   * @return der Viewer oder <code>null</code>, wenn es keinen gibt.
   */
  protected GraphicalViewer getViewer()
  {
    return (GraphicalViewer)getWorkbenchPart().getAdapter(GraphicalViewer.class);
  }

  /**
   * 
   * @param workbenchPart
   */
  protected void setWorkbenchPart(IWorkbenchPart workbenchPart)
  {
    _workbenchPart = workbenchPart;
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {

    CommandStack stack = getViewer().getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

}
