/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.Perspective;
import org.eclipse.ui.internal.WorkbenchPage;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.WanderbaustelleViewParameter;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Action zur Anzeige des Sonderprogramm-Fensters FR No {@link ZustandsDarstellungSonderProgramm}
 * 
 * @author Pittner
 * 
 */
public class ActionParameterWanderBaustelleFrNO implements IWorkbenchWindowActionDelegate
{
  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.wanderBaustelleViewParameterNo";

  private IWorkbenchWindow _workbenchWindow;

  private ClientDavInterface _connection;

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    _workbenchWindow = window;
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

  }

  protected LinkedList<SystemObject> bestimmeObjekteFuerFahrtrichtung(String s)
  {
    LinkedList<SystemObject> re = new LinkedList<SystemObject>();
    if (_connection != null)
    {
      SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaStreckenBeeinflussungsAnlageRichtung");
      if (type != null)
      {
        List<SystemObject> elements = type.getElements();
        for (SystemObject so : elements)
        {
          AttributeGroup atg = so.getDataModel().getAttributeGroup("atg.sswSbaStreckenBeeinflussungsAnlageRichtung");
          Data cd = so.getConfigurationData(atg);
          String fr = cd.getItem("Fahrtrichtung").asTextValue().getValueText();
          if (fr.equals(s))
          {
            re.add(so);
          }
        }
      }
    }
    return re;
  }

  @Override
  public void run(IAction action)
  {

    LinkedList<SystemObject> list = bestimmeObjekteFuerFahrtrichtung("NO");
    for (SystemObject systemObject : list)
    {
      systemObject.getPid();
      oeffneViewMitSecondId(WanderbaustelleViewParameter.ID, systemObject.getPid());
      break;
    }

    // LinkedList<ParameterEditorKapsel> listKapsel = new LinkedList<ParameterEditorKapsel>();
    // String pidAtg = "atg.sswSbaWanderbaustelle";
    // for (SystemObject so : list)
    // {
    // listKapsel.add(new ParameterEditorKapsel(so.getPid(), pidAtg, (short)0));
    // }
    // OpenViewParameterEditor.openView(listKapsel, true, true);

  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  protected void oeffneViewMitSecondId(String viewID, String secondId)
  {
    // IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = _workbenchWindow.getActivePage();

    try
    {
      // System.out.println(viewID + " : " + secondId);

      page.showView(viewID, secondId, IWorkbenchPage.VIEW_CREATE);

      IViewReference ref = page.findViewReference(viewID, secondId);
      Perspective ap = ((WorkbenchPage)page).getActivePerspective();
      if (ap != null)
      {
        if (ref != null)
        {
          ap.getPresentation().detachPart(ref);

          IWorkbenchPart wbPart = ref.getPart(false);
          Shell shell = wbPart.getSite().getShell();
          shell.setSize(600, 700);
          IViewPart vPart = ref.getView(true);

          if (_workbenchWindow != null)
          {
            /*
             * Positionierung des Detached-Windows zentriert auf der Shell des Workbench-Window.
             */
            Point sizeWB = _workbenchWindow.getShell().getSize();
            Point sizeSH = shell.getSize();

            int x = ((int)(((double)sizeWB.x - (double)sizeSH.x) / 2.0d)) + _workbenchWindow.getShell().getLocation().x;
            int y = ((int)(((double)sizeWB.y - (double)sizeSH.y) / 2.0d)) + _workbenchWindow.getShell().getLocation().y;

            shell.setLocation(x, y);
          }
          else
          {
            IllegalArgumentException iae = new IllegalArgumentException(
                "Zentrieren des Fensters nicht mglich, der Action muss ein WorkbenchWindow bergeben werden!");
            Activator.getDefault().getLog()
                .log(
                    new Status(Status.WARNING, Activator.PLUGIN_ID,
                        "Zentrieren des Fensters nicht mglich, der Action muss ein WorkbenchWindow bergeben werden!",
                        iae));
          }
        }
      }
      else
      {
        System.out.println("Perspective ist null!");
      }
    }
    catch (PartInitException e)
    {
      e.printStackTrace();
    }
  }

}
