/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import org.eclipse.gef.GraphicalViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchPage;

import de.bsvrz.buv.plugin.anlagendarstellung.TabelleMqModel;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.TabelleMqA14EditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.TabelleMqB27EditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.views.TabelleMqView;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Action zur Anzeige des Fensters fr die MQ-Tabelle {@link TabelleMqView}
 * 
 * @author Pittner
 *
 */
public class ActionTabelleMqDoppelKlick implements IWorkbenchWindowActionDelegate
{
  private IWorkbenchPart _workbenchPart;

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    _workbenchPart = window.getActivePage().getActivePart();
  }

  @Override
  public void run(IAction action)
  {
    GraphicalViewer viewer = getViewer();
    if (viewer != null)
    {
      ISelection selection = viewer.getSelection();
      StructuredSelection ss = (StructuredSelection)selection;
      Object obj = ss.getFirstElement();

      TabelleMqModel doModel = null;

      if (obj instanceof TabelleMqA14EditPart)
      {
        TabelleMqA14EditPart domep = (TabelleMqA14EditPart)obj;
        doModel = (TabelleMqModel)domep.getModel();
      }
      else if (obj instanceof TabelleMqB27EditPart)
      {
        TabelleMqB27EditPart domep = (TabelleMqB27EditPart)obj;
        doModel = (TabelleMqModel)domep.getModel();
      }

      if (doModel != null)
      {
        SystemObject so = doModel.getSystemObject();
        if (so != null)
        {
          IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
          IWorkbenchPage page = dw.getActivePage();
          try
          {
            starteViewWithSecondaryId(page, so.getPid());

            IViewReference ref = page.findViewReference(TabelleMqView.ID, so.getPid());
            ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
            IWorkbenchPart viewPart = ref.getPart(false);

            Shell shell = viewPart.getSite().getShell();

            Display display = PlatformUI.getWorkbench().getDisplay();
            Point p = display.getCursorLocation();

            shell.setLocation(p.x - 10, p.y - 10);
            shell.setSize(520, 315);
          }
          catch (PartInitException e)
          {
            // TODO Auto-generated catch block
            e.printStackTrace();
          }
        }
      }
    }
  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  /**
   * @return
   */
  protected IWorkbenchPart getWorkbenchPart()
  {
    return _workbenchPart;
  }

  /**
   * Gibt den {@link GraphicalViewer} des Workbench Parts zurck.
   * 
   * @return der Viewer oder <code>null</code>, wenn es keinen gibt.
   */
  protected GraphicalViewer getViewer()
  {
    return (GraphicalViewer)getWorkbenchPart().getAdapter(GraphicalViewer.class);
  }

  /**
   * 
   * @param workbenchPart
   */
  protected void setWorkbenchPart(IWorkbenchPart workbenchPart)
  {
    _workbenchPart = workbenchPart;
  }

  private void starteViewWithSecondaryId(IWorkbenchPage page, String secondaryId) throws PartInitException
  {
    page.showView(TabelleMqView.ID, secondaryId, IWorkbenchPage.VIEW_ACTIVATE);
  }

}
