/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AnzeigeModelTypAbstract} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeTypBEditPart extends AnzeigeTypAEditPart
{
  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  private final Dimension DIM = new Dimension(41, 33);

  @Override
  protected AnzeigeElementFigure createFigure()
  {

    ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = dav.getDataModel();

    AnzeigeModelTypAbstract model = getCastedModel();

    setzteInhalteModell(model, dataModel);

    boolean isFrNord = getCastedModel().getFahrtrichtung() != 1 ? true : false;

    AnzeigeElementFigure aef = new AnzeigeElementFigure("AnzeigeB", AnzeigeElementTyp.WZG_B, AnzeigeInhaltTyp.DUNKEL,
        true, isFrNord);
    aef.setOpaque(true);
    aef.setSize(DIM.width, DIM.height);
    aef.setBorder(new LineBorder(ColorConstants.orange, 2));

    addToolTipListener(aef);

    return aef;

  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);
    
    if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification.getFeature()))
    {
      Object newValue = notification.getNewValue();
      Object oldValue = notification.getOldValue();

      int kontext = 0;
      if (getParent() != null)
      {
        Object parentModel = getParent().getModel();
        {
          if (parentModel instanceof AnzeigeQuerschnittModelTypAbstract)
          {
            AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parentModel;
            kontext = aq.getKontext();
          }
        }
      }

      updateAnzeigeInhalt(kontext, newValue, oldValue, AnzeigeElementTyp.WZG_B);
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      Object object = notification.getNewValue();
      if (object instanceof String)
      {
        String s = (String)object;
        getFigure().setToolTip(new Label(s));
      }
    }
  }
  
//  @Override
//  public Image getIcon()
//  {
//    // TODO Auto-generated method stub
//    return ImageFactory.getImage("icons/legende/anzeige_typ_b.png");
//  }
  
  
}
