/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;

import com.bitctrl.lib.eclipse.draw2d.FigureUtilities2;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.DetektorLveFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionDetektor;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionVerkehrsLage;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.util.LegendeHelper;
import de.bsvrz.buv.plugin.dobj.model.DoKompositum;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link DetektorModelTypAbstract} verwendet.
 * 
 * @author Pittner
 * 
 */
public class DetektorLveEditPart extends DoMitVerbLinieEditPart implements ILegendeBaustein2
{

  private Image image;
  private ArrayList<ILegendeBaustein> legendenBausteine;

  @Override
  protected IFigure createFigure()
  {

    DetektorModelTypAbstract model = getCastedModel();
    Point pos = getCastedModel().getLocation();
    if (pos == null)
      pos = new Point(2, 5);

    Dimension size = getCastedModel().getSize();
    if (size == null)
      size = new Dimension(10, 5);

    model.setSize(size);

    boolean gerunded = model.isGerunded();

    DetektorLveFigure mqf = new DetektorLveFigure("", pos.x, pos.y, size.width, size.height, gerunded);

    mqf.setSize(size);
    
//    DoModelMitVerbLinie target = getModel().getTarget();
//  
//    if (target != null)
//    {            
//      createTargetConnection(getModel(), target);        
//    }

    addToolTipListener(mqf);

    return mqf;

  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);

    final Object notifier = notification.getNotifier();
    final int type = notification.getEventType();
    if (notifier instanceof DetektorModelTypAbstract)
    {
      final int featureID = notification.getFeatureID(DoModel.class);
      switch (type)
      {
        case Notification.SET:
          switch (featureID)
          {
            case AnlagendarstellungPackage.DETEKTOR_LVE__LOCATION:
              refresh();
              break;
            case AnlagendarstellungPackage.DETEKTOR_LVE__SIZE:
              refresh();
              break;
          }
          break;
        default:
      }
    }

    if (AnlagendarstellungPackage.Literals.DETEKTOR_MODEL_TYP_ABSTRACT__GERUNDED.equals(notification.getFeature()))
    {
      boolean gerundet = notification.getNewBooleanValue();
      DetektorLveFigure fig = (DetektorLveFigure)getFigure();
      if (gerundet)
        fig.getRf().setCornerDimensions(new Dimension(90, 90));
      else
        fig.getRf().setCornerDimensions(new Dimension(0, 0));
    }
    else if (AnlagendarstellungPackage.Literals.DETEKTOR_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification
        .getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdateDetektorLVE(result);
    }
    else if (AnlagendarstellungPackage.Literals.DETEKTOR_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      Object object = notification.getNewValue();
      if (object instanceof String)
      {
        String s = (String)notification.getNewValue();
        getFigure().setToolTip(new Label(s));
      }
    }
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension size = getCastedModel().getSize();

    EditPart parent = getParent();
    if (parent instanceof AnschlussStelleEditPart)
    {
      AnschlussStelleEditPart ep = (AnschlussStelleEditPart)parent;
      Point ppos = ep.getModel().getLocation();

      getFigure().setLocation(new Point(pos.x + ppos.x, pos.y + ppos.y));

    }
    else if (parent instanceof FreeFormPolygonEditPart)
    {
      FreeFormPolygonEditPart ep = (FreeFormPolygonEditPart)parent;
      Point ppos = ep.getModel().getLocation();

      getFigure().setLocation(new Point(pos.x + ppos.x, pos.y + ppos.y));

    }
    else
    {
      if (pos != null)
        getFigure().setLocation(pos);
      else
      {
        getFigure().setLocation(new Point(2, 5));
      }
    }

    if (size != null)
    {
      DetektorLveFigure fig = (DetektorLveFigure)getFigure();
      fig.getRf().setSize(size);
      fig.setSize(size);
    }

  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    ResizableEditPolicy policy = new ResizableEditPolicy();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, policy);
  }

  /**
   * @return
   */
  protected DetektorModelTypAbstract getCastedModel()
  {
    DetektorModelTypAbstract model = null;
    if (getModel() instanceof DetektorModelTypAbstract)
    {
      model = (DetektorModelTypAbstract)getModel();
    }
    return model;
  }

  /** {@inheritDoc} */
  @Override
  public List<ILegendeBaustein> getBausteine() {
    if (legendenBausteine == null) {
      legendenBausteine = new ArrayList<ILegendeBaustein>();
      legendenBausteine.addAll(LegendeHelper.erzeugeLegendeBausteine(LegendeDefinitionDetektor.class));
    }

    return legendenBausteine;
  }

  @Override
  public Image getIcon()
  { 
    return null;
  }

  @Override
  public String getText()
  {
    // TODO Auto-generated method stub
    return getModel().getDoTyp().getName();
  }

  @Override
  public String getId()
  {
    // TODO Auto-generated method stub
    return getText();
  }
  
  @Override
  public List getDefinition()
  {
    List list = new ArrayList();
    int kontext = 0;
    
    // Hier in diesem Fall hngt die Information des Kontexts am Viewer, da das DoTyp keine Kontextinformationen hat
    Control control = getViewer().getControl();
    if (control != null){
      Object data = control.getData("kontext");
      if (data instanceof Integer)
      {
        kontext = (Integer)data;        
      }
    }    
    list.addAll(LegendeHelper.erzeugeDetektorLegendeDefinition(getCastedModel(), kontext));
    return list;
  }

}
