/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.FahrStreifenModel;
import de.bsvrz.buv.plugin.anlagendarstellung.StreckenStationModel;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.StreckenstationAnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.StreckenstationFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittProgramm;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittStatusBefehle;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionStreckenStation;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionStreckenStationStatus;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.LegendeHelper;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DobjPackage;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link StreckenStationModel} verwendet.
 * 
 * @author Pittner
 * 
 */
public class StreckenStationEditPart extends DoMitVerbLinieEditPart implements ILegendeBaustein2
{
  private ArrayList<ILegendeBaustein> legendenBausteine;

  @Override
  protected StreckenstationFigure createFigure()
  {

    StreckenStationModel model = getCastedModel();
    String bez = model.getBezeichnung();
    String kilo = model.getKilometrierung();

    if (bez == null || bez.length() == 0)
      bez = "SST 1";

    if (kilo == null || kilo.length() == 0)
      kilo = "km 1,0";

    StreckenstationAnzeigeElementFigure links = new StreckenstationAnzeigeElementFigure(bez, kilo);
    links.setSize(50, 45);
    StreckenstationFigure fig = new StreckenstationFigure(links);
    fig.setSize(70, 45);

    addToolTipListener(fig);

    return fig;

  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);

    final Object notifier = notification.getNotifier();
    final int type = notification.getEventType();
    if (notifier instanceof FahrStreifenModel)
    {
      final int featureID = notification.getFeatureID(DoModel.class);
      switch (type)
      {
        case Notification.SET:
          switch (featureID)
          {
            case DobjPackage.DO_MODEL__LOCATION:
          }
        default:
      }
    }

    if (AnlagendarstellungPackage.Literals.STRECKEN_STATION_MODEL__BEZEICHNUNG.equals(notification.getFeature()))
    {
      String neu = notification.getNewStringValue();
      StreckenstationAnzeigeElementFigure links = (StreckenstationAnzeigeElementFigure)((StreckenstationFigure)getFigure()).getLinks();
      links.getLabelName().setText(neu);
    }
    else if (AnlagendarstellungPackage.Literals.STRECKEN_STATION_MODEL__KILOMETRIERUNG
        .equals(notification.getFeature()))
    {
      String neu = notification.getNewStringValue();
      StreckenstationAnzeigeElementFigure links = (StreckenstationAnzeigeElementFigure)((StreckenstationFigure)getFigure()).getLinks();
      links.getLabelKilometer().setText(neu);
    }
    else if (AnlagendarstellungPackage.Literals.STRECKEN_STATION_MODEL__KONTEXT.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.STRECKEN_STATION_MODEL__CHILDREN.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.STRECKEN_STATION_MODEL__DATA_UPDATE.equals(notification.getFeature()))
    {
      final AnzeigeInhaltTyp status = (AnzeigeInhaltTyp)notification.getNewValue();
      dataUpdateStreckenStationModel(status);
    }
    else if (AnlagendarstellungPackage.Literals.STRECKEN_STATION_MODEL__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      String s = (String)notification.getNewValue();
      getFigure().setToolTip(new Label(s));
    }
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getCastedModel().getLocation();
    getFigure().setLocation(pos);
  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, new ResizableEditPolicy());
  }

  /**
   * @return
   */
  private StreckenStationModel getCastedModel()
  {
    StreckenStationModel model = null;
    if (getModel() instanceof StreckenStationModel)
    {
      model = (StreckenStationModel)getModel();
    }
    return model;
  }
  
  /** {@inheritDoc} */
  @Override
  public List<ILegendeBaustein> getBausteine() {
    if (legendenBausteine == null) {
      legendenBausteine = new ArrayList<ILegendeBaustein>();     
      for (Object obj : getDefinition())
      {
        List<LegendeBaustein> bausteine = LegendeHelper.erzeugeLegendeBausteine(obj);        
        legendenBausteine.addAll(bausteine);        
      }
    }
    return legendenBausteine;
  }

  @Override
  public Image getIcon()
  {
    return null;   
  }

  @Override
  public String getText()
  {
    // TODO Auto-generated method stub
    return getModel().getDoTyp().getName();
  }

  @Override
  public String getId()
  {
    // TODO Auto-generated method stub
    return getText();
  }

  @Override
  public List getDefinition()
  {
    List list = new ArrayList();
    list.add(LegendeDefinitionStreckenStation.class);
    list.add(LegendeDefinitionStreckenStationStatus.class);
    return list;
  }

}
