/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.FahrStreifenModel;
import de.bsvrz.buv.plugin.anlagendarstellung.UfdMessStelleModel;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.UfdSichtweitenFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.UmfeldDatenFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.UmfeldDatenSensorNiederschlagFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.UmfeldDatenSensorOberflaechenFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.UmfeldDatenSensorSichtweitenFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.ZusatzElementTyp;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.internal.DarstellungExtensionPoint;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DobjPackage;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Implementierung fr einen {@link DoModelEditPart} der ein Model-Objekt 
 * vom Typ {@link UfdMessStelleModel} verwendet.
 * 
 * @author Pittner
 * 
 */
public class UfdMessStelleEditPart extends DoModelEditPart<UfdMessStelleModel, UmfeldDatenFigure>
{

  private UmfeldDatenSensorSichtweitenFigure _contentPane = null;

  final private List<AnzeigeModelTypAbstract> _modelChildren = new ArrayList<AnzeigeModelTypAbstract>() {
    {
      AnzeigeModelTypAbstract s1 = AnlagendarstellungFactory.eINSTANCE.createAnzeigeModelUfdSensorSW();
      ConfiguratedDoTyp doTyp = DarstellungExtensionPoint.getDoTyp("UfdSensorEditPart");
      s1.setDoTyp(doTyp);
      s1.setBezeichnung("s1");
      s1.setName("s1");
      s1.setKontext(0);
      s1.setLocation(new Point(1, 13));
      s1.setSystemObject(DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel()
          .getObject("UFDS.HS.B27.4103.S.SW"));
      add(s1);
      AnzeigeModelTypAbstract s2 = AnlagendarstellungFactory.eINSTANCE.createAnzeigeModelUfdSensorFBZ();
      ConfiguratedDoTyp doTyp2 = DarstellungExtensionPoint.getDoTyp("UfdSensorEditPart");
      s2.setDoTyp(doTyp2);
      s2.setBezeichnung("s2");
      s2.setName("s2");
      s2.setKontext(0);
      s2.setLocation(new Point(1, 13));
      s2.setSystemObject(DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel()
          .getObject("UFDS.HS.B27.4101.N.FBZ"));
      add(s1);
      AnzeigeModelTypAbstract s3 = AnlagendarstellungFactory.eINSTANCE.createAnzeigeModelUfdSensorNI();
      ConfiguratedDoTyp doTyp3 = DarstellungExtensionPoint.getDoTyp("UfdSensorEditPart");
      s3.setDoTyp(doTyp3);
      s3.setBezeichnung("s3");
      s3.setName("s3");
      s3.setKontext(0);
      s3.setLocation(new Point(1, 13));
      s3.setSystemObject(DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel()
          .getObject("UFDS.HS.B27.4101.N.NI"));
      add(s1);
    }
  };

  private UmfeldDatenSensorSichtweitenFigure _udsf;

  @Override
  protected void geheOnline()
  {
    super.geheOnline();
  }

  @Override
  protected void geheOffline()
  {
    super.geheOffline();
  }

  @Override
  protected UmfeldDatenFigure createFigure()
  {

    Point pos = getModel().getLocation();

    UmfeldDatenSensorOberflaechenFigure udof = new UmfeldDatenSensorOberflaechenFigure("WT1.", ZusatzElementTyp.OBERFLAECHE,
        ZusatzElementTyp.OBERFLAECHE_TROCKEN);
    UmfeldDatenSensorNiederschlagFigure udnf = new UmfeldDatenSensorNiederschlagFigure(" NS1", ZusatzElementTyp.NIEDERSCHLAG,
        ZusatzElementTyp.NIEDERSCHLAG_TROCKEN);

    UfdSichtweitenFigure ufdSwFigure = new UfdSichtweitenFigure("", pos);
    // ufdSwFigure.setSize(31, 19);
    UmfeldDatenFigure udf = new UmfeldDatenFigure(ufdSwFigure, udof, udnf);
    udf.setSize(70, 70);
    udf.setLabelSichtWeite("1000");

    return udf;
  }

  public void dataUpdate(ResultData result)
  {
    final UfdMessStelleModel model = getModel();
    final IFigure figure = getFigure();/* .getInhalt(); */

    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    if (atg.getPid().equals("atg.berwachungsStatus"))
    {
      Data data = result.getData();
      if (data != null)
      {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {
            String path = AnzeigeElementFigure.createImagePath(AnzeigeElementTyp.STRECKEN_STATION,
                AnzeigeInhaltTyp.TEILSTOERUNG);
            Image image = ImageFactory.getImage(path);
            // inhalt.getSst().setIcon(image);
          }
        });
      }
      else
      {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {
            String path = AnzeigeElementFigure.createImagePath(AnzeigeElementTyp.STRECKEN_STATION,
                AnzeigeInhaltTyp.GESTOERT);
            Image image = ImageFactory.getImage(path);
            // inhalt.getSst().setIcon(image);
          }
        });
      }
    }
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);

    final Object notifier = notification.getNotifier();
    final int type = notification.getEventType();
    if (notifier instanceof FahrStreifenModel)
    {
      final int featureID = notification.getFeatureID(DoModel.class);
      switch (type)
      {
        case Notification.SET:
          switch (featureID)
          {
            case DobjPackage.DO_MODEL__LOCATION:             
          }
          // break;
        default:
      }
    }

    if (AnlagendarstellungPackage.Literals.UFD_MESS_STELLE_MODEL__BEZEICHNUNG.equals(notification.getFeature()))
    {
      String neu = notification.getNewStringValue();
    }
    else if (AnlagendarstellungPackage.Literals.UFD_MESS_STELLE_MODEL__KONTEXT.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.UFD_MESS_STELLE_MODEL__SENSOREN.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.UFD_MESS_STELLE_MODEL__DATA_UPDATE.equals(notification.getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdate(result);
    }
    else if (AnlagendarstellungPackage.Literals.UFD_MESS_STELLE_MODEL__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
    }
    else if (DobjPackage.Literals.DO_MODEL__SYSTEM_OBJECT.equals(notification.getFeature()))
    {
      SystemObject so = (SystemObject)notification.getNewValue();
    }
  }

  @Override
  public void zoomChanged(double zoom)
  {
    super.zoomChanged(zoom);
    getFigure().repaint();
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension size = getFigure().getSize();

    UmfeldDatenFigure fig = (UmfeldDatenFigure)getFigure();
    UfdSichtweitenFigure swf = (UfdSichtweitenFigure)fig.getSichtweite();
    swf.createPointList(pos);
    getFigure().setLocation(pos);
    if (size != null)
      getFigure().setSize(size);
  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, new ResizableEditPolicy());
  }

}
