/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Beispiel - Figur. Es wird ein Rechteck mit einem darin enthaltenen Text gezeichnet (Vorlage BitCtrl).
 * 
 * @author Pittner
 * 
 */
public class TestFigure extends Label implements ScalableFigure
{

  /**
   * Die Ausdehnung der Figur.
   */
  private static final Dimension DIM = new Dimension(100, 30);

  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  public TestFigure()
  {
    super();
    setSize(DIM);

    setText("Hallo Welt!!!");
    setBackgroundColor(ColorConstants.white);
    setBorder(new LineBorder(ColorConstants.black));
  }

  @Override
  public void setLocation(Point p)
  {
    super.setLocation(p);
    updateFigure();
  }

  /**
   * Bestimmt die Bounds der Figur neu.
   */
  private void updateFigure()
  {
    Dimension newDimension = new Dimension(DIM);
    newDimension.scale(scale);
    setBounds(new Rectangle(getLocation(), newDimension));
  }

  /**
   * Sets the zoom level
   * 
   * @param newZoom
   *          The new zoom level
   */
  public void setScale(double newZoom)
  {

    System.out.println("TestFigure.setScale()");

    if (scale == newZoom)
      return;
    scale = newZoom;
    // sorgt dafr, dass die Gre der Figur unabhngig von der Zoomstufe
    // gleich bleibt.
    setSize((int)(DIM.width / scale), (int)(DIM.height / scale));
    // TODO: Die Skalierung des Textes anpassen.
    // updateFigure();
    layout();
    repaint();

  }

  @Override
  public double getScale()
  {
    return scale;
  }

}
