/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.FreeFormFahrStreifenModel;

/**
 * Darstellung der Punktkoordinaten eines Elements eines DOT. DIe Darstellung erfolgt tabellarisch als Sektion in einer
 * TabbedPropertySheetPage.
 * 
 * @author Pittner
 * 
 */
public class PunktKoordinatenSection extends AbstractDobjSection<FreeFormFahrStreifenModel>
{

  /**
   * Aktion zum Anlegen eines neuen Dekorierers. Die Aktion ffnet den Dialog zum Eingeben der gewnschten Parameter des
   * Dekorierers und legt einen neuen Dekorierer in der Liste der Dekorierer des bearbeitenden Elements an.
   */
  private class AddDekoriererAction extends Action
  {

    /** Konstruktor. */
    public AddDekoriererAction()
    {
      super("Punkt hinzufgen");
    }

    @Override
    public void run()
    {
      final PunktkoordinatenManipulationsDialog dialog = new PunktkoordinatenManipulationsDialog(getPart().getSite()
          .getShell(), null);
      if (dialog.open() == Window.OK)
      {
        final Point punkt = dialog.getPunkt();

        System.out.println("ListBildInhalt2Section.AddDekoriererAction.run()");

        if (punkt != null)
        {

          PointList punkte = getElement().getPunkte();
          punkte.addPoint(punkt);
          getElement().setPunkte(punkte);

          EAttribute feature = AnlagendarstellungPackage.Literals.FREE_FORM_FAHR_STREIFEN_MODEL__PUNKTE;
          final SetCommand cmd = new SetCommand(getElement(), feature, punkte);
          getCommandStack().execute(cmd);

          aktualisiereDaten();
        }

      }
    }
  }

  /**
   * Aktion zum Entfernen von Dekorierern eines Elementes eines Darstellungsobjekttyps.
   */
  private class RemoveDekoriererAction extends Action
  {

    /** Konstruktor. */
    public RemoveDekoriererAction()
    {
      super("Punkt entfernen");
    }

    @Override
    public boolean isEnabled()
    {
      return super.isEnabled() && (!decorators.getSelection().isEmpty());
    }

    @Override
    public void run()
    {
      // final Object[] zuLoeschen = ((IStructuredSelection)decorators.getSelection()).toArray();

      int[] iis = decorators.getTable().getSelectionIndices();

      if (iis.length > 0)
      {
        if (MessageDialog.openConfirm(getPart().getSite().getShell(), "Punkte lschen",
            "Sollen die ausgewhlten Punkte wirklich gelscht werden?"))
        {
          PointList punkte = getElement().getPunkte();
          for (int index : iis)
          {
            punkte.removePoint(index);
          }
          getElement().setPunkte(punkte);

          EAttribute feature = AnlagendarstellungPackage.Literals.FREE_FORM_FAHR_STREIFEN_MODEL__PUNKTE;
          final SetCommand cmd = new SetCommand(getElement(), feature, punkte);
          getCommandStack().execute(cmd);
        }
      }
    }
  }

  /**
   * Aktion zum Bearbeiten eine Dekorierers eines Elementes eines Darstellungsobjekttyps. Die Daten eines aktuell
   * ausgewhlten Dekorierers werden in einem Dialog prsentiert und knnen dort angepasst werden. Der Dekorierer wird
   * nach Abschluss des Dialogs durch den aus den eingebenen Daten definierten Dekorierer ersetzt, d.h. der Dekorierer
   * wird nicht wirklich bearbeitet!
   */
  private class EditDekoriererAction extends Action
  {

    /** Konstruktor. */
    public EditDekoriererAction()
    {
      super("Dekorierer bearbeiten");
    }

    @Override
    public boolean isEnabled()
    {
      return super.isEnabled() && ((IStructuredSelection)decorators.getSelection()).size() == 1;
    }

    @Override
    public void run()
    {
      final Point pt = (Point)((IStructuredSelection)decorators.getSelection()).getFirstElement();

      int index = decorators.getTable().getSelectionIndex();

      System.out.println("ListBildInhalt2Section.EditDekoriererAction.run():" + index);

      // if (index != null)
      {
        final PunktkoordinatenManipulationsDialog dialog = new PunktkoordinatenManipulationsDialog(getPart().getSite()
            .getShell(), pt);
        if (dialog.open() == Window.OK)
        {
          Point neu = dialog.getPunkt();
          if (neu != null)
          {
            PointList punkte = getElement().getPunkte();

            punkte.setPoint(pt, index);

            getElement().setPunkte(punkte);

            EAttribute feature = AnlagendarstellungPackage.Literals.FREE_FORM_FAHR_STREIFEN_MODEL__PUNKTE;
            final SetCommand cmd = new SetCommand(getElement(), feature, punkte);
            getCommandStack().execute(cmd);

            aktualisiereDaten();

          }
        }
      }
    }
  }

  /**
   * Aktion zum Bearbeiten eine Dekorierers eines Elementes eines Darstellungsobjekttyps. Die Daten eines aktuell
   * ausgewhlten Dekorierers werden in einem Dialog prsentiert und knnen dort angepasst werden. Der Dekorierer wird
   * nach Abschluss des Dialogs durch den aus den eingebenen Daten definierten Dekorierer ersetzt, d.h. der Dekorierer
   * wird nicht wirklich bearbeitet!
   */
  private class UpDekoriererAction extends Action
  {

    /** Konstruktor. */
    public UpDekoriererAction()
    {
      super("Dekorierer bearbeiten");
    }

    @Override
    public boolean isEnabled()
    {
      return super.isEnabled() && ((IStructuredSelection)decorators.getSelection()).size() == 1
          && decorators.getTable().getSelectionIndex() > 0;
    }

    @Override
    public void run()
    {
      final Point pt = (Point)((IStructuredSelection)decorators.getSelection()).getFirstElement();

      int index = decorators.getTable().getSelectionIndex();

      PointList punkte = getElement().getPunkte();

      if (index > 0)
      {
        Point tmp = punkte.getPoint(index - 1);
        punkte.setPoint(pt, index - 1);
        punkte.setPoint(tmp, index);
      }

      getElement().setPunkte(punkte);

      EAttribute feature = AnlagendarstellungPackage.Literals.FREE_FORM_FAHR_STREIFEN_MODEL__PUNKTE;
      final SetCommand cmd = new SetCommand(getElement(), feature, punkte);
      getCommandStack().execute(cmd);

      aktualisiereDaten();
    }
  }

  /**
   * Aktion zum Bearbeiten eine Dekorierers eines Elementes eines Darstellungsobjekttyps. Die Daten eines aktuell
   * ausgewhlten Dekorierers werden in einem Dialog prsentiert und knnen dort angepasst werden. Der Dekorierer wird
   * nach Abschluss des Dialogs durch den aus den eingebenen Daten definierten Dekorierer ersetzt, d.h. der Dekorierer
   * wird nicht wirklich bearbeitet!
   */
  private class DownDekoriererAction extends Action
  {

    /** Konstruktor. */
    public DownDekoriererAction()
    {
      super("Dekorierer bearbeiten");
    }

    @Override
    public boolean isEnabled()
    {
      int index = decorators.getTable().getSelectionIndex();
      return super.isEnabled() && ((IStructuredSelection)decorators.getSelection()).size() == 1
          && index < decorators.getTable().getItemCount() - 1;
    }

    @Override
    public void run()
    {
      final Point pt = (Point)((IStructuredSelection)decorators.getSelection()).getFirstElement();

      int index = decorators.getTable().getSelectionIndex();

      PointList punkte = getElement().getPunkte();

      if (index < decorators.getTable().getItemCount() - 1)
      {
        Point tmp = punkte.getPoint(index + 1);
        punkte.setPoint(pt, index + 1);
        punkte.setPoint(tmp, index);
      }

      getElement().setPunkte(punkte);

      EAttribute feature = AnlagendarstellungPackage.Literals.FREE_FORM_FAHR_STREIFEN_MODEL__PUNKTE;
      final SetCommand cmd = new SetCommand(getElement(), feature, punkte);
      getCommandStack().execute(cmd);

      aktualisiereDaten();
    }
  }

  /**
   * Provider fr die Textdarstellung der Dekorierer in der Dekorierertabelle.
   */
  private class AttributeTextDecoratorLabelProvider extends LabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(final Object element, final int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(final Object element, final int columnIndex)
    {
      String result = null;
      if (element instanceof Point)
      {
        final Point pt = (Point)element;
        switch (columnIndex)
        {
        // Attributgruppe
          case 0:
            if (pt != null)
            {
              result = "" + pt.x;
            }
            break;
          case 1:
            if (pt != null)
            {
              result = "" + pt.y;
            }
            break;
          default:
            result = "???";
            break;
        }

      }
      if (result == null)
      {
        result = "";
      }
      return result;
    }
  }

  /**
   * die Tabelle zur Darstellung der mit dem aktuellen Element verbundenen Dekorierer.
   */
  private TableViewer decorators;

  /** Schaltflche zum Anlegen eines neuen Dekorierers. */
  private Button addButton;

  /** Schaltflche zum Bearbeiten des aktuell ausgewhlten Dekorierers. */
  private Button editButton;

  /** Schaltflche zum Entfernen der aktuell ausgewhlten Dekorierer. */
  private Button removeButton;

  private Button upButton;

  private Button downButton;

  @Override
  protected void aktualisiereDaten()
  {
    FreeFormFahrStreifenModel model = getElement();
    if (model != null)
    {
      PointList punkte = model.getPunkte();
      if (punkte != null)
      {
        Point[] pts = new Point[punkte.size()];
        for (int i = 0; i < punkte.size(); i++)
        {
          Point pt = punkte.getPoint(i);
          pts[i] = pt;
        }
        decorators.setInput(pts);
      }
    }

  }

  @Override
  public void createUI(final Composite panel)
  {

    final Table table = getWidgetFactory().createTable(panel, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);
    FormData data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(40, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    data.bottom = new FormAttachment(100, -ITabbedPropertyConstants.VSPACE);
    data.height = 150;
    table.setLayoutData(data);
    table.setHeaderVisible(true);
    table.setLinesVisible(true);

    final TableLayout tabLayout = new TableLayout();
    table.setLayout(tabLayout);

    TableColumn col = new TableColumn(table, SWT.NONE);
    col.setText("X-Wert");
    tabLayout.addColumnData(new ColumnWeightData(1));
    col = new TableColumn(table, SWT.NONE);
    col.setText("Y-Wert");
    tabLayout.addColumnData(new ColumnWeightData(1));

    decorators = new TableViewer(table);
    decorators.setContentProvider(new ArrayContentProvider());
    decorators.setLabelProvider(new AttributeTextDecoratorLabelProvider());

    final CLabel label = getWidgetFactory().createCLabel(panel, "Punktkoordinaten:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(table, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    label.setLayoutData(data);

    /* Buttons */
    addButton = getWidgetFactory().createButton(panel, "Hinzufgen", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(table, ITabbedPropertyConstants.HSPACE);
    data.right = new FormAttachment(50);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    addButton.setLayoutData(data);
    addButton.setData(new AddDekoriererAction());

    editButton = getWidgetFactory().createButton(panel, "Bearbeiten", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(table, ITabbedPropertyConstants.HSPACE);
    data.right = new FormAttachment(50);
    data.top = new FormAttachment(addButton);
    editButton.setLayoutData(data);
    editButton.setData(new EditDekoriererAction());

    removeButton = getWidgetFactory().createButton(panel, "Entfernen", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(table, ITabbedPropertyConstants.HSPACE);
    data.right = new FormAttachment(50);
    data.top = new FormAttachment(editButton);
    removeButton.setLayoutData(data);
    removeButton.setData(new RemoveDekoriererAction());

    upButton = getWidgetFactory().createButton(panel, "Hoch", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(table, ITabbedPropertyConstants.HSPACE);
    data.right = new FormAttachment(50);
    data.top = new FormAttachment(removeButton);
    upButton.setLayoutData(data);
    upButton.setData(new UpDekoriererAction());

    downButton = getWidgetFactory().createButton(panel, "Runter", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(table, ITabbedPropertyConstants.HSPACE);
    data.right = new FormAttachment(50);
    data.top = new FormAttachment(upButton);
    downButton.setLayoutData(data);
    downButton.setData(new DownDekoriererAction());

    createDecoratorsContextMenue();
    updateButtons();
  }

  /**
   * aktualisiert die Darstellung der Befehlsschaltflchen entsprechend der aktuellen Auswahl innerhalb der
   * Dekoriererliste.
   */
  private void updateButtons()
  {
    addButton.setEnabled(((Action)addButton.getData()).isEnabled());
    removeButton.setEnabled(((Action)removeButton.getData()).isEnabled());
    editButton.setEnabled(((Action)editButton.getData()).isEnabled());
    upButton.setEnabled(((Action)upButton.getData()).isEnabled());
    downButton.setEnabled(((Action)downButton.getData()).isEnabled());
  }

  /** erzeugt das Kontextmen der Dekoriererliste. */
  private void createDecoratorsContextMenue()
  {
    final MenuManager menuMgr = new MenuManager();
    menuMgr.add(new AddDekoriererAction());
    menuMgr.add(new EditDekoriererAction());
    menuMgr.add(new RemoveDekoriererAction());
    decorators.getControl().setMenu(menuMgr.createContextMenu(decorators.getControl()));
  }

  @Override
  protected void installListeners()
  {
    addButton.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final org.eclipse.swt.events.SelectionEvent e)
      {
        ((Action)addButton.getData()).run();
      }
    });

    editButton.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final org.eclipse.swt.events.SelectionEvent e)
      {
        ((Action)editButton.getData()).run();
      }
    });

    removeButton.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final org.eclipse.swt.events.SelectionEvent e)
      {
        ((Action)removeButton.getData()).run();
      }
    });

    upButton.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final org.eclipse.swt.events.SelectionEvent e)
      {
        ((Action)upButton.getData()).run();
      }
    });

    downButton.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final org.eclipse.swt.events.SelectionEvent e)
      {
        ((Action)downButton.getData()).run();
      }
    });

    decorators.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(final SelectionChangedEvent event)
      {
        updateButtons();
      }
    });
  }

  /** Dialog zum Bearbeiten bzw. Anlegen von Punktkoordinaten. */
  private class PunktkoordinatenManipulationsDialog extends TitleAreaDialog
  {

    /** Der Punkt. */
    private Point punkt;

    /** Eingabefeld fr die X-Koordinate. */
    private Text anfangswertText;

    /** Eingabefeld fr die Y-Koordinate. */
    private Text endwertText;

    /** Schaltflche zum Bearbeiten der X-Koordinate. */
    private Button anfangswertButton;

    /** Schaltflche zum Bearbeiten der Y-Koordinate. */
    private Button endwertButton;

    /**
     * Konstruktor. Der bergebene Punktkoordinate dient zur Initialisierung der Eingabefelder. Wenn eine neue
     * Punktkoordinate angelegt wird, wird der Wert <code>null</code> bergeben.
     * 
     * @param shell
     *          die bergeordnete Shell
     * @param vorlage
     *          der optionale zu bearbeitende Punktkoordinate
     */
    public PunktkoordinatenManipulationsDialog(final Shell shell, final Point punkt)
    {
      super(shell);
      if (punkt != null)
      {
        this.punkt = punkt;
      }
      else
      {
        this.punkt = new Point(0, 0);
      }
    }

    @Override
    protected Control createDialogArea(final Composite parent)
    {

      setTitle("Eingabe von Punktkoordinaten");
      setMessage("Bitte geben Sie die gewnschten Punktkoordinate(n) ein!");

      final Composite panel = new Composite(parent, SWT.NONE);
      GridLayoutFactory.fillDefaults().margins(10, 10).spacing(5, 5).numColumns(3).applyTo(panel);
      GridDataFactory.fillDefaults().grab(true, true).applyTo(panel);

      Label label = new Label(panel, SWT.NONE);

      label = new Label(panel, SWT.NONE);
      label.setText("X-Koordinate:");
      GridDataFactory.fillDefaults().grab(true, false).span(3, 1).applyTo(label);

      anfangswertText = new Text(panel, SWT.BORDER);
      anfangswertText.setEditable(false);
      GridDataFactory.fillDefaults().grab(true, false).applyTo(anfangswertText);
      anfangswertButton = new Button(panel, SWT.PUSH);
      anfangswertButton.setText("...");
      GridDataFactory.fillDefaults().span(2, 1).applyTo(anfangswertButton);
      anfangswertButton.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(final SelectionEvent e)
        {
          int wert;
          try
          {
            wert = Integer.parseInt(anfangswertText.getText());
          }
          catch (final NumberFormatException ex)
          {
            wert = 0;
          }
          final InputDialog dialog = new InputDialog(getShell(), "X-Koordinate", "Geben Sie den Wert ein!", Integer
              .toString(wert), new IInputValidator() {
            @Override
            public String isValid(final String newText)
            {
              try
              {
                punkt.x = Integer.valueOf(newText);
                return null;
              }
              catch (final NumberFormatException ex)
              {
                return "Eingabe kann nicht als Zahl interpretiert werden!";
              }
            }
          });
          if (dialog.open() == Window.OK)
          {
            anfangswertText.setText(dialog.getValue());
          }
          super.widgetSelected(e);
        }
      });

      label = new Label(panel, SWT.NONE);
      label.setText("X-Koordinate:");
      GridDataFactory.fillDefaults().grab(true, false).span(3, 1).applyTo(label);

      endwertText = new Text(panel, SWT.BORDER);
      endwertText.setEditable(false);
      GridDataFactory.fillDefaults().grab(true, false).applyTo(endwertText);
      endwertButton = new Button(panel, SWT.PUSH);
      endwertButton.setText("...");
      GridDataFactory.fillDefaults().span(2, 1).applyTo(endwertButton);
      endwertButton.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(final SelectionEvent e)
        {
          int wert;
          try
          {
            wert = Integer.parseInt(endwertText.getText());
          }
          catch (final NumberFormatException ex)
          {
            wert = 0;
          }
          final InputDialog dialog = new InputDialog(getShell(), "Y-Koordinate", "Geben Sie den Wert ein!", Integer
              .toString(wert), new IInputValidator() {
            @Override
            public String isValid(final String newText)
            {
              try
              {
                punkt.y = Integer.valueOf(newText);
                return null;
              }
              catch (final NumberFormatException ex)
              {
                return "Eingabe kann nicht als Zahl interpretiert werden!";
              }
            }
          });
          if (dialog.open() == Window.OK)
          {
            endwertText.setText(dialog.getValue());
          }
          super.widgetSelected(e);

        }
      });

      initialisiereDaten();

      return panel;
    }

    /** intialisiert die Datenfelder des Dialogs. */
    private void initialisiereDaten()
    {

      if (punkt != null)
      {
        anfangswertText.setText(Integer.toString(punkt.x));
        endwertText.setText(Integer.toString(punkt.y));
      }
      else
      {
        anfangswertText.setText(Integer.toString(0));
        endwertText.setText(Integer.toString(0));
      }

    }

    /**
     * Getter fr den Punkt.
     * 
     * @return der Punkt
     */
    public Point getPunkt()
    {
      return punkt;
    }

  }

}
