/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.einstellungen;

import java.util.List;

import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungPackage;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.bitctrl.eclipse.modell.emf.EMFModellEinstellungen;

/**
 * Zentrale Verwaltungsklasse zum Auslesen von gespeicherten Ansichten aus den Nutzereinstellungen.
 * 
 * @author Pittner
 * 
 */
public class SbaEinstellungen extends EMFModellEinstellungen<Ansicht>
{

  private static final long serialVersionUID = 1L;

  public static final SbaEinstellungen INSTANCE = new SbaEinstellungen();

  private SbaEinstellungen()
  {
    super(Ansicht.class, DarstellungPackage.Literals.ANSICHT);
  }

  /**
   * Liefert eine gespeicherte Ansicht.
   * 
   * @return eine Ansicht.
   */
  public Ansicht getAnsicht(int i)
  {
    List<Ansicht> ansichten = getAlleModellEinstellungen(EinstellungsArt.benutzerLokal);
    if (ansichten.isEmpty())
      ansichten = getAlleModellEinstellungen(EinstellungsArt.allgemeinLokal);

    if (!ansichten.isEmpty())
      return ansichten.get(i);
    return null;
  }

  /**
   * Liefert eine gespeicherte Ansicht.
   * 
   * @return eine Ansicht.
   */
  public Ansicht getAnsicht(String name)
  {
    List<Ansicht> ansichten = getAlleModellEinstellungen(EinstellungsArt.benutzerLokal);
    if (ansichten.isEmpty())
      ansichten = getAlleModellEinstellungen(EinstellungsArt.allgemeinLokal);

    if (!ansichten.isEmpty())
    {
      for (Ansicht ansicht : ansichten)
      {
        if (ansicht.getName().equals(name))
        {
          return ansicht;
        }
      }
    }
    return null;
  }

  /**
   * Holt alle Ansichten
   * 
   * @return
   */
  public List<Ansicht> getAnsichten()
  {
    List<Ansicht> ansichten = getAlleModellEinstellungen(EinstellungsArt.benutzerLokal);
    if (ansichten.isEmpty())
      ansichten = getAlleModellEinstellungen(EinstellungsArt.allgemeinLokal);

    return ansichten;
  }
}
