/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Figure, welche einen AQ eines beliebigen Typs reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class AnzeigeQuerschnittFigure extends Figure implements ScalableFigure
{

  private IFigure _links;

  private IFigure _rechts;

  private IFigure _rechts2;


  public AnzeigeQuerschnittFigure(IFigure links)
  {
    this._links = links;

    setOpaque(true);
    setToolTip(new Label(""));
    setLayoutManager(new XYLayout());

    add(_links);
    setConstraint(_links, new Rectangle(0, 0, _links.getSize().width, _links.getSize().height));
  }

  public AnzeigeQuerschnittFigure()
  {
    setOpaque(true);
    setToolTip(new Label(""));
    setLayoutManager(new XYLayout());
    updateFigure();
  }

  public void addFigure(IFigure figure)
  {
    this._links = figure;
    add(_links);
    setConstraint(_links, new Rectangle(0, 0, _links.getSize().width, _links.getSize().height));
  }

  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  /**
   * Sets the zoom level
   * 
   * @param newZoom
   *          The new zoom level
   */
  public void setScale(double newZoom)
  {
    if (scale == newZoom)
      return;
    scale = newZoom;
    // sorgt dafr, dass die Gre der Figur unabhngig von der Zoomstufe
    // gleich bleibt.

    if (_links != null)
      ((ScalableFigure)_links).setScale(newZoom);
    if (_rechts != null)
      ((ScalableFigure)_rechts).setScale(newZoom);

    // TODO: Die Skalierung des Textes anpassen.
    // setSize((int) (DIM.width * scale), (int) (DIM.height * scale));

    updateFigure();
    layout();
    repaint();

  }

  /**
   * Bestimmt die Bounds der Figur neu.
   */
  private void updateFigure()
  {
    setBounds(new Rectangle(getLocation(), getSize()));
  }

  @Override
  public double getScale()
  {
    return scale;
  }

  public void switchLinksRechts(int fr)
  {

    List children = getChildren();
    for (Object object : children)
    {
      if (object instanceof AnzeigeQuerschnittLinksFigure)
      {
        if (fr != 1)
        {
          _links = (AnzeigeQuerschnittLinksFigure)object;
          ((AnzeigeQuerschnittLinksFigure)_links).switchLinksRechts(fr);
        }
        else
        {
          _rechts = (AnzeigeQuerschnittLinksFigure)object;
          ((AnzeigeQuerschnittLinksFigure)_rechts).switchLinksRechts(fr);
        }

      }
      else if (object instanceof AnzeigeQuerschnittRechtsFigure)
      {
        if (fr != 1)
        {
          _rechts = (AnzeigeQuerschnittRechtsFigure)object;
          ((AnzeigeQuerschnittRechtsFigure)_rechts).switchLinksRechts(fr);
        }
        else
        {
          _links = (AnzeigeQuerschnittRechtsFigure)object;
          ((AnzeigeQuerschnittRechtsFigure)_links).switchLinksRechts(fr);
        }
      }
    }

    add(_links, 0);
    setConstraint(_links, new Rectangle(0, 0, _links.getSize().width, _links.getSize().height));

    add(_rechts, 1);
    if (fr != 1)
      setConstraint(_rechts,
          new Rectangle(_links.getSize().width, 0, _rechts.getSize().width, _rechts.getSize().height));
    else
    {
      if (_links instanceof AnzeigeQuerschnittRechtsFigureA14)
      {
        setConstraint(_rechts, new Rectangle(_links.getSize().width - 4, 0, _rechts.getSize().width,
            _rechts.getSize().height));
      }
      else
      {
        setConstraint(_rechts, new Rectangle(_links.getSize().width - 5, 0, _rechts.getSize().width,
            _rechts.getSize().height));
      }
    }

    updateFigure();
  }

  public AnzeigeQuerschnittFigure(IFigure links, IFigure rechts)
  {
    this._links = links;
    this._rechts = rechts;

    setOpaque(false);
    setToolTip(new Label(""));
    setLayoutManager(new XYLayout());

//    int offset = 20;

    add(_links);
    setConstraint(_links, new Rectangle(0, 0, _links.getSize().width, _links.getSize().height));
    add(_rechts);
    setConstraint(_rechts, new Rectangle(_links.getSize().width, 0, _rechts.getSize().width, _rechts.getSize().height));
  }

  public AnzeigeQuerschnittFigure(IFigure links, IFigure rechts, IFigure rechts2)
  {
    this._links = links;
    this._rechts = rechts;
    this._rechts2 = rechts2;

    setOpaque(true);
    setToolTip(new Label(""));
    ToolbarLayout tl = new ToolbarLayout();

    tl.setVertical(false);

    setLayoutManager(tl);
    add(links);
    add(rechts);
    add(rechts2);

    updateFigure();
  }

  public void setSelected(boolean isSelected)
  {
    LineBorder lineBorder = (LineBorder)getBorder();
    if (isSelected)
    {
      lineBorder.setWidth(1);
      lineBorder.setColor(ColorConstants.black);
    }
    else
    {
      lineBorder.setWidth(2);
      lineBorder.setColor(ColorConstants.orange);
    }
  }

  public IFigure getLinks()
  {

    List children = getChildren();
    for (Object object : children)
    {
      if (object instanceof AnzeigeQuerschnittLinksFigure)
      {
        _links = (AnzeigeQuerschnittLinksFigure)object;
        return _links;
      }
    }
    return null;
  }

  public void setLinks(IFigure links)
  {
    this._links = links;
  }

  public IFigure getRechts()
  {

    List children = getChildren();
    for (Object object : children)
    {
      if (object instanceof AnzeigeQuerschnittRechtsFigure)
      {
        _rechts = (AnzeigeQuerschnittRechtsFigure)object;
        return _rechts;
      }
    }
    return null;
  }

  public void setRechts(IFigure rechts)
  {
    this._rechts = rechts;
  }

  public void repaint()
  {
    super.repaint();
  }

  public IFigure getRechts2()
  {
    return _rechts2;
  }

  public void setRechts2(IFigure rechts2)
  {
    _rechts2 = rechts2;
  }

  public void setExtraText(String text)
  {

    AnzeigeQuerschnittLinksFigure fig = null;

    if (_links instanceof AnzeigeQuerschnittLinksFigure)
    {
      fig = (AnzeigeQuerschnittLinksFigure)_links;
    }
    else if (_rechts instanceof AnzeigeQuerschnittLinksFigure)
    {
      fig = (AnzeigeQuerschnittLinksFigure)_rechts;
    }

    if (fig != null)
      fig.setName(text);

    updateFigure();
  }
}
