/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.Label;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;

/**
 * Figure, welche den rechten Teil eines AQ vom Typ A14 reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class AnzeigeQuerschnittRechtsFigureA14 extends AnzeigeQuerschnittRechtsFigure
{

  private Label _labelProgrammBlind;

  public AnzeigeQuerschnittRechtsFigureA14(int kontext)
  {

    super(kontext);

    switch (kontext)
    {
      case 0:
      case 1:
        createLabelProgrammBlind();
        break;

      default:
    }

  }

  private void createLabelProgrammBlind()
  {
    _labelProgrammBlind = new Label();
    Image image2 = ImageFactory.getImage("icons/Prg_leer.png");

    _labelProgrammBlind.setIcon(image2);
    add(_labelProgrammBlind);
    GridData gridData = new GridData();
    gridData.grabExcessVerticalSpace = false;
    gridData.verticalAlignment = SWT.TOP;
    getLayoutManager().setConstraint(_labelProgrammBlind, gridData);
  }

  public void switchLinksRechts(int fr)
  {

    GridData gd1 = null;
    GridData gd2 = null;
    GridData gd3 = null;
    if (_labelHelligkeit != null)
      gd1 = (GridData)getLayoutManager().getConstraint(_labelHelligkeit);
    if (_labelProgramm != null)
      gd2 = (GridData)getLayoutManager().getConstraint(_labelProgramm);
    if (_labelProgrammBlind != null)
      gd3 = (GridData)getLayoutManager().getConstraint(_labelProgrammBlind);

    int i1 = 0;
    int i2 = 0;
    int i3 = 0;

    if (fr != 1)
    {
      if (gd1 != null)
      {
        gd1.grabExcessVerticalSpace = false;
        gd1.verticalAlignment = SWT.TOP;
      }
      if (gd2 != null)
      {
        gd2.grabExcessVerticalSpace = false;
        gd2.verticalAlignment = SWT.TOP;
      }
      if (gd3 != null)
      {
        gd3.grabExcessVerticalSpace = false;
        gd3.verticalAlignment = SWT.TOP;
      }
      i1 = 0;
      i2 = 1;
      i3 = 2;
    }
    else
    {
      if (gd1 != null)
      {
        gd1.grabExcessVerticalSpace = false;
        gd1.verticalAlignment = SWT.BOTTOM;
      }
      if (gd2 != null)
      {
        gd2.grabExcessVerticalSpace = true;
        gd2.verticalAlignment = SWT.BOTTOM;
      }
      if (gd3 != null)
      {
        gd3.grabExcessVerticalSpace = true;
        gd3.verticalAlignment = SWT.BOTTOM;
      }
      i1 = 2;
      i2 = 1;
      i3 = 0;
    }
    if (_labelHelligkeit != null)
    {
      add(_labelHelligkeit, i1);
      getLayoutManager().setConstraint(_labelHelligkeit, gd1);
    }
    if (_labelProgramm != null)
    {
      if (_labelHelligkeit == null)
        i2 = 0;
      add(_labelProgramm, i2);
      getLayoutManager().setConstraint(_labelProgramm, gd2);
    }
    if (_labelProgrammBlind != null)
    {
      add(_labelProgrammBlind, i3);
      getLayoutManager().setConstraint(_labelProgrammBlind, gd3);
    }
  }

  public Label getLabelProgrammBlind()
  {
    return _labelProgrammBlind;
  }
}
