/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.RoundedRectangle;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;

import de.bsvrz.buv.plugin.anlagendarstellung.DetektorModelTypAbstract;

/**
 * Figure used to represent a table in the schema
 * 
 * @author Pittner
 * 
 */
public class DetektorLveFigure extends Figure /* implements ScalableFigure */
{

  String _name;

  int _x;

  int _y;

  int _width;

  int _height;

  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  private boolean _selected = false;

  RoundedRectangle _rf = new RoundedRectangle();

  public DetektorLveFigure(String name, int x, int y, int width, int height, boolean gerundet)
  {
    _name = name;
    _x = x;
    _y = y;
    _width = width;
    _height = height;

    setOpaque(false);
    setLayoutManager(new XYLayout());

//    setToolTip(new Label(name));

    setBounds(new Rectangle(x, y, width, height));

    setLocation(new Point(x, y));

    _rf.setBounds(new Rectangle(x, y, width, height));
    _rf.setSize(width, height);
    if (gerundet)
      _rf.setCornerDimensions(new Dimension(90, 90));
    else
      _rf.setCornerDimensions(new Dimension(0, 0));

    _rf.setBackgroundColor(ColorConstants.gray);
    // _rf.setBorder(new LineBorder(ColorConstants.black, 1));

    add(_rf);

    setConstraint(_rf, new Rectangle(0, 0, -1, -1));

  }

  public DetektorLveFigure(String name, DetektorModelTypAbstract model)
  {
    _name = name;

    setLayoutManager(new XYLayout());
    setOpaque(true);

//    setToolTip(new Label(name));

    setBackgroundColor(ColorConstants.gray);
    setBorder(new LineBorder(ColorConstants.black, 1));

    ColoredPolygon rf = new ColoredPolygon(ColorConstants.gray);
    rf.setOpaque(true);
    PointList pl = new PointList(5);
    pl.addPoint(0, 0);
    pl.addPoint(10, 0);
    pl.addPoint(10, 5);
    pl.addPoint(0, 5);
    // pl.addPoint(0, 0);
    rf.setPoints(pl);

    add(rf);
    setConstraint(rf, new Rectangle(0, 0, -1, -1));

    updateFigure();

  }

  public RoundedRectangle getRf()
  {
    return _rf;
  }

  public void setRf(RoundedRectangle rf)
  {
    _rf = rf;
  }

  public String getName()
  {
    return _name;
  }

  public void setName(String name)
  {
    _name = name;
  }

  public boolean isSelected()
  {
    return _selected;
  }

  /**
   * Bestimmt die Bounds der Figur neu.
   */
  private void updateFigure()
  {
    setBounds(new Rectangle(getLocation(), getSize()));
  }
}
