package de.bsvrz.buv.plugin.anlagendarstellung.legende;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.part.EditorPart;

import sun.reflect.Reflection;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorLVE;
import de.bsvrz.buv.plugin.anlagendarstellung.FahrStreifenModelAbstract;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungsSpalte;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;
import de.bsvrz.sys.funclib.bitctrl.modell.tmumfelddatenglobal.objekte.UmfeldDatenMessStelle;

/**
 * Legende-Composite fuer die Anlagendarstellung
 * 
 * @author Pittner
 * @version $Id: DoModelLegendeComposite.java,v 1.4 2014/05/16 09:01:57 Pittner Exp $
 */
public class DoModelLegendeComposite extends Composite 
{

	/** {@inheritDoc} */
	@Override
	public boolean setFocus() {
		return false;
	}

	/** Liste der Images. */
	private final List<Image> imagelist = new ArrayList<Image>();
  private Map<String,String> _map;
	
  

	/**
	 * Erzeugt ein Composite f&uuml;r die Legende.
	 * 
	 * @param parent
	 *            Parent-Composite
	 * @param style
	 *            Style
	 */
	public DoModelLegendeComposite(final Composite parent, final int style, AnsichtEditPart epart) {
		super(parent, style);
		setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));
		
		_map = new HashMap<String, String>();

		final GridLayout pageLayout = new GridLayout();
		// pageLayout.numColumns = 2;
		setLayout(pageLayout);
		setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		init(epart);
	
	}



  /**
   * Fhrt die Initialisierung anhand der bergebenen {@link Ansicht} durch.
   * @param epart
   *          der {@link EditorPart} welcher die Ansicht liefert.
   */
  private void init(AnsichtEditPart epart)
  {
    if (epart != null)
    {      
      Map registry = epart.getViewer().getEditPartRegistry();
      Ansicht model = epart.getModel();
      Darstellung darstellung = model.getDarstellung();
      
      if (darstellung instanceof StilisierteDarstellung)
      {
        StilisierteDarstellung darstellungMitSpalten = (StilisierteDarstellung)darstellung;
        EList<DarstellungsSpalte> spalten = darstellungMitSpalten.getSpalten();
        for (DarstellungsSpalte spalte : spalten)
        {
          EList<DoModel> doObjekte = spalte.getDoObjekte();
          processLegende(registry, doObjekte);
        }
      }
    }
  }



  /**
   * Erzeugt die Legende.
   * 
   * @param registry
   *          die Edipart-Registry
   * @param doObjekte
   *          die Liste der {@link DoModel}
   */
  private void processLegende(Map registry, EList<DoModel> doObjekte)
  {
    for (DoModel doModel : doObjekte)
    {
      if (registry.containsKey(doModel))
      {
        Object object = registry.get(doModel);
        if (object instanceof ILegendeBaustein2)
        {
          ILegendeBaustein2 baustein = (ILegendeBaustein2)object;
          List<ILegendeBaustein> bausteine = baustein.getBausteine();

          if (!_map.containsKey(baustein.getId()))
          {
            createGroup(this, baustein, bausteine);
            _map.put(baustein.getId(), baustein.getText());
          }
          if (doModel instanceof FahrStreifenModelAbstract)
          {
            FahrStreifenModelAbstract fsm = (FahrStreifenModelAbstract)doModel;
            EList<DetektorLVE> komponenten = fsm.getKomponenten();
             
            for (DetektorLVE komponente : komponenten)
            {
              if (registry.containsKey(komponente))
              {
                Object object2 = registry.get(komponente);
                if (object2 instanceof ILegendeBaustein2)
                {
                  ILegendeBaustein2 baustein2 = (ILegendeBaustein2)object2;
                  List<ILegendeBaustein> bausteine2 = baustein2.getBausteine();
                  if (!_map.containsKey(baustein2.getId()))
                  {
                    createGroup(this, baustein2, bausteine2);
                    _map.put(baustein2.getId(), baustein2.getText());
                  }
                }
              }
            }                  
          }
          else if (doModel instanceof AnzeigeQuerschnittModelTypAbstract)
          {
            AnzeigeQuerschnittModelTypAbstract aqm = (AnzeigeQuerschnittModelTypAbstract)doModel;
            EList<AnzeigeModelTypAbstract> komponenten = aqm.getKomponenten();
             
            for (AnzeigeModelTypAbstract komponente : komponenten)
            {
              if (registry.containsKey(komponente))
              {
                Object object2 = registry.get(komponente);
                if (object2 instanceof ILegendeBaustein2)
                {
                  ILegendeBaustein2 baustein2 = (ILegendeBaustein2)object2;
                  List<ILegendeBaustein> bausteine2 = baustein2.getBausteine();
                  if (!_map.containsKey(baustein2.getId()))
                  {
                    createGroup(this, baustein2, bausteine2);
                    _map.put(baustein2.getId(), baustein2.getText());
                  }
                }
              }
            }                 
          }
        }
        else if (object instanceof AbstractGraphicalEditPart)
        {
            AbstractGraphicalEditPart aep = (AbstractGraphicalEditPart)object;
            Object adapter = aep.getAdapter(ILegendeBaustein.class);
            if (adapter instanceof ILegendeBaustein)
            {
              ILegendeBaustein baustein = (ILegendeBaustein)adapter;
              ArrayList bausteine = new ArrayList();
              bausteine.add(baustein);
              createGroup(this, new LegendeBaustein(null, "Umfelddaten"), bausteine);
            }                  
        }  
        else{
//          System.out.println("else : " + object);
        }
      }
    }
  }

 
	
  /**
   * Erzeugt die entsprechenden {@link Group}-Elemente anhand des bergebenen Legende-Lieferanten 
   * und der bergebene Legende Inhalte.
   * @param parent
   *            das Parent an welchem die {@link Group}-Elemente angehaengt werden
   * @param baustein
   *            der Lieferant
   * @param bausteine
   *             die Inhalte
   *           
   */
  private void createGroup(Composite parent, ILegendeBaustein baustein, List<ILegendeBaustein> bausteine)
  {
    Group group = new Group(parent, SWT.SHADOW_ETCHED_IN);
    group.setText(baustein.getText());
    group.setLayout(new GridLayout(1, true));
    group.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL));
    group.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));
    
    final TableViewer viewer = new TableViewer(group, SWT.NO_FOCUS);
    viewer.setContentProvider(new ArrayContentProvider());
    viewer.setLabelProvider(new AnlagenStatusLegendeLabelProvider());
    
    // Selection abschalten
    viewer.addSelectionChangedListener(new ISelectionChangedListener() {
      
      @Override
      public void selectionChanged(final SelectionChangedEvent event)
      {
        Object source = event.getSource();
        if (source instanceof TableViewer)
        {
          TableViewer viewer = (TableViewer)source;
          viewer.getTable().deselectAll();
        }
      }
    });
    
    viewer.setInput(bausteine);
  }
	/**
	 * Erzeugt die entsprechenden {@link Group}-Elemente anhand des bergebenen Legende-Lieferanten 
	 * und der bergebene Legende Inhalte.
	 * @param parent
	 *            das Parent an welchem die {@link Group}-Elemente angehaengt werden
	 * @param baustein
	 *            der Lieferant
	 * @param bausteine
	 *             die Inhalte
	 *           
	 */
	private void createGroup(Composite parent, ILegendeBaustein2 baustein, List<ILegendeBaustein> bausteine)
  {
    List definitionen = baustein.getDefinition();
    int offset = 0;
    for (Object obj : definitionen)
    {
      List<ILegendeBaustein> bausteineNeu = new ArrayList<ILegendeBaustein>();
      int len = holeLaengeAusEnumClassObj(obj);
      String zusatz = holeBezeichnungAusEnumClassObj(obj);
      
//      if (len != bausteine.size()){
//        System.out.println("Lnge der Enum-Konstanten stimmt nicht mit der Bausteinanzahl berein: " + len + " " + bausteine.size());
//        len = bausteine.size();
//      }
      

      String text = baustein.getText();
      Group group = new Group(parent, SWT.SHADOW_ETCHED_IN);
      if (offset == 0)
      {
        text += " : " + zusatz;
        // Hier werden die naechsten Groups an das Obrste Group gehaengt
        parent = group;
      }
      else
      {
        text = zusatz;
      }
      group.setText(text);
      group.setLayout(new GridLayout(1, true));
      group.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL));
      group.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));

      final TableViewer viewer = new TableViewer(group, SWT.NO_FOCUS);
      viewer.setContentProvider(new ArrayContentProvider());
      viewer.setLabelProvider(new AnlagenStatusLegendeLabelProvider());

      // Selection abschalten
      viewer.addSelectionChangedListener(new ISelectionChangedListener() {

        @Override
        public void selectionChanged(final SelectionChangedEvent event)
        {
          Object source = event.getSource();
          if (source instanceof TableViewer)
          {
            TableViewer viewer = (TableViewer)source;
            viewer.getTable().deselectAll();
          }
        }
      });

      bausteineNeu.addAll(bausteine.subList(offset, offset + len));
      viewer.setInput(bausteineNeu);
      offset += len;
    }
  }
	
	
	/**
   * Holt per {@link Reflection} die Laenge der im Enum definierten Konstanten aus dem {@link Class} -Objekt der Enum-Definition.
	 * @param obj
	 * @return
	 */
	private int holeLaengeAusEnumClassObj(Object obj){
	  int re = 0;
	   if (obj instanceof Class<?>)
     {
       Class<?> clazz = (Class<?>)obj;
       if (clazz.isEnum()){                       
         Object[] values = clazz.getEnumConstants();
         if (values != null)
           re = values.length;
       }
     }
	  
	  return re;
	}
	
	/**
	 * Holt per {@link Reflection} die Bezeichnung aus dem {@link Class} -Objekt der Enum-Definition.
	 * Die Methode setzt voraus, dass ein statisches Feld mit dem Namen "BEZEICHNUNG" im Enum vorhanden ist.
	 * @param obj 
	 *         das {@link Class}-Objekt
	 * @return
	 *         der Inhalt des statischen {@link String} -Objektes
	 */
	private String holeBezeichnungAusEnumClassObj(Object obj){
	  String re = "";
	  if (obj instanceof Class<?>)
	  {
	    Class<?> clazz = (Class<?>)obj;
	    if (clazz.isEnum()){                       
	      Field field = null;
        try
        {
          field = clazz.getField("BEZEICHNUNG");
          try
          {
            String tmp = (String)field.get(null);
            if (tmp != null)
              re = tmp;
          }
          catch (IllegalArgumentException e)
          {
            // TODO Auto-generated catch block
            e.printStackTrace();
          }
          catch (IllegalAccessException e)
          {
            // TODO Auto-generated catch block
            e.printStackTrace();
          }
        }
        catch (SecurityException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
        catch (NoSuchFieldException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
	    }
	  }
	  
	  return re;
	}
	

	/**
	 * Label-Provider f&uuml;r die Legende.
	 * 
	 * @author BitCtrl Systems GmbH, Gieseler
	 * 
	 */
	class AnlagenStatusLegendeLabelProvider implements ILabelProvider {

		/** {@inheritDoc} */
		@Override
		public Image getImage(final Object element) { 
			if (element instanceof ILegendeBaustein) { 
			  return ((ILegendeBaustein)element).getIcon();
			}
			return null;
		}

		/** {@inheritDoc} */
		@Override
		public String getText(final Object element) { 
			if (element instanceof ILegendeBaustein) {
			  return ((ILegendeBaustein) element).getText();
			}
			return null;
		}

		/** {@inheritDoc} */
		@Override
		public void addListener(final ILabelProviderListener listener) {
			// TODO Auto-generated method stub

		}

		/** {@inheritDoc} */
		@Override
		public void dispose() {
			for (Image i : imagelist) {
				i.dispose();
			}
		}

		/** {@inheritDoc} */
		@Override
		public boolean isLabelProperty(final Object element,
				final String property) {
			// TODO Auto-generated method stub
			return false;
		}

		/** {@inheritDoc} */
		@Override
		public void removeListener(final ILabelProviderListener listener) {
			// TODO Auto-generated method stub

		}
	}
}
