/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.model;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import de.bsvrz.buv.plugin.anlagendarstellung.util.Fg6Helper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.Fg6Helper.FG6_STATUS;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Model Object of a Steuermodul, wird aus Kompatibilittsgrunden bentigt
 * 
 * @deprecated  
 * @author Pittner
 * 
 */
public class SbaDarstellungsModelSteuerModul extends SbaDarstellungsModel
{
  private Collection<String> _fg6De = null;

  private Map<String, String> _atgDe = new HashMap<String, String>();

  private Map<String, StatusTyp> _statusDe = new HashMap<String, StatusTyp>();

  private String _pid;

  public String getPid()
  {
    return _pid;
  }

  public SbaDarstellungsModelSteuerModul(String pid)
  {
    super();

    _pid = pid;
  }

  private Map<String, ResultData> resultMap = new HashMap<String, ResultData>();
  

  public AnzeigeInhaltTyp ermittleStatusAusEmpfangsDaten()
  {

    StatusTyp statusNeu = null;

    /*
     * Feststellen hchster Status aller DE
     */

    for (String dePid : getFg6De())
    {
      StatusTyp status = _statusDe.get(dePid);

      if (statusNeu == null)
      {
        statusNeu = status;
      }
      else if (status == StatusTyp.GESTOERT)
      {
        statusNeu = status;
      }
      else if (status == StatusTyp.KOMMUNIKATIONSAUSFALL)
      {
        if (statusNeu != StatusTyp.GESTOERT)
        {
          statusNeu = status;
        }
      }
      else if (status == StatusTyp.TEILSTOERUNG)
      {
        if (statusNeu != StatusTyp.GESTOERT && statusNeu != StatusTyp.KOMMUNIKATIONSAUSFALL)
        {
          statusNeu = status;
        }
      }
      else if (status == StatusTyp.TUERKONTAKT)
      {
        if (statusNeu != StatusTyp.GESTOERT && statusNeu != StatusTyp.KOMMUNIKATIONSAUSFALL
            && statusNeu != StatusTyp.TEILSTOERUNG)
        {
          statusNeu = status;
        }
      }
      else if (status == StatusTyp.OK)
      {
        if (statusNeu != StatusTyp.GESTOERT && statusNeu != StatusTyp.KOMMUNIKATIONSAUSFALL
            && statusNeu != StatusTyp.TEILSTOERUNG && statusNeu != StatusTyp.TUERKONTAKT)
        {
          statusNeu = status;
        }
      }
    }

    /*
     * Umsetzen Status in Anzeigeinhalt
     */

    AnzeigeInhaltTyp result = AnzeigeInhaltTyp.UNDEFINIERT;

    if (statusNeu == StatusTyp.GESTOERT || statusNeu == StatusTyp.KOMMUNIKATIONSAUSFALL)
    {
      result = AnzeigeInhaltTyp.GESTOERT;
    }
    if (statusNeu == StatusTyp.TEILSTOERUNG || statusNeu == StatusTyp.TUERKONTAKT)
    {
      result = AnzeigeInhaltTyp.TEILSTOERUNG;
    }
    if (statusNeu == StatusTyp.OK)
    {
      result = AnzeigeInhaltTyp.OK;
    }

    return result;
  }

  public Collection<String> getFg6De()
  {

    if (_fg6De == null)
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        _fg6De = new HashSet<String>();
        _fg6De.addAll(Fg6Helper.getAllFg6De(connection, this.getPid()));
        return Collections.unmodifiableCollection(_fg6De);
      }
      else
      {
        return Collections.emptyList();
      }
    }
    else
    {
      return Collections.unmodifiableCollection(_fg6De);
    }
  }

  public void dataChange(ResultData result)
  {

    SystemObject so = result.getObject();
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();
    String atgDe = _atgDe.get(so.getPid());
    Data data = result.getData();

    if (data != null)
    {

      if (atgDe != null)
      {
        if (atgDe.equals(atg.getPid()))
        {
          // Es werden Daten derselbenATG empfangen wie vorher
          setzeStatus(so.getPid(), atg.getPid(), data);
        }
        else
        {
          // Es werden Daten einer ANDEREN ATG empfangen
          _atgDe.put(so.getPid(), atg.getPid());
          setzeStatus(so.getPid(), atg.getPid(), data);
        }
      }
      else
      {
        // Es werden Daten empfangen, vorher war "keine Daten"
        _atgDe.put(so.getPid(), atg.getPid());
        setzeStatus(so.getPid(), atg.getPid(), data);
      }
    }
    else
    {
      if (atgDe != null)
      {
        if (atgDe.equals(atg.getPid()))
        {
          // Vorher empfangene ATG geht auf "keine Daten"
          _atgDe.remove(so.getPid());
        }
      }
    }

    /*
     * Prfen, ob fr alle DE Daten vorliegen, ansonsten liegt Kommunikationsstrung vor
     */
    for (String dePid : getFg6De())
    {
      if (_atgDe.get(dePid) == null)
      {
        _statusDe.put(dePid, StatusTyp.KOMMUNIKATIONSAUSFALL);
      }
    }
  }

  private void setzeStatus(String soPid, String atgPid, Data data)
  {
    FG6_STATUS status = Fg6Helper.getFg6Status(atgPid, data);

    switch (status)
    {
      case OK:
        _statusDe.put(soPid, StatusTyp.OK);
        break;

      case DISTURBED:
        if (atgPid.equals("atg.tlsVltTrKontakt"))
          _statusDe.put(soPid, StatusTyp.TUERKONTAKT);
        else
          _statusDe.put(soPid, StatusTyp.TEILSTOERUNG);
        break;

      case FAILURE:
        _statusDe.put(soPid, StatusTyp.GESTOERT);
        break;

      case UNDEFINED:
        break; // ignore
    }

  }

}
