/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.anlagendarstellung.programmtyp;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

/**
 * Tabellen Modell fr die Parameter SonderProgrammTypen
 * @author Pittner
 *
 */
public class TableModel extends Observable implements Observer
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;

  /**
   * Von
   */
  private Composite _anzeigeTyp;
  
	/**
	 * WVZ-Inhalt der Stufe
	 */
  private Composite _wvzInhalt;

  private Composite _blinkTakt;

  /**
   * Konstruktor der Klasse
   * @param loeschen Lschen
   * @param von Von
   * @param bis Bis
   * @param wvzInhalt WVZ Inhalt
   */
  public TableModel(Composite loeschen, Composite anzeigeTyp, Composite wvzInhalt, Composite blinkTakt)
  {
  	_loeschen = loeschen;  	
  	_anzeigeTyp = anzeigeTyp;  	
  	_wvzInhalt = wvzInhalt;
  	_blinkTakt = blinkTakt;
  }

  /**
   * Methode liefert den Wert des Composite Lschen
   * @return Wert des Composite
   */
  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen);    
  }

  /**
   * Methode setzt den Wert des Composite Lschen
   * @param selWert zu setzender Wert
   */
  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen, selWert, null);
  }
  
  /**
   * Methode liefert den Wert des Composite WVZInhalt
   * @return Wert des Composite
   */
  public String getSelectedWVZInhalt ()
  {
    return getSelectedWert(_wvzInhalt);    
  }

  /**
   * Methode setzt den Wert des Composite WVZInhalt
   * @param selWert zu setzender Wert
   */
  public void setSelectedWVZInhalt (String selWert)
  {
    setSelectedWert(_wvzInhalt, selWert, null);
  }
  
  /**
   * Methode liefert den Wert des Composite WVZInhalt
   * @return Wert des Composite
   */
  public String getSelectedAnzeigeTyp ()
  {
    return getSelectedWert(_anzeigeTyp);    
  }
  
  /**
   * Methode setzt den Wert des Composite WVZInhalt
   * @param selWert zu setzender Wert
   */
  public void setSelectedAnzeigeTyp (String selWert)
  {
    setSelectedWert(_anzeigeTyp, selWert, null);
  }
  
  /**
   * Methode liefert den Wert des Composite WVZInhalt
   * @return Wert des Composite
   */
  public String getSelectedBlinkTakt ()
  {
    return getSelectedWert(_blinkTakt);    
  }
  
  /**
   * Methode setzt den Wert des Composite WVZInhalt
   * @param selWert zu setzender Wert
   */
  public void setSelectedBlinkTakt (String selWert)
  {
    setSelectedWert(_blinkTakt, selWert, null);
  }

  /**
   * Methode liefert den Wert eines Comppsite
   * @param c Composite
   * @return Wert des Composite
   */
  public String getSelectedWert(Composite c)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = (String)cb.getData();
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  /**
   * Methode setzt den Wert eines Composite
   * @param c Composite
   * @param selWert Text auf den das Compisite gesetzt werden soll
   * @param selImage Image auf den das Compoiste gesetzt werden soll
   */
  public void setSelectedWert(Composite c, String selWert, Image selImage)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode setzt die Variable loeschen
   * @param loeschen zu bernehmender Wert fr loeschen
   */
  public void setLoeschen(Composite loeschen)
  {
  	_loeschen = loeschen;
  }

	/**
   * Methode liefert wvzInhalt zurck
   * @return wvzInhalt
   */
  public Composite getWvzInhalt()
  {
  	return _wvzInhalt;
  }

	/**
   * Methode setzt die Variable wvzInhalt
   * @param wvzInhalt zu bernehmender Wert fr wvzInhalt
   */
  public void setWvzInhalt(Composite wvzInhalt)
  {
  	_wvzInhalt = wvzInhalt;
  }

  public Composite getAnzeigeTyp()
  {
    return _anzeigeTyp;
  }

  public void setAnzeigeTyp(Composite anzeigeTyp)
  {
    _anzeigeTyp = anzeigeTyp;
  }
  
  public Composite getBlinkTakt()
  {
    return _blinkTakt;
  }

  public void setBlinkTakt(Composite blinkTakt)
  {
    _blinkTakt = blinkTakt;
  }

}
