/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.util;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.Assert;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ListDialog;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelA81Lsa;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypPrg;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelUfdSbaMessStelleNS;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelUfdSensorFBZ;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelUfdSensorSW;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorLVE;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.FahrStreifenModelAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.MessQuerschnittModel;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerParent;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerVertical;
import de.bsvrz.buv.plugin.anlagendarstellung.StreckenStationModel;
import de.bsvrz.buv.plugin.anlagendarstellung.TabelleMqModel;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.DoMitVerbLinieEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.ZusatzElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.DynamischerWechselText.EnumDynamischerWechseltextTextAusrichtung;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm.ListLabelProvider;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungsSpalte;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
/**
 * Helfer Klasse zum Zugriff auf den Datenverteiler
 * 
 * @author Pittner
 * 
 */
public class DaVHelper
{
  /**
   * String-Definitionen
   */
  private final static String _strTypWVZInhalt = "typ.wvzInhalt";

  private final static String _strAtgWVZInhalt = "atg.wvzInhalt";

  private final static String _strAtgAnzeigeEigenschaftIst = "atg.anzeigeEigenschaftIst";

  private final static String _strAtgSswSbaSchaltbildAnzeige = "atg.sswSbaSchaltbildAnzeige";

  private final static String _strAtgTlsGloDeFehler = "atg.tlsGloDeFehler";

  private final static String _strAtgHandeingriffBetriebsart = "atg.sswSbaHandeingriffBetriebsart";

  private final static String _strAtgHelligkeitProzent = "atg.sswSbaHelligkeitProzentErgebnis";
  
  private static final DecimalFormat _df =   new DecimalFormat  ( "0000.00" );

  /**
   * Holt aus dem String des Bildinhaltes {@link AnzeigeInhaltTyp} das entsprechende WVZ-Objekt
   * 
   * @param connection
   *          der DaV-Verbindung
   * @param bildInhalt
   *          der eindeutige String des WVZ-Inhaltes
   * @return das Objekt vom Typ WVZ-Inhalt
   * 
   */
  public static SystemObject holeWVZInhaltVonBildInhalt(ClientDavInterface connection, String bildInhalt)
  {
    if (connection == null || bildInhalt == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + connection + " - " + bildInhalt);

    SystemObjectType type = connection.getDataModel().getType(_strTypWVZInhalt);
    List<SystemObject> elements = type.getElements();
    for (SystemObject so : elements)
    {
      if (!so.getPid().contains("bw.tls."))
      {
        AttributeGroup atg = connection.getDataModel().getAttributeGroup(_strAtgWVZInhalt);
        Data data = so.getConfigurationData(atg);
        String s = data.getItem("Bildinhalt").asTextValue().getText();
        if (s.equals(bildInhalt))
        {
          return so;
        }
      }
    }
    return null;
  }

  /**
   * Holt anhand des AnzeigeTyps des bergebenen Anzeige aus dem String des Bildinhaltes {@link AnzeigeInhaltTyp} das
   * entsprechende WVZ-Objekt
   * 
   * @param connection
   *          der DaV-Verbindung
   * @param anzeige
   *          das SystemObject der Anzeige
   * @param bildInhalt
   *          der eindeutige String des WVZ-Inhaltes
   * @return das Objekt vom Typ WVZ-Inhalt
   * 
   */
  public static SystemObject holeWVZInhaltVonBildInhalt(ClientDavInterface connection, SystemObject anzeige,
      String bildInhalt)
  {
    if (connection == null || anzeige == null || bildInhalt == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + connection + " - " + bildInhalt);

    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.anzeige");
    Data cd = anzeige.getConfigurationData(atg);
    SystemObject anzeigeTyp = cd.getItem("AnzeigeTyp").asReferenceValue().getSystemObject();

    NonMutableSet set = ((ConfigurationObject)anzeigeTyp).getNonMutableSet("WvzInhalt");
    if (set != null)
    {
      List<SystemObject> elements = set.getElements();
      for (SystemObject so : elements)
      {
        atg = connection.getDataModel().getAttributeGroup(_strAtgWVZInhalt);
        cd = so.getConfigurationData(atg);
        String s = cd.getItem("Bildinhalt").asTextValue().getText();
        if (s.equals(bildInhalt))
        {
          return so;
        }
      }
    }

    return null;
  }

  /**
   * Holt aus dem bergebenen SswSbaAq-Objekt den Betriebskilomter
   * 
   * @param connection
   *          die DaV-Verbindung
   * 
   * @return der Wert des Betriebskilomters
   * 
   */
  public static double holeBetriebKilometerVonSbaAq(ClientDavInterface connection, SystemObject sbaAq)
  {
    if (connection == null || sbaAq == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + connection + " - " + sbaAq);
    
    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
    Data cd1 = sbaAq.getConfigurationData(atg);
    double dv = cd1.getItem("Wert").asUnscaledValue().doubleValue();    
    return dv;
    
  }
  
  /**
   * Holt aus dem bergebenen SswSbaAq-Objekt die Richtung
   * 
   * @param connection
   *          die DaV-Verbindung
   * 
   * @return die Bezeichnung des Richtung
   * 
   */
  public static String holeRichtungVonSbaAq(ClientDavInterface connection, SystemObject sbaAq)
  {
    if (connection == null || sbaAq == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + connection + " - " + sbaAq);
    
    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
    Data cd1 = sbaAq.getConfigurationData(atg);
    String s = cd1.getItem("Richtung").asTextValue().getValueText();    
    return s;
    
  }
  /**
   * Holt aus dem bergebenen WVZ-Objekt den eindeutigen String des Bildinhaltes
   * 
   * @param connection
   *          die DaV-Verbindung
   * 
   * @return der Bildinhalt
   * 
   */
  public static String holeBildInhaltVonWVZInhalt(ClientDavInterface connection, SystemObject wvzInhalt)
  {
    if (connection == null || wvzInhalt == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + connection + " - " + wvzInhalt);

    AttributeGroup atg = connection.getDataModel().getAttributeGroup(_strAtgWVZInhalt);
    Data cd = wvzInhalt.getConfigurationData(atg);
    String bi = cd.getItem("Bildinhalt").asTextValue().getValueText();
    return bi;
  }

  /**
   * Daten-Abarbeitungsmethode der Klasse {@code ZustandsDarstellungIstView} oder {@code ZustandsDarstellungSollView}
   * 
   * @param viewer
   *          der Viewer vom Typ {@code GraphicalViewer}
   * @param result
   *          der DaV-Datensatz
   * @param kontext
   *          welche Zustandsdarstellung (Ist- oder Soll) wird verwendet
   */
  public static void verarbeiteDaVDaten(ClientDavInterface connection, final GraphicalViewer viewer, final ResultData result,
      int kontext)
  {
    final AnsichtEditPart contents = (AnsichtEditPart)viewer.getContents();
    Ansicht ansicht = contents.getModel();

    final StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    darstellung.setSpaltenBorder(true);

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> dos = ebene.getDoObjekte();
      for (DoModel dom : dos)
      {

        if (dom != null)
        {
          SetCommand cmd = null;
          EStructuralFeature feature = null;
          if (dom.getSystemObject() != null && dom.getSystemObject().equals(result.getObject()))
          {
            if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
            {
              if (kontext == 0)
              {
                feature = AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                cmd = new SetCommand(dom, feature, result);
                getCommandStack(viewer).execute(cmd);
              }
            }
            else if (dom instanceof FahrStreifenModelAbstract)
            {
              feature = AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__DATA_UPDATE;
              cmd = new SetCommand(dom, feature, result);
              getCommandStack(viewer).execute(cmd);
            }
            else if (dom instanceof MessQuerschnittModel)
            {
              feature = AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__DATA_UPDATE;
              cmd = new SetCommand(dom, feature, result);
              getCommandStack(viewer).execute(cmd);
            }
            else if (dom instanceof DetektorModelTypAbstract)
            {
              feature = AnlagendarstellungPackage.Literals.DETEKTOR_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              cmd = new SetCommand(dom, feature, result);
              getCommandStack(viewer).execute(cmd);
            }
            else if (dom instanceof TabelleMqModel)
            {
              feature = AnlagendarstellungPackage.Literals.TABELLE_MQ_MODEL__DATA_UPDATE;
              cmd = new SetCommand(dom, feature, result);
              getCommandStack(viewer).execute(cmd);
            }
            else if (dom instanceof AnzeigeModelUfdSensorFBZ)
            {
              feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              cmd = new SetCommand(dom, feature, result);
              getCommandStack(viewer).execute(cmd);
            }
            else if (dom instanceof AnzeigeModelUfdSensorSW)
            {
              feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              cmd = new SetCommand(dom, feature, result);
              getCommandStack(viewer).execute(cmd);
            }
            else if (dom instanceof AnzeigeModelUfdSbaMessStelleNS)
            {
              feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              cmd = new SetCommand(dom, feature, result);
              getCommandStack(viewer).execute(cmd);
            }
            else if (dom instanceof AnzeigeModelA81Lsa)
            {
              feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              cmd = new SetCommand(dom, feature, result);
              getCommandStack(viewer).execute(cmd);
            }
          }
          else if (dom.getSystemObject() == null)
          {
            // tue nichts!!!
          }
          else if (!dom.getSystemObject().equals(result.getObject()))
          {
            if (result.getObject().isOfType("typ.deVlt")
                && !result.getDataDescription().getAttributeGroup().getPid().equals(_strAtgTlsGloDeFehler))
            {
              if (dom instanceof StreckenStationModel)
              {
                Fg6Util.setResultData(dom.getSystemObject().getPid(), result);
                if (Fg6Util.getFg6De().contains(result.getObject().getPid()))
                {
                  feature = AnlagendarstellungPackage.Literals.STRECKEN_STATION_MODEL__DATA_UPDATE;
                  AnzeigeInhaltTyp status = Fg6Util.ermittleStatusAusEmpfangsDaten();
                  cmd = new SetCommand(dom, feature, status);
                  getCommandStack(viewer).execute(cmd);
                }
              }
            }
          }
        }
      }
    }

    if (result.getObject().isOfType("typ.anzeige"))
    {
      AnzeigeModelTypAbstract dom = holeAnzeigeModelTypAbstractVonSystemObject(contents, result.getObject(), kontext);

      if (dom instanceof AnzeigeModelTypAbstract)
      {
        AnzeigeModelTypAbstract anzeige = (AnzeigeModelTypAbstract)dom;

        AnzeigeQuerschnittModelTypAbstract aq = holeAnzeigeQuerschnittVonAnzeige(connection, viewer, anzeige);

        if (aq != null && aq.getKontext() == kontext)
        {
          if (kontext == 0
              && result.getDataDescription().getAttributeGroup().getPid().equals(_strAtgAnzeigeEigenschaftIst))
          {
            if (anzeige.getSystemObject() != null && anzeige.getSystemObject().equals(result.getObject()))
            {
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              SetCommand cmd = new SetCommand(anzeige, feature, result);
              getCommandStack(viewer).execute(cmd);
              // break;
            }
          }
        }
      }
    }
    else if (result.getObject().isOfType("typ.sswSbaStreckenBeeinflussungsAnlage")
        && result.getDataDescription().getAttributeGroup().getPid().equals(_strAtgSswSbaSchaltbildAnzeige))
    {
      if (kontext != 0){
        DaVHelper.verarbeiteDaVDatenSswSbaSchaltbild(connection, viewer, result, kontext);
      }
    }
    else if (result.getObject().isOfType("typ.sswSbaAnzeigeQuerschnitt"))
    {
      SystemObject so = result.getObject();
      AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaAnzeigeQuerschnitt");
      Data cd = so.getConfigurationData(atg);
      if (cd != null)
      {
        SystemObject so2 = cd.getItem("AnzeigeQuerschnitt").asReferenceValue().getSystemObject();

        AnzeigeQuerschnittModelTypAbstract dom = holeAnzeigeQuerschnittModelTypAbstractVonSystemObject(contents, so2,
            kontext);

        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)dom;
          if (aq.getKontext() == kontext)
          {
            if (kontext == 1
                && (result.getDataDescription().getAttributeGroup().getPid().equals(_strAtgHandeingriffBetriebsart) || result
                    .getDataDescription().getAttributeGroup().getPid().equals(_strAtgHelligkeitProzent)))
            {
              DaVHelper.verarbeiteDaVDatenSswSbaAq(connection, viewer, result, kontext);
            }
          }
        }
      }
    }
    else if (result.getObject().isOfType("typ.deLve"))
    {
      DetektorModelTypAbstract dom = holeDetektorModelTypAbstractVonSystemObject(contents, result.getObject(), kontext);
      if (dom instanceof DetektorModelTypAbstract)
      {
        if (dom.getSystemObject() != null && dom.getSystemObject().equals(result.getObject()))
        {
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.DETEKTOR_MODEL_TYP_ABSTRACT__DATA_UPDATE;
          SetCommand cmd = new SetCommand(dom, feature, result);
          getCommandStack(viewer).execute(cmd);
          // break;
        }
      }
    }
    else if (result.getObject().isOfType("typ.fahrStreifen"))
    {
      TabelleMqModel dom = holeTabellMqModelVonSystemObject(contents, result.getObject(), kontext);
      if (dom instanceof TabelleMqModel)
      {
        // if (dom.getSystemObject() != null && dom.getSystemObject().equals(result.getObject()))
        {
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.TABELLE_MQ_MODEL__DATA_UPDATE;
          SetCommand cmd = new SetCommand(dom, feature, result);
          getCommandStack(viewer).execute(cmd);
        }
      }
    }
    else if (result.getObject().isOfType("typ.situation"))
    { 
      if (result != null)
        SituationUtil.verarbeiteSituationUpdate(viewer, result);
    }
  }
 
  /**
   * Holt das Darstellunges-Objekt des AnzeigeQuerschnitts der Anzeige anhand des bergebenen Viewers und des
   * Model-Objekts
   * 
   * @param connection
   *          die Dav-Verbindung
   * @param viewer
   *          der grafischeViewer
   * @param doModel
   *          das Darstellungs-Objekt
   * @return
   */
  public static AnzeigeQuerschnittModelTypAbstract holeAnzeigeQuerschnittVonAnzeige(ClientDavInterface connection,
      GraphicalViewer viewer, DoModel doModel)
  {

    AnzeigeQuerschnittModelTypAbstract re = null;

    EditPart editPart = viewer.getContents();
    if (editPart instanceof AnsichtEditPart)
    {
      AnsichtEditPart contents = (AnsichtEditPart)editPart;
      Ansicht ansicht = contents.getModel();
      Darstellung darstellung = (Darstellung)ansicht.getDarstellung();
      if (darstellung instanceof StilisierteDarstellung)
      {
        ((StilisierteDarstellung)darstellung).setSpaltenBorder(true);
      }

      EList<Ebene> ebenen = darstellung.getEbenen();
      for (Ebene ebene : ebenen)
      {
        EList<DoModel> dos = ebene.getDoObjekte();
        for (DoModel dom : dos)
        {
          if (dom != null)
          {
            if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
            {
              AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)dom;
              EList<AnzeigeModelTypAbstract> komponenten = aq.getKomponenten();
              if (komponenten.contains(doModel))
              {
                {
                  re = aq;
                }
              }
            }
          }
        }
      }
    }
    return re;
  }

  /**
   * Holt die SystemObjekte der aller Darstellungsobjekte aus dem Inhalt eines {@code GraphicalViewer}. Wird zur
   * Anmeldung der Daten bentigt.
   * 
   * @param graphicalViewer
   *          der {@code GraphicalViewer}
   * @return Liste mit Objekten vom Typ {@code SystemObject}
   */
  public static List<SystemObject> holeSystemObjekteAusDarstellung(GraphicalViewer graphicalViewer)
  {
    AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    darstellung.setSpaltenBorder(true);

    List<SystemObject> list = new ArrayList<SystemObject>();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> dos = ebene.getDoObjekte();
      for (DoModel dom : dos)
      {
        if (dom == null)
        {
          System.err.println("dom ist null: " + dom);
        }

        if (dom.getSystemObject() != null && dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          AnzeigeQuerschnittModelTypAbstract model = (AnzeigeQuerschnittModelTypAbstract)dom;
          list.add(model.getSystemObject());

          EList<AnzeigeModelTypAbstract> komponenten = model.getKomponenten();
          for (AnzeigeModelTypAbstract komp : komponenten)
          {
            list.add(komp.getSystemObject());
          }
        }
        if (dom instanceof AnzeigeModelTypAbstract)
        {
          AnzeigeModelTypAbstract ae = (AnzeigeModelTypAbstract)dom;
          list.add(ae.getSystemObject());
        }
        if (dom instanceof FahrStreifenModelAbstract)
        {
          FahrStreifenModelAbstract model = (FahrStreifenModelAbstract)dom;
          list.add(dom.getSystemObject());

          EList<DetektorLVE> komponenten = model.getKomponenten();
          for (DetektorLVE komp : komponenten)
          {
            list.add(komp.getSystemObject());
          }
        }
        if (dom instanceof DetektorModelTypAbstract)
        {
          DetektorModelTypAbstract dt = (DetektorModelTypAbstract)dom;
          list.add(dt.getSystemObject());
        }
        else if (dom instanceof AnzeigeModelTypPrg)
        {
          AnzeigeModelTypAbstract ae = (AnzeigeModelTypPrg)dom;
          list.add(ae.getSystemObject());
        }
        else if (dom instanceof StreckenStationModel)
        {
          StreckenStationModel sm = (StreckenStationModel)dom;
          SystemObject so = sm.getSystemObject();
          if (so != null)
          {
            list.add(so);
          }
        }
        else if (dom instanceof TabelleMqModel)
        {
          TabelleMqModel ae = (TabelleMqModel)dom;
          list.add(ae.getSystemObject());
        }
        else if (dom instanceof MessQuerschnittModel)
        {
          MessQuerschnittModel mq = (MessQuerschnittModel)dom;
          list.add(mq.getSystemObject());
        }
      }
    }
    return list;
  }

  /**
   * Holt die sortierte Liste der AnzeigeQuerschnitte aus dem bergebenen Kilometer-Bereich
   * 
   * @param secId
   *          die Kennung der Fahrtrichtung (NO oder SW)
   * @param absteigend
   *          Schalter, ob die Liste absteigend sortiert werden soll
   * @param von
   *          Start-Km
   * @param bis
   *          Ende-Km
   * @return sortierte Liste der AnzeigeQuerschnitte
   * 
   */
  public static SortedMap<String, SystemObject> holeListeKilometrierungAq(String secId, boolean absteigend, double von,
      double bis)
  {
    // TreeMap<Double,SystemObject> treeMap = new TreeMap<Double, SystemObject>();
    TreeMap<String, SystemObject> treeMap = new TreeMap<String, SystemObject>();
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();
      AttributeGroup atg = dataModel.getAttributeGroup("atg.sswSbaBetriebsKilometer");
      AttributeGroup atg2 = dataModel.getAttributeGroup("atg.sswSbaAnzeigeQuerschnitt");
      AttributeGroup atg3 = dataModel.getAttributeGroup("atg.sswSbaAnzeige");

      for (SystemObject so : dataModel.getType("typ.sswSbaAnzeigeQuerschnitt").getElements())
      {
        if (so.isValid())
        {
          Data cd = so.getConfigurationData(atg);
          Data cd2 = so.getConfigurationData(atg2);
          if (cd2 != null)
          {
            SystemObject aq = cd2.getItem("AnzeigeQuerschnitt").asReferenceValue().getSystemObject();
            NonMutableSet set = ((ConfigurationObject)aq).getNonMutableSet("Anzeigen");

            // ermittelt ob AQ auf Hauptstrecke liegt
            boolean isAqAufHauptstrecke = false;
            if (set != null)
            {
              List<SystemObject> elements = set.getElements();
              for (SystemObject so2 : elements)
              {
                for (SystemObject so3 : dataModel.getType("typ.sswSbaAnzeige").getElements())
                {
                  if (so3.isValid())
                  {
                    Data cd3 = so3.getConfigurationData(atg3);
                    if (cd3 != null)
                    {
                      SystemObject so4 = cd3.getItem("Anzeige").asReferenceValue().getSystemObject();
                      if (so4 != null && so4.equals(so2))
                      {
                        Array array = cd3.getItem("Lage").asArray();
                        if (array.getLength() > 0)
                        {
                          isAqAufHauptstrecke = true;
                          break;
                        }
                      }
                    }
                  }
                }
              }
            }
            if (isAqAufHauptstrecke)
            {
              // entspricht die Richtung der 2nd View-Id?
              String richtung = cd.getItem("Richtung").asTextValue().getText();
              double value = cd.getItem("Wert").asUnscaledValue().doubleValue();
              
//              System.out.println("DaVHelper.holeListeKilometrierungAq(): " + value);
              
              if (value > 0)
                value /= 1000.0;
              if (secId != null && secId.equals(richtung))
              {

                boolean gueltig = false;

                // ermittelt ob der AQ in definierten Bereich der Strecke liegt
                if (von < bis)
                {
                  if (value >= von && value <= bis)
                  {
                    gueltig = true;
                  }
                }
                else if (bis < von)
                {
                  if (value <= von && value >= bis)
                  {
                    gueltig = true;
                  }
                }
                if (gueltig)
                {
                  // treeMap.put(value, so);

                  // Workaround wg. Konfigurationsfehler im SBA AQ
                  if (so.getPid().equals("sba.aq.A8.6130.HFB.W"))
                  {
                    value = 208.43;
                  }
                  
//                  double d = 12345.67890;
                  

//                  String key = "" + value + "_" + so.getPid();
                  
                  String str = _df.format(value);
                  
                  if (str.contains(","))
                    str = str.replace(",", ".");
                  
                  String key = "" + str + "_" + so.getPid();
                  treeMap.put(key, so);

                }

              }
            }
          }
        }
      }
      // Wenn absteigend, dann Liste absteigend sortieren
      if (absteigend)
        return treeMap.descendingMap();
    }
    return treeMap;
  }
  
  /**
   * Holt die sortierte Liste der gepixelten AnzeigeQuerschnitte aus der Trichterung
   * 
   * @param secId
   *          die Kennung der Fahrtrichtung (NO oder SW)
   * @param absteigend
   *          Schalter, ob die Liste absteigend sortiert werden soll
   * @param von
   *          Start-Km
   * @param bis
   *          Ende-Km
   * @return sortierte Liste der AnzeigeQuerschnitte
   * 
   */
  public static SortedMap<Integer,DoMitVerbLinieEditPart> holeAnzeigeQuerschnittModelTypAbstractVonKontext(
      GraphicalViewer viewer, int kontext, boolean absteigend)
    {
   
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    
    List<AnzeigeQuerschnittModelTypAbstract> tmp = new ArrayList<AnzeigeQuerschnittModelTypAbstract>();
    
    if (connection != null)
    {
      EditPart contents = viewer.getContents();
      if (contents instanceof AnsichtEditPart)
      {
        AnsichtEditPart aep = (AnsichtEditPart)contents;
        Ansicht model = aep.getModel();
        Darstellung tmpDarstellung = model.getDarstellung();
        if (tmpDarstellung instanceof StilisierteDarstellung)
        {
          StilisierteDarstellung darstellung = (StilisierteDarstellung)tmpDarstellung;
          EList<DarstellungsSpalte> spalten = darstellung.getSpalten();
          for (DarstellungsSpalte spalte : spalten)
          {
            EList<DoModel> doObjekte = spalte.getDoObjekte();
            for (DoModel doModel : doObjekte)
            {
              if (doModel instanceof AnzeigeQuerschnittModelTypAbstract)
              {
                AnzeigeQuerschnittModelTypAbstract aqModel = (AnzeigeQuerschnittModelTypAbstract)doModel;
                if (aqModel.getKontext() == kontext){
                  
                  EList<AnzeigeModelTypAbstract> komponenten = aqModel.getKomponenten();
                  boolean gefunden = false;
                  for (AnzeigeModelTypAbstract anzModel : komponenten)
                  {
                    Object obj = anzModel.getDataUpdate();
                    if (obj instanceof AnzeigeInhaltTyp)
                    {
                      AnzeigeInhaltTyp inhalt = (AnzeigeInhaltTyp)obj;
                      if (!inhalt.getType().equals(AnzeigeInhaltTyp.BELIEBIG_BI.getType()) && 
                          !inhalt.getType().equals(AnzeigeInhaltTyp.BELIEBIG_BI_TEXT.getType()))
                      {
                        gefunden = true;
                        break;
                      }
                    }
                    else if (obj instanceof VisuAnzeigeKapsel)
                    {
                      gefunden = true;
                      break;
                    }
                    else
                    {
                      System.out.println("else: " + obj);
                    }
                  }
                  
                  if (gefunden)
                    tmp.add(aqModel);
                }
              }
            }
          }        
        }        
      }
    }
    
    Map reg = viewer.getEditPartRegistry();
    TreeMap<Integer,DoMitVerbLinieEditPart> treeMap = new TreeMap<Integer, DoMitVerbLinieEditPart>();
    
    for (AnzeigeQuerschnittModelTypAbstract aqModel : tmp)
    {
      if (reg.containsKey(aqModel)){
        Object object = reg.get(aqModel);
        if (object instanceof DoMitVerbLinieEditPart)
        {
          DoMitVerbLinieEditPart epart = (DoMitVerbLinieEditPart)object;
          IFigure figure = epart.getFigure();
          treeMap.put(figure.getBounds().y, epart);         
        }        
      }
    }
    
    // Wenn absteigend, dann Liste andersherum sortieren
    if (absteigend)
      return treeMap.descendingMap();
    
    
    return treeMap;
  }
  
  /**
   * Holt die sortierte Liste der aller AnzeigeQuerschnitte mit der entsprechenden Fahrtrichtung
   * 
   * @param secId
   *          die Kennung der Fahrtrichtung (NO oder SW)
   * @param absteigend
   *          Schalter, ob die Liste absteigend sortiert werden soll
   * @param von
   *          Start-Km
   * @param bis
   *          Ende-Km
   * @return sortierte Liste der AnzeigeQuerschnitte
   * 
   */
  public static SortedMap<Integer,SystemObject> holeAlleAnzeigeQuerschnittModelTypAbstractVonFahrtRichtung(
      GraphicalViewer viewer, int fahrtRichtung, boolean absteigend)
      {
    
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    
    List<AnzeigeQuerschnittModelTypAbstract> tmp = new ArrayList<AnzeigeQuerschnittModelTypAbstract>();
    
    if (connection != null)
    {
      EditPart contents = viewer.getContents();
      if (contents instanceof AnsichtEditPart)
      {
        AnsichtEditPart aep = (AnsichtEditPart)contents;
        Ansicht model = aep.getModel();
        Darstellung tmpDarstellung = model.getDarstellung();
        if (tmpDarstellung instanceof StilisierteDarstellung)
        {
          StilisierteDarstellung darstellung = (StilisierteDarstellung)tmpDarstellung;
          EList<DarstellungsSpalte> spalten = darstellung.getSpalten();
          for (DarstellungsSpalte spalte : spalten)
          {
            EList<DoModel> doObjekte = spalte.getDoObjekte();
            for (DoModel doModel : doObjekte)
            {
              if (doModel instanceof AnzeigeQuerschnittModelTypAbstract)
              {
                AnzeigeQuerschnittModelTypAbstract aqModel = (AnzeigeQuerschnittModelTypAbstract)doModel;
                
                
                if (aqModel.getFahrtrichtung() == fahrtRichtung){
                    tmp.add(aqModel);
                }
              }
            }
          }        
        }        
      }
    }
    
    Map reg = viewer.getEditPartRegistry();
    TreeMap<Integer,SystemObject> treeMap = new TreeMap<Integer, SystemObject>();
    
    for (AnzeigeQuerschnittModelTypAbstract aqModel : tmp)
    {
      if (reg.containsKey(aqModel))
      {
        Object object = reg.get(aqModel);
        if (object instanceof DoMitVerbLinieEditPart)
        {
          DoMitVerbLinieEditPart epart = (DoMitVerbLinieEditPart)object;
          IFigure figure = epart.getFigure();
          
          int yAq = figure.getBounds().y;
          
          if (treeMap.containsKey(yAq))
              yAq += 1;
            
          treeMap.put(yAq, aqModel.getSystemObject());         
        }        
      }
    }
    
    // Wenn absteigend, dann Liste andersherum sortieren
    if (absteigend)
      return treeMap.descendingMap();
    
    
    return treeMap;
      }

  /**
   * Gibt zurueck ob ein ProgrammTyp eines SOnderprogrammes eine Spursperrung beinhaltet
   * 
   * @param typ
   *          die Bezeichnung des ProgrammTyps
   * 
   * @return true, wenn eine Spursperrung vorliegt
   */
  public static boolean isProgrammTypSpurSperrung(String typ)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if (connection != null)
    {
      AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaProgrammTyp");
      Aspect asp = connection.getDataModel().getAspect("asp.parameterSoll");

      IPreferenceStore store = Activator.getDefault().getPreferenceStore();
      String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
      if (pidSba != null && pidSba.length() > 0)
      {
        SystemObject object = connection.getDataModel().getObject(pidSba);
        if (object != null)
        {
          ResultData result1 = connection.getData(object, new DataDescription(atg, asp), 1500);
          if (result1 != null && result1.hasData())
          {
            Data dta = result1.getData();
            Array array2 = dta.getItem("ProgrammTypen").asArray();
            for (int i = 0; i < array2.getLength(); i++)
            {
              String text = array2.getItem(i).getItem("Bezeichnung").asTextValue().getText();
              if (text != null)
              {
                if (typ.equals(text))
                {
                  boolean b = array2.getItem(i).getItem("Spursperrung").asTextValue().getText().equals("Ja");
                  return b;
                }
              }
            }
          }
        }
      }
    }
    return false;
  }

  /**
   * Holt das Darstellungs-Objekt eines Fahrstreifens
   * 
   * @param contents
   *          die Ansicht
   * @param objekt
   *          das DaV-Objekt
   * @return das Darstellungs-Objekt des Fahrstreifens
   */
  public static FahrStreifenModelAbstract holeFahrStreifenModelTypVonSystemObject(AnsichtEditPart contents,
      SystemObject objekt)
  {
    if (contents == null || objekt == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + objekt);

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> doObjekte = ebene.getDoObjekte();
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof FahrStreifenModelAbstract)
        {
          FahrStreifenModelAbstract fs = (FahrStreifenModelAbstract)dom;
          SystemObject so = fs.getSystemObject();
          if (so != null)
          {
            if (so.equals(objekt))
            {
              return fs;
            }
          }
        }
      }
    }
    return null;
  }

  /**
   * Holt das Darstellungs-Objekt einer Anzeige
   * 
   * @param contents
   *          die Ansicht
   * @param objekt
   *          das DaV-Objekt
   * @return das Darstellungs-Objekt einer Anzeige
   */
  public static AnzeigeModelTypAbstract holeAnzeigeModelTypAbstractVonSystemObject(AnsichtEditPart contents,
      SystemObject objekt, int kontext)
  {
    if (contents == null || objekt == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + objekt);

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> doObjekte = ebene.getDoObjekte();
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)dom;
          int aqKontext = aq.getKontext();
          if (kontext == aqKontext)
          {
            EList<AnzeigeModelTypAbstract> anzeigen = aq.getKomponenten();
            for (AnzeigeModelTypAbstract anzeige : anzeigen)
            {
              SystemObject so = anzeige.getSystemObject();
              if (so != null)
              {
                if (so.equals(objekt))
                {
                  return anzeige;
                }
              }
            }
          }
        }
      }
    }
    return null;
  }

  /**
   * Holt das Darstellungs-Objekt eines Verkehrsdaten-Detektors
   * 
   * @param contents
   *          die Ansicht
   * @param objekt
   *          das DaV-Objekt
   * @return das Darstellungs-Objekt des Detektors
   */
  public static DetektorModelTypAbstract holeDetektorModelTypAbstractVonSystemObject(AnsichtEditPart contents,
      SystemObject objekt, int kontext)
  {
    if (contents == null || objekt == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + objekt);

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> doObjekte = ebene.getDoObjekte();
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof FahrStreifenModelAbstract)
        {
          FahrStreifenModelAbstract fs = (FahrStreifenModelAbstract)dom;
          // int aqKontext = fs.getKontext();
          // if (kontext == aqKontext)
          {

            EList<DetektorLVE> des = fs.getKomponenten();
            for (DetektorLVE de : des)
            {
              SystemObject so = de.getSystemObject();
              if (so != null)
              {
                if (so.equals(objekt))
                {
                  return de;
                }
              }
            }
          }
        }
      }
    }
    return null;
  }

  /**
   * Holt das Darstellungs-Objekt eines AnzeigeQuerschnittes
   * 
   * @param contents
   *          die Ansicht
   * @param objekt
   *          das DaV-Objekt
   * @return das Darstellungs-Objekt des AnzeigeQuerschnittes
   */
  public static AnzeigeQuerschnittModelTypAbstract holeAnzeigeQuerschnittModelTypAbstractVonSystemObject(
      AnsichtEditPart contents, SystemObject objekt, int kontext)
  {
    if (contents == null || objekt == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + objekt);

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    List<SystemObject> list = new ArrayList<SystemObject>();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {

      EList<DoModel> doObjekte = ebene.getDoObjekte();
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {

          AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)dom;
          int aqKontext = aq.getKontext();
          if (kontext == aqKontext)
          {
            SystemObject so = aq.getSystemObject();
            if (so != null)
            {
              if (so.equals(objekt))
              {
                return aq;
              }
            }
          }
        }
      }
    }
    return null;
  }

  /**
   * Holt das Darstellungs-Objekt einer Messquerschnitt-Tabelle
   * 
   * @param contents
   *          die Ansicht
   * @param objekt
   *          das DaV-Objekt
   * @return das Darstellungs-Objekt der Messquerschnitt-Tabelle
   */
  public static TabelleMqModel holeTabellMqModelVonSystemObject(AnsichtEditPart contents, SystemObject objekt,
      int kontext)
  {
    if (contents == null || objekt == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + objekt);

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {

      EList<DoModel> doObjekte = ebene.getDoObjekte();
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof TabelleMqModel)
        {

          TabelleMqModel mq = (TabelleMqModel)dom;

          int aqKontext = mq.getKontext();
          // if (kontext == aqKontext)
          {
            SystemObject so = mq.getSystemObject();
            if (so != null && so.isOfType("typ.messQuerschnitt"))
            {
              NonMutableSet set = ((ConfigurationObject)so).getNonMutableSet("FahrStreifen");
              if (set != null)
              {
                List<SystemObject> elements = set.getElements();
                if (elements != null && elements.contains(objekt))
                {
                  return mq;
                }
              }
            }
          }
        }
      }
    }
    return null;
  }
  
  /**
   * Holt das Darstellungs-Objekt einer Messquerschnitt-Tabelle
   * 
   * @param contents
   *          die Ansicht
   * @param objekt
   *          das DaV-Objekt
   * @return das Darstellungs-Objekt der Messquerschnitt-Tabelle
   */
  public static PickerParent holePickerParentVonSystemObject(AnsichtEditPart contents, SystemObject objekt,
      AttTmcRichtung richtung)
  {
    if (contents == null || objekt == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + objekt);
    
    Ansicht ansicht = contents.getModel();
    
    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    
    EList<Ebene> ebenen = darstellung.getEbenen();
    PickerParent re = null;
    for (Ebene e : ebenen)
    {
      EList<DoModel> doObjekte = e.getDoObjekte();

      for (DoModel dom : doObjekte)
      {
        if (dom instanceof PickerParent)
        {
          PickerParent pv = (PickerParent)dom;
          
          if (pv.getObjektTyp().equals(objekt.getType().getPid())){
            int fr = pv.getFahrtrichtung();
//            System.out.println("DaVHelper.holePickerParentVonSystemObject(): " + richtung + " : " + fr + " -> " + pv);
            if (richtung.equals(AttTmcRichtung.ZUSTAND_1_POSITIV) && fr == 0){
//              System.out.println("1");
              return pv; 
            }
            else if (richtung.equals(AttTmcRichtung.ZUSTAND_1N_NEGATIV) && fr == 1){
//              System.out.println("2");
              return pv; 
            }
            else{
//              System.out.println("DaVHelper.holePickerParentVonSystemObject(else!!!)");
              //re = null;
              //return null;
            }            
          }                    
        }          
      }      
    }
    return re;
  }
  
  /**
   * Holt das Darstellungs-Objekt einer Messquerschnitt-Tabelle
   * 
   * @param contents
   *          die Ansicht
   * @param objekt
   *          das DaV-Objekt
   * @return das Darstellungs-Objekt der Messquerschnitt-Tabelle
   */
  public static PickerParent holePickerParentVonSystemObject2(AnsichtEditPart contents, SystemObject objekt)
  {
    if (contents == null || objekt == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + objekt);
    
    Ansicht ansicht = contents.getModel();
    
    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    
    
//    PickerVertical re = null;
    
    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene e : ebenen)
    {
      EList<DoModel> doObjekte = e.getDoObjekte();
      
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof PickerParent)
        {
          PickerParent pp = (PickerParent)dom;
          EList<PickerVertical> komponenten = pp.getKomponenten();
          for (PickerVertical pv : komponenten)
          {
            if (pv.getSystemObject().getPid().equals(objekt.getPid()))
              return pp;
          }
        }          
      }      
    }
    return null;
  }
  
  /**
   * Holt das Darstellungs-Objekt einer Messquerschnitt-Tabelle
   * 
   * @param contents
   *          die Ansicht
   * @param objekt
   *          das DaV-Objekt
   * @return das Darstellungs-Objekt der Messquerschnitt-Tabelle
   */
  public static PickerParent holePickerParentVonPickerVertical(AnsichtEditPart contents, PickerVertical pickerVertical)
  {
    if (contents == null || pickerVertical == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + pickerVertical);
    
    Ansicht ansicht = contents.getModel();
    
    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    
    
    PickerVertical re = null;
    
    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene e : ebenen)
    {
      EList<DoModel> doObjekte = e.getDoObjekte();
      
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof PickerParent)
        {
          PickerParent pp = (PickerParent)dom;
          EList<PickerVertical> komponenten = pp.getKomponenten();
          for (PickerVertical pv : komponenten)
          {
            if (pv.getSystemObject().getPid().equals(pickerVertical.getSystemObject().getPid()))
              return pp;
          }
        }          
      }      
    }
    return null;
  }

  /**
   * Verarbeitet die Schaltbildaten der Attributgruppe atg.sswSbaSchaltbild. Die Attributgruppe beinhaltet vielfltige
   * Daten, dies sind u.a. die Inhalte der Anzeigen sowie deren Status. Es wird das EMF-Model der Darstellung
   * benachrichtigt, die Abarbeitung der Daten erfolgt an anderer Stelle.
   * 
   * 
   * @param connection
   *          die DaV-Verbindung
   * @param graphicalViewer
   *          der GEF-Viewer
   * @param result
   *          der DaV-Datensatz der Attributgruppe
   * @param kontext
   *          der Kontext der Darstellung (Ist-, Soll-, Sonder- oder Parameter-Darstellung)
   */
  public static void verarbeiteDaVDatenSswSbaSchaltbild(ClientDavInterface connection, GraphicalViewer graphicalViewer,
      ResultData result, int kontext)
  {
    if (connection == null || result == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + connection + " - " + result);

    Data data = result.getData();

    if (data != null)
    {
      Array array = data.getItem("Anzeigen").asArray();
      if (array.getLength() > 0)
      {
        List<AnzeigeQuerschnittModelTypAbstract> listAq = new ArrayList<AnzeigeQuerschnittModelTypAbstract>();
        for (int i = 0; i < array.getLength(); i++)
        {
          Data item = array.getItem(i);
          SystemObject so = item.getItem("Anzeige").asReferenceValue().getSystemObject();
          SystemObject wvzInhaltObj = item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue()
              .getSystemObject();
          Data dynWechselTextData = item.getItem("Schaltbild").getItem("DynamischerWechseltext");

          String blinken = item.getItem("Schaltbild").getItem("Blinken").asTextValue().getValueText();
          String blinkTakt = item.getItem("Schaltbild").getItem("Blinktakt").asTextValue().getValueText();
          
          
          if (connection != null)
          {
            if (wvzInhaltObj != null)
            {
              String bi = DaVHelper.holeBildInhaltVonWVZInhalt(connection, wvzInhaltObj);
              AnzeigeInhaltTyp inhaltTyp = AnzeigeInhaltTyp.setBildInhalt(bi, result.getDataTime());

              AnzeigeModelTypAbstract doModel = DaVHelper.holeAnzeigeModelTypAbstractVonSystemObject(
                  (AnsichtEditPart)graphicalViewer.getContents(), so, kontext);

              if (doModel != null && inhaltTyp != null)
              {
                Object oldFeature = doModel
                    .eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
                if (oldFeature == null || (oldFeature != null && !oldFeature.equals(inhaltTyp)))
                {
                  if (doModel instanceof AnzeigeModelTypAbstract)
                  {
                    EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                    final SetCommand cmd = new SetCommand(doModel, feature, inhaltTyp);
                    getCommandStack(graphicalViewer).execute(cmd);
                    AnzeigeQuerschnittModelTypAbstract aq = holeAnzeigeQuerschnittVonAnzeige(connection, graphicalViewer,
                        doModel);
                    if (!listAq.contains(aq))
                      listAq.add(aq);
                  }
                }
                else 
                {
                  Object oldFeature2 = doModel
                  .eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BLINKEN); 
                   
                  String blinkenAlt = "";
                  if (oldFeature2 != null)
                    blinkenAlt =  oldFeature2.equals(1) ? "Ja" : "Nein";
                  if (oldFeature2 == null || !blinken.equals(blinkenAlt)){                                              
                    if (doModel instanceof AnzeigeModelTypAbstract)
                    {
                      EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BLINKEN;
                      final SetCommand cmd = new SetCommand(doModel, feature, blinken.equals("Ja") ? 1 : 0);
                      getCommandStack(graphicalViewer).execute(cmd);
                      AnzeigeQuerschnittModelTypAbstract aq = holeAnzeigeQuerschnittVonAnzeige(connection, graphicalViewer,
                        doModel);
                      if (!listAq.contains(aq))
                        listAq.add(aq);                      
                    }                        
                  }                     
                }
              }
            }
            else if (dynWechselTextData != null)
            {
              VisuAnzeigeKapsel kapsel = DaVHelper.holeDynamischerWechselTextAusData(dynWechselTextData);

              AnzeigeModelTypAbstract doModel = DaVHelper.holeAnzeigeModelTypAbstractVonSystemObject(
                  (AnsichtEditPart)graphicalViewer.getContents(), so, kontext);
              if (doModel != null && kapsel != null)
              {
                Object oldFeature = doModel
                    .eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);

                if (oldFeature == null || (oldFeature != null && !oldFeature.equals(kapsel)))
                {
                  if (doModel instanceof AnzeigeModelTypAbstract)
                  {
                    EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                    SetCommand cmd = new SetCommand(doModel, feature, kapsel);
                    if (kapsel.isInaktiv())
                    {
                      cmd = new SetCommand(doModel, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                    }
                    getCommandStack(graphicalViewer).execute(cmd);
                    
                    AnzeigeQuerschnittModelTypAbstract aq = holeAnzeigeQuerschnittVonAnzeige(connection, graphicalViewer,
                        doModel);
                    if (!listAq.contains(aq))
                      listAq.add(aq);
                  }
                }
              }
            }
          }
        }

        for (AnzeigeQuerschnittModelTypAbstract aq : listAq)
        {
          if (aq.getKontext() > 0)
          {
            EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__DATA_UPDATE;
            final SetCommand cmd = new SetCommand(aq, feature, result);
            getCommandStack(graphicalViewer).execute(cmd);
          }
        }
      }
    }
  }

  /**
   * Verarbeitet die Schaltbildaten der Attributgruppe atg.sswSbaSchaltbilds fr einen AnzeigeQuerschnitt. Die
   * Attributgruppe beinhaltet vielfltige Daten, dies sind u.a. die Inhalte der Anzeigen sowie deren Status. Es wird
   * das EMF-Model der Darstellung benachrichtigt, die Abarbeitung der Daten erfolgt an anderer Stelle.
   * 
   * 
   * @param connection
   *          die DaV-Verbindung
   * @param aq
   *          das Darstellungs-Objekt des AnzeigeQuerschnitt
   * @param graphicalViewer
   *          der GEF-Viewer
   * @param result
   *          der DaV-Datensatz der Attributgruppe
   * @param kontext
   *          der Kontext der Darstellung (Ist-, Soll-, Sonder- oder Parameter-Darstellung)
   */
  public static void verarbeiteDaVDatenSswSbaSchaltbildAq(ClientDavInterface connection,
      GraphicalViewer graphicalViewer, AnzeigeQuerschnittModelTypAbstract aq, ResultData result, int kontext)
  {
    if (connection == null || result == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + connection + " - " + result);

    Data data = result.getData();

    if (data != null)
    {
      Array array = data.getItem("Anzeigen").asArray();
      if (array.getLength() > 0)
      {
        for (int i = 0; i < array.getLength(); i++)
        {
          Data item = array.getItem(i);
          SystemObject so = item.getItem("Anzeige").asReferenceValue().getSystemObject();
          SystemObject wvzInhaltObj = item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue()
              .getSystemObject();
          if (connection != null)
          {
            String bi = DaVHelper.holeBildInhaltVonWVZInhalt(connection, wvzInhaltObj);
            AnzeigeInhaltTyp inhaltTyp = AnzeigeInhaltTyp.setBildInhalt(bi, result.getDataTime());

            EList<AnzeigeModelTypAbstract> komponenten = aq.getKomponenten();
            for (AnzeigeModelTypAbstract doModel : komponenten)
            {
              if (doModel != null && inhaltTyp != null)
              {
                if (doModel instanceof AnzeigeModelTypAbstract)
                {
                  if (doModel.getSystemObject().equals(so))
                  {
                    EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                    final SetCommand cmd = new SetCommand(doModel, feature, inhaltTyp);
                    getCommandStack(graphicalViewer).execute(cmd);
                  }
                }
              }
            }
          }
        }
        if (kontext < 4)
        {
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__DATA_UPDATE;
          final SetCommand cmd = new SetCommand(aq, feature, result);
          getCommandStack(graphicalViewer).execute(cmd);
        }
      }
    }
  }

  /**
   * Verarbeitet die DaV-Daten der SSW-SBA-AnzeigeQuerschnitt. Es wird das EMF-Model der Darstellung benachrichtigt, die
   * Abarbeitung der Daten erfolgt an anderer Stelle.
   * 
   * @param connection
   *          die DaV-Verbindung
   * @param graphicalViewer
   *          der GEF-Viewer
   * @param result
   *          der DaV-Datensatz der Attributgruppe
   * @param kontext
   *          der Kontext der Darstellung (Ist-, Soll-, Sonder- oder Parameter-Darstellung)
   */
  public static void verarbeiteDaVDatenSswSbaAq(ClientDavInterface connection, GraphicalViewer graphicalViewer,
      ResultData result, int kontext)
  {
    if (connection == null || result == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + connection + " - " + result);

    // Data data = result.getData();
    //
    // if (data != null)
    {

      SystemObject so = result.getObject();
      AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaAnzeigeQuerschnitt");
      Data cd = so.getConfigurationData(atg);
      SystemObject so2 = cd.getItem("AnzeigeQuerschnitt").asReferenceValue().getSystemObject();

      AnzeigeQuerschnittModelTypAbstract doModel = holeAnzeigeQuerschnittModelTypAbstractVonSystemObject(
          (AnsichtEditPart)graphicalViewer.getContents(), so2, kontext);

      if (doModel != null)
      {
        if (doModel instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          if (doModel.getSystemObject().equals(so2))
          {
            EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_QUERSCHNITT_MODEL_TYP_ABSTRACT__DATA_UPDATE;
            final SetCommand cmd = new SetCommand(doModel, feature, result);
            getCommandStack(graphicalViewer).execute(cmd);
          }
        }
      }
    }
  }

  /**
   * @param contents
   * @param objekt
   * @return
   */
  public static DetektorModelTypAbstract holeDetektorModelTypAbstractVonSystemObject(AnsichtEditPart contents,
      SystemObject objekt)
  {
    if (contents == null || objekt == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + objekt);

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    List<SystemObject> list = new ArrayList<SystemObject>();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> doObjekte = ebene.getDoObjekte();
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof FahrStreifenModelAbstract)
        {
          FahrStreifenModelAbstract fs = (FahrStreifenModelAbstract)dom;
          EList<DetektorLVE> komponenten = fs.getKomponenten();
          for (DetektorLVE de : komponenten)
          {
            SystemObject so = de.getSystemObject();
            if (so != null)
            {
              if (so.equals(objekt))
              {
                return de;
              }
            }
          }
        }
      }
    }
    return null;
  }

  /**
   * @param result
   * @return
   */
  public static AnzeigeInhaltTyp holeAnzeigeInhaltAusDaVData(ResultData result)
  {
    AnzeigeInhaltTyp re = null;
    if (result != null)
    {
      AttributeGroup atg = result.getDataDescription().getAttributeGroup();
      if (atg.getPid().equals("atg.anzeigeEigenschaftIst") || atg.getPid().equals("atg.anzeigeEigenschaftSoll"))
      {
        Data data = result.getData();
        if (data != null)
        {
          SystemObject so = data.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();
          ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
          AttributeGroup atg2 = dav.getDataModel().getAttributeGroup("atg.wvzInhalt");

          if (so != null)
          {
            Data data2 = so.getConfigurationData(atg2);
            final String inhalt = data2.getItem("Bildinhalt").asTextValue().getValueText();

            re = AnzeigeInhaltTyp.setBildInhalt(inhalt, result.getDataTime());
          }
          else
          {
            re = AnzeigeInhaltTyp.GESTOERT_BI;
          }
        }
        else
        {
          re = AnzeigeInhaltTyp.GESTOERT_BI;
        }
      }
    }
    return re;
  }

  /**
   * @param result
   * @return
   */
  public static VisuAnzeigeKapsel holeDynamischerWechselTextAusDaVData(ResultData result)
  {
    VisuAnzeigeKapsel re = null;
    if (result != null)
    {
      AttributeGroup atg = result.getDataDescription().getAttributeGroup();
      if (atg.getPid().equals("atg.anzeigeEigenschaftIst") || atg.getPid().equals("atg.anzeigeEigenschaftSoll"))
      {
        Data data = result.getData();
        if (data != null)
        {
          Data item = data.getItem("Eigenschaft").getItem("DynamischerWechseltext");
          SystemObject font = item.getItem("Font").asReferenceValue().getSystemObject();

          String strAusrichtung = item.getItem("TextAusrichtung").asTextValue().getValueText();
          EnumDynamischerWechseltextTextAusrichtung ausrichtung = EnumDynamischerWechseltextTextAusrichtung
              .valueOf(strAusrichtung);

          SystemObject[] textDefinition = item.getItem("Textdefinition").asReferenceArray().getSystemObjectArray();

          List<String> list = new ArrayList<String>();
          for (SystemObject so : textDefinition)
          {
            list.add(so.getPid());
          }

          if (font != null && ausrichtung != null && textDefinition != null)
          {
            ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
            int abstand = 0;
            if (connection != null)
            {
              atg = connection.getDataModel().getAttributeGroup("atg.font");
              Data cd = font.getConfigurationData(atg);
              if (cd != null)
              {
                double dv = cd.getItem("ZeichenAbstandStandard").asScaledValue().doubleValue();
                if (dv > 0.0)
                  abstand = (int)(1 / dv);
              }
            }
            re = new VisuAnzeigeKapsel(list.toArray(new String[list.size()]), font.getPid(), strAusrichtung, abstand);
          }
        }
      }
    }
    return re;
  }
  
  
 public static Dimension holeDimensionVonAnzeige(ClientDavInterface connection, SystemObject so){
    
    AttributeGroup atg1 = connection.getDataModel().getAttributeGroup("atg.anzeige");
    AttributeGroup atg2 = connection.getDataModel().getAttributeGroup("atg.anzeigeTyp");
    
    Data cd = so.getConfigurationData(atg1);
    SystemObject so2 = cd.getItem("AnzeigeTyp").asReferenceValue().getSystemObject();
    Data cd2 = so2.getConfigurationData(atg2);
    int width = cd2.getItem("AuflsungX").asUnscaledValue().intValue();
    int height = cd2.getItem("AuflsungY").asUnscaledValue().intValue();
    return new Dimension(width, height);
     
  }
  
  public static SystemObject[] holeFontsUeberAnzeige(ClientDavInterface connection, SystemObject so){
    
    AttributeGroup atg1 = connection.getDataModel().getAttributeGroup("atg.anzeige");
    AttributeGroup atg2 = connection.getDataModel().getAttributeGroup("atg.anzeigeTyp");
    
    Data cd = so.getConfigurationData(atg1);
    SystemObject so2 = cd.getItem("AnzeigeTyp").asReferenceValue().getSystemObject();
    Data cd2 = so2.getConfigurationData(atg2);
        
    SystemObject[] aa = cd2.getItem("Font").asReferenceArray().getSystemObjectArray();
    if (aa != null)
      return aa;
    
    return null;
  }
  

  /**
   * @param result
   * @return
   */
  public static VisuAnzeigeKapsel holeDynamischerWechselTextAusData(Data item)
  {
    VisuAnzeigeKapsel re = null;

    SystemObject font = item.getItem("Font").asReferenceValue().getSystemObject();

    String strAusrichtung = item.getItem("TextAusrichtung").asTextValue().getValueText();
    EnumDynamischerWechseltextTextAusrichtung ausrichtung = EnumDynamischerWechseltextTextAusrichtung
        .valueOf(strAusrichtung);

    SystemObject[] textDefinition = item.getItem("Textdefinition").asReferenceArray().getSystemObjectArray();

    List<String> list = new ArrayList<String>();
    for (SystemObject so : textDefinition)
    {
      list.add(so.getPid());
    }

    if (font != null && ausrichtung != null && textDefinition != null)
    {

      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

      int abstand = 0;
      if (connection != null)
      {
        AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.font");
        Data cd = font.getConfigurationData(atg);
        if (cd != null)
        {
          double dv = cd.getItem("ZeichenAbstandStandard").asScaledValue().doubleValue();
          if (dv > 0.0)
            abstand = (int)(1 / dv);
        }
      }

      re = new VisuAnzeigeKapsel(list.toArray(new String[list.size()]), font.getPid(), strAusrichtung, abstand);
    }

    return re;
  }

  /**
   * @param viewer
   * @param doAq
   * @param result
   * @return
   */
  public static ZusatzElementTyp ermittleProgrammStatusAq(GraphicalViewer viewer,
      AnzeigeQuerschnittModelTypAbstract doAq, ResultData result)
  {
    ZusatzElementTyp re = ZusatzElementTyp.PROGRAMM_UNDEFINIERT;

    if (doAq != null)
    {
      EList<AnzeigeModelTypAbstract> anzeigen = doAq.getKomponenten();

      int cntHand = 0;
      int cntSonder = 0;
      int cntAuto = 0;

      for (AnzeigeModelTypAbstract doAnzeige : anzeigen)
      {
        SystemObject so = doAnzeige.getSystemObject();

        if (result != null && result.hasData())
        // if (result.hasData())
        {
          Data data = result.getData();
          Array array = data.getItem("Anzeigen").asArray();
          for (int i = 0; i < array.getLength(); i++)
          {
            Data item = array.getItem(i);
            SystemObject ref = item.getItem("Anzeige").asReferenceValue().getSystemObject();
            if (so.equals(ref))
            {
              String status = item.getItem("Programmstatus").asTextValue().getValueText();
              if (status.equals("Handprogramm"))
                cntHand++;
              else if (status.equals("Sonderprogramm"))
                cntSonder++;
              else if (status.equals("Automatikprogramm"))
                cntAuto++;
            }
          }
        }
      }

      if (cntHand > 0)
      {
        re = ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM;
      }
      else if (cntSonder > 0)
      {
        re = ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM;
      }
      else if (cntAuto > 0)
      {
        re = ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM;
      }

    }
    return re;
  }

  /**
   * @param kontextAq
   * @return
   */
  public static ResultData holeSchaltBildAnzeigeDatenSatz(int kontextAq)
  {
    ResultData result = null;

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if (connection != null)
    {
      IPreferenceStore store = Activator.getDefault().getPreferenceStore();
      String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
      if (pidSba != null && pidSba.length() > 0)
      {
        SystemObject object = connection.getDataModel().getObject(pidSba);
        if (object != null)
        {
          AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaSchaltbildAnzeige");

          Aspect asp = null;
          if (kontextAq < 2)
            asp = connection.getDataModel().getAspect("asp.sswSbaBefehle");
          else
            asp = connection.getDataModel().getAspect("asp.sswSbaSonderprogrammSimu");

          result = connection.getData(object, new DataDescription(atg, asp), 1000L);

        }
      }
    }
    return result;
  }

  /**
   * @param doModel
   * @param kontextAq
   * @return
   */
  public static ResultData holeAnzeigeQuerschnittDatenSatz(AnzeigeQuerschnittModelTypAbstract doModel, int kontextAq)
  {

    ResultData result = null;

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();

      SystemObject object = doModel.getSystemObject();
      if (object != null)
      {
        Aspect asp = null;
        AttributeGroup atg = null;
        if (kontextAq > 0)
        {
          atg = dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftSoll");
          asp = dataModel.getAspect("asp.aqVorgabe");
        }
        else
        {
          atg = dataModel.getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst");
          asp = dataModel.getAspect("asp.aqZustand");
        }

        result = connection.getData(object, new DataDescription(atg, asp), 1000L);
      }
    }
    return result;
  }

  /**
   * @param doModel
   * @param kontextAq
   * @return
   */
  public static ResultData holeAnzeigeDatenSatz(AnzeigeModelTypAbstract doModel)
  {

    ResultData result = null;

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();

      SystemObject object = doModel.getSystemObject();
      if (object != null)
      {
        Aspect asp = dataModel.getAspect("asp.aqZustand");
        AttributeGroup atg = dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst");

        result = connection.getData(object, new DataDescription(atg, asp), 1000L);
      }
    }
    return result;
  }

  /**
   * @param doModel
   * @param kontextAq
   * @return
   */
  public static ResultData holeAnzeigeDatenSatz(SystemObject so)
  {

    ResultData result = null;

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();

      if (so != null)
      {
        Aspect asp = dataModel.getAspect("asp.aqZustand");
        AttributeGroup atg = dataModel.getAttributeGroup("atg.anzeigeEigenschaftIst");

        result = connection.getData(so, new DataDescription(atg, asp), 1000L);
      }
    }
    return result;
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected static CommandStack getCommandStack(GraphicalViewer graphicalViewer)
  {

    CommandStack stack = graphicalViewer.getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  /**
   * @param secId
   * @return
   */
  public static boolean ermittleKilometrierungAufsteigendAbsteigend(final String secId)
  {

    String s1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO;
    String s2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO;

    if (secId.equals("SW"))
    {
      s1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW;
      s2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW;
    }

    IPreferenceStore store = Activator.getDefault().getPreferenceStore();

    double d3 = store.getDouble(s1);
    double d4 = store.getDouble(s2);

    boolean absteigend = false;

    if (d3 > d4)
      absteigend = true;
    return absteigend;
  }

  /**
   * @param connection
   * @param secondaryId
   * @return
   */
  public static SystemObject aktionListeAuswahlAq(ClientDavInterface connection, String secondaryId)
  {
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();

    String pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO;
    String pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO;

    if (secondaryId.equals("SW"))
    {
      pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW;
      pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW;
    }

    double d3 = store.getDouble(pref1);
    double d4 = store.getDouble(pref2);

    return aktionListeAuswahlAq(connection, secondaryId, d3, d4);

  }

  /**
   * @param connection
   * @param secondaryId
   * @param von
   * @param bis
   * @return
   */
  public static SystemObject aktionListeAuswahlAq(ClientDavInterface connection, String secondaryId, double von,
      double bis)
  {
    ListDialog listDialog = new ListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell()) {
      protected int getTableStyle()
      {
        return SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER  | SWT.MULTI;
      }

      @Override
      protected void configureShell(Shell shell)
      {
        // Fr die F1 Taste
        PlatformUI.getWorkbench().getHelpSystem().setHelp(shell, Activator.PLUGIN_ID + ".auswahlAnzeigequerschnitt");
        super.configureShell(shell);
      }
    };
    listDialog.setTitle("Auswahl AQ");
    listDialog.setMessage("Folgende AQ stehen zur Auswahl:");
    listDialog.setContentProvider(new ArrayContentProvider());
    listDialog.setLabelProvider(new ListLabelProvider());     

    boolean absteigend = ermittleKilometrierungAufsteigendAbsteigend(secondaryId);

    SortedMap<String, SystemObject> map = DaVHelper.holeListeKilometrierungAq(secondaryId, absteigend, von, bis);

    listDialog.setInput(map.values());

    if (listDialog.open() == ListDialog.OK)
    {
      List model = new ArrayList();
      Object[] results = listDialog.getResult();
      if (results != null)
      {
        model.addAll(Arrays.asList(results));
      }

      if (model.size() > 0)
      {
        return (SystemObject)model.get(0);
      }
    }
    return null;
  }

  /**
   * @param secId
   * @param s1
   * @param s2
   * @return
   */
  public static boolean validiereTextInput(String secId, String s1, String s2)
  {
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();

    String pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO;
    String pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO;

    if (secId.equals("SW"))
    {
      pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW;
      pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW;
    }

    double d3 = store.getDouble(pref1);
    double d4 = store.getDouble(pref2);

    return validiereTextInput(secId, s1, s2, d3, d4);
  }

  /**
   * @param secId
   * @param s1
   * @param s2
   * @param d3
   * @param d4
   * @return
   */
  public static boolean validiereTextInput(String secId, String s1, String s2, double d3, double d4)
  {
    Double d1 = null;
    Double d2 = null;

    String s11 = s1.trim();
    String s21 = s2.trim();
    if (s1.contains(","))
      s11 = s11.replace(",", ".");
    if (s2.contains(","))
      s21 = s21.replace(",", ".");
    try
    {
      d1 = new Double(s11);
    }
    catch (NumberFormatException e)
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Definition km");
      mb.setMessage("Eingabe ungltig: " + s11);
      mb.open();
      return false;
    }

    try
    {
      d2 = new Double(s21);
    }
    catch (NumberFormatException e)
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Definition km");
      mb.setMessage("Eingabe ungltig: " + s21);
      mb.open();
      return false;
    }

    boolean fehler = true;
    if (d1 != null && d2 != null)
    {
      if (d3 < d4)
      {
        if (d1 >= d3 && d1 <= d4)
        {
          if (d1 <= d2)
          {
            fehler = false;
          }
        }
      }
      else if (d4 < d3)
      {
        if (d1 <= d3 && d1 >= d4)
        {
          if (d2 <= d1)
          {
            fehler = false;
          }
        }
      }
    }

    if (fehler)
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Definition km");
      mb.setMessage("Eingabe nicht gem Kilometrierung: " + s11 + " - " + s21);
      mb.open();
      return false;
    }
    return true;
  }

  public static SystemObject holeSystemObjectSswSbaAqVonAq(ClientDavInterface connection, SystemObject systemObject)
  {
    AttributeGroup atgCo = connection.getDataModel().getAttributeGroup("atg.sswSbaAnzeigeQuerschnitt");
    SystemObjectType type = connection.getDataModel().getType("typ.sswSbaAnzeigeQuerschnitt");
    if (type != null)
    {
      List<SystemObject> elements = type.getElements();
      for (SystemObject so : elements)
      {
        Data cd = so.getConfigurationData(atgCo);
        if (cd != null)
        {
          SystemObject ref = cd.getItem("AnzeigeQuerschnitt").asReferenceValue().getSystemObject();
          SystemObject aq = systemObject;

          if (aq != null && ref != null && aq.equals(ref))
            return so;
        }
      }
    }
    return null;
  }

  public static SystemObject holeSystemObjectAqVonSswSbaAq(ClientDavInterface connection, SystemObject systemObject)
  {
    AttributeGroup atgCo = connection.getDataModel().getAttributeGroup("atg.sswSbaAnzeigeQuerschnitt");
    if (systemObject != null)
    {
      Data cd = systemObject.getConfigurationData(atgCo);
      if (cd != null)
      {
        SystemObject ref = cd.getItem("AnzeigeQuerschnitt").asReferenceValue().getSystemObject();
        return ref;
      }
    }
    return null;
  }
  
  
  /**
   * Prft ob im Anzeigeinhalt festgelegt ist, dass der Inhalt blinken soll.
   * 
   * @param result
   *          {@link ResultData} gem ATG
   *          <code>atg.anzeigeEigenschaftIst</code> oder
   *          <code>atg.anzeigeEigenschaftSoll</code>.
   * @return Liefert <code>true</code> wenn der Anzeigeinhalt blinkend
   *         dargestellt werden soll, sonst <code>false</code> (auch wenn nicht
   *         bestimmbar).
   */
  public static boolean holeIsBlinkenInhaltAusDaVData(ResultData result)
  {
    return holeIsBlinkenInhaltAusDaVData(result, null);
  }
  
  /**
   * Prft ob im Anzeigeinhalt festgelegt ist, dass der Inhalt blinken soll.
   * 
   * @param result
   *          {@link ResultData} gem ATG
   *          <code>atg.anzeigeEigenschaftIst</code> oder
   *          <code>atg.anzeigeEigenschaftSoll</code>.
   * @return Liefert <code>true</code> wenn der Anzeigeinhalt blinkend
   *         dargestellt werden soll, sonst <code>false</code> (auch wenn nicht
   *         bestimmbar).
   */
  public static boolean holeIsBlinkenInhaltAusDaVData(ResultData result, SystemObject object)
  {
    boolean blinken = false;

    if (result != null)
    {
      AttributeGroup atg = result.getDataDescription().getAttributeGroup();
      if (atg.getPid().equals("atg.anzeigeEigenschaftIst")
          || atg.getPid().equals("atg.anzeigeEigenschaftSoll"))
      {
        Data data = result.getData();
        if (data != null)
        {
          NumberValue sv = data.getItem("Eigenschaft")
              .getScaledValue("Blinken");
          if (sv != null)
          {
            String txtBlinken = sv.getValueText();
            if ((txtBlinken != null) && (txtBlinken.equalsIgnoreCase("Ja")))
            {
              blinken = true;
            }
          }
        }
      }
      else if (atg.getPid().equals("atg.sswSbaSchaltbildAnzeige"))
      {
        Data data = result.getData();
        if (data != null)
        {
          Array array = data.getItem("Anzeigen").asArray();
          if (array.getLength() > 0)
          {
//            List<AnzeigeQuerschnittModelTypAbstract> listAq = new ArrayList<AnzeigeQuerschnittModelTypAbstract>();
            for (int i = 0; i < array.getLength(); i++)
            {
              Data item = array.getItem(i);
              SystemObject so = item.getItem("Anzeige").asReferenceValue().getSystemObject();
              if (object != null){
                if (so.equals(object)){
//                  SystemObject wvzInhaltObj = item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue()
//                  .getSystemObject();
//                  Data dynWechselTextData = item.getItem("Schaltbild").getItem("DynamischerWechseltext");                  
                  String strBlinken = item.getItem("Schaltbild").getItem("Blinken").asTextValue().getValueText();
//                  String strBlinkTakt = item.getItem("Schaltbild").getItem("Blinktakt").asTextValue().getValueText();
                  
//                  System.out.println("Blinken: " + strBlinken  + " -> " + object.getPid());
                  
                  if (strBlinken.equals("Ja"))
                    blinken = true;
                  
                }
                
              }
            }
          }
        }
      }
      else{
//        System.out.println("else -> ATG:_ "  + atg.getPid());
      }
    }

    return blinken;
  }

}
