/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * ndern der Betriebsart von Anzeigequerschnitten.<br>
 * Die Anzeigequerschnitte, deren Betriebsart gendert werden soll, werden als Textliste (;-getrennt) in der secondary
 * Viev-ID bergeben. Die Textliste kann PID von Objekten vom Typ AnzeigeQuerschnitt oder vom Typ SBAAnzeigeQuerschnitt
 * (auch gemischt) enthalten.
 * 
 * @author Sans
 * 
 */
public class BetriebsartEinstellen extends ViewPart implements IDaVDataUpdateListener
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.betriebsartEinstellen";

  private Composite _parent;

  private Composite _compositeAnzeigeQuerschnitte;

  private Group _gruppeAnzeigequerschnitte;

  private TableViewer _tvAnzeigeQuerschnitte;

  private Composite _compositeSchalten;

  private Group _gruppeSchalten;

  private Button[] _radioButtons = new Button[Betriebsart.values().length];

  private Composite _compositeButton;

  private Button _buttonSchalten;

  private Button _buttonAbbrechen;

  public enum Betriebsart {
    Normalbetrieb, Blindbetrieb;
  }

  private int _radioButtonIndex = 0;

  private Map<String, ModelObject> _mapModel = new LinkedHashMap<String, ModelObject>();

  private SystemObject getSbaAqZuAq(SystemObject aq)
  {
    SystemObject result = null;

    for (SystemObject so : aq.getDataModel().getType("typ.sswSbaAnzeigeQuerschnitt").getElements())
    {
      if (aq == getAqZuSbaAq(so))
      {
        result = so;
        break;
      }
    }
    return result;
  }

  private SystemObject getAqZuSbaAq(SystemObject sbaAq)
  {
    SystemObject result = null;
    AttributeGroup atg = sbaAq.getDataModel().getAttributeGroup("atg.sswSbaAnzeigeQuerschnitt");
    Data data = sbaAq.getConfigurationData(atg);
    if (data != null)
    {
      result = data.getReferenceValue("AnzeigeQuerschnitt").getSystemObject();
    }
    return result;
  }

  private boolean schalten(Betriebsart betriebsart)
  {
    boolean result = false;
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();
      AttributeGroup atg = dataModel.getAttributeGroup("atg.sswSbaHandeingriffBetriebsart");
      Aspect asp = dataModel.getAspect("asp.sswSbaHandschaltung");
      List<ResultData> list = new ArrayList<ResultData>();
      for (String pid : _mapModel.keySet())
      {
        SystemObject so = dataModel.getObject(pid);
        if (so != null)
        {
          Data data = connection.createData(atg);
          data.getTextValue("Betriebsart").setText(betriebsart.toString());
          list.add(new ResultData(so, new DataDescription(atg, asp), connection.getTime(), data));
        }
        else
        {
          throw new IllegalStateException("Objekt zu Pid \"" + pid + "\" nicht gefunden");
        }
      }
      if (!list.isEmpty())
      {
        UrlasserDialog urlasserDialog = new UrlasserDialog(Display.getCurrent().getActiveShell());
        result = urlasserDialog.anzeigen(list).getEingabeOk();
      }
    }
    return result;
  }

  @Override
  public void init(IViewSite site) throws PartInitException
  {
    String secondaryId = site.getSecondaryId();
    String pidsAqText = "";

    if (secondaryId != null)
    {
      String[] components = secondaryId.split("#");
      if (components.length > 0)
      {
        pidsAqText = components[0];
      }
    }

    String[] pidsAqArray = pidsAqText.split(";");

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();
      SystemObject firstSbaAq = null;

      for (String pid : pidsAqArray)
      {
        SystemObject so = dataModel.getObject(pid);
        if (so != null)
        {
          if (so.isOfType("typ.sbaAnzeigeQuerschnitt"))
          {
            SystemObject aq = getSbaAqZuAq(so);
            if (aq != null)
            {
              _mapModel.put(so.getPid(), new ModelObject(aq.getPid(), aq.getName()));
              firstSbaAq = _mapModel.size() == 1 ? so : null;
            }
            else
            {
              throw new PartInitException("Zum " + so.getType().getName() + " mit PID \"" + pid + " existiert kein "
                  + dataModel.getType("typ.anzeigeQuerschnitt").getName());
            }
          }
          else if (so.isOfType("typ.anzeigeQuerschnitt"))
          {
            SystemObject sbaAq = getSbaAqZuAq(so);
            if (sbaAq != null)
            {
              _mapModel.put(sbaAq.getPid(), new ModelObject(so.getPid(), so.getName()));
              firstSbaAq = _mapModel.size() == 1 ? sbaAq : null;
            }
            else
            {
              throw new PartInitException("Zum " + so.getType().getName() + " mit PID \"" + pid + " existiert kein "
                  + dataModel.getType("typ.sbaAnzeigeQuerschnitt").getName());
            }
          }
          else
          {
            throw new PartInitException("Objekt mit PID \"" + pid + "\" ist von nichtuntersttztem Typ "
                + so.getType().getName());
          }
          if (firstSbaAq != null)
          {
            AttributeGroup atg = dataModel.getAttributeGroup("atg.sswSbaHandeingriffBetriebsart");
            Aspect asp = dataModel.getAspect("asp.sswSbaHandschaltungVorgabe");
            Data data = connection.getData(firstSbaAq, new DataDescription(atg, asp), 0L).getData();
            if (data != null)
            {
              String ba = data.getTextValue("Betriebsart").getText();
              if (ba != null)
              {
                Betriebsart betriebsart = Betriebsart.valueOf(ba);
                if (betriebsart != null)
                {
                  _radioButtonIndex = betriebsart.ordinal();
                }
              }
            }
          }

          List<String> list = Arrays.asList(_mapModel.keySet().toArray(new String[_mapModel.keySet().size()]));
          DaVKommunikationDobj.getInstanz().anmeldenAufDatenMQDaten(list, "atg.sswSbaHandeingriffBetriebsart",
              "asp.sswSbaHandschaltungVorgabe");
          DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);
        }
        else
        {
          throw new PartInitException("Objekt mit PID \"" + pid + "\" nicht gefunden");
        }
      }
    }

    super.init(site);
  }

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;
    
    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".aqBetriebsart");

    _parent.setLayout(new GridLayout());
    createCompositeAnzeigeQuerschnitt();
    createCompositeSchalten();
    createCompositeButton();

    _tvAnzeigeQuerschnitte.setInput(_mapModel.values());
    _radioButtons[_radioButtonIndex].setSelection(true);
    _buttonSchalten.setEnabled(_mapModel.size() > 0);

  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }

  private void createCompositeAnzeigeQuerschnitt()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _compositeAnzeigeQuerschnitte = new Composite(_parent, SWT.NONE);
    _compositeAnzeigeQuerschnitte.setLayout(new GridLayout());
    _compositeAnzeigeQuerschnitte.setLayoutData(gridData);
    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;

    GridData gridDataGruppeAnzeigequerschnitte = new GridData();
    gridDataGruppeAnzeigequerschnitte.horizontalAlignment = GridData.FILL;
    gridDataGruppeAnzeigequerschnitte.grabExcessHorizontalSpace = true;
    gridDataGruppeAnzeigequerschnitte.grabExcessVerticalSpace = true;
    gridDataGruppeAnzeigequerschnitte.verticalAlignment = GridData.FILL;
    _gruppeAnzeigequerschnitte = new Group(_compositeAnzeigeQuerschnitte, SWT.NONE);
    _gruppeAnzeigequerschnitte.setLayout(new GridLayout());
    _gruppeAnzeigequerschnitte.setLayoutData(gridDataGruppeAnzeigequerschnitte);
    _gruppeAnzeigequerschnitte.setText("Ausgewhlte Anzeigequerschnitte");

    _tvAnzeigeQuerschnitte = new TableViewer(_gruppeAnzeigequerschnitte, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeigeQuerschnitte, SWT.LEFT);
    column1.getColumn().setWidth(160);
    column1.getColumn().setText("Name");

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeigeQuerschnitte, SWT.LEFT);
    column2.getColumn().setWidth(160);
    column2.getColumn().setText("Pid");

    TableViewerColumn column3 = new TableViewerColumn(_tvAnzeigeQuerschnitte, SWT.LEFT);
    column3.getColumn().setWidth(120);
    column3.getColumn().setText("Betriebsart (Vorgabe)");

    _tvAnzeigeQuerschnitte.setContentProvider(new ContentProvider());
    _tvAnzeigeQuerschnitte.setLabelProvider(new LabelProvider());
    _tvAnzeigeQuerschnitte.getTable().setLayoutData(gridDataTabelle);
    _tvAnzeigeQuerschnitte.getTable().setLinesVisible(true);
    _tvAnzeigeQuerschnitte.getTable().setHeaderVisible(true);
  }

  private void createCompositeSchalten()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    // gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _compositeSchalten = new Composite(_parent, SWT.NONE);
    _compositeSchalten.setLayout(new GridLayout());
    _compositeSchalten.setLayoutData(gridData);

    // Groupbox
    RowLayout layoutGrp = new RowLayout();
    layoutGrp.marginWidth = 5;
    layoutGrp.spacing = 5;
    layoutGrp.pack = false;
    GridData gridDataGrp = new GridData();
    gridDataGrp.horizontalAlignment = GridData.FILL;
    gridDataGrp.grabExcessHorizontalSpace = true;
    gridDataGrp.verticalAlignment = GridData.FILL;
    _gruppeSchalten = new Group(_compositeSchalten, SWT.NONE);
    _gruppeSchalten.setLayoutData(gridDataGrp);
    _gruppeSchalten.setLayout(layoutGrp);
    _gruppeSchalten.setText("Zu schaltende Betriebsart");

    SelectionAdapter selectionListener = new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        for (int i = 0; i < _radioButtons.length; i += 1)
        {
          if (e.widget == _radioButtons[i] && _radioButtons[i].getSelection())
          {
            _radioButtonIndex = i;
            break;
          }
        }
      }
    };

    for (int i = 0; i < _radioButtons.length; i += 1)
    {
      _radioButtons[i] = new Button(_gruppeSchalten, SWT.RADIO);
      _radioButtons[i].setText(Betriebsart.values()[i].toString());
      _radioButtons[i].addSelectionListener(selectionListener);
    }

  }

  private void createCompositeButton()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    _compositeButton = new Composite(_parent, SWT.NONE);
    _compositeButton.setLayoutData(gridData);
    _compositeButton.setLayout(rowLayout);

    _buttonSchalten = new Button(_compositeButton, SWT.NONE);
    _buttonSchalten.setText(" Schalten ");
    _buttonSchalten.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        if (schalten(Betriebsart.values()[_radioButtonIndex]))
        {

        }

      }

    });

    _buttonAbbrechen = new Button(_compositeButton, SWT.NONE);
    _buttonAbbrechen.setText(" Abbrechen ");
    _buttonAbbrechen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        dispose();
      }
    });

  }

  @Override
  public void dispose()
  {
    super.dispose();
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    }
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if (page != null)
    {
      page.hideView(this);
    }
  }

  private static class ContentProvider implements IStructuredContentProvider
  {
    @Override
    public Object[] getElements(Object inputElement)
    {
      return ((Collection)inputElement).toArray();
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

  private static class LabelProvider implements ITableLabelProvider
  {
    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String result = null;
      ModelObject modelObject = (ModelObject)element;

      switch (columnIndex)
      {
        case 0:
          String name = modelObject.getName();
          result = name != null ? name : "";
          break;
        case 1:
          String pid = modelObject.getPid();
          result = pid != null ? pid : "";
          break;
        case 2:
          String betriebsart = modelObject.getBetriebsart();
          result = betriebsart != null ? betriebsart : "";
          break;
        default:
          result = "";
          break;
      }

      return result;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    ResultData rd = e.getResult();
    DataDescription dd = rd.getDataDescription();
    if (dd.getAttributeGroup().getPid().equals("atg.sswSbaHandeingriffBetriebsart")
        && dd.getAspect().getPid().equals("asp.sswSbaHandschaltungVorgabe"))
    {
      SystemObject so = rd.getObject();
      ModelObject modelObject = _mapModel.get(so.getPid());
      if (modelObject != null)
      {
        modelObject.setBetriebsart(null);
        Data data = rd.getData();
        if (data != null)
        {
          String ba = data.getTextValue("Betriebsart").getText();
          if (ba != null)
          {
            Betriebsart betriebsart = Betriebsart.valueOf(ba);
            if (betriebsart != null)
            {
              modelObject.setBetriebsart(betriebsart.toString());
            }
          }
        }
        else
        {
          modelObject.setBetriebsart("Keine Daten");
        }
        _tvAnzeigeQuerschnitte.refresh();
      }
    }
  }

  private static class ModelObject
  {

    private String _pid;

    private String _name;

    private String _betriebsart;

    public ModelObject(String pid, String name)
    {
      _pid = pid;
      _name = name;
    }

    public String getBetriebsart()
    {
      return _betriebsart;
    }

    public void setBetriebsart(String betriebsart)
    {
      _betriebsart = betriebsart;
    }

    public String getPid()
    {
      return _pid;
    }

    public String getName()
    {
      return _name;
    }

  }

}
