/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.event.EventListenerList;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;
import org.osgi.framework.BundleContext;

import de.bsvrz.buv.plugin.baueditor.sts.util.EclipseEngstellenCache;
import de.bsvrz.buv.plugin.baueditor.sts.util.EngstellenCache;
import de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation;
import de.bsvrz.buv.rw.bitctrl.AbstractDAVPlugin;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.sys.funclib.bitctrl.daf.DavProvider;
import de.bsvrz.sys.funclib.bitctrl.modell.systemmodellaoe.objekte.AutarkeOrganisationsEinheit;
import de.bsvrz.sys.funclib.bitctrl.modell.util.KappichModellUtil;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCache;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnittCache;

/**
 * The activator class controls the plug-in life cycle.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: BauEditorPlugin.java,v 1.2 2014/11/14 07:29:41 Pittner Exp $
 */
public class BauEditorPlugin extends AbstractDAVPlugin implements
		PropertyChangeListener {

	/**
	 * Die Plug-in ID.
	 */
	public static final String PLUGIN_ID = "de.bsvrz.buv.plugin.baueditor";

	/** Lock-Objekt. */
	private final Object pluginLock = new Object();

	/** Liste der Eventlistener. */
	private final EventListenerList listeners = new EventListenerList();

	/**
	 * Kommandozeilenparameter, der anzeigt, ob ein gerichtetes Netz vorliegt
	 * oder nicht.
	 */
	private final String STRASSE_EINDEUTIG = "-strasseEindeutig";

	/**
	 * The shared instance.
	 */
	private static BauEditorPlugin plugin;

	/**
	 * Flag, welches angibt, ob dieses Plug-in ausfhrbar ist. Das Plug-in ist
	 * nicht ausfhrbar, wenn kein gltiges Verkehrsmodellnetz als
	 * Aufrufeparameter bergeben wurde.
	 */
	private boolean ausfuehrbar = true;

	/**
	 * Der EngstellenCache, welcher die Engstellen (Baustellen und Unflle)
	 * verwaltet.
	 */
	private EngstellenCache engstellenCache;

	/**
	 * Der Netz-Cache.
	 */
	private NetzCache netzCache;

	/**
	 * Die autarke Organisationseinheit, auf welcher smtliche Ereignisse
	 * basieren.
	 */
	private AutarkeOrganisationsEinheit aoe;

	/**
	 * Flag, das anzeigt, ob der "strasseEindeutig" Kommandozeilenparameter
	 * gesetzt wurde.
	 */
	private boolean strasseEindeutig = false;

	/**
	 * Der Konstruktor.
	 */
	public BauEditorPlugin() {
	}

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		plugin = this;

		if (getObjektFactory() == null) {
			startOffline();
		} else {
			getObjektFactory().addPropertyChangeListener(this);

			if (getObjektFactory().isVerbunden()) {
				startOnline();
			} else {
				startOffline();
			}
		}

		final KmdParameter param = Kommandozeilenparameter.getInstanz()
				.getKommandoZeilenParameter(STRASSE_EINDEUTIG);

		if (param != null && "ja".equalsIgnoreCase(param.getWert())) {
			strasseEindeutig = true;
			getLogger().info(
					"Es werden Straenobjekte mit eindeutiger Richtung verwendet. (Startparameter "
							+ STRASSE_EINDEUTIG + "=ja)");
		}
	}

	/**
	 * Startet Online-Mode.
	 */
	private void startOnline() {
		getLogger().info("Starte Online-Mode");

		synchronized (pluginLock) {

			try {

				// (1)berprfe, ob Plug-in ausfhrbar ist
				ausfuehrbar = StartupPlugin.getDefault().getDefaultNetz() != null;

				if (!ausfuehrbar) {
					final Job uiJob = new UIJob("") {
						@Override
						public IStatus runInUIThread(
								final IProgressMonitor monitor) {
							MessageDialog
									.openError(
											getWorkbench()
													.getActiveWorkbenchWindow()
													.getShell(),
											"Fehler",
											Zeichenketten.FEHLERMELDUNG_NICHT_AUSFUEHRBAR);
							return Status.OK_STATUS;
						}
					};
					uiJob.schedule();
				}

				// (2) Initialisiere die Rahmenwerkseinbindung

				// (3.1) Dummy-Aufruf der Funktionen, um die Rechteverwaltung zu
				// initialisieren !!!!
				getLogger().info(
						"Funktionen der Baustellen- und Unfalleingabe: "
								+ EngstellenVerwaltungPluginFunktion.values());

				// (3.2) final Initialisiere Berechtigungsverwaltung
				getLogger()
						.info("Initialisiere Berechtigungsverwaltung der Baustellen- und Unfalleingabe");
				DaVKommunikation.getInstanz()
						.hinzufuegenOberflaechenBenutzerZuhoerer(
								EngstellenVerwaltungBerechtigungsVerwaltung
										.getInstanz());
				EngstellenVerwaltungBerechtigungsVerwaltung.getInstanz()
						.aktualisiereBerechtigungen();

			} catch (final Error e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Startet Offline-Mode.
	 */
	private void startOffline() {
		getLogger().info("Starte Offline-Mode");
	}

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
		if (DavProvider.PROP_VERBUNDEN.equals(evt.getPropertyName())) {
			if (Boolean.TRUE.equals(evt.getNewValue())) {
				getLogger()
						.info("Baustellen- und Unfalleingabe-Plugin: Verbindung aufgebaut");
				startOnline();
			} else {
				getLogger()
						.info("Baustellen- und Unfalleingabe-Plugin: Verbindung abgebaut");
				startOffline();
			}
			setViewsOnline(Boolean.TRUE.equals(evt.getNewValue()));
		}
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		DaVKommunikation.getInstanz().entferneOberflaechenBenutzerZuhoerer(
				EngstellenVerwaltungBerechtigungsVerwaltung.getInstanz());
		if (engstellenCache != null) {
			engstellenCache.dispose(true);
		}
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 * 
	 * @return the shared instance.
	 */
	public static BauEditorPlugin getDefault() {
		return plugin;
	}

	/**
	 * Liefert den EngstellenCache zurck, welcher die Engstellen (Baustellen
	 * und Unflle) verwaltet.
	 * 
	 * @return den EngstellenCache
	 */
	public EngstellenCache getEngstellenCache() {
		if (istOnline() && engstellenCache == null) {
			final StreckenAbschnittCache streckenAbschnittCache = StartupPlugin
					.getDefault().getDefaultStreckenAbschnittCache();
			engstellenCache = new EclipseEngstellenCache(streckenAbschnittCache);
		}
		return engstellenCache;
	}

	/**
	 * Liefert den Netz-Cache zurck.
	 * 
	 * @return den Netz-Cache
	 */
	public NetzCache getNetzCache() {
		if (istOnline() && netzCache == null) {
			netzCache = StartupPlugin.getDefault().getDefaultNetzCache();
		}
		return netzCache;
	}

	/**
	 * Liefert die autarke Organisationseinheit zurck, auf welcher smtliche
	 * Ereignisse basieren.
	 * 
	 * @return die autarke Organisationseinheit
	 */
	public AutarkeOrganisationsEinheit getAOE() {
		if (istOnline() && aoe == null) {
			aoe = KappichModellUtil.getAOE(getObjektFactory());
		}
		return aoe;
	}

	/**
	 * Liefert <code>true</code> zurck, wenn dieses Plug-in ausfhrbar ist.
	 * 
	 * <p>
	 * Das Plug-in ist nicht ausfhrbar, wenn kein gltiges Verkehrsmodellnetz
	 * als Aufrufeparameter bergeben wurde.
	 * </p>
	 * 
	 * @return <code>true</code>, wenn dieses Plug-in ausfhrbar ist, ansonsten
	 *         <code>false</code>.
	 */
	public boolean isAusfuehrbar() {
		return ausfuehrbar;
	}

	/**
	 * Liefert ein zum gegebenen Pfad gehrenden Image zurck. Ein ggf. neu
	 * erzeugtes Image wird in einer Plug-in eigenen ImageRegistry abgelegt.
	 * 
	 * @param bildPfad
	 *            der Bildpfad
	 * 
	 * @return ein zum gegebenen Pfad gehrenden Image
	 */
	public Image getImage(final String bildPfad) {
		Image image = getImageRegistry().get(bildPfad);
		if (image == null) {
			image = getImageDescriptor(bildPfad).createImage();
			getImageRegistry().put(bildPfad, image);
		}
		return image;
	}

	/**
	 * Liefert die zur gegebenen ID passende Systemfarbe zurck.
	 * 
	 * @param id
	 *            die ID der Systemfarbe, siehe SWT.COLOR_*
	 * 
	 * @return die zur gegebenen ID passende Systemfarbe
	 */
	public static Color getSystemColor(final int id) {
		return PlatformUI.getWorkbench().getDisplay().getSystemColor(id);
	}

	/**
	 * Fgt einen PluginEvent-Listener hinzu.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void addEventListener(final BauEditorEventListener listener) {
		listeners.add(BauEditorEventListener.class, listener);
	}

	/**
	 * Entfernt einen PluginEvent-Listener.
	 * 
	 * @param listener
	 *            den zu entfernenden Listener
	 */
	public void removeEventListener(final BauEditorEventListener listener) {
		listeners.remove(BauEditorEventListener.class, listener);
	}

	/**
	 * Aktualisiert alle angemeldeten Listener mit dem Online-Status.
	 * 
	 * @param online
	 *            <code>true</code> oder <code>false</code>
	 */
	public void setViewsOnline(final boolean online) {
		PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
			public void run() {
				for (final BauEditorEventListener listener : listeners
						.getListeners(BauEditorEventListener.class)) {
					listener.switchOnline(online);
				}
			}
		});
	}

	/**
	 * Gibt den Online-Status fr das Plug-in zurck.
	 * 
	 * @return <code>true</code>, wenn online, sonst <code>false</code>
	 */
	public boolean istOnline() {
		if (getObjektFactory() == null) {
			return false;
		} else {
			return getObjektFactory().isVerbunden();
		}
	}

	/**
	 * @return the strasseEindeutig
	 */
	public final boolean isStrasseEindeutig() {
		return strasseEindeutig;
	}
}
