/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.buv.rw.basislib.berechtigung.IOberflaechenBerechtigungZuhoerer;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.IDaVPreLader;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Zentrale Instanz zur Verwaltung der Berechtigungen fr die Funktionen des
 * Plugins EngstellenVeraltung.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: EngstellenVerwaltungBerechtigungsVerwaltung.java,v 1.2 2014/11/14 07:29:41 Pittner Exp $
 */
public final class EngstellenVerwaltungBerechtigungsVerwaltung implements
		IOberflaechenBerechtigungZuhoerer, IDaVPreLader {

	/**
	 * Der Logger.
	 */
	private final Debug debug = Debug.getLogger();

	/** Das Singleton-Objekt fr die Verwaltung der Berechtigungen. */
	private static final EngstellenVerwaltungBerechtigungsVerwaltung INSTANCE = new EngstellenVerwaltungBerechtigungsVerwaltung();

	/**
	 * Liefert die Instanz des Verwaltungsobjekts.
	 * 
	 * @return das Objekt
	 */
	public static EngstellenVerwaltungBerechtigungsVerwaltung getInstanz() {
		return EngstellenVerwaltungBerechtigungsVerwaltung.INSTANCE;
	}

	/**
	 * Die Liste der Objekte, fr die die Berechtigungen abgefragt werden.
	 * 
	 * Das Berechtigungskonzept des Rahmenwerks erfordert zur Prfung von
	 * Berechtigungen in jedem Fall die bergabe eines Systemobjekts, auch wenn
	 * die Berechtigung an sich nicht objektbezogen ist. Fr die globale
	 * Verwaltung der UDA-Pluginberechtigungen wird deshalb hier das lokale
	 * Applikationsobjekt in die Liste eingetragen und bei der Prfung der
	 * Berechtigungen bergeben.
	 */
	private final List<SystemObject> berechtigunsObjektListe = new ArrayList<SystemObject>();

	/**
	 * Der Konstruktor.
	 */
	private EngstellenVerwaltungBerechtigungsVerwaltung() {
		super();
		DaVPreLader.getInstanz().hinzufuegenListenerPreLader(this);
	}

	/**
	 * Aktualisiert den Zustand der Berechtigungen des Plug-ins.
	 */
	public void aktualisiereBerechtigungen() {
		if (DaVVerbindung.getInstanz().getDaVVerbindung() != null) {
			debug.config("Plug-in Baustellen- und Unfalleingabe: Aktualisiere Berechtigungen");
			if (berechtigunsObjektListe.isEmpty()) {
				berechtigunsObjektListe.add(DaVVerbindung.getInstanz()
						.getDaVVerbindung().getLocalApplicationObject());
			}
			OberflaechenBerechtigung.getInstanz()
					.pruefeBerechtigungElementGruppe(berechtigunsObjektListe,
							BauEditorPlugin.class.getName());
		} else {
			debug.config("Plug-in Baustellen- und Unfalleingabe Offline! Keine Aktualisierung der Berechtigungen");
		}
	}

	@Override
	public void freigabe(final OberflaechenBerechtigungEreignis e) {

		debug.config("Plug-in Baustellen- und Unfalleingabe: Freigabe empfangen");
		List<OberflaechenFunktion> liste = e.getOberflaechenFunktionSperrung();
		if (liste.size() == 0) {
			debug.config("keine gesperrten Elemente");
		} else {
			debug.config(liste.size() + " gesperrte Elemente");
		}
		for (final OberflaechenFunktion oFunk : liste) {
			final EngstellenVerwaltungPluginFunktion funktion = EngstellenVerwaltungPluginFunktion
					.getFunktion(oFunk.getFunktionsName());
			if (funktion != null) {
				funktion.setFreigegeben(false);
			}
		}

		liste = e.getOberflaechenFunktionFreigabe();
		if (liste.size() == 0) {
			debug.config("keine freigegebenen Elemente");
		} else {
			debug.config(liste.size() + " freigegebene Elemente");
		}
		for (final OberflaechenFunktion oFunk : liste) {
			final EngstellenVerwaltungPluginFunktion funktion = EngstellenVerwaltungPluginFunktion
					.getFunktion(oFunk.getFunktionsName());
			if (funktion != null) {
				funktion.setFreigegeben(true);
			}
		}
	}

	@Override
	public void sperrung(final OberflaechenBerechtigungEreignis e) {
		freigabe(e);
	}

	@Override
	public void systemObjekteAktualisierung(final GtmDatenBasis datenBasis) {
		// keine Aktionen erforderlich
	}

	@Override
	public void systemObjekteInitial(final GtmDatenBasis datenBasis) {
		debug.info("Plug-in Baustellen- und Unfalleingabe: Prelader Callback");
		aktualisiereBerechtigungen();
	}
}
