/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.EventListenerList;

import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Definition der Funktionen, fr die Berechtigungen innerhalb des
 * Rahmenwerkes festgelegt werden knnen.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: EngstellenVerwaltungPluginFunktion.java,v 1.2 2014/11/14 07:29:41 Pittner Exp $
 */
public enum EngstellenVerwaltungPluginFunktion {

	/** Aktion zum Anlegen einer Baustelle. */
	BAUSTELLE_ANLEGEN("baustelleAnlegen"),

	/** Aktion zum Duplizieren einer Baustelle. */
	BAUSTELLE_DUPLIZIEREN("baustelleDuplizieren"),

	/** Aktion zum L&ouml;schen einer Baustelle. */
	BAUSTELLE_LOESCHEN("baustelleLoeschen"),

	/** Aktion zum Anzeigen einer Baustelle. */
	BAUSTELLE_BEARBEITEN("baustelleBearbeiten"),

	/** Aktion zum Anlegen eines Unfalls. */
	UNFALL_ANLEGEN("unfallAnlegen"),

	/** Aktion zum Duplizieren eines Unfalls. */
	UNFALL_DUPLIZIEREN("unfallDuplizieren"),

	/** Aktion zum L&ouml;schen eines Unfalls. */
	UNFALL_LOESCHEN("unfallLoeschen"),

	/** Aktion zum Anzeigen eines Unfalls. */
	UNFALL_BEARBEITEN("unfallBearbeiten"),

	/** Aktion zum Anzeigen eines Staus. */
	STAU_ANZEIGEN("stauAnzeigen");

	/**
	 * liefert die Funktion, die mit der bergebenen Id definiert ist. Wenn
	 * keine entsprechende Funktion gefunden werden konnte liefert die Funktion
	 * den Wert <code>null</code>.
	 * 
	 * @param id
	 *            die Id
	 * @return die Funktion oder <code>null</code>
	 */
	public static EngstellenVerwaltungPluginFunktion getFunktion(final String id) {
		EngstellenVerwaltungPluginFunktion result = null;

		for (final EngstellenVerwaltungPluginFunktion item : EngstellenVerwaltungPluginFunktion
				.values()) {
			if (item.funktion.getFunktionsName().equals(id)) {
				result = item;
				break;
			}
		}

		return result;
	}

	/** Definiert den Freigabezustand der Funktion. */
	private boolean freigegeben;

	/** Die zugeordnete Rahmenwerks-Oberflchenfunktion. */
	private OberflaechenFunktion funktion;

	/**
	 * Die Liste der zu benachrichtigenden Listener, wenn sich der
	 * Freigabezustand der Funktion ndert.
	 */
	private final EventListenerList listeners = new EventListenerList();

	/**
	 * Konstruktor, erzeugt eine Funktionsdefinition basierend auf dem
	 * bergebenen Name.
	 * 
	 * Innerhalb des Rahmenwerks wird die Funktion mit der Dialog-ID, die dem
	 * Name des {@link BauEditorPlugin} entspricht und dem Klassenname der
	 * {@link EngstellenVerwaltungPluginFunktion} erweitert um den bergebenen
	 * Name registriert.
	 * 
	 * @param name
	 *            der Name
	 */
	private EngstellenVerwaltungPluginFunktion(final String name) {
		freigegeben = false;
		funktion = new OberflaechenFunktion(
				EngstellenVerwaltungPluginFunktion.class.getName() + "." + name,
				EngstellenVerwaltungBerechtigungsVerwaltung.getInstanz(),
				BauEditorPlugin.class.getName(), name);
		OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(
				funktion);
	}

	/**
	 * Fgt einen Listener zur Beobachtung des Berechtigungszustands der
	 * Funktion hinzu.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void addFunktionsBerechtigungsListener(
			final FunktionsBerechtigungsListener listener) {
		listeners.add(FunktionsBerechtigungsListener.class, listener);
	}

	/**
	 * Liefert den Freigabezustand der Funktion in der aktuellen Umgebung.
	 * 
	 * @return den Zustand
	 */
	public boolean isFreigegeben() {
		if (!freigegeben) {
			final List<SystemObject> objekt = new ArrayList<SystemObject>();
			objekt.add(DaVVerbindung.getInstanz().getDaVVerbindung()
					.getLocalApplicationObject());
			OberflaechenBerechtigung.getInstanz()
					.pruefeBerechtigungElementGruppe(objekt,
							BauEditorPlugin.class.getName());
		}

		return freigegeben;
	}

	/**
	 * Entfernt einen Listener zur Beobachtung des Berechtigungszustands der
	 * Funktion.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void removeFunktionsBerechtigungsListener(
			final FunktionsBerechtigungsListener listener) {
		listeners.remove(FunktionsBerechtigungsListener.class, listener);
	}

	/**
	 * Setzt den Freigabezustand der Funktion.
	 * 
	 * @param freigegeben
	 *            der Zustand
	 */
	void setFreigegeben(final boolean freigegeben) {
		if (freigegeben != this.freigegeben) {
			this.freigegeben = freigegeben;
			for (final FunktionsBerechtigungsListener listener : listeners
					.getListeners(FunktionsBerechtigungsListener.class)) {
				listener.berechtigungsAenderung();
			}
		}
	}
}
