/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.editors;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.UpdateValueStrategy;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.databinding.viewers.ViewersObservables;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import com.bitctrl.lib.eclipse.databinding.validator.MessageValidator;
import com.bitctrl.lib.eclipse.databinding.validator.StringValidator;

import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttBaustellenVeranlasser;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttStatusBaustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Baustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Situation;

/**
 * Der Baustelleneditor.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: BaustellenEditor.java,v 1.2 2014/11/14 07:29:40 Pittner Exp $
 */
public class BaustellenEditor extends EngstellenEditor {

	/**
	 * Die eindeutige ID des Baustelleneditors.
	 */
	public static final String ID = "de.bsvrz.buv.plugin.baueditor.editor.baustelle"; //$NON-NLS-1$

	/** ComboBox zur Anzeige des Status. */
	private ComboViewer statusComboViewer;

	/** ComboBox zur Anzeige des Veranlassers. */
	private ComboViewer veranlasserComboViewer;

	/** Textfeld zur Eingabe der verantwortlichen Firma. */
	private Text firmaText;

	/** Textfeld zur Eingabe der Telefonnummer der verantwortlichen Firma. */
	private Text telefonFirmaText;

	/** Textfeld zur Eingabe der verantwortlichen Person. */
	private Text verantwortlicherText;

	/** Textfeld zur Eingabe der Telefonnummer der verantwortlichen Person. */
	private Text verantwortlicherTelefonText;

	/** Textfeld zur Eingabe der Mobiltelefonnummer der verantwortlichen Person. */
	private Text verantwortlicherMobilText;

	@Override
	public void init(final IEditorSite site, final IEditorInput editorInput)
			throws PartInitException {

		if (!(editorInput instanceof BaustellenEditorInput)) {
			throw new PartInitException(
					"BaustellenEditor muss ein BaustellenEditorInput bergeben werden.");
		}
		super.init(site, editorInput);
	}

	@Override
	protected void erzeugeKopfBereich(final FormToolkit toolkit,
			final Composite parent) {

		// Namen
		final Text nameText = erzeugeNamensTextFeld(toolkit, parent);
		nameText.addFocusListener(getFocusListener(null));
		nameText.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL, 1, 4));

		final Label statusLabel = toolkit.createLabel(parent, "Status:");
		statusLabel.setBackground(null);
		statusLabel.setLayoutData(new TableWrapData(TableWrapData.LEFT,
				TableWrapData.MIDDLE));

		// ComboBox zur Eingabe des Status
		statusComboViewer = new ComboViewer(parent, SWT.READ_ONLY);
		statusComboViewer.setContentProvider(new ArrayContentProvider());
		statusComboViewer.setInput(AttStatusBaustelle.getZustaende());
		statusComboViewer.getControl().addFocusListener(getFocusListener(null));
		statusComboViewer.getControl().setLayoutData(
				new TableWrapData(TableWrapData.LEFT, TableWrapData.MIDDLE));

		// Dummy Label
		final Label dummyLabel = toolkit.createLabel(parent, "    ");
		dummyLabel.setBackground(null);

		// erzeuge Ereignis-Kopfbereichzeile
		final Composite ereignisComposite = erzeugeEreignisKopfbereichZeile(
				toolkit, parent);
		ereignisComposite.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL));
	}

	@Override
	protected void erzeugeSections(final FormToolkit toolkit,
			final Composite composite) {

		final Section lageSection = erzeugeLageSection(toolkit, composite);
		lageSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL, 1, 2));

		final Control segmenteControl = erzeugeSegmenteSection(toolkit,
				composite);
		segmenteControl.setLayoutData(new TableWrapData(TableWrapData.RIGHT,
				TableWrapData.FILL_GRAB, 3, 1));

		final Section dauerSection = erzeugeDauerSection(toolkit, composite);
		dauerSection.setLayoutData(new TableWrapData(TableWrapData.FILL,
				TableWrapData.FILL));

		final Section fahrstreifenSection = erzeugeFahrstreifenSection(toolkit,
				composite);
		fahrstreifenSection.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL));

		final Section verantwortlicherSection = erzeugeBaustellenVerantwortllicherSection(
				toolkit, composite);
		verantwortlicherSection.setLayoutData(new TableWrapData(
				TableWrapData.FILL, TableWrapData.FILL));

		final Section infoSection = erzeugeInfoSection(toolkit, composite);
		infoSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL));
	}

	/**
	 * Erzeugt den Editorbereich "Baustellenverantwortlicher".
	 * 
	 * @param toolkit
	 *            das Forms-Toolkit
	 * @param parent
	 *            das Parent-Composite
	 * 
	 * @return den Editorbereich
	 */
	private Section erzeugeBaustellenVerantwortllicherSection(
			final FormToolkit toolkit, final Composite parent) {

		final Section section = toolkit.createSection(parent,
				SECTION_STYLE_MIT_TWISTIE);

		section.setText("Baustellenverantwortlicher");

		final Composite composite = toolkit.createComposite(section);
		final TableWrapLayout layout = new TableWrapLayout();
		layout.numColumns = 2;
		layout.topMargin = 5;
		composite.setLayout(layout);
		section.setClient(composite);

		toolkit.createLabel(composite, "Firma:");
		firmaText = toolkit.createText(composite, "", SWT.BORDER);
		firmaText.addFocusListener(getFocusListener(null));
		firmaText.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL));

		toolkit.createLabel(composite, "Telefon:");
		telefonFirmaText = toolkit.createText(composite, "", SWT.BORDER);
		telefonFirmaText.addFocusListener(getFocusListener(null));
		telefonFirmaText.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL));

		final Label separator = toolkit.createLabel(composite, "",
				SWT.SEPARATOR | SWT.HORIZONTAL);
		separator.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL, 1, 2));

		toolkit.createLabel(composite, "Verantwortlicher:");
		verantwortlicherText = toolkit.createText(composite, "", SWT.BORDER);
		verantwortlicherText.addFocusListener(getFocusListener(null));
		verantwortlicherText.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL));

		toolkit.createLabel(composite, "Telefon:");
		verantwortlicherTelefonText = toolkit.createText(composite, "",
				SWT.BORDER);
		verantwortlicherTelefonText.addFocusListener(getFocusListener(null));
		verantwortlicherTelefonText.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL));

		toolkit.createLabel(composite, "Mobil:");
		verantwortlicherMobilText = toolkit.createText(composite, "",
				SWT.BORDER);
		verantwortlicherMobilText.addFocusListener(getFocusListener(null));
		verantwortlicherMobilText.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL));

		return section;
	}

	@Override
	protected void erzeugeInfoSectionErweiterung(final FormToolkit toolkit,
			final Composite parent) {

		final Label veranlasserLabel = toolkit.createLabel(parent,
				"Veranlasser:");
		veranlasserLabel.setLayoutData(new TableWrapData(TableWrapData.FILL,
				TableWrapData.MIDDLE));

		// ComboBox zur Eingabe des Veranlassers
		veranlasserComboViewer = new ComboViewer(parent, SWT.READ_ONLY);
		veranlasserComboViewer.getControl().addFocusListener(
				getFocusListener(null));
		veranlasserComboViewer.setContentProvider(new ArrayContentProvider());
		veranlasserComboViewer
				.setInput(AttBaustellenVeranlasser.getZustaende());
		veranlasserComboViewer.setSelection(new StructuredSelection(
				AttBaustellenVeranlasser.ZUSTAND_2_VRZ_INTERN));
		veranlasserComboViewer.getControl().setLayoutData(
				new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL));
	}

	@Override
	protected void bindValues(final DataBindingContext bindingContext,
			final Realm realm) {

		super.bindValues(bindingContext, realm);

		// Status
		final IObservableValue statusSelection = ViewersObservables
				.observeSingleSelection(statusComboViewer);
		bindingContext.bindValue(statusSelection,
				BeansObservables.observeValue(realm, getWrapper(), "status"),
				null, null);

		// Veranlasser
		final IObservableValue veranlasserSelection = ViewersObservables
				.observeSingleSelection(veranlasserComboViewer);
		bindingContext.bindValue(veranlasserSelection, BeansObservables
				.observeValue(realm, getWrapper(), "veranlasser"), null, null);

		// Verantwortliche Firma
		String errorText = "Bitte geben Sie die verantwortliche Firma ein!";
		bindingContext
				.bindValue(
						SWTObservables.observeText(firmaText, SWT.Modify),
						BeansObservables.observeValue(realm, getWrapper(),
								"verantwortlicheFirma"),
						new UpdateValueStrategy()
								.setAfterConvertValidator(new MessageValidator(
										"firma", getMessageManager(),
										firmaText, new StringValidator(
												errorText)
												.setSeverity(IStatus.WARNING))),
						null);

		// Telefon Firma
		errorText = "Bitte geben Sie die Telefonnummer der verantwortliche Firma ein!";
		bindingContext.bindValue(SWTObservables.observeText(telefonFirmaText,
				SWT.Modify), BeansObservables.observeValue(realm, getWrapper(),
				"telefonFirma"), new UpdateValueStrategy()
				.setAfterConvertValidator(new MessageValidator("telefonFirma",
						getMessageManager(), telefonFirmaText,
						new StringValidator(errorText)
								.setSeverity(IStatus.WARNING))), null);

		// Baustellenverantwortlicher
		errorText = "Bitte geben Sie den Namen des Baustellenverantwortlichen ein!";
		bindingContext.bindValue(SWTObservables.observeText(
				verantwortlicherText, SWT.Modify),
				BeansObservables.observeValue(realm, getWrapper(),
						"baustellenVerantwortlicher"),
				new UpdateValueStrategy()
						.setAfterConvertValidator(new MessageValidator(
								"baustellenVerantwortlicher",
								getMessageManager(), verantwortlicherText,
								new StringValidator(errorText)
										.setSeverity(IStatus.WARNING))), null);

		// Telefon Baustellenverantwortlicher
		errorText = "Bitte geben Sie die Telefonnummer des Baustellenverantwortlichen ein!";
		bindingContext.bindValue(SWTObservables.observeText(
				verantwortlicherTelefonText, SWT.Modify), BeansObservables
				.observeValue(realm, getWrapper(),
						"telefonBaustellenVerantwortlicher"),
				new UpdateValueStrategy()
						.setAfterConvertValidator(new MessageValidator(
								"verantwortlicherTelefonText",
								getMessageManager(),
								verantwortlicherTelefonText,
								new StringValidator(errorText)
										.setSeverity(IStatus.WARNING))), null);

		// Mobiltelefon Baustellenverantwortlicher
		errorText = "Bitte geben Sie die Mobiltelefonnummer des Baustellenverantwortlichen ein!";
		bindingContext.bindValue(SWTObservables.observeText(
				verantwortlicherMobilText, SWT.Modify), BeansObservables
				.observeValue(realm, getWrapper(),
						"mobilTelefonBaustellenVerantwortlicher"),
				new UpdateValueStrategy()
						.setAfterConvertValidator(new MessageValidator(
								"verantwortlicherMobilText",
								getMessageManager(), verantwortlicherMobilText,
								new StringValidator(errorText)
										.setSeverity(IStatus.WARNING))), null);
	}

	@Override
	protected void setEnablement(final boolean enabled) {
		super.setEnablement(enabled);

		// Kopfbereich
		statusComboViewer.getCombo().setEnabled(enabled);
		veranlasserComboViewer.getCombo().setEnabled(false);

		// Bereich Baustellenverantwortlicher
		firmaText.setEnabled(enabled);
		telefonFirmaText.setEnabled(enabled);
		verantwortlicherText.setEnabled(enabled);
		verantwortlicherTelefonText.setEnabled(enabled);
		verantwortlicherMobilText.setEnabled(enabled);
	}

	@Override
	protected String getLoeschInfoText(final String nameEngstelle) {
		return "Die Baustelle '"
				+ nameEngstelle
				+ "' wurde im Datenverteiler gelscht, sodass der dazugehrige Editor geschlossen werden muss.";
	}

	@Override
	protected String getPIDPraefix() {
		return "baustelle."; //$NON-NLS-1$
	}

	@Override
	protected Class<? extends Situation> getEngstellenClass() {
		return Baustelle.class;
	}

	@Override
	protected boolean fuegeZuMengeHinzu(final Situation situation) {
		assert situation instanceof Baustelle;
		return StartupPlugin.getDefault().getDefaultNetz().getBaustellen()
				.add((Baustelle) situation);
	}

	@Override
	protected String getEreignisBeschreibung(final String nameEngstelle) {
		return "Ereignis zur Baustelle '" + nameEngstelle + "'";
	}
}
