/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.editors;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;

import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.StauWrapper;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AtlStauVerlaufPrognoseSchritt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Situation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Stau;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdStauVerlauf;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdStauVerlauf.Daten;

/**
 * Der Staueditor (READ_ONLY).
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: StauEditor.java,v 1.2 2014/11/14 07:29:40 Pittner Exp $
 */
public class StauEditor extends EngstellenEditor implements
		DatensatzUpdateListener {

	/**
	 * Die eindeutige ID des Stau.
	 */
	public static final String ID = "de.bsvrz.buv.plugin.baueditor.editor.stau"; //$NON-NLS-1$

	/** der angezeigte Stau. */
	private Stau stau;
	/** die Schrittweite der Stauprognose. */
	private Text schrittweite;
	/** die prognostizierte Dauer des Staus. */
	private Text dauer;
	/** die erwartete Auflsungszeit des Staus. */
	private Text aufloesungszeit;
	/** die maximale Lnge, die der Stau erreicht. */
	private Text maxlaenge;
	/** der Zeitpunkt, zu dem der Stau die maximale Lnge erreicht. */
	private Text maxlaengezeit;
	/** Tabelle zur Darstellung des Prognoseverlaufs. */
	private TableViewer prognoseDaten;

	@Override
	public void init(final IEditorSite site, final IEditorInput editorInput)
			throws PartInitException {

		if (!(editorInput instanceof StauEditorInput)) {
			throw new PartInitException(
					"StauEditor muss ein StauEditorInput bergeben werden.");
		}

		super.init(site, editorInput);
	}

	@Override
	protected void erzeugeKopfBereich(final FormToolkit toolkit,
			final Composite parent) {

		// Name Textfeld
		final Text nameText = erzeugeNamensTextFeld(toolkit, parent);
		nameText.addFocusListener(getFocusListener(null));
		nameText.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL, 1, 4));

		// Dummy Label
		final Label dummyLabel = toolkit.createLabel(parent, "    ");
		dummyLabel.setBackground(null);

		// Ereignis-Definition
		final Composite ereignisComposite = erzeugeEreignisKopfbereichZeile(
				toolkit, parent);
		ereignisComposite.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL, 1, 3));
	}

	@Override
	protected void erzeugeSections(final FormToolkit toolkit,
			final Composite composite) {

		final Section lageSection = erzeugeLageSection(toolkit, composite);
		lageSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL, 1, 2));

		final Control segmenteControl = erzeugeSegmenteSection(toolkit,
				composite);
		segmenteControl.setLayoutData(new TableWrapData(TableWrapData.RIGHT,
				TableWrapData.FILL_GRAB, 3, 1));

		final Section prognoseSection = erzeugePrognoseSection(toolkit,
				composite);
		prognoseSection.setLayoutData(new TableWrapData(TableWrapData.FILL,
				TableWrapData.FILL));

		final Section prognoseVerlaufSection = erzeugePrognoseVerlaufSection(
				toolkit, composite);
		prognoseVerlaufSection.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL));

		final Section dauerSection = erzeugeDauerSection(toolkit, composite);
		dauerSection.setLayoutData(new TableWrapData(TableWrapData.FILL,
				TableWrapData.FILL));

		final Section infoSection = erzeugeInfoSection(toolkit, composite);
		infoSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL));

		final Section fahrstreifenSection = erzeugeFahrstreifenSection(toolkit,
				composite);
		fahrstreifenSection.setLayoutData(new TableWrapData(TableWrapData.LEFT,
				TableWrapData.FILL));
		fahrstreifenSection.setVisible(false);

		setEnablement(false);
		anmelden();
	}

	/** meldet den Empfang von Stauprognosedaten an. */
	private void anmelden() {
		final StauWrapper stauWrapper = ((StauEditorInput) getEditorInput())
				.getStauWrapper();
		stau = (Stau) stauWrapper.getSituation();
		if (stau != null) {
			stau.getOdStauVerlauf().addUpdateListener(
					OdStauVerlauf.Aspekte.PrognoseNormal, this);
			aktualisierePrognose(stau.getOdStauVerlauf().getDatum(
					OdStauVerlauf.Aspekte.PrognoseNormal));
		}
	}

	@Override
	public void dispose() {
		abmelden();
		super.dispose();
	}

	/** meldet den Empfang von Stauprognosedaten ab. */
	private void abmelden() {
		if (stau != null) {
			stau.getOdStauVerlauf().removeUpdateListener(
					OdStauVerlauf.Aspekte.PrognoseNormal, this);
		}
	}

	@Override
	protected void setEnablement(final boolean enabled) {
		// Der Editor ist immer READ-ONLY!
		super.setEnablement(false);
	}

	@Override
	protected String getLoeschInfoText(final String nameEngstelle) {
		return "Der Stau '"
				+ nameEngstelle
				+ "' wurde im Datenverteiler gelscht, sodass der dazugehrige Editor geschlossen werden muss.";
	}

	@Override
	protected String getPIDPraefix() {
		return "stau."; //$NON-NLS-1$
	}

	@Override
	protected Class<? extends Situation> getEngstellenClass() {
		return Stau.class;
	}

	@Override
	protected boolean fuegeZuMengeHinzu(final Situation situation) {
		assert situation instanceof Stau;
		return StartupPlugin.getDefault().getDefaultNetz().getStaus()
				.add((Stau) situation);
	}

	@Override
	protected String getEreignisBeschreibung(final String nameEngstelle) {
		return "Ereignis zum Stau '" + nameEngstelle + "'";
	}

	/**
	 * Erzeugt die Dauer-Section.
	 * 
	 * @param toolkit
	 *            das Forms-Toolkit
	 * @param parent
	 *            das Parent Composite
	 * 
	 * @return die Section
	 */
	protected Section erzeugePrognoseSection(final FormToolkit toolkit,
			final Composite parent) {

		final Section section = toolkit.createSection(parent,
				SECTION_STYLE_MIT_TWISTIE);
		section.setText("Prognose");

		final Composite composite = toolkit.createComposite(section);
		section.setClient(composite);

		// Beachte: In diesem Bereich wird ausnahmsweise ein GridLayout
		// verwendet, da alle Widgets eine feste Gre haben. Auerdem
		// bentigen die DatumZeit- und ZeitDauer-Widgets ein GridLayout.
		GridLayoutFactory.fillDefaults().numColumns(2).spacing(5, 5)
				.margins(5, 5).equalWidth(false).applyTo(composite);

		toolkit.createLabel(composite, "Schrittweite:");
		schrittweite = toolkit.createText(composite, "");
		schrittweite.setEnabled(false);
		GridDataFactory.fillDefaults().grab(true, false).hint(150, SWT.DEFAULT)
				.applyTo(schrittweite);

		toolkit.createLabel(composite, "Dauer");
		dauer = toolkit.createText(composite, "");
		dauer.setEnabled(false);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(dauer);

		toolkit.createLabel(composite, "Auflsungszeit");
		aufloesungszeit = toolkit.createText(composite, "");
		aufloesungszeit.setEnabled(false);
		GridDataFactory.fillDefaults().grab(true, false)
				.applyTo(aufloesungszeit);

		toolkit.createLabel(composite, "max. Lnge");
		maxlaenge = toolkit.createText(composite, "");
		maxlaenge.setEnabled(false);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(maxlaenge);

		toolkit.createLabel(composite, "Zeit der max. Lnge");
		maxlaengezeit = toolkit.createText(composite, "");
		maxlaengezeit.setEnabled(false);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(maxlaengezeit);

		return section;
	}

	/**
	 * Erzeugt die Dauer-Section.
	 * 
	 * @param toolkit
	 *            das Forms-Toolkit
	 * @param parent
	 *            das Parent Composite
	 * 
	 * @return die Section
	 */
	protected Section erzeugePrognoseVerlaufSection(final FormToolkit toolkit,
			final Composite parent) {

		final Section section = toolkit.createSection(parent,
				SECTION_STYLE_MIT_TWISTIE);
		section.setText("Prognoseverlauf");

		final Composite composite = toolkit.createComposite(section);
		section.setClient(composite);

		GridLayoutFactory.fillDefaults().numColumns(1).spacing(SWT.DEFAULT, 0)
				.equalWidth(false).applyTo(composite);

		final Composite tablePanel = new Composite(composite, SWT.NONE);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(tablePanel);

		// von Zeitpunkt
		final Table prognoseTable = toolkit.createTable(tablePanel,
				SWT.HIDE_SELECTION);
		prognoseTable.setLinesVisible(true);
		prognoseTable.setHeaderVisible(true);

		final TableColumnLayout tabLayout = new TableColumnLayout();
		TableColumn col = new TableColumn(prognoseTable, SWT.NONE);
		col.setText("Zufluss");
		tabLayout.setColumnData(col, new ColumnWeightData(1));
		col = new TableColumn(prognoseTable, SWT.NONE);
		col.setText("Kapazitt");
		tabLayout.setColumnData(col, new ColumnWeightData(1));
		col = new TableColumn(prognoseTable, SWT.NONE);
		col.setText("Lnge");
		tabLayout.setColumnData(col, new ColumnWeightData(1));
		col = new TableColumn(prognoseTable, SWT.NONE);
		col.setText("Verlustzeit");
		tabLayout.setColumnData(col, new ColumnWeightData(1));
		col = new TableColumn(prognoseTable, SWT.NONE);
		col.setText("vKfz");
		tabLayout.setColumnData(col, new ColumnWeightData(1));

		tablePanel.setLayout(tabLayout);

		prognoseDaten = new TableViewer(prognoseTable);
		prognoseDaten.setContentProvider(new ArrayContentProvider());
		prognoseDaten.setLabelProvider(new ITableLabelProvider() {

			@Override
			public void removeListener(final ILabelProviderListener listener) {
			}

			@Override
			public boolean isLabelProperty(final Object element,
					final String property) {
				return false;
			}

			@Override
			public void dispose() {
			}

			@Override
			public void addListener(final ILabelProviderListener listener) {
			}

			@Override
			public String getColumnText(final Object element,
					final int columnIndex) {
				String result = null;
				if (element instanceof AtlStauVerlaufPrognoseSchritt) {
					final AtlStauVerlaufPrognoseSchritt schritt = (AtlStauVerlaufPrognoseSchritt) element;
					switch (columnIndex) {
					case 0:
						result = schritt.getZufluss().toString();
						break;
					case 1:
						result = schritt.getKapazitaet().toString();
						break;
					case 2:
						result = schritt.getLaenge().toString();
						break;
					case 3:
						result = schritt.getVerlustZeit().toString();
						break;
					case 4:
						result = schritt.getVKfz().toString();
						break;
					default:
						break;
					}
					// TODO fllen
				}
				if (result == null) {
					result = "???";
				}
				return result;
			}

			@Override
			public Image getColumnImage(final Object element,
					final int columnIndex) {
				return null;
			}
		});

		GridDataFactory.fillDefaults().grab(true, true).applyTo(prognoseTable);
		return section;
	}

	@Override
	public void datensatzAktualisiert(final DatensatzUpdateEvent event) {
		Display.getDefault().asyncExec(new Runnable() {

			@Override
			public void run() {
				aktualisierePrognose(event.getDatum());
			}
		});
	}

	/**
	 * aktualisiert die Daten der Stauprognose.
	 * 
	 * @param datum
	 *            das Datum mit den Information zum Stau
	 */
	private void aktualisierePrognose(final OnlineDatum datum) {

		if ((schrittweite != null) && (!schrittweite.isDisposed())) {

			if (datum instanceof OdStauVerlauf.Daten) {
				final OdStauVerlauf.Daten daten = (Daten) datum;
				schrittweite.setText(daten.getSchrittweite().toString());
				dauer.setText(daten.getDauer().toString());
				aufloesungszeit.setText(daten.getAufloesungsZeit().toString());
				maxlaenge.setText(daten.getMaxLaenge().toString());
				maxlaengezeit.setText(daten.getMaxLaengeZeit().toString());

				prognoseDaten.setInput(daten.getPrognoseverlauf().toArray());

			} else {
				schrittweite.setText("");
				dauer.setText("");
				aufloesungszeit.setText("");
				maxlaenge.setText("");
				maxlaengezeit.setText("");
				prognoseDaten.setInput(new AtlStauVerlaufPrognoseSchritt[0]);
			}
		}
	}
}
