/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.graph.figures;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.ui.ISelectionListener;

import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenKnoten;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnitt;

/**
 * Figure, welches das {@link StrassenKnotenFigure} um die Darstellung des
 * Namens eines Straenknotens ergnzt.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: StrassenKnotenFigureMitLabel.java,v 1.2 2014/11/14 07:29:40 Pittner Exp $
 * 
 * @see StrassenKnotenFigure
 */
public class StrassenKnotenFigureMitLabel extends Figure {

	/**
	 * Gekapseltes {@link StrassenKnotenFigure}.
	 */
	private final StrassenKnotenFigure knotenFigure;

	/**
	 * {@link Label}-Figure fr die erste Zeile.
	 */
	private final Label label1;

	/**
	 * {@link Label}-Figure fr eine mgliche zweite Zeile.
	 */
	private final Label label2;

	/**
	 * Der Konstruktor.
	 * 
	 * @param knoten
	 *            der Straenknoten
	 * @param streckenAbschnitt
	 *            der Streckenabschnitt, auf welchem sich der Straenknoten
	 *            befindet
	 */
	public StrassenKnotenFigureMitLabel(final StrassenKnoten knoten,
			final StreckenAbschnitt streckenAbschnitt) {

		knotenFigure = new StrassenKnotenFigure(knoten, streckenAbschnitt);

		final GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 15;
		gridLayout.verticalSpacing = 2;
		setLayoutManager(gridLayout);

		// Straenknoten-Figure
		final GridData gridData = new GridData(30, 30);
		gridData.verticalSpan = 2;
		add(knotenFigure, gridData);

		// Label 1
		final GridData constraint = new GridData(180, 12);
		constraint.horizontalAlignment = SWT.BEGINNING;
		constraint.verticalAlignment = SWT.TOP;
		label1 = new Label();
		label1.setForegroundColor(ColorConstants.black);
		label1.setBackgroundColor(ColorConstants.lightGray);
		add(label1, constraint);

		// Label 2
		label2 = new Label();
		label2.setForegroundColor(ColorConstants.black);
		label2.setBackgroundColor(ColorConstants.lightGray);
		add(label2, constraint);

		setLabelText(knoten.getName());
	}

	/**
	 * Setzt den Text der Labels.
	 * 
	 * @param name
	 *            der Text
	 */
	private void setLabelText(final String name) {
		String text = name;
		if (name.contains(" [")) {
			text = name.substring(0, name.indexOf(" [") - 1);
		}
		if (text.contains(" ")) {
			final int size = text.length();
			int fromIndex = 0;
			final List<Integer> indexOfSpaces = new ArrayList<Integer>();
			while (true) {
				final int index = text.indexOf(" ", fromIndex + 1);
				if (index == -1) {
					break;
				}
				indexOfSpaces.add(index);
				fromIndex = index;
			}

			int bestSpaceIndex = 0;
			int maxLength = Integer.MAX_VALUE;
			for (final Integer spaceIndex : indexOfSpaces) {
				final int currentMaxLength = Math.max(spaceIndex, size
						- spaceIndex);
				if (currentMaxLength < maxLength) {
					maxLength = currentMaxLength;
					bestSpaceIndex = spaceIndex;
				}
			}

			final String text1 = text.substring(0, bestSpaceIndex);
			final String text2 = text.substring(bestSpaceIndex + 1);
			label1.setText(text1);
			label2.setText(text2);
		} else {
			label1.setText(text);
			label2.setText("");
		}
	}

	/**
	 * Liefert den ConnectionAnchor dieses Figures zurck.
	 * 
	 * @return den ConnectionAnchor dieses Figures
	 */
	public ConnectionAnchor getConnectionAnchor() {
		return new ChopboxAnchor(knotenFigure);
	}

	@Override
	public void setFont(final Font font) {
		label1.setFont(font);
		label2.setFont(font);
	}

	/**
	 * Fgt dem Figure einen {@link ISelectionListener} hinzu.
	 * 
	 * @param listener
	 *            der {@link ISelectionListener}
	 */
	public void addSelectionListener(final ISelectionListener listener) {
		knotenFigure.addSelectionListener(listener);
	}

	/**
	 * Entfernt dem Figure einen {@link ISelectionListener}.
	 * 
	 * @param listener
	 *            der {@link ISelectionListener}
	 */
	public void removeSelectionListener(final ISelectionListener listener) {
		knotenFigure.removeSelectionListener(listener);
	}
}
