/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.util.wrapper;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.Set;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorException;
import de.bsvrz.buv.plugin.baueditor.sts.Konstanten;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Feld;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttFahrStreifenLage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Baustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Stau;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Unfall;
import de.bsvrz.sys.funclib.bitctrl.modell.util.ObjektMitZeitBereich;

/**
 * Abstrakte Wrapper-Klasse fr Engstellen.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: EngstellenWrapper.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 */
public abstract class EngstellenWrapper extends SituationsWrapper implements
		ObjektMitZeitBereich {

	/** Menge der gesperrten Fahrstreifen. */
	private Set<AttFahrStreifenLage> lageGesperrterFahrstreifen;

	/** Die Restkapazitt pro Fahrstreifen. */
	private int restKapazitaetProFahrstreifen;

	/** Die Engpasskapazitt. */
	private int engpassKapazitaet;

	public static final String END_BETRIEBS_KILOMETER_BLOCK_NUMMER = "endBetriebsKilometerBlockNummer";

	public static final String START_BETRIEBS_KILOMETER_BLOCK_NUMMER = "startBetriebsKilometerBlockNummer";

	public static final String END_BETRIEBS_KILOMETER_BEREICH = "endBetriebsKilometerBereich";

	public static final String END_BETRIEBS_KILOMETER = "endBetriebsKilometer";

	public static final String START_BETRIEBS_KILOMETER_BEREICH = "startBetriebsKilometerBereich";

	public static final String START_BETRIEBS_KILOMETER = "startBetriebsKilometer";

	public static final String MAX_END_OFFSET = "maxEndOffset";

	public static final String END_OFFSET = "endOffset";

	public static final String MAX_START_OFFSET = "maxStartOffset";

	public static final String START_OFFSET = "startOffset";

	public static final String ENGPASS_KAPAZITAET = "engpassKapazitaet";

	public static final String REST_KAPAZITAET_PRO_FAHRSTREIFEN = "restKapazitaetProFahrstreifen";

	public static final String LAGE_GESPERRTER_FAHRSTREIFEN = "lageGesperrterFahrstreifen";

	public static final String ANZAHL_FAHRSTREIFEN = "anzahlFahrstreifen";

	public static final String END_KNOTEN = "endKnoten";

	public static final String START_KNOTEN = "startKnoten";

	public static final String RICHTUNGS_AUSWAHL_MOEGLICH = "richtungsAuswahlMoeglich";

	public static final String RICHTUNG = "richtung";

	public static final String STRASSE = "strasse";

	public static final String BESCHREIBUNG = "beschreibung";

	public static final String KURZ_INFO = "kurzInfo";

	public static final String NEUES_ELEMENT = "neuesElement";

	public static final String DAUER = "dauer";

	public static final String END_ZEIT = "endZeit";

	public static final String EREIGNIS_TYP = "ereignisTyp";

	public static final String NAME = "name";

	public static final String STRECKEN_ABSCHNITT = "streckenAbschnitt";

	public static final String STRASSEN_SEGMENTE = "strassenSegmente";

	public static final String EREIGNIS_WARNUNG = "ereignisWarnung";

	public static final String VERKNUEPFE_EREIGNIS = "verknuepfeEreignis";

	public static final String START_ZEIT = "startZeit";

	public static final String EREIGNIS = "ereignis";

	/**
	 * Der Konstruktor.
	 * 
	 * @param strasse
	 *            die Strae, kann <code>null</code> sein
	 * @param richtung
	 *            die Richtung, kann <code>null</code> sein
	 * @param segment
	 *            das Straensegment, kann <code>null</code> sein
	 */
	public EngstellenWrapper(final Strasse strasse,
			final AttTmcRichtung richtung, final StrassenSegment segment) {
		super(strasse, richtung, segment);

		lageGesperrterFahrstreifen = new HashSet<AttFahrStreifenLage>();
		restKapazitaetProFahrstreifen = Konstanten.STANDARDWERT_RESTKAPAZITAET;
		ermittelEngpassKapazitaet();
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param stau
	 *            ein Baustellen-SystemObjekt
	 * 
	 * @throws BauEditorException
	 *             wird geworfen, wenn die gegebene Baustelle nicht geladen
	 *             werden konnte
	 */
	public EngstellenWrapper(final Stau stau) throws BauEditorException {
		super(stau);
		lageGesperrterFahrstreifen = new HashSet<AttFahrStreifenLage>();
		engpassKapazitaet = 0;
		restKapazitaetProFahrstreifen = Konstanten.STANDARDWERT_RESTKAPAZITAET;
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param baustelle
	 *            ein Baustellen-SystemObjekt
	 * 
	 * @throws BauEditorException
	 *             wird geworfen, wenn die gegebene Baustelle nicht geladen
	 *             werden konnte
	 */
	public EngstellenWrapper(final Baustelle baustelle)
			throws BauEditorException {
		super(baustelle);

		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdBaustellenEigenschaftenErweitert.Daten pdEigenschaftenErweitert = baustelle
				.getPdBaustellenEigenschaftenErweitert().getDatum();
		setZeitStempel(pdEigenschaftenErweitert.dGetZeitstempel());
		if (pdEigenschaftenErweitert != null
				&& pdEigenschaftenErweitert.getLageGesperrteFahrstreifen() != null) {
			initGesperrteFahrstreifenSet(pdEigenschaftenErweitert
					.getLageGesperrteFahrstreifen());
		} else {
			lageGesperrterFahrstreifen = new HashSet<AttFahrStreifenLage>();
		}

		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdBaustellenEigenschaften.Daten pdEigenschaften = baustelle
				.getPdBaustellenEigenschaften().getDatum();
		setZeitStempel(pdEigenschaften.dGetZeitstempel());
		if (pdEigenschaften != null
				&& pdEigenschaften.getRestKapazitaet() != null) {
			engpassKapazitaet = pdEigenschaften.getRestKapazitaet().intValue();
			ermittelRestkapazitaetProFahrstreifen();
		} else {
			restKapazitaetProFahrstreifen = Konstanten.STANDARDWERT_RESTKAPAZITAET;
			ermittelEngpassKapazitaet();
		}
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param unfall
	 *            ein Unfall-SystemObjekt
	 * 
	 * @throws BauEditorException
	 *             wird geworfen, wenn der gegebene Unfall nicht geladen werden
	 *             konnte
	 */
	public EngstellenWrapper(final Unfall unfall) throws BauEditorException {
		super(unfall);

		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdUnfallEigenschaftenErweitert.Daten pdEigenschaftenErweitert = unfall
				.getPdUnfallEigenschaftenErweitert().getDatum();
		setZeitStempel(pdEigenschaftenErweitert.dGetZeitstempel());
		if (pdEigenschaftenErweitert != null
				&& pdEigenschaftenErweitert.getLageGesperrteFahrstreifen() != null) {
			initGesperrteFahrstreifenSet(pdEigenschaftenErweitert
					.getLageGesperrteFahrstreifen());
		} else {
			lageGesperrterFahrstreifen = new HashSet<AttFahrStreifenLage>();
		}

		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdUnfallEigenschaften.Daten pdEigenschaften = unfall
				.getPdUnfallEigenschaften().getDatum();
		setZeitStempel(pdEigenschaften.dGetZeitstempel());
		if (pdEigenschaften != null
				&& pdEigenschaften.getRestKapazitaet() != null) {
			engpassKapazitaet = pdEigenschaften.getRestKapazitaet().intValue();
			ermittelRestkapazitaetProFahrstreifen();
		} else {
			restKapazitaetProFahrstreifen = Konstanten.STANDARDWERT_RESTKAPAZITAET;
			ermittelEngpassKapazitaet();
		}
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param wrapper
	 *            ein anderer EngstellenWrapper
	 * @param neuesElement
	 *            <code>true</code>, wenn es sich bei dem neu erstellten
	 *            EngstellenWrapper, um eine neue Engstelle handelt. false, wenn
	 *            ein Klon erstellt werden soll.
	 */
	public EngstellenWrapper(final EngstellenWrapper wrapper,
			final boolean neuesElement) {
		super(wrapper, neuesElement);

		engpassKapazitaet = wrapper.engpassKapazitaet;
		restKapazitaetProFahrstreifen = wrapper.restKapazitaetProFahrstreifen;
		lageGesperrterFahrstreifen = new HashSet<AttFahrStreifenLage>(
				wrapper.lageGesperrterFahrstreifen);
	}

	/**
	 * Initialisiert die gesperrten Fahrstreifen.
	 * 
	 * @param attFahrStreifenLage
	 *            die Fahrstreifen
	 */
	private void initGesperrteFahrstreifenSet(
			final Feld<AttFahrStreifenLage> attFahrStreifenLage) {

		lageGesperrterFahrstreifen = new HashSet<AttFahrStreifenLage>();
		for (final AttFahrStreifenLage lage : attFahrStreifenLage) {
			if (!lageGesperrterFahrstreifen.contains(lage)) {
				lageGesperrterFahrstreifen.add(lage);
			}
		}
	}

	/**
	 * Liefert die durchschnittliche Restkapazitt pro Fahrstreifen zurck.
	 * 
	 * @return die durchschnittliche Restkapazitt pro Fahrstreifen
	 */
	public int getRestKapazitaetProFahrstreifen() {
		return restKapazitaetProFahrstreifen;
	}

	/**
	 * Setzt die durchschnittliche Restkapazitt pro Fahrstreifen.
	 * 
	 * @param neuerWert
	 *            die neue durchschnittliche Restkapazitt pro Fahrstreifen
	 */
	public void setRestKapazitaetProFahrstreifen(final int neuerWert) {
		final int alterWert = restKapazitaetProFahrstreifen;
		restKapazitaetProFahrstreifen = neuerWert;
		firePropertyChange("restKapazitaetProFahrstreifen", alterWert,
				neuerWert);
		ermittelEngpassKapazitaet();
	}

	/**
	 * Ermittelt die Restkapazitt pro Fahrstreifen. Die Restkapazitt pro
	 * Fahrstreifen entspricht dem Quotienten aus Engpasskapazitt und der
	 * Anzahl freier Fahrstreifen.
	 */
	private void ermittelRestkapazitaetProFahrstreifen() {
		restKapazitaetProFahrstreifen = 0;

		if (engpassKapazitaet == 0) {
			return;
		}
		final int anzahlFreierFahrstreifen = getAnzahlFreierFahrstreifen();
		if (anzahlFreierFahrstreifen > 0) {
			restKapazitaetProFahrstreifen = (engpassKapazitaet / anzahlFreierFahrstreifen);
		} else {
			restKapazitaetProFahrstreifen = engpassKapazitaet;
		}
	}

	/**
	 * Liefert die Engpasskapazitt dieser Engstelle zurck.
	 * 
	 * @return die Engpasskapazitt dieser Engstelle
	 */
	public int getEngpassKapazitaet() {
		return engpassKapazitaet;
	}

	/**
	 * Setzt die Engpasskapazitt dieser Engstelle.
	 * 
	 * @param neuerWert
	 *            die neue Engpasskapazitt
	 */
	public void setEngpassKapazitaet(final int neuerWert) {
		final int alterWert = engpassKapazitaet;
		engpassKapazitaet = neuerWert;
		firePropertyChange("engpassKapazitaet", alterWert, neuerWert);
	}

	/**
	 * Ermittelt die Engpasskapazitt. Die Engpasskapazitt entspricht dem
	 * Produkt aus Restkapazitt pro Fahrstreifen und der Anzahl freier
	 * Fahrstreifen.
	 */
	private void ermittelEngpassKapazitaet() {
		final int anzahlFreierFahrstreifen = getAnzahlFreierFahrstreifen();
		if (anzahlFreierFahrstreifen >= 0) {
			setEngpassKapazitaet(anzahlFreierFahrstreifen
					* restKapazitaetProFahrstreifen);
		} else {
			setEngpassKapazitaet(restKapazitaetProFahrstreifen);
		}
	}

	/**
	 * Liefert eine Menge zurck, welche die Lage der gesperrter Fahrstreifen
	 * angibt.
	 * 
	 * @return eine Menge, welche die Lage der gesperrter Fahrstreifen angibt
	 */
	public Set<AttFahrStreifenLage> getLageGesperrterFahrstreifen() {
		return lageGesperrterFahrstreifen;
	}

	/**
	 * Setzt die Lagen der gesperrten Fahrstreifen.
	 * 
	 * @param neuerWert
	 *            die neuen Lagen der gesperrten Fahrstreifen
	 */
	public void setLageGesperrterFahrstreifen(
			final Set<AttFahrStreifenLage> neuerWert) {
		final Set<AttFahrStreifenLage> alterWert = new HashSet<AttFahrStreifenLage>(
				lageGesperrterFahrstreifen);
		lageGesperrterFahrstreifen = neuerWert;
		firePropertyChange("lageGesperrterFahrstreifen", alterWert, neuerWert);
		ermittelEngpassKapazitaet();
	}

	/**
	 * Liefert die Anzahl der freien Fahrstreifen zurck.
	 * 
	 * @return die Anzahl der freien Fahrstreifen, oder <code>-1</code>, wenn
	 *         die Anzahl unbekannt ist.
	 */
	private int getAnzahlFreierFahrstreifen() {
		if (getStreckenAbschnitt() == null) {
			return -1;
		}

		final byte anzahlFahrstreifen = getStreckenAbschnitt()
				.getAnzahlFahrstreifen();
		if (anzahlFahrstreifen < 1 || lageGesperrterFahrstreifen == null) {
			return -1;
		}
		return anzahlFahrstreifen - lageGesperrterFahrstreifen.size();
	}

	@Override
	protected void fuegeStreckenAbschnittListenerHinzu() {
		super.fuegeStreckenAbschnittListenerHinzu();
		ermittelEngpassKapazitaet();
		if (getStreckenAbschnitt() != null) {
			getStreckenAbschnitt().addPropertyChangeListener(
					"anzahlFahrstreifen", new PropertyChangeListener() {
						public void propertyChange(final PropertyChangeEvent evt) {
							ermittelEngpassKapazitaet();
						}
					});
		}
	}

	@Override
	public String toString() {
		return getName();
	}

	@Override
	public String getInfo() {
		final StringBuffer buf = new StringBuffer(super.getInfo());

		buf.append("\nRestkapazitt pro Fahrstreifen: ");
		buf.append(restKapazitaetProFahrstreifen);
		buf.append("\nEngpasskapazitt: ");
		buf.append(engpassKapazitaet);
		buf.append("\nGesperrter Fahrstreifen:");
		for (final AttFahrStreifenLage lage : lageGesperrterFahrstreifen) {
			buf.append("\n\t");
			buf.append(lage);
		}

		return buf.toString();
	}
}
