/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.TableColumn;

/**
 * Bean, welches die Einstellung der Ansicht 'Engstellen' speichert. Dies
 * beinhaltet die Spaltenbreiten, die Spaltensichtbarkeiten, die Sortierung
 * sowie die CheckStates der Aktionen.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: EngstellenViewPartEinstellungen.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 */
public class EngstellenViewPartEinstellungen extends
		AbstractBauEditorViewPartEinstellungen {

	/** Der Index der Tabellenspalte nach welcher sortiert wird. */
	private int sortierSpaltenIndex;

	/** Die Sortierreihenfolge. */
	private int sortierReihenfolge;

	/** Liste mit den Breiten der einzelnen Spalten. */
	private List<Integer> spaltenBreiten;

	/** Liste mit den Sichtbarkeiten der einzelnen Spalten. */
	private List<Boolean> spaltenSichtbarkeit;

	/**
	 * Der Default-Konstruktor.
	 */
	public EngstellenViewPartEinstellungen() {
		super();
		sortierSpaltenIndex = 0;
		sortierReihenfolge = EngstellenTableSorter.AUFSTEIGEND;
		spaltenBreiten = new ArrayList<Integer>();
		spaltenSichtbarkeit = new ArrayList<Boolean>();
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param viewer
	 *            der TableViewer
	 */
	public EngstellenViewPartEinstellungen(final TableViewer viewer) {
		this();
		final TableColumn[] spalten = viewer.getTable().getColumns();
		for (final TableColumn spalte : spalten) {
			spaltenBreiten.add(spalte.getWidth());
			spaltenSichtbarkeit.add(spalte.getResizable());
		}
	}

	/**
	 * Liefert den Index der Spalte zurck, nach welcher sortiert werden soll.
	 * 
	 * @return den Index der Spalte, nach welcher sortiert werden soll
	 */
	public int getSortierSpaltenIndex() {
		return sortierSpaltenIndex;
	}

	/**
	 * Liefert die Sortierreihenfolge zurck.
	 * 
	 * @return die Sortierreihenfolge
	 */
	public int getSortierReihenfolge() {
		return sortierReihenfolge;
	}

	/**
	 * Liefert die Spaltenbreiten zurck.
	 * 
	 * @return die Spaltenbreiten
	 */
	public List<Integer> getSpaltenBreiten() {
		return spaltenBreiten;
	}

	/**
	 * Liefert die Spaltensichtbarkeiten zurck.
	 * 
	 * @return die Spaltensichtbarkeiten
	 */
	public List<Boolean> getSpaltenSichtbarkeit() {
		return spaltenSichtbarkeit;
	}

	/**
	 * Setzt den Index der Spalte, nach welcher sortiert werden soll.
	 * 
	 * @param sortierSpaltenIndex
	 *            der Index der Spalte, nach welcher sortiert werden soll
	 */
	public void setSortierSpaltenIndex(final int sortierSpaltenIndex) {
		this.sortierSpaltenIndex = sortierSpaltenIndex;
	}

	/**
	 * Setzt die Sortierreihenfolge.
	 * 
	 * @param sortierReihenfolge
	 *            die Sortierreihenfolge
	 */
	public void setSortierReihenfolge(final int sortierReihenfolge) {
		this.sortierReihenfolge = sortierReihenfolge;
	}

	/**
	 * Setzt die Spaltenbreiten.
	 * 
	 * @param spaltenBreiten
	 *            die Spaltenbreiten
	 */
	public void setSpaltenBreiten(final List<Integer> spaltenBreiten) {
		this.spaltenBreiten = spaltenBreiten;
	}

	/**
	 * Setzt die Spaltensichtbarkeiten.
	 * 
	 * @param spaltenSichtbarkeit
	 *            die Spaltensichtbarkeiten
	 */
	public void setSpaltenSichtbarkeit(final List<Boolean> spaltenSichtbarkeit) {
		this.spaltenSichtbarkeit = spaltenSichtbarkeit;
	}
}
