/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.views.helper;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.buv.plugin.baueditor.sts.BildDateiPfade;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;

/**
 * Hilfsklasse fr Straenknoten der Ansicht "Engstellen-Navigator".
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: StrassenHelper.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 */
public final class StrassenHelper extends Helper {

	/** Das Straenobjekt, welches durch diesen Helper gekapselt wird. */
	private final Strasse strasse;

	/**
	 * Map, welche zu jedem Parent- und Strae-Objekt den dazugehrigen
	 * StrassenHelper speichert.
	 */
	private static Map<Object, Map<Strasse, StrassenHelper>> map;

	/**
	 * Liefert ein Straenobjekt fr die Ansicht "Engstellen-Navigator" zurck.
	 * 
	 * @param parent
	 *            das Parent-Objekt
	 * @param strasse
	 *            die Strasse
	 * 
	 * @return ein Straenobjekt fr die Ansicht "Engstellen-Navigator"
	 */
	public static StrassenHelper getInstanz(final Object parent,
			final Strasse strasse) {
		if (map == null) {
			map = new HashMap<Object, Map<Strasse, StrassenHelper>>();
		}
		Map<Strasse, StrassenHelper> helperMap = map.get(parent);
		if (helperMap == null) {
			helperMap = new HashMap<Strasse, StrassenHelper>();
			map.put(parent, helperMap);
		}

		StrassenHelper helper = helperMap.get(strasse);
		if (helper == null) {
			helper = new StrassenHelper(parent, strasse);
			helperMap.put(strasse, helper);
		}
		return helper;
	}

	/**
	 * Der private Konstruktor.
	 * 
	 * @param parent
	 *            das Parent-Objekt
	 * @param strasse
	 *            die Strae, welche von diesem Helper gekapselt wird
	 */
	private StrassenHelper(final Object parent, final Strasse strasse) {
		super(parent);
		this.strasse = strasse;
	}

	/**
	 * Liefert die Strae dieses Helpers zurck.
	 * 
	 * @return die Strae
	 */
	public Strasse getStrasse() {
		return strasse;
	}

	@Override
	public String getImagePath() {
		return BildDateiPfade.STRASSE;
	}

	@Override
	public String toString() {
		return strasse.getName();
	}
}
