/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.parameter.tsf.mqParaSteuerung;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchPage;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.parameter.tsf.Activator;
import de.bsvrz.buv.plugin.parameter.tsf.DaVKommunikation;
import de.bsvrz.buv.plugin.parameter.tsf.mqZuordnung.MqZuweisungViewParameter;
import de.bsvrz.buv.plugin.parameter.tsf.mqZuordnung.MqZuweisungViewParameter2;
import de.bsvrz.buv.plugin.parameter.tsf.mqZuordnung.MqZuweisungViewParameter3;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Nach Auswahl des Menpunkts "StreckenBeeinflussungsAnlage/Parameter Quer-Lngsabgleich" in der Rahmenapplikation 
 * ffnet sich das Fenster zur Auswahl der AnzeigeQuerschnitte zum Quer-Lngsabgleich.
 * 
 * @author Krause
 *
 */
public class MqParaSteuerungAuswahlViewParameter extends ViewPart //implements IDaVDataUpdateListener
{

  public static String ID = "de.bsvrz.buv.plugin.parameter.tsf.mqParaSteuerung.mqParaSteuerungAuswahlView";

  /**
   * Objekttyp des DAV, der bercksichtigt wird
   */
  private static final String _objektTyp = "typ.sswSbaUeSeitenstreifenFreigabe"; 

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();
  
  private Composite _parent;
  
  private Composite compositeTop;

  private Group _gruppeUebersicht;

  private Table _tabelleKmdPar;

  private TableViewer _tableViewerKmdPar;

  private ISelectionChangedListener _selectionChangedListener;

  private Composite compositeBottom;

  private Button buttonOk;

  private Button buttonAbbrechen;

//  private int _numViewInstances = 0;

  private ClientDavInterface _connection;

  private Button buttonHilfe;

  private Composite _compositeRight;

  private Button _buttonTsfAblauf;

  private Button _buttonMqZuordnung;

  private Button _buttonEngstelleZuordnung;

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

//    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".mqParaSteuerungAuswahl" );

    _parent.setLayout(new GridLayout());
    createCompositeTop(_parent);   
    createCompositeBottom(_parent);
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    SystemObjectType type = _connection.getDataModel().getType(_objektTyp);
    List<SystemObject> aqs = type.getElements();


    List<SystemObject> list = new ArrayList<SystemObject>();
    list.addAll(aqs);

    Map<String, SystemObject> treeMap = new TreeMap<String, SystemObject>();
    for (SystemObject so : list)
    {
      treeMap.put(so.getName(), so);
    }

    List<SystemObject> sortList = new ArrayList<SystemObject>();
    sortList.addAll(treeMap.values());

//    System.out.println(sortList.size());
    
    _tableViewerKmdPar.setInput(sortList);

  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * Erzeugt gruppeUebersicht
   * 
   */
  private void createCompositeTop(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(parent, SWT.NONE);
    compositeTop.setLayout(new GridLayout(2, false));
    compositeTop.setLayoutData(gridData);
    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;
    GridData gridDataGruppeUebersicht = new GridData();
    gridDataGruppeUebersicht.horizontalAlignment = GridData.FILL;
    gridDataGruppeUebersicht.grabExcessHorizontalSpace = true;
    gridDataGruppeUebersicht.grabExcessVerticalSpace = true;
    gridDataGruppeUebersicht.verticalAlignment = GridData.FILL;
    _gruppeUebersicht = new Group(compositeTop, SWT.NONE);
    _gruppeUebersicht.setLayout(new GridLayout());
    _gruppeUebersicht.setLayoutData(gridDataGruppeUebersicht);
    _gruppeUebersicht.setText("TSF-Messquerschnitte:");
    _tabelleKmdPar = new Table(_gruppeUebersicht, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
    _tabelleKmdPar.setHeaderVisible(true);
    _tabelleKmdPar.setLayoutData(gridDataTabelle);
    _tabelleKmdPar.setLinesVisible(true);
    _tableViewerKmdPar = new TableViewer(_tabelleKmdPar);
    // _tableViewerKmdPar = new CheckboxTableViewer(_tabelleKmdPar);

    TableColumn tcKey = new TableColumn(_tabelleKmdPar, SWT.LEFT);
    tcKey.setText("Name:");
    tcKey.setWidth(250);

    TableColumn tcValue = new TableColumn(_tabelleKmdPar, SWT.LEFT);
    tcValue.setText("Pid:");
    tcValue.setWidth(280);

    _tableViewerKmdPar.setContentProvider(new ContentProvider());
    _tableViewerKmdPar.setLabelProvider(new LabelProvider());

    _selectionChangedListener = new ISelectionChangedListener() {

      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        
        if (selection == null || selection.isEmpty()) {
          _buttonTsfAblauf.setEnabled(false);
          _buttonMqZuordnung.setEnabled(false);
          _buttonEngstelleZuordnung.setEnabled(false);
        }
        else {
          _buttonTsfAblauf.setEnabled(true);
          _buttonMqZuordnung.setEnabled(true);
          _buttonEngstelleZuordnung.setEnabled(true);
        }
        
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof SystemObject)
        {

        }
      }

    };
    _tableViewerKmdPar.addSelectionChangedListener(_selectionChangedListener);
    _tableViewerKmdPar.addDoubleClickListener(new IDoubleClickListener() {

      @Override
      public void doubleClick(DoubleClickEvent event)
      {
        createViewFromSelection();       
      }


    });
    
    createCompositeRight();
  }
  
  
  /**
   * This method initializes _compositeRight
   * 
   */
  private void createCompositeRight()
  {
    RowLayout rowLayout2 = new RowLayout();
    rowLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout2.wrap = false;
    rowLayout2.pack = false;
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.CENTER;
    gridData6.grabExcessVerticalSpace = true;
    gridData6.verticalAlignment = GridData.FILL;
    
    gridData6.verticalIndent = 3;
    
    _compositeRight = new Composite(compositeTop, SWT.NONE);
    _compositeRight.setLayoutData(gridData6);
    _compositeRight.setLayout(rowLayout2);
    
    _buttonTsfAblauf = new Button(_compositeRight, SWT.NONE);
    _buttonTsfAblauf.setText("Parameter MQ-Zuordnung");
    _buttonTsfAblauf.addMouseListener(new MouseAdapter() {
    
    @Override
    public void mouseDown(MouseEvent e)
    {
      if (_tableViewerKmdPar.getTable().getSelectionCount() > 0) {
        createViewFromSelection(MqZuweisungViewParameter.ID, 650, 400);        
      }
      else {
        MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
            mb1.setText( "Fehler" );
            mb1.setMessage( "Es ist keine TSF-UrsachenEinheit selektiert!" );
            mb1.open(); 
      }
    }
  
    });
    _buttonTsfAblauf.setEnabled(false);
      
    _buttonMqZuordnung = new Button(_compositeRight, SWT.NONE);
    _buttonMqZuordnung.setText("Parameter Input-Zuordnung");
    _buttonMqZuordnung.addMouseListener(new MouseAdapter() {
    
    @Override
    public void mouseDown(MouseEvent e)
    {
      if (_tableViewerKmdPar.getTable().getSelectionCount() > 0) {
           createViewFromSelection(MqZuweisungViewParameter2.ID, 650, 400);        
      }
      else {
        MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
            mb1.setText( "Fehler" );
            mb1.setMessage( "Es ist keine TSF-UrsachenEinheit selektiert!" );
            mb1.open(); 
      }
    }
  
    });
    _buttonMqZuordnung.setEnabled(false);
    
    _buttonEngstelleZuordnung = new Button(_compositeRight, SWT.NONE);
    _buttonEngstelleZuordnung.setText("Parameter Engstelle-Zuordnung");
    _buttonEngstelleZuordnung.addMouseListener(new MouseAdapter() {
      
      @Override
      public void mouseDown(MouseEvent e)
      {
        if (_tableViewerKmdPar.getTable().getSelectionCount() > 0) {
          createViewFromSelection(MqZuweisungViewParameter3.ID, 650, 520);        
        }
        else {
          MessageBox mb1 = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
          mb1.setText( "Fehler" );
          mb1.setMessage( "Es ist keine TSF-UrsachenEinheit selektiert!" );
          mb1.open(); 
        }
      }
      
    });
    _buttonEngstelleZuordnung.setEnabled(false);
   
  }
  
  /**
   * Aufruf der ViewParameter fr die selektierten Objekte 
   * 
   */
  private void createViewFromSelection(String viewId, int width, int height)
  {
    IStructuredSelection selection = (IStructuredSelection)_tableViewerKmdPar.getSelection();
    
    Object firstElement = selection.getFirstElement();
    if (firstElement instanceof SystemObject)
    {      
        Object[] array = selection.toArray();
        List<SystemObject> listObj = new ArrayList<SystemObject>();
        for (Object object : array)
        {
          SystemObject so = (SystemObject)object;
          listObj.add(so);
        }
        
        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = dw.getActivePage();
        try
        {
          if (listObj.size() > 0){
            
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < listObj.size(); i++)
            {
              SystemObject so = listObj.get(i);
              if ( i < listObj.size()-1 )
                sb.append(so.getPid() + ";");
              else
                sb.append(so.getPid());
              
            }
            
            page.showView(viewId, sb.toString(), IWorkbenchPage.VIEW_CREATE);
            IViewReference ref = page.findViewReference(viewId, sb.toString());
            ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
            IWorkbenchPart wbPart = ref.getPart(false);
            Shell shell = wbPart.getSite().getShell();
            shell.setSize(width, height);
                         
          }
          
        }
        catch (PartInitException e1)
        {
          // TODO Auto-generated catch block
          e1.printStackTrace();
        }
        
        
    }
  }
  
  
  private void createViewFromSelection()
  {
    IStructuredSelection selection = (IStructuredSelection)_tableViewerKmdPar.getSelection();
    
    Object firstElement = selection.getFirstElement();
    if (firstElement instanceof SystemObject)
    {      
        Object[] array = selection.toArray();
        List<SystemObject> listKBA = new ArrayList<SystemObject>();
         
        for (Object object : array)
        {
          SystemObject so = (SystemObject)object;
          listKBA.add(so);          
        }
        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = dw.getActivePage();
        try
        {
          if (listKBA.size() > 0){
            
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < listKBA.size(); i++)
            {
              SystemObject so = listKBA.get(i);
              if (i<listKBA.size()-1)
                sb.append(so.getPid() + ";");
              else
                sb.append(so.getPid());
              
            }
            
            page.showView(MqZuweisungViewParameter.ID, sb.toString(), IWorkbenchPage.VIEW_CREATE);
            IViewReference ref = page.findViewReference(MqZuweisungViewParameter.ID, sb.toString());
            ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
            IWorkbenchPart wbPart = ref.getPart(false);
            Shell shell = wbPart.getSite().getShell();
            shell.setSize(500, 280);
            
//            page.showView(MqParaSteuerungViewParameter.ID, sb.toString(), IWorkbenchPage.VIEW_CREATE);
//            IViewReference ref = page.findViewReference(MqParaSteuerungViewParameter.ID, sb.toString());
//            ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
//            IWorkbenchPart wbPart = ref.getPart(false);
//            Shell shell = wbPart.getSite().getShell();
//            shell.setSize(450, 350);
//            IViewPart vPart = ref.getView(true);
//            if (vPart instanceof MqParaSteuerungViewParameter)
//            {
//              MqParaSteuerungViewParameter view = (MqParaSteuerungViewParameter) vPart;
////              view.erzeugeTabItems(listKBA);
//            }                
          }        
        }
        catch (PartInitException e1)
        {
          e1.printStackTrace();
        }
        
        
    }
  }
  

//  private void aktionOeffneAnlagenstatusView()
//  {
//    if (_tabelleKmdPar.getSelection() != null)
//    {
//
//    }
//    else
//    {
//      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
//      mb.setText("Fehler Auswahl UZ");
//      mb.setMessage("Es muss ein Eintrag ausgewhlt werden");
//      mb.open();
//    }
//  }

  private void createCompositeBottom(Composite parent)
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(parent, SWT.NONE);
    compositeBottom.setLayoutData(gridData3);
    compositeBottom.setLayout(rowLayout);
    buttonOk = new Button(compositeBottom, SWT.NONE);
    buttonOk.setText("Ok");
    buttonOk.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
         createViewFromSelection();       
      }
   

    });

    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }

    });
    
    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHilfe();
      }

    });
  }

  protected void aktionAbbrechen()
  {
    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(this);
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  protected void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".mqParaSteuerungAuswahl";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  private class ContentProvider implements IStructuredContentProvider
  {
    @Override
    public Object[] getElements(Object inputElement)
    {
      List list = (List)inputElement;

      Object[] re = new Object[list.size()];

      for (int i = 0; i < list.size(); i++)
      {
        re[i] = list.get(i);
      }
      return re;
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class LabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {

      // String[] ss = (String[])element;

      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          returnString = ((SystemObject)element).getName();
          break;
        case 1:
          returnString = ((SystemObject)element).getPid();
          break;

        default:
          returnString = "default";
      }

      return returnString;

    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

//  @Override
//  public void daVDataUpdate(DaVDataUpdateEvent e)
//  {
//    ResultData result = e.getResult();
//
//    if (result != null)
//    {
//      System.out.println(result);
//      
//    }
//
//  }

}
