/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.parameter.tsf.mqParaSteuerung;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.parameter.tsf.Activator;
import de.bsvrz.buv.plugin.parameter.tsf.DaVKommunikation;
import de.bsvrz.buv.plugin.parameter.tsf.common.DataHelper;
import de.bsvrz.buv.plugin.parameter.tsf.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.parameter.tsf.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.parameter.tsf.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Fenster zur nderung der Parameter fr AnzeigeQuerschnitte zum Quer-Lngsabgleich.
 * 
 * @author Krause
 * 
 */
public class MqParaSteuerungViewParameter extends ViewPart implements IDaVDataUpdateListener
{

  public static final String ID = "de.bsvrz.buv.plugin.parameter.tsf.mqParaSteuerung.mqParamSteuerungView";
  
  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributGruppe = "atg.sswSbaUeMqZuordnungBereich"; 
  
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private CTabFolder _tabFolder = null;

  private Composite _composite = null;

  private Composite _compositeBottom;

  private Button buttonAlle;
  
  private Button buttonOk;

  private Button buttonUebernehemen;

  private Button buttonAbbrechen;

  private Button buttonHilfe;

  private IMemento _memento = null;

  private String _secondViewId;

  private ClientDavInterface _connection;

  private boolean _sperreFrNeueDaten = false;

  /**
   * Fertig angelegte Tabs
   */
  private int _tabCnt = 0;
  
  @Override
  public void createPartControl( Composite parent )
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _composite = new Composite( parent, SWT.NONE );
    _composite.setLayout( gridLayout );
    _composite.setLayoutData( gridData );
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _composite, Activator.PLUGIN_ID + ".mqParaSteuerung" );   
    
    // Darstellung aufbauen
    createCompositeTab();
    createCompositeBottom();

    restoreState();

    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    List<SystemObject> list = new ArrayList<SystemObject>();

    if ( _secondViewId.contains( ";" ) )
    {

      String[] split = _secondViewId.split( ";" );

      for (String s : split)
      {
        SystemObject so = _connection.getDataModel().getObject( s );
        list.add( so );
      }

    }
    else
    {
      SystemObject so = _connection.getDataModel().getObject( _secondViewId );
      list.add( so );
    }

//  System.out.println( "Anzahl Tabs:" + list.size() );
//  for (SystemObject so : list)
//  {
//    System.out.println( "\tPids: " + so.getPid() );
//  }
    
    erzeugeTabItems( list );

    DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener( this );
    
    DaVKommunikation.getInstanz().anmeldeDaten( list, _attributGruppe, "asp.parameterSoll" );

  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_composite != null)
      _composite.setFocus();
  }

  /**
   * This method initializes _tabFolder
   * 
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder( _composite, SWT.NONE );
    _tabFolder.setLayout( new GridLayout() );
    _tabFolder.setLayoutData( gridData3 );
    _tabFolder.addSelectionListener( new SelectionAdapter()
    {
      @Override
      public void widgetSelected( SelectionEvent e )
      {

        CTabFolder folder = (CTabFolder) e.getSource();
        CTabItem itemSel = folder.getSelection();

        Composite cSel = (Composite) itemSel.getControl();

        if ( cSel.getChildren() != null && cSel.getChildren().length > 0 )
        {
          MqParaSteuerungCompositeParameter compParSel = (MqParaSteuerungCompositeParameter) cSel.getChildren()[0];

          TableViewer tvParameter[] = new TableViewer[1];
          for (int i = 0; i < 1; i++)
          {
            tvParameter[i] = compParSel.getTvAnzeige();          
            tvParameter[i].refresh();
          }
        }
      }
    } );
  }

  /**
   * This method initializes _tabItem for the list of system objects
   * 
   * @param list  list of system objects
   */
  public void erzeugeTabItems( List<SystemObject> list )
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    
    for (SystemObject so : list)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );
      tabItem.setText( so.getName() );
      tabItem.setData( so.getName(), so.getPid() );

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }

  /**
   * This method initializes _compositeBottom
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    _compositeBottom = new Composite( _composite, SWT.NONE );
    _compositeBottom.setLayoutData( gridData3 );
    _compositeBottom.setLayout( rowLayout );

    buttonAlle = new Button( _compositeBottom, SWT.NONE );
    buttonAlle.setText( "Alle setzen" );
    buttonAlle.setToolTipText( "Einstellungen bei allen ausgewhlten gleichartigen AQs setzen" );
    buttonAlle.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionAlle();
      }
    } );
   
    buttonOk = new Button( _compositeBottom, SWT.NONE );
    buttonOk.setText( "Ok" );
    buttonOk.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionSchalten();
      }
    } );

    buttonUebernehemen = new Button( _compositeBottom, SWT.NONE );
    buttonUebernehemen.setText( "bernehmen" );
    buttonUebernehemen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionUebernehmen();
      }
    } );

    buttonAbbrechen = new Button( _compositeBottom, SWT.NONE );
    buttonAbbrechen.setText( "Abbrechen" );
    buttonAbbrechen.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        dispose();
      }

    } );
    buttonHilfe = new Button( _compositeBottom, SWT.NONE );
    buttonHilfe.setText( "Hilfe" );
    buttonHilfe.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionHilfe();
      }

    } );
  }

  /**
   * Zeigt die Hilfe an.
   * 
   */
  protected void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".mqParaSteuerung";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }
  
  protected void aktionAlle()
  {
//    System.out.println("Aktion alle!!!");
    
    // aktueller Reiter
    CTabItem itemSel = _tabFolder.getSelection();

    Composite compSel = null;

    if ( itemSel == null )
      compSel = (Composite) _tabFolder.getItems()[0].getControl();
    else
      compSel = (Composite) itemSel.getControl();

    if ( compSel.getChildren() != null )
    {
      MqParaSteuerungCompositeParameter compParSel = (MqParaSteuerungCompositeParameter) compSel.getChildren()[0];

      CTabItem[] items = _tabFolder.getItems();
      
      for (CTabItem item : items)
      {
        if ( !item.equals( itemSel ) )
        {
          Composite c = (Composite) item.getControl();
  
          MqParaSteuerungCompositeParameter compPar = (MqParaSteuerungCompositeParameter) c.getChildren()[0];
          
          compPar.setTvAnzeige( compParSel.getTvAnzeige() );            
        }
      }

      _sperreFrNeueDaten = true;

    }
    else
      System.out.println("Kein selektierter Tab!!!");
  }

  protected void aktionSchalten()
  {
    if ( schaltenUndParametrieren() )
    {
      dispose();
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  protected void aktionUebernehmen()
  {
    if ( schaltenUndParametrieren() )
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  @Override
  public void init( IViewSite site, IMemento memento ) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init( site, memento );
  }

  private void restoreState()
  {

    if ( _memento == null )
    {
      return;
    }

    _secondViewId = _memento.getString( "pidMemento" );

  }

  @Override
  public void saveState( IMemento memento )
  {
    memento.putString( "pidMemento", _secondViewId );
    super.saveState( memento );
  }

  /**
   * Schalten und Parametrieren im DAV durchfhren
   * 
   * @return Durchfhrung erfolgreich
   */
  private Boolean schaltenUndParametrieren()
  {   
    DataModel dataModel = _connection.getDataModel();

    Aspect asp = dataModel.getAspect( "asp.parameterVorgabe" );
    AttributeGroup atg = dataModel.getAttributeGroup( _attributGruppe );
 
    Map<MqParaSteuerungCompositeParameter, ResultData> map = new LinkedHashMap<MqParaSteuerungCompositeParameter, ResultData>();

    //Tabs durchlaufen
    CTabItem[] items = _tabFolder.getItems();
    for (CTabItem item : items)
    {
      Composite c = (Composite)item.getControl();
      MqParaSteuerungCompositeParameter compPar = (MqParaSteuerungCompositeParameter)c.getChildren()[0];
      
      Data data = _connection.createData(atg);

      for (int i = 0; i < compPar.getAnzahlGruppen(); i++)
      {
        //TabelleElemente durchlaufen
        for (int j = 0; j < compPar.getAnzahlItems( i ); j++)
        {
          String t1 = compPar.getItems(i)[j].getSelectedWert();
          Label lb = (Label)compPar.getItems(i)[j].getAttribut().getChildren()[0];
          
          data.getItem(lb.getText()).asTextValue().setText(t1);
        }      
      }
      
      if (!DataHelper.isEqual(data, compPar.getResultData().getData())) {
    	  SystemObject so = dataModel.getObject((String)item.getData(item.getText()));      
          ResultData rd = new ResultData(so, new DataDescription(atg, asp), new Date().getTime(), data);
          map.put(compPar, rd);  
      }
      
    }

//    UrlasserDialog urlasserDialog = new UrlasserDialog( Display.getCurrent().getActiveShell() );
//    Boolean schlieenOk = (urlasserDialog.anzeigen( list )).getEingabeOk();

    if (!map.isEmpty()) {
    	UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench()
    	        .getDisplay().getActiveShell());
    	            
//   	    UrlasserDialog.UrlasserDialogInfo anzeigen = uld.anzeigen(map.values(), Berechtigungen.parametrieren);
   	    UrlasserDialog.UrlasserDialogInfo anzeigen = uld.anzeigen(map.values());
   	    boolean eingabeOk = anzeigen.getEingabeOk();
   	    if (eingabeOk && (anzeigen.getFehlerInfo() == null)) {
	    	for (Map.Entry<MqParaSteuerungCompositeParameter, ResultData> entry: map.entrySet()) {
	    		entry.getKey().setResultData(entry.getValue());
	    	}
	    	return true;
	    }
	    else {
	    	return false;
	    }
    }
    else {  // Keine nderung
    	return true;
    }
  }

  @Override
  public void daVDataUpdate( DaVDataUpdateEvent e )
  {
    if ( e != null )
    {
      if ( !_sperreFrNeueDaten )
        verarbeiteDavDaten( e );
      else
        System.out.println( "Neue Daten -> kein Update, da Sperre fr neue Daten aktiv" );
    }
  }

  private void verarbeiteDavDaten( DaVDataUpdateEvent e )
  {
	  System.out.println( "Neue Daten -> Verarbeitung" ); 
	  
    SystemObject so = e.getResult().getObject();

    CTabItem[] items = _tabFolder.getItems();
    
    //Tab zum Systemobject ermitteln
    CTabItem item = null;
    for (CTabItem ti : items)
    {
      String pid = (String)ti.getData(ti.getText());

      if (pid == so.getPid())
      {
        item = ti;
        break;
      }
    }

    //Tab gefunden
    if (item != null)
    {
      Composite c = (Composite)item.getControl();
      
      Control[] children = c.getChildren();

      //Tab neu anlegen, wenn noch leer
      if (children.length == 0)
      {
        _tabCnt++;
        
        MqParaSteuerungCompositeParameter compPar = new MqParaSteuerungCompositeParameter( c, SWT.None, e
        .getResult() );
        compPar.layout( true );
        c.layout( true );
      }
      
      //Sperre fr neue Daten, wenn fertige Tabs == Anzahl Tabs
      if (_tabCnt == _tabFolder.getItemCount()){            
        _sperreFrNeueDaten = true;
//        System.out.println("Sperre fr neue Daten ist aktiv");
      }
    }
    
    if ( _tabFolder.getItemCount() <= 1 )
      buttonAlle.setEnabled( false );
    
  }

  @Override
  public void dispose()
  {
    super.dispose();
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener( this );
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if ( page != null )
    {
      page.hideView( this );
    }
  }

}
