package de.bsvrz.buv.plugin.parameter.ufd.haqparam;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.List;
import java.util.ListIterator;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;

import de.bsvrz.buv.plugin.parameter.ufd.common.AbstractDataContainerComposite;
import de.bsvrz.buv.plugin.parameter.ufd.common.MinMaxColumnEditingSupport;
import de.bsvrz.buv.plugin.parameter.ufd.common.SpinnerColumnEditingSupport;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;

public class HAQSensorStoerungComposite extends AbstractDataContainerComposite
{
  
  private Button _btnAddDatum;
  private Button _btnRemoveDatum;
  private Button _btnAddZeit;
  private Button _btnRemoveZeit;
  
  private Spinner _spVonTag;
  private Spinner _spVonMonat;
  private Spinner _spBisTag;
  private Spinner _spBisMonat;
  
  private Spinner _spStunde;
  private Spinner _spMinute;
  
  private TableViewer _tvDatum;
  private TableViewer _tvZeit;
  
  private List<DatumEntry> _modelDatum = new ArrayList<DatumEntry>();
  private ZeitEntry[] _modelZeitEmpty = new ZeitEntry[0];

  protected HAQSensorStoerungComposite(Composite parent, int style)
  {
    super(parent, style);
  }

  @Override
  protected void createControls()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
  
    Group groupParameter = new Group(this, SWT.NONE);
    groupParameter.setLayout(new GridLayout(4, false));
    groupParameter.setLayoutData(gridData);
    groupParameter.setText("Sensorstrung");
    
    createDatumComposite(groupParameter);
    createTableDatum(groupParameter);
    createZeitComposite(groupParameter);
    createTableZeit(groupParameter);
    
    handleSelectionDatumTable();
  }

  @Override
  public Data getUpdate()
  {
    if (!isInitialized())
      return null;
    
    Data result = _data.createModifiableCopy();
    Array array = result.getArray("Datumsbereich");
    array.setLength(0);
    
    for (int i = 0; i < _modelDatum.size(); i++) {
      DatumEntry entry = _modelDatum.get(i);
      int index = array.getLength();
      array.setLength(index + 1);
      Data item = array.getItem(index);
      
      Data subItem = item.getItem("DatumsbereichVon");
      subItem.getItem("Tag").asUnscaledValue().set(entry.tagVon);
      subItem.getItem("Monat").asUnscaledValue().set(entry.monatVon);
      
      subItem = item.getItem("DatumsbereichBis");
      subItem.getItem("Tag").asUnscaledValue().set(entry.tagBis);
      subItem.getItem("Monat").asUnscaledValue().set(entry.monatBis);
      
      subItem = item.getItem("TageszeitbereichStufe");
      subItem.getItem("StufeAb00:00Uhr").asUnscaledValue().set(entry.stufeAb0Uhr);
      
      Array innerArray = subItem.getArray("StundeMinuteStufe");
      
      for (int j = 0; j < entry.stundeMinuteStufe.size(); j++) {
        ZeitEntry innerEntry = entry.stundeMinuteStufe.get(j);
        int innerIndex = innerArray.getLength();
        innerArray.setLength(innerIndex + 1);
        Data innerItem = innerArray.getItem(innerIndex);
        
        Data innerSubItem = innerItem.getItem("StundeMinute");
        innerSubItem.getItem("Stunde").asUnscaledValue().set(innerEntry.stunde);
        innerSubItem.getItem("Minute").asUnscaledValue().set(innerEntry.minute);
        
        innerItem.getItem("Stufe").asUnscaledValue().set(innerEntry.stufe);
      }
      
    }
    
    return result;
  }

  @Override
  public void setUpdate(Data data)
  {
    if (!isInitialized())
      return;
          
    Array array = data.getArray("Datumsbereich");
    _modelDatum.clear();
    
    for (int i = 0; i < array.getLength(); i++) {
      Data item = array.getItem(i);
      
      Data subItem = item.getItem("DatumsbereichVon");
      int tagVon = subItem.getItem("Tag").asUnscaledValue().intValue();
      int monatVon = subItem.getItem("Monat").asUnscaledValue().intValue();
      
      subItem = item.getItem("DatumsbereichBis");
      int tagBis = subItem.getItem("Tag").asUnscaledValue().intValue();
      int monatBis = subItem.getItem("Monat").asUnscaledValue().intValue();
      
      subItem = item.getItem("TageszeitbereichStufe");
      int stufeAb0Uhr = subItem.getItem("StufeAb00:00Uhr").asUnscaledValue().intValue();
      
      DatumEntry entry = new DatumEntry(tagVon, monatVon, tagBis, monatBis);
      entry.stufeAb0Uhr = stufeAb0Uhr;
      
      Array innerArray = subItem.getArray("StundeMinuteStufe");
      for (int j = 0; j < innerArray.getLength(); j++) {
        Data innerItem = innerArray.getItem(j);
        
        Data innerSubItem = innerItem.getItem("StundeMinute");
        int stunde = innerSubItem.getItem("Stunde").asUnscaledValue().intValue();
        int minute = innerSubItem.getItem("Minute").asUnscaledValue().intValue();
        
        int stufe = innerItem.getItem("Stufe").asUnscaledValue().intValue();
        
        ZeitEntry innerEntry = new ZeitEntry(stunde, minute);
        innerEntry.stufe = stufe;
        
        entry.stundeMinuteStufe.add(innerEntry);
      }
      
      _modelDatum.add(entry);
    }
    
//    Collections.sort(_modelDatum);
    _tvDatum.refresh();
    handleSelectionDatumTable();
  }
  
  private void createDatumComposite(Composite parent) {
    GridLayout gridLayout = new GridLayout(1, false);
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
//    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayoutData(gridData);
    composite.setLayout(gridLayout);
    
    gridData = new GridData();
//    gridData.grabExcessHorizontalSpace = true;
//    gridData.grabExcessVerticalSpace = true;
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.BEGINNING;
    
    Group grpDatumVon = new Group(composite, SWT.None);
    grpDatumVon.setText("DatumsbereichVon");
    grpDatumVon.setLayoutData(gridData);
    grpDatumVon.setLayout(new GridLayout(2, false));
    
    new Label(grpDatumVon, SWT.NONE).setText("Tag:");
    
    _spVonTag = new Spinner(grpDatumVon, SWT.BORDER);
    _spVonTag.setMinimum(1);
    _spVonTag.setMaximum(31);
    _spVonTag.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        enableDisableDatumButtons();
      }
      
    });
    
    new Label(grpDatumVon, SWT.NONE).setText("Monat:");
    
    _spVonMonat = new Spinner(grpDatumVon, SWT.BORDER);
    _spVonMonat.setMinimum(1);
    _spVonMonat.setMaximum(12);
    _spVonMonat.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        int days = getDaysInMonth(_spVonMonat.getSelection());
        if (days != _spVonTag.getSelection()) {
          if (days < _spVonTag.getSelection()) {
            _spVonTag.setSelection(days);  
          }
          _spVonTag.setMaximum(days);
        }
        enableDisableDatumButtons();
      }
      
    });
    
    gridData = new GridData();
    //  gridData.grabExcessHorizontalSpace = true;
    //  gridData.grabExcessVerticalSpace = true;
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.BEGINNING;
  
    Group grpDatumBis = new Group(composite, SWT.None);
    grpDatumBis.setText("DatumsbereichBis");
    grpDatumBis.setLayoutData(gridData);
    grpDatumBis.setLayout(new GridLayout(2, false));
  
    new Label(grpDatumBis, SWT.NONE).setText("Tag:");
    
    _spBisTag = new Spinner(grpDatumBis, SWT.BORDER);
    _spBisTag.setMinimum(1);
    _spBisTag.setMaximum(31);
    _spBisTag.addSelectionListener(new SelectionAdapter() {
  
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        enableDisableDatumButtons();
      }
      
    });
    
    new Label(grpDatumBis, SWT.NONE).setText("Monat:");
    
    _spBisMonat = new Spinner(grpDatumBis, SWT.BORDER);
    _spBisMonat.setMinimum(1);
    _spBisMonat.setMaximum(12);
    _spBisMonat.addSelectionListener(new SelectionAdapter() {
  
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        int days = getDaysInMonth(_spBisMonat.getSelection());
        if (days != _spBisTag.getSelection()) {
          if (days < _spBisTag.getSelection()) {
            _spBisTag.setSelection(days);  
          }
          _spBisTag.setMaximum(days);
        }
        enableDisableDatumButtons();
      }
      
    });
    
    gridData = new GridData();
    gridData.horizontalSpan = 2;
    gridData.horizontalAlignment = GridData.END;
        
    _btnAddDatum = new Button(composite, SWT.NONE);
    _btnAddDatum.setText("Hinzufgen >>");
    _btnAddDatum.setLayoutData(gridData);
    _btnAddDatum.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        actionAddDatum();
      }
      
    });
       
    _btnRemoveDatum = new Button(composite, SWT.NONE);
    _btnRemoveDatum.setText("Entfernen <<");
    _btnRemoveDatum.setLayoutData(gridData);
    _btnRemoveDatum.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        actionRemoveDatum();
      }
      
    });

  }
  
  public static int getDaysInMonth(int month) {
//    switch (month + 1) {
    switch (month) {
      case 1:
      case 3:
      case 5:
      case 7:
      case 8:
      case 10:
      case 12:
        return 31;
        
      case 2:
//        return 28;
        return 29;
        
      case 4:
      case 6:
      case 9:
      case 11:
        return 30;
        
      default:
        throw new IllegalArgumentException("Illegal month " + month);
    }
  }
 
  private void actionAddDatum() {
    
    int vonTag = _spVonTag.getSelection();
    int vonMonat = _spVonMonat.getSelection();
    int bisTag = _spBisTag.getSelection();
    int bisMonat = _spBisMonat.getSelection();
    
    _modelDatum.add(new DatumEntry(vonTag, vonMonat, bisTag, bisMonat));
    
    Collections.sort(_modelDatum);
    
    _tvDatum.refresh();
      
    handleSelectionDatumTable();
  }
  
  private void actionRemoveDatum() {
    ISelection sel = _tvDatum.getSelection();
    if (!sel.isEmpty() && sel instanceof StructuredSelection) {
      
      Object[] array = ((StructuredSelection)sel).toArray();
      ListIterator<DatumEntry> it = _modelDatum.listIterator();
      while(it.hasNext()) {
        DatumEntry entry = it.next();
        for (int i = 0; i < array.length; i++) {
          if (entry.equals(array[i])) {
            it.remove();
            break;
          }
        }
      }

      _tvDatum.refresh();
      
      handleSelectionDatumTable();
    }
  }
  
  private boolean checkDatum(int vonTag, int vonMonat, int bisTag, int bisMonat) {
    
//    for (DatumEntry entry: _modelDatum) {
//      if (entry.tagVon == vonTag && entry.monatVon == vonMonat &&
//          entry.tagBis == bisTag && entry.monatBis == bisMonat) {
//        return false;
//      }
//    }
    
//    if ((vonMonat > bisMonat) || vonMonat == bisMonat && vonTag >= bisTag) {
    if ((vonMonat > bisMonat) || vonMonat == bisMonat && vonTag > bisTag) {
      return false; // Von-Datum > Bis-Datum
    }

    // Initialisiere Datum 01.01.2008
    Calendar cal = Calendar.getInstance();
    cal.set(Calendar.MILLISECOND, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.HOUR_OF_DAY, 0);
//    cal.set(Calendar.DATE, 1);
//    cal.set(Calendar.MONTH, 0);
    cal.set(Calendar.YEAR, 2008);
    
    cal.set(Calendar.DATE, vonTag);
    cal.set(Calendar.MONTH, vonMonat - 1);
    int von = cal.get(Calendar.DAY_OF_YEAR);
    
    cal.set(Calendar.DATE, bisTag);
    cal.set(Calendar.MONTH, bisMonat - 1);
    int bis = cal.get(Calendar.DAY_OF_YEAR);
    
    for (DatumEntry entry: _modelDatum) {
      
      cal.set(Calendar.DATE, entry.tagVon);
      cal.set(Calendar.MONTH, entry.monatVon - 1);
      int min = cal.get(Calendar.DAY_OF_YEAR);
      
      cal.set(Calendar.DATE, entry.tagBis);
      cal.set(Calendar.MONTH, entry.monatBis - 1);
      int max = cal.get(Calendar.DAY_OF_YEAR);
      
//      if ((von > min && von < max) || (bis > min && bis < max)) {
      if ((von >= min && von <= max) || (bis >= min && bis <= max)) {
        return false;
      }

    }
    
    return true;
  }
  
  private void enableDisableDatumButtons() {
    
    int vonTag = _spVonTag.getSelection();
    int vonMonat = _spVonMonat.getSelection();
    int bisTag = _spBisTag.getSelection();
    int bisMonat = _spBisMonat.getSelection();
        
    _btnAddDatum.setEnabled(checkDatum(vonTag, vonMonat, bisTag, bisMonat));
        
    _btnRemoveDatum.setEnabled(_tvDatum.getTable().getSelectionCount() > 0);
  }
  
  private void createZeitComposite(Composite parent) {
    GridLayout gridLayout = new GridLayout(1, false);
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
//    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayoutData(gridData);
    composite.setLayout(gridLayout);
    
    gridData = new GridData();
//    gridData.grabExcessHorizontalSpace = true;
//    gridData.grabExcessVerticalSpace = true;
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.BEGINNING;
    
    Group grpZeit = new Group(composite, SWT.None);
    grpZeit.setText("Tageszeit");
    grpZeit.setLayoutData(gridData);
    grpZeit.setLayout(new GridLayout(2, false));
    
    new Label(grpZeit, SWT.NONE).setText("Stunde:");
    
    _spStunde = new Spinner(grpZeit, SWT.BORDER);
    _spStunde.setMinimum(0);
    _spStunde.setMaximum(23);
    _spStunde.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        enableDisableZeitButtons();
      }
      
    });
    
    new Label(grpZeit, SWT.NONE).setText("Minute:");
    
    _spMinute = new Spinner(grpZeit, SWT.BORDER);
    _spMinute.setMinimum(0);
    _spMinute.setMaximum(59);
    _spMinute.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        enableDisableZeitButtons();
      }
      
    });
     
    
    gridData = new GridData();
    gridData.horizontalSpan = 2;
    gridData.horizontalAlignment = GridData.END;
        
    _btnAddZeit = new Button(composite, SWT.NONE);
    _btnAddZeit.setText("Hinzufgen >>");
    _btnAddZeit.setLayoutData(gridData);
    _btnAddZeit.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        actionAddZeit();
      }
      
    });
       
    _btnRemoveZeit = new Button(composite, SWT.NONE);
    _btnRemoveZeit.setText("Entfernen <<");
    _btnRemoveZeit.setLayoutData(gridData);
    _btnRemoveZeit.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        actionRemoveZeit();
      }
      
    });

  }
  
  private void actionAddZeit() {
    
    ISelection sel = _tvDatum.getSelection();
    if (sel instanceof StructuredSelection && ((StructuredSelection)sel).size() == 1) {
      DatumEntry entry = (DatumEntry)((StructuredSelection)sel).getFirstElement();
      
      int stunde = _spStunde.getSelection();
      int minute = _spMinute.getSelection();
      
      entry.stundeMinuteStufe.add(new ZeitEntry(stunde, minute));
      
      Collections.sort(entry.stundeMinuteStufe);
      
      _tvZeit.refresh();
      
      enableDisableZeitButtons();
    }
  }
  
  private void actionRemoveZeit() {
    
    ISelection sel = _tvDatum.getSelection();
    if (sel instanceof StructuredSelection && ((StructuredSelection)sel).size() == 1) {
      DatumEntry entry = (DatumEntry)((StructuredSelection)sel).getFirstElement();
     
      ISelection innerSel = _tvZeit.getSelection();
      if (!innerSel.isEmpty() && innerSel instanceof StructuredSelection) {
        
        Object[] array = ((StructuredSelection)innerSel).toArray();
        ListIterator<ZeitEntry> it = entry.stundeMinuteStufe.listIterator();
        while(it.hasNext()) {
          ZeitEntry innerEntry = it.next();
          for (int i = 0; i < array.length; i++) {
            if (innerEntry.equals(array[i])) {
              it.remove();
              break;
            }
          }
        }

        _tvZeit.refresh();
        
        enableDisableZeitButtons();
      }
      
    }

  }
  
  private void enableDisableZeitButtons() {
  
    ISelection sel = _tvDatum.getSelection();
    
     
    if (sel instanceof StructuredSelection && ((StructuredSelection)sel).size() == 1) {
      DatumEntry entry = (DatumEntry)((StructuredSelection)sel).getFirstElement();
      
      boolean gefunden = false;
      int stunde = _spStunde.getSelection();
      int minute = _spMinute.getSelection();
      
      for (ZeitEntry innerEntry: entry.stundeMinuteStufe) {
        if (innerEntry.stunde == stunde && innerEntry.minute == minute) {
          gefunden = true;
          break;
        }
      }
      _btnAddZeit.setEnabled(!gefunden);
    }
    else {
      _btnAddZeit.setEnabled(false);
    }
    
    _btnRemoveZeit.setEnabled(_tvZeit.getTable().getSelectionCount() > 0);
  }
  
  private void createTableDatum(Composite parent) {
    _tvDatum = new TableViewer(parent, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvDatum.setContentProvider(new ArrayContentProvider());
    
    TableViewerColumn column = new TableViewerColumn(_tvDatum, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof DatumEntry) {
          return ((DatumEntry)element).getDatumsbereichVon();
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(110);
    column.getColumn().setText("DatumsbereichVon");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
    column = new TableViewerColumn(_tvDatum, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof DatumEntry) {
          return ((DatumEntry)element).getDatumsbereichBis();
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(110);
    column.getColumn().setText("DatumsbereichBis");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
    column = new TableViewerColumn(_tvDatum, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof DatumEntry) {
          return (String.valueOf(((DatumEntry)element).stufeAb0Uhr));
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(100);
    column.getColumn().setText("StufeAb00:00Uhr");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
//    column.setEditingSupport(new MinMaxColumnEditingSupport(_tvDatum, 0, 127) {
    column.setEditingSupport(new SpinnerColumnEditingSupport(_tvDatum, 0, 127) {

      @Override
      protected Object getValue(Object element)
      {
        if (element instanceof DatumEntry) {
          return Integer.valueOf(((DatumEntry)element).stufeAb0Uhr);
//          return String.valueOf(((DatumEntry)element).stufeAb0Uhr);
        }
        return null;
      }

      @Override
      protected void setValue(Object element, Object value)
      {
        if (element instanceof DatumEntry && value instanceof Integer) {
          ((DatumEntry)element).stufeAb0Uhr = ((Integer)value).intValue();
          _tvDatum.update(element, null); 
        }
//        if (element instanceof DatumEntry) {
//          try {
//            ((DatumEntry)element).stufeAb0Uhr = value != null ? Integer.parseInt(value.toString()) : 0;
//            _tvDatum.update(element, null);  
//          }
//          catch(NumberFormatException e) {
//            // tue nichts
//          }
//          
//        }
        
      }
      
    });
    
    Table table = _tvDatum.getTable();
    table.setLinesVisible(true);
    table.setHeaderVisible(true);
    
    GridData gridData = new GridData(SWT.DEFAULT, 
        10 * table.getItemHeight() + table.getHeaderHeight());
//    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;

    table.setLayoutData(gridData);
    
    _tvDatum.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        handleSelectionDatumTable();
      }
    });
    
     _tvDatum.setInput(_modelDatum);
  }
  
  private void handleSelectionDatumTable() {
    
    ISelection sel = _tvDatum.getSelection();
    if (sel instanceof StructuredSelection && ((StructuredSelection)sel).size() == 1) {
      
      _spStunde.setEnabled(true);
      _spMinute.setEnabled(true);
      _tvZeit.getTable().setEnabled(true);
      
      DatumEntry entry = (DatumEntry)((StructuredSelection)sel).getFirstElement();
      
      _tvZeit.setInput(entry.stundeMinuteStufe);
      
    }
    else {
      _spStunde.setEnabled(false);
      _spMinute.setEnabled(false);
      _tvZeit.setInput(_modelZeitEmpty);
      _tvZeit.getTable().setEnabled(false);
    }
    
    enableDisableDatumButtons();
    enableDisableZeitButtons();
  }
  
  private void createTableZeit(Composite parent) {
    _tvZeit = new TableViewer(parent, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvZeit.setContentProvider(new ArrayContentProvider());
    
    TableViewerColumn column = new TableViewerColumn(_tvZeit, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof ZeitEntry) {
          return ((ZeitEntry)element).getStundeMinute();
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(90);
    column.getColumn().setText("StundeMinute");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
    column = new TableViewerColumn(_tvZeit, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof ZeitEntry) {
          return String.valueOf(((ZeitEntry)element).stufe);
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(50);
    column.getColumn().setText("Stufe");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
//    column.setEditingSupport(new MinMaxColumnEditingSupport(_tvZeit, 0, 127) {
    column.setEditingSupport(new SpinnerColumnEditingSupport(_tvZeit, 0, 127) {

      @Override
      protected Object getValue(Object element)
      {
//        if (element instanceof ZeitEntry) {
//          return String.valueOf(((ZeitEntry)element).stufe);
//        }
        if (element instanceof ZeitEntry) {
          return Integer.valueOf(((ZeitEntry)element).stufe);
        }
        return null;
      }

      @Override
      protected void setValue(Object element, Object value)
      {
        if (element instanceof ZeitEntry && value instanceof Integer) {
          ((ZeitEntry)element).stufe = ((Integer)value).intValue();
          _tvZeit.update(element, null);  
        }
//        if (element instanceof ZeitEntry) {
//          try {
//            ((ZeitEntry)element).stufe = value != null ? Integer.parseInt(value.toString()) : 0;
//            _tvZeit.update(element, null);  
//          }
//          catch(NumberFormatException e) {
//            // tue nichts
//          }
//          
//        }
        
      }
      
    });
    
    Table table = _tvZeit.getTable();
    table.setLinesVisible(true);
    table.setHeaderVisible(true);
    
    GridData gridData = new GridData(SWT.DEFAULT, 
        10 * table.getItemHeight() + table.getHeaderHeight());
//    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;

    table.setLayoutData(gridData);
    
    _tvZeit.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        enableDisableZeitButtons();
      }
    });
    
//    _tvZeit.setInput(_modelZeitEmpty);
    
  }
  
  private class DatumEntry implements Comparable<DatumEntry> {
    
    public final int tagVon;
    public final int monatVon;
    
    public final int tagBis;
    public final int monatBis;
    
    public int stufeAb0Uhr = 0;
    
    public final List<ZeitEntry> stundeMinuteStufe = new ArrayList<ZeitEntry>();
    
    public DatumEntry(int tagVon, int monatVon, int tagBis, int monatBis)
    {
      super();
      this.tagVon = tagVon;
      this.monatVon = monatVon;
      this.tagBis = tagBis;
      this.monatBis = monatBis;
    }
    
    public String getDatumsbereichVon() {
      return String.format("%02d.%02d", tagVon, monatVon);
    }
    
    public String getDatumsbereichBis() {
      return String.format("%02d.%02d", tagBis, monatBis);
    }

    @Override
    public int compareTo(DatumEntry o)
    {
      int result = String.format("%02d%02d%02d%02d", tagVon, monatVon, 
          tagBis, monatBis).compareTo(String.format("%02d%02d%02d%02d", 
              o.tagVon, o.monatVon, o.tagBis, o.monatBis));
      if (result == 0) {
        if (stufeAb0Uhr > o.stufeAb0Uhr) {
          result = 1; 
        }
        else if (stufeAb0Uhr < o.stufeAb0Uhr) {
          result = -1;
        }
      }
      return result;
    }
    
  }
  
  private class ZeitEntry implements Comparable<ZeitEntry>{
    
    public final int stunde;
    public final int minute;
    
    public int stufe = 0;
    
    public ZeitEntry(int stunde, int minute)
    {
      super();
      this.stunde = stunde;
      this.minute = minute;
    }
    
    public String getStundeMinute() {
      return String.format("%d:%02d", stunde, minute);
    }

    @Override
    public int compareTo(ZeitEntry o)
    {
      int result = String.format("%02d%02d", stunde, minute).compareTo(
          String.format("%02d%02d", o.stunde, o.minute));
      if (result == 0) {
        if (stufe > o.stufe) {
          result = 1; 
        }
        else if (stufe < o.stufe) {
          result = -1;
        }
      }
      return result;
    }
    
  }

}
