package de.bsvrz.buv.plugin.parameter.ufd.haqparam;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.ListIterator;

import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DateTime;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;

import de.bsvrz.buv.plugin.parameter.ufd.common.AbstractDataContainerComposite;
import de.bsvrz.buv.plugin.parameter.ufd.common.MinMaxColumnEditingSupport;
import de.bsvrz.buv.plugin.parameter.ufd.common.SpinnerColumnEditingSupport;
import de.bsvrz.buv.plugin.parameter.ufd.dav.DaVKommunikation;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;


public class HAQSteuerungStufeComposite extends AbstractDataContainerComposite
{
   
  private List<StufeEntry> _stufen = new ArrayList<StufeEntry>();
  
  private DateTime _dtStandzeit;
  private Combo _cbSteuerung;
  
  Spinner _spStufe;
  
  private Button _btnHinzufuegen;
  private Button _btnEntfernen;
  
  private TableViewer _tvHAQ;
  
  private ControlDecoration _tableDecor;
  
  private IntegerAttributeType _iatSteuerung; 
  
  public HAQSteuerungStufeComposite(Composite parent, int style) {
    super(parent, style);
  }

  @Override
  public void initialize() {
 
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    if (connection != null) {
      DataModel dataModel = connection.getDataModel();
      AttributeType at = dataModel.getAttributeType("att.sswSbaHelligkeitSteuerung");
      
      if (at instanceof IntegerAttributeType) {
        _iatSteuerung = (IntegerAttributeType)at;
        
      }
    }   
    super.initialize();
  }
  
  
  protected void createControls()
  {
    GridLayout gridLayout = new GridLayout(2, false);
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
  
    Group groupParameter = new Group(this, SWT.NONE);
    groupParameter.setLayout(gridLayout);
    groupParameter.setLayoutData(gridData);
    groupParameter.setText("SteuerungStufe");
    
    createComposite(groupParameter);
   
    createTableHAQ(groupParameter);
     
  }

  
  private void createComposite(Composite parent) {
    
    GridLayout gridLayout = new GridLayout(2, false);
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
//    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayoutData(gridData);
    composite.setLayout(gridLayout);

    new Label(composite, SWT.NONE).setText("Standzeit:");
    
    _dtStandzeit = new DateTime(composite, SWT.BORDER | SWT.TIME);
    _dtStandzeit.setTime(0, 0, 0);
    
    new Label(composite, SWT.NONE).setText("Steuerung:");
    
    _cbSteuerung = new Combo(composite, SWT.READ_ONLY);
    if (_iatSteuerung != null) {
      IntegerValueRange range = _iatSteuerung.getRange();
      if (range != null) {
        for (long value = range.getMinimum(); value <= range.getMaximum(); value++) {
          String name = null;
          for (IntegerValueState state: _iatSteuerung.getStates()) {
            if (value == state.getValue()) {
              name = state.getName();
              break;
            }
          }
          if (name != null) {
            _cbSteuerung.add(name);
          }
          else {
            _cbSteuerung.add("" + value);
          }
        }  
      }
      else {
        for (IntegerValueState state: _iatSteuerung.getStates()) {
          _cbSteuerung.add(state.getName());
        }
      }
    }
    
    gridData = new GridData();
    gridData.horizontalSpan = 2;
//    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
//    gridData.horizontalAlignment = GridData.BEGINNING;
    gridData.verticalAlignment = GridData.BEGINNING;
    
    Group groupStufe = new Group(composite, SWT.NONE);
    groupStufe.setText("Stufen");
    groupStufe.setLayoutData(gridData);
    groupStufe.setLayout(new GridLayout(3, false));
    
    new Label(groupStufe, SWT.NONE).setText("Helligkeitswert:");
    
    _spStufe = new Spinner(groupStufe, SWT.BORDER);
    _spStufe.setMinimum(0);
    _spStufe.setMaximum(100);
    _spStufe.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub
       enableDisableButtons();
      }
      
    });
    
    new Label(groupStufe, SWT.NONE).setText("%");
    
    gridData = new GridData();
    gridData.horizontalSpan = 3;
    gridData.horizontalAlignment = GridData.END;
        
    _btnHinzufuegen = new Button(groupStufe, SWT.NONE);
    _btnHinzufuegen.setText("Hinzufgen >>");
    _btnHinzufuegen.setLayoutData(gridData);
    _btnHinzufuegen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        actionHinzufuegenStufe();
      }
      
    });
       
    _btnEntfernen = new Button(groupStufe, SWT.NONE);
    _btnEntfernen.setText("Entfernen <<");
    _btnEntfernen.setLayoutData(gridData);
    _btnEntfernen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        actionEntfernenStufen();
      }
      
    });
   
  }
    
  private void createTableHAQ(Composite parent) {
    
    _tvHAQ = new TableViewer(parent, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvHAQ.setContentProvider(new ArrayContentProvider());
    
    TableViewerColumn column = new TableViewerColumn(_tvHAQ, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof StufeEntry) {
          int result = _stufen.indexOf((StufeEntry)element);
          if (result >= 0 && result < _stufen.size()) {
            return String.valueOf(result + 1);
          }
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(100);
    column.getColumn().setText("Stufe");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
    column = new TableViewerColumn(_tvHAQ, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof StufeEntry) {
          return (String.valueOf(((StufeEntry)element).helligkeitsWert));
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(130);
    column.getColumn().setText("Helligkeitswert [%]");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
    column = new TableViewerColumn(_tvHAQ, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof StufeEntry) {
          return (String.valueOf(((StufeEntry)element).grenzwertEin));
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(130);
    column.getColumn().setText("GrenzwertEin [Lx]");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
//    column.setEditingSupport(new MinMaxColumnEditingSupport(_tvHAQ, 0, 60000) {
    column.setEditingSupport(new SpinnerColumnEditingSupport(_tvHAQ, 0, 60000) {

      @Override
      protected Object getValue(Object element)
      {
//        if (element instanceof StufeEntry) {
//          return String.valueOf(((StufeEntry)element).grenzwertEin);
//        }
        if (element instanceof StufeEntry) {
          return Integer.valueOf(((StufeEntry)element).grenzwertEin);
        }
        return null;
      }

      @Override
      protected void setValue(Object element, Object value)
      {
        if (element instanceof StufeEntry && value instanceof Integer) {
          ((StufeEntry)element).grenzwertEin = ((Integer)value).intValue();
          _tvHAQ.update(element, null);
        }
//        if (element instanceof StufeEntry) {
//          try {
//            ((StufeEntry)element).grenzwertEin = value != null ? Integer.parseInt(value.toString()) : 0;
//            _tvHAQ.update(element, null);
//            validateTableData();
//          }
//          catch(NumberFormatException e) {
//            // tue nichts
//          }
//          
//        }
        
      }
      
    });
    
    
    column = new TableViewerColumn(_tvHAQ, SWT.NONE);
    column.setLabelProvider(new ColumnLabelProvider() {

      @Override
      public String getText(Object element)
      {
        if (element instanceof StufeEntry) {
          return (String.valueOf(((StufeEntry)element).grenzwertAus));
        }
        return null;
      }
      
    });
    
    column.getColumn().setWidth(130);
    column.getColumn().setText("GrenzwertAus [Lx]");
    column.getColumn().setMoveable(true);
    column.getColumn().setResizable(true);
    
//    column.setEditingSupport(new MinMaxColumnEditingSupport(_tvHAQ, 0, 60000) {
    column.setEditingSupport(new SpinnerColumnEditingSupport(_tvHAQ, 0, 60000) {

      @Override
      protected Object getValue(Object element)
      {
//        if (element instanceof StufeEntry) {
//          return String.valueOf(((StufeEntry)element).grenzwertAus);
//        }
        if (element instanceof StufeEntry) {
          return Integer.valueOf(((StufeEntry)element).grenzwertAus);
        }
        return null;
      }

      @Override
      protected void setValue(Object element, Object value)
      {
        if (element instanceof StufeEntry && value instanceof Integer) {
          ((StufeEntry)element).grenzwertAus = ((Integer)value).intValue();
          _tvHAQ.update(element, null);
        }
//        if (element instanceof StufeEntry) {
//          try {
//            ((StufeEntry)element).grenzwertAus = value != null ? Integer.parseInt(value.toString()) : 0;
//            _tvHAQ.update(element, null);
//            validateTableData();
//          }
//          catch(NumberFormatException e) {
//            // tue nichts
//          }
//          
//        }
        
      }
      
    });

    
    Table table = _tvHAQ.getTable();
    table.setLinesVisible(true);
    table.setHeaderVisible(true);
    
    _tableDecor = new ControlDecoration(table, SWT.LEFT | SWT.TOP);
    Image image = FieldDecorationRegistry.getDefault()
          .getFieldDecoration(FieldDecorationRegistry.DEC_WARNING)
          .getImage();
    _tableDecor.setImage(image);
//    _tableDecor.setMarginWidth(2);
//    _tableDecor.setShowHover(false);
    _tableDecor.hide();
       
    GridData gridData = new GridData(SWT.DEFAULT, 
        10 * table.getItemHeight() + table.getHeaderHeight());
//    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    // Schaffe Platz fr Decorator Image
    gridData.horizontalIndent = image.getBounds().width - 2;
    table.setLayoutData(gridData);
    
    _tvHAQ.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        enableDisableButtons();
      }
    });
    
     _tvHAQ.setInput(_stufen);
  }
  
  
  private void validateTableData() {
    
    final StringBuffer textBuffer = new StringBuffer();
    
    if (!_stufen.isEmpty()) {
      List<StufeEntry> pruefListe = new ArrayList<StufeEntry>();
      pruefListe.addAll(_stufen);
      
      int wert = -1;
      boolean komplett = false;
      
      // Prfe unsortierte Liste bezgl. Reihenfolge Helligkeitswerte
      for (StufeEntry entry: pruefListe ) {
        if (!(entry.helligkeitsWert > wert)) {
          if (komplett) {
            textBuffer.append("Die Werte von 'Helligkeitswert' mssen aufsteigend gem den Stufen sein");          
          }
          else {
            textBuffer.append("Die Werte von 'Helligkeitswert' mssen aufsteigend gem den Stufen 100% berstreichen");          
          }
          break;
        }
        wert = entry.helligkeitsWert;
        if (entry.helligkeitsWert == 100) {
          komplett = true;
        }
      }
      if (!komplett && textBuffer.length() == 0) {
        textBuffer.append("Die Werte von 'Helligkeitswert' mssen 100% berstreichen");
      }
      
      Collections.sort(pruefListe);  
      wert = -1;
      boolean ein = false;
      boolean aus = false;
      
      for (StufeEntry entry: pruefListe ) {
        if (!ein && !(entry.grenzwertEin > wert)) {
          if (textBuffer.length() > 0) {
            textBuffer.append("\n");
          }
          textBuffer.append("Die Werte von 'GrenzwertEin' mssen aufsteigend gem der Helligkeitswerte sein");          
          ein = true;
        }
        wert = entry.grenzwertEin;

        if (!aus && entry.grenzwertAus > entry.grenzwertEin) {
          if (textBuffer.length() > 0) {
            textBuffer.append("\n");
          }
          textBuffer.append("Die Werte von 'GrenzwertAus' drfen nicht grsser sein als die jeweils zugehrigen Werte von 'GrenzwertEin'");          
          aus = true;
        }
        
        if (ein && aus) {
          break;
        }
      }
    }
    
    _tableDecor.setDescriptionText(textBuffer.toString());
    
    Display.getCurrent().asyncExec(new Runnable() {

      @Override
      public void run()
      {
        if (textBuffer.length() > 0) {
          _tableDecor.show();
          layout(true);
        }
        else {
          _tableDecor.hide();
        }
        
      }
      
    });
    
  }
   
  @Override
  public Data getUpdate()
  {
    if (!isInitialized())
      return null;
    
    Data result = _data.createModifiableCopy();

    String steuerung = _cbSteuerung.getText();
    if (_iatSteuerung != null) {
      boolean fertig = false;
      for (IntegerValueState state: _iatSteuerung.getStates()) {
        if (state.getName().equals(steuerung)) {
          result.getItem("Steuerung").asTextValue().setText(steuerung);
          fertig = true;
          break;
        }
      }
      if (!fertig) {
        IntegerValueRange range = _iatSteuerung.getRange();
        if (range != null) {
          for (long value = range.getMinimum(); value <= range.getMaximum(); value++) {
            if (("" + value).equals(steuerung)) {
              result.getItem("Steuerung").asTextValue().setText(steuerung);
              break;
            }
          }
        }
      }
    }
    
    long sekunden = _dtStandzeit.getHours() * 3600 + _dtStandzeit.getMinutes() * 60 + _dtStandzeit.getSeconds();
    result.getItem("Standzeit").asTimeValue().setSeconds(sekunden);
        
    Array array = result.getArray("Stufen");
    array.setLength(0);
    
    for (int i = 0; i < _stufen.size(); i++) {
      StufeEntry entry = _stufen.get(i);
      int index = array.getLength();
      array.setLength(index + 1);
      Data item = array.getItem(index);
      item.getItem("Helligkeitswert").asUnscaledValue().set(entry.helligkeitsWert);
      item.getItem("GrenzwertEin").asUnscaledValue().set(entry.grenzwertEin);
      item.getItem("GrenzwertAus").asUnscaledValue().set(entry.grenzwertAus);
    }
    
    return result;
  }

  @Override
  public void setUpdate(Data data)
  {
    if (!isInitialized())
      return;
    
    Data copy = data.createModifiableCopy();
    
    Data item = copy.getItem("Steuerung");
    if (item.isDefined()) {
      String steuerung = item.asTextValue().getText();
      if (Arrays.asList(_cbSteuerung.getItems()).contains(steuerung)) {
        _cbSteuerung.setText(steuerung);  
      }
      else if (_cbSteuerung.getItemCount() > 0) {
        _cbSteuerung.setText(_cbSteuerung.getItem(0));
      }
    }
    else if (_cbSteuerung.getItemCount() > 0) {
      _cbSteuerung.setText(_cbSteuerung.getItem(0));
    }
     
    item = copy.getItem("Standzeit");
    if (item.isDefined()) {
      long standzeit = data.getItem("Standzeit").asTimeValue().getSeconds();
      if (standzeit < 24 * 3600) {
        int h = (int)standzeit / 3600;
        int m = (int)(standzeit % 3600) / 60;
        int s = (int)(standzeit % 3600) % 60;
        _dtStandzeit.setTime(h, m, s);
      }
      else {
        _dtStandzeit.setTime(23, 59, 59);
      }
    }
    else {
      _dtStandzeit.setTime(0, 0, 0);
    }
    
    Array array = data.getArray("Stufen");
    _stufen.clear();
    
    for (int i = 0; i < array.getLength(); i++) {
      item = array.getItem(i);
      int helligkeitsWert = item.getItem("Helligkeitswert").asUnscaledValue().intValue();
      StufeEntry entry = new StufeEntry(helligkeitsWert);
      entry.grenzwertEin = item.getItem("GrenzwertEin").asUnscaledValue().intValue();
      entry.grenzwertAus = item.getItem("GrenzwertAus").asUnscaledValue().intValue();
      
      _stufen.add(entry);
    }
    
//    Collections.sort(_stufen);
    _tvHAQ.refresh();
    validateTableData();
    enableDisableButtons();
  }
  
  private void actionHinzufuegenStufe() {
    int value = this._spStufe.getSelection();
    
    _stufen.add(new StufeEntry(value));
    
    Collections.sort(_stufen);
    
    _tvHAQ.refresh();
      
    validateTableData();
    enableDisableButtons();
  }
  
  private void actionEntfernenStufen() {
    ISelection sel = _tvHAQ.getSelection();
    if (!sel.isEmpty() && sel instanceof StructuredSelection) {
      
      Object[] array = ((StructuredSelection)sel).toArray();
      ListIterator<StufeEntry> it = _stufen.listIterator();
      while(it.hasNext()) {
        StufeEntry entry = it.next();
        for (int i = 0; i < array.length; i++) {
          if (entry.equals(array[i])) {
            it.remove();
            break;
          }
        }
      }

      _tvHAQ.refresh();
      
      validateTableData();
      enableDisableButtons();
    }
  }
  
  private void enableDisableButtons() {
    
    boolean gefunden = false;
    int value = _spStufe.getSelection();
    
    for (StufeEntry entry: _stufen) {
      if (entry.helligkeitsWert == value) {
        gefunden = true;
        break;
      }
    }
    
    _btnHinzufuegen.setEnabled(!gefunden);
        
    _btnEntfernen.setEnabled(_tvHAQ.getTable().getSelectionCount() > 0);
  }
  
  private class StufeEntry implements Comparable<StufeEntry >{
    
    public final int helligkeitsWert;
    public int grenzwertEin;
    public int grenzwertAus;
    
    public StufeEntry(int helligkeitsWert) {
      this.helligkeitsWert = helligkeitsWert;
    }

    @Override
    public int compareTo(StufeEntry o)
    {
      if (helligkeitsWert > o.helligkeitsWert) {
        return 1;
      }
      else if (helligkeitsWert < o.helligkeitsWert) {
        return -1;
      }
      else {
        if (grenzwertEin > o.grenzwertEin) {
          return 1;
        }
        else if (grenzwertEin < o.grenzwertEin) {
         return -1; 
        }
        else {
          if (grenzwertAus > o.grenzwertAus) {
            return 1;
          }
          else if (grenzwertAus < o.grenzwertAus) {
           return -1; 
          }
          else {
            return 0;
          }
        }
      }
    }
  }
}
