/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.dav;

import java.util.Date;
import java.util.List;

/**
 * Startet Anfragen ans Archiv als Thread. Entkoppelt die Oberflche vom endgltigen Ergebnis. Damit wird ein Blockieren
 * verhindert.
 * 
 * @author HofmannJ
 * 
 */
public class ArchiveRequestStarter implements Runnable
{
  /**
   * Archivanfrageklasse.
   */
  private ArchiveRequest _archiveRequest = null;

  /**
   * Einzelne {@link AnfrageDaten}.
   */
  private AnfrageDaten _anfrageDaten = null;

  /**
   * Liste von {@link AnfrageDaten}.
   */
  private List<AnfrageDaten> _lAnfrageDaten = null;

  /**
   * Startet eine Archivanfrage als Thread mit einzelnen {@link AnfrageDaten}.
   * 
   * @param archiveRequest
   *          Archivanfrage.
   * @param anfrageDaten
   *          Einzelne {@link AnfrageDaten}.
   */
  public ArchiveRequestStarter(ArchiveRequest archiveRequest, AnfrageDaten anfrageDaten)
  {
    this._archiveRequest = archiveRequest;
    this._anfrageDaten = anfrageDaten;
    startThread();
  }

  /**
   * Startet eine Archivanfrage als Thread mit mehreren {@link AnfrageDaten}.
   * 
   * @param archiveRequest
   *          Archivanfrage.
   * @param lAnfrageDaten
   *          Liste von {@link AnfrageDaten}.
   */
  public ArchiveRequestStarter(ArchiveRequest archiveRequest, List<AnfrageDaten> lAnfrageDaten)
  {
    this._archiveRequest = archiveRequest;
    this._lAnfrageDaten = lAnfrageDaten;
    startThread();
  }

  /**
   * Startet den Thread.
   */
  private void startThread()
  {
    Thread th = new Thread(this);
    th.setName("ArchiveRequestStarter " + new Date().toString());
    th.start();
  }

  @Override
  public void run()
  {
    if (_anfrageDaten != null)
    {
      _archiveRequest.startRequest(_anfrageDaten);
    }
    else if (_lAnfrageDaten != null)
    {
      _archiveRequest.startRequest(_lAnfrageDaten);
    }

  }

}
