/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.expose;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.font.DaVHelper;
import de.bsvrz.buv.plugin.pua.basis.font.VisuAnzeigeFonts;
import de.bsvrz.buv.plugin.pua.basis.font.VisuAnzeigeFontsZeichen;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.views.ProtokollAusgabeSicht;
import de.bsvrz.buv.plugin.pua.basis.views.Tools;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResultData;
import de.bsvrz.pua.prot.data.ProtocolData;

public class NachbereitungSchaltbildAnzeige implements INachbereitung {
	
	public Collection<SystemObject> getObjekte(Collection<SystemObject> objekte,
			ProtokollInformationen protokollInformationen) {
		if (protokollInformationen == null) {
			throw new IllegalArgumentException("ProtokollInformationen drfen nicht null sein");
		}
		return getSelektierteObjekte(protokollInformationen);
		
	}
	
	protected List<SystemObject> getSelektierteObjekte(ProtokollInformationen protokollInformationen) {
		List<SystemObject> result = new ArrayList<SystemObject>();
		
		ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
		if (connection != null) {
			DataModel dataModel = connection.getDataModel();
			String infoText = protokollInformationen.getInformationsText();
	    	int index = infoText.indexOf('[');
	    	if (index == 0) {
	    		index = infoText.indexOf(']');
	    		if (index > 0) {
	    			String verborgenerText = infoText.substring(1, index);
	    			String[] zerlegung1 = verborgenerText.split(":");
	    			if (zerlegung1.length > 1) {
	    				String pidList = zerlegung1[1];
	    				String[] zerlegung2 = pidList.split(",");
	    				for (int i = 0; i < zerlegung2.length; i += 1) {
	    					String pid = zerlegung2[i];
	    					SystemObject so = dataModel.getObject(pid);
	    					if (so != null) {
	    						result.add(so);
	    					}
	    				}
	    			}
	    		}
	    	}
		}
		return result;
	}
	
	protected SystemObject getAnzeigeQuerschnitt(SystemObject anzeige) {
		SystemObject result = null;
		DataModel dataModel = anzeige.getDataModel();
		
		for (SystemObject aq: dataModel.getType("typ.anzeigeQuerschnitt").getElements()) {
			NonMutableSet set = ((ConfigurationObject)aq).getNonMutableSet("Anzeigen");
			if (set != null) {
				for (SystemObject element: set.getElements()) {
					if (element == anzeige) {
						result = aq;
						break;
					}
				}
			}
		}
		
		return result;
	}
	
	public List<String[]> getZellenInhalte(List<SystemObject> objekte, int[] objektZuordnung, 
			ProtocolResultData prd, IProjektInfo projektInfo, ProtokollInformationen protokollInformationen) {
		if (projektInfo == null) {
			throw new IllegalArgumentException("ProjektInfo darf nicht null sein");
		}
		if (protokollInformationen == null) {
			throw new IllegalArgumentException("ProtokollInformationen drfen nicht null sein");
		}

		List<String[]> result = new ArrayList<String[]>();
		String zeitStempel = SDF.format(new Date(prd.getTimeStamp()));
		int anzahlAttribute = projektInfo.getAnzahlAttribute();
		Map<String, String[]> zellenwerteMap = new LinkedHashMap<String, String[]>();
		List<SystemObject> selektierteObjekte = getSelektierteObjekte(protokollInformationen);
		
		for (int i = 0; i < objekte.size(); i++) {
			SystemObject sba = objekte.get(i);
			
			// Prfe ob Daten vorhanden sind
			    if (i >= prd.values.length) {
			      break;
			    }
        	if (prd.values[i].getValue() != null && !prd.values[i].getValue().hasValue()) {
        		continue;
        	}
        	
        	DataModel dataModel = sba.getDataModel();
        	ProtocolData protocolData = prd.getData(dataModel, i);
			
			// Bearbeite Attributgruppe atg.anzeigeEigenschaftSoll
			if (protocolData.getName().equals(dataModel.getAttributeGroup("atg.sswSbaSchaltbildAnzeige").getName()) &&
					protocolData.isList()) {
				
				Array anzeigen = protocolData.getArray("Anzeigen");
				for (int j = 0; j < anzeigen.getLength(); j += 1) {
					Data item = anzeigen.getItem(j);
					SystemObject anzeige = item.getReferenceValue("Anzeige").getSystemObject();
					if (anzeige != null) {
						SystemObject aq = getAnzeigeQuerschnitt(anzeige);
						if (aq != null && selektierteObjekte.contains(aq)) {
							String[] zellenwerte  = zellenwerteMap.get(sba.getPid() + ":" + aq.getPid());
			       	  		if (zellenwerte == null) {
			       	  			zellenwerte = new String[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + anzahlAttribute];
			           	  		zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = zeitStempel;
			           	  		zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = aq.getNameOrPidOrId();
			 					zellenwerteMap.put(sba.getPid() + ":" + aq.getPid(), zellenwerte);
			       	  		}
			       	  		
				       	 	int index = projektInfo.getSpaltenIndexRelativ(anzeige.getPid());
		            		if (index >= 0) {
		            			String zellenText = "";
		            			Data schaltbild = item.getItem("Schaltbild");
//		            			SystemObject anzeigeInhalt = schaltbild.getReferenceValue("AnzeigeInhalt").getSystemObject();
//	//	            			System.out.println("Bearbeite Attribut \"AnzeigeInhalt\" fr Objekt " + anfrageObj.getPid() + " am Index " + i);
//		            			if (anzeigeInhalt != null)
//		            			{ 
//		            				String text = "";
//		            				AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");
//		            				Data data = anzeigeInhalt.getConfigurationData(atg);
//		            				if (data != null) {
//		            					String bildInhalt = data.getTextValue("Bildinhalt").getValueText();
//		            					if (bildInhalt != null && ! bildInhalt.isEmpty()) {
//		            						text = projektInfo.getBildInhalt(bildInhalt);
//		            					}
//		            					else {
//		            						text = bildInhalt;
//		            					}
//		            				}
//		            				zellenText += text;
//		            			}
//		            			else {
//		            				zellenText += "undefiniert";
//		            			}
		                  
		                  // bearbeite AnzeigeInhalt
		            			
		            			if (schaltbild != null){
		            			  SystemObject anzeigeInhalt = null;
		            			  try {
		            			    anzeigeInhalt = schaltbild.getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();  
		            			  }
		            			  catch (ArithmeticException e) {
		            			    zellenText = Tools.toText(protocolData);
		            			  }                 
		            			  
		            			  if (anzeigeInhalt != null)
		            			  {
		            			    AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");
		            			    Data data = anzeigeInhalt.getConfigurationData(atg);
		            			    if (data != null) {
		            			      String bildInhalt = data.getTextValue("Bildinhalt").getValueText();
		            			      if (bildInhalt != null && ! bildInhalt.isEmpty()) {
		            			        zellenText = projektInfo.getBildInhalt(bildInhalt);
		            			      }
		            			      else {
		            			        zellenText = bildInhalt;
		            			      }
		            			    } 
		            			  }
		            			  else 
		            			  {
		            			    // bearbeite Wechseltext              
		            			    SystemObject soWT = null;
		            			    try {
		            			      soWT = schaltbild.getItem("Wechseltext").asReferenceValue().getSystemObject();
		            			    }
		            			    catch (ArithmeticException e) {
		            			      zellenText = Tools.toText(protocolData);
		            			    }                 
		            			    
		            			    if (soWT != null)
		            			    {
		            			      AttributeGroup atg = dataModel.getAttributeGroup("atg.wechseltext");
		            			      Data dataWT = soWT.getConfigurationData(atg);
		            			      
		            			      if (dataWT != null) {
		            			        String pidFont = dataWT.getItem("Font").asTextValue().getText();
		            			        String ausrichtung = dataWT.getItem("TextAusrichtung").asTextValue().getText();
		            			        SystemObject[] sos = dataWT.getItem("Textdefinition").asReferenceArray().getSystemObjectArray();
		            			        System.out.println(pidFont + " " + ausrichtung + "->2: " + sos.length);
		            			        for (SystemObject so : sos)
		            			        {
		            			          dataModel.getObject(pidFont);
		            			          VisuAnzeigeFontsZeichen zeichen = VisuAnzeigeFonts.getInstanz().getFontZeichenUeberPid(pidFont, so.getPid());
		            			          Integer code = zeichen.getAsciiCode();                    
		            			          if (DaVHelper.getMapTextKonserve().containsKey(code)){
		            			            zellenText += DaVHelper.getMapTextKonserve().get(code);
		            			          }else{
		            			            char c = (char)code.intValue();                        
		            			            zellenText += c;
		            			          }
		            			        }
		            			      }else {
		            			        zellenText = projektInfo.getBildInhalt("ReferenzUndefiniert");
		            			        if (zellenText == null)
		            			        {
		            			          zellenText = "<unbekannt>";
		            			        }
		            			      }
		            			    }
		            			    else
		            			    {
		            			      // bearbeite dynamischer Wechseltext
		            			      
		            			      Data dataDWT = null;
		            			      try {
		            			        dataDWT = schaltbild.getItem("DynamischerWechseltext");
		            			        
		            			      }
		            			      catch (ArithmeticException e) {
//		                            System.err.println(e.getMessage());
		            			        zellenText = Tools.toText(protocolData);
		            			      }                 
		            			      
		            			      if (dataDWT != null)
		            			      {
		            			        SystemObject font = dataDWT.getItem("Font").asReferenceValue().getSystemObject();
		            			        if (font != null) {
		            			          String pidFont = font.getPid();
		            			          String ausrichtung = dataDWT.getItem("TextAusrichtung").asTextValue().getText();
		            			          SystemObject[] sos = dataDWT.getItem("Textdefinition").asReferenceArray().getSystemObjectArray();
		            			          
		            			          for (SystemObject so : sos)
		            			          {
		            			            dataModel.getObject(pidFont);
		            			            VisuAnzeigeFontsZeichen zeichen = VisuAnzeigeFonts.getInstanz().getFontZeichenUeberPid(pidFont, so.getPid());
		            			            Integer code = zeichen.getAsciiCode();
		            			            
		            			            if (DaVHelper.getMapTextKonserve().containsKey(code)){
		            			              zellenText += DaVHelper.getMapTextKonserve().get(code);
		            			            }else{
		            			              char c = (char)code.intValue();                        
		            			              zellenText += c;
		            			            }
		            			            
		            			          }  
		            			        }
		            			        else {
		            			          zellenText = projektInfo.getBildInhalt("ReferenzUndefiniert");
		            			          if (zellenText == null)
		            			          {
		            			            zellenText = "<unbekannt>";
		            			          }
		            			        }
		            			      }
		            			    }
		            			  }
		            			  
		            			}
		            			
		            			zellenText += ",";
		            			zellenText += item.getTextValue("Prioritt").getText();
		            			zellenText += ",";
		            			zellenText += item.getTextValue("Programmstatus").getText();
		            			zellenText += ",";
		            			SystemObject quelle = item.getReferenceValue("Quelle").getSystemObject();
		            			zellenText += quelle != null ? quelle.getNameOrPidOrId() : "undefiniert";
		            			zellenText += ",";
		            			SystemObject schaltgrund = item.getReferenceValue("Schaltgrund").getSystemObject();
		            			zellenText += schaltgrund != null ? schaltgrund.getNameOrPidOrId() : "undefiniert";
		            				
		            			zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + index] = zellenText;
		            		}	
						}
					}
				}
			}
		}
		
		result.addAll(zellenwerteMap.values());
		
		return result;
	}

}
