/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.expose;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.views.ProtokollAusgabeSicht;
import de.bsvrz.buv.plugin.pua.basis.views.Tools;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResultData;
import de.bsvrz.pua.prot.data.ProtocolData;

public class NachbereitungTSFZustandArgoscan extends NachbereitungDefault {
	
	
	
	@Override
	public List<String[]> getZellenInhalte(List<SystemObject> objekte, int[] objektZuordnung, 
			ProtocolResultData prd, IProjektInfo projektInfo, ProtokollInformationen protokollInformationen) {
		
		if (projektInfo == null) {
			throw new IllegalArgumentException("ProjektInfo darf nicht null sein");
		}
		List<String[]> zellenwerteListe = new ArrayList<String[]>();
		
		ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
		if (connection != null) {
			List<ProtocolData> dataList = prd.getDataList(connection.getDataModel());
		    String zeitStempel = INachbereitung.SDF.format(new Date(prd.getTimeStamp()));
		    Map<String, String[]> zellenwerteMap = new LinkedHashMap<String, String[]>();
		    Map<String, Integer> positionsMap = new HashMap<String, Integer>();
		    
		    int anzahlSpalten = ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + projektInfo.getAnzahlAttribute();
    	  
		    for (int i = 0; i < dataList.size(); i++)
		    {
    		  int objektIndex = objektZuordnung[i];
    		  SystemObject anfrageObj = objekte.get(objektIndex);
    		  String pid = anfrageObj.getPid();
    		  String[] zellenwerte = zellenwerteMap.get(pid);
    		  
    		  if (zellenwerte == null) {
     	  		zellenwerte = new String[anzahlSpalten];
					zellenwerteMap.put(pid, zellenwerte);
					// Datum + Objektbezeichner
	                zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = zeitStempel;
	                String name = anfrageObj.getNameOrPidOrId();
	                zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = name;
	                positionsMap.put(pid,ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL);
    		  }
    		  
    		  int position = positionsMap.get(pid);
    		  ProtocolData protocolData = dataList.get(i);
    		  
    		  if (protocolData.isPlain())
              {
    			zellenwerte[position] = Tools.toText(protocolData);	  
    			  
    			if ("Zustand".equals(protocolData.getName())) {
    				// Sonderbehandlung Feld "Zustand"
    				
    				if (protocolData.getAttributeType() instanceof IntegerAttributeType) {
    					IntegerAttributeType attType = (IntegerAttributeType)protocolData.getAttributeType();
    					
//    					System.out.println("TYPE: " + attType.getName());
//    					for (IntegerValueState state :attType.getStates()) {
//    						System.out.println("\tSTATE: " + state.getName() + " VALUE: " + state.getValue());
//    					}
    					try
    					{
	    					long value = protocolData.asUnscaledValue().longValue();
	    					
	    					if (value == 9) {
	    						zellenwerte[position] += " (Initialisierung abgeschlossen)";
	    					}
    					}
    					catch (ArithmeticException e) {
    						// Datum hat keinen Zahlenwert
    						// tue nichts
    					}
    				}
    			}
                if (position < zellenwerte.length) {
                	position++;	
                }
                else {
                	System.err.println("Spaltenberlauf bei Spalte " + i + " Position " + 
                			position + " fr Objekt " + pid);
                }
              }
              else
              {
            	  List<String> zellenwerteFlach = new ArrayList<String>();
                  doZellenwerteAufloesen(protocolData, zellenwerteFlach);
                  // hor9tr: Bugfix - Array und Liste haben nur einen Header aber viele Werte -> Liste von Werten
                  //                  in einen flachen String kopieren und der Spalte des Headers hinzufuegen
                  if((anzahlSpalten - ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL) == 1 && zellenwerteFlach.size() > 1)
                  {
                    String zellenwert = zellenwerteFlach.get(0);
                    for(int m = 1; m < zellenwerteFlach.size(); m++)
                    {
                      zellenwert = zellenwert + ", " + zellenwerteFlach.get(m);
                    }
                    zellenwerteFlach.clear();
                    zellenwerteFlach.add(zellenwert);
                  }
                  for (int k = 0; k < zellenwerteFlach.size() && position < anzahlSpalten; k++)
                  {
                    zellenwerte[position] = zellenwerteFlach.get(k);
                    if (position < zellenwerte.length) {
                    	position++;	
                    }
                    else {
                    	System.err.println("Spaltenberlauf bei Spalte " + i + " Position " + 
                    			position + " fr Objekt " + pid);
                    	break;
                    }
                  }
                  
                  if (zellenwerteFlach.size() == 0) // Leere Zelle mit Leerstring belegen und dadurch berspringen
                  {
                    zellenwerte[position] = "";
                    
                    if (position < zellenwerte.length) 
                    {
                      position++;
                    }
                  }
              }
    		  positionsMap.put(pid, position);
          }	
    	  zellenwerteListe.addAll(zellenwerteMap.values());  
		}  
		return zellenwerteListe;
	}
}
