/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.expose;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.views.ProtokollAusgabeSicht;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResultData;
import de.bsvrz.pua.prot.data.ProtocolData;

public class NachbereitungTsfBefehle implements INachbereitung {
	
	public Collection<SystemObject> getObjekte(Collection<SystemObject> objekte,
			ProtokollInformationen protokollInformationen) {
		
		List<SystemObject> result = new ArrayList<SystemObject>();
		
		for (SystemObject so: objekte) {
			if (so != null && so.isOfType("typ.streckenBeeinflussungsAnlageBereich")) {
				result.add(so);
			}
		}
		
		return result;
	}
	
	protected SystemObject getAnzeigeQuerschnitt(SystemObject anzeige) {
		SystemObject result = null;
		DataModel dataModel = anzeige.getDataModel();
		
		for (SystemObject aq: dataModel.getType("typ.anzeigeQuerschnitt").getElements()) {
			NonMutableSet set = ((ConfigurationObject)aq).getNonMutableSet("Anzeigen");
			if (set != null) {
				for (SystemObject element: set.getElements()) {
					if (element == anzeige) {
						result = aq;
						break;
					}
				}
			}
		}
		
		return result;
	}
	
	public List<String[]> getZellenInhalte(List<SystemObject> objekte, int[] objektZuordnung, 
			ProtocolResultData prd, IProjektInfo projektInfo, ProtokollInformationen protokollInformationen) {
		if (projektInfo == null) {
			throw new IllegalArgumentException("ProjektInfo darf nicht null sein");
		}
		List<String[]> result = new ArrayList<String[]>();
		String zeitStempel = SDF.format(new Date(prd.getTimeStamp()));
		int anzahlAttribute = projektInfo.getAnzahlAttribute();
		
//		System.out.println("OBJEKTE: " + objekte.size() + "  PRD: " + prd.values.length);
		
		for (int i = 0; i < objekte.size(); i++) {

			SystemObject anfrageObj = objekte.get(i);
        
            // Attributwerte
            if (i < prd.values.length) {
            	
            	// Prfe ob Daten vorhanden sind
            	if (prd.values[i].getValue() != null && !prd.values[i].getValue().hasValue()) {
            		continue;
            	}
            	
            	DataModel dataModel = anfrageObj.getDataModel();
				ProtocolData protocolData = prd.getData(dataModel, i);

				// Bearbeite Attributgruppe atg.sbaUeSchaltBild
				if (protocolData.getName().equals(dataModel.getAttributeGroup("atg.sbaUeSchaltbild").getName()) &&
						protocolData.isList()) {
					Array anzeigen = protocolData.getArray("Anzeigen");
					Map<String, String[]> zellenwerteMap = new LinkedHashMap<String, String[]>();
//					System.out.println("Bearbeite Array \"Anzeigen\" fr Objekt " + objekte.get(i) + " am Index " + i);
					for (int k = 0; k < anzeigen.getLength(); k++) {
						Data item = anzeigen.getItem(k);
						SystemObject anzeige = item.getReferenceValue("Anzeige").getSystemObject();
						int index = projektInfo.getSpaltenIndexRelativ(anzeige.getPid());
						if (index >= 0) {
							SystemObject aq = getAnzeigeQuerschnitt(anzeige);
							if (aq != null) {
								String[] zellenwerte = zellenwerteMap.get(aq.getPid());
								if (zellenwerte == null) {
									zellenwerte = new String[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + anzahlAttribute];
									zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = zeitStempel;
						            zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = aq.getNameOrPidOrId();
						            zellenwerteMap.put(aq.getPid(), zellenwerte);
								}
								Data schaltbild = item.getItem("Schaltbild");
								SystemObject anzeigeInhalt = schaltbild.getReferenceValue("AnzeigeInhalt").getSystemObject();
								String text = "";
								if (anzeigeInhalt != null) {
									AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");
									Data data = anzeigeInhalt.getConfigurationData(atg);
									if (data != null) {
										String bildInhalt = data.getTextValue("Bildinhalt").getValueText();
										if (bildInhalt != null && ! bildInhalt.isEmpty()) {
											text = projektInfo.getBildInhalt(bildInhalt);
										}
										else {
											text = bildInhalt;
										}
									}  
								}
								zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + index] = text;	
							}
						}
					}	
//					Data urlasser = protocolData.getItem("Urlasser");
////					System.out.println("Bearbeite Attributliste \"Urlasser\" fr Objekt " + objekte.get(i) + " am Index " + i);
//					SystemObject benutzer = urlasser.getReferenceValue("BenutzerReferenz").getSystemObject();
//					// Benutzername kommt in die drittletzte Spalte
//					zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + projektInfo.getAnzahlAttribute() - 3] = 
//						benutzer != null ? benutzer.getNameOrPidOrId() : "";
////					System.out.println("Benutzer: " + (benutzer != null ? benutzer.getName() : benutzer));
//					String ursache = urlasser.getTextValue("Ursache").getText();
//					// "Ursache" kommt in die zweitletzte Spalte
//					zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + projektInfo.getAnzahlAttribute() - 2] = ursache;
//					String veranlasser = urlasser.getTextValue("Veranlasser").getText();
//					// "Veranlasser" kommt in die letzte Spalte
//					zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + projektInfo.getAnzahlAttribute() - 1] = veranlasser;
					
					for (String[] werte:zellenwerteMap.values()) {
						result.add(werte);
					}
				}	
            }
		}
		return result;
	}
}
