/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.expose;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.views.ProtokollAusgabeSicht;
import de.bsvrz.buv.plugin.pua.basis.views.Tools;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResultData;
import de.bsvrz.pua.prot.data.ProtocolData;

public class NachbereitungZraVorschlaege extends NachbereitungZraAktiv implements INachbereitung {
	
	private static String PREFIX = "ErgebnisZufluss";
	
	@Override
	public List<String[]> getZellenInhalte(List<SystemObject> objekte, int[] objektZuordnung, 
			ProtocolResultData prd, IProjektInfo projektInfo, ProtokollInformationen protokollInformationen) {
		
		List<String[]> result = new ArrayList<String[]>();
		ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
		if (connection != null) {
			DataModel dataModel = connection.getDataModel();
			List<ProtocolData> dataList = prd.getDataList(dataModel);
	  	  	String zeitStempel = INachbereitung.SDF.format(new Date(prd.getTimeStamp()));
	  	    int anzahlAttribute = ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + projektInfo.getAnzahlAttribute();
	  	    	
	  	  	for (int i = 0; i < objekte.size(); i++)
	        {
	          String name = objekte.get(i).getNameOrPidOrId();

	          // Attributwerte
	          for (int j = i; j < dataList.size(); j += objekte.size())
	          {
	        	Map<String, String[]> zellenWertMap = new LinkedHashMap<String, String[]>();  
	            ProtocolData protocolData = dataList.get(j);
	            String attr = protocolData.getName();
	            if (attr.startsWith(PREFIX)) {
	            	String verfahren = attr.substring(PREFIX.length());
	            	if (verfahren.equals("PrognoseFR") ||verfahren.equals("PrognoseFSr") || verfahren.equals("ALINEA")) {
	            		String[] zellenwerte = zellenWertMap.get(verfahren);
	            		if (zellenwerte == null) {
	            			zellenwerte = new String[anzahlAttribute];
	            			// Datum + Objektbezeichner
	            	        zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = zeitStempel;
	            	        zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = name;
	            	        zellenWertMap.put(verfahren, zellenwerte);
	            		}
	            		if (protocolData.isList()) {
	            			zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 1] = verfahren;
	            			if (verfahren.equals("PrognoseFR") || verfahren.equals("PrognoseFSr")) {
	            				String qb = protocolData.getTextValue("QB").getValueText();
	            				zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 2] = qb;
	            				String k = protocolData.getTextValue("k").getValueText();
	            				zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 3] = k;
	            				String stufeqb = protocolData.getTextValue("StufeQB").getValueText();
	            				zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 4] = stufeqb;
	            				String stufek = protocolData.getTextValue("StufeK").getValueText();
	            				zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 5] = stufek;
	                		}
	                		else { // verfahren.equals("ALINEA") 
	                			String umlaufzeit = protocolData.getTextValue("Umlaufzeit").getValueText();
	            				zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 6] = umlaufzeit;
	            				String stufe = protocolData.getTextValue("Stufe").getValueText();
	            				zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 7] = stufe;
	                		}
	            		    String rotschaltzeit = protocolData.getTextValue("Rotschaltzeit").getValueText();
	            		    if ("0".equals(rotschaltzeit)) {
	            		    	rotschaltzeit = "aus";
	                    	}
	                    	else if ("241".equals(rotschaltzeit)) {
	                    		rotschaltzeit = "Dauerrot";
	                    	}
	            			zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 8] = rotschaltzeit;
	            		    String fahrzeuganzahl = protocolData.getTextValue("Fahrzeuganzahl").getValueText();
	            		    zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 9] = fahrzeuganzahl;
	            		}
	            		else {
	            			zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX + 1] = Tools.toText(protocolData);
	            		}
	            	}
	            }
	            result.addAll(zellenWertMap.values());
	          }
	        }
		}	
 	  	return result;
	}

}
