/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.protokolleUpdateListener;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.dav.daf.main.archive.ArchiveData;

/**
 * Die Klasse kapselt die Verwaltungsmethoden fr die Listener-Funktionalitt 
 * des IDaVDataUpdateListener. Diese Klasse implemntiert das Singleton-Pattern
 * und stellt dadurch sicher dass der Listener nur einmal Instanziiert wird. 
 * 
 * 
 * @author Pittner
 *
 */
public class ProtokolleUpdateListener
{
  /**
   * Liste der {@link IProtokolleUpdateListener}, die ber einen neue Protokoll-Daten informiert werden mchten.
   */
  private LinkedList<IProtokolleUpdateListener> _protokolleDataUpdateListener = new LinkedList<IProtokolleUpdateListener>();
  
  /**
   * Singleton-Instanz
   */
  private static ProtokolleUpdateListener _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static ProtokolleUpdateListener getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new ProtokolleUpdateListener();
    }
    return _instanz;
  }
    

  /**
   * Fgt einen Listener zur Liste der ProtokollDataUpdate-Listener hinzu.
   * 
   * @param listener
   */
  public void addProtokolleDataUpdateListener(IProtokolleUpdateListener listener)
  {
    if (!_protokolleDataUpdateListener.contains(listener))
    {
      _protokolleDataUpdateListener.add(listener);
    }
    else
    {
      System.err
          .println("Der Listener ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt");
    }
  }
  
  /**
   * Benachrichtigt alle registrierten {@link IProtokolleUpdateListener} darber, dass neue Daten vorhanden sind.
   * 
   */
  public void notifyProtokolleDataUpdateListener(Object[] result)
  {

    ProtokolleUpdateEvent event = new ProtokolleUpdateEvent(this, result);

    Iterator<IProtokolleUpdateListener> it = _protokolleDataUpdateListener.iterator();
    while (it.hasNext())
    {
      IProtokolleUpdateListener listener = it.next();
      listener.protokolleDataUpdate(event);
    }
  }
  
  /**
   * Entfernt einen {@link IProtokolleUpdateListener} von der Liste der Zuhrer.
   * 
   * @param IProtokolleUpdateListener
   *          Zu entfernender {@link IProtokolleUpdateListener}.
   */
  public void removeProtokolleUpdateListener(IProtokolleUpdateListener listener)
  {
    if (_protokolleDataUpdateListener.contains(listener))
    {
      _protokolleDataUpdateListener.remove(listener);
    }
    else
    {
      System.err.println("Der Listener ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }
  
}
