/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.pua.basis.pua;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;

import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.SkriptInformationen;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.ProtocolResultStream;
import de.bsvrz.pua.prot.util.ProcessingParameter;
import de.bsvrz.pua.prot.util.ProtocolRequestResult;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.datk.AtlMeta;
import de.bsvrz.sys.funclib.losb.datk.PidScript;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;
import de.bsvrz.sys.funclib.losb.exceptions.LoggerException;
import de.bsvrz.sys.funclib.losb.util.Tuple;

/**
 * Hauptklasse der Applikation ZwischenschichtAq.<br>
 * <br>
 * Die Applikation ZwischenschichtAq stellt die Verbindung zwischen dem Modell der KExTls und dem Modell AnzeigeGlobal
 * her. Damit die Applikation erfolgreich arbeiten kann, muss das Modell tmAnzeigenGlobalZwischenschichtTls sinnvoll
 * versorgt sein. <br>
 * <br>
 * Folgende Startparameter sind mglich:<br>
 * <br>
 * DAF-Startparameter wie in SSB beschrieben und durch StandardApplication verarbeitet:<br>
 * <br>
 * Beispiele:<br>
 * -datenverteiler=rechnerDav:8083<br>
 * -debugLevelStdErrText=FINE<br>
 * -benutzer=Tester<br>
 * -authentifizierung=C:\kernsoftware-system-20-03-2006\passwd<br>
 * <br>
 * Spezielle Startparameter:<br>
 * <br>
 * -bereiche=zu verwendende Konfigurationsbereiche durch ':' getrennt (optional, bei keiner Angabe Verwendung aller
 * vorhandenen Konfigurationsbereiche)<br>
 * <br>
 * Beispiele:<br>
 * -bereiche=kb.objekte.VerkehrMqFs.uz.Dambach <br>
 * 
 * @version $Revision: 1.3 $ / $Date: 2012/12/21 09:56:27 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 * 
 */
public class ProtokolleUndAuswertungenTest implements StandardApplication
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug;

  /**
   * Datenverteilerverbindung.
   */
  private ClientDavInterface m_connection = null;

  /**
   * Datenmodell.
   */
  private DataModel m_dataModel;

  /**
   * 
   */
  private ProtokolleUndAuswertungen m_protokolleUndAuswertungen;

  /**
   * Format fr Datumseingabe festlegen.
   */
  protected static final SimpleDateFormat m_sdf = new SimpleDateFormat("dd.MM.yyyyHH:mm:ss:SSS"); // yyyyHH OHNE Space
                                                                                                  // ist korrekt!

  /**
   * Konstruktor.
   */
  public ProtokolleUndAuswertungenTest()
  {
  }

  /**
   * Programmeinstieg.<br>
   * 
   * @param args
   *          Kommandozeilenargumente.
   */
  public static void main(String[] args)
  {
    StandardApplicationRunner.run(new ProtokolleUndAuswertungenTest(), args);
  }

  /*
   * (non-Javadoc)
   * 
   * @see sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList) throws Exception
  {
    m_debug = Debug.getLogger();

    m_debug.fine("argumentList = " + argumentList);
  }

  /*
   * (non-Javadoc)
   * 
   * @see sys.funclib.application.StandardApplication#initialize(stauma.dav.clientside.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection) throws Exception
  {
    try
    {
      m_connection = connection;

      m_dataModel = connection.getDataModel();

      m_protokolleUndAuswertungen = new ProtokolleUndAuswertungen(m_connection);

      System.out.println("Archiv verfgbar " + m_connection.getArchive().isArchiveAvailable());

      // client = new PuaClient(m_connection, m_connection.getLocalConfigurationAuthority());

      // aktualisiere Liste ungelesene Protokolle
      boolean ok;

      if (true)
      {
        ok = m_protokolleUndAuswertungen.aktualisiereListeUngeleseneProtokolle(BenutzerAngemeldet.getInstanz().getName());

        if (ok)
        {
          System.out.println("Liste ungelesene Protokolle:");

          for (ProtokollInformationen protokoll : m_protokolleUndAuswertungen.getListeUngeleseneProtokolle())
          {
            System.out.println("- " + protokoll);
          }
        }
      }

      // Zugriff auf ungelesenes Protokoll
      if (true)
      {
        System.out.println("Ungelesenes Protokoll:");

        ProtocolResultStream stream = m_protokolleUndAuswertungen.zugriffAufUngelesenesProtokoll(1249999944687L);

        if (stream != null)
          m_protokolleUndAuswertungen.printStream(stream);
        else
          System.out.println("Protokoll nicht verfgbar !");
      }

      // aktualisiere Liste gespeicherte Protokolle
      if (false)
      {
        ok = m_protokolleUndAuswertungen.aktualisiereListeGespeicherteProtokolle(BenutzerAngemeldet.getInstanz().getName());

        if (ok)
        {
          System.out.println("Liste gespeicherte Protokolle:");

          for (ProtokollInformationen protokoll : m_protokolleUndAuswertungen.getListeGespeicherteProtokolle())
          {
            System.out.println("- " + protokoll);
          }
        }
      }

      // Zugriff auf gespeichertes Protokoll
      if (false)
      {
        System.out.println("Gespeichertes Protokoll:");

        ProtocolResultStream stream = m_protokolleUndAuswertungen.zugriffAufGespeichertesProtokoll(1239181528496L);

        if (stream != null)
          m_protokolleUndAuswertungen.printStream(stream);
        else
          System.out.println("Protokoll nicht verfgbar !");
      }

      // Skript erstellen
      if (false)
      {
        SkriptInformationen skriptInformationen = new SkriptInformationen(
            "pua.skript.nw.atgVerkehrsDatenKurzZeitMqWerte1", "AtgVerkehrsDatenKurzZeitMqWerte1",
            "Abfrage Werte der ATG VerkehrsDatenKurzZeitMq", "Dambach-Werke GmbH, Th. Drapp", "Geprft", "08.04.2009",
            "pua.skript.nw.atgVerkehrsDatenKurzZeitMqWerte1");

        boolean erg = m_protokolleUndAuswertungen.erstelleSkript(skriptInformationen);

        System.out.println("erstelleSkript erfolgreich ? " + erg);

        System.out.println("requestResult '" + m_protokolleUndAuswertungen.getRequestResult() + "'");
      }

      if (false)
      {
        SkriptInformationen skriptInformationen = new SkriptInformationen(
            "pua.skript.nw.atgVerkehrsDatenKurzZeitGeglaettetMqWerte1", "AtgVerkehrsDatenKurzZeitGeglttetMqWerte1",
            "Abfrage Werte der ATG VerkehrsDatenKurzZeitGeglttetMq", "Dambach-Werke GmbH, Th. Drapp", "Geprft",
            "08.04.2009", "pua.skript.nw.atgVerkehrsDatenKurzZeitGeglaettetMqWerte1");

        boolean erg = m_protokolleUndAuswertungen.erstelleSkript(skriptInformationen);

        System.out.println("erstelleSkript erfolgreich ? " + erg);

        System.out.println("requestResult '" + m_protokolleUndAuswertungen.getRequestResult() + "'");
      }

      if (false)
      {
        SkriptInformationen skriptInformationen = new SkriptInformationen(
            "pua.skript.nw.atgVerkehrsDatenKurzZeitTrendExtraPolationMqWerte1",
            "AtgVerkehrsDatenKurzZeitTrendExtraPolationMqWerte1",
            "Abfrage Werte der ATG VerkehrsDatenKurzZeitTrendExtraPolationMq", "Dambach-Werke GmbH, Th. Drapp",
            "Geprft", "08.04.2009", "pua.skript.nw.atgVerkehrsDatenKurzZeitTrendExtraPolationMqWerte1");

        boolean erg = m_protokolleUndAuswertungen.erstelleSkript(skriptInformationen);

        System.out.println("erstelleSkript erfolgreich ? " + erg);

        System.out.println("requestResult '" + m_protokolleUndAuswertungen.getRequestResult() + "'");
      }

      // Protokoll erstellen
      if (false)
      {
        ProtokollInformationen protokollInformationen = new ProtokollInformationen();

//        ArchiveDataKind[] archiveDataKind = new ArchiveDataKind[] { ArchiveDataKind.ONLINE };
        Set<ArchiveDataKind> archiveDataKind = new HashSet<ArchiveDataKind>();
        archiveDataKind.add(ArchiveDataKind.ONLINE);
        protokollInformationen.setArchivDatenArt(archiveDataKind);

        Hashtable<String, String> bindungen = new Hashtable<String, String>();
        bindungen.put("?1", "asp.analyse");
        protokollInformationen.setBindungen(bindungen);

        String pid = "mq.MQ_1.010_AB_NO_R";

        protokollInformationen.setHauptObjekt(m_dataModel.getObject(pid));

        HashSet<SystemObject> objekte = new HashSet<SystemObject>();
        objekte.add(m_dataModel.getObject(pid));
        protokollInformationen.setObjekte(objekte);

        Hashtable<String, String[]> pseudoObjekte = new Hashtable<String, String[]>();
        pseudoObjekte.put("mq0", new String[] { pid });
        protokollInformationen.setPseudoObjekte(pseudoObjekte);

        protokollInformationen.setProtokollSpeichern(true);

        protokollInformationen.setSimlationsVariante((short)0);

        SystemObject skript = m_dataModel.getObject("pua.skript.nw.atgVerkehrsDatenKurzZeitMqWerte1");
        protokollInformationen.setSkript(skript);

        String periods = "01.04.200908:00:00:000-29.04.200908:10:00:000";

        List<Tuple<Long, Long>> periodList = new ArrayList<Tuple<Long, Long>>();

        if (periods != null)
        {
          String[] pList = periods.replace(" ", "").split(",");

          for (int i = 0; (pList != null && i < pList.length); i++)
          {
            String first = pList[i].split("-")[0];
            String last = pList[i].split("-")[1];

            try
            {
              GregorianCalendar gc = new GregorianCalendar();
              gc.setTime(m_sdf.parse(first));
              long start = gc.getTimeInMillis();
              gc.clear();
              gc.setTime(m_sdf.parse(last));
              long end = gc.getTimeInMillis();
              periodList.add(new Tuple<Long, Long>(start, end));
            }
            catch (ParseException e)
            {
              throw new FailureException(
                  "Zeitbereich wurde fehlerhaft angegeben: Erwartetes Format: dd.MM.yyyy HH:mm:ss:SSS.",
                  LoggerException.ERROR);
            }
          }
        }

        protokollInformationen.setIntervalle(periodList);

        try
        {
          ProcessingParameter processingParameter = protokollInformationen.getProcessingParameter();

          ProtocolResultStream stream = m_protokolleUndAuswertungen
              .erstelleProtokollMitSofortigerAusgabe(processingParameter);

          if (stream != null)
          {
            m_protokolleUndAuswertungen.printStream(stream);
          }
          else
            System.out.println("Protokoll nicht verfgbar !");
        }
        catch (FailureException e)
        {
          e.printStackTrace();
        }
      }

      if (false)
      {
        ProtokollInformationen protokollInformationen = new ProtokollInformationen();

//        ArchiveDataKind[] archiveDataKind = new ArchiveDataKind[] { ArchiveDataKind.ONLINE };
        Set<ArchiveDataKind> archiveDataKind = new HashSet<ArchiveDataKind>();
        archiveDataKind.add(ArchiveDataKind.ONLINE);
        protokollInformationen.setArchivDatenArt(archiveDataKind);

        Hashtable<String, String> bindungen = new Hashtable<String, String>();
        bindungen.put("?1", "asp.prognoseNormal");
        protokollInformationen.setBindungen(bindungen);

        protokollInformationen.setHauptObjekt(m_dataModel.getObject("mq.MQ_1.010_HFB_NO"));

        HashSet<SystemObject> objekte = new HashSet<SystemObject>();
        objekte.add(m_dataModel.getObject("mq.MQ_1.010_HFB_NO"));
        protokollInformationen.setObjekte(objekte);

        Hashtable<String, String[]> pseudoObjekte = new Hashtable<String, String[]>();
        pseudoObjekte.put("mq0", new String[] { "mq.MQ_1.010_HFB_NO", "mq.MQ_1.010_HFB_SW", "mq.MQ_1.020_HFB_NO",
            "mq.MQ_3070027_ZU_NO_L" });
        protokollInformationen.setPseudoObjekte(pseudoObjekte);

        protokollInformationen.setProtokollSpeichern(true);

        protokollInformationen.setSimlationsVariante((short)0);

        SystemObject skript = m_dataModel.getObject("pua.skript.nw.atgVerkehrsDatenKurzZeitGeglaettetMqWerte1");
        protokollInformationen.setSkript(skript);

        String periods = "07.04.200908:00:00:000-07.04.200908:10:00:000,07.04.200908:20:00:000-07.04.200908:30:00:000";

        List<Tuple<Long, Long>> periodList = new ArrayList<Tuple<Long, Long>>();

        if (periods != null)
        {
          String[] pList = periods.replace(" ", "").split(",");

          for (int i = 0; (pList != null && i < pList.length); i++)
          {
            String first = pList[i].split("-")[0];
            String last = pList[i].split("-")[1];

            try
            {
              GregorianCalendar gc = new GregorianCalendar();
              gc.setTime(m_sdf.parse(first));
              long start = gc.getTimeInMillis();
              gc.clear();
              gc.setTime(m_sdf.parse(last));
              long end = gc.getTimeInMillis();
              periodList.add(new Tuple<Long, Long>(start, end));
            }
            catch (ParseException e)
            {
              throw new FailureException(
                  "Zeitbereich wurde fehlerhaft angegeben: Erwartetes Format: dd.MM.yyyy HH:mm:ss:SSS.",
                  LoggerException.ERROR);
            }
          }
        }

        protokollInformationen.setIntervalle(periodList);

        try
        {
          ProcessingParameter processingParameter = protokollInformationen.getProcessingParameter();

          ProtocolResultStream stream = m_protokolleUndAuswertungen
              .erstelleProtokollMitSofortigerAusgabe(processingParameter);

          if (stream != null)
            m_protokolleUndAuswertungen.printStream(stream);
          else
            System.out.println("Protokoll nicht verfgbar !");
        }
        catch (FailureException e)
        {
          e.printStackTrace();
        }
      }

      if (false)
      {
        ProtokollInformationen protokollInformationen = new ProtokollInformationen();

//        ArchiveDataKind[] archiveDataKind = new ArchiveDataKind[] { ArchiveDataKind.ONLINE };
        Set<ArchiveDataKind> archiveDataKind = new HashSet<ArchiveDataKind>();
        archiveDataKind.add(ArchiveDataKind.ONLINE);
        protokollInformationen.setArchivDatenArt(archiveDataKind);

        Hashtable<String, String> bindungen = new Hashtable<String, String>();
        bindungen.put("?1", "asp.prognoseNormal");
        protokollInformationen.setBindungen(bindungen);

        protokollInformationen.setHauptObjekt(m_dataModel.getObject("mq.MQ_1.010_HFB_NO"));

        HashSet<SystemObject> objekte = new HashSet<SystemObject>();
        objekte.add(m_dataModel.getObject("mq.MQ_1.010_HFB_NO"));
        protokollInformationen.setObjekte(objekte);

        Hashtable<String, String[]> pseudoObjekte = new Hashtable<String, String[]>();
        pseudoObjekte.put("mq0", new String[] { "mq.MQ_1.010_HFB_NO", "mq.MQ_1.010_HFB_SW", "mq.MQ_1.020_HFB_NO",
            "mq.MQ_3070027_ZU_NO_L" });
        protokollInformationen.setPseudoObjekte(pseudoObjekte);

        protokollInformationen.setProtokollSpeichern(true);

        protokollInformationen.setSimlationsVariante((short)0);

        SystemObject skript = m_dataModel.getObject("pua.skript.nw.atgVerkehrsDatenKurzZeitTrendExtraPolationMqWerte1");
        protokollInformationen.setSkript(skript);

        String periods = "07.04.200908:00:00:000-07.04.200908:10:00:000,07.04.200908:20:00:000-07.04.200908:30:00:000";

        List<Tuple<Long, Long>> periodList = new ArrayList<Tuple<Long, Long>>();

        if (periods != null)
        {
          String[] pList = periods.replace(" ", "").split(",");

          for (int i = 0; (pList != null && i < pList.length); i++)
          {
            String first = pList[i].split("-")[0];
            String last = pList[i].split("-")[1];

            try
            {
              GregorianCalendar gc = new GregorianCalendar();
              gc.setTime(m_sdf.parse(first));
              long start = gc.getTimeInMillis();
              gc.clear();
              gc.setTime(m_sdf.parse(last));
              long end = gc.getTimeInMillis();
              periodList.add(new Tuple<Long, Long>(start, end));
            }
            catch (ParseException e)
            {
              throw new FailureException(
                  "Zeitbereich wurde fehlerhaft angegeben: Erwartetes Format: dd.MM.yyyy HH:mm:ss:SSS.",
                  LoggerException.ERROR);
            }
          }
        }

        protokollInformationen.setIntervalle(periodList);

        try
        {
          ProcessingParameter processingParameter = protokollInformationen.getProcessingParameter();

          ProtocolResultStream stream = m_protokolleUndAuswertungen
              .erstelleProtokollMitSofortigerAusgabe(processingParameter);

          if (stream != null)
          {
            if (stream.isRequestSuccessful())
            {
              m_protokolleUndAuswertungen.printStream(stream);
            }
            else
            {
              System.out.println("ErrorMsg " + stream.getErrorMsg());
            }
          }
          else
          {
            System.out.println("Protokoll nicht verfgbar !");
          }

          System.out.println("requestResult '" + m_protokolleUndAuswertungen.getRequestResult() + "'");
        }
        catch (FailureException e)
        {
          e.printStackTrace();
        }
      }

      // Skript lschen

      if (false)
      {
        boolean erg = false;
        DynamicObject dyn = (DynamicObject)m_dataModel.getObject("pua.skript.nw.3");

        erg = m_protokolleUndAuswertungen.loescheSkript(dyn);

        if (erg)
        {
          System.out.println("loescheSkript erfolgreich ? " + erg);
        }

        System.out.println("requestResult '" + m_protokolleUndAuswertungen.getRequestResult() + "'");
      }

      // Skript ausgeben

      if (false)
      {
        ProtocolRequestResult requestResult;
        DynamicObject dyn = (DynamicObject)m_dataModel
            .getObject("pua.skript.nw.atgVerkehrsDatenKurzZeitTrendExtraPolationMqWerte1");

        StringBuffer buffer = new StringBuffer();
        AtlMeta meta = new AtlMeta();
        requestResult = m_protokolleUndAuswertungen.holeSkriptInformationen(dyn, buffer, meta);

        if (requestResult != null)
        {
          System.out.println("zeigeSkript erfolgreich ? " + requestResult);

          String retVal = "\tErfolgreich?\t"
              + (requestResult.isRequestSuccessful() ? "Ja." : "Nein.")
              + ((requestResult.isRequestSuccessful()) ? ("\n---\nSkript Details:\n" + meta + "\n---\nQuelltext:\n" + buffer)
                  : ("\n\tFehler:\n\t\t" + requestResult.getErrorMsg()));
          System.out.println(retVal);

          System.out.println("-> Start");
          StringBuilder sb = new StringBuilder();
          sb.append(meta.atlBase.toString() + "\n");
          sb.append("------------------" + "\n");
          sb.append(meta.atlDetail.toString() + "\n");
          sb.append("------------------" + "\n");
          for (int i = 0; i < meta.atlVersions.size(); i++)
            sb.append(meta.atlVersions.get(i) + "\n");

          System.out.println(sb.toString());
          System.out.println("<- Ende");

          System.out.println("-> Start2");

          System.out.println("Beschreibung " + meta.atlBase.description);

          System.out.println("<- Ende2");
        }

        System.out.println("requestResult '" + m_protokolleUndAuswertungen.getRequestResult() + "'");
      }

      // alle in der Menge vorhandenen Skripte ausgeben

      if (false)
      {
        List<SystemObject> liste = m_dataModel.getConfigurationAuthority().getMutableSet(PidScript.objectSet)
            .getElements();

        ProtocolRequestResult requestResult;

        for (SystemObject skript : liste)
        {
          StringBuffer buffer = new StringBuffer();
          AtlMeta meta = new AtlMeta();
          requestResult = m_protokolleUndAuswertungen.holeSkriptInformationen((DynamicObject)skript, buffer, meta);

          if (requestResult != null)
          {
            System.out.println("holeSkriptInformationen erfolgreich ? " + requestResult);

            String retVal = "\tErfolgreich?\t"
                + (requestResult.isRequestSuccessful() ? "Ja." : "Nein.")
                + ((requestResult.isRequestSuccessful()) ? ("\n---\nSkript Details:\n" + meta + "\n---\nQuelltext:\n" + buffer)
                    : ("\n\tFehler:\n\t\t" + requestResult.getErrorMsg()));
            System.out.println(retVal);
          }
        }
      }

      // m_protokolleUndAuswertungen.
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    System.out.println("Ende!!!");

  }
}
