/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.views;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Comparator;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.AnfrageThread;
import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.pua.basis.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.pua.basis.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.pua.basis.extension.Bindung;
import de.bsvrz.buv.plugin.pua.basis.extension.ExtensionVerwaltung;
import de.bsvrz.buv.plugin.pua.basis.extension.SkriptInfo;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;
import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStruktur;
import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStrukturZeitGruppe;
import de.bsvrz.buv.rw.basislib.objektzusammenstellung.ObjektSpeicher;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Nach Auswahl des Menpunkts "Protokolle und Auswertungen/Protokolle/..." in der Rahmenapplikation muss sich die View
 * zur Auswahl fr die Protokollierung ffnen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-8</li>
 * </ul>
 * 
 * @version $Revision: 1.13 $ / $Date: 2014/02/21 08:45:08 $ / ($Author: Sans $)
 * @author HofmannJ
 * 
 */
public class ProtokollierungView extends ViewPart implements IDaVDataUpdateListener
{

  public static final String ID = "de.bsvrz.buv.plugin.pua.protokollierungView";

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private Composite _parent;

  private Composite _top = null;

  private Composite compositeBottom;

  private Button buttonUebernehemen;

  private Button buttonAbbrechen;

  private Button buttonHilfe;

  protected String _programm;

  protected Combo combo;

  private ViewerFilter _searchFilter;

  private Integer _numberInstances = 0; // @jve:decl-index=0:

  private boolean _sperreFrNeueDaten = false;

  private Composite _composite;

  private KalenderZeitFilterDatenStruktur _DatenStruktur;

  private LinkedList<de.bsvrz.buv.plugin.pua.basis.views.ProtokollierungView.ZeitGruppenZeile> _listeZeitGruppenZeilen; // @jve:decl-index=0:

  private Composite _composite5;

  private Group _group0;

  private Button b1;

  private Button b2;

  private Button b3;

  private Button b31;

  private Group groupFilterDefinitionen11;

  private Label label0;

  private Label label1;

  private org.eclipse.swt.widgets.List list;

  private Composite composite1;

  private org.eclipse.swt.widgets.List list2;

  private Group groupFilterDefinitionen12;

  private ScrolledComposite _verschiebbaresCompositeZeitgruppen;

  private Composite _compositeZeitGruppenInhalt;

  private Composite composite2;

  private Button button;

  private Composite composite3;

  private Button button2;

  private Button button3;

  private Button button1;

  private Group _group111;

  private Button b1111;

  private Button b1112; 

  private Label l1111;
  
  private Text t1111;
  
  private Button _buttonAenderungsProtokoll;

  private Button b2111;

  private Group _group1111;

  private Label l111111;

  private Spinner spinner;
  
  private Label _labelBindungen;
  
  private Combo _comboBindungen;

  private TableViewer _tvVorhandeneObjekte;

  private TableViewer _tvAusgewaehlteObjekte;

  private Composite composite41 = null;

  private Composite composite42 = null;

  private ClientDavInterface _connection; // @jve:decl-index=0:

  private Composite composite43 = null;

  private Composite composite44 = null;

  private Text text = null;

  private Button button4 = null;

  private Button button5 = null;

  private Button button51;

  private Text _text;

  private Text _text2;

  private Composite composite45 = null;

  private SkriptInfo _skriptInfo = null;
  
  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".auswahl");

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _top = new Composite(_parent, SWT.NONE);
    _top.setLayoutData(gridData);
    _top.setLayout(new FillLayout());

    createCompositeTop();
    createCompositeBottom();
    
    String secondaryId = getViewSite().getSecondaryId();
    if (secondaryId != null) {
    	try {
    		_skriptInfo = ExtensionVerwaltung.getInstanz().getSkriptInfo(secondaryId);
    	}
    	catch (RuntimeException e) {
    		String msg = "Fr \"" + secondaryId + "\" konnte kein Skriptinfo bestimmt werden";
  	      	Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
  	      	System.err.println(msg);
  	      	e.printStackTrace(System.err);
  	      	return;
    	}
    	
    	b1.setSelection(_skriptInfo.isAktuell());
    	b2.setSelection(_skriptInfo.isNachgeliefert());
    	b3.setSelection(_skriptInfo.isNachgefordertAktuell());
    	b31.setSelection(_skriptInfo.isNachgefordertNachgeliefert());
    	
    	_buttonAenderungsProtokoll.setEnabled(_skriptInfo.isAenderungsProtokoll());
    	
    	List<Bindung> bindungen = _skriptInfo.getBindungen();
    	if (bindungen.size() > 0) {
    		Bindung bindung = bindungen.get(0);
    		int anzahl = bindung.getAnzahlBindungsAttribute(); 
    		if (anzahl > 0) {
    			Collection<String> auswahlNamen = bindung.getAuswahlNamen();
    			_comboBindungen.removeAll();
    			for (String text: auswahlNamen) {
    				_comboBindungen.add(text);
    			}
    			_comboBindungen.setText(auswahlNamen.iterator().next());
    			_comboBindungen.setToolTipText(_comboBindungen.getText());
    			_comboBindungen.setEnabled(anzahl > 1);
    			_comboBindungen.setData(bindung);
    		}
    	}
    	
        if (_connection != null)
        {
          DataModel dataModel = _connection.getDataModel();	
        	
          SystemObject soSkript = dataModel.getObject(_skriptInfo.getPid());
          if (soSkript == null) {
      		String msg = "Fr die Pid \"" + _skriptInfo.getPid() + "\" konnte kein Pua-Skript-Objekt gefunden werden";
  	      	Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg));
  	      	System.err.println(msg);
  	      	return;
          }
//          setPartName(getPartName() + " " + soSkript.getName());
          String partName = ExtensionVerwaltung.getInstanz().getAktionsText(_skriptInfo.getAktionsId());
          if (partName != null) {
        	  setPartName(partName);
          }
          
          Collection<SystemObject> elements;
          String pidSystemObjectType = _skriptInfo.getTyp();
          try {
        	  elements = _skriptInfo.getObjektProvider().getObjekte(pidSystemObjectType,
        			  _skriptInfo.isAusfilternDE255(), _skriptInfo.getUmsetungsModulFilterText());
          }
          catch (RuntimeException e) {
        	  String msg = "Fr den Typ \"" + pidSystemObjectType + "\" konnte keine Objektliste erzeugt werden";
    	      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
    	      System.err.println(msg);
    	      e.printStackTrace(System.err);
           	  return;
          }
          
          SortedMap<String, SystemObject> sortedMap = new TreeMap<String, SystemObject>();

          for (SystemObject so : elements)
          {
            if (so.getPid() != null && so.getPid().startsWith("swarco.pua")) {
              continue;
            }
            sortedMap.put(so.getPid(), so);
          }

          List<SystemObject> list = new ArrayList<SystemObject>();
          list.addAll(sortedMap.values());

          if (list != null)
            _tvVorhandeneObjekte.setInput(list);

          _text.setText(_tvVorhandeneObjekte.getTable().getSelectionCount() + " / "
              + _tvVorhandeneObjekte.getTable().getItemCount());
          composite44.layout(true);
        }
    }

    	

//      else if (_pidPuaSkript.equals(PuaSkript.TlsGloDeFehler.info.getPid()))
////        else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsGloDeFehler1"))
//      {
//
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.de").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          int deKanal = cd.getItem("DEKanal").asUnscaledValue().intValue();
//
//          if (deKanal != 255)
//          {
//            elements.add(so);
//          }
//        }
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsLveErgaenzendeDeFehlerMeldung.info.getPid())
//          || _pidPuaSkript.equals(PuaSkript.TlsLveErgebnisMeldungVersion0Bis4.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsLveErgaenzendeDeFehlerMeldung1")
////          || _pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsLveErgebnisMeldungVersion0Bis4"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deLve").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          int deKanal = cd.getItem("DEKanal").asUnscaledValue().intValue();
//
//          if (deKanal != 255)
//          {
//            elements.add(so);
//          }
//        }
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdLuftTemperatur.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungLuftTemperaturLT"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//          
//          if (modul.contains("Fg003Typ048"))
//          {
//            elements.add(so);
//          }
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdFahrbahnTemperatur.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungFahrbahnOberFlaechenTemperaturFBT"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg003Typ049"))
//          {
//            elements.add(so);
//          }
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdRestSalz.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungRestSalzRS"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg003Typ052"))
//          {
//            elements.add(so);
//          }
//
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdLuftFeuchte.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungRelativeLuftFeuchteRLF"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg003Typ055"))
//          {
//            elements.add(so);
//          }
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdHelligkeit.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungHelligkeitHK"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg003Typ061"))
//          {
//            elements.add(so);
//          }                    
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdGefrierTemperatur.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungGefrierTemperaturGT"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg003Typ065"))
//          {
//            elements.add(so);
//          }
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdTaupunktTemperatur.info.getPid()))
////      else if (_pidPuaSkript.equals("ua.skript.nw.atgTlsUfdErgebnisMeldungTaupunktTemperaturTPT"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg003Typ066"))
//          {
//            elements.add(so);
//          }
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdFahrbahnZustand.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungZustandDerFahrbahnOberFlaecheFBZ"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg003Typ070"))
//          {
//            elements.add(so);
//          }
//        
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdNiederschlagsArt.info.getPid()))
////        else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungNiederschlagsArtNS"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg003Typ071"))
//          {
//            elements.add(so);
//          }
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdNiederschlagsIntensitaet.info.getPid()))
////        else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungNiederschlagsArtNS"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//          
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//          
//          Data cd = co.getConfigurationData(atg);
//          
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//          
//          if (modul.contains("Fg003Typ053"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdTemperaturTiefe1TT1.info.getPid()))
////        else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungNiederschlagsArtNS"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//          
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//          
//          Data cd = co.getConfigurationData(atg);
//          
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//          
//          if (modul.contains("Fg003Typ067"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdTemperaturTiefe3TT3.info.getPid()))
////        else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungNiederschlagsArtNS"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//          
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//          
//          Data cd = co.getConfigurationData(atg);
//          
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//          
//          if (modul.contains("Fg003Typ069"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdSichtWeite.info.getPid()))
////        else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungNiederschlagsArtNS"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//          
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//          
//          Data cd = co.getConfigurationData(atg);
//          
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//          
//          if (modul.contains("Fg003Typ060"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsUfdWasserfilmDicke.info.getPid()))
////        else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsUfdErgebnisMeldungWasserFilmDickeWFD"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deUfd").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg003Typ072"))
//          {
//            elements.add(so);
//          }
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsVltHeizung.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsVltTuerKontakt"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deVlt").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//          
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//          
//          Data cd = co.getConfigurationData(atg);
//          
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//          
//          if (modul.contains("Fg006Typ052"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsVltLicht.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsVltTuerKontakt"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deVlt").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//          
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//          
//          Data cd = co.getConfigurationData(atg);
//          
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//          
//          if (modul.contains("Fg006Typ050"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsVltTuerKontakt.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsVltTuerKontakt"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deVlt").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg006Typ048"))
//          {
//            elements.add(so);
//          }
//        }
//
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsVltTemperaturUeberwachung.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsVltTuerKontakt"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deVlt").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//          
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//          
//          Data cd = co.getConfigurationData(atg);
//          
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//          
//          if (modul.contains("Fg006Typ049"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsVltStromVersorgung.info.getPid()))
////        else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsVltStromVersorgung"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deVlt").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg006Typ051"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsVltLueftung.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsVltLueftung"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deVlt").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg006Typ053"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsVltUeberspannungsSchutz.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsVltUeberspannungsSchutz"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deVlt").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg006Typ054"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.TlsVltDiebstahlVandalismusSchutz.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsVltDiebstahlVandalismusSchutz"))
//      {
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deVlt").getElements();
//        
//        elements = new ArrayList<SystemObject>();
//        
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          String modul = cd.getItem("UmsetzungsModul").asTextValue().getValueText();
//
//          if (modul.contains("Fg006Typ055"))
//          {
//            elements.add(so);
//          }
//        }
//        
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.WVZDefekteLampen.info.getPid()) 
//          || _pidPuaSkript.equals(PuaSkript.WVZNichtDarstellbareWVZ.info.getPid()))
////      else if (_pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsWzgDefekteLampen") 
////          || _pidPuaSkript.equals("pua.skript.st.uz.a14.atgTlsWzgNichtDarstellbareWVZ"))
//      {
//
//        List<SystemObject> temp = _connection.getDataModel().getType("typ.deWzg").getElements();
//
//        elements = new ArrayList<SystemObject>();
//
//        for (SystemObject so : temp)
//        {
//          ConfigurationObject co = (ConfigurationObject)so;
//
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");
//
//          Data cd = co.getConfigurationData(atg);
//
//          int deKanal = cd.getItem("DEKanal").asUnscaledValue().intValue();
//
//          if (deKanal < 193)
//          {
//            elements.add(so);
//          }
//        }
//      }
//      else if (_pidPuaSkript.equals(PuaSkript.UEEinzelsituationVorstufe.info.getPid()))
//      {
//        elements = _connection.getDataModel().getType("typ.sswSbaUrsachenEinheit").getElements();
//      }
//
//      SortedMap<String, SystemObject> sortedMap = new TreeMap<String, SystemObject>();
//
//      for (SystemObject so : elements)
//      {
//        sortedMap.put(so.getPid(), so);
//      }
//
//      List<SystemObject> list = new ArrayList<SystemObject>();
//      list.addAll(sortedMap.values());
//
//      if (list != null)
//        _tvBetriebsMeldungen1.setInput(list);
//
//      _text.setText(_tvBetriebsMeldungen1.getTable().getSelectionCount() + " / "
//          + _tvBetriebsMeldungen1.getTable().getItemCount());
//      composite44.layout(true);
//
//    }
//    else
//      System.err.println("Connection ist null");

  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }

  /**
   * This method initializes compositeBottom
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(_composite, SWT.NONE);
    compositeBottom.setLayoutData(gridData3);
    compositeBottom.setLayout(rowLayout);

    buttonUebernehemen = new Button(compositeBottom, SWT.NONE);
    buttonUebernehemen.setText("Anfordern");
    buttonUebernehemen.setEnabled(_tvAusgewaehlteObjekte.getTable().getItemCount() > 0);
    buttonUebernehemen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        // TODO Auto-generated method stub
        super.mouseDown(e);
        
        int index = 0;
        Iterator<ZeitGruppenZeile> it = _listeZeitGruppenZeilen.iterator();
        while (it.hasNext()) {
        	index++;
        	ZeitGruppenZeile zeile = it.next();
        	Date von = zeile._dzVon.getDatum();
        	Date bis = zeile._dzBis.getDatum();
        	
        	if (von == null) {
        		MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
                mb.setText("Fehler Datum/Zeit");
                if (_listeZeitGruppenZeilen.size() > 1) {
                	mb.setMessage("Fr den " + index + ". Zeitbereich fehlt der 'von'-Zeitpunkt");
                }
                else {
                	mb.setMessage("Es fehlt der 'von'-Zeitpunkt");	
                }
                mb.open();
                return;
        	}
        	else if (bis == null) {
        		MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
                mb.setText("Fehler Datum/Zeit");
                if (_listeZeitGruppenZeilen.size() > 1) {
                	mb.setMessage("Fr den " + index + ". Zeitbereich fehlt der 'bis'-Zeitpunkt");
                }
                else {
                	mb.setMessage("Es fehlt der 'bis'-Zeitpunkt");	
                }
                mb.open();
                return;
        	}
        	else if (von.after(bis)) {
        		MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
                mb.setText("Fehler Datum/Zeit");
                if (_listeZeitGruppenZeilen.size() > 1) {
                	mb.setMessage("Fr den " + index + ". Zeitbereich liegt der 'von'-Zeitpunkt nach dem 'bis'-Zeitpunkt");
                }
                else {
                	mb.setMessage("Der 'von'-Zeitpunkt liegt nach dem 'bis'-Zeitpunkt");	
                }
                mb.open();
                return;
        	}
        }
        
        IStructuredSelection selection = (IStructuredSelection)_tvAusgewaehlteObjekte.getSelection();

        Object input = _tvAusgewaehlteObjekte.getInput();

        if (input instanceof ArrayList)
        {
          List list = (ArrayList)input;
          if (list != null && list.size() > 0)
          {
            aktionAnfordern();
          }
          else
          {
            MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
            mb.setText("Fehler Auswahl Objekte");
            mb.setMessage("Es mssen Objekte in der Tabelle vorhandene Objekte bernommen werden");
            mb.open();

          }
        }
        else
        {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
          mb.setText("Fehler Auswahl Objekte");
          mb.setMessage("Es mssen Objekte in der Tabelle vorhandene Objekte bernommen werden");
          mb.open();

        }

      }
    });

    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        // TODO Auto-generated method stub
        super.mouseDown(e);
        dispose();
      }

    });
    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        aktionHilfe();
      }

    });
  }

  /**
   * Zeigt die Hilfe zur View an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".auswahl";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  // /**
  // * Fhrt das Schalten durch.
  // *
  // */
  // protected void aktionSchalten()
  // {
  // if (schaltenUndParametrieren())
  // {
  // dispose();
  // }
  // else
  // {
  // MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
  // mb.setText("Fehler Urlasser");
  // mb.setMessage("Objekt konnte nicht parametriert werden: " + "");
  // mb.open();
  // return;
  // }
  //
  // }

  // protected void aktionAnfordern()
  // {
  // IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
  // IWorkbenchPage page = dw.getActivePage();
  // try
  // {
  // ArchiveQueryPriority archiveQueryPriority = ArchiveQueryPriority.HIGH;
  // TimingType timingType = TimingType.DATA_TIME;
  // boolean startRelative = false;
  // ArchiveDataKindCombination archiveDataKindCombination = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE);
  // ArchiveOrder archiveOrder = ArchiveOrder.BY_INDEX;
  // ArchiveRequestOption archiveRequestOption = ArchiveRequestOption.NORMAL;
  //
  // List<SystemObject> objekte = (List<SystemObject>)_tvBetriebsMeldungen2.getInput();
  //
  // if (objekte != null && objekte.size() > 0)
  // {
  //
  // AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.betriebsMeldung");
  // Aspect asp = _connection.getDataModel().getAspect("asp.information");
  //
  // DataDescription dd = new DataDescription(atg, asp);
  //
  // LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe> zeitgruppen = new
  // LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe>();
  //
  // Date d1 = new Date();
  // Date d2 = new Date();
  // SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
  //
  // for (ZeitGruppenZeile zgz : _listeZeitGruppenZeilen)
  // {
  // KalenderZeitFilterDatenStrukturZeitGruppe dszg = zgz.getDatenStrukturZeitGruppe();
  //
  // d1.setTime(dszg.getStartzeitpunkt());
  // System.out.println("...from1 :" + sdf.format(d1));
  //
  // d1.setTime(dszg.getEndzeitpunkt());
  // System.out.println("...to1 :" + sdf.format(d2));
  //
  // zeitgruppen.add(dszg);
  // }
  //
  // AnfrageDaten anfrageDaten;
  // try
  // {
  // anfrageDaten = new AnfrageDaten(objekte, zeitgruppen, dd, archiveQueryPriority, timingType, startRelative,
  // archiveDataKindCombination, archiveOrder, archiveRequestOption);
  //
  // page.showView(MeldungsanzeigeSicht.ID, _numberInstances.toString(), IWorkbenchPage.VIEW_VISIBLE);
  //
  // IViewReference ref = page.findViewReference(MeldungsanzeigeSicht.ID, _numberInstances.toString());
  // MeldungsanzeigeSicht view = (MeldungsanzeigeSicht)ref.getView(true);
  // view.setAnfrageDaten(anfrageDaten);
  //
  // _numberInstances++;
  //
  // new ArchiveRequest().startRequest(anfrageDaten);
  // }
  // catch (IllegalArgumentException ie)
  // {
  // ie.printStackTrace();
  // }
  // }
  // else
  // {
  //
  // System.out.println("hier");
  //
  // MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
  // mb.setText("Fehler Anforderung");
  //
  // String s = "Es muss ein Eintrag in der Tabelle \"vorhandene Objekte\"\n";
  // s += "vorhanden sein!";
  //
  // mb.setMessage(s);
  // mb.open();
  // return;
  // }
  //
  // }
  // catch (PartInitException e)
  // {
  // e.printStackTrace();
  // }
  //
  // }
  //
  // private Boolean schaltenUndParametrieren()
  // {
  // return false;
  // }

  /*
   * (non-Javadoc)
   * 
   * @see de.bsvrz.buv.plugin.knotenbeeinflussung.daVDataUpdateListener.IDaVDataUpdateListener#daVDataUpdate(de.bsvrz.buv.plugin.knotenbeeinflussung.daVDataUpdateListener.DaVDataUpdateEvent)
   */
  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    if (e != null)
    {
      if (!_sperreFrNeueDaten)
      {
        // verarbeiteDavDaten(e);
      }
      else
      {
        // System.out.println("Neue Daten -> kein Update, da Sperre fr neue Daten aktiv");
      }
    }
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void createCompositeTop()
  {

    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    _composite = new Composite(_top, SWT.NONE);
    _composite.setLayout(gridLayout);

    createGroupAuswahlObjekte();

    _DatenStruktur = new KalenderZeitFilterDatenStruktur();
    _listeZeitGruppenZeilen = new LinkedList<ZeitGruppenZeile>();

    createGroupZeitgruppen();

    createGroupArchivanfrage();

    // createGroupZeitgruppenTest();

    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    if (_connection != null)
    {
      _text.setText(_tvVorhandeneObjekte.getTable().getSelectionCount() + " / "
          + _tvVorhandeneObjekte.getTable().getItemCount());
      _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
          + _tvAusgewaehlteObjekte.getTable().getItemCount());
      composite43.layout(true);
      composite44.layout(true);

    }

  }

  /**
   * This method initializes groupFilterDefinitionen11
   * 
   */
  private void createGroupArchivanfrage()
  {

    GridData gridData23 = new GridData();
    gridData23.horizontalAlignment = GridData.FILL;
    gridData23.grabExcessHorizontalSpace = true;
    gridData23.grabExcessVerticalSpace = true;
    gridData23.verticalAlignment = GridData.FILL;
    gridData23.horizontalSpan = 2;
    GridLayout gridLayout3 = new GridLayout();
    gridLayout3.numColumns = 7;
    gridLayout3.marginWidth = 0;
    gridLayout3.marginHeight = 0;
    gridLayout3.horizontalSpacing = 5;
    gridLayout3.makeColumnsEqualWidth = true;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 1;
    gridLayout1.marginWidth = 5;
    gridLayout1.verticalSpacing = 5;
    gridLayout1.makeColumnsEqualWidth = false;
    gridLayout1.horizontalSpacing = 5;
    gridLayout1.marginHeight = 5;
    _composite5 = new Composite(_composite, SWT.NONE);
    _composite5.setLayoutData(gridData);
    _composite5.setLayout(gridLayout3);
    _group0 = new Group(_composite5, SWT.NONE);
    _group0.setText("Art der Archivanfrage");
    _group0.setLayoutData(gridData23);
    _group0.setLayout(gridLayout1);
    b1 = new Button(_group0, SWT.CHECK);
    b1.setText("aktuelle Daten");
    b1.setSelection(true);
    b2 = new Button(_group0, SWT.CHECK);
    b2.setText("nachgelieferte Daten");
    b3 = new Button(_group0, SWT.CHECK);
    b3.setText("nachgefordert-aktuelle Daten");
    b31 = new Button(_group0, SWT.CHECK);
    b31.setText("nachgefordert-nachgelieferte Daten");
    create_group111();
    create_group1111();
  }

  /**
   * This method initializes groupFilterDefinitionen11
   * 
   */
  private void createGroupAuswahlObjekte()
  {
    RowLayout rowLayout3 = new RowLayout();
    rowLayout3.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout3.justify = true;
    rowLayout3.wrap = false;
    rowLayout3.pack = false;
    rowLayout3.spacing = 25;
    rowLayout3.fill = false;
    GridData gridData22 = new GridData();
    gridData22.horizontalAlignment = GridData.CENTER;
    gridData22.grabExcessVerticalSpace = true;
    gridData22.grabExcessHorizontalSpace = false;
    gridData22.verticalAlignment = GridData.FILL;
    GridData gridData10 = new GridData();
    gridData10.horizontalSpan = 2;
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.END;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.verticalAlignment = GridData.CENTER;
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.BEGINNING;
    gridData4.verticalAlignment = GridData.CENTER;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = false;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.BEGINNING;
    gridData11.verticalAlignment = GridData.CENTER;
    GridData gridData211 = new GridData();
    gridData211.horizontalAlignment = GridData.BEGINNING;
    gridData211.verticalAlignment = GridData.CENTER;
    groupFilterDefinitionen11 = new Group(_composite, SWT.NONE);
    groupFilterDefinitionen11.setText("Auswahl Objekte");
    groupFilterDefinitionen11.setLayout(new FillLayout());
    groupFilterDefinitionen11.setLayoutData(gridData2);

    GridData gridData41 = new GridData();
    gridData41.horizontalAlignment = GridData.FILL;
    gridData41.grabExcessHorizontalSpace = true;
    gridData41.grabExcessVerticalSpace = false;
    gridData41.verticalAlignment = GridData.FILL;
    gridData41.heightHint = 100;

    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = false;
    gridData3.verticalAlignment = GridData.FILL;
    gridData3.heightHint = 100;

    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.CENTER;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.CENTER;
    gridData12.grabExcessHorizontalSpace = false;
    gridData12.verticalAlignment = GridData.CENTER;

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 4;
    gridLayout.marginHeight = 5;
    gridLayout.verticalSpacing = 5;
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.marginWidth = 5;
    _top = new Composite(groupFilterDefinitionen11, SWT.NONE);
    label0 = new Label(_top, SWT.NONE);
    label0.setText("mgliche Objekte");
    label0.setLayoutData(gridData12);
    _top.setLayout(gridLayout);
    Label filler4 = new Label(_top, SWT.NONE);
    label1 = new Label(_top, SWT.NONE);
    label1.setText("vorhandene Objekte");
    label1.setLayoutData(gridData21);
    Label filler3 = new Label(_top, SWT.NONE);
    createComposite44();
    Label filler1 = new Label(_top, SWT.NONE);
    createComposite43();
    Label filler2 = new Label(_top, SWT.NONE);
    createComposite41();
    erzeugeTabelle1();

    composite1 = new Composite(_top, SWT.NONE);
    composite1.setLayoutData(gridData22);
    createComposite2();
    composite1.setLayout(rowLayout3);
    createComposite3();

    createComposite42();
    erzeugeTabelle2();
  }

  private class MyComparatorNeu implements Comparator<String>
  {

    @Override
    public int compare(String o1, String o2)
    {
      Integer i1 = new Integer(o1);
      Integer i2 = new Integer(o2);

      int ret = 0;

      if (i1 < i2)
        ret = -1;
      else if (i1 == i2)
        ret = 0;
      else if (i1 > i2)
        ret = 1;
      return ret;
    }

  }

  /**
   * Liest die Datenstruktur ein
   * 
   */
  private void leseDatenstruktur()
  {
    // Zeitgruppen setzen
    List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitGruppen = _DatenStruktur.getListeZeitGruppen();
    // Wenn die ZeitGruppenListe nicht leer ist, dann auslesen und darstellen

    if (zeitGruppen.size() > 0)
    {
      Iterator<KalenderZeitFilterDatenStrukturZeitGruppe> it = zeitGruppen.iterator();
      while (it.hasNext())
      {
        KalenderZeitFilterDatenStrukturZeitGruppe zg = it.next();
        generiereZeitgruppeMitVorgabe(zg);
      }
    }
    else
    // Sonst nur eine Leerzeile darstellen.
    {
      // System.out.println("hier!!!");
      Date d = new Date();
      generiereZeitgruppeMitVorgabe(d.getTime(), d.getTime() + 1);

      // generiereZeitgruppeLeer();
    }

  }

  /**
   * This method initializes groupFilterDefinitionen12
   * 
   */
  private void createGroupZeitgruppen()
  {
    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    groupFilterDefinitionen12 = new Group(_composite, SWT.NONE);
    groupFilterDefinitionen12.setText("Zeitbereiche");
    groupFilterDefinitionen12.setLayout(fillLayout);
    groupFilterDefinitionen12.setLayoutData(gridData2);

    _verschiebbaresCompositeZeitgruppen = new ScrolledComposite(groupFilterDefinitionen12, SWT.H_SCROLL | SWT.V_SCROLL);
    _verschiebbaresCompositeZeitgruppen.getVerticalBar().setIncrement(20);

    _compositeZeitGruppenInhalt = new Composite(_verschiebbaresCompositeZeitgruppen, SWT.NONE);
    _compositeZeitGruppenInhalt.setLayout(new FillLayout());
    GridLayout glGroupZeitgruppen = new GridLayout();
    glGroupZeitgruppen.numColumns = 1;
    glGroupZeitgruppen.makeColumnsEqualWidth = false;

    _verschiebbaresCompositeZeitgruppen.setContent(_compositeZeitGruppenInhalt);
    _verschiebbaresCompositeZeitgruppen.setMinSize(500, 1000);
    _verschiebbaresCompositeZeitgruppen.setExpandHorizontal(true);
    _verschiebbaresCompositeZeitgruppen.setExpandVertical(true);

    _compositeZeitGruppenInhalt.setLayout(glGroupZeitgruppen);

    leseDatenstruktur();

  }

  private void generiereZeitgruppeMitVorgabe(long lStartZeitPunkt, long lEndZeitPunkt)
  {
    generiereZeitgruppeLeer();

    _listeZeitGruppenZeilen.get(0).setDatenStrukturZeitGruppe(
        new KalenderZeitFilterDatenStrukturZeitGruppe(lStartZeitPunkt, lEndZeitPunkt));

  }

  /**
   * Generiert eine Zeitgruppe mit Zeitvorgabe in Form einer Instanz der Klasse
   * KalenderZeitFilterDatensStrukturZeitGruppe
   * 
   * @param zg
   *          Vorgabe von Start- und Endzeitpunkt
   */
  private void generiereZeitgruppeMitVorgabe(KalenderZeitFilterDatenStrukturZeitGruppe zg)
  {
    _DatenStruktur.fuegeZeitGruppeHinzu(zg);
    ZeitGruppenZeile zgz = new ZeitGruppenZeile(_compositeZeitGruppenInhalt, zg);
    zgz.generiereComposite();
    _listeZeitGruppenZeilen.add(zgz);
    _compositeZeitGruppenInhalt.layout();
  }

  public org.eclipse.swt.widgets.List getList()
  {
    return list;
  }

  public org.eclipse.swt.widgets.List getList2()
  {
    return list2;
  }

  /**
   * This method initializes composite2
   * 
   */
  private void createComposite2()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    composite2 = new Composite(composite1, SWT.NONE);
    composite2.setLayout(rowLayout);
    button = new Button(composite2, SWT.NONE);
    button.setText(">");
    button.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
    	 // _tvVorhandeneObjekte.refresh();
        IStructuredSelection selection = (IStructuredSelection)_tvVorhandeneObjekte.getSelection();
        
        List<SystemObject> x = new ArrayList<SystemObject>();
        Object firstElement = selection.getFirstElement();
        if (firstElement instanceof SystemObject)
        {
          SystemObject fe = (SystemObject)firstElement;
          System.out.println("firstElement: " + fe.getPid());

          Iterator it = selection.iterator();
          while (it.hasNext())
          {
            SystemObject so = (SystemObject)it.next();
            x.add(so);
          }
        }
        else
        {
          // System.out.println(firstElement.getClass().getSimpleName());
        }

        List<SystemObject> y = new ArrayList<SystemObject>();
        List<SystemObject> input1 = (List<SystemObject>)_tvVorhandeneObjekte.getInput();
               
        if (input1 != null)
        {
          y.addAll(input1);
          y.removeAll(x);
        }
        _tvVorhandeneObjekte.getTable().clearAll();

        SortedMap<String, SystemObject> sortedMap = new TreeMap<String, SystemObject>();

        for (SystemObject so : y)
        {
          sortedMap.put(so.getPid(), so);
        }

        List<SystemObject> list = new ArrayList<SystemObject>();
        list.addAll(sortedMap.values());

        _tvVorhandeneObjekte.setInput(list);

        List<SystemObject> input = (List<SystemObject>)_tvAusgewaehlteObjekte.getInput();
        if (input != null)
        {
          x.addAll(input);
        }
        _tvAusgewaehlteObjekte.getTable().clearAll();

        SortedMap<String, SystemObject> sortedMap2 = new TreeMap<String, SystemObject>();

        for (SystemObject so : x)
        {
          sortedMap2.put(so.getPid(), so);
        }

        List<SystemObject> list2 = new ArrayList<SystemObject>();
        list2.addAll(sortedMap2.values());

        _tvAusgewaehlteObjekte.setInput(list2);
        _text.setText(_tvVorhandeneObjekte.getTable().getSelectionCount() + " / "
            + _tvVorhandeneObjekte.getTable().getItemCount());
        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        buttonUebernehemen.setEnabled(_tvAusgewaehlteObjekte.getTable().getItemCount() > 0);
        composite43.layout(true);
        composite44.layout(true);

      }

    });
    button1 = new Button(composite2, SWT.NONE);
    button1.setText(">>");
    button1.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {

        List<SystemObject> x = new ArrayList<SystemObject>();

        List<SystemObject> y = new ArrayList<SystemObject>();
        y.addAll((List<SystemObject>)_tvVorhandeneObjekte.getInput());

        List<SystemObject> input1 = new ArrayList<SystemObject>();

        if (_tvVorhandeneObjekte.getFilters().length > 0)
        {
          TableItem[] items = _tvVorhandeneObjekte.getTable().getItems();
          for (TableItem ti : items)
          {
            SystemObject so = (SystemObject)ti.getData();
            input1.add(so);
          }
        }
        else
          input1 = (List<SystemObject>)_tvVorhandeneObjekte.getInput();

        List<SystemObject> input2 = (List<SystemObject>)_tvAusgewaehlteObjekte.getInput();
        if (input1 != null)
        {
          x.addAll(input1);
        }
        if (input2 != null)
        {
          x.addAll(input2);
        }

        if (_tvVorhandeneObjekte.getFilters().length > 0)
        {

          y.removeAll(input1);

          SortedMap<String, SystemObject> sortedMap = new TreeMap<String, SystemObject>();

          for (SystemObject so : y)
          {
            sortedMap.put(so.getPid(), so);
          }

          List<SystemObject> list = new ArrayList<SystemObject>();
          list.addAll(sortedMap.values());

          _tvVorhandeneObjekte.setInput(list);

        }
        else
          _tvVorhandeneObjekte.setInput(new ArrayList<SystemObject>());

        SortedMap<String, SystemObject> sortedMap = new TreeMap<String, SystemObject>();

        for (SystemObject so : x)
        {
          sortedMap.put(so.getPid(), so);
        }

        List<SystemObject> list = new ArrayList<SystemObject>();
        list.addAll(sortedMap.values());

        _tvAusgewaehlteObjekte.setInput(list);

        _text.setText(_tvVorhandeneObjekte.getTable().getSelectionCount() + " / "
            + _tvVorhandeneObjekte.getTable().getItemCount());
        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        buttonUebernehemen.setEnabled(_tvAusgewaehlteObjekte.getTable().getItemCount() > 0);
        composite43.layout(true);
        composite44.layout(true);

      }

    });

  }

  /**
   * This method initializes composite3
   * 
   */
  private void createComposite3()
  {
    RowLayout rowLayout1 = new RowLayout();
    rowLayout1.pack = false;
    composite3 = new Composite(composite1, SWT.NONE);
    composite3.setLayout(rowLayout1);
    button2 = new Button(composite3, SWT.NONE);
    button2.setText("<<");
    button2.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {

        List<SystemObject> x = new ArrayList<SystemObject>();
        List<SystemObject> input1 = (List<SystemObject>)_tvVorhandeneObjekte.getInput();
        List<SystemObject> input2 = (List<SystemObject>)_tvAusgewaehlteObjekte.getInput();
        if (input1 != null)
        {
          x.addAll(input1);
        }
        if (input2 != null)
        {
          x.addAll(input2);
        }

        SortedMap<String, SystemObject> sortedMap = new TreeMap<String, SystemObject>();

        for (SystemObject so : x)
        {
          sortedMap.put(so.getPid(), so);
        }

        List<SystemObject> list = new ArrayList<SystemObject>();
        list.addAll(sortedMap.values());

        _tvAusgewaehlteObjekte.setInput(new ArrayList<SystemObject>());
        _tvVorhandeneObjekte.setInput(list);

        _text.setText(_tvVorhandeneObjekte.getTable().getSelectionCount() + " / "
            + _tvVorhandeneObjekte.getTable().getItemCount());
        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        buttonUebernehemen.setEnabled(_tvAusgewaehlteObjekte.getTable().getItemCount() > 0);
        composite43.layout(true);
        composite44.layout(true);

      }

    });
    button3 = new Button(composite3, SWT.NONE);
    button3.setText("<");
    button3.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        IStructuredSelection selection = (IStructuredSelection)_tvAusgewaehlteObjekte.getSelection();
        List<SystemObject> x = new ArrayList<SystemObject>();
        Object firstElement = selection.getFirstElement();
        if (firstElement instanceof SystemObject)
        {
          SystemObject fe = (SystemObject)firstElement;

          Iterator it = selection.iterator();
          while (it.hasNext())
          {
            SystemObject so = (SystemObject)it.next();
            x.add(so);
          }
        }
        else
        {
          // System.out.println(firstElement.getClass().getSimpleName());
        }

        List<SystemObject> y = new ArrayList<SystemObject>();
        List<SystemObject> input1 = (List<SystemObject>)_tvAusgewaehlteObjekte.getInput();
        if (input1 != null)
        {
          y.addAll(input1);
          y.removeAll(x);
        }
        _tvAusgewaehlteObjekte.getTable().clearAll();
        _tvAusgewaehlteObjekte.setInput(y);

        List<SystemObject> input = (List<SystemObject>)_tvVorhandeneObjekte.getInput();

        if (input != null)
        {
          x.addAll(input);

        }

        SortedMap<String, SystemObject> sortedMap = new TreeMap<String, SystemObject>();

        for (SystemObject so : x)
        {
          sortedMap.put(so.getPid(), so);
        }

        List<SystemObject> list = new ArrayList<SystemObject>();
        list.addAll(sortedMap.values());

        _tvVorhandeneObjekte.getTable().clearAll();
        _tvVorhandeneObjekte.setInput(list);

        _text.setText(_tvVorhandeneObjekte.getTable().getSelectionCount() + " / "
            + _tvVorhandeneObjekte.getTable().getItemCount());
        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        buttonUebernehemen.setEnabled(_tvAusgewaehlteObjekte.getTable().getItemCount() > 0);
        composite43.layout(true);
        composite44.layout(true);

      }

    });
  }

  /**
   * Bildet die Struktur einer ZeitgruppenZeile
   * 
   * @author HofmannJ
   * 
   */
  public class ZeitGruppenZeile
  {
    private Composite _parent = null;

    private KalenderZeitFilterDatenStrukturZeitGruppe _daten = null;

    private Label _labelVon;

    private Label _labelBis;

    private Label _labelInfo;

    private Button _schaltflaecheHinzufuegen = null;

    private Button _schaltflaecheLoeschen = null;

    /**
     * Diese Composite enthlt die Darstellung eines Zeitgruppen-Eintrags
     */
    private Composite _returnComposite;

    public DatumZeit _dzVon;

    public DatumZeit _dzBis;

    /**
     * Konstruktor
     * 
     * @param parent
     *          Eltern-Composite, in dass die Zeitgruppenzeile eingeblendet werden soll.
     * @param Daten
     *          Von-Bis.
     */
    public ZeitGruppenZeile(Composite parent, KalenderZeitFilterDatenStrukturZeitGruppe Daten)
    {
      this._parent = parent;
      this._daten = Daten;
    }

    /**
     * Sichert die erfassten Zeiten in daten
     * 
     */
    public void sichereErfassteZeiten()
    {
      _daten.setStartzeitpunkt(_dzVon.getDatum().getTime());
      _daten.setEndzeitpunkt(_dzBis.getDatum().getTime());

    }

    /**
     * Liefert die {@link KalenderZeitFilterDatenStrukturZeitGruppe} dieser {@link ZeitGruppenZeile}.
     * 
     * @return {@link KalenderZeitFilterDatenStrukturZeitGruppe}
     */
    public KalenderZeitFilterDatenStrukturZeitGruppe getDatenStrukturZeitGruppe()
    {
      return this._daten;
    }

    /**
     * Setzt die {@link KalenderZeitFilterDatenStrukturZeitGruppe} dieser {@link ZeitGruppenZeile}
     * 
     * @param datenStrukturZeitGruppe
     *          {@link KalenderZeitFilterDatenStrukturZeitGruppe}
     */
    public void setDatenStrukturZeitGruppe(KalenderZeitFilterDatenStrukturZeitGruppe datenStrukturZeitGruppe)
    {
      this._daten = datenStrukturZeitGruppe;
      _dzVon.setDatum(new Date(_daten.getStartzeitpunkt()));
      _dzBis.setDatum(new Date(_daten.getEndzeitpunkt()));
    }

    /**
     * Liefert true zurck, wenn der Startzeitpunkt VOR dem Endzeitpunkt liegt. Andernfalls wird false geliefert.
     * 
     * @return true, wenn der Startzeitpunkt VOR dem Endzeitpunkt liegt.
     */
    private boolean pruefePlausibilitaet()
    {
      boolean returnWert = true;
      if (_daten.getStartzeitpunkt() > _daten.getEndzeitpunkt())
      {
        returnWert = false;
      }

      return returnWert;
    }

    /**
     * Dispose-Methode fr das {@link Composite} dieser {@link ZeitGruppenZeile}
     */
    public void dispose()
    {
      _returnComposite.dispose();
    }

    /**
     * Lscht diese {@link ZeitGruppenZeile}.
     */
    private void aktionZeitgruppenzeileLoeschen()
    {
      loescheZeitGruppenZeile(this);
    }

    /**
     * Hngt eine Zeitgruppenzeile an
     */
    private void aktionZeitgruppenzeileAnhaengen()
    {
      // generiereZeitgruppeLeer();
      einfuegenZeitGruppenZeileNachPosition(this);
    }

    /**
     * Generiert das Composuite
     * 
     * @return Ein Composite
     */
    public Composite generiereComposite()
    {
      _returnComposite = new Composite(_parent, SWT.BORDER);

      RowLayout rl = new RowLayout(SWT.HORIZONTAL);
      rl.pack = false;
      _returnComposite.setLayout(rl);

      _labelVon = new Label(_returnComposite, SWT.NONE);
      _labelVon.setText("von: ");

      _dzVon = new DatumZeit(_returnComposite, SWT.NONE, DatumZeit.Eingabetyp.datumuhr, false, false);
      _dzVon.setToolTipText("Datum + Zeit, mit Auswhler.");

      _dzVon.setDatum(new Date(_daten.getStartzeitpunkt()));
      _dzVon.setLayout(new GridLayout());

      _labelBis = new Label(_returnComposite, SWT.NONE);
      _labelBis.setText("bis: ");

      _dzBis = new DatumZeit(_returnComposite, SWT.NONE, DatumZeit.Eingabetyp.datumuhr, false, false);
      _dzBis.setToolTipText("Datum + Zeit, mit Auswhler.");

      _dzBis.setDatum(new Date(_daten.getEndzeitpunkt()));
      _dzBis.setLayout(new GridLayout());

      _labelInfo = new Label(_returnComposite, SWT.NONE);

      _dzVon.hinzufuegenSelektionsZuhoerer(new SelectionListener() {

        private long _startzeitpunkt = -1;

        public void widgetDefaultSelected(SelectionEvent e)
        {
          gemeinsam(e);
        }

        public void widgetSelected(SelectionEvent e)
        {
          gemeinsam(e);
        }

        private void gemeinsam(SelectionEvent e)
        {
          if (e.data instanceof Date)
          {
            Date datum = (Date)e.data;
            _startzeitpunkt = datum.getTime();
            _daten.setStartzeitpunkt(_startzeitpunkt);

            if (!pruefePlausibilitaet())
            {
              Color color = new Color(Display.getDefault(), 255, 0, 0);
              _labelInfo.setForeground(color);
              _labelInfo.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
              _compositeZeitGruppenInhalt.layout();
            }
            else
            {
              _labelInfo.setText("");
              _compositeZeitGruppenInhalt.layout();
            }

            // fireEvent();
          }
        }

      });

      _dzBis.hinzufuegenSelektionsZuhoerer(new SelectionListener() {

        private long _endzeitpunkt = -1;

        public void widgetDefaultSelected(SelectionEvent e)
        {
          gemeinsam(e);
        }

        public void widgetSelected(SelectionEvent e)
        {
          gemeinsam(e);
        }

        private void gemeinsam(SelectionEvent e)
        {
          if (e.data instanceof Date)
          {
            Date datum = (Date)e.data;
            _endzeitpunkt = datum.getTime();
            _daten.setEndzeitpunkt(_endzeitpunkt);

            if (!pruefePlausibilitaet())
            {
              Color color = new Color(Display.getDefault(), 255, 0, 0);
              _labelInfo.setForeground(color);
              _labelInfo.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
              _compositeZeitGruppenInhalt.layout();
            }
            else
            {
              _labelInfo.setText("");
              _compositeZeitGruppenInhalt.layout();
            }

            // fireEvent();
          }
        }

      });

      _schaltflaecheHinzufuegen = new Button(_returnComposite, SWT.NONE);
      _schaltflaecheHinzufuegen.setText("+");
      _schaltflaecheLoeschen = new Button(_returnComposite, SWT.NONE);
      _schaltflaecheLoeschen.setText("-");

      _schaltflaecheHinzufuegen.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
        public void mouseDown(org.eclipse.swt.events.MouseEvent e)
        {
          aktionZeitgruppenzeileAnhaengen();
        }
      });

      _schaltflaecheLoeschen.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
        public void mouseDown(org.eclipse.swt.events.MouseEvent e)
        {
          aktionZeitgruppenzeileLoeschen();
        }
      });

      GridData gridData = new GridData();
      gridData.horizontalAlignment = GridData.FILL;
      gridData.grabExcessHorizontalSpace = false;
      gridData.grabExcessVerticalSpace = false;
      gridData.verticalAlignment = GridData.CENTER;
      _returnComposite.setLayoutData(gridData);

      GridLayout gridLayout = new GridLayout();
      gridLayout.numColumns = 7;
      _returnComposite.setLayout(gridLayout);

      return _returnComposite;
    }

  }

  private void loescheZeitGruppenZeile(ZeitGruppenZeile zgz)
  {
    if (_listeZeitGruppenZeilen.contains(zgz) && _listeZeitGruppenZeilen.size() > 1)
    {
      _listeZeitGruppenZeilen.remove(zgz);
      zgz.dispose();
      _compositeZeitGruppenInhalt.layout();
    }
  }

  private void einfuegenZeitGruppenZeileNachPosition(ZeitGruppenZeile zg)
  {
    generiereZeitgruppeLeer();

    LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe> _dsAlt = new LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe>();

    int idx = _listeZeitGruppenZeilen.indexOf(zg) + 1;

    if (idx < _listeZeitGruppenZeilen.size())
    {
      if (idx > -1)
      {
        for (int i = idx; i < (_listeZeitGruppenZeilen.size() - 1); i++)
        {
          _dsAlt.add(_listeZeitGruppenZeilen.get(i).getDatenStrukturZeitGruppe());
        }
      }

      int i = idx + 1;
      Iterator<KalenderZeitFilterDatenStrukturZeitGruppe> it = _dsAlt.iterator();
      while (it.hasNext())
      {
        KalenderZeitFilterDatenStrukturZeitGruppe datenStrukturZeitGruppe = it.next();
        _listeZeitGruppenZeilen.get(i).setDatenStrukturZeitGruppe(datenStrukturZeitGruppe);
        i++;
      }

      /*
       * Neue Zeile mit Nullstellung erzeugen
       */
      GregorianCalendar gcNullStellung = new GregorianCalendar();
      gcNullStellung.set(Calendar.HOUR_OF_DAY, 0);
      gcNullStellung.set(Calendar.MINUTE, 0);
      gcNullStellung.set(Calendar.SECOND, 0);
      gcNullStellung.set(Calendar.MILLISECOND, 0);

      _listeZeitGruppenZeilen.get(idx).setDatenStrukturZeitGruppe(
          new KalenderZeitFilterDatenStrukturZeitGruppe(gcNullStellung.getTimeInMillis(), gcNullStellung
              .getTimeInMillis()));
    }

  }

  /**
   * Generiert eine leere Zeitgruppe
   * 
   */
  private void generiereZeitgruppeLeer()
  {
    Date d = new Date();
    long nu = d.getTime();

    KalenderZeitFilterDatenStrukturZeitGruppe kz = new KalenderZeitFilterDatenStrukturZeitGruppe(nu, nu);
    _DatenStruktur.fuegeZeitGruppeHinzu(kz);
    ZeitGruppenZeile zgz = new ZeitGruppenZeile(_compositeZeitGruppenInhalt, kz);
    zgz.generiereComposite();

    _listeZeitGruppenZeilen.add(zgz);
    _compositeZeitGruppenInhalt.layout();
  }

  /**
   * This method initializes _group111
   * 
   */
  private void create_group111()
  {
    GridData gridData7 = new GridData();
    gridData7.horizontalAlignment = GridData.FILL;
    gridData7.grabExcessHorizontalSpace = true;
    gridData7.grabExcessVerticalSpace = true;
    gridData7.verticalAlignment = GridData.FILL;
    gridData7.horizontalSpan = 3;
    GridLayout gridLayout211 = new GridLayout();
    gridLayout211.numColumns = 2;
    gridLayout211.makeColumnsEqualWidth = false;
    gridLayout211.marginHeight = 5;
    gridLayout211.marginWidth = 5;
    gridLayout211.verticalSpacing = 5;
    gridLayout211.horizontalSpacing = 5;
    _group111 = new Group(_composite5, SWT.NONE);
    _group111.setText("Ausgabe Optionen");
    _group111.setLayoutData(gridData7);
    _group111.setLayout(gridLayout211);
    b2111 = new Button(_group111, SWT.CHECK);
    b2111.setText("Ausgabe sofort");
    b2111.setSelection(true);
    GridData gridData = new GridData();
    gridData.horizontalSpan = 2;
    b2111.setLayoutData(gridData);

    b1111 = new Button(_group111, SWT.CHECK);
    b1111.setText("Speichern");
    // b1111.setSelection(true);
    b1112 = new Button(_group111, SWT.CHECK);
    b1112.setText("ffentlich");
    b1112.setSelection(true);
    // b1112.setEnabled(false);
    
    l1111 = new Label(_group111, SWT.LEFT);
    l1111.setText("Informationstext");
    // l1111.setEnabled(false);
    t1111 = new Text(_group111, SWT.BORDER | SWT.SINGLE);
    // t1111.setEnabled(false);
    
    _buttonAenderungsProtokoll = new Button(_group111, SWT.CHECK);
    _buttonAenderungsProtokoll.setText("nderungsprotokoll");
    
    GridData gridDataT = new GridData();
    gridDataT.horizontalAlignment = GridData.FILL;
    gridDataT.grabExcessHorizontalSpace = true;
    // gridDataT.grabExcessVerticalSpace = true;
    // gridDataT.verticalAlignment = GridData.BEGINNING;
    t1111.setLayoutData(gridDataT);
    
    /* b1111.addSelectionListener(new SelectionAdapter()
    {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        boolean selected = b1111.getSelection(); 
        b1112.setEnabled(selected);
        l1111.setEnabled(selected);
        t1111.setEnabled(selected);
      }
    }); */
  }

  /**
   * This method initializes _group1111
   * 
   */
  private void create_group1111()
  {
    GridData gridData6 = new GridData();
//    gridData6.horizontalAlignment = GridData.END;
    gridData6.horizontalAlignment = GridData.BEGINNING;
    gridData6.grabExcessHorizontalSpace = false;
    gridData6.verticalAlignment = GridData.CENTER;
    GridData gridData8 = new GridData();
    gridData8.grabExcessHorizontalSpace = true;
    gridData8.verticalAlignment = GridData.FILL;
    gridData8.grabExcessVerticalSpace = true;
    gridData8.horizontalAlignment = GridData.FILL;
    gridData8.horizontalSpan = 2;
    GridLayout gridLayout2111 = new GridLayout();
    gridLayout2111.numColumns = 2;
    gridLayout2111.makeColumnsEqualWidth = false;
    gridLayout2111.marginHeight = 5;
    gridLayout2111.marginWidth = 5;
    gridLayout2111.verticalSpacing = 5;
    gridLayout2111.horizontalSpacing = 5;
    _group1111 = new Group(_composite5, SWT.NONE);
    _group1111.setText("Parameter");
    _group1111.setLayoutData(gridData8);
    _group1111.setLayout(gridLayout2111);
    l111111 = new Label(_group1111, SWT.NONE);
    l111111.setText("Simulationsvariante");
    spinner = new Spinner(_group1111, SWT.BORDER);
    spinner.setMaximum(254);
    spinner.setLayoutData(gridData6);
    
    _labelBindungen = new Label(_group1111, SWT.NONE);
    _labelBindungen.setText("Bindung");
    _comboBindungen = new Combo(_group1111, SWT.READ_ONLY);
    _comboBindungen.add("-");
	_comboBindungen.setText("-");
    _comboBindungen.setEnabled(false);
	_comboBindungen.addSelectionListener(new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
			_comboBindungen.setToolTipText(_comboBindungen.getText());
		}
	});
    GridData gridData9 = new GridData();
    gridData9.horizontalAlignment = GridData.BEGINNING;
    gridData9.grabExcessHorizontalSpace = true;
    gridData9.verticalAlignment = GridData.CENTER;
    _comboBindungen.setLayoutData(gridData9);
  }

  @Override
  public void dispose()
  {
    super.dispose();
    ObjektSpeicher.getInstanz().getSystemObjekte().clear();
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if (page != null)
    {

      _listeZeitGruppenZeilen = null;
      _DatenStruktur = null;
      page.hideView(this);
    }

  }
  
  public class MyViewerComparator extends ViewerComparator {
		private int propertyIndex;
		private static final int ASCENDING = 0;
	  private static final int DESCENDING = 1;
		private int direction = ASCENDING;

		public MyViewerComparator() {
			this.propertyIndex = 0;
			direction = ASCENDING;
		}

		public int getDirection() {
			return direction == DESCENDING ? SWT.DOWN : SWT.UP;
		}

		public void setColumn(int column) {
			if (column == this.propertyIndex) {
				// Same column as last sort; toggle the direction
				direction = 1 - direction;
			} else {
				// New column; do an ascending sort
				this.propertyIndex = column;
				direction = ASCENDING;
			}
		}

		@Override
		public int compare(Viewer viewer, Object e1, Object e2) {
			SystemObject p1 = (SystemObject) e1;
			SystemObject p2 = (SystemObject) e2;
			
			int rc = 0;
			switch (propertyIndex) {
			case 0:
				rc = p1.toString().compareTo(p2.toString());
				break;
		
			default:
				rc = 0;
			}
			// If descending order, flip the direction
			if (direction == DESCENDING) {
				rc = -rc;
			}
			return rc;
		}

	}

  private void erzeugeTabelle1()
  {
    _tvVorhandeneObjekte = new TableViewer(composite41, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL
        | SWT.V_SCROLL);
    _tvVorhandeneObjekte.setData("name", "viewer1");

    ViewerSorter sorter;
    
    _tvVorhandeneObjekte.setComparator(new MyViewerComparator());
    
    final TableColumn tc0 = new TableColumn(_tvVorhandeneObjekte.getTable(), SWT.LEFT);
    tc0.setText("Name (Pid)");
    tc0.setWidth(300);
    
    tc0.addSelectionListener(new SelectionAdapter() {
    	
    	private boolean sortFlag = false;
    	
    	@Override
    	public void widgetSelected(SelectionEvent e) {
    		
    		sortFlag = !sortFlag;
    		
    		((MyViewerComparator)_tvVorhandeneObjekte.getComparator()).setColumn(0);
			int dir = sortFlag ? SWT.DOWN : SWT.UP;
			_tvVorhandeneObjekte.getTable().setSortDirection(dir);
			_tvVorhandeneObjekte.getTable().setSortColumn(tc0);
			_tvVorhandeneObjekte.refresh();
		}
    });
    
//    tc0.addListener(SWT.Selection, new Listener() {
//      private int colPos = 0;
//
//      private boolean sortFlag = false;
//
//      public void handleEvent(Event event)
//      {
//        // System.out.println(".handleEvent()");
//        sortFlag = !sortFlag;
//        Table table = _tvVorhandeneObjekte.getTable();
//
//        BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];
//
//        for (int i = 0; i < table.getItemCount(); i++)
//        {
//          TableItem item = table.getItem(i);
//
//          String[] strings = new String[table.getColumnCount()];
//          for (int j = 0; j < table.getColumnCount(); j++)
//          {
//            String text = item.getText(j);
//            strings[j] = text;
//          }
//          comparables[colPos][i] = new StringComparator(strings, i, colPos);
//        }
//
//        sortTable(table, table.getItems(), colPos, comparables, sortFlag);
//      }
//
//    });
    // getSite().setSelectionProvider(_tvBetriebsMeldungen1);

    _tvVorhandeneObjekte.getTable().setHeaderVisible(true);
    _tvVorhandeneObjekte.getTable().setLinesVisible(true);

    _tvVorhandeneObjekte.setContentProvider(new TabelleInhaltsVersorger());
    _tvVorhandeneObjekte.setLabelProvider(new TabelleDarstellungsVersorger());

    /** ************************************************************************************** */

    // int ops = DND.DROP_COPY | DND.DROP_MOVE;
    //
    // List<SystemObject> liste = ObjektSpeicher.getInstanz().getSystemObjekte();
    //
    // _meldungsAnzeigeFallAdapter = new MeldungsanzeigeFallAdapter(_tvBetriebsMeldungen1, liste);
    // Transfer[] dropTransfers = new Transfer[] { TextTransfer.getInstance() };
    // _tvBetriebsMeldungen1.addDropSupport(ops, dropTransfers, _meldungsAnzeigeFallAdapter);
    // _zugZuhoererMeldung = new ZugZuhoererMeldung(SichtObjektzusammenstellung.getTabelleSystemObjekte(),
    // ObjektSpeicher.getInstanz().getSystemObjekte());
    // Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
    // _tvBetriebsMeldungen1.addDragSupport(ops, dragTransfers, _zugZuhoererMeldung);
    /** ************************************************************************************** */

    _tvVorhandeneObjekte.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof SystemObject)
        {
          _text.setText(_tvVorhandeneObjekte.getTable().getSelectionCount() + " / "
              + _tvVorhandeneObjekte.getTable().getItemCount());
          composite44.layout(true);
        }
      }
    });

  }

  private void erzeugeTabelle2()
  {
    _tvAusgewaehlteObjekte = new TableViewer(composite42, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL
        | SWT.V_SCROLL);
    _tvAusgewaehlteObjekte.setData("name", "viewer2");

    final TableColumn tc0 = new TableColumn(_tvAusgewaehlteObjekte.getTable(), SWT.LEFT);
    tc0.setText("Name (Pid)");
    tc0.setWidth(300);
    // tc0.setMoveable(true);

    _tvAusgewaehlteObjekte.getTable().setHeaderVisible(true);
    _tvAusgewaehlteObjekte.getTable().setLinesVisible(true);

    _tvAusgewaehlteObjekte.setContentProvider(new TabelleInhaltsVersorger());
    _tvAusgewaehlteObjekte.setLabelProvider(new TabelleDarstellungsVersorger());

    /** ************************************************************************************** */
    // int ops = DND.DROP_COPY | DND.DROP_MOVE;
    //
    // List<SystemObject> liste = ObjektSpeicher.getInstanz().getSystemObjekte();
    //
    // _meldungsAnzeigeFallAdapter = new MeldungsanzeigeFallAdapter(_tvBetriebsMeldungen2, liste);
    // Transfer[] dropTransfers = new Transfer[] { TextTransfer.getInstance() };
    // _tvBetriebsMeldungen2.addDropSupport(ops, dropTransfers, _meldungsAnzeigeFallAdapter);
    // _zugZuhoererMeldung = new ZugZuhoererMeldung(_tvBetriebsMeldungen2, _objektSpeicher.getSystemObjekte());
    // Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
    // _tvBetriebsMeldungen2.addDragSupport(ops, dragTransfers, _zugZuhoererMeldung);
    /** ************************************************************************************** */

    _tvAusgewaehlteObjekte.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof SystemObject)
        {
          _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
              + _tvAusgewaehlteObjekte.getTable().getItemCount());
          composite43.layout(true);
        }
      }
    });

    // getSite().setSelectionProvider(_tvBetriebsMeldungen2);

  }

  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {
    public Object[] getElements(Object inputElement)
    {
      return ((List)inputElement).toArray();
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {

    }

  }

  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object object, int columnIndex)
    {
      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          returnString = ((SystemObject)object).toString();
          break;
        // case 1:
        // returnString = ((SystemObject)object).getName();
        // break;
        default:
          returnString = "";

      }

      return returnString;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /**
   * This method initializes composite41
   * 
   */
  private void createComposite41()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = false;
    gridData.verticalAlignment = GridData.FILL;
    gridData.heightHint = 200;
    composite41 = new Composite(_top, SWT.NONE);
    composite41.setLayout(new FillLayout());
    composite41.setLayoutData(gridData);
  }

  /**
   * This method initializes composite42
   * 
   */
  private void createComposite42()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = false;
    gridData.verticalAlignment = GridData.FILL;
    gridData.heightHint = 200;

    composite42 = new Composite(_top, SWT.NONE);
    composite42.setLayout(new FillLayout());
    composite42.setLayoutData(gridData);
  }

  /**
   * This method initializes composite43
   * 
   */
  private void createComposite43()
  {
    GridData gridData15 = new GridData();
    gridData15.horizontalAlignment = GridData.FILL;
    gridData15.grabExcessHorizontalSpace = true;
    gridData15.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout4 = new GridLayout();
    gridLayout4.numColumns = 3;
    gridLayout4.marginHeight = 0;
    gridLayout4.marginWidth = 0;
    GridData gridData9 = new GridData();
    gridData9.grabExcessHorizontalSpace = true;
    gridData9.verticalAlignment = GridData.CENTER;
    gridData9.horizontalAlignment = GridData.FILL;
    composite43 = new Composite(_top, SWT.NONE);
    composite43.setLayoutData(gridData9);
    createComposite45();
    composite43.setLayout(gridLayout4);
    _text2 = new Text(composite43, SWT.BORDER);
    _text2.setEditable(false);
    _text2.setText("0 / 0");
    button51 = new Button(composite43, SWT.BUTTON5);
    button51.setText("X");
    button51.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        _tvAusgewaehlteObjekte.setSelection(null);

        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        composite43.layout(true);
      }

    });
    // button51.setEnabled(false);
  }

  /**
   * This method initializes composite44
   * 
   */
  private void createComposite44()
  {
    GridData gridData14 = new GridData();
    gridData14.horizontalAlignment = GridData.BEGINNING;
    gridData14.grabExcessVerticalSpace = false;
    gridData14.grabExcessHorizontalSpace = false;
    gridData14.verticalAlignment = GridData.CENTER;
    GridData gridData13 = new GridData();
    gridData13.horizontalAlignment = GridData.FILL;
    gridData13.grabExcessHorizontalSpace = true;
    gridData13.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout2 = new GridLayout();
    gridLayout2.numColumns = 4;
    gridLayout2.marginWidth = 0;
    gridLayout2.marginHeight = 0;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    composite44 = new Composite(_top, SWT.NONE);
    composite44.setLayoutData(gridData1);
    composite44.setLayout(gridLayout2);
    text = new Text(composite44, SWT.BORDER);
    text.setLayoutData(gridData13);
    button4 = new Button(composite44, SWT.NONE);
    button4.setText("Filtern");
    button4.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        _searchFilter = new ViewerFilter() {
          @Override
          public boolean select(Viewer viewer, Object parentElement, Object element)
          {
            return ((SystemObject)element).toString().contains(text.getText());
          }
        };
        _tvVorhandeneObjekte.setFilters(new ViewerFilter[] { _searchFilter });
        _text.setText(_tvVorhandeneObjekte.getTable().getSelectionCount() + " / "
            + _tvVorhandeneObjekte.getTable().getItemCount());
        composite44.layout(true);
      }
    });
    _text = new Text(composite44, SWT.BORDER);
    _text.setEditable(false);
    _text.setText("0 / 0");
    button5 = new Button(composite44, SWT.NONE);
    button5.setText("X");
    button5.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        _tvVorhandeneObjekte.setSelection(null);

        _text.setText(_tvVorhandeneObjekte.getTable().getSelectionCount() + " / "
            + _tvVorhandeneObjekte.getTable().getItemCount());
        composite44.layout(true);
      }

    });
  }

  /**
   * This method initializes composite45
   * 
   */
  private void createComposite45()
  {
    GridData gridData16 = new GridData();
    gridData16.horizontalAlignment = GridData.FILL;
    gridData16.grabExcessHorizontalSpace = true;
    gridData16.verticalAlignment = GridData.CENTER;
    composite45 = new Composite(composite43, SWT.NONE);
    composite45.setLayout(new GridLayout());
    composite45.setLayoutData(gridData16);
  }

  /**
   * Hier werden die Daten der Spalten des "original" TableItem[] in einen MyComperatorArray umkopiert und darauf hin
   * wird dieser mittels der Array.sort(Object[] o) Methode sortiert. Da sich der MyComperator die ursprngliche
   * Position in der Tabelle gemerkt hat, kann man nun ganz einfach die entsprechenden Werte im Original Array tauschen
   * und schon ist man fertig.
   * 
   * @param tableItems
   */
  protected final void sortTable(Table table, TableItem[] tableItems, int colPos, BaseComapator[][] comparables,
      boolean sortFlag)
  {

    // SortFlag bei allen richtig setzen ... leider kann ich bei inneren
    // keine static Variablen anlegen ansonsten wre das hier unntig...
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      comparables[colPos][i]._sortFlag = sortFlag;
    }

    Arrays.sort(comparables[colPos]);

    String[] tmp;
    String[] textBuffer = null;
    textBuffer = new String[comparables.length];
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      int rowNum = comparables[colPos][i]._oldRowNum;
      tmp = comparables[colPos][i]._textBuffer;
      tableItems[rowNum].setText(comparables[colPos][rowNum]._textBuffer);
      tableItems[i].setText(tmp);
    }

  }

  /**
   * Basisklasse fr alle weiteren Comparatoren...
   * 
   */
  class BaseComapator implements Comparable
  {

    String[] _textBuffer;

    int _oldRowNum;

    int _colPos;

    boolean _sortFlag;

    public BaseComapator(String[] textBuffer, int oldRowNum, int colPos)
    {
      _textBuffer = textBuffer;
      _oldRowNum = oldRowNum;
      _colPos = colPos;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o)
    {

      return 0;
    }
  }

  /**
   * 
   * @author Administrator Eigener Comparator der das Vergleichen der einzelnen Tabellenzeilen vornimmt.
   */
  class StringComparator extends BaseComapator implements Comparable
  {

    /**
     * Konstruktor ...
     * 
     * @param textBuffer :
     *          Der Text der Zeile
     * @param rowNum :
     *          Die Zeilennr der Tabellenzeile
     * @param sortFlag :
     *          Aufsteigend false, Absteigend true
     * @param colPos :
     *          Die Spalte nach der Sortiert werden soll
     */

    public StringComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
    }

    public final int compareTo(Object arg0)
    {
      int ret = _textBuffer[_colPos].compareTo(((StringComparator)arg0)._textBuffer[_colPos]);
      return _sortFlag ? ret : ret * -1;
    }

  }

  public Text getText()
  {
    return _text;
  }

  private void aktionAnfordern()
  {
    boolean ausgabe = b2111.getSelection();

    if (ausgabe)
    {

      IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
      IWorkbenchPage page = dw.getActivePage();
      try
      {
        
        
//        String s = _skriptInfo.getPid() + ";" + _numberInstances.toString();
        String s = _skriptInfo.getAktionsId() + ";" + _numberInstances.toString();
        
//        System.out.println("Erzeuge sec: " + s);
        
        page.showView(ProtokollAusgabeSicht.ID, s, IWorkbenchPage.VIEW_ACTIVATE);

        _numberInstances++;
      }
      catch (PartInitException e1)
      {
        e1.printStackTrace();
      }

    }

    List<ArchiveDataKind> kinds = new ArrayList<ArchiveDataKind>();

    if (b1.getSelection())
      kinds.add(ArchiveDataKind.ONLINE);
    if (b2.getSelection())
      kinds.add(ArchiveDataKind.ONLINE_DELAYED);
    if (b3.getSelection())
      kinds.add(ArchiveDataKind.REQUESTED);
    if (b31.getSelection())
      kinds.add(ArchiveDataKind.REQUESTED_DELAYED);

    ArchiveDataKind[] adks = new ArchiveDataKind[kinds.size()];
    adks = kinds.toArray(adks);

    boolean speichern = b1111.getSelection();
    boolean oeffentlich = b1112.getSelection(); // speichern ? b1112.getSelection() : true;
    String informationsText = "[" + _skriptInfo.getAktionsId() + "]" + t1111.getText(); // speichern ? t1111.getText() : "";
    String ersteller = BenutzerAngemeldet.getInstanz().getName();
    int simvar = spinner.getSelection();
    boolean aenderungsProtokoll = _buttonAenderungsProtokoll.getSelection();
    
    Map<String, String> bindungen = new HashMap<String, String>();
    Bindung bindung = (Bindung)_comboBindungen.getData();
    if (bindung != null) {
    	String name = _comboBindungen.getText();
    	String wert = bindung.getWertZuNamen(name);
    	if (wert != null) {
    		bindungen.put("?" + bindung.getBezeichner(), wert);
    	}
    }

    AnfrageThread th = new AnfrageThread(_connection, _tvAusgewaehlteObjekte, _listeZeitGruppenZeilen, adks, speichern, 
    		oeffentlich, informationsText, aenderungsProtokoll,ersteller, ausgabe, simvar, bindungen, _numberInstances - 1, 
    		_skriptInfo);

  }
  
} // @jve:decl-index=0:visual-constraint="10,10,847,295"
