/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.views;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import net.sf.paperclips.DefaultGridLook;
import net.sf.paperclips.GridColumn;
import net.sf.paperclips.GridPrint;
import net.sf.paperclips.ImagePrint;
import net.sf.paperclips.LineBorder;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.Print;
import net.sf.paperclips.ScalePrint;
import net.sf.paperclips.TextPrint;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.buv.rw.basislib.seiteneinstellungen.DefsSeiteneinstellungen;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.DruckvorschauDialog.SeitenDekoration;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.ReferenceAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.data.AttributeProtocolData;
import de.bsvrz.pua.prot.data.ProtocolData;
import de.bsvrz.pua.prot.util.ExpressionResult;
import de.bsvrz.pua.prot.util.ExpressionResult.ResultType;

/**
 * @author Pittner
 *
 */
public class Tools
{
  public static Print createPrint( Table table, String titel ) 
  {
    // Create GridPrint with all columns at default size.
    DefaultGridLook look = new DefaultGridLook();
    look.setCellBorder( new LineBorder() );
    RGB background = table.getDisplay().getSystemColor( SWT.COLOR_WIDGET_BACKGROUND ).getRGB();
    look.setHeaderBackground( background );

    GridPrint grid = new GridPrint( look );

    // Add header and footer to match table column names.
    TableColumn[] columns = table.getColumns();
    for ( int i = 0; i < columns.length; i++ ) {
      TableColumn col = columns[i];
      grid.addColumn( new GridColumn( col.getAlignment(), SWT.DEFAULT, 0 ) );
      Print cell = createCell( col.getImage(), col.getText(), SWT.CENTER );
      grid.addHeader( cell );
    }

    // Add content rows
    TableItem[] items = table.getItems();
    for ( int i = 0; i < items.length; i++ ) {
      TableItem item = items[i];
      for ( int j = 0; j < columns.length; j++ )
        grid.add( createCell( item.getImage( j ), item.getText( j ), columns[j].getAlignment() ) );
    }
    
    
    ScalePrint scalePrint = new ScalePrint(grid);
    
    PagePrint page = new PagePrint(scalePrint);

    de.bsvrz.buv.rw.basislib.Activator activator = de.bsvrz.buv.rw.basislib.Activator.getDefault();
    
    IPreferenceStore preferenceStore = activator.getPreferenceStore();
   
    page.setHeader(new SeitenDekoration(preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileLinks),
        preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileMitte), preferenceStore
        .getString(DefsSeiteneinstellungen.kopfzeileRechts), true, titel));

    page.setHeaderGap(5);
    page.setFooter(new SeitenDekoration(preferenceStore.getString(DefsSeiteneinstellungen.fusszeileLinks),
        preferenceStore.getString(DefsSeiteneinstellungen.fusszeileMitte), preferenceStore
            .getString(DefsSeiteneinstellungen.fusszeileRechts), false));
    page.setFooterGap(5);
    

    return page;
  }

  public static Print createCell( Image image, String text, int align ) {
    if ( image == null )
      return new TextPrint( text, align );

    GridPrint grid = new GridPrint( "p, d" );
    grid.add( new ImagePrint( image.getImageData(), image.getDevice().getDPI() ) );
    grid.add( new TextPrint( text, align ) );
    return grid;
  }
  
  /**
   * Workarround:
   * 
   * Script mit  ....:QKfz.Wert als "QKfz"  liefert  "nicht ermittelbar"  fuer Spalte "QKfz [Fz/h]" 
   *   bei Verwendung von 'protocolData.asTextValue().getValueText()'
   *   
   * Script mit  ....:QKfz als "QKfz"  liefert  "-"  fuer Spalte "QKfz.Wert [Fz/h]"
   *   bei Verwendung von 'protocolData.asTextValue().getValueText()'
   *   
   * ExpressionResult ist in diesem Fall: ResultType.NONE, 
   *   aber  _isState == true && _string.equals("nicht ermittelbar") == true
   * --> ist hier ein Widerspruch
   *   
   */
  public static String toText(ProtocolData protocolData)
  {
    String ret = null;
    
    try
    {
      if (protocolData instanceof AttributeProtocolData)
      {
    	if (protocolData.getAttributeType() instanceof ReferenceAttributeType) {
    		ret = "";
    		try {
    			ReferenceValue ref = ((AttributeProtocolData)protocolData).asReferenceValue();
    			SystemObject so = ref.getSystemObject();
    			if (so != null) {
    				ret = so.toString();
    			}
    		}
    		catch (Exception e) {
    			// ignore - "" returned
    			// e.printStackTrace();
    		}
    		return ret;
    	}
    	
    	ExpressionResult result = getInternalValue((AttributeProtocolData)protocolData);
    	   	
        if (result != null)
        {
          if (result.getType() == ResultType.NONE && result.getString() != null && result.getString().length() > 0)
          {
            ret = result.getString();
          }
        }
      }
    }
    catch(Exception e)
    {
    }
    
    if(ret == null)
    {
      try {
    	  ret = protocolData.asTextValue().getValueText();  
      }
      catch (ArithmeticException e)
      {
    	  ret = e.getMessage();
      }
    }
    
    return (ret);
  }
  
  @SuppressWarnings("unchecked")
  public static ExpressionResult getInternalValue(AttributeProtocolData data) throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException
  {
    Class attributeProtocolDataClass = data.getClass();
    Method getInternalValueMethod = attributeProtocolDataClass.getDeclaredMethod("getInternalValue", new Class[]{});
    getInternalValueMethod.setAccessible(true);
    
    return (ExpressionResult)getInternalValueMethod.invoke(data, new Object[]{});
  }

}
