/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.streckenprofil.sts.a8;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.buv.rw.basislib.berechtigung.IOberflaechenBerechtigungZuhoerer;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.IDaVPreLader;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Zentrale Instanz zur Verwaltung der Berechtigungen fr die Funktionen des
 * Parametrierungs-Plug-ins.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * 
 * @version $Id: StreckenprofilBerechtigungsVerwaltung.java,v 1.3 2014/04/17 15:51:18 Pittner Exp $
 */
public final class StreckenprofilBerechtigungsVerwaltung implements
		IOberflaechenBerechtigungZuhoerer, IDaVPreLader {

	/**
	 * Debug-Logger zur Beobachtung der Rahmenwerks-Callbacks.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/** das Singleton-Objekt fr die Verwaltung der Berechtigungen. */
	private static final StreckenprofilBerechtigungsVerwaltung INSTANCE = new StreckenprofilBerechtigungsVerwaltung();

	/**
	 * liefert die Instanz des Verwaltungsobjekts.
	 * 
	 * @return das Objekt
	 */
	public static StreckenprofilBerechtigungsVerwaltung getInstanz() {
		return StreckenprofilBerechtigungsVerwaltung.INSTANCE;
	}

	/**
	 * die Liste der Objekte, fr die die Berechtigungen abgefragt werden.
	 * 
	 * Das Berechtigungskonzept des Rahmenwerks erfordert zur Prfung von
	 * Berechtigungen in jedem Fall die bergabe eines Systemobjekts, auch wenn
	 * die Berechtigung an sich nicht objektbezogen ist. Fr die globale
	 * Verwaltung der UDA-Pluginberechtigungen wird deshalb hier das lokale
	 * Applikationsobjekt in die Liste eingetragen und bei der Prfung der
	 * Berechtigungen bergeben.
	 */
	private final List<SystemObject> berechtigunsObjektListe = new ArrayList<SystemObject>();

	/** Konstruktor. */
	private StreckenprofilBerechtigungsVerwaltung() {
		super();
		DaVPreLader.getInstanz().hinzufuegenListenerPreLader(this);
	}

	/** aktualisiert den Zustand der Berechtigungen des Plugins. */
	public void aktualisiereBerechtigungen() {
		if (DaVVerbindung.getInstanz().getDaVVerbindung() != null) {
			DEBUG.config("Streckenprofil-Plugin Aktualisiere Berechtigungen");
			if (berechtigunsObjektListe.isEmpty()) {
				berechtigunsObjektListe.add(DaVVerbindung.getInstanz()
						.getDaVVerbindung().getLocalApplicationObject());
			}
			OberflaechenBerechtigung.getInstanz()
					.pruefeBerechtigungElementGruppe(berechtigunsObjektListe,
							Activator.class.getName());
		} else {
			DEBUG
					.config("Streckenprofil-Plugin Offline! Keine Aktualisierung der Berechtigungen");
		}
	}

	@Override
	public void freigabe(final OberflaechenBerechtigungEreignis e) {
		List<OberflaechenFunktion> liste = e.getOberflaechenFunktionSperrung();
		DEBUG.config("Streckenprofil-Plugin Freigabe empfangen");
		if (liste.size() == 0) {
			DEBUG.config("keine gesperrten Elemente");
		} else {
			DEBUG.config(liste.size() + " gesperrte Elemente");
		}
		for (final OberflaechenFunktion oFunk : liste) {
			final StreckenprofilPluginFunktion funktion = StreckenprofilPluginFunktion
					.getFunktion(oFunk.getFunktionsName());
			if (funktion != null) {
				funktion.setFreigegeben(false);
			}
		}

		liste = e.getOberflaechenFunktionFreigabe();
		if (liste.size() == 0) {
			DEBUG.config("keine freigegebenen Elemente");
		} else {
			DEBUG.config(liste.size() + " freigegebene Elemente");
		}
		for (final OberflaechenFunktion oFunk : liste) {
			final StreckenprofilPluginFunktion funktion = StreckenprofilPluginFunktion
					.getFunktion(oFunk.getFunktionsName());
			if (funktion != null) {
				funktion.setFreigegeben(true);
			}
		}
	}

	@Override
	public void sperrung(final OberflaechenBerechtigungEreignis e) {
		freigabe(e);
	}

	@Override
	public void systemObjekteAktualisierung(final GtmDatenBasis datenBasis) {
		// keine Aktionen erforderlich
	}

	@Override
	public void systemObjekteInitial(final GtmDatenBasis datenBasis) {
		DEBUG.info("Streckenprofil-Plugin Prelader Callback");
		aktualisiereBerechtigungen();
	}
}
