/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8;

import java.util.Map;

import org.eclipse.birt.chart.model.attribute.ColorDefinition;
import org.eclipse.birt.chart.model.attribute.impl.ColorDefinitionImpl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypDatenQuellen;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.fachmodellglobal.attribute.AttProzent;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttBemessungsFahrzeugDichte;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttBemessungsVerkehrsStaerkeStunde;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttFahrStreifenLage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttFahrzeugDichte;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttGeschwindigkeit;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttVerkehrsStaerkeIntervall;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttVerkehrsStaerkeStunde;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.FahrStreifen;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnitt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitGeglaettetFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitGeglaettetMq;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitIntervall;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitMq;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitTrendExtraPolationFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitTrendExtraPolationMq;

/**
 * Util fr Streckenprofil.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilUtil.java,v 1.5 2014/04/22 13:49:40 Pittner Exp $
 */
public final class StreckenprofilUtil {

	/**
	 * Konstruktor verstecken.
	 */
	private StreckenprofilUtil() {
		// nichts
	}

	/**
	 * Der Context fr alle Streckenprofil Charts. Damit wird der ShortCut zum
	 * exportieren der Verkehrsdaten aktiviert und deaktiviert.
	 */
//	public static final String STRECKENPROFIL_CHART_KONTEXT = "de.bsvrz.buv.plugin.streckenprofil.sts.a81.context";
	public static final String STRECKENPROFIL_CHART_KONTEXT = "de.bsvrz.buv.plugin.streckenprofil.sts.a8.context";

	/**
	 * Ermittelt das Verkehrsdatum vom Typ
	 * {@link OdVerkehrsDatenKurzZeitGeglaettetMq.Daten} fr einen
	 * {@link FahrStreifen} in einem Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param mq
	 *            der {@link MessQuerschnitt}, dessen Verkehrsdaten abgefragt
	 *            werden.
	 * @param od
	 *            der Verkehrsdatensatz des {@link MessQuerschnitt}s
	 * @return Datum als einfache Zahl.
	 */
	private static Double getFSValueOdVerkehrsDatenKurzZeitGeglaettetFs(
			final LinienEigenschaften lp, final MessQuerschnitt mq,
			final OdVerkehrsDatenKurzZeitGeglaettetFs.Daten od) {
		Double result = null;
		if (od instanceof OdVerkehrsDatenKurzZeitGeglaettetFs.Daten) {
			final OdVerkehrsDatenKurzZeitGeglaettetFs.Daten fsDatum = od;
			switch (lp.getTypVerkehrsDaten()) {
			case QKFZ:
				final AttVerkehrsStaerkeStunde wertQKFZ = fsDatum.getQKfzG()
						.getWert();
				if (wertQKFZ != null) {
					result = wertQKFZ.doubleValue();
				}
				break;
			case QLKW:
				final AttVerkehrsStaerkeStunde wertQLKW = fsDatum.getQLkwG()
						.getWert();
				if (wertQLKW != null) {
					result = wertQLKW.doubleValue();
				}
				break;
			case QPKW:
				final AttVerkehrsStaerkeStunde wertQPKW = fsDatum.getQPkwG()
						.getWert();
				if (wertQPKW != null) {
					result = wertQPKW.doubleValue();
				}
				break;
			case VKFZ:
				final AttGeschwindigkeit wertVKFZ = fsDatum.getVKfzG()
						.getWert();
				if (wertVKFZ != null) {
					result = wertVKFZ.doubleValue();
				}
				break;
			case VLKW:
				final AttGeschwindigkeit wertVLKW = fsDatum.getVLkwG()
						.getWert();
				if (wertVLKW != null) {
					result = wertVLKW.doubleValue();
				}
				break;
			case VPKW:
				final AttGeschwindigkeit wertVPKW = fsDatum.getVPkwG()
						.getWert();
				if (wertVPKW != null) {
					result = wertVPKW.doubleValue();
				}
				break;
			case ALKW:
				final AttProzent wertALKW = fsDatum.getALkwG().getWert();
				if (wertALKW != null) {
					result = wertALKW.doubleValue();
				}
				break;
			case KB:
				final AttBemessungsFahrzeugDichte wertKB = fsDatum.getKBG()
						.getWert();
				if (wertKB != null) {
					result = wertKB.doubleValue();
				}
				break;
			case KKFZ:
				final AttFahrzeugDichte wertKKFZ = fsDatum.getKKfzG().getWert();
				if (wertKKFZ != null) {
					result = wertKKFZ.doubleValue();
				}
				break;
			case KLKW:
				final AttFahrzeugDichte wertKLKW = fsDatum.getKLkwG().getWert();
				if (wertKLKW != null) {
					result = wertKLKW.doubleValue();
				}
				break;
			case KPKW:
				final AttFahrzeugDichte wertKPKW = fsDatum.getKPkwG().getWert();
				if (wertKPKW != null) {
					result = wertKPKW.doubleValue();
				}
				break;
			case QB:
				final AttBemessungsVerkehrsStaerkeStunde wertQB = fsDatum
						.getQBG().getWert();
				if (wertQB != null) {
					result = wertQB.doubleValue();
				}
				break;
			default:
				result = null;
			}
		}
		return result;
	}

	/**
	 * 
	 * Ermittelt das Verkehrsdatum vom Typ
	 * {@link OdVerkehrsDatenKurzZeitFs.Daten} fr einen {@link FahrStreifen} in
	 * einem Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param mq
	 *            der {@link MessQuerschnitt} zu dem Fahrstreifen gehren.
	 * @param od
	 *            der Verkehrsdatensatz eines {@link FahrStreifen}s
	 * @return Datum als einfache Zahl.
	 */
	private static Double getFSValueOdVerkehrsDatenKurzZeitFs(
			final LinienEigenschaften lp, final MessQuerschnitt mq,
			final OdVerkehrsDatenKurzZeitFs.Daten od) {
		Double result = null;
		if (od instanceof OdVerkehrsDatenKurzZeitFs.Daten) {
			final OdVerkehrsDatenKurzZeitFs.Daten fsDatum = od;
			switch (lp.getTypVerkehrsDaten()) {
			case QKFZ:
				final AttVerkehrsStaerkeStunde wertQKFZ = fsDatum.getQKfz()
						.getWert();
				if (wertQKFZ != null) {
					result = wertQKFZ.doubleValue();
				}
				break;
			case QLKW:
				final AttVerkehrsStaerkeStunde wertQLKW = fsDatum.getQLkw()
						.getWert();
				if (wertQLKW != null) {
					result = wertQLKW.doubleValue();
				}
				break;
			case QPKW:
				final AttVerkehrsStaerkeStunde wertQPKW = fsDatum.getQPkw()
						.getWert();
				if (wertQPKW != null) {
					result = wertQPKW.doubleValue();
				}
				break;
			case VKFZ:
				final AttGeschwindigkeit wertVKFZ = fsDatum.getVKfz().getWert();
				if (wertVKFZ != null) {
					result = wertVKFZ.doubleValue();
				}
				break;
			case VLKW:
				final AttGeschwindigkeit wertVLKW = fsDatum.getVLkw().getWert();
				if (wertVLKW != null) {
					result = wertVLKW.doubleValue();
				}
				break;
			case VPKW:
				final AttGeschwindigkeit wertVPKW = fsDatum.getVPkw().getWert();
				if (wertVPKW != null) {
					result = wertVPKW.doubleValue();
				}
				break;
			case ALKW:
				final AttProzent wertALKW = fsDatum.getALkw().getWert();
				if (wertALKW != null) {
					result = wertALKW.doubleValue();
				}
				break;
			case KB:
				final AttBemessungsFahrzeugDichte wertKB = fsDatum.getKB()
						.getWert();
				if (wertKB != null) {
					result = wertKB.doubleValue();
				}
				break;
			case KKFZ:
				final AttFahrzeugDichte wertKKFZ = fsDatum.getKKfz().getWert();
				if (wertKKFZ != null) {
					result = wertKKFZ.doubleValue();
				}
				break;
			case KLKW:
				final AttFahrzeugDichte wertKLKW = fsDatum.getKLkw().getWert();
				if (wertKLKW != null) {
					result = wertKLKW.doubleValue();
				}
				break;
			case KPKW:
				final AttFahrzeugDichte wertKPKW = fsDatum.getKPkw().getWert();
				if (wertKPKW != null) {
					result = wertKPKW.doubleValue();
				}
				break;
			case QB:
				final AttBemessungsVerkehrsStaerkeStunde wertQB = fsDatum
						.getQB().getWert();
				if (wertQB != null) {
					result = wertQB.doubleValue();
				}
				break;
			default:
				result = null;
			}
		}
		return result;
	}

	/**
	 * 
	 * Ermittelt das Verkehrsdatum vom Typ
	 * {@link OdVerkehrsDatenKurzZeitIntervall.Daten} fr einen
	 * {@link FahrStreifen} in einem Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param od
	 *            der Verkehrsdatensatz des {@link FahrStreifen}s
	 * @return Datum als einfache Zahl.
	 */
	private static Double getFSValueOdVerkehrsDatenKurzZeitFs(
			final LinienEigenschaften lp,
			final OdVerkehrsDatenKurzZeitIntervall.Daten od) {
		Double result = null;
		if (od instanceof OdVerkehrsDatenKurzZeitIntervall.Daten) {
			final OdVerkehrsDatenKurzZeitIntervall.Daten fsDatum = od;
			switch (lp.getTypVerkehrsDaten()) {
			case QKFZ:
				final AttVerkehrsStaerkeIntervall wertQKFZ = fsDatum.getQKfz()
						.getWert();
				if (wertQKFZ != null) {
					result = wertQKFZ.doubleValue();
				}
				break;
			case QLKW:
				final AttVerkehrsStaerkeIntervall wertQLKW = fsDatum.getQLkw()
						.getWert();
				if (wertQLKW != null) {
					result = wertQLKW.doubleValue();
				}
				break;
			case QPKW:
				final AttVerkehrsStaerkeIntervall wertQPKW = fsDatum.getQPkw()
						.getWert();
				if (wertQPKW != null) {
					result = wertQPKW.doubleValue();
				}
				break;
			case VKFZ:
				final AttGeschwindigkeit wertVKFZ = fsDatum.getVKfz().getWert();
				if (wertVKFZ != null) {
					result = wertVKFZ.doubleValue();
				}
				break;
			case VLKW:
				final AttGeschwindigkeit wertVLKW = fsDatum.getVLkw().getWert();
				if (wertVLKW != null) {
					result = wertVLKW.doubleValue();
				}
				break;
			case VPKW:
				final AttGeschwindigkeit wertVPKW = fsDatum.getVPkw().getWert();
				if (wertVPKW != null) {
					result = wertVPKW.doubleValue();
				}
				break;
			default:
				result = null;
			}
		}
		return result;
	}

	/**
	 * 
	 * Ermittelt das Verkehrsdatum vom Typ
	 * {@link OdVerkehrsDatenKurzZeitTrendExtraPolationMq.Daten} fr einen
	 * {@link FahrStreifen} in einem Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param mq
	 *            der {@link MessQuerschnitt} zu dem Fahrstreifen gehren.
	 * @param od
	 *            der Verkehrsdatensatz eines {@link FahrStreifen}s
	 * @return Datum als einfache Zahl.
	 */
	private static Double getFSValueOdVerkehrsDatenKurzZeitTrendExtraPolationFs(
			final LinienEigenschaften lp, final MessQuerschnitt mq,
			final OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Daten od) {
		Double result = null;
		if (od instanceof OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Daten) {
			final OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Daten fsDatum = od;
			switch (lp.getTypVerkehrsDaten()) {
			case QKFZ:
				final AttVerkehrsStaerkeStunde wertQKFZ = fsDatum.getQKfzP()
						.getWert();
				if (wertQKFZ != null) {
					result = wertQKFZ.doubleValue();
				}
				break;
			case QLKW:
				final AttVerkehrsStaerkeStunde wertQLKW = fsDatum.getQLkwP()
						.getWert();
				if (wertQLKW != null) {
					result = wertQLKW.doubleValue();
				}
				break;
			case QPKW:
				final AttVerkehrsStaerkeStunde wertQPKW = fsDatum.getQPkwP()
						.getWert();
				if (wertQPKW != null) {
					result = wertQPKW.doubleValue();
				}
				break;
			case VKFZ:
				final AttGeschwindigkeit wertVKFZ = fsDatum.getVKfzP()
						.getWert();
				if (wertVKFZ != null) {
					result = wertVKFZ.doubleValue();
				}
				break;
			case VLKW:
				final AttGeschwindigkeit wertVLKW = fsDatum.getVLkwP()
						.getWert();
				if (wertVLKW != null) {
					result = wertVLKW.doubleValue();
				}
				break;
			case VPKW:
				final AttGeschwindigkeit wertVPKW = fsDatum.getVPkwP()
						.getWert();
				if (wertVPKW != null) {
					result = wertVPKW.doubleValue();
				}
				break;
			case ALKW:
				final AttProzent wertALKW = fsDatum.getALkwP().getWert();
				if (wertALKW != null) {
					result = wertALKW.doubleValue();
				}
				break;
			case KB:
				final AttBemessungsFahrzeugDichte wertKB = fsDatum.getKBP()
						.getWert();
				if (wertKB != null) {
					result = wertKB.doubleValue();
				}
				break;
			case KKFZ:
				final AttFahrzeugDichte wertKKFZ = fsDatum.getKKfzP().getWert();
				if (wertKKFZ != null) {
					result = wertKKFZ.doubleValue();
				}
				break;
			case KLKW:
				final AttFahrzeugDichte wertKLKW = fsDatum.getKLkwP().getWert();
				if (wertKLKW != null) {
					result = wertKLKW.doubleValue();
				}
				break;
			case KPKW:
				final AttFahrzeugDichte wertKPKW = fsDatum.getKPkwP().getWert();
				if (wertKPKW != null) {
					result = wertKPKW.doubleValue();
				}
				break;
			case QB:
				final AttBemessungsVerkehrsStaerkeStunde wertQB = fsDatum
						.getQBP().getWert();
				if (wertQB != null) {
					result = wertQB.doubleValue();
				}
				break;
			default:
				result = null;
			}
		}
		return result;
	}

	/**
	 * Ermittelt das Verkehrsdatum fr einen {@link MessQuerschnitt} in einem
	 * Chart.
	 * 
	 * @param lp
	 *            Eigenschaften der Linie bzw. Series
	 * @param mq
	 *            das Betrachtete Modell - Objekt
	 * @param onlineDatum
	 *            die aktuellsten Verkehrsdaten
	 * @return Datum als einfache Zahl.
	 */
	public static Double getMQValue(final LinienEigenschaften lp,
			final MessQuerschnittAllgemein mq, final OnlineDatum onlineDatum) {
		Double ergebnis = null;
		if (onlineDatum instanceof OdVerkehrsDatenKurzZeitMq.Daten) {
			ergebnis = getMQValueOdVerkehrsDatenKurzZeitMq(lp, mq,
					(OdVerkehrsDatenKurzZeitMq.Daten) onlineDatum);
		} else if (onlineDatum instanceof OdVerkehrsDatenKurzZeitGeglaettetMq.Daten) {
			ergebnis = getMQValueOdVerkehrsDatenKurzZeitGeglaettetMq(lp, mq,
					(OdVerkehrsDatenKurzZeitGeglaettetMq.Daten) onlineDatum);
		} else if (onlineDatum instanceof OdVerkehrsDatenKurzZeitTrendExtraPolationMq.Daten) {
			ergebnis = getMQValueOdVerkehrsDatenKurzZeitTrendExtraPolationMq(
					lp,
					mq,
					(OdVerkehrsDatenKurzZeitTrendExtraPolationMq.Daten) onlineDatum);
		} else if (onlineDatum instanceof OdVerkehrsDatenKurzZeitIntervall.Daten) {
			ergebnis = getMQValueOdVerkehrsDatenKurzZeitIntervall(lp, mq,
					(OdVerkehrsDatenKurzZeitIntervall.Daten) onlineDatum);
		}
		return ergebnis;
	}

	/**
	 * 
	 * Ermittelt ein Verkehrsdatum fr einen {@link FahrStreifen} in einem
	 * Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param mqAllgemein
	 *            der {@link MessQuerschnitt} zu dem Fahrstreifen gehren.
	 * @param daten
	 *            der Verkehrsdatensatz eines {@link FahrStreifen}s
	 * @return Datum als einfache Zahl.
	 */
	public static Double getFSValue(final LinienEigenschaften lp,
			final MessQuerschnittAllgemein mqAllgemein,
			final Map<SystemObjekt, OnlineDatum> daten) {
		Double result = null;
		if (mqAllgemein instanceof MessQuerschnitt) {
			final MessQuerschnitt mq = (MessQuerschnitt) mqAllgemein;
			for (final FahrStreifen fs : mq.getFahrStreifen()) {
				final AttFahrStreifenLage fsLage = fs.getKdFahrStreifen()
						.getDatum().getLage();
				if ((AttFahrStreifenLage.ZUSTAND_0_HFS.equals(fsLage) && TypDatenQuellen.HFS
						.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_1__1UEFS.equals(fsLage) && TypDatenQuellen.UEFS1
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_2__2UEFS.equals(fsLage) && TypDatenQuellen.UEFS2
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_3__3UEFS.equals(fsLage) && TypDatenQuellen.UEFS3
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_4__4UEFS.equals(fsLage) && TypDatenQuellen.UEFS4
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_5__5UEFS.equals(fsLage) && TypDatenQuellen.UEFS5
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_6__6UEFS.equals(fsLage) && TypDatenQuellen.UEFS6
								.equals(lp.getTypDatenQuelle()))) {
					final OnlineDatum od = daten.get(fs);
					result = getFSValue(lp, fs, od);

				}
			}
		}
		return result;
	}

	/**
	 * 
	 * Ermittelt ein Verkehrsdatum fr einen {@link FahrStreifen} in einem
	 * Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param mqAllgemein
	 *            der {@link MessQuerschnitt} zu dem Fahrstreifen gehren.
	 * @param daten
	 *            der Verkehrsdatensatz eines {@link FahrStreifen}s
	 * @return Datum als einfache Zahl.
	 */
	public static Double getFSValue(final LinienEigenschaften lp,
			final MessQuerschnittAllgemein mqAllgemein, final OnlineDatum daten) {
		Double result = null;
		if (mqAllgemein instanceof MessQuerschnitt) {
			final MessQuerschnitt mq = (MessQuerschnitt) mqAllgemein;
			for (final FahrStreifen fs : mq.getFahrStreifen()) {
				final AttFahrStreifenLage fsLage = fs.getKdFahrStreifen()
						.getDatum().getLage();
				if ((AttFahrStreifenLage.ZUSTAND_0_HFS.equals(fsLage) && TypDatenQuellen.HFS
						.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_1__1UEFS.equals(fsLage) && TypDatenQuellen.UEFS1
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_2__2UEFS.equals(fsLage) && TypDatenQuellen.UEFS2
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_3__3UEFS.equals(fsLage) && TypDatenQuellen.UEFS3
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_4__4UEFS.equals(fsLage) && TypDatenQuellen.UEFS4
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_5__5UEFS.equals(fsLage) && TypDatenQuellen.UEFS5
								.equals(lp.getTypDatenQuelle()))
						|| (AttFahrStreifenLage.ZUSTAND_6__6UEFS.equals(fsLage) && TypDatenQuellen.UEFS6
								.equals(lp.getTypDatenQuelle()))) {
					result = getFSValue(lp, fs, daten);

				}
			}
		}
		return result;
	}

	/**
	 * 
	 * Ermittelt ein Verkehrsdatum fr einen {@link FahrStreifen} in einem
	 * Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param fs
	 *            der {@link FahrStreifen}.
	 * @param datum
	 *            der Verkehrsdatensatz eines {@link FahrStreifen}s
	 * @return Datum als einfache Zahl.
	 */
	public static Double getFSValue(final LinienEigenschaften lp,
			final FahrStreifen fs, final OnlineDatum datum) {
		Double result = null;
		if (datum instanceof OdVerkehrsDatenKurzZeitFs.Daten) {
			result = StreckenprofilUtil.getFSValueOdVerkehrsDatenKurzZeitFs(lp,
					null, (OdVerkehrsDatenKurzZeitFs.Daten) datum);
		} else if (datum instanceof OdVerkehrsDatenKurzZeitGeglaettetFs.Daten) {
			result = StreckenprofilUtil
					.getFSValueOdVerkehrsDatenKurzZeitGeglaettetFs(lp, null,
							(OdVerkehrsDatenKurzZeitGeglaettetFs.Daten) datum);
		} else if (datum instanceof OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Daten) {
			result = StreckenprofilUtil
					.getFSValueOdVerkehrsDatenKurzZeitTrendExtraPolationFs(
							lp,
							null,
							(OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Daten) datum);
		} else if (datum instanceof OdVerkehrsDatenKurzZeitIntervall.Daten) {
			result = StreckenprofilUtil.getFSValueOdVerkehrsDatenKurzZeitFs(lp,
					(OdVerkehrsDatenKurzZeitIntervall.Daten) datum);
		}
		return result;
	}

	/**
	 * 
	 * Ermittelt das Verkehrsdatum vom Typ
	 * {@link OdVerkehrsDatenKurzZeitGeglaettetMq.Daten} fr einen
	 * {@link MessQuerschnittAllgemein} in einem Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param mq
	 *            der {@link MessQuerschnittAllgemein}.
	 * @param onlineDatum
	 *            der Verkehrsdatensatz eines {@link MessQuerschnittAllgemein}s
	 * @return Datum als einfache Zahl.
	 */
	private static Double getMQValueOdVerkehrsDatenKurzZeitGeglaettetMq(
			final LinienEigenschaften lp, final MessQuerschnittAllgemein mq,
			final OdVerkehrsDatenKurzZeitGeglaettetMq.Daten onlineDatum) {
		Double result = null;
		switch (lp.getTypVerkehrsDaten()) {
		case QKFZ:
			final AttVerkehrsStaerkeStunde wertQKFZ = onlineDatum.getQKfzG()
					.getWert();
			if (wertQKFZ != null) {
				result = wertQKFZ.doubleValue();
			}
			break;
		case QLKW:
			final AttVerkehrsStaerkeStunde wertQLKW = onlineDatum.getQLkwG()
					.getWert();
			if (wertQLKW != null) {
				result = wertQLKW.doubleValue();
			}
			break;
		case QPKW:
			final AttVerkehrsStaerkeStunde wertQPKW = onlineDatum.getQPkwG()
					.getWert();
			if (wertQPKW != null) {
				result = wertQPKW.doubleValue();
			}
			break;
		case VKFZ:
			final AttGeschwindigkeit wertVKFZ = onlineDatum.getVKfzG()
					.getWert();
			if (wertVKFZ != null) {
				result = wertVKFZ.doubleValue();
			}
			break;
		case VLKW:
			final AttGeschwindigkeit wertVLKW = onlineDatum.getVLkwG()
					.getWert();
			if (wertVLKW != null) {
				result = wertVLKW.doubleValue();
			}
			break;
		case VPKW:
			final AttGeschwindigkeit wertVPKW = onlineDatum.getVPkwG()
					.getWert();
			if (wertVPKW != null) {
				result = wertVPKW.doubleValue();
			}
			break;
		case ALKW:
			final AttProzent wertALKW = onlineDatum.getALkwG().getWert();
			if (wertALKW != null) {
				result = wertALKW.doubleValue();
			}
			break;
		case KB:
			final AttBemessungsFahrzeugDichte wertKB = onlineDatum.getKBG()
					.getWert();
			if (wertKB != null) {
				result = wertKB.doubleValue();
			}
			break;
		case KKFZ:
			final AttFahrzeugDichte wertKKFZ = onlineDatum.getKKfzG().getWert();
			if (wertKKFZ != null) {
				result = wertKKFZ.doubleValue();
			}
			break;
		case KLKW:
			final AttFahrzeugDichte wertKLKW = onlineDatum.getKLkwG().getWert();
			if (wertKLKW != null) {
				result = wertKLKW.doubleValue();
			}
			break;
		case KPKW:
			final AttFahrzeugDichte wertKPKW = onlineDatum.getKPkwG().getWert();
			if (wertKPKW != null) {
				result = wertKPKW.doubleValue();
			}
			break;
		case QB:
			final AttBemessungsVerkehrsStaerkeStunde wertQB = onlineDatum
					.getQBG().getWert();
			if (wertQB != null) {
				result = wertQB.doubleValue();
			}
			break;
		default:
			result = null;
		}

		return result;
	}

	/**
	 * 
	 * Ermittelt das Verkehrsdatum vom Typ
	 * {@link OdVerkehrsDatenKurzZeitMq.Daten} fr einen
	 * {@link MessQuerschnittAllgemein} in einem Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param mq
	 *            der {@link MessQuerschnittAllgemein}.
	 * @param onlineDatum
	 *            der Verkehrsdatensatz eines {@link MessQuerschnittAllgemein}s
	 * @return Datum als einfache Zahl.
	 */
	private static Double getMQValueOdVerkehrsDatenKurzZeitMq(
			final LinienEigenschaften lp, final MessQuerschnittAllgemein mq,
			final OdVerkehrsDatenKurzZeitMq.Daten onlineDatum) {
		Double result = null;
		switch (lp.getTypVerkehrsDaten()) {
		case QKFZ:
			final AttVerkehrsStaerkeStunde wertQKFZ = onlineDatum.getQKfz()
					.getWert();
			if (wertQKFZ != null) {
				result = wertQKFZ.doubleValue();
			}
			break;
		case QLKW:
			final AttVerkehrsStaerkeStunde wertQLKW = onlineDatum.getQLkw()
					.getWert();
			if (wertQLKW != null) {
				result = wertQLKW.doubleValue();
			}
			break;
		case QPKW:
			final AttVerkehrsStaerkeStunde wertQPKW = onlineDatum.getQPkw()
					.getWert();
			if (wertQPKW != null) {
				result = wertQPKW.doubleValue();
			}
			break;
		case VKFZ:
			final AttGeschwindigkeit wertVKFZ = onlineDatum.getVKfz().getWert();
			if (wertVKFZ != null) {
				result = wertVKFZ.doubleValue();
			}
			break;
		case VLKW:
			final AttGeschwindigkeit wertVLKW = onlineDatum.getVLkw().getWert();
			if (wertVLKW != null) {
				result = wertVLKW.doubleValue();
			}
			break;
		case VPKW:
			final AttGeschwindigkeit wertVPKW = onlineDatum.getVPkw().getWert();
			if (wertVPKW != null) {
				result = wertVPKW.doubleValue();
			}
			break;
		case ALKW:
			final AttProzent wertALKW = onlineDatum.getALkw().getWert();
			if (wertALKW != null) {
				result = wertALKW.doubleValue();
			}
			break;
		case KB:
			final AttBemessungsFahrzeugDichte wertKB = onlineDatum.getKB()
					.getWert();
			if (wertKB != null) {
				result = wertKB.doubleValue();
			}
			break;
		case KKFZ:
			final AttFahrzeugDichte wertKKFZ = onlineDatum.getKKfz().getWert();
			if (wertKKFZ != null) {
				result = wertKKFZ.doubleValue();
			}
			break;
		case KLKW:
			final AttFahrzeugDichte wertKLKW = onlineDatum.getKLkw().getWert();
			if (wertKLKW != null) {
				result = wertKLKW.doubleValue();
			}
			break;
		case KPKW:
			final AttFahrzeugDichte wertKPKW = onlineDatum.getKPkw().getWert();
			if (wertKPKW != null) {
				result = wertKPKW.doubleValue();
			}
			break;
		case QB:
			final AttBemessungsVerkehrsStaerkeStunde wertQB = onlineDatum
					.getQB().getWert();
			if (wertQB != null) {
				result = wertQB.doubleValue();
			}
			break;
		default:
			result = null;
		}

		return result;
	}

	/**
	 * 
	 * Ermittelt das Verkehrsdatum vom Typ
	 * {@link OdVerkehrsDatenKurzZeitIntervall.Daten} fr einen
	 * {@link MessQuerschnittAllgemein} in einem Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param mq
	 *            der {@link MessQuerschnittAllgemein}.
	 * @param onlineDatum
	 *            der Verkehrsdatensatz eines {@link MessQuerschnittAllgemein}s
	 * @return Datum als einfache Zahl.
	 */
	private static Double getMQValueOdVerkehrsDatenKurzZeitIntervall(
			final LinienEigenschaften lp, final MessQuerschnittAllgemein mq,
			final OdVerkehrsDatenKurzZeitIntervall.Daten onlineDatum) {
		Double result = null;
		switch (lp.getTypVerkehrsDaten()) {
		case QKFZ:
			final AttVerkehrsStaerkeIntervall wertQKFZ = onlineDatum.getQKfz()
					.getWert();
			if (wertQKFZ != null) {
				result = wertQKFZ.doubleValue();
			}
			break;
		case QLKW:
			final AttVerkehrsStaerkeIntervall wertQLKW = onlineDatum.getQLkw()
					.getWert();
			if (wertQLKW != null) {
				result = wertQLKW.doubleValue();
			}
			break;
		case QPKW:
			final AttVerkehrsStaerkeIntervall wertQPKW = onlineDatum.getQPkw()
					.getWert();
			if (wertQPKW != null) {
				result = wertQPKW.doubleValue();
			}
			break;
		case VKFZ:
			final AttGeschwindigkeit wertVKFZ = onlineDatum.getVKfz().getWert();
			if (wertVKFZ != null) {
				result = wertVKFZ.doubleValue();
			}
			break;
		case VLKW:
			final AttGeschwindigkeit wertVLKW = onlineDatum.getVLkw().getWert();
			if (wertVLKW != null) {
				result = wertVLKW.doubleValue();
			}
			break;
		case VPKW:
			final AttGeschwindigkeit wertVPKW = onlineDatum.getVPkw().getWert();
			if (wertVPKW != null) {
				result = wertVPKW.doubleValue();
			}
			break;
		default:
			result = null;
		}

		return result;
	}

	/**
	 * 
	 * Ermittelt das Verkehrsdatum vom Typ
	 * {@link OdVerkehrsDatenKurzZeitTrendExtraPolationMq.Daten} fr einen
	 * {@link MessQuerschnittAllgemein} in einem Chart.
	 * 
	 * @param lp
	 *            die {@link LinienEigenschaften}, welche bestimmen welche
	 *            Verkehrsdaten aus dem Datensatz extrahiert werden sollen.
	 * @param mq
	 *            der {@link MessQuerschnittAllgemein}.
	 * @param onlineDatum
	 *            der Verkehrsdatensatz eines {@link MessQuerschnittAllgemein}s
	 * @return Datum als einfache Zahl.
	 */
	private static Double getMQValueOdVerkehrsDatenKurzZeitTrendExtraPolationMq(
			final LinienEigenschaften lp, final MessQuerschnittAllgemein mq,
			final OdVerkehrsDatenKurzZeitTrendExtraPolationMq.Daten onlineDatum) {
		Double result = null;
		switch (lp.getTypVerkehrsDaten()) {
		case QKFZ:
			final AttVerkehrsStaerkeStunde wertQKFZ = onlineDatum.getQKfzP()
					.getWert();
			if (wertQKFZ != null) {
				result = wertQKFZ.doubleValue();
			}
			break;
		case QLKW:
			final AttVerkehrsStaerkeStunde wertQLKW = onlineDatum.getQLkwP()
					.getWert();
			if (wertQLKW != null) {
				result = wertQLKW.doubleValue();
			}
			break;
		case QPKW:
			final AttVerkehrsStaerkeStunde wertQPKW = onlineDatum.getQPkwP()
					.getWert();
			if (wertQPKW != null) {
				result = wertQPKW.doubleValue();
			}
			break;
		case VKFZ:
			final AttGeschwindigkeit wertVKFZ = onlineDatum.getVKfzP()
					.getWert();
			if (wertVKFZ != null) {
				result = wertVKFZ.doubleValue();
			}
			break;
		case VLKW:
			final AttGeschwindigkeit wertVLKW = onlineDatum.getVLkwP()
					.getWert();
			if (wertVLKW != null) {
				result = wertVLKW.doubleValue();
			}
			break;
		case VPKW:
			final AttGeschwindigkeit wertVPKW = onlineDatum.getVPkwP()
					.getWert();
			if (wertVPKW != null) {
				result = wertVPKW.doubleValue();
			}
			break;
		case ALKW:
			final AttProzent wertALKW = onlineDatum.getALkwP().getWert();
			if (wertALKW != null) {
				result = wertALKW.doubleValue();
			}
			break;
		case KB:
			final AttBemessungsFahrzeugDichte wertKB = onlineDatum.getKBP()
					.getWert();
			if (wertKB != null) {
				result = wertKB.doubleValue();
			}
			break;
		case KKFZ:
			final AttFahrzeugDichte wertKKFZ = onlineDatum.getKKfzP().getWert();
			if (wertKKFZ != null) {
				result = wertKKFZ.doubleValue();
			}
			break;
		case KLKW:
			final AttFahrzeugDichte wertKLKW = onlineDatum.getKLkwP().getWert();
			if (wertKLKW != null) {
				result = wertKLKW.doubleValue();
			}
			break;
		case KPKW:
			final AttFahrzeugDichte wertKPKW = onlineDatum.getKPkwP().getWert();
			if (wertKPKW != null) {
				result = wertKPKW.doubleValue();
			}
			break;
		case QB:
			final AttBemessungsVerkehrsStaerkeStunde wertQB = onlineDatum
					.getQBP().getWert();
			if (wertQB != null) {
				result = wertQB.doubleValue();
			}
			break;
		default:
			result = null;
		}

		return result;
	}

	/**
	 * Liste von Farben, die der Reihe nach den Linien bzw. Balken zugewiesen
	 * werden.
	 */
	private static final int[] FARBEN = new int[] { SWT.COLOR_BLACK,
			SWT.COLOR_BLUE, SWT.COLOR_CYAN, SWT.COLOR_GREEN, SWT.COLOR_MAGENTA,
			SWT.COLOR_RED, SWT.COLOR_WHITE, SWT.COLOR_YELLOW };

	/**
	 * Index, der zuletzt benutzten Farbe (initial 0).
	 */
	private static int letzteFarbe;

	/**
	 * Liefert die nchste Farbe.
	 * 
	 * @return die nchste Farbe als {@link RGB}
	 */
	public static RGB getNaechsteFarbe() {
		letzteFarbe++;
		if (letzteFarbe >= FARBEN.length || letzteFarbe < 0) {
			letzteFarbe = 0;
		}
		return Display.getDefault().getSystemColor(FARBEN[letzteFarbe])
				.getRGB();
	}

	/**
	 * Macht aus einem {@link RGB} eine {@link ColorDefinitionImpl}.
	 * 
	 * @param rgb
	 *            fr diesen RGB Wert wird eine {@link ColorDefinition} gesucht.
	 * @return die {@link ColorDefinition} des RGB Wertes.
	 */
	public static ColorDefinition createColorDefinition(final RGB rgb) {
		return null != rgb ? ColorDefinitionImpl.create(rgb.red, rgb.green,
				rgb.blue) : ColorDefinitionImpl.BLACK().copyInstance();
	}
}
