/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.daten;

import java.awt.Color;
import java.awt.Font;
import java.util.HashMap;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.type.StreckenProfilElementTyp;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.type.StreckenProfilInhaltTyp;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.util.Tupel;
 
/**
 * Modelliert den Zugriff auf die Diagramm-Daten eines Streckenprofils
 * 
 * @author Pittner
 *
 */
public class StreckenProfilDiagrammData implements IStreckenProfilDaten
{
  private static final long serialVersionUID = -6713427173153117154L;
  private Map<String, Map<String, SortedMap<Long, Object>>> diagrammDaten = null;
  private String editor = null;
  private boolean simuAktiv = false;
  private Long datenZeit = null;

  private StreckenProfilDiagrammData()
  {
  }
  
  public StreckenProfilDiagrammData(Map<String, Map<String, SortedMap<Long, Object>>> diagrammDaten, String editor, boolean simuAktiv, Long datenZeit)
  {		 
    this.editor = editor;
    this.datenZeit = datenZeit;
    this.simuAktiv = simuAktiv;
//    this.diagrammDaten = diagrammDaten;
    
    if(diagrammDaten != null)
    {
      this.diagrammDaten = new HashMap<String, Map<String, SortedMap<Long, Object>>>();
      for(String objekt : diagrammDaten.keySet())
      {
        Map<String, SortedMap<Long, Object>> mapDatenreiheEbeneKopie = new HashMap<String, SortedMap<Long,Object>>();
        this.diagrammDaten.put(objekt, mapDatenreiheEbeneKopie);
        Map<String, SortedMap<Long, Object>> mapDatenreiheEbene = diagrammDaten.get(objekt);
        for(String bezeichner : mapDatenreiheEbene.keySet())
        {
          SortedMap<Long, Object> datenKopie = new TreeMap<Long, Object>(mapDatenreiheEbene.get(bezeichner));
          mapDatenreiheEbeneKopie.put(bezeichner, datenKopie);
        }
      }
    }
  }
  
  public static StreckenProfilDiagrammData erzeugeDifferenzdaten(StreckenProfilDiagrammData datenVorgaenger, StreckenProfilDiagrammData datenNachfolger)
  {
    StreckenProfilDiagrammData ret = null;
    
    if(datenVorgaenger != null)
    {
      try
      {
        Map<String, Map<String, SortedMap<Long, Object>>> diagrammDaten = null;
        if(datenNachfolger.diagrammDaten != null)
        {
          diagrammDaten = new HashMap<String, Map<String, SortedMap<Long, Object>>>();
          for(String objekt : datenNachfolger.diagrammDaten.keySet())
          {
            Map<String, SortedMap<Long, Object>> mapDatenreiheEbeneVorgaenger = datenVorgaenger.diagrammDaten.get(objekt);
            Map<String, SortedMap<Long, Object>> mapDatenreiheEbeneNachfolger = datenNachfolger.diagrammDaten.get(objekt);
            if(mapDatenreiheEbeneVorgaenger == null)
            {
              diagrammDaten.put(objekt, mapDatenreiheEbeneNachfolger);
            }
            else
            {
              Map<String, SortedMap<Long, Object>> mapDatenreiheEbeneKopie = new HashMap<String, SortedMap<Long, Object>>();
              diagrammDaten.put(objekt, mapDatenreiheEbeneKopie);
              for(String bezeichner : mapDatenreiheEbeneNachfolger.keySet())
              {
                SortedMap<Long, Object> sortierteDatenVorgaenger = mapDatenreiheEbeneVorgaenger.get(bezeichner);
                SortedMap<Long, Object> sortierteDatenNachfolger = mapDatenreiheEbeneNachfolger.get(bezeichner);
                if(sortierteDatenVorgaenger == null || !StreckenProfilDiagrammData.vergleiche(sortierteDatenVorgaenger, sortierteDatenNachfolger))
                {
                  mapDatenreiheEbeneKopie.put(bezeichner, sortierteDatenNachfolger);
                }
              }
            }
          }
        }
        
        ret = new StreckenProfilDiagrammData();
        ret.diagrammDaten = diagrammDaten;
        ret.editor = datenNachfolger.editor;
        ret.datenZeit = datenNachfolger.datenZeit;
        ret.simuAktiv = datenNachfolger.simuAktiv;
      }
      catch(Throwable t)
      {
        System.err.println("gescheitert beim erzeugen der differenzdaten - " + t.getMessage());
      }
    }
    
    if(ret == null)
    {
      ret = datenNachfolger;
    }
    
    return (ret);
  }

  public Map<String, Map<String, SortedMap<Long, Object>>> getDiagrammDaten()
  {
    return (diagrammDaten);
  }
  
  public String getEditor()
  {
    return editor;
  }

  public Long getDatenZeit()
  {
    return datenZeit;
  }

  /** @see java.lang.Object#equals(java.lang.Object) */
	@Override
    public boolean equals(Object object) {
		if(object == null) {
			return false;
		}
		else if(object instanceof StreckenProfilDiagrammData) {
			StreckenProfilDiagrammData other = (StreckenProfilDiagrammData)object;
			
			return this.diagrammDaten.equals(other.diagrammDaten);
		}
		else {
			return false;
		}
	}

	/** @see java.lang.Object#toString() */
	@Override
    public String toString() {
		StringBuilder sb = new StringBuilder();
		
		for (Map.Entry<String, Map<String, SortedMap<Long, Object>>> me : diagrammDaten.entrySet()) {
			String key = me.getKey();
			sb.append("StreckenprofilDiagrammData: " + key + "\n");
			Map<String, SortedMap<Long, Object>> values = me.getValue();
			for (Map.Entry<String, SortedMap<Long, Object>> me2 : values.entrySet()) {
				String key2 = me2.getKey();
				sb.append("Series : " + key2 + "\n");
				SortedMap<Long, Object> values2 = me2.getValue();
				for (Map.Entry<Long, Object> me3 : values2.entrySet()) {
					Long key3 = me3.getKey();
					Object value = me3.getValue();
					if (value instanceof String)
          {
            String s = (String)value;
            sb.append("\t" + key3 + " : " + s + "\n");
            
          }else if (value instanceof Tupel[])
          {
            Tupel[] tupels = (Tupel[])value;
            for (int i = 0; i < tupels.length; i++)
            {
              Tupel tupel = tupels[i];
              Object first = tupel.getFirst();
              Object second = tupel.getSecond();
              
              StreckenProfilElementTyp typ = null;
              StreckenProfilInhaltTyp inhalt = null;
              
              if (first instanceof StreckenProfilElementTyp)
              {
                typ = (StreckenProfilElementTyp)first;                
              }
              if (second instanceof StreckenProfilInhaltTyp)
              {
                inhalt = (StreckenProfilInhaltTyp)second;                
              }
              sb.append("\t" + key3 + " : " + typ.getType() + " --- " + inhalt.getType() + "\n");             
              
            }
            
          }
				}
			}			
		}
	
		return sb.toString();
	}
	 
	/* (non-Javadoc)
	 * @see de.bsvrz.buv.plugin.streckenprofil.sts.daten.IStreckenProfilDaten#getType()
	 */
	public int getType() {
		return IStreckenProfilDaten.DIAGRAMM_DATA;
	}

  // hilfsfunktionen
  private static boolean vergleiche(SortedMap<Long, Object> v, SortedMap<Long, Object> n)
  {
    boolean ret = false;
    
    if(v.size() == n.size())
    {
      ret = true;
      if(v.size() > 0)
      {
        Long[] vXWert = v.keySet().toArray(new Long[0]);
        Long[] nXWert = n.keySet().toArray(new Long[0]);
        Tupel[] vYWert = v.values().toArray(new Tupel[0]);
        Tupel[] nYWert = n.values().toArray(new Tupel[0]);
        for(int pos = 0; pos < vXWert.length && ret; pos++)
        {
          if(vXWert[pos].longValue() == nXWert[pos].longValue())
          {
            Object vErster = vYWert[pos].getFirst();
            Object vZweiter = vYWert[pos].getSecond();
            Object nErster = nYWert[pos].getFirst();
            Object nZweiter = nYWert[pos].getSecond();
            if(vErster instanceof IStreckenProfilToolTipDecoration && nErster instanceof IStreckenProfilToolTipDecoration && vZweiter instanceof String && nZweiter instanceof String)
            {
              if(!vZweiter.equals(nZweiter))
              {
                ret = false;
              }
              else
              {
                String[] vToolTipList = ((IStreckenProfilToolTipDecoration)vErster).getToolTipText();
                String[] nToolTipList = ((IStreckenProfilToolTipDecoration)nErster).getToolTipText();
                if(!StreckenProfilDiagrammData.vergleicheStringFeld(vToolTipList, nToolTipList))
                {
                  ret = false;
                }
              }
            }
            else if(vErster instanceof IStreckenProfilToolTipDecoration && nErster instanceof IStreckenProfilToolTipDecoration && vZweiter instanceof StreckenProfilInhaltTyp && nZweiter instanceof StreckenProfilInhaltTyp)
            {
              if(!((StreckenProfilInhaltTyp)vZweiter).getType().equals(((StreckenProfilInhaltTyp)nZweiter).getType()))
              {
                ret = false;
              }
              else
              {
                String[] vToolTipList = ((IStreckenProfilToolTipDecoration)vErster).getToolTipText();
                String[] nToolTipList = ((IStreckenProfilToolTipDecoration)nErster).getToolTipText();
                if(!StreckenProfilDiagrammData.vergleicheStringFeld(vToolTipList, nToolTipList))
                {
                  ret = false;
                }
              }
            }
            else if(vZweiter instanceof StreckenProfilKonfigurationData.TextKonfiguration && nZweiter instanceof StreckenProfilKonfigurationData.TextKonfiguration)
            {
              StreckenProfilKonfigurationData.TextKonfiguration vTextKonfiguration = (StreckenProfilKonfigurationData.TextKonfiguration)vZweiter;
              StreckenProfilKonfigurationData.TextKonfiguration nTextKonfiguration = (StreckenProfilKonfigurationData.TextKonfiguration)nZweiter;
              if(!StreckenProfilDiagrammData.vergleicheString(vTextKonfiguration.getText(), nTextKonfiguration.getText()))
              {
                ret = false;
              }
              else if(!StreckenProfilDiagrammData.vergleicheFarbe(vTextKonfiguration.getColor(), nTextKonfiguration.getColor()))
              {
                ret = false;
              }
              else if(!StreckenProfilDiagrammData.vergleicheSchrift(vTextKonfiguration.getFont(), nTextKonfiguration.getFont()))
              {
                ret = false;
              }
              else if(!StreckenProfilDiagrammData.vergleicheFliesskomma(vTextKonfiguration.getRotation(), nTextKonfiguration.getRotation()))
              {
                ret = false;
              }
              else if(!StreckenProfilDiagrammData.vergleicheStringFeld(vTextKonfiguration.getTooltip(), nTextKonfiguration.getTooltip()))
              {
                ret = false;
              }
            }
            else
            {
              System.err.println("gescheitert beim vergleichen von unbekannten daten");
              ret = false;
            }
          }
          else
          {
            ret = false;
          }
        }
      }
    }
    
    return (ret);
  }
  
  private static boolean vergleicheStringFeld(String[] vString, String[] nString)
  {
    boolean ret = false;
    
    if(vString == null && nString == null)
    {
      ret = true;
    }
    else if(vString != null && nString != null)
    {
      if(vString.length == nString.length)
      {
        ret = true;
        for(int i = 0; i < vString.length; i++)
        {
          if(!StreckenProfilDiagrammData.vergleicheString(vString[i], nString[i]))
          {
            ret = false;
            break;
          }
        }
      }
    }
    
    return (ret);
  }
  
  private static boolean vergleicheString(String vString, String nString)
  {
    boolean ret = false;
    
    if(vString == null && nString == null)
    {
      ret = true;
    }
    else if(vString != null && nString != null)
    {
      if(vString.equals(nString))
      {
        ret = true;
      }
    }
    
    return (ret);
  }

  private static boolean vergleicheFarbe(Color vFarbe, Color nFarbe)
  {
    boolean ret = false;
    
    if(vFarbe == null && nFarbe == null)
    {
      ret = true;
    }
    else if(vFarbe != null && nFarbe != null)
    {
      if(vFarbe.equals(nFarbe))
      {
        ret = true;
      }
    }
    
    return (ret);
  }

  private static boolean vergleicheSchrift(Font vSchrift, Font nSchrift)
  {
    boolean ret = false;
    
    if(vSchrift == null && nSchrift == null)
    {
      ret = true;
    }
    else if(vSchrift != null && nSchrift != null)
    {
      if(vSchrift.equals(nSchrift))
      {
        ret = true;
      }
    }
    
    return (ret);
  }

  private static boolean vergleicheFliesskomma(double vWert, double nWert)
  {
    boolean ret = false;
    
    if(Double.isNaN(vWert) && Double.isNaN(nWert))
    {
      ret = true;
    }
    else if(vWert == nWert)
    {
      ret = true;
    }
    
    return (ret);
  }

  public boolean isSimuAktiv()
  {
    return simuAktiv;
  }

//  public void setSimuAktiv(boolean simuAktiv)
//  {
//    this.simuAktiv = simuAktiv;
//  }
}
