/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.dialog;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmrechner.objekte.Rechner;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AtlStartInfo;

/**
 * Dialog fr Auswahl des StartStopp-Blocks.
 * 
 * /****************************************************************************
 * *** Copyright (c) 2000, 2008 IBM Corporation and others. All rights reserved.
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: IBM Corporation - initial API and implementation
 ******************************************************************************* 
 * 
 * @author BitCtrl Systems GmbH, Albrecht Uhlmann/Hoesel
 * @version $Id: StartStoppBlockAuswahlDialog.java,v 1.3 2014/04/17 15:51:21 Pittner Exp $
 */
public class StartStoppBlockAuswahlDialog extends Dialog {

	/**
	 * The title of the dialog.
	 */
	private final String title;

	/**
	 * The message to display, or <code>null</code> if none.
	 */
	private final String message;

	/**
	 * The input value; the empty string by default.
	 */
	private AtlStartInfo value = new AtlStartInfo();

	/**
	 * Ok button widget.
	 */
	private Button okButton;

	/**
	 * Der Dateiname des StartStopp - Skriptes.
	 */
	private Text skriptString;

	/**
	 * {@link TableViewer} in dem die {@link Rechner} angezeigt werden.
	 */
	private TableViewer viewer;

	/**
	 * Konstruktor bernimmt verschiedene statische Initialisierungswerte.
	 * 
	 * @param parentShell
	 *            Eltern-Shell
	 */
	public StartStoppBlockAuswahlDialog(final Shell parentShell) {
		super(parentShell);
		title = "Allgemeine Einstellungen zur Streckenprofil-Offline-Simulation";
		message = "Bitte whlen Sie einen Rechner aus, auf dem die SWE Start/Stopp gestartet ist.";

		value = Activator.getDefault().ermittleSimulationStartInfo();
	}

	/*
	 * (non-Javadoc) Method declared on Dialog.
	 */
	@Override
	protected void buttonPressed(final int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			final Object obj = ((StructuredSelection) viewer.getSelection())
					.getFirstElement();
			if (obj instanceof Rechner) {
				value = new AtlStartInfo();
				value.setStartskript(skriptString.getText());
				value.setRechner((Rechner) obj);
				speichereStartInfo();
			}
		} else {
			value = null;
		}
		super.buttonPressed(buttonId);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets
	 * .Shell)
	 */
	@Override
	protected void configureShell(final Shell shell) {
		super.configureShell(shell);
		if (title != null) {
			shell.setText(title);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse
	 * .swt.widgets.Composite)
	 */
	@Override
	protected void createButtonsForButtonBar(final Composite parent) {
		// create OK and Cancel buttons by default
		okButton = createButton(parent, IDialogConstants.OK_ID,
				IDialogConstants.OK_LABEL, true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/*
	 * (non-Javadoc) Method declared on Dialog.
	 */
	@Override
	protected Control createDialogArea(final Composite parent) {
		// create composite
		final Composite composite = (Composite) super.createDialogArea(parent);
		// create message
		if (message != null) {
			final Label label = new Label(composite, SWT.WRAP);
			label.setText(message);
			final GridData data = new GridData(GridData.GRAB_HORIZONTAL
					| GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_CENTER);
			data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
			label.setLayoutData(data);
			label.setFont(parent.getFont());
		}
		final Table table = new Table(composite, getInputTableStyle());
		table.setLinesVisible(true);
		viewer = new TableViewer(table);
		table.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
				| GridData.HORIZONTAL_ALIGN_FILL));
		viewer.setContentProvider(new ArrayContentProvider());
		viewer.setInput(Activator.getDefault().getObjektFactory()
				.bestimmeModellobjekte(Rechner.PID).toArray(new Rechner[0]));
		if (value != null && value.getRechner() != null) {
			viewer.setSelection(new StructuredSelection(value.getRechner()));
		}

		final Label skriptLabel = new Label(composite, SWT.NONE);
		skriptLabel
				.setText("Bitte geben Sie den Dateiname eines StartStopp - Skriptes,\n mit leerem StatStopp - Block, auf dem oben ausgewhlten Rechner an.");

		skriptString = new Text(composite, SWT.BORDER);
		if (value != null && value.getStartskript() != null) {
			skriptString.setText(value.getStartskript());
		}
		skriptString.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
				| GridData.HORIZONTAL_ALIGN_FILL));

		applyDialogFont(composite);
		return composite;
	}

	/**
	 * Returns the ok button.
	 * 
	 * @return the ok button
	 */
	protected Button getOkButton() {
		return okButton;
	}

	/**
	 * Returns the item selected in this dialog.
	 * 
	 * @return the item
	 */
	public AtlStartInfo getValue() {
		return value;
	}

	/**
	 * Speichert die StartInfos der Simulation in den Benutzereinstellungen.
	 */
	private void speichereStartInfo() {
		if (value != null) {
			Activator.getDefault();
			final String rechnerPid = Activator.PLUGIN_ID
					+ ".simulation.rechnerpid";
			final String startStoppLeer = Activator.PLUGIN_ID
					+ ".startstoppskript";
			final EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
			final EinstellungenSpeicher esp = esw
					.getEinstellungen(EinstellungsArt.allgemeinLokal);
			esp.ueberschreibeParameter(startStoppLeer, value.getStartskript());

			esp.ueberschreibeParameter(rechnerPid, value.getRechner().getPid());
			esw.sichereSpeicherAllgemeinLokal();
		}
	}

	/**
	 * Returns the style bits that should be used for the input table field.
	 * Defaults to a single line entry. Subclasses may override.
	 * 
	 * @return the integer style bits that should be used when creating the
	 *         input table
	 * 
	 * @since 3.4
	 */
	protected int getInputTableStyle() {
		return SWT.SINGLE | SWT.BORDER | SWT.FULL_SELECTION;
	}

}
