/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.simulation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditor;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.MessQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.Strecke;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.sys.funclib.bitctrl.daf.DavTools;
import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.Aspekt;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensendeException;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Feld;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.modell.metamodellglobal.attribute.AttJaNein;
import de.bsvrz.sys.funclib.bitctrl.modell.metamodellglobal.objekte.Attributgruppe;
import de.bsvrz.sys.funclib.bitctrl.modell.systemmodellaoe.objekte.AutarkeOrganisationsEinheit;
import de.bsvrz.sys.funclib.bitctrl.modell.systemmodellglobal.objekte.Applikation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.objekte.Anzeige;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.onlinedaten.OdAnzeigeEigenschaftIst;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttFahrStreifenLage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdFahrStreifen;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.FahrStreifen;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitGeglaettetFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitGeglaettetMq;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitMq;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitTrendExtraPolationFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitTrendExtraPolationMq;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdVerkehrsLageVerfahren1;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AtlDatenSpezifikationen;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AtlErzeugeSimulationAnfrage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AtlErzeugeSimulationAntwort;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AtlOfflineSimulationsZeitbereich;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AtlParameterSatzSimulationsdaten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AtlParameterSpezifikationen;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AttErzeugeSimulationFehler;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AttErzeugeSimulationZustand;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AttSimulationsArt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AttSimulationsZustand;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.OfflineSimulation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.SimulationsStrecke;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.onlinedaten.OdSimulationVerwaltungAnfrage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.onlinedaten.OdSimulationVerwaltungAntwort;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.onlinedaten.OdSimulationsSteuerungOffline;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.parameter.PdSimulationsStreckenBeschreibung;
import de.bsvrz.sys.funclib.bitctrl.modell.util.KappichModellUtil;
import de.bsvrz.sys.funclib.dynobj.DynObjektException;
import de.bsvrz.sys.funclib.dynobj.DynamischeObjekte;

/**
 * Job, welcher ein Dynamisches Objekt vom Typ
 * <code>typ.simulationsStrecke</code> erzeugt, wartet, bis die Parametrierung
 * die Default-Parameterstze versendet hat und dann das Objekt in eine Menge
 * einfgt. Danach wird noch der Name des dynamischen Objektes, den der Nutzer
 * eingegeben hat, in das Feld "Name" der Attributgruppe
 * {@link PdSimulationsStreckenBeschreibung} eingefgt und der Parametersatz
 * (mit Urlasser undefiniert!) versendet.
 * 
 * 
 * @author BitCtrl Systems GmbH, Albrecht Uhlmann / Christian Hoesel
 * @version $Id: SimulationErzeugenUndStartenJob.java 29510 2011-06-09 13:45:24Z
 *          hoesel $
 * 
 */
public class SimulationErzeugenUndStartenJob extends Job implements
		DatensatzUpdateListener {

	/**
	 * Die Daten des Parametersatzes, der den Namen enthlt.
	 */
	private PdSimulationsStreckenBeschreibung.Daten pdDatum;

	/**
	 * Der {@link StreckenprofilEditor}, der Daten der Offline Simulation
	 * verwenden soll.
	 */
	private final StreckenprofilEditor streckenprofilEditor;

	/**
	 * Der Rckgabewert des Jobs.
	 */
	private IStatus status = Status.OK_STATUS;

	/**
	 * Die erzeugte {@link OfflineSimulation}.
	 */
	private OfflineSimulation simulation;

	/**
	 * Die Simulationsgeschwindigkeit. Default 1.
	 */
	private double simulationsGeschwindigkeit = 1;

	/**
	 * Kennzeichnung des {@link Job}s fr die Simulationsverwaltung.
	 */
	private final String simuZeichen = "StreckenprofilOfflineSimulation_"
			+ System.currentTimeMillis();

	/**
	 * Der Startzeitpunkt der Offline Simulation.
	 */
	private final Zeitstempel startzeitpunkt;

	/**
	 * Der Endzeitpunkt der Offline Simulation.
	 */
	private final Zeitstempel endZeitpunkt;

	/**
	 * Die {@link Strecke} der Streckenprofils durch die der Name der
	 * Simulationsstrecke bestimmt wird.
	 */
	private final Strecke strecke;

	/**
	 * Menge aller {@link MessQuerschnitt}e, die die {@link SimulationsStrecke}
	 * enthalten soll.
	 */
	private final Collection<MessQuerschnitt> mqMenge = new ArrayList<MessQuerschnitt>();

	/**
	 * Menge aller {@link AnzeigeQuerschnitt}e, die die
	 * {@link SimulationsStrecke} enthalten soll.
	 */
	private final Collection<AnzeigeQuerschnitt> aqMenge = new ArrayList<AnzeigeQuerschnitt>();

	/**
	 * Konstruktor bestimmt nur den Namen des Jobs und merkt sich den Namen der
	 * anzulegenden Simulationsstrecke.
	 * 
	 * @param streckenprofilEditor
	 *            der {@link StreckenprofilEditor}, der Daten der Offline
	 *            Simulation verwenden soll.
	 * @param model
	 *            das Streckenprofil der anzulegenden Simulationsstrecke
	 */
	public SimulationErzeugenUndStartenJob(
			final StreckenprofilEditor streckenprofilEditor, final Modell model) {
		super("Streckenprofil-Simulation" + " anlegen ...");
		Assert.isNotNull(model.getErsteStrecke().getName(),
				"Es wurde kein Streckenprofil fr die Simulation bestimmt.");
		Assert.isNotNull(streckenprofilEditor,
				"Ungltiger Editor, fr die Erstellung einer Streckenprofil - Simulation.");
		this.streckenprofilEditor = streckenprofilEditor;

		startzeitpunkt = new Zeitstempel(model.getChartProperties()
				.getHistorieStartZeitpunkt().getTime());

		endZeitpunkt = new Zeitstempel(model.getChartProperties()
				.getHistorieEndZeitpunkt().getTime());

		strecke = model.getErsteStrecke();

		mqMenge.addAll(model.getAlleMessquerschnitte(false));

		aqMenge.addAll(model.getAlleAnzeigequerschnitte());

	}

	@Override
	protected IStatus run(final IProgressMonitor monitor) {

		Assert.isTrue(
				startzeitpunkt.getTime() < endZeitpunkt.getTime(),
				"Der Startzeitpunkt fr eine historische Abfrage mu vom dem Endzeitpunkt der selbigen. liegen.");

		Assert.isTrue(!mqMenge.isEmpty(),
				"Innerhalb des gewhlten Streckenzugs, sind kein Messquerschnitte aktiv.");

		monitor.beginTask("Streckenprofil-Offline-Simulation anlegen...",
				IProgressMonitor.UNKNOWN);
		try {
			monitor.subTask("Simulationsstrecke erstellen ...");
			final SimulationsStrecke modellObjekt = erzeugeSimulationsStrecke();
			monitor.subTask("Simulation erstellen ...");
			erzeugeSimulation(modellObjekt);
			if (simulation == null) {
				status = new Status(
						IStatus.ERROR,
						Activator.PLUGIN_ID,
						"Die Simulationsverwaltung konnte kein Simulationsobjekt anlegen.",
						null);
			} else {
				try {
					starteSimulation(monitor);
				} catch (final InterruptedException e) {
					e.printStackTrace();
				}

			}
			status = Status.OK_STATUS;
		} catch (final DynObjektException e) {
			final String msg = "Das Erzeugen einer neuen Simulationsstrecke ist fehlgeschlagen.";
			Activator.getDefault().getLogger().warning(msg, e);
			status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, msg, e);
		} catch (final AnmeldeException e) {
			final String msg = "Anmeldung als Sender fr die Simulationsverwaltung fehlgeschlagen.";
			Activator.getDefault().getLogger().warning(msg, e);
			status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, msg, e);
		} catch (final DatensendeException e) {
			final String msg = "Ein Datensatz konnte nicht an die Simulationsverwaltung versendet werden.";
			Activator.getDefault().getLogger().warning(msg, e);
			status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, msg, e);
		}
		monitor.done();

		return status;
	}

	/**
	 * Startet die Offline Simulation.
	 * 
	 * @param monitor
	 *            der {@link IProgressMonitor} des {@link Job}s.
	 * @throws AnmeldeException
	 *             die Anmeldung als Sender fr die Simulationssteuerung war
	 *             nicht erfolgreich.
	 * @throws DatensendeException
	 *             das Senden eines neuen Simulationszustandes an die
	 *             Simulationsverwatung war nicht erfolgreich.
	 * @throws InterruptedException
	 *             das Warten auf neue Simulationszustnde wurde unterbrochen.
	 */
	private void starteSimulation(final IProgressMonitor monitor)
			throws AnmeldeException, DatensendeException, InterruptedException {

		final Aspekt asp = OdSimulationsSteuerungOffline.Aspekte.Vorgabe;
		final OfflineSimulation sim = simulation;
		final OdSimulationsSteuerungOffline simSteuerung = sim
				.getOdSimulationsSteuerungOffline();
		simSteuerung.anmeldenSender(asp);
		final OdSimulationsSteuerungOffline.Daten datum = simSteuerung
				.createDatum();

		datum.setSimulationsgeschwindigkeit(1d);
		datum.dSetZeitstempel(new Zeitstempel());

		final long wartezeit = 500L;
		int i = 0;

		while (AttSimulationsZustand.ZUSTAND_0_NEU.equals(simSteuerung
				.getDatum(OdSimulationsSteuerungOffline.Aspekte.Zustand)
				.getSimulationsZustand())) {
			if (monitor.isCanceled()) {
				final SimulationBeendenUndVernichtenJob j = new SimulationBeendenUndVernichtenJob(
						simulation);
				j.schedule();
				return;
			}
			Thread.sleep(wartezeit);
			i++;
			datum.setSimulationsZustand(AttSimulationsZustand.ZUSTAND_1_VORSTART);
			datum.setSimulationsgeschwindigkeit(simulationsGeschwindigkeit);
			simSteuerung.sendeDatum(asp, datum);
		}

		while (AttSimulationsZustand.ZUSTAND_1_VORSTART.equals(simSteuerung
				.getDatum(OdSimulationsSteuerungOffline.Aspekte.Zustand)
				.getSimulationsZustand())) {
			if (monitor.isCanceled()) {
				final SimulationBeendenUndVernichtenJob j = new SimulationBeendenUndVernichtenJob(
						simulation);
				j.schedule();
				return;
			}
			Thread.sleep(wartezeit);
			i++;
			datum.setSimulationsZustand(AttSimulationsZustand.ZUSTAND_2_START);
			datum.setSimulationsgeschwindigkeit(simulationsGeschwindigkeit);
			simSteuerung.sendeDatum(asp, datum);
		}

		Activator
				.getDefault()
				.getLogger()
				.info("Nach eine Anlaufzeit von "
						+ i
						* wartezeit
						+ " ms, ist die Simulation fr den Streckenprofileditor erfolgreich gestartet.");

		simSteuerung.abmeldenSender(asp);

		streckenprofilEditor.setAktuelleSimulation(simulation);

	}

	/**
	 * Erzeugt eine neue {@link OfflineSimulation} anhand der bergebenen
	 * {@link SimulationsStrecke}.
	 * 
	 * @param simulationsStrecke
	 *            die Simulationsstrecke.
	 * @throws AnmeldeException
	 *             die Anmeldung als Sender fr die Simulationsverwaltung ist
	 *             fehlgeschlagen.
	 * @throws DatensendeException
	 *             das Senden von Daten an die Simulationsverwaltung war nicht
	 *             erfolgreich.
	 */
	private void erzeugeSimulation(final SimulationsStrecke simulationsStrecke)
			throws AnmeldeException, DatensendeException {

		final Applikation app = (Applikation) Activator
				.getDefault()
				.getObjektFactory()
				.getModellobjekt(
						Activator.getDefault().getObjektFactory().getDav()
								.getLocalApplicationObject());
		app.getOdSimulationVerwaltungAntwort().setSenke(
				OdSimulationVerwaltungAntwort.Aspekte.Antwort, true);
		app.getOdSimulationVerwaltungAntwort().addUpdateListener(
				OdSimulationVerwaltungAntwort.Aspekte.Antwort, this);

		final AutarkeOrganisationsEinheit sv = KappichModellUtil
				.getAOE(Activator.getDefault().getObjektFactory());
		final OdSimulationVerwaltungAnfrage anfrage = sv
				.getOdSimulationVerwaltungAnfrage();
		anfrage.anmeldenSender(OdSimulationVerwaltungAnfrage.Aspekte.Anfrage);
		final OdSimulationVerwaltungAnfrage.Daten datum = anfrage.createDatum();

		final AtlErzeugeSimulationAnfrage erzeugeAnfrage = new AtlErzeugeSimulationAnfrage();
		erzeugeAnfrage.setName("StreckenprofilSimu"
				+ System.currentTimeMillis());
		erzeugeAnfrage.setReferenzSimulationsStrecke(simulationsStrecke);
		erzeugeAnfrage.setSimulationsArt(AttSimulationsArt.ZUSTAND_2_OFFLINE);
		final AtlOfflineSimulationsZeitbereich zeitBereich = new AtlOfflineSimulationsZeitbereich();
		zeitBereich.setSimulationsBeginn(startzeitpunkt);
		zeitBereich.setSimulationsEnde(endZeitpunkt);
		erzeugeAnfrage.getOfflineSimulationsZeitbereich().add(zeitBereich);
		datum.getErzeugeSimulationAnfrage().add(erzeugeAnfrage);
		datum.setAbsenderId(KappichModellUtil.getApplikation(Activator
				.getDefault().getObjektFactory()));
		datum.setAbsenderZeichen(simuZeichen);
		datum.dSetZeitstempel(new Zeitstempel());

		anfrage.sendeDatum(OdSimulationVerwaltungAnfrage.Aspekte.Anfrage, datum);
		anfrage.abmeldenSender(OdSimulationVerwaltungAnfrage.Aspekte.Anfrage);

		long remainingWaitTime = 3000L;
		final long endTime = System.currentTimeMillis() + remainingWaitTime;
		synchronized (this) {
			while (simulation == null
					&& System.currentTimeMillis() < (endTime - 50)) {
				try {
					wait(remainingWaitTime);
				} catch (final InterruptedException e) {
					Logger.getLogger(getClass().getName()).log(Level.WARNING,
							e.getLocalizedMessage(), e);
					status = new Status(IStatus.WARNING, Activator.PLUGIN_ID,
							e.getLocalizedMessage(), e);
					remainingWaitTime = endTime - System.currentTimeMillis();
				}
			}
		}
		app.getOdSimulationVerwaltungAntwort().removeUpdateListener(
				OdSimulationVerwaltungAntwort.Aspekte.Antwort, this);

	}

	/**
	 * Erzeugt eine neue {@link SimulationsStrecke}.
	 * 
	 * @return die neue {@link SimulationsStrecke}.
	 * @throws DynObjektException
	 *             ein neues {@link DynObjektException} fr die
	 *             {@link SimulationsStrecke} konnte nicht angelegt werden.
	 * @throws AnmeldeException
	 *             Anmeldung als Sender fr die Simulationsbeschreibung war
	 *             nicht erfolgreich.
	 * @throws DatensendeException
	 *             Die Simulationsbeschreibung der {@link SimulationsStrecke}
	 *             konnte nicht versendet werden.
	 */
	private SimulationsStrecke erzeugeSimulationsStrecke()
			throws DynObjektException, AnmeldeException, DatensendeException {
		PdSimulationsStreckenBeschreibung pdb = null;
		final String pid = DavTools.generierePID(
				strecke.getName() + System.currentTimeMillis(),
				"Streckenprofil Simulation");
		final DynamischeObjekte dynObjektVerwaltung = DynamischeObjekte
				.getInstanz(Activator.getDefault().getObjektFactory().getDav());
		final DynamicObjectType dot = (DynamicObjectType) Activator
				.getDefault().getObjektFactory().getDav().getDataModel()
				.getType(SimulationsStrecke.PID);
		final DynamicObject dynobjekt = dynObjektVerwaltung.erzeugeObjekt(dot,
				strecke.getName(), pid);
		final SimulationsStrecke modellObjekt = (SimulationsStrecke) Activator
				.getDefault().getObjektFactory().getModellobjekt(dynobjekt);

		pdb = modellObjekt.getPdSimulationsStreckenBeschreibung();

		long remainingWaitTime = 60000L;
		long endTime = System.currentTimeMillis() + remainingWaitTime;
		boolean waitRegularReturn = false;
		synchronized (this) {
			pdb.addUpdateListener(SimulationErzeugenUndStartenJob.this);
			while (!waitRegularReturn
					&& System.currentTimeMillis() < (endTime - 50)) {
				try {
					wait(remainingWaitTime);
					waitRegularReturn = true;
				} catch (final InterruptedException e) {
					Logger.getLogger(getClass().getName()).log(Level.WARNING,
							e.getLocalizedMessage(), e);
					status = new Status(IStatus.WARNING, Activator.PLUGIN_ID,
							e.getLocalizedMessage(), e);
					remainingWaitTime = endTime - System.currentTimeMillis();

				}
			}
		}

		final MutableSet menge = Activator.getDefault().getObjektFactory()
				.getDav().getLocalConfigurationAuthority()
				.getMutableSet("SimulationsStrecken");
		dynObjektVerwaltung.fuegeObjektInMengeEin(dynobjekt, menge);
		if (pdDatum != null) {
			pdDatum.setName(strecke.getName());
			pdDatum.getStartInfo().add(
					Activator.getDefault().ermittleSimulationStartInfo());
			pdb.anmeldenSender();
			pdb.sendeDatum(pdDatum);
			remainingWaitTime = 60000L;
			endTime = System.currentTimeMillis() + remainingWaitTime;
			synchronized (this) {
				while (!waitRegularReturn
						&& System.currentTimeMillis() < (endTime - 50)) {
					try {
						wait(remainingWaitTime);
						waitRegularReturn = true;
					} catch (final InterruptedException e) {
						Logger.getLogger(getClass().getName()).log(
								Level.WARNING, e.getLocalizedMessage(), e);
						status = new Status(IStatus.WARNING,
								Activator.PLUGIN_ID, e.getLocalizedMessage(), e);
						remainingWaitTime = endTime
								- System.currentTimeMillis();
					}
				}
			}

			final AtlParameterSatzSimulationsdaten d = new AtlParameterSatzSimulationsdaten();
			pdDatum.getParameterSatz().add(d);

			final AtlParameterSpezifikationen param = new AtlParameterSpezifikationen();
			param.getAttributGruppe().add(
					(Attributgruppe) Activator.getDefault().getObjektFactory()
							.getModellobjekt(PdVerkehrsLageVerfahren1.PID));

			d.getEinstellungen().setParametrieren(AttJaNein.ZUSTAND_1_JA);
			final Feld<AtlDatenSpezifikationen> eingangsDatenFeld = d
					.getEingangsdaten();
			final AtlDatenSpezifikationen eingangsDatenSpezifikationMQ = new AtlDatenSpezifikationen();
			final AtlDatenSpezifikationen eingangsDatenSpezifikationFS = new AtlDatenSpezifikationen();
			final AtlDatenSpezifikationen eingangsDatenSpezifikationAQ = new AtlDatenSpezifikationen();

			for (final MessQuerschnitt modelMq : mqMenge) {
				eingangsDatenSpezifikationMQ.getObjekt().add(
						modelMq.getMessQuerschnitt().getSystemObject());
				param.getObjekt().add(
						modelMq.getModellObjekt().getSystemObject());
				final MessQuerschnittAllgemein mqAllgemein = modelMq
						.getMessQuerschnitt();
				if (mqAllgemein instanceof de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnitt) {
					for (final FahrStreifen f : ((de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnitt) mqAllgemein)
							.getFahrStreifen()) {
						eingangsDatenSpezifikationFS.getObjekt().add(
								f.getSystemObject());
						param.getObjekt().add(f.getSystemObject());
					}
				}
			}

			if (!eingangsDatenSpezifikationMQ.getObjekt().isEmpty()) {
				eingangsDatenFeld.add(eingangsDatenSpezifikationMQ);
			}
			if (!eingangsDatenSpezifikationFS.getObjekt().isEmpty()) {
				eingangsDatenFeld.add(eingangsDatenSpezifikationFS);
			}
			if (!param.getObjekt().isEmpty()) {
				d.getDatenSpezifikation().add(param);
			}

			eingangsDatenSpezifikationFS.getAttributGruppe().add(
					(Attributgruppe) Activator.getDefault().getObjektFactory()
							.getModellobjekt(OdVerkehrsDatenKurzZeitFs.PID));
			eingangsDatenSpezifikationFS.getAttributGruppe().add(
					(Attributgruppe) Activator
							.getDefault()
							.getObjektFactory()
							.getModellobjekt(
									OdVerkehrsDatenKurzZeitGeglaettetFs.PID));
			eingangsDatenSpezifikationFS
					.getAttributGruppe()
					.add((Attributgruppe) Activator
							.getDefault()
							.getObjektFactory()
							.getModellobjekt(
									OdVerkehrsDatenKurzZeitTrendExtraPolationFs.PID));

			eingangsDatenSpezifikationMQ.getAttributGruppe().add(
					(Attributgruppe) Activator
							.getDefault()
							.getObjektFactory()
							.getModellobjekt(
									OdVerkehrsDatenKurzZeitGeglaettetMq.PID));
			eingangsDatenSpezifikationMQ.getAttributGruppe().add(
					(Attributgruppe) Activator.getDefault().getObjektFactory()
							.getModellobjekt(OdVerkehrsDatenKurzZeitMq.PID));
			eingangsDatenSpezifikationMQ
					.getAttributGruppe()
					.add((Attributgruppe) Activator
							.getDefault()
							.getObjektFactory()
							.getModellobjekt(
									OdVerkehrsDatenKurzZeitTrendExtraPolationMq.PID));

			// wir suchen alle anzeigen, die auf der Hauptfahrbahn liegen.
			for (final AnzeigeQuerschnitt aq : aqMenge) {
				for (final Anzeige anz : aq.getModellObjekt().getAnzeigen()) {
					for (final FahrStreifen fs : anz.getFahrStreifen()) {
						final KdFahrStreifen.Daten fsDaten = fs
								.getKdFahrStreifen().getDatum();
						if (fsDaten != null) {
							if (AttFahrStreifenLage.ZUSTAND_0_HFS
									.equals(fsDaten.getLage())) {
								eingangsDatenSpezifikationAQ.getObjekt().add(
										anz);
							}
						}
					}
				}
			}
			if (!eingangsDatenSpezifikationAQ.getObjekt().isEmpty()) {
				eingangsDatenFeld.add(eingangsDatenSpezifikationAQ);
			}
			eingangsDatenSpezifikationAQ.getAttributGruppe().add(
					(Attributgruppe) Activator.getDefault().getObjektFactory()
							.getModellobjekt(OdAnzeigeEigenschaftIst.PID));

			// pdbDatum.getUrlasser().setBenutzerReferenz(b);
			// pdbDatum.getUrlasser().setUrsache(ursache);
			// pdbDatum.getUrlasser().setVeranlasser(info.getVeranlasser());
			pdDatum.dSetZeitstempel(new Zeitstempel());
			pdb.anmeldenSender();
			pdb.sendeDatum(pdDatum);
			pdb.abmeldenSender();
		}

		pdb.removeUpdateListener(SimulationErzeugenUndStartenJob.this);

		return modellObjekt;

	}

	@Override
	public void datensatzAktualisiert(final DatensatzUpdateEvent event) {
		synchronized (this) {
			final OnlineDatum datum = event.getDatum();
			if (datum instanceof PdSimulationsStreckenBeschreibung.Daten
					&& datum.dContainsDaten()) {
				pdDatum = (PdSimulationsStreckenBeschreibung.Daten) event
						.getDatum();
				notifyAll();
			} else if (datum instanceof OdSimulationVerwaltungAntwort.Daten
					&& datum.dContainsDaten()) {
				final String meinZeichen = ((OdSimulationVerwaltungAntwort.Daten) datum)
						.getAbsenderZeichen();
				if (simuZeichen.equals(meinZeichen)) {
					// die Nachricht ist fr mich
					for (final AtlErzeugeSimulationAntwort antwort : ((OdSimulationVerwaltungAntwort.Daten) datum)
							.getErzeugeSimulationAntwort()) {
						if (AttErzeugeSimulationZustand.ZUSTAND_0_ERZEUGT
								.equals(antwort.getErzeugeSimulationZustand())
								&& AttErzeugeSimulationFehler.ZUSTAND_0_KEIN_FEHLER
										.equals(antwort
												.getErzeugeSimulationFehler())) {
							simulation = (OfflineSimulation) antwort
									.getReferenzErzeugteSimulation();
							notifyAll();
							break;
						} else {
							// Fehler
							simulation = null;
							break;
						}
					}
				}
			}
		}
	}

	/**
	 * Liefert die Simulationsgeschwindigkeit.
	 * 
	 * @return the simulationsGeschwindigkeit
	 */
	public final double getSimulationsGeschwindigkeit() {
		return simulationsGeschwindigkeit;
	}

	/**
	 * Setzt die Simulationsgeschwindigkeit.
	 * 
	 * @param simulationsGeschwindigkeit
	 *            the simulationsGeschwindigkeit to set
	 */
	public final void setSimulationsGeschwindigkeit(
			final double simulationsGeschwindigkeit) {

		this.simulationsGeschwindigkeit = simulationsGeschwindigkeit;
	}
}
