/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.histdata;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.MessQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypDatenQuellen;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypPrognoseTyp;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.objekte.Anzeige;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.onlinedaten.OdAnzeigeEigenschaftIst;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.attribute.AttFahrStreifenLage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.FahrStreifen;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitGeglaettetFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitGeglaettetMq;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitMq;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitTrendExtraPolationFs;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdVerkehrsDatenKurzZeitTrendExtraPolationMq;

/**
 * Datenzusammenstellung Default-Implementierung fr Streckenprofil-Modell 
 * 
 * @author Sans
 *
 */
public class DatenZusammenstellung implements IDatenZusammenstellung
{
  public DatenZusammenstellung(Modell model) {
   
    // Auswerten Prognosetyp (Wert aus Aspekt-Combobox)
    String atgPidMq;
    String aspPidMq;
    String atgPidFs;
    String aspPidFs;
    TypPrognoseTyp prognoseTyp = model.getChartProperties().getTypPrognoseTyp();
    switch (prognoseTyp) {
      case ANALYSE:
        atgPidMq = OdVerkehrsDatenKurzZeitMq.PID;
        aspPidMq = OdVerkehrsDatenKurzZeitFs.Aspekte.Analyse.getPid();
        atgPidFs = OdVerkehrsDatenKurzZeitFs.PID;
        aspPidFs = OdVerkehrsDatenKurzZeitFs.Aspekte.Analyse.getPid();
        break;
      case FLINK_GEGLAETTET:
        atgPidMq = OdVerkehrsDatenKurzZeitGeglaettetMq.PID;
        aspPidMq = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseFlink.getPid();
        atgPidFs = OdVerkehrsDatenKurzZeitGeglaettetFs.PID;
        aspPidFs = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseFlink.getPid();
        break;
      case NORMAL_GEGLAETTET:
        atgPidMq = OdVerkehrsDatenKurzZeitGeglaettetMq.PID;
        aspPidMq = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseNormal.getPid();
        atgPidFs = OdVerkehrsDatenKurzZeitGeglaettetFs.PID;
        aspPidFs = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseNormal.getPid();
        break;
      case TRAEGE_GEGLAETTET:
        atgPidMq = OdVerkehrsDatenKurzZeitGeglaettetMq.PID;
        aspPidMq = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseTraege.getPid();
        atgPidFs = OdVerkehrsDatenKurzZeitGeglaettetFs.PID;
        aspPidFs = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseTraege.getPid();
        break;
      case FLINK_EXTRA_POLATION:
        atgPidMq = OdVerkehrsDatenKurzZeitTrendExtraPolationMq.PID;
        aspPidMq = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseFlink.getPid();
        atgPidFs = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.PID;
        aspPidFs = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseFlink.getPid();
        break;
      case NORMAL_EXTRA_POLATION:
        atgPidMq = OdVerkehrsDatenKurzZeitTrendExtraPolationMq.PID;
        aspPidMq = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseNormal.getPid();
        atgPidFs = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.PID;
        aspPidFs = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseNormal.getPid();
        break;
      case TRAEGE_EXTRA_POLATION:
        atgPidMq = OdVerkehrsDatenKurzZeitTrendExtraPolationMq.PID;
        aspPidMq = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseTraege.getPid();
        atgPidFs = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.PID;
        aspPidFs = OdVerkehrsDatenKurzZeitTrendExtraPolationFs.Aspekte.PrognoseTraege.getPid();
        break;
      default:
          throw new IllegalStateException("Nicht untersttzter Prognosetyp");
    }
   
    List<SystemObject> mqList = new ArrayList<SystemObject>();
    List<SystemObject> fsList = new ArrayList<SystemObject>();
    
    for (MessQuerschnitt modelMq : model.getAlleMessquerschnitte(true)) {
      
      for (LinienEigenschaften lp : model.getLineProperties()) {
        if (TypDatenQuellen.MQ.equals(lp.getTypDatenQuelle())) {
          mqList.add(modelMq.getModellObjekt().getSystemObject());
//        System.out.println("MQ " + modelMq.getModellObjekt().getSystemObject().getPid() + " hinzugefgt");
        }
      }
      MessQuerschnittAllgemein mqAllgemein = modelMq.getMessQuerschnitt();
      if (mqAllgemein instanceof de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnitt) {
        for (FahrStreifen fs : ((de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnitt) mqAllgemein).getFahrStreifen()) {
          AttFahrStreifenLage fsLage = fs.getKdFahrStreifen().getDatum().getLage();
          for (final LinienEigenschaften lp : model.getLineProperties()) {
            if ((AttFahrStreifenLage.ZUSTAND_0_HFS.equals(fsLage) && TypDatenQuellen.HFS
                .equals(lp.getTypDatenQuelle()))
                || (AttFahrStreifenLage.ZUSTAND_1__1UEFS
                    .equals(fsLage) && TypDatenQuellen.UEFS1
                    .equals(lp.getTypDatenQuelle()))
                || (AttFahrStreifenLage.ZUSTAND_2__2UEFS
                    .equals(fsLage) && TypDatenQuellen.UEFS2
                    .equals(lp.getTypDatenQuelle()))
                || (AttFahrStreifenLage.ZUSTAND_3__3UEFS
                    .equals(fsLage) && TypDatenQuellen.UEFS3
                    .equals(lp.getTypDatenQuelle()))
                || (AttFahrStreifenLage.ZUSTAND_4__4UEFS
                    .equals(fsLage) && TypDatenQuellen.UEFS4
                    .equals(lp.getTypDatenQuelle()))
                || (AttFahrStreifenLage.ZUSTAND_5__5UEFS
                    .equals(fsLage) && TypDatenQuellen.UEFS5
                    .equals(lp.getTypDatenQuelle()))
                || (AttFahrStreifenLage.ZUSTAND_6__6UEFS
                    .equals(fsLage) && TypDatenQuellen.UEFS6
                    .equals(lp.getTypDatenQuelle()))) {
              
              fsList.add(fs.getSystemObject());
    //          System.out.println("FS " + f.getSystemObject().getPid() + " hinzugefgt");
            }
          }
        }
      }
    }
    if (!mqList.isEmpty()) {
      Aspect asp = (Aspect)Activator.getDefault().getObjektFactory().getModellobjekt(aspPidMq).getSystemObject();
      AttributeGroup atg = (AttributeGroup)Activator.getDefault().getObjektFactory().getModellobjekt(atgPidMq).getSystemObject();
      DataDescription dd = new DataDescription(atg, asp);
      Collection<SystemObject> objekte  = daten.get(dd);
      if (objekte == null) {
        objekte = new ArrayList<SystemObject>();
        daten.put(dd, objekte);
      }
      objekte.addAll(mqList);
    }
    if (!fsList.isEmpty()) {
      Aspect asp = (Aspect)Activator.getDefault().getObjektFactory().getModellobjekt(aspPidFs).getSystemObject();
      AttributeGroup atg = (AttributeGroup)Activator.getDefault().getObjektFactory().getModellobjekt(atgPidFs).getSystemObject();
      DataDescription dd = new DataDescription(atg, asp);
      Collection<SystemObject> objekte  = daten.get(dd);
      if (objekte == null) {
        objekte = new ArrayList<SystemObject>();
        daten.put(dd, objekte);
      }
      objekte.addAll(fsList);
    }
          
//    SbaAnzeigeDaten sbaAnzeigeDaten = new SbaAnzeigeDaten();
    
    List<SystemObject> anzList = new ArrayList<SystemObject>();
    for (AnzeigeQuerschnitt aq : model.getAlleAnzeigequerschnitte()) {
      for (Anzeige anz : aq.getModellObjekt().getAnzeigen()) {
        
        SystemObject anzeige = anz.getSystemObject();
        
        anzList.add(anzeige);
//        System.out.println("ANZ " + anzeige.getName() + " hinzugefgt");
        
//        Collection<FahrStreifen> fahrStreifen = anz.getFahrStreifen();
//        if (fahrStreifen != null) {
//          for (FahrStreifen fs : fahrStreifen) {
//            KdFahrStreifen.Daten fsDaten = fs.getKdFahrStreifen().getDatum();
//            if (fsDaten != null) {
//              if (AttFahrStreifenLage.ZUSTAND_0_HFS.equals(fsDaten.getLage())) {
//                anzList.add(anzeige);
////                System.out.println("ANZ " + anzeige.getPid() + " hinzugefgt");
//              }
//            }
//          }  
//        }
//        else {
//          Data data = sbaAnzeigeDaten.get(anzeige);
//          if (data != null) {
//            Array array = data.getArray("Lage");
//            for (int i = 0; i < array.getLength(); i++) {
//              if ("HFS".equals(array.getItem(i).asTextValue().getText())) {
//                anzList.add(anzeige);
////                System.out.println("ANZ " + anzeige.getPid() + " hinzugefgt");
//              }
//            }
//          }
//          else {
//            Activator.getDefault().getLogger().warning("Fr Anzeige " + 
//                anzeige.getPid() + " ist keine Fahrstreifenzuordnung versorgt");            
//          }
//        }
      }
    }
    if (!anzList.isEmpty()) {
      Aspect asp = (Aspect)Activator.getDefault().getObjektFactory().getModellobjekt("asp.aqZustand").getSystemObject();
      AttributeGroup atg = (AttributeGroup)Activator.getDefault().getObjektFactory().getModellobjekt(OdAnzeigeEigenschaftIst.PID).getSystemObject();
      DataDescription dd = new DataDescription(atg, asp);
      Collection<SystemObject> objekte  = daten.get(dd);
      if (objekte == null) {
        objekte = new ArrayList<SystemObject>();
        daten.put(dd, objekte);
      }
      objekte.addAll(anzList);  
    }
    
//    System.out.println("Datenzusammenstellung");
//    if (!daten.isEmpty()) {
//      for (DataDescription dd: daten.keySet()) {
//        System.out.println("\t" + dd);
//        for (SystemObject so: daten.get(dd)) {
//          System.out.println("\t\t" + so.getName());
//        }
//      }
//    }
//    else {
//      System.out.println("\t<EMPTY>");
//    }
  }
     
  private  Map<DataDescription, Collection<SystemObject>> daten =
    new HashMap<DataDescription, Collection<SystemObject>>();

  @Override
  public Map<DataDescription, Collection<SystemObject>> getDaten()
  {
    return Collections.unmodifiableMap(daten);
  }

}
