/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.histdata;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Stellt die Datenstze der ATG SbaAnzeige von SbaAnzeigen unter dem Schlssel der zugehrigen 
 * Anzeige zur Verfgung
 *  
 * @author Sans
 *
 */
public class SbaAnzeigeDaten
{
  private Map<SystemObject, Data> sbaAnzeigen = new HashMap<SystemObject, Data>();
  
  public SbaAnzeigeDaten() {
    ClientDavInterface connection = Activator.getDefault().getObjektFactory().getDav();
    if (connection != null) {
      DataModel dataModel = connection.getDataModel();
      SystemObjectType type = dataModel.getType("typ.sswSbaAnzeige");
      if (type != null) {
        for (SystemObject so: type.getElements()) {
          if (so != null && so.isValid()) {
            AttributeGroup atg = dataModel.getAttributeGroup("atg.sswSbaAnzeige");
            if (atg != null) {
              Data data = so.getConfigurationData(atg);
              if (data != null) {
                SystemObject anzeige = data.getItem("Anzeige").asReferenceValue().getSystemObject();
                if (anzeige != null) {
                  sbaAnzeigen.put(anzeige, data);
                }
              }
            }
          }
        }
      }
    }
  }
  
  public Data get(SystemObject anzeige) {
    return sbaAnzeigen.get(anzeige);
  }
}
