/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.adapter;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

/**
 * Adapter der ein "derived" Attribut jedes Mal neu berechnet, wenn sich ein
 * entsprechendes "navigation" oder "dependant" Feature aus der Objekthierachie
 * gendert hat, fr das der Adapter registriert ist.
 * 
 * @author BitCtrl Systems GmbH, Schnepel / Hoesel
 * @version $Id: SetDerivedAttributeAdapter.java,v 1.3 2014/04/17 15:51:19 Pittner Exp $
 */
public abstract class SetDerivedAttributeAdapter extends
		AbstractDependantAttributeAdapter {

	/**
	 * Das {@link EStructuralFeature}, das im Ereignisfall neu berechnet wird.
	 */
	private final EStructuralFeature derivedFeature;

	/**
	 * Konstruktor des {@link SetDerivedAttributeAdapter}s.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param derivedFeature
	 *            das {@link EStructuralFeature} eines Attributes, das als
	 *            "derived" im EMF Modell markiert ist und dessen Wert neu
	 *            berechent wird.
	 * @param navigationFeature
	 *            das {@link EStructuralFeature} an dem einem {@link EObject},
	 *            das wiederum andere {@link EObject}s als Kinder hat, und auf
	 *            dessen nderungen reagiert werden soll.
	 * @param dependantFeature
	 *            das {@link EStructuralFeature} eines Kindobjektes, auf das bei
	 *            nderungen reagiert werden soll.
	 * @param localFeature
	 *            das {@link EStructuralFeature} auf das an dem Wurzelobjekt der
	 *            Objekthierarchie reagiert wird.
	 */
	public SetDerivedAttributeAdapter(final EObject source,
			final EStructuralFeature derivedFeature,
			final EStructuralFeature navigationFeature,
			final EStructuralFeature dependantFeature,
			final EStructuralFeature localFeature) {
		this(source, derivedFeature);
		addNavigatedDependency(navigationFeature, dependantFeature);
		addLocalDependency(localFeature);
	}

	/**
	 * Konstruktor des {@link SetDerivedAttributeAdapter}s.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param derivedFeature
	 *            das {@link EStructuralFeature} eines Attributes, das als
	 *            "derived" im EMF Modell markiert ist und dessen Wert neu
	 *            berechent wird.
	 * @param navigationFeature
	 *            das {@link EStructuralFeature} an dem einem {@link EObject},
	 *            das wiederum andere {@link EObject}s als Kinder hat, und auf
	 *            dessen nderungen reagiert werden soll.
	 * @param dependantFeature
	 *            das {@link EStructuralFeature} eines Kindobjektes, auf das bei
	 *            nderungen reagiert werden soll.
	 */
	public SetDerivedAttributeAdapter(final EObject source,
			final EStructuralFeature derivedFeature,
			final EStructuralFeature navigationFeature,
			final EStructuralFeature dependantFeature) {
		this(source, derivedFeature);
		addNavigatedDependency(navigationFeature, dependantFeature);
	}

	/**
	 * Konstruktor des {@link SetDerivedAttributeAdapter}s.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param derivedFeature
	 *            das {@link EStructuralFeature} eines Attributes, das als
	 *            "derived" im EMF Modell markiert ist und dessen Wert neu
	 *            berechent wird.
	 * @param localFeature
	 *            das {@link EStructuralFeature} auf das an dem Wurzelobjekt der
	 *            Objekthierarchie reagiert wird.
	 */
	public SetDerivedAttributeAdapter(final EObject source,
			final EStructuralFeature derivedFeature,
			final EStructuralFeature localFeature) {
		this(source, derivedFeature);
		addLocalDependency(localFeature);
	}

	/**
	 * Konstruktor des {@link SetDerivedAttributeAdapter}s.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param derivedFeature
	 *            das {@link EStructuralFeature} eines Attributes, das als
	 *            "derived" im EMF Modell markiert ist und dessen Wert neu
	 *            berechent wird.
	 */
	public SetDerivedAttributeAdapter(final EObject source,
			final EStructuralFeature derivedFeature) {
		super(source);
		this.derivedFeature = derivedFeature;
	}

	@Override
	protected void notifyNavigationTargetChange(final Notification notification) {
		notifyChange();
	}

	@Override
	protected void notifyDependantChange(final Notification notification) {
		notifyChange();
	}

	/**
	 * Wird bei {@link Notification}s innerhalb der Objekthierarchie aufgerufen
	 * und veranlasst die Neuberechnung des
	 * {@link SetDerivedAttributeAdapter#derivedFeature}s.
	 */
	private void notifyChange() {
		final InternalEObject source = getSource();
		final Object old = source.eGet(derivedFeature);
		final Object computed = compute();
		if (old != computed && (old == null || !old.equals(computed))) {
			source.eSet(derivedFeature, computed);
		}
	}

	/**
	 * Berechnet das {@link SetDerivedAttributeAdapter#derivedFeature}.
	 * 
	 * @return der neue Value des
	 *         {@link SetDerivedAttributeAdapter#derivedFeature}
	 */
	protected abstract Object compute();
}
