/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.util;

import java.net.MalformedURLException;

import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;

/**
 * Verwaltungsklasse fr Images. Implementiert das Singleton-Pattern.
 *  
 * @author Pittner
 * 
 */
public class ImageFactory
{
  /**
   * Singleton-Instanz
   */
  private static ImageFactory _instanz = null;

  /**
   * Image Registry
   */
  private static ImageRegistry _imageRegistry = null;

  /**
   * Das Display
   */
  private static Display _display = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static ImageFactory getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new ImageFactory();
    }
    return _instanz;
  }

  /**
   * Privater Konstruktor
   */
  private ImageFactory()
  {
    _display = Display.getDefault();
    _imageRegistry = new ImageRegistry(_display);
    try
    {
      loadImages();
    }
    catch (MalformedURLException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  /**
   * Fgt die Bilder der Registry hinzu
   * 
   * @throws MalformedURLException
   * 
   */
  private static void loadImages() throws MalformedURLException
  {
    _imageRegistry.put("GRID_OFF", Activator.getDefault().getImageDescriptor("icons/grid_off.gif"));
    _imageRegistry.put("GRID_OSLEFT", Activator.getDefault().getImageDescriptor("icons/grid_osleft.gif"));
    _imageRegistry.put("GRID_LEFT", Activator.getDefault().getImageDescriptor("icons/grid_left.gif"));
    _imageRegistry.put("GRID_RIGHT", Activator.getDefault().getImageDescriptor("icons/grid_right.gif"));
    _imageRegistry.put("GRID_OSRIGHT", Activator.getDefault().getImageDescriptor("icons/grid_osright.gif"));

    _imageRegistry.put("EXPORT_A4_PT", Activator.getDefault().getImageDescriptor("icons/export_pdf_a4_pt.gif"));
    _imageRegistry.put("EXPORT_A4_LS", Activator.getDefault().getImageDescriptor("icons/export_pdf_a4_ls.gif"));
    _imageRegistry.put("PRINTVIEW_TSK", Activator.getDefault().getImageDescriptor("icons/printview_tsk.gif"));
    
    _imageRegistry.put("LEGENDE", Activator.getDefault().getImageDescriptor("icons/outline_co.gif"));
  }

  /**
   * Holt das entsprechende Image aus der Registry
   * 
   * @param key der Schluessel          
   * @return das Image
   */
  public Image getImage(String key)
  {
    return _imageRegistry.get(key);
  }

  /**
   * Benachrichtigt die ImageRegistry. Es werden alle registrierten Images disposed.
   */
  public void dispose()
  {
    _display.dispose();
  }

}
