/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.util;

/**
 * Ein Tupel mit zwei Elementen oder ein geordnetes Paar.
 * 
 * @author Pittner
 * @version $Id: Tupel.java,v 1.3 2014/04/17 15:51:10 Pittner Exp $
 * @param <S1>
 *            der Typ des ersten Elements.
 * @param <S2>
 *            der Typ des zweiten Elements.
 */
public class Tupel<S1, S2> implements Cloneable {

	private S1 first;
	private S2 second;

	/**
	 * Der Standardkonstruktor tut nix.
	 */
	public Tupel() {
		// tut nix
	}

	/**
	 * Initialisierten die beiden Elemente.
	 * 
	 * @param first
	 *            das erste Element.
	 * @param second
	 *            das zweite Element.
	 */
	public Tupel(final S1 first, final S2 second) {
		this.first = first;
		this.second = second;
	}

	public S1 getFirst() {
		return first;
	}

	public S2 getSecond() {
		return second;
	}

	public void setFirst(final S1 first) {
		this.first = first;
	}

	public void setSecond(final S2 second) {
		this.second = second;
	}

	@Override
	public boolean equals(final Object obj) {
		if (obj == this) {
			return true;
		}
		if (obj instanceof Tupel<?, ?>) {
			final Tupel<?, ?> other = (Tupel<?, ?>) obj;
			boolean equals = true;
			equals &= first != null && first.equals(other.first)
					|| first == other.first;
			equals &= second != null && second.equals(other.second)
					|| second == other.second;
			return equals;
		}
		return false;

	}
	
	
	@Override
	protected Object clone() throws CloneNotSupportedException
	{
	  // TODO Auto-generated method stub
	  return new Tupel<S1, S2>(first, second);
	}

	@Override
	public int hashCode() {
		return (null != first ? first.hashCode() : 0)
				^ (null != second ? second.hashCode() : 0);
	}

	@Override
	public String toString() {
		return "(" + first + ", " + second + ")";
	}

}
