/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.autarkprogramm;

import java.util.Arrays;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.buv.plugin.ursacheneinheit.erweiterterstaualgorithmus.TableModel;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Fenster zum Versorgen der Algorithmus abhngigen Parameter
 * @author ObertM
 */
public class CompositeParameterModus extends Composite
{
	private final String _default = "Default";
	
	private final String _textDaVModus1 = "Modus 1";
	private final String _textGUIModus1 = "fest hinterlegtes Zeichen";

	private final String _textDaVModus2 = "Modus 2";
	private final String _textGUIModus2 = "letztes zurckgemeldetes Zeichen";

	/**
	 * Debug
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnis vom DaV
   */
  private ResultData _result;

  /**
   * Daten vom DaV
   */
  private Data _data;

  /**
   * Gesmatcomposite
   */
  private Composite _composite;

  /**
   * Composite (Gruppe) fr die Parameter
   */
  private Group _groupParameter;

  /**
   * Composite mit der Tabelle
   */
  private Composite _compositeTabelle = null;

  /**
   * Compoiste fr die Tabelle
   */
  private Composite _composite3 = null;

  /**
   * Table Viewer
   */
  private TableViewer _tvParameter;;

  /**
   * ComboBoxen
   */
  private Combo[] _comboReaktion;

  /**
   * ComboBoxen
   */
  private Combo[] _comboModus;

	private List<String> _listeMoeglicherAnzeigeQuerschnitte;

	private String _anlage;

  /**
   * Konstruktor der Klasse
   * @param parent Aufrufendes Composite
   * @param style Stil
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
   * @param listeMoeglicherAnzeigeQuerschnitte Liste mit den Namen der mglichen Anzeigequerschnitte 
   * @param anlage  Name der Anlage
   */
  CompositeParameterModus(Composite parent, int style, ResultData result, Data data, String anlage, List<String> listeMoeglicherAnzeigeQuerschnitte ) 
  {
    super(parent, style);
    this._result = result;
    this._data = data;

    _anlage = anlage;
    
    _listeMoeglicherAnzeigeQuerschnitte = listeMoeglicherAnzeigeQuerschnitte;
    
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    createGroupParameter();

    Table table = _tvParameter.getTable();
    TableModel[] input = createModel(table);
    _tvParameter.setInput(input);

    updateData();
  }

  /**
   * Updatefunktion fr Aufruf von aussen
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
   */
  public void update (ResultData result, Data data)
  {
    _result  = result;
    this._data = data;
    
    updateData();
    
  }
  
  /**
   * Updatefunktion
   */
  private void updateData ()
  {
    if ( _result != null )
    {
      if ( _result.hasData() )
      {
      	/*
      	 * Anlagenparameter
      	 */
      	String reaktion = _data.getTextValue("Reaktion").getText();
      	Combo cb = (Combo) _comboReaktion [0];
      	auswerteReaktion (cb, reaktion);
      	
      	String modus    = _data.getTextValue("Modus").getText();
      	cb = (Combo) _comboModus [0];
      	auswerteModus (cb, modus);
        
        /*
         * Anzeigequerschnitt spezifische Parameter
         */
        Array a = _data.getArray("AnzeigeQuerschnitt");
        for (int i=0; i<a.getLength(); ++i)
        {
        	Data aktuell = a.getItem(i);
        	SystemObject so = aktuell.getReferenceValue("AnzeigeQuerschnitt").getSystemObject();
        	
        	for (int j=0; j<_listeMoeglicherAnzeigeQuerschnitte.size(); ++j)
        	{
        		if (Tools.getName(so).equals(_listeMoeglicherAnzeigeQuerschnitte.get(j)))
    				{
        			int indexCombo = j + 1;  
        				
            	reaktion = aktuell.getTextValue("Reaktion").getText();
            	cb = (Combo) _comboReaktion [indexCombo];
            	auswerteReaktion(cb, reaktion);

            	modus    = aktuell.getTextValue("Modus").getText();
            	cb = (Combo) _comboModus [indexCombo];
            	auswerteModus(cb, modus);
              
              break;
    				}
        	}
        }
      }
    }

    getTvParameter().refresh();
    layout(true);
  }

  private void auswerteModus(Combo cb, String modus)
  {
  	if (modus.equals(_textDaVModus1))
  		modus = _textGUIModus1;

  	if (modus.equals(_textDaVModus2))
  		modus = _textGUIModus2;

    for (int k = 0; k < cb.getItemCount(); k++)
    {
      if (modus.equals(cb.getItem( k )))
        cb.select(k);      
    }
  }

	private void auswerteReaktion(Combo cb, String reaktion)
  {
    for (int k = 0; k < cb.getItemCount(); k++)
    {
      if (reaktion.equals(cb.getItem( k )))
        cb.select(k);      
    }
  }

  /**
   * Methode zum Anlegen des Composite mit den Parametern
   */
  private void createGroupParameter()
  {
    FillLayout fillLayout2 = new FillLayout();
    fillLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(fillLayout2);
    _groupParameter.setLayoutData(gridData2);
    
    //GridLayout mit Anzahl Spalten
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    
    _compositeTabelle = new Composite(_groupParameter, SWT.NONE);
    _compositeTabelle.setLayout(gridLayout);

    erzeugeTabelle();
  }

  /**
   * Tabelle fr die Parameter erzeugen
   */
  private void erzeugeTabelle()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL; 

    _composite3 = new Composite(_compositeTabelle, SWT.None);
    _composite3.setLayout(fillLayout);
    _composite3.setLayoutData( gridData );
    
    _tvParameter = new TableViewer(_composite3, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvParameter.setContentProvider(new TableContentProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column1.getColumn().setWidth(200);
    column1.getColumn().setText("Objekt");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("Reaktion auf");
    column2.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column3 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column3.getColumn().setWidth(200);
    column3.getColumn().setText("Anforderung");
    column3.setLabelProvider(new MyCellLabelProvider());

    /*
     * Zeilenhhe anpassen, nachdem die Daten eingegangen sind
     */
    _tvParameter.getTable().addListener(SWT.MeasureItem, new Listener() 
    {
      @Override
      public void handleEvent(Event event)
      {
        event.height = 29;
      }
    });

    _tvParameter.getTable().setLinesVisible(true);
    _tvParameter.getTable().setHeaderVisible(true);
  }

  /**
   * Contentprovider fr die Tabelle
   * @author ObertM
   *
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
    private TableViewer tableViewer;

    /*
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
      return (TableModel[])inputElement;
    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {

    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     *      java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.deleteObserver(this);
          }

        }

      }
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.addObserver(this);
          }
        }

      }

    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();

      }
    }
  }

  /**
   * Tabellenmodell fr die Parameter erzeugen
   * @param table Tabelle
   * @return TabellenModell
   */
  private TableModel[] createModel( Table table )
  {
  	int anzahl = 1 + _listeMoeglicherAnzeigeQuerschnitte.size();
  	
    TableModel[] elements = new TableModel[anzahl];

    _comboReaktion = new Combo [anzahl];
    _comboModus    = new Combo [anzahl];

    List<String> reaktion = Arrays.asList("keine Daten", "keine Quelle");
    List<String> modus    = Arrays.asList(_textGUIModus1, _textGUIModus2);

    List<String> reaktionAQ = Arrays.asList(_default, "keine Daten", "keine Quelle");
    List<String> modusAQ    = Arrays.asList(_default, _textGUIModus1, _textGUIModus2);

   	elements [0] = createComboElement (table, 0, _anlage, reaktion, modus);
    
    for (int i=0; i<_listeMoeglicherAnzeigeQuerschnitte.size(); ++i)
    {
    	elements [i + 1] = createComboElement (table, i + 1, _listeMoeglicherAnzeigeQuerschnitte.get(i), reaktionAQ, modusAQ);
    }
    
    return elements;
  }

  private TableModel createComboElement(Table table, int index, String objektName, List<String> reaktion, List<String> modus)
  {
  	/*
  	 * Composite 0: Objekt 
  	 */
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessVerticalSpace = true;
    
    Composite comp0 = new Composite(table, SWT.NONE);
    comp0.setBackground(table.getBackground());
    comp0.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp0.setLayoutData(gd0);
    comp0.setLayout(new GridLayout());
    
    Label label = new Label(comp0, SWT.NONE);
    label.setText( objektName );

    /*
     * Composite 1: Combobox fr Reaktion
     */
    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;
    gd1.verticalAlignment = GridData.END;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout() );
    
    _comboReaktion[index] = new Combo( comp1, SWT.BORDER );
    _comboReaktion[index].setLayoutData( gdTableItem );
    
    String[] t = new String [reaktion.size()];
    for (int i=0; i<reaktion.size(); ++i)
    	t[i] = reaktion.get(i);
    	
    _comboReaktion[index].setItems(t);
    _comboReaktion[index].select(0);

    /*
     * Composite 2: Combobox fr Modus
     */
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.grabExcessVerticalSpace = true;
    gd2.verticalAlignment = GridData.END;

    GridData gdTableItem2 = new GridData();
    gdTableItem2.horizontalAlignment = GridData.FILL;
    gdTableItem2.grabExcessHorizontalSpace = true;
    gdTableItem2.verticalAlignment = GridData.CENTER;

    Composite comp2 = new Composite( table, SWT.NONE );
    comp2.setBackground( table.getBackground() );
    comp2.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp2.setLayoutData( gd2 );
    comp2.setLayout( new GridLayout() );
    
    _comboModus[index] = new Combo( comp2, SWT.BORDER );
    _comboModus[index].setLayoutData( gdTableItem2 );
    
    String[] t2 = new String [modus.size()];
    for (int i=0; i<modus.size(); ++i)
    	t2[i] = modus.get(i);
    	
    _comboModus[index].setItems(t2);
    _comboModus[index].select(0);

    return new TableModel(comp0, comp1, comp2);
  }

  /**
   * Methode liefert den TableViewer
   * @return TableViewer
   */
  public TableViewer getTvParameter()
  {
    return _tvParameter;
  }

  /**
   * Methode liefert die ComboBox zu einem Index
   * @param index Index des Spinners
   * @return Spinner
   */
  public Combo getComboBox (int index)
  {
    return _comboReaktion [index];
  }

  /**
   * Labelprovider
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

      if (item.getData("EDITOR") != null)
      {
        TableEditor editor = (TableEditor)item.getData("EDITOR");
        editor.dispose();
      }


      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getBezeichnung();
          break;
        case 1:
          comp = model.getAttribut();
          break;
        case 2:
          comp = model.getEinheit();
          break;
        default:
      }
      TableEditor editor = new TableEditor(item.getParent());
      
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);
    }
  }

  /**
   * Methode schreibt die aktuell eingestellten Daten in eine Attributgruppe fr den DaV
   * @param data Attributgruppe fr den DaV
   */
	public void getData(Data data)
  {
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
  	List<SystemObject> objekte = connection.getDataModel().getType( "typ.anzeigeQuerschnitt" ).getElements();

	  /*
	   * Anlagenspezifisch	
	   */
  	Combo cb = (Combo) _comboReaktion [0];
  	String selectedReaktionAnlage = cb.getItem(cb.getSelectionIndex());
  	data.getTextValue("Reaktion").setText(selectedReaktionAnlage);
  	
  	cb = (Combo) _comboModus [0];
  	String selectedModusAnlage = cb.getItem(cb.getSelectionIndex());
  	if (selectedModusAnlage.equals(_textGUIModus1))
  		selectedModusAnlage = _textDaVModus1;
  	else if (selectedModusAnlage.equals(_textGUIModus2))
  		selectedModusAnlage = _textDaVModus2;
  	
  	data.getTextValue("Modus").setText(selectedModusAnlage);
  	
  	/*
  	 * Anzeigequerschnitte
  	 */

  	String selectedReaktion;
		String selectedModus;

  	/*
  	 * Prfen wieviele Anzeigequerschnitte extra versorgt sind
  	 */
  	int anzahl = 0;
		for (int i=0; i<_listeMoeglicherAnzeigeQuerschnitte.size(); ++i)
  	{
  		int indexCombo = i + 1;

    	cb = (Combo) _comboReaktion [indexCombo];
    	selectedReaktion = cb.getItem(cb.getSelectionIndex());
    	
    	cb = (Combo) _comboModus [indexCombo];
    	selectedModus = cb.getItem(cb.getSelectionIndex());

    	if (!selectedReaktion.equals(_default) || !selectedModus.equals(_default))
    		anzahl++;
  	}
  		
  	Array a = data.getArray("AnzeigeQuerschnitt");
  	a.setLength(anzahl);

  	int index = 0;
  	for (int i=0; i<_listeMoeglicherAnzeigeQuerschnitte.size(); ++i)
  	{
  		int indexCombo = i + 1;

    	cb = (Combo) _comboReaktion [indexCombo];
    	selectedReaktion = cb.getItem(cb.getSelectionIndex());
    	
    	cb = (Combo) _comboModus [indexCombo];
    	selectedModus = cb.getItem(cb.getSelectionIndex());

    	if (selectedModus.equals(_textGUIModus1))
    		selectedModus = _textDaVModus1;
    	else if (selectedModus.equals(_textGUIModus2))
    		selectedModus = _textDaVModus2;

    	if (!selectedReaktion.equals(_default) || !selectedModus.equals(_default))
    	{
    		Data aktuell = a.getItem(index++);

    		SystemObject so = null;
    		for (int j=0; j<objekte.size(); ++j)
    			if (Tools.getName(objekte.get(j)).equals(_listeMoeglicherAnzeigeQuerschnitte.get(i)))
    				so = objekte.get(j);
    		
    		aktuell.getReferenceValue("AnzeigeQuerschnitt").setSystemObject(so);
    		
    		if (selectedReaktion.equals(_default))
    			selectedReaktion = selectedReaktionAnlage;
    		
    		if (selectedModus.equals(_default))
    		 selectedModus = selectedModusAnlage;
    		
      	aktuell.getTextValue("Reaktion").setText(selectedReaktion);
      	aktuell.getTextValue("Modus").setText(selectedModus);
    	}
  	}
  }
}
