/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.luftschadstoffe;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.ursacheneinheit.Activator;
import de.bsvrz.buv.plugin.ursacheneinheit.defaultParameter.DefaultParameterHandler;
import de.bsvrz.buv.plugin.ursacheneinheit.defaultParameter.IDefaultParameterVersorgung;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.buv.plugin.ursacheneinheit.verwaltung.VersorgungsArt;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;

/**
 * View zum Versorgen der Parameter einer Ursacheneinheit vom Typ "typ.sswSbaUeLuftschadstoffe"
 * @author ObertM
 */
public class ViewParameterLuftSchadstoffe extends ViewPart implements IDefaultParameterVersorgung
{
	/**
	 * eindeutige ID der View
	 */
  public static final String ID = "de.bsvrz.buv.plugin.ursacheneinheit.parameter.typ.sswSbaUeLuftschadstoffe"; 

  /**
   * Schalter fr Debug Ausgaben
   */
  private static final boolean _isDebug = false;

  /**
   * Attributgruppen des DAV, die bercksichtigt wird
   */
  public static final String _pidAtgAlg [] = new String [] 
  { 
  	"atg.sswSbaUeLuftschadstoffe",
  };
  
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();
  
  /**
   * Aufrufendes Compoiste
   */
  private Composite _parent;
  
  /**
   * Compoiste fr die Reiter
   */
  private CTabFolder _tabFolder = null;

  /**
   * Hauptcomposite
   */
  private Composite _composite = null;

  /**
   * Composite mit den Buttons
   */
  private Composite _compositeBottom;

  /**
   * Button "OK" 
   */
  private Button _buttonOk;

  /**
   * Button "bernehmen"
   */
  private Button _buttonUebernehemen;

  /**
   * Button "Abbrechen"
   */
  private Button _buttonAbbrechen;

  /**
   * Button Hilfe
   */
  private Button _buttonHilfe;

  /**
   * Button "Fr alle bernehmen"
   */
  private Button _buttonAlle;

  /**
   * Memento
   */
  private IMemento _memento = null;

  /**
   * Second View ID
   */
  private String _secondViewId;

  /**
   * Verbindung zum DaV
   */
  private ClientDavInterface _connection;

  /**
   * Ergebnisdaten des DaV fr die Algorithmus spezifischen Parameter in einer Hashmap. Als Key dient die Pid
   * des Systemobjekts, als Value das Ergebnis aus dem DaV
   */
  private HashMap<String,ResultData[]> _resultDataAlg = new HashMap<String, ResultData[]>();

  /**
   * Liste mit denzu versorgenden Ursacheneinheiten;
   * 
   */
  List<SystemObject> _ursachenEinheiten = new ArrayList<SystemObject>();
  
  /**
   * Merker ob die Versorgung der Zuordnung der Messquerschnitte angeboten wird
   */
	private boolean _auswahlMQ;

	private List<SystemObject> _listeMoeglicherUmfeldDatenSensor;

	private List<SystemObject> _listeMoeglicherMessQuerschnitt;

	private List<SystemObject> _listeMoeglicherEreignisTypen;

  @Override
  public void createPartControl( Composite parent )
  {
    if ( _isDebug )
      System.out.println( "--> ViewParameter: createPartControl" );

    _parent = parent;

    /*
     * Wenn mehrere ursacheneinheiten bergeben werden, wird die Versorgung der Messquerschnitte
     * nicht angeboten, da es unwahrscheinlich ist, dass mehrere Ursacheneinheiten vom gleichen Typ
     * die selben Messquerschnitte zugeordnet haben. Die Parameter fr die Algorithmen dagegen knnen
     * durchaus identisch sein.
     */
   	_auswahlMQ = !_secondViewId.contains( ";" );
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".parameterLuftSchadstoffe" );

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _composite = new Composite( parent, SWT.NONE );
    _composite.setLayout( gridLayout );
    _composite.setLayoutData( gridData );

    // Darstellung aufbauen
    
    createCompositeTab();
    createCompositeBottom();

    restoreState();

    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    if ( _secondViewId.contains( ";" ) )
    {

      String[] split = _secondViewId.split( ";" );

      for (String s : split)
      {
        SystemObject so = _connection.getDataModel().getObject( s );
        _ursachenEinheiten.add( so );
      }

    }
    else
    {
      SystemObject so = _connection.getDataModel().getObject( _secondViewId );
      _ursachenEinheiten.add( so );
    }

    if ( _isDebug )
      System.out.println( "Listener anmelden!" );

    if ( _isDebug )
    {
      System.out.println( "Anzahl Tabs:" + _ursachenEinheiten.size() );

      for (SystemObject so : _ursachenEinheiten)
      {
        System.out.println( "\tPids: " + so.getPid() );
      }
    }
    
    // Reiter erstellen

    erzeugeTabItems( _ursachenEinheiten );
    
    leseDatenVonDav(_ursachenEinheiten);

  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * Methode liest die bentigten Parameter direkt vom DaV
   * @param liste Liste mit Systemobjekten die versorgt werden sollen
   */
  private void leseDatenVonDav (List<SystemObject> liste)
  {
  	for (int i=0; i<liste.size(); ++i)
  	{
  		String pidObj = liste.get(i).getPid();
  		
  		ResultData result = null;
  		
  		for (int j=0; j<_pidAtgAlg.length; ++j)
  		{
    		result = DaVKommunikation.getInstanz().leseDatenDirektVonDaV(pidObj, _pidAtgAlg [j], "asp.parameterSoll");
    		
    		/*
    		 * Leeres Feld anlegen
    		 */
    		if (!_resultDataAlg.containsKey(pidObj))
    		{
    		  ResultData [] rd = new ResultData [_pidAtgAlg.length];
    		  _resultDataAlg.put(pidObj, rd);
    		}
    		
    		_resultDataAlg.get(pidObj)[j] = result;
  		}
  		
  		verarbeiteDavDaten(pidObj);
		}
  }

  /**
   * This method initializes compositeTab (Bereich ber der ButtonLeiste)
   * 
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData3);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub

      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();        
        CTabItem itemSel = folder.getSelection();
        
        Composite cSel = (Composite)itemSel.getControl();
        
        if (((Composite)cSel).getChildren().length > 0)
        {
          CompositeParameterGesamt compParSel = (CompositeParameterGesamt)cSel.getChildren()[0];
          
          CompositeParameterLuftSchadstoffe composite1 = compParSel.getCompositeParameterLuftSchadStoffe();
          
          if (composite1 != null)
           	if (composite1 != null && composite1.getTvParameter() != null)
           		composite1.getTvParameter().refresh();
        }
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<SystemObject> list )
  {
    for (SystemObject so : list)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = Tools.getName(so);
      tabItem.setText( tabName );
      tabItem.setData( tabName, so.getPid() );

      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }

  /**
   * Reiter fr Untergruppen des SystemObjekts erstellen
   * 
   * @param so
   *          SystemObjekt
   * @param ug
   *          Name der Untergruppe
   * @param pos
   *          Position des Reiters
   */
  public CTabItem erzeugeTabItems( SystemObject so, String ug, int pos )
  {
    CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE, pos );

    String tabName = Tools.getName(so);
    tabItem.setText( tabName );
    tabItem.setData( tabName, so.getPid() );

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData2 );

    tabItem.setControl( c1 );

    return tabItem;
  }

  /**
   * This method initializes compositeBottom (ButtonLeiste)
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    _compositeBottom = new Composite( _composite, SWT.NONE );
    _compositeBottom.setLayoutData( gridData3 );
    _compositeBottom.setLayout( rowLayout );

    new DefaultParameterHandler(_compositeBottom, this);
    
    _buttonAlle = new Button( _compositeBottom, SWT.NONE );
    _buttonAlle.setText( "Alle setzen" );
    _buttonAlle.setToolTipText( "Einstellungen bei allen ausgewhlten gleichartigen AQs setzen" );
    _buttonAlle.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionAlle();
      }
    } );
    _buttonAlle.setEnabled(!_auswahlMQ);

    _buttonOk = new Button( _compositeBottom, SWT.NONE );
    _buttonOk.setText( "Ok" );
    _buttonOk.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionSchalten();
      }
    } );

    _buttonUebernehemen = new Button( _compositeBottom, SWT.NONE );
    _buttonUebernehemen.setText( "bernehmen" );
    _buttonUebernehemen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionUebernehmen();
      }
    } );

    _buttonAbbrechen = new Button( _compositeBottom, SWT.NONE );
    _buttonAbbrechen.setText( "Abbrechen" );
    _buttonAbbrechen.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        dispose();
      }

    } );
    _buttonHilfe = new Button( _compositeBottom, SWT.NONE );
    _buttonHilfe.setText( "Hilfe" );
    _buttonHilfe.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionHilfe();
      }

    } );
  }

  protected void aktionHilfe()
  {
	String contextId = Activator.PLUGIN_ID + ".parameterLuftSchadstoffe";  
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  protected void aktionSchalten()
  {
    if ( schaltenUndParametrieren() )
    {
      dispose();
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  protected void aktionUebernehmen()
  {
    if ( schaltenUndParametrieren() )
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  @Override
  public void init( IViewSite site, IMemento memento ) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init( site, memento );
  }

  private void restoreState()
  {

    if ( _memento == null )
    {
      return;
    }

    _secondViewId = _memento.getString( "pidMemento" );

  }

  @Override
  public void saveState( IMemento memento )
  {
    memento.putString( "pidMemento", _secondViewId );
    super.saveState( memento );
  }
  
  /**
   * Schalten und Parametrieren im DAV durchfhren
   * 
   * @return
   */
  private Boolean schaltenUndParametrieren()
  {
    List<ResultData> list = new ArrayList<ResultData>();
    
  	list.addAll(erzeugeResultData ());

    UrlasserDialog urlasserDialog = new UrlasserDialog( Display.getCurrent().getActiveShell() );
    Boolean schlieenOk = (urlasserDialog.anzeigen( list )).getEingabeOk();

    return schlieenOk;
  }

  /**
   * Methode erzeugt die Resultdaten fr den DaV fr alle Reiter fr die Algorithmus Parameter
   * @return Resultdaten fr den DaV
   */
  private List<ResultData> erzeugeResultData ()
  {
    List<ResultData> list = new ArrayList<ResultData>();

    DataModel dataModel = _connection.getDataModel();

    Aspect asp = dataModel.getAspect( "asp.parameterVorgabe" );

    CTabItem[] items = _tabFolder.getItems();
    for (CTabItem ti : items)
    {
    	/*
    	 * Luftschadstoffe
    	 */
    	Data data1 = erzeugeAttributGruppeLaerm(ti);
      
      String pid1 = (String) ti.getData( ti.getText() );
      SystemObject so1 = dataModel.getObject( pid1 );
       
      AttributeGroup atg1 = dataModel.getAttributeGroup( _pidAtgAlg [0] );
      ResultData rd1 = new ResultData( so1, new DataDescription( atg1, asp ), new Date().getTime(), data1 );
      list.add( rd1 );
    }

    return list;
  }

  /**
   * Methode erzeugt ein Datenfeld mit den Daten eines bergebenen Reiters
   * @param itemSel auszuwertender Reiter
   * @return Datenfeld
   */
  private Data erzeugeAttributGruppeLaerm (CTabItem itemSel)
  {
    DataModel dataModel = _connection.getDataModel();

    AttributeGroup atg = dataModel.getAttributeGroup( _pidAtgAlg [0] );
    Data data = _connection.createData( atg );
    
    Composite c = (Composite) itemSel.getControl();
    CompositeParameterGesamt compPar = (CompositeParameterGesamt) c.getChildren()[0];

    CompositeParameterLuftSchadstoffe cAlg = compPar.getCompositeParameterLuftSchadStoffe();
    
    if (cAlg != null)
    {
    	/*
    	 * Comboboxen 
    	 */
    	int anzahlCB = cAlg.getAnzahlComboBoxItems();
    	for (int i=0; i<anzahlCB; ++i)
    	{
        String item = cAlg.getItemComboBoxName(i);
        Combo cb = cAlg.getComboBox(i);
        String selected = cb.getItem( cb.getSelectionIndex() );
        
    		/*
    		 * Referenzen auf Objekte
    		 */
    		 if (i < 3)
    		 {
           SystemObject so = null;
           
           if (!selected.equals(Tools._undefiniert))
           {
          	 /*
          	  * UmfeldDatenSensor
          	  */
          	 if (i == 0)
          	 {
          		 for (SystemObject s : _listeMoeglicherUmfeldDatenSensor)
          			 if (Tools.getName(s).equals(selected))
          				 so = s;
          	 }
          	 /*
          	  * MessQuerschnitt
          	  */
          	 if (i == 1)
          	 {
          		 for (SystemObject s : _listeMoeglicherMessQuerschnitt)
          			 if (Tools.getName(s).equals(selected))
          				 so = s;
          	 }
          	 /*
          	  * EreignisTyp
          	  */
          	 if (i == 2)
          	 {
          		 for (SystemObject s : _listeMoeglicherEreignisTypen)
          			 if (Tools.getName(s).equals(selected))
          				 so = s;
          	 }
          	 
           	 data.getReferenceValue(item).setSystemObject(so);
           }
    		 }
    		 else
    		 {
    			 data.getTextValue(item).setText(selected);
    		 }
    	}
    	
    	/*
    	 * Spinner
    	 */
      for (int j=0; j<cAlg.getAnzahlSpinnerItems(); ++j)
      {
        String item = cAlg.getItemSpinnerName(j);
        int wert = getWert(cAlg.getSpinner(j));
        data.getUnscaledValue(item).set(wert);
      }
      
    	/*
    	 * Zeitpunkte
    	 */
      for (int j=0; j<cAlg.getAnzahlZeitPunktItems(); ++j)
      {
        String item = cAlg.getItemZeitPunktName(j);
        DatumZeit zpe = cAlg.getZeitPunktEingabe(j);
        long wert = zpe.getDatum().getTime();
        
        data.getTimeValue(item).setMillis(wert);
      }

    }
    	
    return data;
  }

	/**
	 * Methode liefert den Wert einers Spinners
	 * @param spinner Spinner
	 * @return eingestellter Wert
	 */
	public int getWert (Spinner spinner)
	{
	  Integer i1 = spinner.getSelection();
		return i1;
	}

  /**
   * Methode zum Verarbeiten der Daten des DaV
   * @param pidObj Pid des Systemobjekts
   */
  private void verarbeiteDavDaten (String pidObj)
  {
    if ( _isDebug )
      System.out.println( "--> ViewParameter: VerarbeiteDavDaten" );
    
    /*
     *  einmalig alle mglichen Objekte bestimmen
     */
    
    if ( _listeMoeglicherUmfeldDatenSensor == null )
    	_listeMoeglicherUmfeldDatenSensor = Tools.getSystemObjekte("typ.umfeldDatenSensor"); 
    		
    if ( _listeMoeglicherMessQuerschnitt == null )
    	_listeMoeglicherMessQuerschnitt = Tools.getMessQuerschnitte();
    		
    if ( _listeMoeglicherEreignisTypen == null )
    	_listeMoeglicherEreignisTypen = Tools.getSystemObjekte("typ.ereignisTyp" ); 
    		
    /*
     *  alle Reiter zur Pid ermitteln
     */
    CTabItem[] items = _tabFolder.getItems();
    List<CTabItem> itemPidList = new ArrayList<CTabItem>();
    for (CTabItem ti : items)
    {
      String pid = (String) ti.getData( ti.getText() );

      if ( pid == pidObj )
      {
        itemPidList.add( ti );
      }
    }

    if ( itemPidList != null )
    {
      CTabItem item = null;
  
      for (CTabItem ti : itemPidList)
      {
        item = ti;

        Composite c = (Composite) item.getControl();
        Control[] children = c.getChildren();
        CompositeParameterGesamt compPar = null;

        if ( children.length == 0 )
        {
          compPar = new CompositeParameterGesamt( c, SWT.None, _resultDataAlg.get(pidObj), 
          		Tools.bestimmeObjektNamen(_listeMoeglicherUmfeldDatenSensor, true),
          		Tools.bestimmeObjektNamen(_listeMoeglicherMessQuerschnitt, true),
          		Tools.bestimmeObjektNamen(_listeMoeglicherEreignisTypen, true));
          compPar.layout( true );
          c.layout( true );

        }
        else
        {
          compPar = (CompositeParameterGesamt) children[0];
          compPar.update( _resultDataAlg.get(pidObj) );
          
          compPar.layout( true );
          c.layout( true );
        }
      }
    }
  }

  @Override
  public void dispose()
  {
    super.dispose();
//    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener( this );
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if ( page != null )
    {
      page.hideView( this );
    }
  }

  /**
   * Methode bernimmt die eingestellten Parameter fr alle Tabs. Hierzu werden die Tabellenviewer des
   * angewhlten Tab an die anderen Tabs bergeben.
   */
  protected void aktionAlle()
  {
  	/*
  	 * Angewhlten Tab (= Ursacheneinheit bestimmen)
  	 */
    CTabItem itemSel = _tabFolder.getSelection();
    Composite c = (Composite) itemSel.getControl();
    CompositeParameterGesamt compPar = (CompositeParameterGesamt) c.getChildren()[0];
    
    /*
     * Parameter Luftschadstoffe kopieren
     */
    kopiereDaten(compPar, 0);

    /*
     * Fr alle Reiter alle Attributgruppe erzeugen
     */
    CTabItem[] items = _tabFolder.getItems();

    for (CTabItem ti : items)
    {
      String pid = (String) ti.getData( ti.getText() );
      verarbeiteDavDaten(pid);
    }
  }
  
  
  /**
   * Methode kopiert eine Parameterart auf alle geffneten Ursacheneinheiten (Reiter)
   * @param compPar Compoiste der Gesamtparamter
   * @param index Index der Parameter entsprechend dem Feld _pidAtgAlg
   */
  private void kopiereDaten (CompositeParameterGesamt compPar, int index)
  {
    List<ResultData> list = new ArrayList<ResultData>();
    
    DataModel dataModel = _connection.getDataModel();
    Aspect asp = dataModel.getAspect( "asp.parameterVorgabe" );

    AttributeGroup atg = dataModel.getAttributeGroup( _pidAtgAlg[index] );
    Data data = _connection.createData( atg );

    /*
     * aktueller Reiter einlesen
     */
    CTabItem itemSel = _tabFolder.getSelection();

    /*
     * Lrm
     */
    if (index == 0)
    {
    	data = erzeugeAttributGruppeLaerm (itemSel);
    }

    /*
     * Fr alle Reiter alle Attributgruppe erzeugen
     */
    CTabItem[] items = _tabFolder.getItems();

    for (CTabItem ti : items)
    {
      String pid = (String) ti.getData( ti.getText() );
      SystemObject so = dataModel.getObject( pid );

      ResultData rd1 = new ResultData( so, new DataDescription( atg, asp ), new Date().getTime(), data );
      list.add(rd1);
    }

    /*
     * Daten umkopieren
     */
    
    for (ResultData rData : list)
    {
    	String pid = rData.getObject().getPid();
  		_resultDataAlg.get(pid)[index] = rData;
    }
  }
  
  @Override
  public String getParameterAtgPid()
  {
    return _pidAtgAlg[0];
  }

  @Override
  public String getRootObjektPid()
  {
    String result = null;
    
    switch (getVersorgungsArt()) {
      case EinReiterProUrsachenEinheit:
        CTabItem item = _tabFolder.getSelection();
        if (item != null) {
          result = (String)item.getData(item.getText());
        }  
        break;
      case EinReiterFuerAlleUrsachenEinheiten:
        for (SystemObject so : _ursachenEinheiten)
        {
          if (result == null) {
            result = so.getType().getPid();
          }
          else {
            if (!result.equals(so.getType().getPid())) {
              result = null;
              break;
            }
          }
        }
        break;
    }
    
    return result;
  }

  @Override
  public VersorgungsArt getVersorgungsArt()
  {
    return VersorgungsArt.EinReiterProUrsachenEinheit;
  }
  
  @Override
  public void setData(Data data) {
    CTabItem item = _tabFolder.getSelection();
    if (item != null) {
      Composite c = (Composite) item.getControl();
      Control[] children = c.getChildren();
      
      if ( children.length > 0 )
      {
        CompositeParameterGesamt compPar = (CompositeParameterGesamt) children[0];
        
        String pid = (String)item.getData(item.getText());
      
        ResultData result[] = _resultDataAlg.get(pid);
        result[0] = new ResultData(result[0].getObject(), result[0].getDataDescription(),
            result[0].getDataTime(), data);
//        _resultDataAlg.put(pid, result);
               
        compPar.update( _resultDataAlg.get(result[0]) );
      }
    }  
  }
  
}
