/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.messquerschnitte;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.buv.plugin.ursacheneinheit.tools.AuswahlObjekte;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Darstellung des Fensters zur Parameternderung mit den Messquerschnitten und den Algorithmus spezifischen
 * Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Messquerschnitte
   */
  private ResultData _resultDataMq;

  /**
   * Liste mit den mglichen Messquerschnitte
   */
  private List<String> _listeMoeglicherObjekte;
  
  /**
   * Hauptcomposite
   */
  private Composite _composite;

	/**
	 * Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
	private boolean _auswahlMQ;

	private AuswahlObjekte _auswahlObjekteMQ;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param listMoeglicheMessQuerschnitte Liste mit den mglichen Messquerschnitten
	 * @param auswahlMQ Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData resultDataMq, List<String> listMoeglicheMessQuerschnitte, boolean auswahlMQ)
  {
    super(parent, style);
    
    this._resultDataMq = resultDataMq;
    this._auswahlMQ = auswahlMQ;
    
    this._listeMoeglicherObjekte = listMoeglicheMessQuerschnitte;
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    if (_auswahlMQ)
    	_auswahlObjekteMQ = new AuswahlObjekte(_composite, "Messquerschnitte");

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataMq Ergebnis Messquerschnitte
   */
  public synchronized void update (ResultData resultDataMq)
  {
    _resultDataMq = resultDataMq;

    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataMq != null)
    {
    	if (_auswahlMQ)
        updateDataMessQuerschnitt();
    }
  }

  /**
   * Updatefunktion ber die Messquerschnitte
   */
	private void updateDataMessQuerschnitt()
  {
		List<SystemObject> l = Tools.auswerteResultDataMQ(_resultDataMq);
		
		List<String> angewaehlteObjekte = new ArrayList<String>();
		for (SystemObject so : l)
	    angewaehlteObjekte.add (Tools.getName(so));
	  
    _auswahlObjekteMQ.setAngewaehlteObjekte(_listeMoeglicherObjekte, angewaehlteObjekte);
  }
	
  /**
   * Methode liefert die Liste der angewhlten Messstellen
   * @return Liste der angewhlten Messstellen
   */
  public org.eclipse.swt.widgets.List getZugeordneteMQ()
  {
    return _auswahlObjekteMQ.getZugeordneteObjekte();
  }

}
