/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.naessewarnung;

import java.util.Comparator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

/**
 * Klasse erzeugt ein Composite mit denen es mglich ist Objekte zuzuordnen
 * @author ObertM
 */
public class AuswahlObjekte
{
	/**
	 * Parent Composite
	 */
  private Composite _parent;

  /**
   * Liste mit den mglichen Messquerschnitten
   */
  private org.eclipse.swt.widgets.List _list1 = null;

  /**
   * Liste mit den angewhlten Messquerschnitten
   */
  private org.eclipse.swt.widgets.List _list2 = null;

  /**
   * Comparator fr die Messquerschnitte
   */
  private MyComparator _comp = new MyComparator();

  /**
   * Textuelle Beschreibung der Objekte
   */
	private String _kennung;

	/**
	 * Konstruktor der Klasse
	 * @param parent parent Composite
	 * @param kennung Kennung die bei der Auswahl eingeblendet wird (z.B. Messquerschnitte)
	 */
  public AuswahlObjekte (Composite parent, String kennung)
  {
  	_parent = parent;
  	
  	_kennung = kennung;
  	
  	erzeugeEingabeElemente();
  }
	
	/**
	 * Methode erzeugt die Eingabeelemente
	 */
  private void erzeugeEingabeElemente()
  {
    FillLayout fillLayout2 = new FillLayout();
    fillLayout2.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    RowLayout rowLayout3 = new RowLayout();
    rowLayout3.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout3.justify = false;
    rowLayout3.wrap = false;
    rowLayout3.pack = false;
    rowLayout3.spacing = 25;
    rowLayout3.fill = false;
    
    GridData gridData22 = new GridData();
    gridData22.horizontalAlignment = GridData.CENTER;
    gridData22.grabExcessVerticalSpace = false;
    gridData22.grabExcessHorizontalSpace = false;
    gridData22.verticalAlignment = GridData.CENTER;

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = false;
    gridData2.heightHint = 110;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.CENTER;

    Group groupObjekte = new Group(_parent, SWT.NONE);
    groupObjekte.setText(_kennung);
    groupObjekte.setLayout(fillLayout2);
    groupObjekte.setLayoutData(gridData2);
    
    GridData gridData41 = new GridData();
    gridData41.horizontalAlignment = GridData.FILL;
    gridData41.grabExcessHorizontalSpace = true;
    gridData41.grabExcessVerticalSpace = true;
    gridData41.verticalAlignment = GridData.FILL;
    
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.CENTER;
    gridData21.verticalAlignment = GridData.CENTER;
    
    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.CENTER;
    gridData12.verticalAlignment = GridData.CENTER;
    
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    gridData3.grabExcessVerticalSpace = true;

    //GridLayout mit Anzahl Spalten
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 4;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    
    Composite compositeObjekte = new Composite(groupObjekte, SWT.NONE);
    Label label = new Label(compositeObjekte, SWT.NONE);
    label.setText("mgliche " + _kennung);
    label.setLayoutData(gridData12);
    compositeObjekte.setLayout(gridLayout);
    new Label(compositeObjekte, SWT.NONE);
    
    Label label1 = new Label(compositeObjekte, SWT.NONE);
    label1.setText("zugeordnete " + _kennung);
    label1.setLayoutData(gridData21);
    new Label(compositeObjekte, SWT.NONE);
 
    _list1 = new org.eclipse.swt.widgets.List(compositeObjekte, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _list1.setLayoutData(gridData3);

    Composite _composite1 = new Composite(compositeObjekte, SWT.NONE);
    _composite1.setLayoutData(gridData22);
    _composite1.setLayout(rowLayout3);
    
    Button _addButton = new Button(_composite1, SWT.NONE);
    _addButton.setText("<");
    _addButton.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        String[] selection = _list2.getSelection();
        for (String string : selection)
        {
          _list1.add(string);
          _list2.remove(string);
        }

        String[] items = _list1.getItems();
        java.util.Arrays.sort(items, _comp);
        _list1.setItems(items);

        String[] items2 = _list2.getItems();
        java.util.Arrays.sort(items2, _comp);
        _list2.setItems(items2);

      }
    });
    Button _subButton = new Button(_composite1, SWT.NONE);
    _subButton.setText(">");
    _subButton.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        String[] selection = _list1.getSelection();
        for (String string : selection)
        {
          _list2.add(string);
          _list1.remove(string);
        }

        String[] items = _list2.getItems();
        java.util.Arrays.sort(items, _comp);
        _list2.setItems(items);

        String[] items2 = _list1.getItems();
        java.util.Arrays.sort(items2, _comp);
        _list1.setItems(items2);

      }
    });
    _list2 = new org.eclipse.swt.widgets.List(compositeObjekte, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _list2.setLayoutData(gridData41);

    _list1.setSize(_list1.getSize().x, 100);
  }

  /**
   * Interne Klasse zum Vergleich
   * 
   * @author Krause
   * 
   */
  private class MyComparator implements Comparator<String>
  {

    @Override
    public int compare(String o1, String o2)
    {
      int ret = 0;
      
      //String-Sortierung
      if ( o1 != null )
        ret = o1.compareTo( o2 );
     
//      //Integer-Sortierung
//      Integer i1 = new Integer(o1);
//      Integer i2 = new Integer(o2);
//
//      if (i1 < i2)
//        ret = -1;
//      else if (i1 == i2)
//        ret = 0;
//      else if (i1 > i2)
//        ret = 1;
      
      return ret;
      
    }
  }
  
  /**
   * Methode definiert die mglichen Objekte
   * @param moeglicheObjekte mgliche Objekte
   */
  private void setMoeglicheObjekte (List<String> moeglicheObjekte)
  {
		_list1.removeAll();
		_list2.removeAll();
		
    for ( String str : moeglicheObjekte )
      _list1.add( str );
    
    String[] items = _list1.getItems();
    java.util.Arrays.sort( items, _comp );
    _list1.setItems( items );

    String[] items2 = _list2.getItems();
    java.util.Arrays.sort( items2, _comp );
    _list2.setItems( items2 );
  }
  
  /**
   * Methode definiert die bereits angewhlten Objekte
   * @param angewaehlteObjekte angewhlte Objekte
   * @param moeglicheObjekte mgliche Objekte 
   */
  public void setAngewaehlteObjekte (List<String> moeglicheObjekte, List<String> angewaehlteObjekte)
  {
  	setMoeglicheObjekte(moeglicheObjekte);
  	
  	for (String s : angewaehlteObjekte)
  	{
      _list1.remove ( s );
      _list2.add    ( s );

  	}
  }

  /**
   * Methode liefert eine Liste mit den angewhlten Objekten
   * @return Liste mit den angewhlten Objekten
   */
	public org.eclipse.swt.widgets.List getZugeordneteObjekte()
  {
	  return _list2;
  }
}
