/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.temposplit;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Darstellung des Fensters zur Parameternderung mit den Messquerschnitten und den Algorithmus spezifischen
 * Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData _resultDataAlg;

  /**
   * Liste mit den mglichen Messquerschnitte
   */
  private List<String> _listeMoeglicherObjekte;
  
  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Gruppe mit den allgemeinen Parameter
   */
  private Group _groupAllgemein;

  /**
   * Composite mit den Reitern
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private ArrayList<String> _list;

	/**
	 * Compoiste mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterAlgorithmusProFahrstreifen[] _compositeAlg;

	/**
	 * Gruppe Parameter
	 */
	private Group _groupParameter;

	/**
	 * Composite fr die allgemeinen Parameter
	 */
	private CompositeParameterAlgorithmusAllgemein _compositeAlgAllgemein;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param listMoeglicheMessQuerschnitte Liste mit den mglichen Messquerschnitten
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData resultDataAlg, List<String> listMoeglicheMessQuerschnitte)
  {
    super(parent, style);
    
    this._resultDataAlg = resultDataAlg;
    
    this._listeMoeglicherObjekte = listMoeglicheMessQuerschnitte;
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    createCompositeObjekte();
    
    createCompositeTab();
    
    /*
     * Reiter anlegen fr die unterschiedliche Anzahl Fahrstreifen
     */
    _list = new ArrayList<String>();
    _list.add ("1 Fahrstreifen");
    _list.add ("2 Fahrstreifen");
    _list.add ("3 Fahrstreifen");
    _list.add ("4 Fahrstreifen");

  	_compositeAlg = new CompositeParameterAlgorithmusProFahrstreifen [_list.size()];

    erzeugeTabItems (_list);

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void update (ResultData resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;

    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataAlg != null)
    {
    	updataDataAlgorithmus ();
    }
  }
  
  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus()
  {
  	updateDataAllgemein();
  	
    /*
     * Loop ber die Fahrstreifen	
     */
  
  	int index = 0;
  	for (String s : _list)
  	{
      /*
       * Reiter emrmitten
       */

  		CTabItem[] items = _tabFolder.getItems();
      CTabItem item = null;
      
       for (CTabItem ti : items)
       {
         String pid = (String) ti.getData( ti.getText() );
         if ( pid.equals(s) )
        	 item = ti;
       }

       if ( item != null )
       {
          Composite c = (Composite) item.getControl();
          Control[] children = c.getChildren();
          CompositeParameterAlgorithmusProFahrstreifen compPar = null;

          Data data = null;
          if (_resultDataAlg.hasData())
          {
            String kennung = s.replace(" ", "");
            data = _resultDataAlg.getData().getItem(kennung);
          }
          
          if ( children.length == 0 )
          {
            compPar = new CompositeParameterAlgorithmusProFahrstreifen ( c, SWT.None, _resultDataAlg, data );
            _compositeAlg [index++] = compPar;
            
            compPar.layout( true );
            c.layout( true );
            
            compPar.getTvParameter().refresh();
          }
          else
          {
            compPar = (CompositeParameterAlgorithmusProFahrstreifen) children[0];
            compPar.update ( _resultDataAlg, data );
            
            compPar.layout( true );
            c.layout( true );

            compPar.getTvParameter().refresh();
          }
        }
      }
  }

  private void updateDataAllgemein ()
  {
  	/*
  	 * Parameter allgemein
  	 */
    Composite c = _groupAllgemein;
    Control[] children = c.getChildren();
    CompositeParameterAlgorithmusAllgemein compParObjekte = null;

    Data data = null;
    if (_resultDataAlg.hasData())
    {
      data = _resultDataAlg.getData();
    
	    if ( children.length == 0 )
	    {
	    	compParObjekte = new CompositeParameterAlgorithmusAllgemein ( c, SWT.None, _resultDataAlg, data, _listeMoeglicherObjekte );
	      _compositeAlgAllgemein = compParObjekte;
	      
	      compParObjekte.layout( true );
	      c.layout( true );
	      
	      compParObjekte.getTvParameter().refresh();
	    }
	    else
	    {
	    	compParObjekte = (CompositeParameterAlgorithmusAllgemein) children[0];
	    	compParObjekte.update ( _resultDataAlg, data );
	      
	    	compParObjekte.layout( true );
	      c.layout( true );
	
	      compParObjekte.getTvParameter().refresh();
	    }
    }
  }

  /**
   * Methode zum Anlegen des Composite fr die Objekte
   */
  private void createCompositeObjekte()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;

    _groupAllgemein = new Group(_composite, SWT.NONE);
    _groupAllgemein.setText("Parameter allgemein");
    _groupAllgemein.setLayout(new GridLayout(1, false));
    _groupAllgemein.setLayoutData(gridDataGroup);
  }

  /**
   * Methode zum Anlegen des Composite fr die Tabelle
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;

    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter pro Fahrstreifen");
    _groupParameter.setLayout(new GridLayout(1, false));
    _groupParameter.setLayoutData(gridDataGroup);
  
    _tabFolder = new CTabFolder(_groupParameter, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();        
        CTabItem itemSel = folder.getSelection();
        
        Composite cSel = (Composite)itemSel.getControl();
        
        if (((Composite)cSel).getChildren().length > 0)
        {
          CompositeParameterAlgorithmusProFahrstreifen compParSel = (CompositeParameterAlgorithmusProFahrstreifen)cSel.getChildren()[0];
        
          TableViewer tvParameter = compParSel.getTvParameter();
          tvParameter.refresh();
        }
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<String> list )
  {
  	for (int i=0; i<list.size(); ++i)
  	{
      String so = list.get(i);
     
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = so;
      tabItem.setText( tabName );
      tabItem.setData( tabName, so );

      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
    
  }

  /**
   * Methode liefert die einzelnen Composite fr die Algorithmus spezifischen Parameter zurck
   * @return Composites
   */
  public CompositeParameterAlgorithmusProFahrstreifen[] getCompositeAlg()
  {
  	return _compositeAlg;
  }

	/**
   * Methode liefert compositeAlgAllgemein zurck
   * @return compositeAlgAllgemein
   */
  public CompositeParameterAlgorithmusAllgemein getCompositeAlgAllgemein()
  {
  	return _compositeAlgAllgemein;
  }
  
}
