/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.verwaltung;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Die Klasse {@link EintragUrsachenEinheit} reprsentiert die Datenstze fr die Liste 
 * {@link ListeUrsachenEinheiten}, die in der Liste im Dialog {@link VerwaltungsSicht} dargestellt werden. <br>
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @author Krause
 * 
 */
public class EintragUrsachenEinheit implements Cloneable
{
  /**
   * Pid des UrsachenEinheitEintrags.
   */
  private String _pid = null;

  /**
   * Name des UrsachenEinheitEintrags.
   */
  private String _name = null;

  /**
   * Unter-Objekttyp der UrsachenEinheit.
   */
  private SystemObjectType _objTyp = null;
  
  /**
   * Liste fr die Eintrge der DefaultSchaltgrnde
   */
  private List<EintragSystemObjekt> _lEintragDefaultSchaltgruende = null;

  /**
   * Liste fr die Eintrge der Schaltgrnde
   */
  private List<EintragSystemObjekt> _lEintragSchaltgruende = null;

  /**
   * Liste fr die Eintrge der Strategien
   */
  private List<EintragSystemObjekt> _lEintragStrategien = null;

  /**
   * Liefert einen Clone dieser Klasse und setzt den Prfix "Kopie von " vor den Namen.
   * 
   * @return Clone dieser Klasse.
   */
  public EintragUrsachenEinheit clone()
  {
    EintragUrsachenEinheit ursachenEinheitDatensatz = new EintragUrsachenEinheit();
    String neuerName = "Kopie von " + _name;
    ursachenEinheitDatensatz.setName(neuerName);

    SystemObjectType objTyp = _objTyp;
    ursachenEinheitDatensatz.setObjTyp( objTyp );
    
    if (_lEintragDefaultSchaltgruende != null)
    {
      List<EintragSystemObjekt> list1 = new ArrayList<EintragSystemObjekt>();
      list1.addAll( _lEintragDefaultSchaltgruende );
      ursachenEinheitDatensatz.setListeEintragDefaultSchaltgruende( list1 );
    }
    
    if (_lEintragSchaltgruende != null)
    {
      List<EintragSystemObjekt> list2 = new ArrayList<EintragSystemObjekt>();
      list2.addAll( _lEintragSchaltgruende );
      ursachenEinheitDatensatz.setListeEintragSchaltgruende( list2 );
    }
    
    if (_lEintragStrategien != null)
    {
      List<EintragSystemObjekt> list3 = new ArrayList<EintragSystemObjekt>();
      list3.addAll( _lEintragStrategien );    
      ursachenEinheitDatensatz.setListeEintragStrategien( list3 );
    }
    
    return ursachenEinheitDatensatz;
  }

  /**
   * Liefert den Namen des UrsachenEinheit Datensatzes.
   * 
   * @return Name des UrsachenEinheit Datensatzes.
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Setzt den Namen des UrsachenEinheit Datensatzes.
   * 
   * @param name
   *          Name des UrsachenEinheit Datensatzes.
   */
  public void setName(String name)
  {
    this._name = name;
  }

  /**
   * Liefert die Pid der UrsachenEinheit.
   * 
   * @return Pid der UrsachenEinheit.
   */
  public String getPid()
  {
    return _pid;
  }

  /**
   * Setzt die Pid der UrsachenEinheit.
   * 
   * @param pid
   *          Pid der UrsachenEinheit.
   */
  public void setPid(String pid)
  {
    this._pid = pid;
  }

  /**
   * Liefert die Pid des Unter-Objekttyps der UrsachenEinheit.
   * 
   * @return Pid des Unter-Objekttyps der UrsachenEinheit.
   */
  public String getPidObjTyp()
  {
    return _objTyp.getPid();
  }

  /**
   * Liefert den Unter-Objekttyp der UrsachenEinheit.
   * 
   * @return Unter-Objekttyp der UrsachenEinheit.
   */
  public SystemObjectType getObjTyp()
  {
    return _objTyp;
  }
  
  /**
   * Setzt den Unter-Objekttyp der UrsachenEinheit.
   * 
   * @param objTyp
   *          Unter-Objekttyp der UrsachenEinheit.
   */
  public void setObjTyp( SystemObjectType objTyp )
  {
    _objTyp = objTyp;
  }
  
  /**
   * Liefert die Liste der DefaultSchaltgrundEintrge
   * 
   * @return Liste der DefaultSchaltgrundEintrge
   */
  public List<EintragSystemObjekt> getListeEintragDefaultSchaltgruende()
  {
    return _lEintragDefaultSchaltgruende;
  }

  /**
   * Setzt die Liste der DefaultSchaltgrundEintrge
   * 
   * @param eintragSystemObjekte
   *          Liste der DefaultSchaltgrundEintrge
   */
  public void setListeEintragDefaultSchaltgruende(List<EintragSystemObjekt> eintragSystemObjekte)
  {
    _lEintragDefaultSchaltgruende = eintragSystemObjekte;
  }

  /**
   * Liefert die Liste der SchaltgrundEintrge
   * 
   * @return Liste der SchaltgrundEintrge
   */
  public List<EintragSystemObjekt> getListeEintragSchaltgruende()
  {
    return _lEintragSchaltgruende;
  }

  /**
   * Setzt die Liste der SchaltgrundEintrge
   * 
   * @param eintragSystemObjekte
   *          Liste der SchaltgrundEintrge
   */
  public void setListeEintragSchaltgruende(List<EintragSystemObjekt> eintragSystemObjekte)
  {
    _lEintragSchaltgruende = eintragSystemObjekte;
  }
  
  /**
   * Liefert die Liste der StrategieEintrge
   * 
   * @return Liste der StrategieEintrge
   */
  public List<EintragSystemObjekt> getListeEintragStrategien()
  {
    return _lEintragStrategien;
  }

  /**
   * Setzt die Liste der StrategieEintrge
   * 
   * @param eintragSystemObjekte
   *          Liste der StrategieEintrge
   */
  public void setListeEintragStrategien(List<EintragSystemObjekt> eintragSystemObjekte)
  {
    _lEintragStrategien = eintragSystemObjekte;
  }

  /**
   * Methode prft ob die Ursacheneinheit bearbeitet wird. 
   * @return Info ob Ursacheneinheit bearbeitet wird
   */
  public boolean isVerwenden ()
  {
  	String[] pidUrsachenEinheiten = NichtZuBetrachtendeUrsachenEinheiten.getPidUrsachenEinheiten();
  	
  	boolean verwenden = true;
  	
  	if (getPidObjTyp() != null)
  	{
  		for (String pid : pidUrsachenEinheiten)
  			if (pid.equals(getPidObjTyp()))
  				verwenden = false;
  	}
  	  
  	return verwenden;
  }
}
