/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.verwaltung;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import de.bsvrz.buv.plugin.ursacheneinheit.harmobelastung.DaVKommunikation;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse zum Speichern der Parameter VerwaltungDynamischerObjekte (atg.verwaltungDynamischerObjekte)
 * @author ObertM
 */
public class ParaVerwaltungDynamischerObjekte implements ClientReceiverInterface
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static ParaVerwaltungDynamischerObjekte _instance;
  
  /**
   * Verbindung zum DaV
   */
	private ClientDavInterface _connection;

	/**
	 * Pid der AOE
	 */
	private String _pidAOE;

	/**
	 * Pid der Attributgruppe mit den Parametern
	 */
	private String _pidAtg = "atg.verwaltungDynamischerObjekte";

	/**
	 * Hashmap mit der Verwaltung der Zuordnungen. Als Key dient die Pid des Typs, als Value die
	 * Pid des Konfigurationsbereichs.
	 */
	private HashMap<String, String> _verwaltung = new HashMap<String, String>();

	/**
	 * Pid des Defaultbereichs
	 */
	private String _pidDefaultBereich;
	
  /**
   * Privater Konstruktor der Klasse
   */
  private ParaVerwaltungDynamischerObjekte ()
  {
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

  	bestimmeAOE ();
  	
  	ConfigurationArea ca = bestimmeDefaultBereich(_connection);
  	_pidDefaultBereich = ca.getPid();
  	
  	anmeldeDaV ();
  }
  
  /**
   * Methode zum Anmelden am DaV
   */
  private void anmeldeDaV ()
  {
    ResultData result = leseDatenDirektVonDaV(_pidAOE, _pidAtg, "asp.parameterSoll");
    if (result != null)
    	update(new ResultData []{ result });
    
    anmeldeLeseDaV(_pidAOE, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  }

  /**
   * Methode bestimmt die Autarke Organisations Einheit
   */
  private void bestimmeAOE()
  {
  	_pidAOE = _connection.getDataModel().getConfigurationAuthorityPid();
  }

	/**
   * Methode liefert die einzige Instanz der Klasse
   * @return einzige Instanz der Klasse
   */
	public static ParaVerwaltungDynamischerObjekte getInstance ()
  {
  	if (_instance == null)
  	{
  		synchronized (ParaVerwaltungDynamischerObjekte.class)
      {
	      if (_instance == null)
	      {
	      	_instance = new ParaVerwaltungDynamischerObjekte();
	      }
      }
  	}
  	
  	return _instance;
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode liest fr das Objekt mit der PID objPid die Attibutgruppe atgPid 
   * unter dem Aspekt aspPid direkt vom DaV und ruft sofort die Auswertemethode auf
   * (nicht ber den Thread) 
   * @param objPid Pid des Objekts 
   * @param atgPid Attributgruppe die angemeldet werden soll
   * @param aspPid Apekt der angemeldet werden soll
   */
  private ResultData leseDatenDirektVonDaV (String objPid, String atgPid, String aspPid)
  {
    ClientDavInterface verb = _connection;

    AttributeGroup atg = verb.getDataModel().getAttributeGroup(atgPid);

    Aspect asp = verb.getDataModel().getAspect(aspPid);

    DataDescription dd = new DataDescription(atg, asp);

    SystemObject obj = verb.getDataModel().getObject(objPid); 

    try
    {
      ResultData data = verb.getData( obj, dd, 10000 );
      
      ResultData[] results = new ResultData [] {data};
      
      if (results.length == 1)
      	return results [0];
    }
    catch (Exception e) 
    {
      e.printStackTrace();
    }
    
    return null;
  }

  /**
   * Update Methode des DaV
   */
	@Override
  public void update(ResultData[] arg0)
  {
		for (ResultData result : arg0)
		{
			if (result.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
			{
			  if (result.hasData())
			  {
			    Data data = result.getData();

			    auswertePara (data);
			  }
			}
		}
  }

	/**
	 * Methode zum Auswerten der Parameter
	 * @param data empfangene Daten des DaV
	 */
	private void auswertePara(Data data)
  {
		Array a = data.getArray("ZuordnungDynamischerObjektTypZuKB");
		int anzahl = a.getLength();

		synchronized (_verwaltung)
    {
			_verwaltung.clear();
			
			for (int i=0; i<anzahl; ++i)
			{
			  String typ = a.getItem(i).getReferenceValue("DynamischerTypReferenz").getSystemObjectPid();
			  String kb  = a.getItem(i).getReferenceValue("KonfigurationsBereichReferenz").getSystemObjectPid();
	
			  synchronized (_verwaltung)
	      {
		      _verwaltung.put(typ, kb);
	      }
			}
    }
  }
	
	/**
	 * Method prft ob fr eine bestimmten dynamischen Typ ein Konfigurationsbereich definiert ist.
	 * @param pidTyp Pid des Typs
	 * @return Info ob Konfigurationsbreich definiert ist
	 */
	public boolean isKonfigurationsBereichDefiniert (String pidTyp)
	{
		synchronized (_verwaltung)
    {
	    return _verwaltung.containsKey(pidTyp);
    }
	}

	/**
	 * Methode erzeugt eine Zuordnung des dynamischen Typs zum Defaultkonfigurationsbereich
	 * @param pidTyp Pid des Typs
	 */
	public void erzeugeZuordnungZuDefaultBereich (String pidTyp)
	{
		synchronized (_verwaltung)
    {
	    _verwaltung.put(pidTyp, _pidDefaultBereich);
    }
	}

	/**
	 * Methode lscht eine Zuordnung des dynamischen Typs zum Defaultkonfigurationsbereich
	 * @param pidTyp Pid des Typs
	 */
	public void loescheZuordnungZuDefaultBereich (String pidTyp)
	{
		synchronized (_verwaltung)
    {
	    _verwaltung.remove(pidTyp);
    }
	}

  /**
   * Bestimmt den Default-Konfigurationsbereich des DaV
   * 
   * @param connection
   * @return Konfigurationsbereich
   */
  private ConfigurationArea bestimmeDefaultBereich(ClientDavInterface connection)
  {
    Data data = connection.getDataModel().getConfigurationAuthority().getConfigurationData(
        connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften"));

    if (data != null)
    {
      TextArray array = data.getTextArray("defaultBereich");

      if (array.getLength() == 1)
      {
        return (ConfigurationArea)connection.getDataModel().getObject(array.getText(0));
      }
    }

    return null;
  }

	/**
   * Methode liefert pidAOE zurck
   * @return pidAOE
   */
  public String getPidAOE()
  {
  	return _pidAOE;
  }

	/**
   * Methode setzt die Variable pidAOE
   * @param pidAOE zu bernehmender Wert fr pidAOE
   */
  public void setPidAOE(String pidAOE)
  {
  	_pidAOE = pidAOE;
  }

	/**
   * Methode liefert pidAtg zurck
   * @return pidAtg
   */
  public String getPidAtg()
  {
  	return _pidAtg;
  }

	/**
   * Methode setzt die Variable pidAtg
   * @param pidAtg zu bernehmender Wert fr pidAtg
   */
  public void setPidAtg(String pidAtg)
  {
  	_pidAtg = pidAtg;
  }

	/**
   * Methode liefert pidDefaultBereich zurck
   * @return pidDefaultBereich
   */
  public String getPidDefaultBereich()
  {
  	return _pidDefaultBereich;
  }

	/**
   * Methode setzt die Variable pidDefaultBereich
   * @param pidDefaultBereich zu bernehmender Wert fr pidDefaultBereich
   */
  public void setPidDefaultBereich(String pidDefaultBereich)
  {
  	_pidDefaultBereich = pidDefaultBereich;
  }

  /**
   * Methode erzeugt eine Liste mit den Daten fr den DaV
   * @return Liste mit Daten fr den DaV
   */
  public List<ResultData> getDatenFuerDaV () 
  {
    List<ResultData> list = new ArrayList<ResultData>();
    
    DataModel dataModell = _connection.getDataModel();
    
    Aspect asp = dataModell.getAspect( "asp.parameterVorgabe" );
    AttributeGroup atg = dataModell.getAttributeGroup( _pidAtg );

    SystemObject so = dataModell.getObject(_pidAOE);      
    ResultData rd = new ResultData(so, new DataDescription(atg, asp), new Date().getTime(), erzeugeAttributGruppe());
    list.add(rd);
    
    return list;
  }

  /**
   * Methode erzeugt eine Attributgruppe mit den Parametern fr den DaV
   * @return Attributgruppe fr DaV
   */
  private Data erzeugeAttributGruppe ()
  {
    DataModel dataModel = _connection.getDataModel();

    AttributeGroup atg = dataModel.getAttributeGroup( _pidAtg );
    Data data = _connection.createData( atg );

    synchronized (_verwaltung)
    {
	    Array a = data.getArray("ZuordnungDynamischerObjektTypZuKB");
	    a.setLength(_verwaltung.entrySet().size());
	    
	    int i=0;
	    for (Entry<String, String> entry : _verwaltung.entrySet())
	    {
	    	a.getItem(i).getReferenceValue("DynamischerTypReferenz").setSystemObject (dataModel.getObject(entry.getKey()));
	    	a.getItem(i).getReferenceValue("KonfigurationsBereichReferenz").setSystemObject (dataModel.getObject(entry.getValue()));
	    	
	    	i++;
	    }
    }
    
    return data;
  }
}
