/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.zeitdauer;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;


import de.bsvrz.buv.plugin.ursacheneinheit.zeitdauer.MyZeitDauer.SelektierInfo;
import de.bsvrz.buv.plugin.ursacheneinheit.zeitdauer.MyZeitDauer.ValidierungsInfo;
import de.bsvrz.buv.plugin.ursacheneinheit.zeitdauer.MyZeitDauer.ZeitKlasse;
import de.bsvrz.buv.rw.basislib.kalender.ZeitDauerGranularitaet;


public class MyCompositeZeitDauer extends Composite
{
	/**
	 * Debug-Flag
	 */
	private boolean _debug = false;

	/**
	 * Feld zur Eingabe der Zeitdauer
	 */
	private Text _textEingabe = null;

	/**
	 * Erhhen der selektierten Zeit
	 */
	private Button _schaltflaechePlus = null;

	/**
	 * Verringern der selektierten Zeit
	 */
	private Button _schaltflaecheMinus = null;

	/**
	 * Dialog zur Erfassung der Zeitdauer
	 */
	private Button _schaltflaecheZeitDauerWaehlerOeffnen = null;

	/**
	 * Validierung durchfhren
	 */
	private Button _schaltflaecheValidieren = null;

	/**
	 * Vorgabe
	 */
	private String _vorgabe = null;

	// /**
	// * Ausgangszeitpunkt
	// */
	// private long _ausgangsZeitpunkt = 0;

	@SuppressWarnings("unused")
	private MyCompositeZeitDauer _this = null;

	/**
	 * Zeitdauer-Hilfsklasse
	 */
	private MyZeitDauer _zeitdauer = null;

	public MyCompositeZeitDauer(Composite parent, int style,
			long ausgangsZeitpunkt, boolean kurzDarstellung)
	{
		super(parent, style);
		_this = this;

		_zeitdauer = new MyZeitDauer(ausgangsZeitpunkt);
		_zeitdauer.setAllesEditierbar(true);
		_zeitdauer.setKurzdarstellung(kurzDarstellung);

		init();
		listenerAnhaengen();
	}

	public MyCompositeZeitDauer(Composite parent, int style,
			long ausgangsZeitpunkt, long zeitdauer, boolean kurzDarstellung)
	{
		super(parent, style);
		_this = this;

		_zeitdauer = new MyZeitDauer(ausgangsZeitpunkt);
		_zeitdauer.setAllesEditierbar(true);
		_zeitdauer.setKurzdarstellung(kurzDarstellung);

		init();
		listenerAnhaengen();

		setZeitDauerInBezugAufAnfangsZeitpunkt(zeitdauer);
	}

	public MyCompositeZeitDauer(Composite parent, int style, String vorgabe,
			long ausgangsZeitpunkt)
	{
		super(parent, style);
		_this = this;

		_zeitdauer = new MyZeitDauer(ausgangsZeitpunkt);

		init();

		_textEingabe.setText(vorgabe);
		listenerAnhaengen();
	}

	private void listenerAnhaengen()
	{
		_textEingabe.addListener(SWT.Verify, new Listener()
		{

			@Override
			public void handleEvent(Event e)
			{
//				System.out.println("Event: (" + e.type + ") = " + e.keyCode);
				// System.out.println("Start: " + e.start);
				// System.out.println("Ende: " + e.end);

				e.doit = false;

				if ((e.keyCode != SWT.TAB) && (e.keyCode != SWT.ARROW_LEFT)
						&& (e.keyCode != SWT.ARROW_RIGHT)) // berlesen der Tab-Taste
				// if (e.keyCode != SWT.TAB) // berlesen der Tab-Taste
				{
					_schaltflaechePlus.setVisible(false);
					_schaltflaecheMinus.setVisible(false);					

					/*
					 * Verarbeitung von Texten, die z.B. per Copy & Paste eingefgt
					 * wurden.
					 */
					if (e.text.length() > 1)
					{
						if (_zeitdauer.verifyEingabe(e.text))
						{
							e.doit = true;
						}
					} else
					{
						if (_vorgabe == null)
						{
							if (_zeitdauer.verifyEingabe(e.text, e.start, _textEingabe
									.getText(), true))
							{
								e.doit = true;
							}
						} else if (_vorgabe != null)
						{
							if (_zeitdauer.verifyEingabe(e.text, e.start, _textEingabe
									.getText(), false))
							{
								e.doit = true;
							}
						}
					}
				}

			}

		});

		_textEingabe.addMouseListener(new MouseAdapter()
		{
			public void mouseDown(MouseEvent e)
			{
				aktionMausSelektion();

			}
		});

		_textEingabe.addTraverseListener(new TraverseListener()
		{

			@Override
			public void keyTraversed(TraverseEvent e)
			{
				System.out.println("Traverse KeyCode: " + e.keyCode + " Detail: "
						+ e.detail);

				// if (e.keyCode == SWT.TAB)
				// {
				// selektiereTab(true);
				// _textEingabe.setFocus();
				// e.doit = false;
				// }

				// System.err.println("=>Detail: " + e.detail);
				if (e.detail == SWT.TRAVERSE_TAB_NEXT)
				{
					selektiereTab(true);
					_textEingabe.setFocus();
					e.doit = false;
				} else if (e.detail == SWT.TRAVERSE_TAB_PREVIOUS)
				{
					selektiereTab(false);
					_textEingabe.setFocus();
					e.doit = false;
				}

			}

		});

	}

	/**
	 * Aktion hinter der Selektion eines Zahlenfeldes mit der Maus
	 */
	private void aktionMausSelektion()
	{
		int pos = _textEingabe.getCaretPosition();
		// System.out.println("Position in Text: " + pos);
		int[] bestimmeMarkierung = _zeitdauer.bestimmeMarkierung(_textEingabe
				.getText(), pos);
		if (bestimmeMarkierung != null)
		{
			if (bestimmeMarkierung.length == 2)
			{
				/*
				 * Setzen der Selektion
				 */
				_textEingabe.setSelection(bestimmeMarkierung[0],
						bestimmeMarkierung[1] + 1);

				_schaltflaechePlus.setVisible(true);
				_schaltflaecheMinus.setVisible(true);
			} else
			{
				_schaltflaechePlus.setVisible(false);
				_schaltflaecheMinus.setVisible(false);
			}
		} else
		{
			_schaltflaechePlus.setVisible(false);
			_schaltflaecheMinus.setVisible(false);
		}
	}

	private void init()
	{   
		_textEingabe = new Text(this, SWT.BORDER);
		_textEingabe.addListener(SWT.DefaultSelection, new Listener()
		{
			public void handleEvent(Event e)
			{
				aktionNormalisieren();
			}
		});

		_schaltflaecheZeitDauerWaehlerOeffnen = new Button(this, SWT.ARROW | SWT.DOWN);
		_schaltflaecheZeitDauerWaehlerOeffnen
				.addMouseListener(new org.eclipse.swt.events.MouseAdapter()
				{
					public void mouseDown(org.eclipse.swt.events.MouseEvent e)
					{
						aktionZeitDauerWaehler();
					}
				});

		_schaltflaechePlus = new Button(this, SWT.NONE);
		_schaltflaechePlus.setText("+");
		_schaltflaechePlus
				.addMouseListener(new org.eclipse.swt.events.MouseAdapter()
				{
					public void mouseDown(org.eclipse.swt.events.MouseEvent e)
					{
						aktionPlus();
					}
				});
		_schaltflaechePlus.setVisible(false);

		_schaltflaecheMinus = new Button(this, SWT.NONE);
		_schaltflaecheMinus.setText("-");
		_schaltflaecheMinus
				.addMouseListener(new org.eclipse.swt.events.MouseAdapter()
				{
					public void mouseDown(org.eclipse.swt.events.MouseEvent e)
					{
						aktionMinus();
					}
				});
		_schaltflaecheMinus.setVisible(false);

		if (_debug)
		{
			_schaltflaecheValidieren = new Button(this, SWT.PUSH);
			_schaltflaecheValidieren.setText("VAL");
			_schaltflaecheValidieren.addMouseListener(new MouseAdapter()
			{
				public void mouseDown(org.eclipse.swt.events.MouseEvent e)
				{
					aktionValidieren();
				}
			});
		}

		GridData gridDataText = new GridData();
		gridDataText.horizontalAlignment = GridData.FILL;
		gridDataText.grabExcessHorizontalSpace = true;
		gridDataText.verticalAlignment = GridData.CENTER;
		gridDataText.horizontalIndent = 0;
		_textEingabe.setLayoutData(gridDataText);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 4;

		gridLayout.horizontalSpacing = 0;

		this.setLayout(gridLayout);
		// setSize(new Point(300, 200));
		
		this.setJahreDarstellen(false);
    this.setMonateDarstellen(false);
    this.setTageDarstellen(false);
    this.setStundenDarstellen(true);
    this.setMinutenDarstellen(true);
    this.setSekundenDarstellen(true);
    this.setTausendstelDarstellen(false);
	}

	/**
	 * Fhrt die Selektion des nchsten Zahlenfeldet mit Hilfe der TAB-Taste
	 * durch.
	 * 
	 * @param naechstesFeld
	 *          Wenn <code>true</code>, wenn wird das nchste Zahlenfeld nach
	 *          rechts hin gesucht, bei <code>false</code> wird in linker
	 *          Richtung vorgegangen.
	 */
	private void selektiereTab(boolean naechstesFeld)
	{
		// System.out.println("Selektiere Tab: ");
		// textEingabe.setText("123 Jahre");

		int caretPosition = _textEingabe.getCaretPosition();

		SelektierInfo naechstesZahlenfeld = null;

		if (naechstesFeld)
		{
			naechstesZahlenfeld = _zeitdauer.bestimmeNaechstesZahlenfeld(_textEingabe
					.getText(), caretPosition, _textEingabe.getSelection().x,
					_textEingabe.getSelection().y);
		} else
		{
			naechstesZahlenfeld = _zeitdauer.bestimmeVorhergehendesZahlenfeld(
					_textEingabe.getText(), caretPosition, _textEingabe.getSelection().x,
					_textEingabe.getSelection().y);
		}

		if (naechstesZahlenfeld != null)
		{
			_textEingabe.setSelection(naechstesZahlenfeld.getStartPos(),
					naechstesZahlenfeld.getEndPos() + 1);
			_schaltflaechePlus.setVisible(true);
			_schaltflaecheMinus.setVisible(true);
		}
		_textEingabe.setFocus();
	}
	
	
	private void aktionZeitDauerWaehler()
	{
		aktionNormalisieren();
		MyZeitDauerDialog zdd = new MyZeitDauerDialog(PlatformUI.getWorkbench().getDisplay(), this);
		zdd.oeffnen();
	}

	/**
	 * Fhrt auf einem Zahlenfeld einer Zeitklasse die Addition von 1 durch.
	 */
	private void aktionPlus()
	{
		String text = _textEingabe.getText();
		int caretPosition = _textEingabe.getCaretPosition();
		String selectionText = _textEingabe.getSelectionText();
		SelektierInfo selektierInfo = _zeitdauer.wertAendern(text, selectionText,
				caretPosition, 1);

		if (selektierInfo != null)
		{
			_textEingabe.setText(selektierInfo.getNeuerText());
			_textEingabe.setSelection(selektierInfo.getStartPos(), selektierInfo
					.getEndPos() + 1);
		}

		_schaltflaechePlus.setVisible(true);
		_schaltflaecheMinus.setVisible(true);

		_textEingabe.setFocus();

		// aktionNormalisieren();
	}

	/**
	 * Fhrt auf einem Zahlenfeld einer Zeitklasse die Subtraktion von 1 durch.
	 */
	private void aktionMinus()
	{
		String text = _textEingabe.getText();
		int caretPosition = _textEingabe.getCaretPosition();
		String selectionText = _textEingabe.getSelectionText();

		int wert = -1;
		SelektierInfo selektierInfo = _zeitdauer.wertAendern(text, selectionText,
				caretPosition, wert);

		if (selektierInfo != null)
		{
			_textEingabe.setText(selektierInfo.getNeuerText());
			_textEingabe.setSelection(selektierInfo.getStartPos(), selektierInfo
					.getEndPos() + 1);
		}

		_schaltflaechePlus.setVisible(true);
		_schaltflaecheMinus.setVisible(true);

		_textEingabe.setFocus();

		// aktionNormalisieren();
	}

	/**
	 * Enthlt das Textfeld einen korrekten Zeitdauer String, so wird dieser
	 * Validiert und entsprechend dargestellt.
	 */
	public void aktionValidieren()
	{
		if (_zeitdauer != null)
		{
			if (_textEingabe.getText().length() > 0)
			{
				ValidierungsInfo val = _zeitdauer.validiereFelder(_textEingabe
						.getText());

				if (validierungsFehlerauswertung(val))
				{
					_textEingabe.setText(val.getFeldText());
				}
			}
		}
	}

	/**
	 * Normalisiert den Zeitdauer-String des Textfeldes an Hand des
	 * gregorianischen Kalenders. Es wird vorher eine Validierung durchgefhrt.
	 */
	public void aktionNormalisieren()
	{
		if (_zeitdauer != null)
		{
			if (_textEingabe.getText().length() > 0)
			{
				ValidierungsInfo validierungsInfo = _zeitdauer
						.validiereFelder(_textEingabe.getText());

				if (validierungsFehlerauswertung(validierungsInfo))
				{
					String normalisiert = _zeitdauer.normalisieren(validierungsInfo
							.getFeldText());
					_textEingabe.setText(normalisiert);
				}
			}
		}
	}

	private boolean validierungsFehlerauswertung(ValidierungsInfo validierungsInfo)
	{
		boolean re = true;

		String meldung = "";

		LinkedList<String> nichtBestimmbareFelder = validierungsInfo
				.getNichtBestimmbareFelder();
		if (nichtBestimmbareFelder != null)
		{
			if (nichtBestimmbareFelder.size() > 0)
			{
				meldung = meldung + "Nicht bestimmbare Felder:\n";
				meldung = meldung + "=========================\n";
				int i = 0;
				Iterator<String> it = nichtBestimmbareFelder.iterator();
				while (it.hasNext())
				{
					String string = it.next();
					meldung = meldung + i + ". Feld: " + string + "\n";
					i++;
				}
			}
		}

		String sonstigeInfos = validierungsInfo.getSonstigeInfos();
		if (sonstigeInfos != null)
		{
			if (sonstigeInfos.length() > 0)
			{
				meldung = meldung + "\n";
				meldung = meldung + "Sonstige Informationen:\n";
				meldung = meldung + "=======================\n";
				meldung = meldung + sonstigeInfos;
			}
		}

		if (meldung.length() > 0)
		{
			meldung = "Die Validierung konnte nicht komplett durchgefhrt werden,\n"
					+ "da einige Angaben unvollstndig oder fehlerhaft sind. Nachfolgend\n"
					+ "erhalten Sie eine Auflistung der unvollstndigen oder fehlerhafen\n"
					+ "Angaben:\n" + meldung;

			meldung = meldung
					+ "\n\nDrcken Sie OK um fortzufahren, fehlerhafte Eingaben gehen\n"
					+ "dabei verloren, oder Abbrechen um Ihre fehlerhaften Eingaben\n"
					+ "zu korrigieren.";

			MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.CANCEL | SWT.ICON_QUESTION);
			msg.setText("Validierung nicht erfolgreich");
			msg.setMessage(meldung);

			int open = msg.open();

			if (open == SWT.OK)
			{
				re = true;
			} else if (open == SWT.CANCEL)
			{
				re = false;
			}
		}

		return re;
	}

	/**
	 * Liefert den Ausgangszeitpunkt in Millisekunden seit dem 1.1.1970.
	 * 
	 * @return Ausgangszeitpunkt
	 */
	public long getAusgangsZeitpunkt()
	{
		return _zeitdauer.getAusgangszeitpunkt();
	}

	/**
	 * Setzt den Ausgangszeitpunkt in Millisekunden seit dem 1.1.1970. Wird der
	 * Wert gendert, muss der Entwickler auch die Zeitdauer erneut setzen.
	 * 
	 * @param ausgangsZeitpunkt
	 *          Ausgangszeitpunkt
	 */
	public void setAusgangsZeitpunkt(long ausgangsZeitpunkt)
	{
		_zeitdauer.setAusgangsZeitPunkt(ausgangsZeitpunkt);
	}

	/**
	 * Prft, ob die Kurzdarstellung gewhlt wurde oder nicht.
	 * 
	 * @return Liefert <code>true</code>, wenn die Kurzdarstellung gewhlt
	 *         wurde, sonst <code>false</code>.
	 */
	public boolean istKurzDarstellung()
	{
		return _zeitdauer.istKurzdarstellung();
	}

	/**
	 * Setzt, ob die Darstellung in der Kurzdarstellung erfolgen soll. Per Default
	 * ist dieser Wert false.
	 * 
	 * @param kurzDarstellung
	 *          Muss <code>true</code> sein, wenn eine Kurzdarstellung erfolgen
	 *          soll, sonst <code>false</code>.
	 */
	public void setKurzDarstellung(boolean kurzDarstellung)
	{
		_zeitdauer.setKurzdarstellung(kurzDarstellung);
	}

	/**
	 * Liefert die Zeitdauer in Millisekunden zurck.
	 * 
	 * @return Zeitdauer in Millisekunden
	 */
	public long getZeitDauer()
	{
		long re = -1;

		if ((_zeitdauer != null) && (_textEingabe.getText().length() > 0))
		{
			re = _zeitdauer.getZeitdauerInBezugAufAnfangszeitpunkt(_textEingabe
					.getText());
		}

		return re;
	}

	/**
	 * Setzt die Zeitdauer in Bezug zum gesetzten Anfangszeitpunkt.
	 * 
	 * @param zeitDauer
	 *          Zeitdauer in Millisekunden
	 */
	public void setZeitDauerInBezugAufAnfangsZeitpunkt(long zeitDauer)
	{
		String inBezugAufAnfangszeitpunkt = _zeitdauer
				.setZeitdauerInBezugAufAnfangszeitpunkt(_zeitdauer
						.getAusgangszeitpunkt(), zeitDauer);
		_textEingabe.setText(inBezugAufAnfangszeitpunkt);
	}

	/**
	 * Setzt auf direktem Wege die Zeitdauer, in dem Jahre, Monate, Tage, Stunden,
	 * Minuten, Sekunden und Tausendstel der Zeitdauer direkt angegebene werden.
	 * Die Werte werden ausgehend vom gewhlten Ausgangszeitpunkt normalisiert.
	 * 
	 * @param jahre
	 *          Anzahl der Jahre
	 * @param monate
	 *          Anzahl der Monate
	 * @param tage
	 *          Anzahl der Tage
	 * @param stunden
	 *          Anzahl der Stunden
	 * @param minuten
	 *          Anzahl der Minuten
	 * @param sekunden
	 *          Anzahl der Sekunden
	 * @param tausendstel
	 *          Anzahl der Tausendstel
	 */
	public void setZeitDauer(int jahre, int monate, int tage, int stunden,
			int minuten, int sekunden, int tausendstel)
	{
		String zeitString = _zeitdauer.baueZeitString(jahre, monate, tage, stunden,
				minuten, sekunden, tausendstel);
		_textEingabe.setText(_zeitdauer.normalisieren(zeitString));
	}

	/**
	 * Liefert die Anzahl der Jahre zurck.
	 * 
	 * @return Anzahl der Jahre oder -1, wenn keine Jahre gesetzt wurden.
	 */
	public int getJahre()
	{
		return _zeitdauer.getJahre(_textEingabe.getText());
	}
	
	public void setJahre(int jahre)
	{
		String wert = _zeitdauer.setJahre(_textEingabe.getText(), jahre);
		_textEingabe.setText(wert);
	}

	/**
	 * Liefert die Anzahl der Monate zurck.
	 * 
	 * @return Anzahl der Monate oder -1, wenn keine Monate gesetzt wurden.
	 */
	public int getMonate()
	{
		return _zeitdauer.getMonate(_textEingabe.getText());
	}
	
	public void setMonate(int monate)
	{
		String wert = _zeitdauer.setMonate(_textEingabe.getText(), monate);
		_textEingabe.setText(wert);
	}

	/**
	 * Liefert die Anzahl der Tage zurck.
	 * 
	 * @return Anzahl der Tage oder -1, wenn keine Tage gesetzt wurden.
	 */
	public int getTage()
	{
		return _zeitdauer.getTage(_textEingabe.getText());
	}
	
	public void setTage(int tage)
	{
		String wert = _zeitdauer.setTage(_textEingabe.getText(), tage);
		_textEingabe.setText(wert);
	}

	/**
	 * Liefert die Anzahl der Stunden zurck.
	 * 
	 * @return Anzahl der Stunden oder -1, wenn keine Stunden gesetzt wurden.
	 */
	public int getStunden()
	{
		return _zeitdauer.getStunden(_textEingabe.getText());
	}
	
	public void setStunden(int stunden)
	{
		String wert = _zeitdauer.setStunden(_textEingabe.getText(), stunden);
		_textEingabe.setText(wert);
	}

	/**
	 * Liefert die Anzahl der Minuten zurck.
	 * 
	 * @return Anzahl der Minuten oder -1, wenn keine Minuten gesetzt wurden.
	 */
	public int getMinuten()
	{
		return _zeitdauer.getMinuten(_textEingabe.getText());
	}
	
	public void setMinuten(int minuten)
	{
		String wert = _zeitdauer.setMinuten(_textEingabe.getText(), minuten);
		_textEingabe.setText(wert);
	}

	/**
	 * Liefert die Anzahl der Sekunden zurck.
	 * 
	 * @return Anzahl der Sekunden oder -1, wenn keine Sekunden gesetzt wurden.
	 */
	public int getSekunden()
	{
		return _zeitdauer.getSekunden(_textEingabe.getText());
	}
	
	public void setSekunden(int sekunden)
	{
		String wert = _zeitdauer.setSekunden(_textEingabe.getText(), sekunden);
		_textEingabe.setText(wert);
	}

	/**
	 * Liefert die Anzahl der Tausendstel zurck.
	 * 
	 * @return Anzahl der Tausendstel oder -1, wenn keine Tausendstel gesetzt
	 *         wurden.
	 */
	public int getTausendstel()
	{
		return _zeitdauer.getTausendstel(_textEingabe.getText());
	}
	
	public void setTausendstel(int tausendstell)
	{
		String wert = _zeitdauer.setTausendstel(_textEingabe.getText(), tausendstell);
		_textEingabe.setText(wert);
	}

	/**
	 * Prft, ob Jahre dargestellt werden sollen.
	 * 
	 * @return Liefert <code>true</code>, wenn Jahre dargestellt werden sollen,
	 *         sonst <code>false</code>.
	 */
	public boolean istJahreDarstellen()
	{
		return _zeitdauer._jahreDarstellen;
	}

	/**
	 * Setzt, ob Jahre dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *          Wenn <code>true</code>, dann werden Jahre dargestellt, bei
	 *          <code>false</code> nicht.
	 */
	public void setJahreDarstellen(boolean darstellen)
	{
		this._zeitdauer._jahreDarstellen = darstellen;
		aktionNormalisieren();
	}

	/**
	 * Prft, ob Monate dargestellt werden sollen.
	 * 
	 * @return Liefert <code>true</code>, wenn Monate dargestellt werden
	 *         sollen, sonst <code>false</code>.
	 */
	public boolean istMonateDarstellen()
	{
		return _zeitdauer._monateDarstellen;
	}

	/**
	 * Setzt, ob Monate dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *          Wenn <code>true</code>, dann werden Monate dargestellt, bei
	 *          <code>false</code> nicht.
	 */
	public void setMonateDarstellen(boolean darstellen)
	{
		this._zeitdauer._monateDarstellen = darstellen;
		aktionNormalisieren();
	}

	/**
	 * Prft, ob Tage dargestellt werden sollen.
	 * 
	 * @return Liefert <code>true</code>, wenn Tage dargestellt werden sollen,
	 *         sonst <code>false</code>.
	 */
	public boolean istTageDarstellen()
	{
		return _zeitdauer._tageDarstellen;
	}

	/**
	 * Setzt, ob Tage dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *          Wenn <code>true</code>, dann werden Tage dargestellt, bei
	 *          <code>false</code> nicht.
	 */
	public void setTageDarstellen(boolean darstellen)
	{
		this._zeitdauer._tageDarstellen = darstellen;
		aktionNormalisieren();
	}

	/**
	 * Prft, ob Stunden dargestellt werden sollen.
	 * 
	 * @return Liefert <code>true</code>, wenn Stunden dargestellt werden
	 *         sollen, sonst <code>false</code>.
	 */
	public boolean istStundenDarstellen()
	{
		return _zeitdauer._stundenDarstellen;
	}

	/**
	 * Setzt, ob Stunden dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *          Wenn <code>true</code>, dann werden Stunden dargestellt, bei
	 *          <code>false</code> nicht.
	 */
	public void setStundenDarstellen(boolean darstellen)
	{
		this._zeitdauer._stundenDarstellen = darstellen;
		aktionNormalisieren();
	}

	/**
	 * Prft, ob Minuten dargestellt werden sollen.
	 * 
	 * @return Liefert <code>true</code>, wenn Minuten dargestellt werden
	 *         sollen, sonst <code>false</code>.
	 */
	public boolean istMinutenDarstellen()
	{
		return _zeitdauer._minutenDarstellen;
	}

	/**
	 * Setzt, ob Minuten dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *          Wenn <code>true</code>, dann werden Minuten dargestellt, bei
	 *          <code>false</code> nicht.
	 */
	public void setMinutenDarstellen(boolean darstellen)
	{
		this._zeitdauer._minutenDarstellen = darstellen;
		aktionNormalisieren();
	}

	/**
	 * Prft, ob Sekunden dargestellt werden sollen.
	 * 
	 * @return Liefert <code>true</code>, wenn Sekunden dargestellt werden
	 *         sollen, sonst <code>false</code>.
	 */
	public boolean istSekundenDarstellen()
	{
		return _zeitdauer._sekundenDarstellen;
	}

	/**
	 * Setzt, ob Sekunden dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *          Wenn <code>true</code>, dann werden Sekunden dargestellt, bei
	 *          <code>false</code> nicht.
	 */
	public void setSekundenDarstellen(boolean darstellen)
	{
		this._zeitdauer._sekundenDarstellen = darstellen;
		aktionNormalisieren();
	}

	/**
	 * Prft, ob Tausendstel dargestellt werden sollen.
	 * 
	 * @return Liefert <code>true</code>, wenn Tausendstel dargestellt werden
	 *         sollen, sonst <code>false</code>.
	 */
	public boolean istTausendstelDarstellen()
	{
		return _zeitdauer._tausendstelDarstellen;
	}

	/**
	 * Setzt, ob Tausendstel dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *          Wenn <code>true</code>, dann werden Tausendstel dargestellt,
	 *          bei <code>false</code> nicht.
	 */
	public void setTausendstelDarstellen(boolean darstellen)
	{
		this._zeitdauer._tausendstelDarstellen = darstellen;
		aktionNormalisieren();
	}

	/**
	 * Setzt welche Zeitklassen dargestellt werden sollen. Wird diese Funktion
	 * nachtrglich aufgerufen sollte eine Normalisierung durchgefhrt werden.
	 * 
	 * @param jahre
	 *          Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param monate
	 *          Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param tage
	 *          Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param stunden
	 *          Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param minuten
	 *          Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param sekunden
	 *          Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param tausendstel
	 *          Werden bei <code>true</code> dargestellt, sonst nicht.
	 */
	public void setZeitklassenDarstellen(boolean jahre, boolean monate,
			boolean tage, boolean stunden, boolean minuten, boolean sekunden,
			boolean tausendstel)
	{
		this._zeitdauer._jahreDarstellen = jahre;
		this._zeitdauer._monateDarstellen = monate;
		this._zeitdauer._tageDarstellen = tage;
		this._zeitdauer._stundenDarstellen = stunden;
		this._zeitdauer._minutenDarstellen = minuten;
		this._zeitdauer._sekundenDarstellen = sekunden;
		this._zeitdauer._tausendstelDarstellen = tausendstel;
		aktionNormalisieren();
	}

	/**
	 * Prft, ob der Text der Zeitklassen im Eingabefeld editierbar ist oder
	 * nicht.
	 * 
	 * @return Liefert <code>true</code>, wenn der Text im Eingabefeld
	 *         editierbar ist, sonst <code>false</code>.
	 */
	public boolean istZeitKlassenTextEditierbar()
	{
		return _zeitdauer.istAllesEditierbar();
	}

	/**
	 * Setzt die Editierbarkeit der Zeitklassentexte im Eingabefeld.
	 * 
	 * @param editierbar
	 *          Wenn <code>true</code>, dann ist der Text der Zeitklassentexte
	 *          editierbar, bei <code>false</code> nicht.
	 */
	public void setZeitKlassenTextEditierbar(boolean editierbar)
	{
		this._zeitdauer.setAllesEditierbar(editierbar);
	}

	/**
	 * Liefert die Granularitt dieses Eingabefeldes.
	 * 
	 * @return
	 */
	public ZeitDauerGranularitaet getGranularitaet()
	{
		return _zeitdauer.getGranularitaet(_textEingabe.getText());
	}
	
	
	/**
	 * Liefert die {@link ZeitKlasse} Jahr.
	 * 
	 * @return Jahr
	 */
	public ZeitKlasse getZeitKlasseJahr()
	{
		return _zeitdauer.getZeitKlasseJahr();
	}

	/**
	 * Liefert die {@link ZeitKlasse} Monat.
	 * 
	 * @return Monat
	 */
	public ZeitKlasse getZeitKlasseMonat()
	{
		return _zeitdauer.getZeitKlasseMonat();
	}

	/**
	 * Liefert die {@link ZeitKlasse} Tag.
	 * 
	 * @return Tag
	 */
	public ZeitKlasse getZeitKlasseTag()
	{
		return _zeitdauer.getZeitKlasseTag();
	}

	/**
	 * Liefert die {@link ZeitKlasse} Stunde.
	 * 
	 * @return Stunde
	 */
	public ZeitKlasse getZeitKlasseStunde()
	{
		return _zeitdauer.getZeitKlasseStunde();
	}

	/**
	 * Liefert die {@link ZeitKlasse} Minute.
	 * 
	 * @return Minute
	 */
	public ZeitKlasse getZeitKlasseMinute()
	{
		return _zeitdauer.getZeitKlasseMinute();
	}

	/**
	 * Liefert die {@link ZeitKlasse} Sekunde.
	 * 
	 * @return Sekunde
	 */
	public ZeitKlasse getZeitKlasseSekunde()
	{
		return _zeitdauer.getZeitKlasseSekunde();
	}

	/**
	 * Liefert die {@link ZeitKlasse} Tausendstel.
	 * 
	 * @return Tausendstel.
	 */
	public ZeitKlasse getZeitKlasseTausendstel()
	{
		return _zeitdauer.getZeitKlasseTausendstel();
	}

}
