/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas.compsites;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreas;

/**
 * Erweitertes {@link Composite} zur Auswahl eines Konfigurationsbereiches aus
 * allen vorhandenen.
 * 
 * @author HofmannJ
 * 
 */
public class CompositeConfigurationAreaChooser extends Composite
{
	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * Enumerator zur Auswahl der Selektion: Single oder Multi
	 */
	private EnumSelection _enumSelection = null;

	/**
	 * Liste zur Darstellung der Konfigurationsbereiche.
	 */
	private List _listConfigurationAreas;

	/**
	 * {@link ListViewer} fr die Konfigurationsbereiche.
	 */
	private ListViewer _lvConfigurationAreas;

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 * @param style
	 *          SWT-Stil
	 * @param connection
	 *          Verbindung zum DaV
	 * @param enumSelection
	 *          Gibt an, ob Einzel- oder Mehrfachselektion mglich ist.
	 */
	public CompositeConfigurationAreaChooser(Composite parent, int style,
			ClientDavInterface connection, EnumSelection enumSelection)
	{
		super(parent, style);
		this._connection = connection;
		this._enumSelection = enumSelection;
		createContent();
	}

	/**
	 * Erzeugt die Darstellung
	 */
	private void createContent()
	{
		if ((_enumSelection == null) || _enumSelection.equals(EnumSelection.MULTI))
		{
			_listConfigurationAreas = new List(this, SWT.NONE | SWT.MULTI
					| SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		}
		else if (_enumSelection.equals(EnumSelection.SINGLE))
		{
			_listConfigurationAreas = new List(this, SWT.NONE | SWT.FULL_SELECTION
					| SWT.H_SCROLL | SWT.V_SCROLL);
		}

		_lvConfigurationAreas = new ListViewer(_listConfigurationAreas);
		_lvConfigurationAreas.setSorter(new ViewerSorter());
		_lvConfigurationAreas.setContentProvider(new CP());

		ConfigurationAreas konfigurationsbereiche = new ConfigurationAreas(
				_connection);
		_lvConfigurationAreas.setInput(konfigurationsbereiche);

		/*
		 * Layout
		 */
		GridLayout gl = new GridLayout(1, false);
		this.setLayout(gl);

		GridData gdList = new GridData();
		gdList.grabExcessHorizontalSpace = true;
		gdList.horizontalAlignment = GridData.FILL;
		gdList.grabExcessVerticalSpace = true;
		gdList.verticalAlignment = GridData.FILL;
		_listConfigurationAreas.setLayoutData(gdList);
	}

	/**
	 * Liefert eine Liste der SystemObjekte der selektierten
	 * Konfigurationsbereiche.
	 * 
	 * @return Liste der SystemObjekte der selektierten Konfigurationsbereiche
	 */
	public LinkedList<SystemObject> getSelectedConfigurationAreasSystemObjects()
	{
		LinkedList<SystemObject> re = new LinkedList<SystemObject>();

		ISelection selection = _lvConfigurationAreas.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;
			if (ss.size() > 0)
			{
				@SuppressWarnings("unchecked")
				Iterator it = ss.iterator();
				while (it.hasNext())
				{
					Object object = it.next();
					if (object instanceof SystemObject)
					{
						SystemObject systemObject = (SystemObject) object;
						re.add(systemObject);
					}
				}
			}
		}

		return re;
	}

	/**
	 * Liefert eine Liste der Pids der selektierten Konfigurationsbereiche.
	 * 
	 * @return Liste der Pids der selektierten Konfigurationsbereiche
	 */
	public LinkedList<String> getSelectedConfigurationAreasPids()
	{
		LinkedList<String> re = new LinkedList<String>();

		LinkedList<SystemObject> selectedConfigurationAreas = getSelectedConfigurationAreasSystemObjects();
		if (selectedConfigurationAreas.size() > 0)
		{
			Iterator<SystemObject> it = selectedConfigurationAreas.iterator();
			while (it.hasNext())
			{
				SystemObject systemObject = it.next();
				re.add(systemObject.getPid());
			}
		}

		return re;
	}

	/**
	 * Setzt die zu selektierenden Konfigurationsbereiche an Hand einer Liste von
	 * {@link SystemObject}s, welche die Konfigurationsbereiche reprsentieren.
	 * 
	 * @param systemObjects
	 *          {@link SystemObject}s der zu selektierenden
	 *          Konfigurationsbereiche
	 */
	public void setSelectedConfigurationAreasPidsSystemObjects(
			Collection<SystemObject> systemObjects)
	{
		if (systemObjects.size() > 0)
		{
			LinkedList<String> pids = new LinkedList<String>();
			Iterator<SystemObject> it = systemObjects.iterator();
			while (it.hasNext())
			{
				pids.add(it.next().getPid());
			}
		}
	}

	/**
	 * Setzt die zu selektierenden Konfigurationsbereiche an Hand einer Liste von
	 * Pids der zu selektierenden Konfigurationsbereiche.
	 * 
	 * @param pidSystemObjects
	 *          Liste von Pids der zu selektierenden Konfigurationsbereiche
	 */
	public void setSelectedConfigurationAreasPids(
			Collection<String> pidSystemObjects)
	{
		LinkedList<SystemObject> col = new LinkedList<SystemObject>();

		int itemCount = _listConfigurationAreas.getItemCount();

		if (itemCount > 0)
		{
			for (int i = 0; i < itemCount; i++)
			{
				Object elementAt = _lvConfigurationAreas.getElementAt(i);
				if (elementAt instanceof SystemObject)
				{
					SystemObject sysObjLv = (SystemObject) elementAt;
					Iterator<String> itPid = pidSystemObjects.iterator();
					while (itPid.hasNext())
					{
						String pid = itPid.next();
						if (sysObjLv.getPid().equals(pid))
						{
							col.add(sysObjLv);
						}
					}
				}
			}

			if (col.size() > 0)
			{
				_lvConfigurationAreas.setSelection(new StructuredSelection(col), true);
			}
		}
	}

	/**
	 * ContentProvider fr den {@link ListViewer}. Wertet Objekte vom Typ
	 * {@link ConfigurationAreas} aus.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = new Object[] { "Kein Element verfgbar!" };

			if (inputElement instanceof ConfigurationAreas)
			{
				ConfigurationAreas konfigurationsbereiche = (ConfigurationAreas) inputElement;
				re = konfigurationsbereiche.getKonfigurationsBereiche().toArray();
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

}
