/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas.compsites;

import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreaControler;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreaModel;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreaModelRecord;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreas;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.dialogs.DialogAssociationObjTypConfAreaEditor;
import de.bsvrz.sba.plugin.dynobj.tools.Tools;

/**
 * Editor-{@link Composite} fr das {@link ConfigurationAreaModel}
 * 
 * @author HofmannJ
 * 
 */
public class CompositeConfigurationAreaModelEditor extends Composite
{
	/**
	 * ID des {@link Composite}s
	 */
	private static final String ID = "de.bsvrz.sba.plugin.dynobj.configurationAreas.compsites.CompositeConfigurationAreaModelEditor";

	/*
	 * Strings des Composites
	 */

	/**
	 * Bezeichnung der Tabellenspalte fr die Objekttypen
	 */
	private static final String P_OBJTYP = "Pid ObjektTyp:";

	/**
	 * Bezeichnung der Tabellenspalte fr die Konfigurationsbereiche
	 */
	private static final String P_CONFAREA = "Pid KonfigurationsBereich:";

	/**
	 * Text Action Hinzufgen
	 */
	private static final String S_ADD = "Hinzufgen";

	/**
	 * ToolTip Action Hinzufgen
	 */
	private static final String S_ADD_TT = "Fgt einen neuen Eintrag hinzu.";

	/**
	 * Text Action ndern
	 */
	private static final String S_CHANGE = "ndern";

	/**
	 * ToolTip Action ndern
	 */
	private static final String S_CHANGE_TT = "ndert den selektierten Eintrag.";

	/**
	 * Text Action Entfernen
	 */
	private static final String S_DELETE = "Entfernen";

	/**
	 * ToolTip Action Entfernen
	 */
	private static final String S_DELETE_TT = "Lscht den selektierten Eintrag.";

	/**
	 * Text Action Import
	 */
	private static final String S_IMPORT = "Import";

	/**
	 * ToolTip Action Import
	 */
	private static final String S_IMPORT_TT = "Importiert die Datenstruktur aus einer CSV oder XML Datei.";

	/**
	 * Text Action Export
	 */
	private static final String S_EXPORT = "Export";

	/**
	 * ToolTip Action Export
	 */
	private static final String S_EXPORT_TT = "Exportiert die Datenstruktur in ein CSV oder XML Datei.";

	/**
	 * Text Action Lade aus Prferenzen
	 */
	private static final String S_IMPORT_PREF = "Lade aus Pref.";

	/**
	 * ToolTip Action Lade aus Prferenzen
	 */
	private static final String S_IMPORT_PREF_TT = "Ldt die Datenstruktur aus dem PreferenceStore.";

	/**
	 * Text Action Sichere in Prferenzen
	 */
	private static final String S_EXPORT_PREF = "Sichere in Pref.";

	/**
	 * ToolTip Action Sichere in Prferenzen
	 */
	private static final String S_EXPORT_PREF_TT = "Sichert die Datenstruktur in den PreferenceStore.";

	// ////Ende der Strings//////

	/**
	 * Zu bearbeitendes Modell
	 */
	private ConfigurationAreaModel _model = null;

	/**
	 * Linker Bereich
	 */
	private Composite _left = null;

	/**
	 * Linker Bereich oben
	 */
	private Group _leftUp = null;

	/**
	 * Linker Bereich Unten
	 */
	private Group _leftDown = null;

	/**
	 * Rechter Bereich
	 */
	private Composite _right = null;

	/**
	 * Tabelle des Editors
	 */
	private Table _tableEditor = null;

	/**
	 * TableViewer des Editors
	 */
	private TableViewer _tvEditor = null;

	/**
	 * PreferenceStore
	 */
	private IPreferenceStore _preferenceStore = null;

	/**
	 * Hinzufgen
	 */
	private Button _bAdd = null;

	/**
	 * ndern
	 */
	private Button _bChange = null;

	/**
	 * Hrt auf die Verbindung zum DaV und regelt das (De-)Aktivieren der
	 * Hinzufgen- und Bearbeiten-Buttons.
	 */
	private IDaVVerbindungZuhoerer _daVConnectionListener = null;

	/**
	 * Action Hinzufgen
	 */
	private Action _actionAdd;

	/**
	 * Action ndern/Bearbeiten
	 */
	private Action _actionChange;

	/**
	 * Action Lschen
	 */
	private Action _actionDelete;

	/**
	 * Action Import aus Datei
	 */
	private Action _actionImport;

	/**
	 * Action Export in Datei
	 */
	private Action _actionExport;

	/**
	 * Action Import aus PreferenceStore
	 */
	private Action _actionImportPrefs;

	/**
	 * Action Export in PreferenceStore
	 */
	private Action _actionExportPrefs;

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}
	 * @param style
	 *          SWT-Stil
	 * @param model
	 *          {@link ConfigurationAreaModel}, welches editiert werden soll.
	 * @param preferenceStore
	 *          {@link IPreferenceStore}, in dem Daten abgelegt werden knnen.
	 */
	public CompositeConfigurationAreaModelEditor(Composite parent, int style,
			ConfigurationAreaModel model, IPreferenceStore preferenceStore)
	{
		super(parent, style);
		this._model = model;
		this._preferenceStore = preferenceStore;
		createContent();
	}

	/**
	 * Setzt das Modell des {@link ConfigurationAreaModel} und aktualisiert die
	 * Darstellung des {@link TableViewer}s.
	 * 
	 * @param model
	 *          Darzustellendes {@link ConfigurationAreaModel}
	 */
	public void setModel(ConfigurationAreaModel model)
	{
		this._model = model;
		_tvEditor.setInput(_model);
	}

	/**
	 * Erzeugt die Darstellung.
	 */
	private void createContent()
	{
		createLeft(this);

		createRight(this);
		createActions();
		createConextMenu();

		/*
		 * Layout
		 */
		setLayout(new GridLayout(2, false));

		GridData gdLeft = new GridData();
		gdLeft.grabExcessHorizontalSpace = true;
		gdLeft.horizontalAlignment = GridData.FILL;
		gdLeft.grabExcessVerticalSpace = true;
		gdLeft.verticalAlignment = GridData.FILL;
		_left.setLayoutData(gdLeft);

		GridData gdRight = new GridData();
		gdRight.grabExcessVerticalSpace = true;
		gdRight.verticalAlignment = GridData.FILL;
		_right.setLayoutData(gdRight);

	}

	private Composite createLeft(Composite parent)
	{
		if (_left == null)
		{
			_left = new Composite(parent, SWT.NONE);
			createLeftUp(_left);
			createLeftDown(_left);

			/*
			 * Layout
			 */
			GridLayout gl = new GridLayout(1, false);
			_left.setLayout(gl);

			GridData gdLeftUp = new GridData();
			gdLeftUp.grabExcessHorizontalSpace = true;
			gdLeftUp.horizontalAlignment = GridData.FILL;
			_leftUp.setLayoutData(gdLeftUp);

			GridData gdLeftDown = new GridData();
			gdLeftDown.grabExcessHorizontalSpace = true;
			gdLeftDown.horizontalAlignment = GridData.FILL;
			gdLeftDown.grabExcessVerticalSpace = true;
			gdLeftDown.verticalAlignment = GridData.FILL;
			_leftDown.setLayoutData(gdLeftDown);
		}

		return _left;
	}

	private Composite createLeftUp(Composite parent)
	{
		if (_leftUp == null)
		{
			_leftUp = new Group(parent, SWT.NONE);
			_leftUp.setText("Default Konfigurationsbereich:");

			Text textDefConfArea = new Text(_leftUp, SWT.NONE);
			textDefConfArea.setEditable(false);
			if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
			{
				ConfigurationAreas ca = new ConfigurationAreas(DaVVerbindung
						.getInstanz().getDaVVerbindung());
				textDefConfArea.setText(ca.bestimmeDefaultBereich());
			}
			else
			{
				textDefConfArea
						.setText("Z.Zt. ist keine Verbindung zum DaV aktiv. Default Konfigurationsbereich nicht bestimmbar!");
			}

			/*
			 * Layout
			 */
			_leftUp.setLayout(new GridLayout(1, false));

			GridData gdTextCA = new GridData();
			gdTextCA.grabExcessHorizontalSpace = true;
			gdTextCA.horizontalAlignment = GridData.FILL;
			textDefConfArea.setLayoutData(gdTextCA);

		}

		return _leftUp;
	}

	/**
	 * Erzeugt das {@link Composite} zur Darstellung der Zuordnung von Objekttypen
	 * zu Konfigurationsbereichen.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 * @return Liefert das erzeugte {@link Composite}.
	 */
	private Composite createLeftDown(Composite parent)
	{
		if (_leftDown == null)
		{
			_leftDown = new Group(parent, SWT.NONE);
			_leftDown.setText("Zuordnung Objekttyp - Konfigurationsbereich:");
			_tableEditor = new Table(_leftDown, SWT.MULTI | SWT.FULL_SELECTION
					| SWT.H_SCROLL | SWT.V_SCROLL | SWT.NONE);
			_tableEditor.setLinesVisible(true);
			_tableEditor.setHeaderVisible(true);

			TableColumn tcObjectTyp = new TableColumn(_tableEditor, SWT.LEFT);
			tcObjectTyp.setText(P_OBJTYP);
			tcObjectTyp
					.setToolTipText("Diese Spalte enthlt die Pids der jeweiligen ObjektTypen.");
			tcObjectTyp.setWidth(200);

			TableColumn tcConfArea = new TableColumn(_tableEditor, SWT.LEFT);
			tcConfArea.setText(P_CONFAREA);
			tcConfArea
					.setToolTipText("Diese Spalte enthlt die Pids der jeweiligen Konfigurationsbereiche.");
			tcConfArea.setWidth(200);

			_tvEditor = new TableViewer(_tableEditor);
			_tvEditor.setColumnProperties(new String[] { P_OBJTYP, P_CONFAREA });
			CellEditor[] editors = new CellEditor[] {
					new TextCellEditor(_tableEditor), new TextCellEditor(_tableEditor) };

			_tvEditor.setContentProvider(new CP());
			_tvEditor.setLabelProvider(new LP());
			_tvEditor.setCellEditors(editors);
			_tvEditor.setCellModifier(new CellMod());
			_tvEditor.setSorter(new ViewerSorter());

			if (_model != null)
			{
				setModel(_model);
			}

			/*
			 * Layout
			 */
			_leftDown.setLayout(new GridLayout(1, false));

			GridData gdTable = new GridData();
			gdTable.grabExcessHorizontalSpace = true;
			gdTable.horizontalAlignment = GridData.FILL;
			gdTable.grabExcessVerticalSpace = true;
			gdTable.verticalAlignment = GridData.FILL;
			_tableEditor.setLayoutData(gdTable);
		}

		return _leftDown;
	}

	/**
	 * Erzeugt das rechte {@link Composite} mit den Buttons zur Steuerung der
	 * Basisfunktionen.
	 * 
	 * @param parent
	 *          Eltern-Composite, auf dessen Basis die Darstellung erfolgen soll.
	 * @return Liefert das erzeugte {@link Composite} zurck.
	 */
	private Composite createRight(Composite parent)
	{
		if (_right == null)
		{
			/*
			 * Composite erezeugen
			 */
			_right = new Composite(parent, SWT.NONE);

			/*
			 * Hinzufgen
			 */
			_bAdd = new Button(_right, SWT.PUSH);
			_bAdd.setText(S_ADD);
			_bAdd.setToolTipText(S_ADD_TT);
			_bAdd.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					actionAdd();
				}
			});

			/*
			 * ndern
			 */
			_bChange = new Button(_right, SWT.PUSH);
			_bChange.setText(S_CHANGE);
			_bChange.setToolTipText(S_CHANGE_TT);
			_bChange.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					actionChange();
				}
			});

			/*
			 * Hinzufgen/ndern deaktivieren, wenn keine DaV-Verbindung besteht
			 */
			DaVVerbindung daVVerbindung = DaVVerbindung.getInstanz();
			if (!daVVerbindung.istDaVVerbindungAktiv())
			{
				_bAdd.setEnabled(false);
				_bChange.setEnabled(false);
			}

			/*
			 * Listener auf die DaV-Verbindung anmelden. Wird ber die Dispose-Methode
			 * wieder entfernt.
			 */
			_daVConnectionListener = new IDaVVerbindungZuhoerer()
			{

				@Override
				public void verbindungAbgebaut(DaVVerbindungEreignis e)
				{
					PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable()
					{

						@Override
						public void run()
						{
							_bAdd.setEnabled(false);
							_bChange.setEnabled(false);
						}
					});
				}

				@Override
				public void verbindungAufgebaut(DaVVerbindungEreignis e)
				{
					PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable()
					{

						@Override
						public void run()
						{
							_bAdd.setEnabled(true);
							_bChange.setEnabled(true);
						}
					});
				}

			};
			daVVerbindung.hinzufuegenZuhoerer(_daVConnectionListener);

			/*
			 * Lschen
			 */
			Button bDel = new Button(_right, SWT.PUSH);
			bDel.setText(S_DELETE);
			bDel.setToolTipText(S_DELETE_TT);
			bDel.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					actionDelete();
				}
			});

			/*
			 * Importieren aus Datei
			 */
			Button bImport = new Button(_right, SWT.PUSH);
			bImport.setText(S_IMPORT);
			bImport.setToolTipText(S_IMPORT_TT);
			bImport.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					actionImport();
				}
			});

			/*
			 * Exportieren in Datei
			 */
			Button bExport = new Button(_right, SWT.PUSH);
			bExport.setText(S_EXPORT);
			bExport.setToolTipText(S_EXPORT_TT);
			bExport.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					actionExport();
				}
			});

//			/*
//			 * Laden aus den PreferenceStore
//			 */
//			Button bLoadPref = new Button(_right, SWT.PUSH);
//			bLoadPref.setText(S_IMPORT_PREF);
//			bLoadPref.setToolTipText(S_IMPORT_PREF_TT);
//			bLoadPref.addSelectionListener(new SelectionAdapter()
//			{
//				public void widgetSelected(SelectionEvent e)
//				{
//					importFromPrefStore();
//				}
//			});

			/*
			 * Sichern im PreferenceStore
			 */
//			Button bSavePref = new Button(_right, SWT.PUSH);
//			bSavePref.setText(S_EXPORT_PREF);
//			bSavePref.setToolTipText(S_EXPORT_PREF_TT);
//			bSavePref.addSelectionListener(new SelectionAdapter()
//			{
//				public void widgetSelected(SelectionEvent e)
//				{
//					exportToPrefStore();
//				}
//			});

			/*
			 * Layout
			 */
			RowLayout rl = new RowLayout(SWT.VERTICAL);
			rl.pack = false;
			_right.setLayout(rl);
		}

		return _right;
	}

	/**
	 * Erzeugt {@link Action}s zu den verfgbaren Funktionen
	 */
	private void createActions()
	{
		_actionAdd = new Action()
		{
			public void run()
			{
				actionAdd();
			}
		};
		_actionAdd.setId(ID + ".actionAdd");
		_actionAdd.setText(S_ADD);
		_actionAdd.setToolTipText(S_ADD_TT);

		_actionChange = new Action()
		{
			public void run()
			{
				actionChange();
			}
		};
		_actionChange.setId(ID + ".actionChange");
		_actionChange.setText(S_CHANGE);
		_actionChange.setToolTipText(S_CHANGE_TT);

		_actionDelete = new Action()
		{
			public void run()
			{
				actionDelete();
			}
		};
		_actionDelete.setId(ID + ".actionDelete");
		_actionDelete.setText(S_DELETE);
		_actionDelete.setToolTipText(S_DELETE_TT);

		_actionImport = new Action()
		{
			public void run()
			{
				actionImport();
			}
		};
		_actionImport.setId(ID + ".actionImport");
		_actionImport.setText(S_IMPORT);
		_actionImport.setToolTipText(S_IMPORT_TT);

		_actionExport = new Action()
		{
			public void run()
			{
				actionExport();
			}
		};
		_actionExport.setId(ID + ".actionExport");
		_actionExport.setText(S_EXPORT);
		_actionExport.setToolTipText(S_EXPORT_TT);

		_actionImportPrefs = new Action()
		{
			public void run()
			{
				importFromPrefStore();
			}
		};
		_actionImportPrefs.setId(ID + ".actionImportPrefs");
		_actionImportPrefs.setText(S_IMPORT_PREF);
		_actionImportPrefs.setToolTipText(S_IMPORT_PREF_TT);

		_actionExportPrefs = new Action()
		{
			public void run()
			{
				exportToPrefStore();
			}
		};
		_actionExportPrefs.setId(ID + ".actionExportPrefs");
		_actionExportPrefs.setText(S_EXPORT_PREF);
		_actionExportPrefs.setToolTipText(S_EXPORT_PREF_TT);
	}

	/**
	 * Erzeugt das Kontextmen und hngt es an den Tableviewer an.
	 */
	private void createConextMenu()
	{
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener()
		{
			public void menuAboutToShow(IMenuManager manager)
			{
				fillContextMenu(manager);
			}
		});

		Menu menu = menuMgr.createContextMenu(_tvEditor.getControl());
		_tvEditor.getControl().setMenu(menu);
	}

	/**
	 * Fllt das Kontextmen whrend des Aufbaus der Darstellung.
	 * 
	 * @param manager
	 *          {@link IMenuManager}, welcher die Basis der Kontextmens bildet.
	 */
	private void fillContextMenu(IMenuManager manager)
	{
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			manager.add(_actionAdd);
			manager.add(_actionChange);
		}
		manager.add(_actionDelete);
		manager.add(_actionImport);
		manager.add(_actionExport);
		manager.add(_actionImportPrefs);
		manager.add(_actionExportPrefs);
	}

	/**
	 * Fgt einen neuen Eintrag zum {@link ConfigurationAreaModel} hinzu. Kann
	 * auch verwendet werden, um einen vorhandenen Eintrag zu berschreiben.
	 */
	private void actionAdd()
	{
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			DialogAssociationObjTypConfAreaEditor dlg = new DialogAssociationObjTypConfAreaEditor(
					PlatformUI.getWorkbench().getDisplay().getActiveShell(),
					DaVVerbindung.getInstanz().getDaVVerbindung(), DaVPreLader
							.getInstanz().getGtmDatenBasis(), null);
			ConfigurationAreaModelRecord modelRecord = dlg.open(false);
			if (modelRecord != null)
			{
				/*
				 * Zeigt an, dass fr einen Objekttyp noch keine Zuordnung vorhanden
				 * ist.
				 */
				boolean nichtVorhanden = true;

				String pidConfigurationArea = _model
						.getPidConfigurationArea(modelRecord.getPidObjectType());
				if (pidConfigurationArea != null)
				{
					if (!pidConfigurationArea.equals(modelRecord
							.getPidConfigurationArea()))
					{
						nichtVorhanden = false;

						MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell(), SWT.ICON_QUESTION | SWT.YES
								| SWT.NO);
						mb.setText("Zuordnung zu Objekttyp \""
								+ modelRecord.getPidObjectType() + "\" bereits vorhanden!");
						String message = "";
						message = message + "Dem Objekttyp \""
								+ modelRecord.getPidObjectType();
						message = message + "\" ist bereits\n";
						message = message + "der Konfigurationsbereich \""
								+ pidConfigurationArea + "\" zugeordnet.\n";
						message = message + "Soll der Konfigurationsbereich durch \""
								+ modelRecord.getPidConfigurationArea() + "\" ersetzt werden?";
						mb.setMessage(message);
						int open = mb.open();
						if (open == SWT.YES)
						{
							nichtVorhanden = true;
						}
					}
				}

				if (nichtVorhanden)
				{
					_model.add(modelRecord.getPidObjectType(), modelRecord
							.getPidConfigurationArea());
					_tvEditor.refresh();
				}
			}
		}
		else
		{
			showMessageKeineDaVVerbindung();
		}
	}

	/**
	 * Gibt eine Meldung aus, dass z.Zt. keine Verbindung zum DaV aktiv ist
	 */
	private void showMessageKeineDaVVerbindung()
	{
		MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
				.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
		mb.setText("Keine Verbindung zum DaV");
		String text = "";
		text = text
				+ "Diese Funktion steht z.Zt. noch nicht zur Verfgung, da z.Zt.\n";
		text = text + "keine Verbindung zum DaV aktiv ist.\n";
		text = text + "Bauen Sie eine Verbindung zum DaV auf und versuchen\n";
		text = text + "Sie es erneut.\n";
		mb.setMessage(text);
		mb.open();
	}

	/**
	 * ndert den aktuell selektierten Eintrag. Es wird ein entsprechender Dialog
	 * angezeigt. Der Objekttyp lsst sich nicht anpassen, nur der zugeordnete
	 * Konfigurationsbereich lsst sich ndern.
	 */
	private void actionChange()
	{
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			ISelection selection = _tvEditor.getSelection();
			if (selection instanceof StructuredSelection)
			{
				StructuredSelection ss = (StructuredSelection) selection;
				Object firstElement = ss.getFirstElement();
				if ((firstElement != null)
						&& (firstElement instanceof ConfigurationAreaModelRecord))
				{
					ConfigurationAreaModelRecord record = (ConfigurationAreaModelRecord) firstElement;

					DialogAssociationObjTypConfAreaEditor dlg = new DialogAssociationObjTypConfAreaEditor(
							PlatformUI.getWorkbench().getDisplay().getActiveShell(),
							DaVVerbindung.getInstanz().getDaVVerbindung(), DaVPreLader
									.getInstanz().getGtmDatenBasis(), record);
					ConfigurationAreaModelRecord dialogRecord = dlg.open(false);

					if (dialogRecord != null)
					{
						_model.add(dialogRecord.getPidObjectType(), dialogRecord
								.getPidConfigurationArea());
						_tvEditor.refresh();
					}
				}
			}
		}
		else
		{
			showMessageKeineDaVVerbindung();
		}
	}

	/**
	 * Entfernt den aktuell selektierten Eintrag, inkl. Sicherheitsabfrage.
	 */
	private void actionDelete()
	{
		LinkedList<ConfigurationAreaModelRecord> deleteList = new LinkedList<ConfigurationAreaModelRecord>();

		ISelection selection = _tvEditor.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;

			/*
			 * Lschliste aufbauen
			 */
			@SuppressWarnings("unchecked")
			Iterator it = ss.iterator();
			while (it.hasNext())
			{
				Object object = it.next();
				if (object instanceof ConfigurationAreaModelRecord)
				{
					ConfigurationAreaModelRecord record = (ConfigurationAreaModelRecord) object;
					deleteList.add(record);
				}
			}

			/*
			 * Infotext + Abfrage aufbauen
			 */
			String records = "";
			Iterator<ConfigurationAreaModelRecord> itSicherheitsabfrage = deleteList
					.iterator();
			while (itSicherheitsabfrage.hasNext())
			{
				ConfigurationAreaModelRecord record = itSicherheitsabfrage.next();
				records = records + record + "\n";
			}
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			mb.setText("Wirklich lschen?");
			mb.setMessage("Sind Sie sicher, dass Sie die Eintrge\n" + records
					+ "lschen mchten?");
			int open = mb.open();

			/*
			 * Wenn Abfragem mit Ja besttigt wurde, Lschen der Elemente der
			 * Lschliste aus dem Modell.
			 */
			if (open == SWT.YES)
			{
				Iterator<ConfigurationAreaModelRecord> itDel = deleteList.iterator();
				while (itDel.hasNext())
				{
					ConfigurationAreaModelRecord object = itDel.next();
					ConfigurationAreaModelRecord record = (ConfigurationAreaModelRecord) object;
					_model.remove(record.getPidObjectType());
				}
			}

			_tvEditor.refresh();
		}
	}

	/**
	 * ffnet einen Filedialog, mit dessen Hilfe sich der Import einer Struktur
	 * aus einer CSV oder XML Datei anstoen lsst.
	 */
	private void actionImport()
	{
		boolean saveOk = true;

		if (_model.isDirty())
		{
			saveOk = false;

			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			mb.setText("Aktuelle Struktur wurde nicht gesichert!");
			mb
					.setMessage("Die aktuelle Struktur wurde nicht gesichert. Soll der Import trotzdem durchgefhrt werden? Hierbei geht die aktuelle Struktur verloren!");
			int open = mb.open();
			if (open == SWT.YES)
			{
				saveOk = true;
			}
		}

		if (saveOk)
		{
			FileDialog fld = new FileDialog(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OPEN);
			fld.setText("CSV/XML Datei importieren");
			fld.setFilterNames(new String[] { "CSV Datei", "XML Datei" });
			fld.setFilterExtensions(new String[] { "*.csv", "*.xml" });

			String filePath = fld.open();

			importFromFile(filePath);
		}
	}

	/**
	 * ffnet einen {@link FileDialog}, mit dessen Hilfe sich der Export der
	 * aktuellen Struktur in eine CSV oder XML Datei anstoen lsst.
	 */
	private void actionExport()
	{
		FileDialog fld = new FileDialog(PlatformUI.getWorkbench().getDisplay()
				.getActiveShell(), SWT.SAVE);
		fld.setText("CSV/XML Datei exportieren");
		fld.setFilterNames(new String[] { "CSV Datei", "XML Datei" });
		fld.setFilterExtensions(new String[] { "*.csv", "*.xml" });

		String filePath = fld.open();

		exportToFile(filePath);
	}

	/**
	 * Importiert die Struktur fr das verwendete {@link ConfigurationAreaModel}
	 * aus einer CVS oder XML Datei.
	 * 
	 * @param filePath
	 *          Pfad zur CSV oder XML Datei
	 */
	private void importFromFile(String filePath)
	{
		if (filePath.length() > 4)
		{
			File file = new File(filePath);
			if (file.exists())
			{
				if (file.isFile())
				{
					String suffix = filePath.substring(filePath.length() - 4, filePath
							.length());

					if (suffix.toLowerCase().equals(".csv"))
					{
						ConfigurationAreaControler cont = new ConfigurationAreaControler();
						String fileContent = Tools.readStringFromFile(filePath);
						cont.readFromCsvString(fileContent);
						this._model = cont.getModel();
						_tvEditor.setInput(_model);
					}
					else if (suffix.toLowerCase().equals(".xml"))
					{
						ConfigurationAreaControler cont = new ConfigurationAreaControler();
						String fileContent = Tools.readStringFromFile(filePath);
						cont.readFromXmlString(fileContent);
						this._model = cont.getModel();
						_tvEditor.setInput(_model);
					}
					else
					{
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
						mb.setText("Ungltige Dateiendung!");
						mb
								.setMessage("Die Dateiendung \""
										+ suffix
										+ "\" ist ungltig. Nur .csv und .xml werden akzeptiert. Es ist kein Import mglich.");
						mb.open();
					}
				}
				else
				{
					MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
							.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
					mb.setText("Keine Datei!");
					mb
							.setText("Der angegebene Pfad \""
									+ filePath
									+ "\" beschreibt nicht den Pfad zu einer Datei, somit ist kein ffnen mglich");
					mb.open();
				}
			}
			else
			{
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Datei existiert nicht!");
				mb.setMessage("Die Datei \"" + filePath
						+ "\" ist nicht existent und kann somit nicht geffnet werden.");
				mb.open();
			}
		}
		else
		{
			String msg = "Der Dateipfad \"" + filePath + "\" ist zu kurz.";
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
			mb.setText("Pfad zu kurz!");
			mb.setMessage(msg);
			mb.open();
		}
	}

	/**
	 * Exportiert die Struktur des aktuellen {@link ConfigurationAreaModel}s in
	 * eine CSV oder XML Datei.
	 * 
	 * @param filePath
	 *          Pfad zur CSV oder XML Datei
	 */
	private void exportToFile(String filePath)
	{
		if (filePath.length() > 4)
		{
			String suffix = filePath.substring(filePath.length() - 4, filePath
					.length());

			if (suffix.toLowerCase().equals(".csv"))
			{
				ConfigurationAreaControler cont = new ConfigurationAreaControler(_model);
				String csvString = cont.writeToCsvString();
				if (csvString.length() > 0)
				{
					Tools.writeStringToFile(filePath, true, csvString);
					_model.resetDirtyFlag();
				}
			}
			else if (suffix.toLowerCase().equals(".xml"))
			{
				ConfigurationAreaControler cont = new ConfigurationAreaControler(_model);
				String xmlString = cont.writeToXmlString();
				if (xmlString.length() > 0)
				{
					Tools.writeStringToFile(filePath, true, xmlString);
					_model.resetDirtyFlag();
				}
			}
			else
			{
				MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
						.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Ungltige Dateiendung!");
				mb
						.setMessage("Die Dateiendung \""
								+ suffix
								+ "\" ist ungltig. Nur .csv und .xml werden akzeptiert. Es findet kein Export statt.");
				mb.open();
			}

		}
		else
		{
			String msg = "Der Dateipfad \"" + filePath + "\" ist zu kurz.";
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
			mb.setText("Pfad zu kurz!");
			mb.setMessage(msg);
			mb.open();
		}
	}

	/**
	 * Importiert das {@link ConfigurationAreaModel}, welches im
	 * {@link PreferenceStore} abgelegt ist.
	 */
	private void importFromPrefStore()
	{
		if (_preferenceStore != null)
		{
			ConfigurationAreaControler cont = new ConfigurationAreaControler();
			cont.readFromPreferenceStore(_preferenceStore);
			this._model = cont.getModel();
			_tvEditor.setInput(_model);
		}
		else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
			mb.setText("Kein PreferenceStore verfgbar!");
			mb.setMessage("Es wurde kein PreferenceStore ber geben.");
			mb.open();
		}
	}

	/**
	 * Sichert das {@link ConfigurationAreaControler} in den
	 * {@link PreferenceStore}.
	 */
	public void exportToPrefStore()
	{
		if (_preferenceStore != null)
		{
			ConfigurationAreaControler cont = new ConfigurationAreaControler(_model);
			cont.writeToPreferenceStore(_preferenceStore);
			_model.resetDirtyFlag();
		}
		else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
			mb.setText("Kein PreferenceStore verfgbar!");
			mb.setMessage("Es wurde kein PreferenceStore ber geben.");
			mb.open();
		}
	}

	/**
	 * ContentProvider fr den {@link TableViewer}. Wertet als InputElement ein
	 * {@link ConfigurationAreaModel} aus.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof ConfigurationAreaModel)
			{
				ConfigurationAreaModel model = (ConfigurationAreaModel) inputElement;

				LinkedList<ConfigurationAreaModelRecord> structurNew = model
						.getStructure();
				if (structurNew.size() > 0)
				{
					re = structurNew.toArray();
				}
			}

			/*
			 * Work-Arround: Wenn die Struktur leer ist, ein leeres Array statts null
			 * zurckliefern.
			 */
			if (re == null)
			{
				re = new Object[] {};
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * LabelProvider
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if (element instanceof ConfigurationAreaModelRecord)
			{
				ConfigurationAreaModelRecord rec = (ConfigurationAreaModelRecord) element;
				switch (columnIndex)
				{
				case 0:
					re = rec.getPidObjectType();
					break;
				case 1:
					re = rec.getPidConfigurationArea();
					break;
				default:
					re = "Undefiniert";
					break;
				}
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	/**
	 * CellModifier
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class CellMod implements ICellModifier
	{

		@Override
		public boolean canModify(Object element, String property)
		{
			boolean re = false;

			if (property.equals(P_CONFAREA))
			{
				re = true;
			}

			return re;
		}

		@Override
		public Object getValue(Object element, String property)
		{
			Object re = null;

			if (element != null)
			{
				if (element instanceof ConfigurationAreaModelRecord)
				{
					ConfigurationAreaModelRecord rec = (ConfigurationAreaModelRecord) element;
					if (property.equals(P_CONFAREA))
					{
						re = rec.getPidConfigurationArea();
					}
				}
			}

			return re;
		}

		@Override
		public void modify(Object element, String property, Object value)
		{
			if (element instanceof TableItem)
			{
				TableItem item = (TableItem) element;
				Object data = item.getData();
				if (data instanceof ConfigurationAreaModelRecord)
				{
					ConfigurationAreaModelRecord rec = (ConfigurationAreaModelRecord) data;
					if (value instanceof String)
					{
						_model.setPidConfigurationArea(rec.getPidObjectType(),
								(String) value);
						_tvEditor.refresh();
					}
				}
			}
		}

	}

	public void dispose()
	{
		if (_daVConnectionListener != null)
		{
			DaVVerbindung.getInstanz().entferneZuhoerer(_daVConnectionListener);
		}
		super.dispose();
	}

}
