/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatorView;


import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sba.plugin.navigator.Activator;

/**
 * ImageRegistry-Klasse, speichert Images und ldt noch nicht vorhandene Images
 * ber den Activator nach
 * 
 * @author Krause
 * 
 */

public class MyImageRegistry extends ImageRegistry
{
  /**
   * Konstruktor
   */
  public MyImageRegistry()
  {
    super();
  }

  /**
   * Laden des Bildes ber den Activator und den angegebene Pfad und speichern
   * unter dem angegebenen Namen
   * 
   * @param name
   *          Name unter dem das Bild gespeichert werden soll
   * @param imageFilePath
   *          Pfad zum Bild
   */
  public void putImage( String name, String imageFilePath )
  {
    // Pfadname muss vorhanden sein
    if ( imageFilePath == null || imageFilePath.equals( "" ) )
      return;
    
    // Falls kein Name angegeben wurde, wird der Pfadname verwendet
    String regName = name;
    if ( regName == null || regName.equals( "" ) )
      regName = imageFilePath;

    // Bild laden, falls nicht bereits vorhanden
    Image bild = this.get( regName );
    if ( bild == null  )
    {
      try
      {
        ImageDescriptor iDesc = Activator.getImageDescriptor( imageFilePath );
        this.put( regName, iDesc );
      }
      catch (Exception e)
      {
        this.put( regName, PlatformUI.getWorkbench().getSharedImages().getImage( ISharedImages.IMG_OBJS_WARN_TSK ) );
      }
    }
  }

  /**
   * Liefert das Bild zu dem gegebenen relativen Pfad - falls dieses nicht
   * vorhanden ist, wird es geladen -
   * 
   * @param imageFilePath
   *          Pfad zum Bild
   * @return Image (Standard-Bild, falls Pfadangabe falsch)
   */
  public Image getImage( String imageFilePath )
  {
    return getImage( imageFilePath, imageFilePath );
  }

  /**
   * Liefert das Bild zu dem gegebenen Namen - falls dieses nicht
   * vorhanden ist, wird es ber den gegebenen relativen Pfad geladen -
   *
   * @param name
   *          Name unter dem das Bild gespeichert werden soll
   * @param imageFilePath
   *          Pfad zum Bild
   * @return Image (Standard-Bild, falls Pfadangabe falsch)
   */
  public Image getImage( String name, String imageFilePath )
  {
    Image bild = this.get( name );

    if ( bild == null && imageFilePath != null )
    {
      putImage( name, imageFilePath );
      
      bild = this.get( name );
    }
    else if ( bild == null )
    {
      bild = PlatformUI.getWorkbench().getSharedImages().getImage( ISharedImages.IMG_OBJS_WARN_TSK );
    }

    return bild;
  }

}
