/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatorView;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.preference.IPreferenceStore;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sba.plugin.navigator.Activator;
import de.bsvrz.sba.plugin.navigator.navigatorModell.INavigatorElement;
import de.bsvrz.sba.plugin.navigator.navigatorModell.NavigatorElement;
import de.bsvrz.sba.plugin.navigator.navigatorModell.NavigatorElementUmfeldDaten;
import de.bsvrz.sba.plugin.navigator.navigatorModell.NavigatorElementUrsachenEinheit;
import de.bsvrz.sba.plugin.navigator.navigatorModell.INavigatorElement.STATUS;
import de.bsvrz.sba.plugin.navigator.navigatorModell.NavigatorElement.KATEGORIE;
import de.bsvrz.sba.plugin.navigator.navigatoreinstellungen.Navigatorkategorien;

/**
 * Klasse zum Erstellen des Navigatorbaumes
 * 
 * @author Krause
 * 
 */
public class NavigatorDaten
{
  /**
   * Flag zum Debuggen
   */
  protected static final boolean m_debug = false;

  /**
   * Testparameter zum Generieren von Testdaten
   */
  private static final boolean m_generiereTest = false;

  /**
   * mgliche Kategorien von Datenendgerten
   */
  public static final HashMap<String, KATEGORIE> DEKATEGORIE = new HashMap<String, KATEGORIE>();
  static
  {
    DEKATEGORIE.put( "typ.de", KATEGORIE.Datenendgeraet );
    DEKATEGORIE.put( "typ.de128", KATEGORIE.DatenendgeraetDe128 );
    DEKATEGORIE.put( "typ.deAxl", KATEGORIE.DatenendgeraetDeAxl );
    DEKATEGORIE.put( "typ.deLve", KATEGORIE.DatenendgeraetDeLve );
    DEKATEGORIE.put( "typ.deSys", KATEGORIE.DatenendgeraetDeSys );
    DEKATEGORIE.put( "typ.deUfd", KATEGORIE.DatenendgeraetDeUfd );
//    DEKATEGORIE.put( "typ.deUfdNW", KATEGORIE.DatenendgeraetDeUfdNW );
    DEKATEGORIE.put( "typ.deVlt", KATEGORIE.DatenendgeraetDeVlt );
    DEKATEGORIE.put( "typ.deWww", KATEGORIE.DatenendgeraetDeWww );
    DEKATEGORIE.put( "typ.deWzg", KATEGORIE.DatenendgeraetDeWzg );
  }

  /**
   * Wurzel der Navigatorelemente
   */
  private INavigatorElement m_root;

  /**
   * Verbindung zum Datenverteiler
   */
  private ClientDavInterface m_connection;

  /**
   * DatenModell zum Datenverteiler
   */
  private DataModel m_dataModel;

  /**
   * Elternknoten zu den Ereignissen
   */
  private INavigatorElement m_ereignisse;

  /**
   * Elternknoten zu den Ereignistypen
   */
  private INavigatorElement m_ereignistypen;
  
  /**
   * Elternknoten zum Systemkalender
   */
  private INavigatorElement m_systemKalender;

  /**
   * Elternknoten zu den UrsachenEinheiten
   */
  private INavigatorElement m_ursachenEinheiten;

  /**
   * ProgressMonitor zur Fortschrittsanzeige
   */
  private IProgressMonitor m_progressMonitor;

  /**
   * Speicher mit den Prferenzen (Navigatoreinstellungen)
   */
  private IPreferenceStore m_preferenceStore;

  private HashMap<String, INavigatorElement> m_listeUrsachenEinheitTypen;

  // -----------------------------------------------
  /**
   * Konstruktor zur Erstellung des Navigatorbaumes
   */
  public NavigatorDaten( IProgressMonitor monitor )
  {
    m_root = new NavigatorElement( null, null, null, null );

    m_progressMonitor = monitor;

    // Hole den Prferenzen Speicher
    m_preferenceStore = Activator.getDefault().getPreferenceStore();
   
    m_listeUrsachenEinheitTypen = new HashMap<String, INavigatorElement>();

    String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[0];
    int int1 = m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel );
    
    if ( m_generiereTest )
      generiereTestDaten();
    else
      generiereBaumStruktur();
  }

  /**
   * Testdaten generieren
   */
  private void generiereTestDaten()
  {
    String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[0];
    if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) != 0 )
    {
	    INavigatorElement navElement0 = new NavigatorElement( m_root, elemTitel,
	        KATEGORIE.Wetterstation, null );
	    for (int i = 0; i < 3; i++)
	    {
	      INavigatorElement navKind = new NavigatorElement( navElement0,
	          "Konfigurationsbereich Darstellung WWW AQs " + i, KATEGORIE.KonfigBereich, null );
	      for (int j = 0; j < 2; j++)
	      {
	        INavigatorElement navKindKind = new NavigatorElement( navKind, "Unterzentralen " + j,
	            KATEGORIE.Unterzentrale, null );
	        if ( j == 0 )
	          navKindKind.setStatus( STATUS.Stoerung );
	        if ( j == 1 )
	          navKindKind.setStatus( STATUS.IO );
	      }
	    }
    }

    elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[1];
    if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) != 0 )
    {
		INavigatorElement navElement1 = new NavigatorElement( m_root, elemTitel,
		    KATEGORIE.Anzeigequerschnitte, null );
		for (int i = 4; i >= 0; i--)
		{
		  INavigatorElement navKind = new NavigatorElement( navElement1, "Anzeigequerschnitt " + i,
		      KATEGORIE.Anzeigequerschnitt, null );
		  if ( i == 2 )
		    navKind.setStatus( STATUS.IO );
		  navKind = new NavigatorElement( navElement1, "Anzeige " + i, KATEGORIE.Anzeige, null );
		  navKind.setStatus( STATUS.Stoerung );
		}
    }

    elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[4];
    if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) != 0 )
    {
	    INavigatorElement navElement2 = new NavigatorElement( m_root, elemTitel, KATEGORIE.Messquerschnitte, null );
	    for (int i = 0; i < 5; i++)
	    {
	      INavigatorElement navKind = new NavigatorElement( navElement2, "Fahrspur " + i, KATEGORIE.Fahrspur, null );
	      if ( i == 0 )
	        navKind.setStatus( STATUS.IO );
	    }
    }

    elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[3];
    if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) != 0 )
    {
	    INavigatorElement navElement3 = new NavigatorElement( m_root, elemTitel, KATEGORIE.Messquerschnitte, null );
	    for (int i = 0; i < 5; i++)
	    {
	      INavigatorElement navKind = new NavigatorElement( navElement3, "Fahrspur " + i, KATEGORIE.Fahrspur, null );
	      navKind.setStatus( STATUS.IO );
	    }
	    for (int i = 0; i < 3; i++)
	    {
	      INavigatorElement navKind = new NavigatorElement( navElement3, "Messquerschnitt " + i,
	          KATEGORIE.Messquerschnitt, null );
	      navKind.setStatus( STATUS.IO );
	    }
    }

    elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[5];
    if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) != 0 )
    {
	    INavigatorElement navElement4 = new NavigatorElement( m_root, elemTitel, KATEGORIE.Ordner, null );
	    for (int i = 0; i < 5; i++)
	    {
	      INavigatorElement navKind = new NavigatorElement( navElement4, "Ereignis " + i, KATEGORIE.Ereignis, null );
	      if ( i == 1 )
	        navKind.setStatus( STATUS.Stoerung );
	    }
    }
  }

  // -------------------------------------------------------------
  /**
   * Baumstruktur mit den Daten vom Datenverteiler generieren
   */
  private void generiereBaumStruktur()
  {
    // Verbindung zum Datenverteiler aufbauen
    m_connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if ( m_connection != null )
    {
      if ( m_debug )
        System.out.println( "Verbindung zum Datenverteiler aufgebaut" );
      m_dataModel = m_connection.getDataModel();
      
      // Men Kommunikation extern
      bestimmeMenuKommunikationExtern( m_dataModel, m_root);
      
      m_progressMonitor.worked( 2 );
      if ( m_progressMonitor.isCanceled() )
        return;

      // Men Anzeigequerschnitte
      bestimmeMenuAnzeigequerschnitte( m_dataModel, m_root );
  
      m_progressMonitor.worked( 2 );
      if ( m_progressMonitor.isCanceled() )
        return;

      // Messquerschnitte
      bestimmeMenuMessquerschnitte( m_dataModel, m_root );

      m_progressMonitor.worked( 2 );
      if ( m_progressMonitor.isCanceled() )
        return;

      // Men Ereignisse
      bestimmeMenuEreignisse( m_connection, m_root );

      m_progressMonitor.worked( 2 );
      if ( m_progressMonitor.isCanceled() )
        return;

      // Men Ereignistypen
      bestimmeMenuEreignistypen( m_connection, m_root );

      m_progressMonitor.worked( 2 );
      if ( m_progressMonitor.isCanceled() )
        return;

      // Men Systemkalender
      bestimmeMenuSystemkalender( m_connection, m_root );

      m_progressMonitor.worked( 2 );
      if ( m_progressMonitor.isCanceled() )
        return;

      // Men SswSbaUrsacheneinheiten
      bestimmeMenuSswSbaUrsacheneinheiten(m_connection, m_root);
     
      m_progressMonitor.worked(2);
      if (m_progressMonitor.isCanceled())
        return;

      // Men UmfeldDatenMessstellen
      bestimmeMenuUmfeldDatenMessstellen(m_dataModel, m_root);
     
      m_progressMonitor.worked(2);
      if (m_progressMonitor.isCanceled())
        return;
    }
    else
    {
      new NavigatorElement( m_root, "Keine Verbindung zum Datenverteiler", null, null );
    }

  }

  /**
   * Baumstruktur fr das Men Systemkalender generieren
   * 
   * @param connection
   *          Verbindung zum Datenverteiler
   * @param root
   *          Knoten des Elternteils
   */
  private void bestimmeMenuSystemkalender( ClientDavInterface connection, INavigatorElement root )
  {
    String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[7];
    if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) == 0 )
      return;
    
    int anzVerborgen = 0;
    m_systemKalender = new NavigatorElement( root, elemTitel, KATEGORIE.Ordner, null );
    MutableSet setsk = connection.getDataModel().getConfigurationAuthority().getMutableSet( "SystemKalenderEintrge" );

    List<SystemObject> objekte = setsk.getElements();

    if ( objekte.size() != 0 )
    {
      for (SystemObject systemObject : objekte)
      {
        if ( !systemObject.isValid() )
        {
          if ( m_debug )
            System.out.println( "Invalid: " + systemObject.getPidOrNameOrId() );
          continue;
        }

        if ( erstelleSystemKalenderObjekt( m_systemKalender, systemObject ) == null )
          anzVerborgen++;
      }
      if ( anzVerborgen > 0 )
        m_systemKalender.setName( "Systemkalendereintrge (" + anzVerborgen + " Eintrge verborgen)" );
    }

    setsk.addChangeListener( new MutableSetChangeListener()
    {

      @Override
      public void update( MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects )
      {
        removeAddElements( "Systemkalender", m_systemKalender, addedObjects, removedObjects );
      }

    } );

    if ( m_debug )
      System.out.println( elemTitel + " fertig!" );
  }

  /**
   * SystemKalenderObjekt in der Baumstruktur generieren
   * 
   * @param root
   *          Elternteil zu dem hinzugefgt oder entfertn werden soll
   * @param systemObject
   *          Objekt des Datenverteilers
   */
  public INavigatorElement erstelleSystemKalenderObjekt( INavigatorElement root, SystemObject systemObject )
  {
    ConfigurationArea area = systemObject.getConfigurationArea();

    if ( !(m_preferenceStore.getBoolean( Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId() )) )
      return null;

    INavigatorElement element = new NavigatorElement( root, systemObject.getNameOrPidOrId(),
        KATEGORIE.Kalender, getLinkedList( systemObject ) );

    DynamicObject dobjekt = (DynamicObject) systemObject;

    dobjekt.addListenerForInvalidation( new InvalidationListener()
    {

      @Override
      public void invalidObject( DynamicObject dynamicObject )
      {
        System.out.println( "Invalid: " + dynamicObject.getPidOrNameOrId() );

        if ( dynamicObject != null && dynamicObject.isOfType( "typ.systemKalenderEintrag" ) )
        {
          Object[] children = m_systemKalender.getChildren();
          for (int i = 0; i < children.length; i++)
          {
            INavigatorElement child = (INavigatorElement) children[i];
            if ( child.hasObjekt( dynamicObject ) )
            {
              INavigatorElement parent = (INavigatorElement) child.getParent();
              parent.remove( child );
              break;
            }

          }// for
        }// if
      }

    } );

    return element;
  }

  
  /**
   * Baumstruktur fr das Men Systemkalender generieren
   * 
   * @param connection
   *          Verbindung zum Datenverteiler
   * @param root
   *          Knoten des Elternteils
   */
  private void bestimmeMenuSswSbaUrsacheneinheiten(ClientDavInterface connection, INavigatorElement root)
  {
	DataModel dataModel = connection.getDataModel();
	  
    String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[8];
    if ((m_preferenceStore.getInt(Activator.PLUGIN_ID + ":" + elemTitel)) == 0)
      return;
    
    SystemObjectType sot = dataModel.getType("typ.sswSbaStreckenBeeinflussungsAnlage");
    
    if (sot == null)
      return;

    List<SystemObject> objekte = sot.getElements();
    List<SystemObject> objekteGefiltert = new Vector<SystemObject>();
    
    for (SystemObject objekt : objekte) 
    {
      ConfigurationArea area = objekt.getConfigurationArea();
    	
   	  if (m_preferenceStore.getBoolean(Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId()))
      {
    	objekteGefiltert.add(objekt);
      }
	}

    if (objekteGefiltert.size() > 0)
    {
      int anzVerborgen = 0;
      m_ursachenEinheiten = new NavigatorElement(root, elemTitel, KATEGORIE.Ordner, null);
    
      for (SystemObject systemObject : objekteGefiltert) 
      {
    	// Anlage
    	  
    	INavigatorElement elem = new NavigatorElement(m_ursachenEinheiten, systemObject.getNameOrPidOrId(), KATEGORIE.Ordner, null);
    	
        MutableSet setertyp = ((ConfigurationObject)systemObject).getMutableSet("UrsachenEinheiten");
        
        if (setertyp != null)
        {
          List<SystemObject> ursacheneinheiten = setertyp.getElements();

          if (ursacheneinheiten.size() != 0)
          {
            for (SystemObject ursacheneinheit : ursacheneinheiten)
            {
              if (!ursacheneinheit.isValid())
              {
                continue;
              }
              
              // UrsachenEinheiten fr Sonderprogramme nicht anzeigen
              
              if (ursacheneinheit.getType().getPid().equals("typ.sswSbaUeSonderprogramm"))
              {
            	continue;
              }
    
        	  INavigatorElement elem1 = holeBzwErstelleNavigatorElementUrsachenEinheitTyp(elem, ursacheneinheit);
        	  
              if (erstelleSswSbaUrsachenEinheitObjekt(elem1, ursacheneinheit) == null)
                anzVerborgen++;
            }
          }
        
//          setertyp.addChangeListener(new MutableSetChangeListener()
//          {
//          @Override
//            public void update(MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects)
//            {
//              removeAddElements("UrsachenEinheiten", m_ursachenEinheiten, addedObjects, removedObjects);
//            }
//          });
        }
	  }
    }

    if (m_debug)
      System.out.println(elemTitel + " fertig!");
  }

  private INavigatorElement holeBzwErstelleNavigatorElementUrsachenEinheitTyp(INavigatorElement elem, SystemObject ursacheneinheit) 
  {
	INavigatorElement elem1 = m_listeUrsachenEinheitTypen.get(ursacheneinheit.getType().getPidOrNameOrId());
	
	if (elem1 == null)
	{
	  elem1 = new NavigatorElement(elem, ursacheneinheit.getType().getNameOrPidOrId(), KATEGORIE.Ordner, null);
	  
	  m_listeUrsachenEinheitTypen.put(ursacheneinheit.getType().getPidOrNameOrId(), elem1);
	}
	
	return elem1;
  }
  
  /**
   * Baumstruktur fr das Men Ereignistypen generieren
   * 
   * @param connection
   *          Verbindung zum Datenverteiler
   * @param root
   *          Knoten des Elternteils
   */
  private void bestimmeMenuEreignistypen( ClientDavInterface connection, INavigatorElement root )
  {
    String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[6];
    if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) == 0 )
      return;
    
    int anzVerborgen = 0;
    m_ereignistypen = new NavigatorElement( root, elemTitel, KATEGORIE.Ordner, null );
    MutableSet setertyp = connection.getDataModel().getConfigurationAuthority().getMutableSet( "EreignisTypen" );

    List<SystemObject> objekte = setertyp.getElements();

    if ( objekte.size() != 0 )
    {
      for (SystemObject systemObject : objekte)
      {
        if ( erstelleEreignistypObjekt( m_ereignistypen, systemObject ) == null )
          anzVerborgen++;
      }
      if ( anzVerborgen > 0 )
        m_ereignistypen.setName( "Ereignistypen (" + anzVerborgen + " Eintrge verborgen)" );
    }
    
    setertyp.addChangeListener( new MutableSetChangeListener()
    {

      @Override
      public void update( MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects )
      {
        removeAddElements( "Ereignistypen", m_ereignistypen, addedObjects, removedObjects );
      }

    } );
    
    if ( m_debug )
      System.out.println( elemTitel + " fertig!" );
  }

  /**
   * EreignistypenObjekt in der Baumstruktur generieren
   * 
   * @param root
   *          Elternteil zu dem hinzugefgt oder entfertn werden soll
   * @param systemObject
   *          Objekt des Datenverteilers
   */
  private INavigatorElement erstelleEreignistypObjekt( INavigatorElement root, SystemObject systemObject )
  {
    ConfigurationArea area = systemObject.getConfigurationArea();

    if ( !(m_preferenceStore.getBoolean( Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId() )) )
      return null;

    INavigatorElement element = new NavigatorElement( root, systemObject.getNameOrPidOrId(),
        KATEGORIE.Ereignistyp, getLinkedList( systemObject ) );

    DynamicObject dobjekt = (DynamicObject) systemObject;

    dobjekt.addListenerForInvalidation( new InvalidationListener()
    {

      @Override
      public void invalidObject( DynamicObject dynamicObject )
      {
        if ( dynamicObject != null && dynamicObject.isOfType( "typ.EreignisTyp" ) )
        {
          Object[] children = m_ereignistypen.getChildren();
          for (int i = 0; i < children.length; i++)
          {
            INavigatorElement child = (INavigatorElement) children[i];
            if ( child.hasObjekt( dynamicObject ) )
            {
              INavigatorElement parent = (INavigatorElement) child.getParent();
              parent.remove( child );
              break;
            }

          }// for
        }// if
      }

    } );

    return element;
  }

  /**
   * Baumstruktur fr das Men Ereignisse generieren
   * 
   * @param connection
   *          Verbindung zum Datenverteiler
   * @param root
   *          Knoten des Elternteils
   */
  private void bestimmeMenuEreignisse( ClientDavInterface connection, INavigatorElement root )
  {
    String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[5];
    if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) == 0 )
      return;
    
    int anzVerborgen = 0;
    m_ereignisse = new NavigatorElement( root, elemTitel, KATEGORIE.Ordner, null );
    MutableSet seter = connection.getDataModel().getConfigurationAuthority().getMutableSet( "Ereignisse" );

    List<SystemObject> objekte = seter.getElements();

    if ( objekte.size() != 0 )
    {
      for (SystemObject systemObject : objekte)
      {
        if ( erstelleEreignisObjekt( m_ereignisse, systemObject ) == null )
          anzVerborgen++;
      }
      if ( anzVerborgen > 0 )
        m_ereignisse.setName( "Ereignisse (" + anzVerborgen + " Eintrge verborgen)" );

    }

    seter.addChangeListener( new MutableSetChangeListener()
    {

      @Override
      public void update( MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects )
      {
        removeAddElements( "Ereignisse", m_ereignisse, addedObjects, removedObjects );
      }

    } );
    
    if ( m_debug )
      System.out.println( elemTitel + " fertig!" );
  }

  /**
   * EreignisObjekt in der Baumstruktur generieren
   * 
   * @param root
   *          Elternteil zu dem hinzugefgt oder entfertn werden soll
   * @param systemObject
   *          Objekt des Datenverteilers
   */
  private INavigatorElement erstelleEreignisObjekt( INavigatorElement root, SystemObject systemObject )
  {
    ConfigurationArea area = systemObject.getConfigurationArea();

    if ( !(m_preferenceStore.getBoolean( Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId() )) )
      return null;

    AttributeGroup atg = m_dataModel.getAttributeGroup( "atg.ereignis" );
    Aspect asp = m_dataModel.getAspect( "asp.zustand" );
    DataDescription dd = new DataDescription( atg, asp );

    INavigatorElement element = new NavigatorElement( root, systemObject.getNameOrPidOrId(),
        KATEGORIE.Ereignis, getLinkedList( systemObject ) );

    DynamicObject dobjekt = (DynamicObject) systemObject;

    dobjekt.addListenerForInvalidation( new InvalidationListener()
    {

      @Override
      public void invalidObject( DynamicObject dynamicObject )
      {
        if ( dynamicObject != null && dynamicObject.isOfType( "typ.Ereignis" ) )
        {
          Object[] children = m_ereignisse.getChildren();
          for (int i = 0; i < children.length; i++)
          {
            INavigatorElement child = (INavigatorElement) children[i];
            if ( child.hasObjekt( dynamicObject ) )
            {
              INavigatorElement parent = (INavigatorElement) child.getParent();
              parent.remove( child );
              break;
            }

          }// for
        }// if
      }

    } );

    // Listener auf die Statusaktualisierung setzen --> startet Methode update
    // des NavigatorElements
    m_connection.subscribeReceiver( element, systemObject, dd, ReceiveOptions.normal(), ReceiverRole.receiver() );

    return element;
  }

  /**
   * Hinzufgen bzw. Entfernen von Navigatorelementen an Hand der hinzugefgten
   * bzw. entfernten Objekte
   * 
   * @param root
   *          Elternteil zu dem hinzugefgt oder entfertn werden soll
   * @param addedObjects
   *          Hinzuzufgende Objekte
   * @param removedObjects
   *          Zu entfernende Objekte
   */
  private void removeAddElements( String art, INavigatorElement root, SystemObject[] addedObjects, SystemObject[] removedObjects )
  {
    if ( m_debug )
      System.out.println( "Anzahl weg: " + removedObjects.length );

    // removedObjects entfernen
    for (int i = 0; i < removedObjects.length; i++)
    {
      SystemObject systemObject = removedObjects[i];

      if ( systemObject != null )
      {
        Object[] children = root.getChildren();
        for (int ii = 0; ii < children.length; ii++)
        {
          INavigatorElement child = (INavigatorElement) children[ii];
          if ( child.hasObjekt( systemObject ) )
          {
            INavigatorElement parent = (INavigatorElement) child.getParent();
            if ( child != null )
              parent.remove( child );
            break;
          }
        }// for children
      }// if
    }// for removedObjects

    // addedObjects hinzufgen
    for (int i = 0; i < addedObjects.length; i++)
    {
      SystemObject systemObject = addedObjects[i];

      if ( systemObject != null )
      {
        INavigatorElement child = null;
        if (art.equals( "Systemkalender" ))
          child = erstelleSystemKalenderObjekt( null, systemObject );
        else if (art.equals( "Ereignistypen" ))
          child = erstelleEreignistypObjekt( null, systemObject );
        else if (art.equals( "Ereignisse" ))
          child = erstelleEreignisObjekt( null, systemObject );
        
        if ( child != null )
          root.add( child );
      }// if
    }// for addedObjects
  }
  
  /**
   * Baumstruktur fr Mens der Anzeigequerschnitte generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   */
  private void bestimmeMenuAnzeigequerschnitte( DataModel dataModel, INavigatorElement root )
  {
    SystemObjectType sot = dataModel.getType( "typ.anzeigeQuerschnitt" );
    List<SystemObject> objekte = sot.getElements();
    
    if ( objekte.size() != 0 )
    {
      String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[1];
      if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) != 0 )
      {
        INavigatorElement elem = new NavigatorElement( root, elemTitel,
          KATEGORIE.Anzeigequerschnitte, null );
        
        bestimmeKonfigurationsBereicheFuerAnzeigequerschnitte( dataModel, elem, objekte );
      }

      elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[2];
      if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) != 0 )
      {
        INavigatorElement elem = new NavigatorElement( root, elemTitel,
            KATEGORIE.Anzeigequerschnitte, null );
        
        bestimmeAnzeigequerschnitte( dataModel, elem, objekte );
      }
    }
    
    if ( m_debug )
      System.out.println( "Anzeigequerschnitte fertig!" );
     
  }

  /**
   * Baumstruktur fr die Konfigurationsbereiche der Anzeigequerschnitte
   * generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeKonfigurationsBereicheFuerAnzeigequerschnitte( DataModel model, INavigatorElement root,
      List<SystemObject> objekte )
  {
    List<ConfigurationArea> konfBereiche = new Vector<ConfigurationArea>();

    // Objekte der Konfigurationsbereiche ermitteln
    for (Iterator iterator = objekte.iterator(); iterator.hasNext();)
    {
      SystemObject objekt = (SystemObject) iterator.next();

      ConfigurationArea area = objekt.getConfigurationArea();

      if ( !konfBereiche.contains( area )
          && m_preferenceStore.getBoolean( Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId() ) )
      {
        konfBereiche.add( area );
      }
    }

    // Konfigurationsbereiche
    for (Iterator iterator = konfBereiche.iterator(); iterator.hasNext();)
    {
      ConfigurationArea ca = (ConfigurationArea) iterator.next();
      INavigatorElement elem1 = new NavigatorElement( root, "Konfigurationsbereich " + ca.getNameOrPidOrId(),
          KATEGORIE.KonfigBereich, null );

      // Anzeigequerschnitt
      for (Iterator iterator1 = objekte.iterator(); iterator1.hasNext();)
      {
        AttributeGroup atgAQ = m_dataModel.getAttributeGroup( "atg.anzeigeQuerschnittEigenschaftIst" );
        Aspect aspAQ = m_dataModel.getAspect( "asp.aqZustand" );
        DataDescription ddAQ = new DataDescription( atgAQ, aspAQ );

        SystemObject objekt = (SystemObject) iterator1.next();
        ConfigurationArea area = objekt.getConfigurationArea();

        if ( ca.equals( area ) )
        {
          INavigatorElement elem2 = new NavigatorElement( elem1, "Anzeigequerschnitt "
              + objekt.getNameOrPidOrId(), KATEGORIE.Anzeigequerschnitt, getLinkedList( objekt ) );

          
          // Listener auf die Statusaktualisierung setzen --> startet Methode
          // update des NavigatorElements
          m_connection.subscribeReceiver( elem2, objekt, ddAQ, ReceiveOptions.normal(), ReceiverRole.receiver() );

          // Anzeige
          AttributeGroup atg = m_dataModel.getAttributeGroup( "atg.anzeigeEigenschaftIst" );
          Aspect asp = m_dataModel.getAspect( "asp.aqZustand" );
          DataDescription dd = new DataDescription( atg, asp );

          ConfigurationObject cobjekt = (ConfigurationObject) objekt;
          NonMutableSet anzeige = cobjekt.getNonMutableSet( "Anzeigen" );
          List<SystemObject> anz = new LinkedList<SystemObject>();
          if ( anzeige != null )
            anz = anzeige.getElements();

          for (SystemObject systemObject : anz)
          {
            INavigatorElement elem3 = new NavigatorElement( elem2,
                "Anzeige " + systemObject.getNameOrPidOrId(), KATEGORIE.Anzeige, getLinkedList( systemObject ) );
            
            // Listener auf die Statusaktualisierung setzen --> startet Methode
            // update des NavigatorElements
            m_connection.subscribeReceiver( elem3, systemObject, dd, ReceiveOptions.normal(), ReceiverRole.receiver() );
          }// for systemObject
        }// if area
      }// for Anzeigequerschnitte

      m_progressMonitor.worked( 1 );
      if ( m_progressMonitor.isCanceled() )
        return;
    }// for Konfigurationsbereiche
  }

  /**
   * Baumstruktur fr die Anzeigequerschnitte generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeAnzeigequerschnitte( DataModel dataModel, INavigatorElement root, List<SystemObject> objekte )
  {
    for (Iterator iterator1 = objekte.iterator(); iterator1.hasNext();)
    {
      AttributeGroup atgAQ = m_dataModel.getAttributeGroup( "atg.anzeigeQuerschnittEigenschaftIst" );
      Aspect aspAQ = m_dataModel.getAspect( "asp.aqZustand" );
      DataDescription ddAQ = new DataDescription( atgAQ, aspAQ );

      SystemObject objekt = (SystemObject) iterator1.next();
      ConfigurationArea area = objekt.getConfigurationArea();

      if ( !(m_preferenceStore.getBoolean( Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId() )) )
        continue;

      INavigatorElement elem = new NavigatorElement( root, "Anzeigequerschnitt " + objekt.getNameOrPidOrId(),
          KATEGORIE.Anzeigequerschnitt, getLinkedList( objekt ) );

      // Listener auf die Statusaktualisierung setzen --> startet Methode update
      // des NavigatorElements
      m_connection.subscribeReceiver( elem, objekt, ddAQ, ReceiveOptions.normal(), ReceiverRole.receiver() );

      // Anzeigen
      AttributeGroup atg = m_dataModel.getAttributeGroup( "atg.anzeigeEigenschaftIst" );
      Aspect asp = m_dataModel.getAspect( "asp.aqZustand" );
      DataDescription dd = new DataDescription( atg, asp );

      ConfigurationObject cobjekt = (ConfigurationObject) objekt;
      NonMutableSet anzeige = cobjekt.getNonMutableSet( "Anzeigen" );
      List<SystemObject> anz = new LinkedList<SystemObject>();
      if ( anzeige != null )
        anz = anzeige.getElements();

      for (SystemObject systemObject : anz)
      {
        INavigatorElement elem3 = new NavigatorElement( elem, "Anzeige " + systemObject.getNameOrPidOrId(),
            KATEGORIE.Anzeige, getLinkedList( systemObject ) );

        // Listener auf die Statusaktualisierung setzen --> startet Methode
        // update des NavigatorElements
        m_connection.subscribeReceiver( elem3, systemObject, dd, ReceiveOptions.normal(), ReceiverRole.receiver() );
      }// for Anzeigen
    }// for Anzeigequerschnitt

    m_progressMonitor.worked( 1 );
    if ( m_progressMonitor.isCanceled() )
      return;

  }

  /**
   * Baumstruktur fr Mens der Messquesrschnitte generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   */
  private void bestimmeMenuMessquerschnitte( DataModel dataModel, INavigatorElement root )
  {
    SystemObjectType sot = dataModel.getType( "typ.messQuerschnittAllgemein" );

    List<SystemObject> objekte = sot.getElements();

    if ( objekte.size() != 0 )
    {
      String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[3];
      if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) != 0 )
      {
        INavigatorElement elem = new NavigatorElement( root, elemTitel, KATEGORIE.Messquerschnitte, null );
        bestimmeKonfigurationsBereicheFuerMessquerschnitte( dataModel, elem, objekte );
      }
      elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[4];
      if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) != 0 )
      {
        INavigatorElement elem = new NavigatorElement( root, elemTitel, KATEGORIE.Messquerschnitte, null );
        bestimmeMessquerschnitte( dataModel, elem, objekte );
      }
    }
    if ( m_debug )
      System.out.println( "Messquerschnitte fertig!" );
  }

  /**
   * Baumstruktur fr die Konfigurationsbereiche der Messquerschnitte generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeKonfigurationsBereicheFuerMessquerschnitte( DataModel dataModel, INavigatorElement root,
      List<SystemObject> objekte )
  {
    List<ConfigurationArea> konfBereiche = new Vector<ConfigurationArea>();

    // Objekte der Konfigurationsbereiche ermitteln
    for (Iterator iterator = objekte.iterator(); iterator.hasNext();)
    {
      SystemObject objekt = (SystemObject) iterator.next();

      ConfigurationArea area = objekt.getConfigurationArea();

      if ( !konfBereiche.contains( area )
          && m_preferenceStore.getBoolean( Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId() ) )
      {
        konfBereiche.add( area );
      }
    }

    // Konfigurationsbereiche
    for (Iterator iterator = konfBereiche.iterator(); iterator.hasNext();)
    {
      ConfigurationArea ca = (ConfigurationArea) iterator.next();
      INavigatorElement elem1 = new NavigatorElement( root, "Konfigurationsbereich " + ca.getNameOrPidOrId(),
          KATEGORIE.KonfigBereich, null );

      // Messquerschnitt
      for (Iterator iterator1 = objekte.iterator(); iterator1.hasNext();)
      {
        SystemObject objekt = (SystemObject) iterator1.next();
        ConfigurationArea area = objekt.getConfigurationArea();

        if ( ca.equals( area ) )
        {
          INavigatorElement elem2 = new NavigatorElement( elem1,
              "Messquerschnitt " + objekt.getNameOrPidOrId(), KATEGORIE.Messquerschnitt, getLinkedList( objekt ) );

          ConfigurationObject cobjekt = (ConfigurationObject) objekt;
          NonMutableSet fahrstreifen = null;
      		try {
      			fahrstreifen = cobjekt.getNonMutableSet( "FahrStreifen" );
      		} catch (Exception e) {
//      			e.printStackTrace();
      		}
          List<SystemObject> fahr = new LinkedList<SystemObject>();
          if ( fahrstreifen != null )
            fahr = fahrstreifen.getElements();

          // Fahrspur
          for (SystemObject systemObject : fahr)
          {
            INavigatorElement elem3 = new NavigatorElement( elem2, "Fahrstreifen "
                + systemObject.getNameOrPidOrId(), KATEGORIE.Fahrspur, getLinkedList( systemObject ) );
          }
        }// if area
      }// for Messquerschnitt

      m_progressMonitor.worked( 1 );
      if ( m_progressMonitor.isCanceled() )
        return;

    }// for Konfigurationsbereiche
  }

  /**
   * Baumstruktur fr die Messquerschnitte generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeMessquerschnitte( DataModel dataModel, INavigatorElement root, List<SystemObject> objekte )
  {
    for (Iterator iterator1 = objekte.iterator(); iterator1.hasNext();)
    {
      SystemObject objekt = (SystemObject) iterator1.next();
      ConfigurationArea area = objekt.getConfigurationArea();

      if ( !(m_preferenceStore.getBoolean( Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId() )) )
        continue;

      INavigatorElement elem = new NavigatorElement( root, "Messquerschnitt " + objekt.getNameOrPidOrId(),
          KATEGORIE.Messquerschnitt, getLinkedList( objekt ) );

      ConfigurationObject cobjekt = (ConfigurationObject) objekt;
      NonMutableSet fahrstreifen = null;
  		try {
  			fahrstreifen = cobjekt.getNonMutableSet( "FahrStreifen" );
  		} catch (Exception e) {		
  	//		e.printStackTrace();
  		}
      List<SystemObject> fahr = new LinkedList<SystemObject>();
      if ( fahrstreifen != null )
        fahr = fahrstreifen.getElements();

      for (SystemObject systemObject : fahr)
      {
        INavigatorElement elem3 = new NavigatorElement( elem,
            "Fahrstreifen " + systemObject.getNameOrPidOrId(), KATEGORIE.Fahrspur, getLinkedList( systemObject ) );
      }
    }
  }

  /**
   * Baumstruktur fr die Kommunikation extern generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   */
  private void bestimmeMenuKommunikationExtern( DataModel dataModel, INavigatorElement root )
  {
    String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[0];
    if ( (m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel )) == 0 )
      return;
    
    SystemObjectType sot = dataModel.getType( "typ.uz" );

    List<SystemObject> objekte = sot.getElements();

    if ( objekte.size() != 0 )
    {
      INavigatorElement elem = new NavigatorElement( root, elemTitel, KATEGORIE.Wetterstation,
          null );
      bestimmeKonfigurationsBereicheFuerUzen( dataModel, elem, objekte );
    }

    if ( m_debug )
      System.out.println( elemTitel + " fertig!" );
  }

  /**
   * Baumstruktur fr die Konfigurationsbereiche der UZ generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeKonfigurationsBereicheFuerUzen( DataModel dataModel, INavigatorElement root,
      List<SystemObject> objekte )
  {
    // Objekte der Konfigurationsbereiche ermitteln
    List<ConfigurationArea> konfBereiche = new Vector<ConfigurationArea>();

    for (Iterator iterator = objekte.iterator(); iterator.hasNext();)
    {
      SystemObject objekt = (SystemObject) iterator.next();

      ConfigurationArea area = objekt.getConfigurationArea();

      if ( !konfBereiche.contains( area )
          && m_preferenceStore.getBoolean( Activator.PLUGIN_ID + ":" + area.getPid() ) )
      {
        konfBereiche.add( area );
      }
    }

    // Konfigurationsbereiche
    for (Iterator iterator = konfBereiche.iterator(); iterator.hasNext();)
    {
      ConfigurationArea ca = (ConfigurationArea) iterator.next();
      INavigatorElement elem1 = new NavigatorElement( root, "Konfigurationsbereich " + ca.getNameOrPidOrId(),
          KATEGORIE.KonfigBereich, null );
      

      // Unterzentralen
      for (Iterator iterator1 = objekte.iterator(); iterator1.hasNext();)
      {
        SystemObject objekt = (SystemObject) iterator1.next();
        ConfigurationArea area = objekt.getConfigurationArea();

        if ( ca.equals( area ) )
        {
          INavigatorElement elem2 = new NavigatorElement( elem1, "Unterzentrale " + objekt.getNameOrPidOrId(),
              KATEGORIE.Unterzentrale, getLinkedList( objekt ) );
          
          bestimmeUzPartner( dataModel, elem2, objekt );
        }
      }// for Unterzentralen

      m_progressMonitor.worked( 1 );
      if ( m_progressMonitor.isCanceled() )
        return;
    }// for Konfigurationsbereiche
  }

  /**
   * Baumstruktur fr die Partner der Unterzentralen generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeUzPartner( DataModel dataModel, INavigatorElement root, SystemObject objekt )
  {
    ConfigurationObject cobjekt = (ConfigurationObject) objekt;
    NonMutableSet set = cobjekt.getNonMutableSet( "AnschlussPunkteGert" );

    List<SystemObject> listeAP = set.getElements();

    for (SystemObject systemObject : listeAP)
    {
      ConfigurationObject csystemObjekt = (ConfigurationObject) systemObject;
      NonMutableSet set1 = csystemObjekt.getNonMutableSet( "AnschlussPunkteKommunikationsPartner" );

      List<SystemObject> listeAPKP = set1.getElements();

      SystemObject systemObjectKP = listeAPKP.get( 0 );

      Data data = systemObjectKP.getConfigurationData( dataModel
          .getAttributeGroup( "atg.anschlussPunktKommunikationsPartner" ) );

      SystemObject partner = data.getReferenceValue( "KommunikationsPartner" ).getSystemObject();

      if ( partner.getType().getPid().equals( "typ.steuerModul" ) )
      {
        // Inselbus

        Data dataAP = systemObject.getConfigurationData( dataModel.getAttributeGroup( "atg.anschlussPunkt" ) );

        INavigatorElement elem = new NavigatorElement( root, "Inselbus "
            + dataAP.getScaledValue( "PortNummer" ).getValueText(), KATEGORIE.Inselbus, getLinkedList( systemObject ) );
        
        bestimmeSm( dataModel, elem, systemObject );
      }
      else
      {
        // KRI

        Data dataKRI = partner.getConfigurationData( dataModel.getAttributeGroup( "atg.gert" ) );

        INavigatorElement elem = new NavigatorElement( root, dataKRI.getTextValue( "Bezeichnung" )
            .getValueText(), KATEGORIE.KommRechnerIF, getLinkedList( systemObject ) );
        
        bestimmeUzPartner( dataModel, elem, partner );
      }
    }
  }

  /**
   * Baumstruktur fr die Steuermodule generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeSm( DataModel dataModel, INavigatorElement root, SystemObject systemObject )
  {
    ConfigurationObject csystemObjekt = (ConfigurationObject) systemObject;
    NonMutableSet set1 = csystemObjekt.getNonMutableSet( "AnschlussPunkteKommunikationsPartner" );

    List<SystemObject> listeAPKP = set1.getElements();

    for (SystemObject systemObjectKP : listeAPKP)
    {
      Data data = systemObjectKP.getConfigurationData( dataModel
          .getAttributeGroup( "atg.anschlussPunktKommunikationsPartner" ) );

      SystemObject partner = data.getReferenceValue( "KommunikationsPartner" ).getSystemObject();

      Data dataSM = partner.getConfigurationData( dataModel.getAttributeGroup( "atg.gert" ) );

      INavigatorElement elem = new NavigatorElement( root, "SteuerModul "
          + dataSM.getTextValue( "Bezeichnung" ).getValueText(), KATEGORIE.Streckenstation, getLinkedList( partner ) );
      
      
      bestimmeEak( dataModel, elem, partner );
    }
  }

  /**
   * Baumstruktur fr die EinAusgabeKonzentrator generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeEak( DataModel dataModel, INavigatorElement root, SystemObject partner )
  {
    ConfigurationObject csystemObjekt = (ConfigurationObject) partner;
    NonMutableSet set1 = csystemObjekt.getNonMutableSet( "Eak" );

    List<SystemObject> listeEAK = set1.getElements();

    for (SystemObject systemObjectEAK : listeEAK)
    {
      Data data = systemObjectEAK.getConfigurationData( dataModel.getAttributeGroup( "atg.eak" ) );
      
      INavigatorElement elem = new NavigatorElement( root, data.getTextValue( "Bezeichnung" ).getValueText(),
          KATEGORIE.EinAusKonz, getLinkedList( systemObjectEAK ) );
      
      bestimmeDe( dataModel, elem, systemObjectEAK );
    }

  }

  /**
   * Baumstruktur fr die Datenendgerte generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeDe( DataModel dataModel, INavigatorElement root, SystemObject partner )
  {
    
    
    
    AttributeGroup atg = m_dataModel.getAttributeGroup( "atg.tlsGloDeFehler" );
    Aspect asp = m_dataModel.getAspect( "asp.tlsAntwort" );
    DataDescription dd = new DataDescription( atg, asp );

    ConfigurationObject csystemObjekt = (ConfigurationObject) partner;
    NonMutableSet set1 = csystemObjekt.getNonMutableSet( "De" );

    List<SystemObject> listeDE = set1.getElements();

    for (SystemObject systemObjectDE : listeDE)
    {
      Data data = systemObjectDE.getConfigurationData( dataModel.getAttributeGroup( "atg.de" ) );

      if ( data.getUnscaledValue( "DEKanal" ).intValue() == 255 ) // Sammelkanal
      // herausfiltern
      {
        continue;
      }

      KATEGORIE deKategorie = bestimmeDeKategorie( systemObjectDE.getType() );

      
      
      INavigatorElement elem = new NavigatorElement( root, "DE " + systemObjectDE.getNameOrPidOrId(),
          deKategorie, getLinkedList( systemObjectDE ) );

      // INavigatorElement elem = new NavigatorElementMitStati( root, "DE "
      // + data.getTextValue( "Bezeichnung" ).getValueText(), deKategorie,
      // getLinkedList( systemObjectDE ) );

      // Listener auf die Statusaktualisierung setzen --> startet Methode update
      // des NavigatorElements
      m_connection.subscribeReceiver( elem, systemObjectDE, dd, ReceiveOptions.normal(), ReceiverRole.receiver() );
    }

  }

  /**
   * Kategorie der Datenendgerte bestimmen
   * 
   * @param type
   *          Objekttyp
   * @return Kategorie fr NavigatorElement ( Default: KATEGORIE.Datenendgeraet )
   */
  private KATEGORIE bestimmeDeKategorie( SystemObjectType type )
  {
    KATEGORIE kat = DEKATEGORIE.get( type.getPid() );

    // rekursiv nach einem gltigen SuperType suchen
    if ( kat == null && !(type.getPid().equals( "type.de" )) )
      kat = bestimmeDeKategorie( type.getSuperTypes().get( 0 ) );

    // Defaultwert
    if ( kat == null )
      kat = KATEGORIE.Datenendgeraet;

    return kat;
  }

  /**
   * List-Objekt in eine LinkList-Objekt umwandeln
   * 
   * @param objekte
   *          Liste von Systemobjekten
   * @return LinkedList von Systemobjekten
   */
  private LinkedList<SystemObject> getLinkedList( List<SystemObject> objekte )
  {
    LinkedList<SystemObject> lstobj = new LinkedList<SystemObject>();
    if ( objekte.size() != 0 )
    {
      for (Iterator iterator = objekte.iterator(); iterator.hasNext();)
      {
        SystemObject systemObject = (SystemObject) iterator.next();
        lstobj.add( systemObject );
      }
    }
    return lstobj;
  }

  /**
   * Objekt in eine LinkList-Objekt umwandeln
   * 
   * @param objekt
   *          Systemobjekt
   * @return LinkedList von Systemobjekten
   */
  private LinkedList<SystemObject> getLinkedList( SystemObject objekt )
  {
    LinkedList<SystemObject> lstobj = new LinkedList<SystemObject>();
    if ( objekt != null )
      lstobj.add( objekt );

    return lstobj;
  }

  /**
   * Liefert die Wurzel des Navigators
   */
  public INavigatorElement getNavigatorRoot()
  {
    return m_root;
  }

  /**
   * EreignistypenObjekt in der Baumstruktur generieren
   * 
   * @param root
   *          Elternteil zu dem hinzugefgt oder entfertn werden soll
   * @param systemObject
   *          Objekt des Datenverteilers
   */
  private INavigatorElement erstelleSswSbaUrsachenEinheitObjekt(INavigatorElement root, SystemObject systemObject)
  {
    ConfigurationArea area = systemObject.getConfigurationArea();

    if (!(m_preferenceStore.getBoolean( Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId())))
      return null;

    INavigatorElement element = new NavigatorElementUrsachenEinheit(root, systemObject.getNameOrPidOrId(),
        KATEGORIE.UrsachenEinheit, getLinkedList(systemObject));

    AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaEinzelsituationVorstufe");
    Aspect asp = m_dataModel.getAspect("asp.zustand");
    
    if ((atg != null) && (asp != null))
    {
      DataDescription dd = new DataDescription(atg, asp);

      DynamicObject dobjekt = (DynamicObject)systemObject;

      dobjekt.addListenerForInvalidation(new InvalidationListener()
      {
        @Override
        public void invalidObject(DynamicObject dynamicObject)
        {
          if (dynamicObject != null)
          {
        	navigatorElementAusBaumEntfernen(m_ursachenEinheiten, dynamicObject);
          }
        }
      });

      // Listener auf die Statusaktualisierung setzen --> startet Methode update
      // des NavigatorElements
      m_connection.subscribeReceiver(element, systemObject, dd, ReceiveOptions.delta(), ReceiverRole.receiver());
    }
    
    return element;
  }

  /**
   * Methode fr das Lschen eines Baumelemnts aus der Baumstruktur mittels iterativem Suchen. 
   * 
   * @param iNavigatorElement Baumelement, von dem aus in den Kindern oder in ihm selbst gesucht wird.
   * @param dynamicObject Dynamisches Objekt das gesucht wird.
   * @return <code>true</code> wenn Baumelement gelscht, sonst <code>false</code>. 
   */
  private boolean navigatorElementAusBaumEntfernen(INavigatorElement iNavigatorElement, DynamicObject dynamicObject)
  {
    Object[] children = iNavigatorElement.getChildren();
    
    for (int i = 0; i < children.length; i++)
    {
      INavigatorElement child = (INavigatorElement)children[i];
      
      if (child.hasChildren())
      {
        boolean geloescht = navigatorElementAusBaumEntfernen(child, dynamicObject);
        
        if (geloescht)
        {
          return true;
        }
      }
      else
      {
  	    if (child.hasObjekt(dynamicObject))
    	{
    	  INavigatorElement parent = (INavigatorElement)child.getParent();
    	  parent.remove(child);
    	  
    	  // TODO angemeldete Daten abmelden und NavigatorElement lschen
    		  
    	  return true;
   	    }
      }
    }
	  
	return false;
  }
  
  /**
   * Baumstruktur fr Mens der Umfelddatenmessstellen generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   */
  private void bestimmeMenuUmfeldDatenMessstellen(DataModel dataModel, INavigatorElement root)
  {
    SystemObjectType sot = dataModel.getType("typ.umfeldDatenMessStelle");

    if (sot != null)
    {
      List<SystemObject> objekte = sot.getElements();

      if (objekte.size() != 0)
      {
        String elemTitel = Navigatorkategorien.NAVIGATORKATEGORIE[9];
        if ((m_preferenceStore.getInt(Activator.PLUGIN_ID + ":" + elemTitel )) != 0)
        {
          INavigatorElement elem = new NavigatorElement(root, elemTitel, KATEGORIE.Ordner, null);
          bestimmeKonfigurationsBereicheFuerUmfeldDatenMessstellen(dataModel, elem, objekte);
        }
      }
    
      if (m_debug)
        System.out.println("UmfeldDatenMessstellen fertig!");
    }
  }

  /**
   * Baumstruktur fr die Konfigurationsbereiche der Umfelddatenmessstellen generieren
   * 
   * @param dataModel
   *          Datenmodell des Datenverteilers
   * @param root
   *          Knoten des Elternteils
   * @param objekte
   *          Objekte des Datenverteilers
   */
  private void bestimmeKonfigurationsBereicheFuerUmfeldDatenMessstellen(DataModel dataModel, INavigatorElement root,
      List<SystemObject> objekte)
  {
    List<ConfigurationArea> konfBereiche = new Vector<ConfigurationArea>();

    // Objekte der Konfigurationsbereiche ermitteln
    for (Iterator iterator = objekte.iterator(); iterator.hasNext();)
    {
      SystemObject objekt = (SystemObject)iterator.next();

      ConfigurationArea area = objekt.getConfigurationArea();

      if ( !konfBereiche.contains(area)
          && m_preferenceStore.getBoolean(Activator.PLUGIN_ID + ":" + area.getPidOrNameOrId()))
      {
        konfBereiche.add(area);
      }
    }

    // Konfigurationsbereiche
    for (Iterator iterator = konfBereiche.iterator(); iterator.hasNext();)
    {
      ConfigurationArea ca = (ConfigurationArea)iterator.next();
      INavigatorElement elem1 = new NavigatorElement(root, "Konfigurationsbereich " + ca.getNameOrPidOrId(),
          KATEGORIE.Ordner, null);

      // Umfelddatenmessstellen
      for (Iterator iterator1 = objekte.iterator(); iterator1.hasNext();)
      {
        SystemObject objekt = (SystemObject)iterator1.next();
        ConfigurationArea area = objekt.getConfigurationArea();

        if (ca.equals(area))
        {
          INavigatorElement elem2 = new NavigatorElementUmfeldDaten( elem1,
              "UmfeldDatenMessstelle " + objekt.getNameOrPidOrId(), KATEGORIE.UmfeldDatenMessStelle, getLinkedList( objekt ) );

          ConfigurationObject cobjekt = (ConfigurationObject)objekt;

          NonMutableSet umfelddatensensoren = cobjekt.getNonMutableSet("UmfeldDatenSensoren");

          if (umfelddatensensoren != null)
          {
        	List<SystemObject> ufds = umfelddatensensoren.getElements();

            // UmfeldDatenSensor
            for (SystemObject systemObject : ufds)
            {
              KATEGORIE kat = KATEGORIE.UmfeldDatenSensor;

        	  if (systemObject.getType().getPid().equals("typ.ufdsHelligkeit"))
        	  {
                kat = KATEGORIE.UmfeldDatenSensorHelligkeit;
        	  }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsFahrBahnOberFlchenTemperatur"))
              {
                kat = KATEGORIE.UmfeldDatenSensorFahrBahnOberflaechenTemperatur;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsFahrBahnOberFlchenZustand"))
              {
                kat = KATEGORIE.UmfeldDatenSensorFahrBahnOberflaechenZustand;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsGefrierTemperatur"))
              {
                kat = KATEGORIE.UmfeldDatenSensorGefrierTemperatur;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsLuftTemperatur"))
              {
                kat = KATEGORIE.UmfeldDatenSensorLuftTemperatur;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsNiederschlagsArt"))
              {
                kat = KATEGORIE.UmfeldDatenSensorNiederschlagsart;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsNiederschlagsIntensitt"))
              {
                kat = KATEGORIE.UmfeldDatenSensorNiederschlagsintensitaet;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsRelativeLuftFeuchte"))
              {
                kat = KATEGORIE.UmfeldDatenSensorRelativeLuftFeuchte;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsRestSalz"))
              {
                kat = KATEGORIE.UmfeldDatenSensorRestSalz;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsSichtWeite"))
              {
                kat = KATEGORIE.UmfeldDatenSensorSichtWeite;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsTaupunktTemperatur"))
              {
                kat = KATEGORIE.UmfeldDatenSensorTaupunktTemperatur;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsTemperaturInTiefe1"))
              {
                kat = KATEGORIE.UmfeldDatenSensorTemperaturInTiefe1;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsTemperaturInTiefe3"))
              {
                kat = KATEGORIE.UmfeldDatenSensorTemperaturInTiefe3;
              }
        	  else if (systemObject.getType().getPid().equals("typ.ufdsWasserFilmDicke"))
              {
                kat = KATEGORIE.UmfeldDatenSensorWasserFilmDicke;
              }
        	  
              new NavigatorElementUmfeldDaten(elem2, "UmfelddatenSensor " + systemObject.getNameOrPidOrId(), kat, getLinkedList(systemObject));
            }
          }
        }// if area
      }// for Messquerschnitt

      m_progressMonitor.worked( 1 );
      if ( m_progressMonitor.isCanceled() )
        return;

    } // for Konfigurationsbereiche
  }
}
