/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatoreinstellungen;


import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sba.plugin.navigator.Activator;
import de.bsvrz.sba.plugin.navigator.navigatorView.NavigatorView;

/**
 * Liefert die {@link PreferencePage} zum Durchfhren der Navigatoreinstellungen zu den Konfigurationsbereichen
 * 
 * @author KrauseS
 * 
 */
public class Konfigurationsbereiche extends PreferencePage implements IWorkbenchPreferencePage
{
  /**
   * ID der {@link PreferencePage}
   */
  public static String ID = "de.bsvrz.sba.plugin.navigator.pageKonfigurationsbereiche";

  /**
   * Flag zum Debuggen
   */
  protected static final boolean m_debug = false;

  /**
   * Label zu den Konfigurationsbereichen
   */
  private Label m_lbKonfig;
  
  /**
   * Anzahl der ausgewhlten Konfigurationsbereiche
   */
  private int m_prefKonfig;
  
  /**
   * Anzahl der Konfigurationsbereiche
   */
  private int m_anzKonfig;
  
  /**
   * Tabelle der Konfigurationsbereiche
   */
  private Table m_tableKonfig;
  
  /**
   * Verbindung zum Datenverteiler
   */
  private ClientDavInterface m_connection;

  /**
   * DatenModell zum Datenverteiler
   */
  private DataModel m_dataModel;

  /**
   * Konstruktor, parameterlos.
   */
  public Konfigurationsbereiche()
  {
  }

  /**
   * Konstruktor, setzt den Titel der Navigatoreinstellungen zu den Konfigurationsbereichen.
   * 
   * @param title
   *          Titel
   */
  public Konfigurationsbereiche( String title )
  {
    super( title );
  }

  /**
   * Konstruktor, setzt den Titel und das Bild der Navigatoreinstellungen zu den Konfigurationsbereichen.
   * 
   * @param title
   *          Titel
   * @param image
   *          Bild
   */
  public Konfigurationsbereiche( String title, ImageDescriptor image )
  {
    super( title, image );
  }

  @Override
  protected Control createContents( Composite parent )
  {  
    Composite top = new Composite(parent, SWT.NONE);
    
//    Label lbinfo = new Label( parent, SWT.HORIZONTAL | SWT.WRAP );
    Label lbinfo = new Label( top, SWT.NONE);
    lbinfo.setText( "Diese Navigatoreinstellungen ermglichen die Aktivierung und Deaktivierung bestimmter Konfigurationsbereiche der Datensicht im Navigator.\n"
        + "Durch Deaktivierung der nicht bentigten Konfigurationsbereiche wird die Ladezeit fr die Daten des Naigators verkrzt.\n\n" );
    
    // Tabelle der Konfigurationsbereiche erzeugen
    erzeugeKonfigTabelle( top );

    // zustzliche Buttons erzeugen
    erzeugeButton( top );

    // Daten laden
    ladeDaten();
    
    // Einstellungen laden
    ladeEinstellungen();
    
    /*
     * Layout
     */
    GridLayout glTop = new GridLayout(1, false);
    top.setLayout( glTop );
    
    GridData gdTree = new GridData();
    gdTree.grabExcessHorizontalSpace = true;
    gdTree.horizontalAlignment = GridData.FILL;
    gdTree.grabExcessVerticalSpace = true;
    gdTree.verticalAlignment = GridData.FILL;
    top.setLayoutData( gdTree );

    return null;
  }

  /**
   * Initialisierungen fr die Tabelle der Konfigurationsbereiche
   * 
   * @param parent
   *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
   */
  private void erzeugeKonfigTabelle( Composite parent )
  {

    m_lbKonfig = new Label( parent, SWT.NONE );
    m_lbKonfig.setText( "Konfigurationsbereiche:" );

    GridData data = new GridData( SWT.FILL, SWT.FILL, true, true );
    data.heightHint = 200;
    data.widthHint = 500;
    
    m_tableKonfig = new Table( parent, SWT.CHECK | SWT.BORDER );
    m_tableKonfig.setLayoutData( data );
    
    m_tableKonfig.addListener( SWT.Selection, new Listener()
    {
      @Override
      public void handleEvent( Event event )
      {
        // Zhler hochzhlen
         TableItem item = (TableItem) event.item;
         m_prefKonfig = item.getChecked() ? ++m_prefKonfig : --m_prefKonfig;

        // Anzahl der ausgewhlten Konfigurationsbereiche
        if ( m_anzKonfig != 0 )
        {
          m_lbKonfig.setText( "Konfigurationsbereiche: " + m_prefKonfig + " von " + m_anzKonfig + " ausgewhlt" );
        }

      }

    } );
   
  }

  /**
   * Initialisierungen fr die zustzlichen Buttons
   * 
   * @param parent
   *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
   */
  private void erzeugeButton( Composite parent )
  {
    Composite comp = new Composite( parent, SWT.NONE );
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    comp.setLayout( rowLayout );
    
    GridData data = new GridData( SWT.FILL, SWT.FILL, true, true );
    data.verticalAlignment = SWT.BEGINNING;
    comp.setLayoutData( data );
    
    Button btnAktivAll = new Button( comp, SWT.PUSH );
    btnAktivAll.setText( "Alle aktivieren" );
    btnAktivAll.addSelectionListener( new SelectionAdapter()
    {
      public void widgetSelected( SelectionEvent event )
      {
        TableItem[] tia = m_tableKonfig.getItems();

        for (int i = 0; i < tia.length; i++)
        {
          tia[i].setChecked( true );
        }
        
        m_prefKonfig = m_anzKonfig;
        
        //Anzahl der ausgewhlten Konfigurationsbereiche
        if ( m_anzKonfig != 0 )
        {
          m_lbKonfig.setText( "Konfigurationsbereiche: " + m_prefKonfig + " von " + m_anzKonfig + " ausgewhlt");
        }
      }
    } );
    Button btnDeaktivAll = new Button( comp, SWT.PUSH );
    btnDeaktivAll.setText( "Alle deaktivieren" );
    btnDeaktivAll.addSelectionListener( new SelectionAdapter()
    {
      public void widgetSelected( SelectionEvent event )
      {
        TableItem[] tia = m_tableKonfig.getItems();

        for (int i = 0; i < tia.length; i++)
        {
          tia[i].setChecked( false );
        }
        
        m_prefKonfig = 0;
        
        //Anzahl der ausgewhlten Konfigurationsbereiche
        if ( m_anzKonfig != 0 )
        {
          m_lbKonfig.setText( "Konfigurationsbereiche: " + m_prefKonfig + " von " + m_anzKonfig + " ausgewhlt");
        }
      }
    } );

  }

  /**
   * Ldt die Daten aus dem Datenverteiler
   */
  private void ladeDaten()
  {
    // Verbindung zum Datenverteiler aufbauen
    m_connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if ( m_connection != null )
    {
      if ( m_debug )
        System.out.println( "Verbindung zum Datenverteiler aufgebaut" );
      m_dataModel = m_connection.getDataModel();

      // KonfigurationsBereiche laden
      bestimmeKonfigurationsBereiche();
    }
    else
    {
      TableItem item = new TableItem( m_tableKonfig, SWT.NONE );
      item.setText( "Keine Verbindung zum Datenverteiler!" );
      item.setChecked( false );
      m_tableKonfig.setEnabled( false );
    }

  }
  
  /**
   * Ermitteln der Konfigurationsbereiche und Eintrag in die Tabelle
   */
  private void bestimmeKonfigurationsBereiche()
  {
    SystemObjectType sot = m_dataModel.getType( "typ.konfigurationsBereich" );

    List<SystemObject> objekte = sot.getElements();
    
    if ( objekte.size() != 0 )
    {
      //Konfigurationsbereiche merken
      HashMap<String, KonfigEintrag> listeKonfig = new HashMap<String, KonfigEintrag>();
      
      for (Iterator<SystemObject> iterator = objekte.iterator(); iterator.hasNext();)
      {
        SystemObject objekt = (SystemObject) iterator.next();

        ConfigurationArea ca = objekt.getConfigurationArea();
        
        KonfigEintrag konfigEintrag = new KonfigEintrag(ca);
        
        listeKonfig.put(konfigEintrag.getNameKonfigurationsBereich(), konfigEintrag);
      }
      
      //Sortierung der Konfigurationsbereiche
      TreeSet<String> treeSet = new TreeSet<String>(listeKonfig.keySet());
      
      //Anzahl der Konfigurationsbereiche
      m_anzKonfig = treeSet.size();
      
      for (Iterator<String> iterator = treeSet.iterator(); iterator.hasNext();)
      {
        String key = (String)iterator.next();
        
        KonfigEintrag konfigEintrag = listeKonfig.get(key);
        
        TableItem item = new TableItem(m_tableKonfig, SWT.NONE);
        item.setText(konfigEintrag.toString());
        item.setChecked( false );
        item.setData(Activator.PLUGIN_ID + ":" + konfigEintrag.getPidKonfigurationsBereich());
      }
            
    }
    else
    {
      TableItem item = new TableItem( m_tableKonfig, SWT.NONE );
      item.setText( "Keine Konfigurationsbereich vorhanden!" );
      item.setChecked( false );
      m_tableKonfig.setEnabled( false );
    }

  }

 
  /**
   * Ldt die Einstellungen
   */
  private void ladeEinstellungen()
  {
    System.out.println("Konfigurationsbereiche.ladeEinstellungen()");
    
    // Hole den Prferenzen Speicher
    IPreferenceStore preferenceStore = getPreferenceStore();
    m_prefKonfig = 0;
    assert preferenceStore != null;

    TableItem[] tia = m_tableKonfig.getItems();
    
    for (int i = 0; i < tia.length; i++)
    {
      boolean check = preferenceStore.getBoolean( tia[i].getData().toString() );
      tia[i].setChecked( check );
      if (check)
        m_prefKonfig++;
    }
    
    //Anzahl der ausgewhlten Konfigurationsbereiche
    if ( m_anzKonfig != 0 )
    {
      m_lbKonfig.setText( "Konfigurationsbereiche: " + m_prefKonfig + " von " + m_anzKonfig + " ausgewhlt");
    }    
  }
 
  /**
   * Speichert die Einstellungen
   * 
   */
  private void sichernEinstellungen()
	{
		// Hole den Prferenzen Speicher
		IPreferenceStore preferenceStore = getPreferenceStore();

		// Speichern, ob das Element angezeigt werden soll
		TableItem[] tia = m_tableKonfig.getItems();

		for (int i = 0; i < tia.length; i++)
		{
			preferenceStore.setValue(tia[i].getData().toString(), tia[i].getChecked());
		}

		// Navigator aktualisieren
		IWorkbench workbench = Activator.getDefault().getWorkbench();
		try
		{
			IWorkbenchPage activePage = workbench.getActiveWorkbenchWindow().getActivePage();
			IViewPart view = activePage.showView(NavigatorView.ID);
			((NavigatorView) view).ladeNavigator();
		} 
		catch (Exception e)
		{
		  
		}

	}

  @Override
  protected void performApply()
  {
    sichernEinstellungen();
  }

  @Override
  public boolean performOk()
  {
    performApply();
    return super.performOk();
  }

  @Override
  public void performHelp()
  {
    String helpContextId = Activator.PLUGIN_ID + ".konfigurationsbereiche";
//    PlatformUI.getWorkbench().getHelpSystem().displayHelp( helpContextId );
  }

  @Override
  public void init( IWorkbench workbench )
  {
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    setPreferenceStore( store );

  }
  
  @Override
  protected void performDefaults()
  {
    ladeEinstellungen();
    super.performDefaults();

  }


  /**
   * Eintrag eines Konfigurationsbereiches in den Prferenzen
   * 
   * @author Krause
   *
   */
  private class KonfigEintrag
  {
    ConfigurationArea m_ca;
    
    public KonfigEintrag(ConfigurationArea ca)
    {
      m_ca = ca;
    }
    
    public String getPidKonfigurationsBereich()
    {
      return m_ca.getPidOrNameOrId();
    }

    public String getNameKonfigurationsBereich()
    {
      return m_ca.getNameOrPidOrId();
    }

    @Override
    public String toString()
    {
      return m_ca.getNameOrPidOrId() + " (KonfigurationsVerantwortlicher: " + m_ca.getConfigurationAuthority().getNameOrPidOrId() + ")";
    }
  }
  
}
