/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatoreinstellungen;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sba.plugin.navigator.Activator;
import de.bsvrz.sba.plugin.navigator.navigatorView.NavigatorView;

/**
 * Liefert die {@link PreferencePage} zum Durchfhren der Navigatoreinstellungen
 * zu den Navigatorkategorien
 * 
 * @author KrauseS
 * 
 */
public class Navigatorkategorien extends PreferencePage implements
		IWorkbenchPreferencePage
{
	/**
	 * ID der {@link PreferencePage}
	 */
	public static String ID = "de.bsvrz.sba.plugin.navigator.pageNavigatorkategorien";

	/**
	 * Flag zum Debuggen
	 */
	protected static final boolean m_debug = false;

	/**
	 * Titel des Prferenzen-Seite
	 */
	private String m_titel;

	/**
	 * Liste der Navigatorkategorien
	 */
	public static final String[] NAVIGATORKATEGORIE = new String[] {
			"Kommunikation extern", "Anzeigequerschnitte",
			"Anzeigequerschnitte ohne Unterteilung", "Messquerschnitte",
			"Messquerschnitte ohne Unterteilung", "Ereignisse", "Ereignistypen",
			"Systemkalendereintrge", "UrsachenEinheiten", "UmfeldDatenMessStellen"};

	/**
	 * Label zu den Navigatorkategorien
	 */
	private Label m_lbKategorien;

	/**
	 * Tabelle der Navigatorkategorien
	 */
	private Table m_tableKategorien;

	/**
	 * Selektierte Navigatorkategorie
	 */
	private TableItem m_selKategorie;

	/**
	 * Button fr die Einsortierung nach oben
	 */
	private Button m_btnUp;

	/**
	 * Button fr die Einsortierung nach unten
	 */
	private Button m_btnDown;

	/**
	 * Speicher der Prferenzen
	 */
	private IPreferenceStore m_preferenceStore;

	/**
	 * Konstruktor, parameterlos.
	 */
	public Navigatorkategorien()
	{
	}

	/**
	 * Konstruktor, setzt den Titel der Navigatoreinstellungen zu den
	 * Navigatorkategorien.
	 * 
	 * @param title
	 *          Titel
	 */
	public Navigatorkategorien(String title)
	{
		super(title);
	}

	/**
	 * Konstruktor, setzt den Titel und das Bild der Navigatoreinstellungen zu den
	 * Navigatorkategorien.
	 * 
	 * @param title
	 *          Titel
	 * @param image
	 *          Bild
	 */
	public Navigatorkategorien(String title, ImageDescriptor image)
	{
		super(title, image);
	}

	@Override
	protected Control createContents(Composite parent)
	{
		m_titel = getTitle();

		Composite top = new Composite(parent, SWT.NONE);

		// 1.Teil des Fensters: Einstellungen der Navigatorkategorien
		Label lbinfo = new Label(top, SWT.NONE);
		lbinfo
				.setText("Diese Navigatoreinstellungen ermglichen die Aktivierung und Deaktivierung bestimmter Navigatorkategorien im Navigator.\n"
						+ "Durch Deaktivierung der nicht bentigten Navigatorkategorien wird die Ladezeit fr die Daten des Naigators verkrzt.\n\n");

		m_lbKategorien = new Label(top, SWT.NONE);
		m_lbKategorien.setText("Navigatorkategorien:");

		Composite comp = new Composite(top, SWT.FILL);
		GridLayout gridLayout = new GridLayout(2, false);
		comp.setLayout(gridLayout);

		// Tabelle der Navigatorkategorien erzeugen
		erzeugeTabelle(comp);

		// zustzliche Buttons erzeugen
		erzeugeButtonRechts(comp);
		erzeugeButtonUnten(comp);

		// Daten setzen
		setzeDaten();

		// Einstellungen laden
		ladeEinstellungen();

		// 2.Teil des Fensters: Import oder Export der Einstellungen
		TableItem[] tia = m_tableKategorien.getItems();
		LinkedList<String> prefKeys = new LinkedList<String>();

		for (int i = 0; i < tia.length; i++)
		{
			prefKeys.add(tia[i].getData().toString());
		}
		String[] arrayPrefKeys = prefKeys.toArray(new String[0]);

		PreferenceImportExport preferenceIO = new PreferenceImportExport(top,
				SWT.NONE, m_titel, m_preferenceStore, arrayPrefKeys,
				PreferenceImportExport.TYP.Integer);
		preferenceIO.addListener(new IPreferenceListener()
		{

			@Override
			public void neueWerte(IPreferenceStore ips)
			{
				if (ips.equals(m_preferenceStore))
				{
					ladeEinstellungen();
				}
			}

		});

		/*
		 * Layout
		 */
		GridLayout glTop = new GridLayout(1, false);
		top.setLayout(glTop);

		GridData gdTree = new GridData();
		gdTree.grabExcessHorizontalSpace = true;
		gdTree.horizontalAlignment = GridData.FILL;
		gdTree.grabExcessVerticalSpace = true;
		gdTree.verticalAlignment = GridData.FILL;
		top.setLayoutData(gdTree);

		return null;
	}

	/**
	 * Initialisierungen fr die Tabelle der Navigatorkategorien
	 * 
	 * @param parent
	 *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
	 */
	private void erzeugeTabelle(Composite parent)
	{

		GridData data = new GridData(SWT.NONE, SWT.FILL, true, true);
		data.heightHint = 300;
		data.widthHint = 500;

		m_tableKategorien = new Table(parent, SWT.CHECK | SWT.BORDER);
		m_tableKategorien.setLayoutData(data);

		m_tableKategorien.addListener(SWT.Selection, new Listener()
		{
			@Override
			public void handleEvent(Event event)
			{
				// Selektion merken
				TableItem item = (TableItem) event.item;
				m_selKategorie = item;

				// entsprechende Up-Down-Buttons de-/aktivieren
				TableItem[] tia = m_tableKategorien.getItems();

				if (tia[0].equals(m_selKategorie))
					m_btnUp.setEnabled(false);
				else
					m_btnUp.setEnabled(true);

				if (tia[tia.length - 1].equals(m_selKategorie))
					m_btnDown.setEnabled(false);
				else
					m_btnDown.setEnabled(true);

			}

		});

	}

	/**
	 * Initialisierungen fr die zustzlichen rechten Buttons
	 * 
	 * @param parent
	 *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
	 */
	private void erzeugeButtonRechts(Composite parent)
	{
		// Buttons fr die Positionierung
		Composite compRechts = new Composite(parent, SWT.NONE);
		RowLayout rowLayout = new RowLayout(SWT.VERTICAL);
		rowLayout.pack = false;
		compRechts.setLayout(rowLayout);

		GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
		data.verticalAlignment = SWT.BEGINNING;
		compRechts.setLayoutData(data);

		m_btnUp = new Button(compRechts, SWT.ARROW | SWT.UP);
		m_btnUp.setEnabled(false);
		m_btnUp.addSelectionListener(new SelectionAdapter()
		{
			// Selektiertes Element nach oben schieben
			public void widgetSelected(SelectionEvent event)
			{
				if (m_selKategorie != null)
				{
					TableItem[] tia = m_tableKategorien.getItems();

					for (int i = 0; i < tia.length; i++)
					{
						if (tia[i].equals(m_selKategorie) && (i != 0))
						{
							String selText = m_selKategorie.getText();
							boolean selCheck = m_selKategorie.getChecked();
							Object selData = m_selKategorie.getData();

							tia[i].setText(tia[i - 1].getText());
							tia[i].setChecked(tia[i - 1].getChecked());
							tia[i].setData(tia[i - 1].getData());

							tia[i - 1].setText(selText);
							tia[i - 1].setChecked(selCheck);
							tia[i - 1].setData(selData);

							m_tableKategorien.setSelection(i - 1);
							m_selKategorie = tia[i - 1];
							break;
						}
					}

				}

			}
		});
		m_btnDown = new Button(compRechts, SWT.ARROW | SWT.DOWN);
		m_btnDown.setEnabled(false);
		m_btnDown.addSelectionListener(new SelectionAdapter()
		{
			// Selektiertes Element nach unten schieben
			public void widgetSelected(SelectionEvent event)
			{
				if (m_selKategorie != null)
				{
					TableItem[] tia = m_tableKategorien.getItems();

					for (int i = 0; i < tia.length; i++)
					{
						if (tia[i].equals(m_selKategorie) && (i != tia.length - 1))
						{
							String selText = m_selKategorie.getText();
							boolean selCheck = m_selKategorie.getChecked();
							Object selData = m_selKategorie.getData();

							tia[i].setText(tia[i + 1].getText());
							tia[i].setChecked(tia[i + 1].getChecked());
							tia[i].setData(tia[i + 1].getData());

							tia[i + 1].setText(selText);
							tia[i + 1].setChecked(selCheck);
							tia[i + 1].setData(selData);

							m_tableKategorien.setSelection(i + 1);
							m_selKategorie = tia[i + 1];
							break;
						}
					}
				}
			}
		});

	}

	/**
	 * Initialisierungen fr die zustzlichen Buttons unten
	 * 
	 * @param parent
	 *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
	 */
	private void erzeugeButtonUnten(Composite parent)
	{
		// Buttons fr die Aktivierung und Deaktivierung
		Composite compUnten = new Composite(parent, SWT.NONE);
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		compUnten.setLayout(rowLayout);

		GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
		data.verticalAlignment = SWT.BEGINNING;
		compUnten.setLayoutData(data);

		Button btnAktivAll = new Button(compUnten, SWT.PUSH);
		btnAktivAll.setText("Alle aktivieren");
		btnAktivAll.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent event)
			{
				TableItem[] tia = m_tableKategorien.getItems();

				for (int i = 0; i < tia.length; i++)
				{
					tia[i].setChecked(true);
				}

			}
		});
		Button btnDeaktivAll = new Button(compUnten, SWT.PUSH);
		btnDeaktivAll.setText(" Alle deaktivieren ");
		btnDeaktivAll.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent event)
			{
				TableItem[] tia = m_tableKategorien.getItems();

				for (int i = 0; i < tia.length; i++)
				{
					tia[i].setChecked(false);
				}
			}
		});

	}

	/**
	 * Setzt die Daten
	 */
	private void setzeDaten()
	{
		for (int i = 0; i < NAVIGATORKATEGORIE.length; i++)
		{
			TableItem item = new TableItem(m_tableKategorien, SWT.NONE);
			item.setText(NAVIGATORKATEGORIE[i]);
			item.setChecked(false);
			item.setData(Activator.PLUGIN_ID + ":" + NAVIGATORKATEGORIE[i]);
		}

	}

	/**
	 * Ldt die Einstellungen
	 */
	private void ladeEinstellungen()
	{
		assert m_preferenceStore != null;

		TableItem[] tia = m_tableKategorien.getItems();
		TreeMap<Integer, String> sortList = new TreeMap<Integer, String>();
		LinkedList<String> deaktivList = new LinkedList<String>();

		// Sortierung merken
		for (int i = 0; i < tia.length; i++)
		{
			int position = m_preferenceStore.getInt(tia[i].getData().toString());
			if (position != 0)
				sortList.put(position - 1, tia[i].getText());
			else
				deaktivList.add(tia[i].getText());
		}

		int index = 0;

		// die ausgewhlten und positionierten Navigatorkategorien
		for (Map.Entry<Integer, String> me : sortList.entrySet())
		{
			String titel = me.getValue();
			tia[index].setText(titel);
			tia[index].setChecked(true);
			tia[index].setData(Activator.PLUGIN_ID + ":" + titel);
			index++;
		}

		// die nicht ausgewhlten Navigatorkategorien
		for (Iterator<String> iterator = deaktivList.iterator(); iterator.hasNext();)
		{
			String titel = (String) iterator.next();
			tia[index].setText(titel);
			tia[index].setChecked(false);
			tia[index].setData(Activator.PLUGIN_ID + ":" + titel);
			index++;
		}

	}

	/**
	 * Speichert die Einstellungen
	 * 
	 */
	private void sichernEinstellungen()
	{

		// Speichern, in welcher Reihenfolge das Element angezeigt werden soll (bei
		// 1 beginnend), keine Anzeige = 0
		TableItem[] tia = m_tableKategorien.getItems();

		for (int i = 0; i < tia.length; i++)
		{
			if (tia[i].getChecked())
				m_preferenceStore.setValue(tia[i].getData().toString(), i + 1);
			else
				m_preferenceStore.setValue(tia[i].getData().toString(), 0);
		}

		// Navigator aktualisieren
		IWorkbench workbench = Activator.getDefault().getWorkbench();
		try
		{
			IWorkbenchPage activePage = workbench.getActiveWorkbenchWindow()
					.getActivePage();
			IViewPart view = activePage.showView(NavigatorView.ID);
			((NavigatorView) view).ladeNavigator();
		}
		catch (Exception e)
		{
		}

	}

	@Override
	protected void performApply()
	{
		sichernEinstellungen();
	}

	@Override
	public boolean performOk()
	{
		performApply();
		return super.performOk();
	}

	@Override
	public void performHelp()
	{
		String helpContextId = Activator.PLUGIN_ID + ".navigatorKategorien";
//		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	@Override
	public void init(IWorkbench workbench)
	{
		// Hole den Prferenzen Speicher
		m_preferenceStore = Activator.getDefault().getPreferenceStore();
		setPreferenceStore(m_preferenceStore);

	}

	@Override
	protected void performDefaults()
	{

		ladeEinstellungen();
		super.performDefaults();

	}

}
