/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatoreinstellungen;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;
import org.jdom.Attribute;
import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.bsvrz.sba.plugin.navigator.Activator;
import de.bsvrz.sba.plugin.navigator.tools.Tools;

public class PreferenceImportExport extends Composite
{

	public static enum TYP
	{
		Boolean, Integer, String
	};

	/**
	 * Titel des Prferenzen-Seite
	 */
	private String m_titlePreference;

	/**
	 * dieses Composite
	 */
	private Composite m_top;

	/**
	 * Button fr den Import
	 */
	private Button m_btnImport;

	/**
	 * Button fr den Export
	 */
	private Button m_btnExport;

	/**
	 * Speicher der Prferenzen
	 */
	private IPreferenceStore m_preferenceStore;

	/**
	 * Liste der Keys der Prferenzen
	 */
	private String[] m_prefKeys;

	/**
	 * Typ der Werte der Prferenzen (Boolean, Integer, String)
	 */
	private TYP m_typPrefWert;

	/**
	 * Listener fr die Benachtichtigung, wenn der Import abgeschlossen ist
	 */
	private List<IPreferenceListener> m_angemeldeteListener = new LinkedList<IPreferenceListener>();

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}
	 * @param style
	 *          Stil
	 * @param title
	 *          Titel der Prferenzen-Seite
	 * @param store
	 *          Speicher mit den Prferenzen
	 * @param prefKeys
	 *          Liste der Keys der Prferenzen
	 * @param typ
	 *          Typ der Werte der Prferenzen
	 */
	public PreferenceImportExport(Composite parent, int style, String title,
			IPreferenceStore store, String[] prefKeys, TYP typ)
	{
		super(parent, style);
		m_top = this;
		m_titlePreference = title;
		m_preferenceStore = store;
		m_prefKeys = prefKeys;
		m_typPrefWert = typ;
		init();
	}

	/**
	 * Fhrt die Initialisierung durch
	 */
	private void init()
	{
		// m_lbio = new Label( this, SWT.NONE);
		// m_lbio.setText( "Import/Export:" );

		m_btnImport = new Button(m_top, SWT.PUSH);
		m_btnImport.setText("Import...");
		m_btnImport.addSelectionListener(new SelectionAdapter()
		{
			// FileEditor fr die Auswahl einer XML.Datei ffnen
			public void widgetSelected(SelectionEvent e)
			{
				FileDialog openDialog = new FileDialog(m_top.getShell(), SWT.OPEN);
				openDialog.setText("Import " + m_titlePreference);
				openDialog.setFilterNames(new String[] { "XML Dateien" });
				openDialog.setFilterExtensions(new String[] { "*.xml" }); // Windows
				// wild cards

				String openFile = openDialog.open();
				if (openFile != null)
				{
					// XML-Datei auslesen
					Element elementXML = leseRootElementXML(openFile);

					if (elementXML != null)
						setPreferences(elementXML);
				}

			}

		});

		m_btnExport = new Button(m_top, SWT.PUSH);
		m_btnExport.setText("  Export...  ");
		m_btnExport.addSelectionListener(new SelectionAdapter()
		{
			// FileEditor fr die Speicherung in einer XML.Datei ffnen
			public void widgetSelected(SelectionEvent e)
			{
				FileDialog openDialog = new FileDialog(m_top.getShell(), SWT.SAVE);
				openDialog.setText("Export " + m_titlePreference);
				openDialog.setFilterNames(new String[] { "XML Dateien" });
				openDialog.setFilterExtensions(new String[] { "*.xml" }); // Windows
				// wild cards

				String saveFile = openDialog.open();

				if (saveFile != null)
				{
					if (m_preferenceStore != null)
					{
						LinkedList<KategorieDatenSatz> exportListe = new LinkedList<KategorieDatenSatz>();

						for (int i = 0; i < m_prefKeys.length; i++)
						{
							String wert = "";
							switch (m_typPrefWert)
							{
							case Boolean:
								wert = String.valueOf(m_preferenceStore
										.getBoolean(m_prefKeys[i]));
								break;
							case Integer:
								wert = String.valueOf(m_preferenceStore.getInt(m_prefKeys[i]));
								break;
							case String:
								wert = m_preferenceStore.getString(m_prefKeys[i]);
								break;
							default:
								break;
							}// switch

							if (!wert.equals(""))
							{
								KategorieDatenSatz ds = new KategorieDatenSatz(m_prefKeys[i],
										"" + m_typPrefWert, wert);
								exportListe.add(ds);
							}
						}// for

						if (exportListe.size() > 0)
						{
							exportXml(exportListe, saveFile);
						}

					}// if store

				}// if file

			}

		});

		/*
		 * Layout
		 */

		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		this.setLayout(rowLayout);

		GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
		data.horizontalAlignment = SWT.BEGINNING;
		this.setLayoutData(data);

	}

	/**
	 * Exportiert eine Liste von {@link KategorieDatenSatz} in ein XML-Datei. Die
	 * Struktur entspricht <code>preferences.dtd</code>. Existiert die zu
	 * schreibende Datei bereits wird nachgefragt, ob die Datei berschrieben
	 * werden darf.
	 * 
	 * @param exportList
	 *          Liste der zu exportierenden {@link KategorieDatenSatz}
	 * @param filePath
	 *          Pfad zur XML-Datei, in welche die Struktur geschrieben werden
	 *          soll.
	 */
	private void exportXml(LinkedList<KategorieDatenSatz> exportList,
			String filePath)
	{
		Element root = new Element(DefsXml.ELE_PREFERENCES_BASIS);
		Document doc = new Document(root);

		Iterator<KategorieDatenSatz> it = exportList.iterator();
		while (it.hasNext())
		{
			KategorieDatenSatz datenSatz = it.next();
			Element element = new Element(DefsXml.ELE_ELEMENT);
			Attribute attKey = new Attribute(DefsXml.ATT_KEY, datenSatz.getKey());
			element.setAttribute(attKey);
			Attribute attTyp = new Attribute(DefsXml.ATT_TYP, datenSatz.getTyp());
			element.setAttribute(attTyp);
			Attribute attWert = new Attribute(DefsXml.ATT_WERT, datenSatz.getWert());
			element.setAttribute(attWert);
			root.addContent(element);
		}
		System.out.println("Wurzel: " + root);

		/*
		 * DocType
		 */
		DocType type = new DocType(DefsXml.xmlDocTypeElementName,
				DefsXml.xmlDocTypePublicId, DefsXml.xmlDocSystemId);
		doc.setDocType(type);

		/*
		 * Formatierung und Ausgabe
		 */
		Format prettyFormat = Format.getPrettyFormat();
		XMLOutputter out = new XMLOutputter(prettyFormat);
		Format f = out.getFormat();
		f.setEncoding(DefsXml.xmlCodierung);
		out.setFormat(f);
		String outputString = out.outputString(doc);
		System.out.println("XML:\n" + outputString);

		/*
		 * Prfen ob Datei existiert und ggf. berschreiben J/N Meldung
		 */
		boolean schreiben = false;
		File file = new File(filePath);
		if (file.exists())
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			mb.setText("Datei \"" + filePath + "\" existiert bereits!");
			mb.setMessage("Die Datei \"" + filePath
					+ "\" existiert bereits! Soll diese berschrieben werden?");
			int open = mb.open();
			if (open == SWT.YES)
			{
				schreiben = true;
			}
		}
		else
		{
			schreiben = true;
		}

		/*
		 * Schreiben, wenn Schreiben ok ist.
		 */
		if (schreiben)
		{
			Tools.writeStringToFile(filePath, true, outputString);
		}

	}

	/**
	 * Beschreibt einen DatenSatz der Navigatorkategorie, bestehend aus Key, Typ
	 * und Wert. Wird fr den Export bentigt.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class KategorieDatenSatz
	{
		/**
		 * Schlssel
		 */
		private String _key = null;

		/**
		 * Typ
		 */
		private String _typ = null;

		/**
		 * Wert
		 */
		private String _wert = null;

		/**
		 * Konstruktor
		 * 
		 * @param key
		 *          Schlssel des Datensatzes
		 * @param typ
		 *          Typ des Datensatzes
		 * @param wert
		 *          Wert des Datensatzes
		 */
		public KategorieDatenSatz(String key, String typ, String wert)
		{
			this._key = key;
			this._typ = typ;
			this._wert = wert;
		}

		/**
		 * Liefert den Schlssel des Datensatzes.
		 * 
		 * @return Schlssel
		 */
		public String getKey()
		{
			return _key;
		}

		/**
		 * Liefert den Typ des Datensatzes.
		 * 
		 * @return Typ
		 */
		public String getTyp()
		{
			return _typ;
		}

		/**
		 * Liefert den Wert des Datensatzes.
		 * 
		 * @return Wert
		 */
		public String getWert()
		{
			return _wert;
		}

		public String toString()
		{
			String re = "Key: " + _key + " Type: " + _typ + " Wert: " + _wert;
			return re;
		}
	}

	/**
	 * Setzen der Prferenzen
	 * 
	 * @param elementXML
	 *          Wurzelelement aus einer XML-Datei
	 */
	@SuppressWarnings("unchecked")
	private void setPreferences(Element element)
	{
		List children = element.getChildren();

		if (children == null)
			return;

		for (Iterator iterator = children.iterator(); iterator.hasNext();)
		{
			Element child = (Element) iterator.next();

			Boolean booWert = false;
			Integer intWert = 0;
			String strkey = child.getAttributeValue("key");
			String strTyp = child.getAttributeValue("typ");
			String strWert = child.getAttributeValue("wert");

			try
			{
				if (strTyp.equals("Boolean"))
				{
					// Wert ist Boolean
					booWert = Boolean.valueOf(strWert);
					m_preferenceStore.setValue(strkey, booWert);
				}

				if (strTyp.equals("Integer"))
				{
					// Wert ist Integer
					intWert = Integer.valueOf(strWert);
					m_preferenceStore.setValue(strkey, intWert);
				}
			}// try
			catch (Exception ex)
			{
				// Wert ist String
				m_preferenceStore.setValue(strkey, strWert);
			}

		}// for

		benachrichtigeListener();

	}

	/**
	 * Methode liest das Rootelement aus einer XML-Datei aus. Vorher wird geprft,
	 * ob die Datei berhaupt existiert und ob sie der DTD Beschreibung
	 * entspricht.
	 * 
	 * @param datei
	 *          Name der Datei die in eine XML-Struktur eingelesen werden soll mit
	 *          absoluter und relativer Pfad-Angabe
	 * @return Element RootElement der XML-Struktur oder null im Fehlerfall
	 */
	private Element leseRootElementXML(String datei)
	{
		// Datei mit absolutem oder relativem Pfad ?

		if (datei.matches("[a-zA-Z]:.+"))
		{
			// Prfen ob Datei berhaupt existiert

			if (!(new File(datei)).exists())
			{
				System.out.println("Datei \"" + datei + "\" nicht gefunden !");
				return null;
			}
		}

		// Einlesen der Datei in eine XML-Struktur

		boolean validate = true; // Validierung gegen die jeweilige DTD

		SAXBuilder builder = new SAXBuilder(validate);
		builder.setEntityResolver(new EntityResolver()
		{
			public InputSource resolveEntity(String publicId, String systemId)
					throws SAXException, IOException
			{
				if ((systemId != null))
				{
					URL url = Activator.getDefault().getBundle().getResource(
							"XML/preferences.dtd");
					assert url != null : this.getClass();

					return new InputSource(url.toExternalForm());
				}

				return null;
			}
		});

		Document jdomDokument = null;

		try
		{
			jdomDokument = builder.build(new File(datei));
		}
		catch (JDOMException e)
		{
			System.out.println("JDOMException: " + e.getMessage());
			return null;
		}
		catch (IOException e)
		{
			System.out.println("IOException: " + e.getMessage());
			return null;
		}

		// Rootelement aus eingelesener Datei bestimmen

		Element rootElement = jdomDokument.getRootElement();

		return rootElement;
	}

	// ----------------------------------------------------------------------------------------------
	// Listener Funktionalitten
	// ----------------------------------------------------------------------------------------------

	/**
	 * Methode zum Hinzufgen eines Listeners
	 * 
	 * @param listener
	 *          Listener der hinzugefgt werden soll
	 */
	public void addListener(IPreferenceListener listener)
	{
		m_angemeldeteListener.add(listener);
	}

	/**
	 * Methode zum Entfernen eines Listeners
	 * 
	 * @param listener
	 *          Listener der entfernt werden soll
	 */
	public void removeListener(IPreferenceListener listener)
	{
		m_angemeldeteListener.remove(listener);
	}

	/**
	 * Methode mit der die Klasse alle bei ihr angemeldeten Listener ber die
	 * nderung der Selektion informiert.
	 * 
	 * @param objekte
	 *          Objekte aus der Listbox
	 */
	private void benachrichtigeListener()
	{
		// zu bergebende Daten eintragen
		Iterator<IPreferenceListener> it = m_angemeldeteListener.iterator();
		while (it.hasNext())
		{
			IPreferenceListener l = it.next();
			l.neueWerte(m_preferenceStore);
		}
	}

}
