package de.bsvrz.sba.plugin.parameter.steuerungskern.dialog;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.sba.plugin.parameter.steuerungskern.Activator;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;

/**
 * Figure used to hold the column labels
 * 
 * @author Krause
 *
 */
public class AnzeigeElementFigure extends Label {

	private String _pid;

	private AnzeigeInhaltTyp _inhalt;

	private AnzeigeElementTyp _typ;

	private boolean _aktiv = true;

	private static AnzeigeInhaltAuswahlDialog _anzeigeInhaltAuswahlDialog;

	public static Color columnColor = new Color(null, 255, 255, 255);

	public AnzeigeElementFigure (String pid, AnzeigeElementTyp typ,	AnzeigeInhaltTyp inhalt, boolean aktiv) 
	{
		_pid = pid;
		_inhalt = inhalt;
		_typ = typ;
		_aktiv = aktiv;

		Image image = null;
		
		if (true) // !typ.equals(AnzeigeElementTyp.FREITEXT))
		{
			IFigure fig = getParent();
			
			if (fig != null)			
			  System.out.println(fig.getClass().getSimpleName());
			
			if (_aktiv)
				image = getImage(typ, inhalt);
			else 
			{
				image = getImage(typ, AnzeigeInhaltTyp.INAKTIV);
			}
			
			if (image != null) 
				setIcon(image);
		}			
		
  	setBorder(new LineBorder(ColorConstants.gray, 2));
		setToolTip(new Label(pid));
		setOpaque(true);
	}

	public void setSelected(boolean isSelected) {
		LineBorder lineBorder = (LineBorder) getBorder();
		if (isSelected) {
			lineBorder.setWidth(2);
		} else {
			lineBorder.setWidth(1);
		}
	}

	public static List<AnzeigeInhaltTyp> createAnzeigeInhalte(AnzeigeElementTyp typ) 
	{
		return createAnzeigeInhalte(typ, false, false, false);
	}

	public static List<AnzeigeInhaltTyp> createAnzeigeInhalteMitBeliebig (AnzeigeElementTyp typ)
	{
		return createAnzeigeInhalte(typ, true, false, false);
	}

	public static List<AnzeigeInhaltTyp> createAnzeigeInhalteMitBeliebigundNegiertenInhalten (AnzeigeElementTyp typ)
	{
		return createAnzeigeInhalte(typ, true, false, true);
	}

	public static List<AnzeigeInhaltTyp> createAnzeigeInhalteMitDontCare (AnzeigeElementTyp typ)
	{
		return createAnzeigeInhalte(typ, false, true, false);
	}

	public static List<AnzeigeInhaltTyp> createAnzeigeInhalte(AnzeigeElementTyp typ, boolean mitBeliebig, boolean mitDontCare, boolean mitNegiertenInhalten) 
	{
		return VerwaltungWvzInhalte.getInstance().getAnzeigeInhaltTypen (typ, mitBeliebig, mitDontCare, mitNegiertenInhalten);
	}

	private static HashMap<String, Image> m_images = new HashMap<String, Image>();

  private static Map _map = Collections.synchronizedMap(m_images);
	
	public static AnzeigeInhaltAuswahlDialog getAnzeigeInhaltAuswahlDialog() {
		return _anzeigeInhaltAuswahlDialog;
	}

	public static void setAnzeigeInhaltAuswahlDialog(
			AnzeigeInhaltAuswahlDialog dialog) {
		// TODO Auto-generated method stub
		_anzeigeInhaltAuswahlDialog = dialog;

	}

	public String getPid() {
		return _pid;
	}

	public void setPid(String pid) {
		_pid = pid;
	}

	public AnzeigeInhaltTyp getInhalt() {
		return _inhalt;
	}

	public void setInhalt(AnzeigeInhaltTyp inhalt) {
		_inhalt = inhalt;
	}

	public AnzeigeElementTyp getTyp() {
		return _typ;
	}

	public void setTyp(AnzeigeElementTyp typ) {
		_typ = typ;
	}

	public boolean isAktiv() {
		return _aktiv;
	}

	public void setAktiv(boolean aktiv) {
		_aktiv = aktiv;

	}

  /**
   * Methode holt ein Image aus den Ressourcen. Es wird dabei aber nur eine Instanz des Images erzeugt.
   * @param pfad Pfad des zu Ladenden Images
   * @return das Image Image
   */
  private static Image getImage (String pfad, boolean negiert)
  {
    if (pfad != null)
    {
    	String key = (negiert ? "!" : "" ) + pfad;
//    	System.out.println("AnzeigeElementFigure.getImage() " + key);
    	
      Image erg = (Image)_map.get (key);

      if (erg == null)
      {
      	Image image = null;
      	
      	try
      	{
   	      image = Activator.getImageDescriptor(pfad).createImage();
      	}
        catch (Exception e) 
        {
          System.err.println("Kein Image fr Pfad " + pfad + " gefunden !!");
        }

        if (image != null)
        {
        	if (negiert)
        	{
        		GC gc = new GC (image);
  
        		Rectangle r = image.getBounds();
        		
        		
        		Color c = Display.getCurrent().getSystemColor(SWT.COLOR_GRAY); 

        		gc.setForeground(c);

//        		gc.drawRectangle(0, 0, r.width, r.height);

        		int art = 1;
        		
        		// Rahmen zeichnen
        		
        		if (art == 0)
        		{
	        		int width  = r.width;
	        		int height = r.height;
	        		
	        		for (int i=0; i<2; ++i)
	        		{
	          		gc.drawRectangle(0 + i, 0 + i, width - 1, height - 1);
	          		width  -= 2;
	          		height -= 2;
	        		}
        		}
        		
        		// X ber den inhalt zeichnen
        		
        		if (art == 1)
        		{
	        		gc.setLineWidth(2);
	        		gc.drawLine(0, 0, r.width,     r.height);
	        		gc.drawLine(0, r.height, r.width, 0);
        		}
        		
        		// Ausrufezeichen auf Bild malen
        		
        		if (art == 2)
           		gc.drawText(" ! ", 0, 0);
        		
        		gc.dispose();
        	}
        	
          _map.put (key, image);

          return image;
        }
      }
      else
        return erg;
    }

    return null;
  }
  
	/**
	 * Methode liefert ein Image zu einem Bildinhalt in Abhngigkeit vom Anzeigtyp
	 * @param typ Anzeigetyp
	 * @param inhalt Bildinhalt 
	 * @return Image
	 */
	public static Image getImage (AnzeigeElementTyp typ,	AnzeigeInhaltTyp inhalt) 
	{
		String pfad = createImagePath (typ, inhalt);
		
		return getImage (pfad, inhalt.isNegiert());
	}

	/**
	 * Methode liefert den Namen eines Images zu einem Bildinhalt in Abhngigkeit vom Anzeigtyp
	 * @param typ Anzeigetyp
	 * @param inhalt Bildinhalt 
	 * @return Name des Images
	 */
	public static String createImagePath (AnzeigeElementTyp typ,	AnzeigeInhaltTyp inhalt)
	{
		if (typ == null || inhalt == null)
		{
			if (typ == null)
				System.out.println("AnzeigeElementFigure.createImagePath() Typ = null");

			if (inhalt == null)
				System.out.println("AnzeigeElementFigure.createImagePath() Inhalt = null");

			return null;
		}

		String imagepath = null;

		if (typ.equals(AnzeigeElementTyp.WZG_A)) 
		{
			if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG)) 
			{
				imagepath = "icons/a1_beliebig.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV)) 
			{
				imagepath = "icons/a1_inaktiv.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE)) 
			{
				imagepath = "icons/a1_inaktiv.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || 
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL)) 
			{
				imagepath = "icons/a1_leer.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL_STAU) || 
							 inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL_STAU)) 
			{
				imagepath = "icons/a1_dunkelStau.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP) ||
							 inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL_HP)) 
			{
				imagepath = "icons/a1_dunkelHP.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_FAHRSTREIFEN_GESPERRT)) 
			{
				imagepath = "icons/a1_111.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS)) 
			{
				imagepath = "icons/a1_113.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS) ||
							 inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS)) 
			{
				imagepath = "icons/a1_114.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_40) ||
							 inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ZULAESSIGE_HOECHST_40)) 
			{
				imagepath = "icons/a1_22.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_50) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ZULAESSIGE_HOECHST_50)) 
			{
				imagepath = "icons/a1_23.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_60) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ZULAESSIGE_HOECHST_60)) 
			{
				imagepath = "icons/a1_24.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_80) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ZULAESSIGE_HOECHST_80)) 
			{
				imagepath = "icons/a1_26.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_100) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ZULAESSIGE_HOECHST_100)) 
			{
				imagepath = "icons/a1_28.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_120) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ZULAESSIGE_HOECHST_120)) 
			{
				imagepath = "icons/a1_30.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_130) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ZULAESSIGE_HOECHST_130)) 
			{
				imagepath = "icons/a1_33.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_40) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_ZULAESSIGE_HOECHST_40)) 
			{
				imagepath = "icons/a1_42.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_50) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_ZULAESSIGE_HOECHST_50)) 
			{
				imagepath = "icons/a1_43.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_60) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_ZULAESSIGE_HOECHST_60)) 
			{
				imagepath = "icons/a1_44.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_80) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_ZULAESSIGE_HOECHST_80)) 
			{
				imagepath = "icons/a1_46.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_100) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_ZULAESSIGE_HOECHST_100)) 
			{
				imagepath = "icons/a1_48.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_120) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_ZULAESSIGE_HOECHST_120)) 
			{
				imagepath = "icons/a1_50.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_ALLER_STRECKENVERBOTE)) 
			{
				imagepath = "icons/a1_53.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_130) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_ZULAESSIGE_HOECHST_130)) 
			{
				imagepath = "icons/a1_54.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_STAU)) 
			{
				imagepath = "icons/a1_9.png";
			} 
			else 
			{
				imagepath = "icons/a1_unbekannt.png";
			}
		} 
		else if (typ.equals(AnzeigeElementTyp.WZG_B)) 
		{
			if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG)) 
			{
				imagepath = "icons/b1_beliebig.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV)) 
   		{
				imagepath = "icons/b1_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE)) 
   		{
				imagepath = "icons/b1_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL)) 
			{
				imagepath = "icons/b1_leer.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL_STAU) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL_STAU)) 
			{
				imagepath = "icons/b1_dunkelStau.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL_HP)) 
			{
				imagepath = "icons/b1_dunkelHP.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GEFAHRENSTELLE) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_GEFAHRENSTELLE)) 
   		{
				imagepath = "icons/b1_1.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GLAETTEGEFAHR) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_GLAETTEGEFAHR)) 
   		{
				imagepath = "icons/b1_3.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_SCHLEUDERGEFAHR) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_SCHLEUDERGEFAHR)) 
   		{
				imagepath = "icons/b1_4.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_BAUSTELLE) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_BAUSTELLE)) 
   		{
				imagepath = "icons/b1_8.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_STAU)) 
   		{
				imagepath = "icons/b1_9.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_ALLGEMEIN) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_UEBERHOLVERBOT_ALLGEMEIN)) 
   		{
				imagepath = "icons/b1_31.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_LKW) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_UEBERHOLVERBOT_LKW)) 
   		{
				imagepath = "icons/b1_32.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN)) 
   		{
				imagepath = "icons/b1_51.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_LKW) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_UEBERHOLVERBOT_LKW)) 
   		{
				imagepath = "icons/b1_52.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_ENDE_ALLER_STRECKENVERBOTE)) 
   		{
				imagepath = "icons/b1_53.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.BAG_LKW)) 
			{
				imagepath = "icons/b1_222.png";
			} 
   		else 
   		{
				imagepath = "icons/b1_unbekannt.png";
			}
		} 
		if (typ.equals(AnzeigeElementTyp.WZG_C)) 
		{
			if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG)) 
			{
				imagepath = "icons/c1_beliebig.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV)) 
   		{
				imagepath = "icons/c1_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE)) 
   		{
				imagepath = "icons/c1_inaktiv.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL)) 
			{
				imagepath = "icons/c1_leer.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL_STAU) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL_STAU)) 
			{
				imagepath = "icons/c1_dunkelStau.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL_HP)) 
			{
				imagepath = "icons/c1_dunkelHP.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_STAU)) 
   		{
				imagepath = "icons/c1_62.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAUGEFAHR) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_STAUGEFAHR)) 
   		{	
				imagepath = "icons/c1_63.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_NEBEL) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_NEBEL)) 
   		{
				imagepath = "icons/c1_64.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_NAESSE) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_NAESSE)) 
   		{
				imagepath = "icons/c1_65.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_UNFALL) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_UNFALL)) 
   		{
				imagepath = "icons/c1_66.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_SICHT) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_SICHT)) 
   		{
				imagepath = "icons/c1_67.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_SMOG) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_SMOG)) 
   		{
				imagepath = "icons/c1_68.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_500M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_500M)) 
   		{
				imagepath = "icons/c1_81.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1000M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_1000M)) 
   		{
				imagepath = "icons/c1_82.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1500M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_1500M)) 
   		{
				imagepath = "icons/c1_83.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2000M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_2000M)) 
   		{
				imagepath = "icons/c1_84.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2500M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_2500M)) 
   		{
				imagepath = "icons/c1_85.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_3000M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_3000M)) 
   		{
				imagepath = "icons/c1_86.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_500M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_AUF_500M)) 
   		{
				imagepath = "icons/c1_91.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1000M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_AUF_1000M)) 
   		{
				imagepath = "icons/c1_92.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1500M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_AUF_1500M)) 
   		{
				imagepath = "icons/c1_93.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2000M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_AUF_2000M)) 
   		{
				imagepath = "icons/c1_94.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2500M) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_AUF_2500M)) 
   		{
				imagepath = "icons/c1_95.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_7KOMMA5TONNEN)) 
   		{
				imagepath = "icons/c1_102.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GLAETTE) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_GLAETTE)) 
   		{
				imagepath = "icons/c1_203.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_OELSPUR) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_OELSPUR)) 
   		{
				imagepath = "icons/c1_204.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_TIERE) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_TIERE)) 
   		{
				imagepath = "icons/c1_214.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_KONTROLLE) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_KONTROLLE)) 
   		{
				imagepath = "icons/c1_213.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1KM) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_1KM)) 
   		{
				imagepath = "icons/c1_75.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1KOMMA5KM) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_1KOMMA5KM)) 
   		{
				imagepath = "icons/c1_76.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2KM) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_2KM)) 
   		{
				imagepath = "icons/c1_218.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2KOMMA5KM) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_2KOMMA5KM)) 
   		{
				imagepath = "icons/c1_77.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_3KM) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_3KM)) 
   		{
				imagepath = "icons/c1_219.png";
			} 
   		else 
   		{
				imagepath = "icons/c1_unbekannt.png";
			}
		}
		if (typ.equals(AnzeigeElementTyp.WZG_D)) 
		{
			if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG)) 
			{
				imagepath = "icons/d1_beliebig.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV)) 
   		{
				imagepath = "icons/d1_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE)) 
   		{
				imagepath = "icons/d1_inaktiv.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL)) 
			{
				imagepath = "icons/d1_leer.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_HALTEN) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_FAHRSTREIFEN_HALTEN)) 
			{
				imagepath = "icons/d1_112.png";
			} 
   		else 
   		{
				imagepath = "icons/d1_unbekannt.png";
			}
		}
		else if (typ.equals(AnzeigeElementTyp.BLINKER)) 
		{
			if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG)) 
			{
				imagepath = "icons/blinker_beliebig.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV)) 
   		{
				imagepath = "icons/blinker_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE)) 
   		{
				imagepath = "icons/blinker_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL)) 
			{
				imagepath = "icons/blinker_aus.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_GELBES_BLINKLICHT)) 
   		{
				imagepath = "icons/blinker_ein.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_SYNCHRON) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_GELBES_BLINKLICHT_SYNCHRON)) 
   		{
				imagepath = "icons/blinker_synchron.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_ASYNCHRON) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TLS_GELBES_BLINKLICHT_ASYNCHRON)) 
   		{
				imagepath = "icons/blinker_asynchron.png";
			} 
   		else 
   		{
				imagepath = "icons/blinker_unbekannt.png";
			}
		} 
		else if (typ.equals(AnzeigeElementTyp.WZG_BAQ)) 
		{
   		if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV)) 
   		{
				imagepath = "icons/b2_inaktiv.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG)) 
			{
				imagepath = "icons/b2_beliebig.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE)) 
   		{
				imagepath = "icons/b2_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL)) 
			{
				imagepath = "icons/b2_leer.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.BAQ_BA_LKW_STOP) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_BAQ_BA_LKW_STOP)) 
			{
				imagepath = "icons/b2_1.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.BAQ_BA_LKW) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_BAQ_BA_LKW)) 
   		{
				imagepath = "icons/b2_2.png";
			} 
   		else 
   		{
				imagepath = "icons/b2_unbekannt.png";
			}
		} 
		else if (typ.equals(AnzeigeElementTyp.WZG_TAQ)) 
		{
   		if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG)) 
   		{
				imagepath = "icons/t1_beliebig.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV)) 
   		{
				imagepath = "icons/t1_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE)) 
   		{
				imagepath = "icons/t1_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TAQ_NEUTRAL) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TAQ_NEUTRAL)) 
			{
				imagepath = "icons/t1_neutral_s1.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.TAQ_NEUTRAL_S3) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_TAQ_NEUTRAL_S3)) 
			{
				imagepath = "icons/t1_neutral_s3.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TAQ_BEFAHREN) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TAQ_BEFAHREN)) 
   		{
				imagepath = "icons/t1_1.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TAQ_NICHT_BEFAHREN) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TAQ_NICHT_BEFAHREN)) 
   		{
				imagepath = "icons/t1_2.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.TAQ_RAEUMEN) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_TAQ_RAEUMEN)) 
   		{
				imagepath = "icons/t1_3.png";
			} 
   		else 
   		{
				imagepath = "icons/t1_unbekannt.png";
			}
		} 
		else if (typ.equals(AnzeigeElementTyp.WZG_WAQ)) 
		{
   		if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV)) 
   		{
				imagepath = "icons/w1_inaktiv.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG)) 
			{
				imagepath = "icons/w1_beliebig.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE)) 
   		{
				imagepath = "icons/w1_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.WAQ_SEITE_1) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_WAQ_SEITE_1)) 
			{
				imagepath = "icons/w1_1.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.WAQ_SEITE_2) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_WAQ_SEITE_2)) 
			{
				imagepath = "icons/w1_2.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.WAQ_SEITE_3) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_WAQ_SEITE_3)) 
			{
				imagepath = "icons/w1_3.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.WAQ_SEITE_4) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_WAQ_SEITE_4)) 
			{
				imagepath = "icons/w1_4.png";
			} 
   		else 
   		{
				imagepath = "icons/w1_unbekannt.png";
			}
		} 
		else if (typ.equals(AnzeigeElementTyp.WZG_BAG)) 
		{
   		if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV)) 
   		{
				imagepath = "icons/bag_inaktiv.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG)) 
			{
				imagepath = "icons/bag_beliebig.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE)) 
   		{
				imagepath = "icons/bag_inaktiv.png";
			} 
   		else if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) ||
   				     inhalt.equals(AnzeigeInhaltTyp.NICHT_DUNKEL)) 
			{
				imagepath = "icons/bag_leer.png";
			} 
			else if (inhalt.equals(AnzeigeInhaltTyp.BAG_LKW) ||
					     inhalt.equals(AnzeigeInhaltTyp.NICHT_BAG_LKW)) 
			{
				imagepath = "icons/bag_1.png";
			} 
   		else 
   		{
				imagepath = "icons/bag_unbekannt.png";
			}
		} 

		return imagepath;
	}
}