package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallandereszeichen;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeElementFigure;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltAuswahlDialog;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Versorgung der Parameter
 */
public class CompositeParameterAusfallAnderesZeichen extends Composite
{
	private final String _anwenden = "anwenden";

	private final String _nichtAnwenden = "nicht anwenden";
	
	private final int _anzBilder = 3;

	private WVZInhalt _dontCare = VerwaltungWvzInhalte.getInstance().getWvzInhaltDontCare();

	/**
	 * Aktuelles Composite
	 */
  private Composite _composite;

  /**
   * Composite der Tabelle
   */
  private Composite _compositeTabelle;

  /**
   * Gruppe mit den Regeln
   */
  private Group _groupRegeln;

  /**
   * Tabelviewer der Tabelle
   */
  private TableViewer _tvAnzeige;

  /**
   * Aktuelle Parameter
   */
  private ListeMitRegeln _aktuelleParameter;

  /**
   * Composite fr die Buttons
   */
	private Composite _compositeButtons;

	/**
	 * Button Hinzufgen
	 */
	private Button _buttonHinzufuegen;

	/**
	 * Button Lschen
	 */
	private Button _buttonLoeschen;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModel;
	
	/**
	 * Anzeigetyp fr den die Parameter versorgt werden
	 */
	private AnzeigeElementTyp _anzeigeTyp;

	private Button _buttonAlleLoeschen;

//	private Button _buttonDefault;

  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil des Fenstern
   * @param aktuell Aktuelle Parameter
   * @param pidAnzeigeTyp Pid des Anzeigetyps
   */
  CompositeParameterAusfallAnderesZeichen (Composite parent, int style, ListeMitRegeln aktuell, String pidAnzeigeTyp)
  {
    super (parent, style);
    
    this._aktuelleParameter = erzeugeKopie(aktuell);
    
    _anzeigeTyp = AnzeigeElementTyp.getTypByPid(pidAnzeigeTyp);
    
    initialize();
  }

  /**
   * Methode legt eine Arbeitskopie der akutellen Daten an
   * @param aktuell aktuelle Daten
   * @return Arbeitskopie
   */
  private ListeMitRegeln erzeugeKopie(ListeMitRegeln aktuell)
  {
	  ListeMitRegeln kopie = new ListeMitRegeln();
	  
	  Vector<RegelAusfallAnderesZeichen> parameter = aktuell.getParameter();
	  for (RegelAusfallAnderesZeichen regel : parameter)
	  {
	  	List<WVZInhalt> inhaltZ = new ArrayList<WVZInhalt>();
	  	for (WVZInhalt inhalt : regel.getAngeforderterAnzeigenInhaltZusaetzlich())
	  		inhaltZ.add(inhalt);

	  	List<WVZInhalt> inhaltV = new ArrayList<WVZInhalt>();
	  	for (WVZInhalt inhalt : regel.getAngeforderterAnzeigenInhaltVerschiebung())
	  		inhaltV.add(inhalt);

	  	kopie.addRegel(regel.getRichtung(),
	  			           regel.getAnzeigenInhalt(),
	  			           regel.getAngeforderterAnzeigenInhalt(),
	  			           regel.getAuswahlAnzeige(),
	  			           regel.isPruefeAndereZeichen(),
	  			           regel.getAnzeigeTypZusaetzlich(),
	  			           regel.getAuswahlAnzeigeQuerschnitt(),
	  			           regel.getAngeforderterAnzeigenInhaltZusaetzlich(),
	  			           regel.getAnzeigeTypVerschiebung(),
	  			           regel.getAngeforderterAnzeigenInhaltVerschiebung(),
	  			           regel.getEbene(),
	  			           regel.getSchaltgrundRegel());
	  }
	  
	  return kopie;
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
    erzeugeKommentar();
    
    //Initialisierungen der Arrays
    
    List<TableModel[]> inputList = new ArrayList<TableModel[]>();
    
    erzeugeGruppeRegeln();
    
    Table table = _tvAnzeige.getTable();
    _tableModel = createModel1(table);
    
    if ( _tableModel != null )
      _tvAnzeige.setInput(_tableModel); 
    
    createButtons();
    
    updateData(_aktuelleParameter);
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Ausfall anderes Zeichen:\n\n" + 
    "Wenn an einer Anzeige der Anzeigeinhalt = \"X\" ist\n" +
    "und Inhalt \"X\" an dieser Anzeige nicht darstellbar ist\n" +
    "und Inhalt \"X\" an keiner anderen Anzeige dieses AQ dargestellt wird (dieser Regelteil ist abschaltbar)\n" +
    "dann \"Y\" anfordern (entweder an keiner Anzeige, an dieser Anzeige oder an allen Anzeigen des gleichen Typs)\n" +
    "und an allen Anzeigen vom Typ \"A\" (wahlweise an diesem AQ oder in Lngsvorgnger/Lngsnachfoglerrichtung)\n" +
    "das erste Zeichen aus der Reihe \"X\", \"Y\", \"Z\" anzeigen, welches dort darstellbar ist.\n" +
    "Bleibt der zweite Zweig undefiniert oder liefert es kein Ergebnis, so wird nur \"Y\" geschaltet\n\n" +
    "Wird im zweiten Zweig kein Ergebnis ermittelt wird versucht an allen Vorgngern/Nachfolgern vom WZG-Typ \"B\"\n" +
    "in Lngsrichtung bis zur Ebene \"i\" den ersten Inhalt aus der Reihe \"X\", \"Y\", \"Z\" anzufordern, welcher\n" +
    "dort darstellbar ist und eine hhere Prioritt als der dort aktuelle anliegende Anzeigeninhalt hat (Verschiebung).\n\n";
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Methode bernimmt die Daten und zeigt diese in der Tabelle an
   */
  private void updateData (ListeMitRegeln parameter)
  {
  	int anzahlParameter = parameter.getParameter().size();
  	
  	erzeugeTabellenModell (anzahlParameter);
  	
  	for (int j=0; j<anzahlParameter; ++j)
  	{
      TableModel tm = _tableModel [j];    	
    	RegelAusfallAnderesZeichen aktuell = parameter.getParameter().get(j);
    	
    	/*
    	 * Lschen
    	 */
    	tm.setSelectedLoeschen("false");
    	
    	/*
    	 * Schaltgrund
    	 */
    	
    	if (aktuell.getSchaltgrundRegel() == null)
    	{
    	  tm.setSelectedSchaltgrund("");
    	}
    	else
    	{
  	    tm.setSelectedSchaltgrund(aktuell.getSchaltgrundRegel().getName());
    	}
    	
    	/*
    	 * Richtung
    	 */
  	  tm.setSelectedRichtung(aktuell.getRichtung().getName());

  	  /*
  	   * X
  	   */
  	  Label imX = (Label) tm.getX().getChildren()[0];
  	  WVZInhalt x = aktuell.getAnzeigenInhalt();
  	  
  	  Tools.setzeBildInhalte(imX, x, _anzeigeTyp);
  	  
  	  /*
  	   * Regelteil anwenden
  	   */
  	  tm.setSelectedRegelTeilAnwenden(aktuell.isPruefeAndereZeichen() ? _anwenden : _nichtAnwenden);
  	  
  	  /*
  	   * Y
  	   */

  	  Label imY = (Label) tm.getY().getChildren()[0];
  	  WVZInhalt y = aktuell.getAngeforderterAnzeigenInhalt();
  	  
  	  Tools.setzeBildInhalte(imY, y, _anzeigeTyp);

  	  /*
  	   * Auswahl Anzeige
  	   */
  	  tm.setSelectedSelektionAnzeige(aktuell.getAuswahlAnzeige().getBezeichnung());
  	  
  	  /*
  	   * Anzeigetyp A
  	   */
  	  AnzeigeElementTyp aetA = AnzeigeElementTyp.getTypBySystemObject(aktuell.getAnzeigeTypZusaetzlich());
  	  if (aetA != null)
  	    tm.setSelectedWZGTypZusaetzlich(aetA.getName());

  	  /*
  	   * Auswahl Anzeigequerschnitt
  	   */
  	  
  	  tm.setSelectedSelektionAQ(aktuell.getAuswahlAnzeigeQuerschnitt().getBezeichnung());
  	  
  	  /*
  	   * zustzlicher Anzeigeinhalt
  	   */

  	  Control[] childrenZ = tm.getInhaltZusaetzlich().getChildren();
  	  Label [] imZ = new Label [childrenZ.length];
  	  for (int i=0; i<childrenZ.length; ++i)
  	  	imZ [i] = (Label) childrenZ [i];
  	  
  	  List<WVZInhalt> z = aktuell.getAngeforderterAnzeigenInhaltZusaetzlich();
  	  AnzeigeElementTyp anzeigeTypZ = AnzeigeElementTyp.getTypByPid(aktuell.getAnzeigeTypZusaetzlich().getPid());
  	  
  	  for (int i=0; i<z.size(); ++i)
  	  	Tools.setzeBildInhalte(imZ[i], z.get(i), anzeigeTypZ);
  	  
  	  for (int i=z.size(); i<_anzBilder; ++i)
  	  	Tools.setzeBildInhalte(imZ[i], _dontCare, anzeigeTypZ);

  	  /*
  	   * Anzeigetyp B
  	   */
  	  AnzeigeElementTyp aetB = AnzeigeElementTyp.getTypBySystemObject(aktuell.getAnzeigeTypVerschiebung());
  	  if (aetB != null)
  	    tm.setSelectedWZGTypVerschieben(aetB.getName());

  	  /*
  	   * Ebene
  	   */
  	  tm.setSelectedEbene("" + aktuell.getEbene());
  	  
  	  /*
  	   * Anzeigeinhalt verschieben
  	   */

  	  Control[] childrenV = tm.getInhaltVerschiebung().getChildren();
  	  Label [] imV = new Label [childrenV.length];
  	  for (int i=0; i<childrenV.length; ++i)
  	  	imV [i] = (Label) childrenV [i];
  	  
  	  List<WVZInhalt> v = aktuell.getAngeforderterAnzeigenInhaltVerschiebung();
  	  AnzeigeElementTyp anzeigeTypV = AnzeigeElementTyp.getTypByPid(aktuell.getAnzeigeTypVerschiebung().getPid());

  	  for (int i=0; i<v.size(); ++i)
  	  	Tools.setzeBildInhalte(imV[i], v.get(i), anzeigeTypV);
  	  
  	  for (int i=v.size(); i<_anzBilder; ++i)
  	  	Tools.setzeBildInhalte(imV[i], _dontCare, anzeigeTypV);
    }
  	
  	_buttonAlleLoeschen.setEnabled (anzahlParameter != 0);
  	pruefeObLoeschenAngewaehlt();
  }

  /**
   * Methode erzeugt ein Tabellenmodell einer bestimmten Gre. Dabei werden bereits vorhandene
   * Composite in der Tabelle weiter verwendet
   * @param anzahlParameter Gre des erzeugten Tabellenmodells
   */
  private void erzeugeTabellenModell (int anzahlParameter)
  {
  	/*
  	 * Hat sich die Gre gendert
  	 */
  	if (_tableModel.length != anzahlParameter)
  	{
  		/*
  		 * Tabelle erweitern
  		 */
  		if (_tableModel.length < anzahlParameter)
  		{
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  
  		  for (int i=0; i<_tableModel.length; ++i)
  		  	v.add(_tableModel [i]);
  		  
  		  for (int i=_tableModel.length; i<anzahlParameter; ++i)
  		  	v.add (erzeugeZeileInTabelle(_tvAnzeige.getTable()));
  		  
  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		/*
  		 * Tabelle reduzieren
  		 */
  		else if (_tableModel.length > anzahlParameter)
  		{
  			/*
  			 * Nicht mehr bentigte Elemente lschen
  			 */
  			
  			for (int i = _tableModel.length - 1; i > anzahlParameter - 1; i--)
  			{
  				_tableModel [i].getInhaltVerschiebung().dispose();
  				_tableModel [i].getAuswahlVerschieben().dispose();
  				_tableModel [i].getInhaltZusaetzlich().dispose();
  				_tableModel [i].getAuswahlZusaetzlich().dispose();
  				_tableModel [i].getSelektionAnzeige().dispose();
  				_tableModel [i].getY().dispose();
  				_tableModel [i].getRegelteilAnwenden().dispose();
  				_tableModel [i].getX().dispose();
  				
  				_tableModel [i].getRichtung().dispose();
  				_tableModel [i].getSchaltgrund().dispose();
  				_tableModel [i].getLoeschen().dispose();
  			}
  				
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  for (int i=0; i<anzahlParameter; ++i)
  		  	v.add(_tableModel [i]);

  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		
  	  _tvAnzeige.setInput(_tableModel);
  	  _tvAnzeige.getTable().layout(true);
  	}
  }
  
  /**
   * Methode erzeugt die Gruppe mit den Regeln
   */
  private void erzeugeGruppeRegeln()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupRegeln = new Group(_composite, SWT.NONE);
    _groupRegeln.setText("Regeln");
    
    _groupRegeln.setLayout(fillLayout1);
    _groupRegeln.setLayoutData(gridData2);

    erzeugeTabelle();
  }

  /**
   * Methode erzeugt das Composite mit den Buttons
   */
  private void createButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeButtons = new Composite( _composite, SWT.NONE );
    _compositeButtons.setLayoutData( gridData3 );
    _compositeButtons.setLayout( rowLayout );

    _buttonHinzufuegen = new Button( _compositeButtons, SWT.NONE );
    _buttonHinzufuegen.setText( "Regel hinzufgen" );
    _buttonHinzufuegen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelHinzufuegen();
      }
    } );

    _buttonLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonLoeschen.setText( "Selektierte Regeln loeschen" );
    _buttonLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelLoeschen();
      }

    } );
    _buttonLoeschen.setEnabled(false);

    _buttonAlleLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonAlleLoeschen.setText( "Alle Regeln loeschen" );
    _buttonAlleLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        aktionAlleRegelLoeschen();
      }
    } );
    
//    if (_anzeigeTyp == AnzeigeElementTyp.WZG_A)
//    {
//	    _buttonDefault = new Button( _compositeButtons, SWT.NONE );
//	    _buttonDefault.setText( "Defaultwerte" );
//	    _buttonDefault.addMouseListener( new MouseAdapter()
//	    {
//	      @Override
//	      public void mouseDown( MouseEvent e )
//	      {
//	        aktionDefault ();
//	      }
//	    } );
//    }
  }

	/**
   * Methode zum Lschen aller Meldungen
   */
  protected void aktionAlleRegelLoeschen()
  {
		Vector<RegelAusfallAnderesZeichen> regeln = new Vector<RegelAusfallAnderesZeichen>();
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }

  
  
  /**
   * Methode generiert automatisch alle B/C Kombinationen
   */
  protected void aktionDefault()
  {
		Vector<RegelAusfallAnderesZeichen> regeln = new Vector<RegelAusfallAnderesZeichen>();
	  
    _aktuelleParameter.setParameter(regeln);
    updateData(_aktuelleParameter);
  }

  /**
   * Methode prft ob Regeln zum Lschen ausgewhlt sind
   */
  private void pruefeObLoeschenAngewaehlt ()
  {
  	boolean angewaehlt = false;
  	
		for (TableModel tm : _tableModel)
			if (tm.getSelectedLoeschen().equals("true"))
				angewaehlt = true;
		
		_buttonLoeschen.setEnabled(angewaehlt);
  }
  
  /**
   * Methode zum Lschen der Regeln
   */
	private void aktionRegelLoeschen()
  {
		Vector<RegelAusfallAnderesZeichen> regeln = leseAktuelleEinstellungen().getParameter();
		
    Vector<RegelAusfallAnderesZeichen> loeschen = new Vector<RegelAusfallAnderesZeichen>();
    
    for (int i=0; i<_tableModel.length; ++i)
    	if (!_tableModel [i].getSelectedLoeschen().equals("false"))
    		loeschen.add(regeln.get(i));

    for (RegelAusfallAnderesZeichen r : loeschen)
    {
    	regeln.remove(r);
    }
    
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }

	/**
	 * Aktion Regel hinzufgen
	 */
	private void aktionRegelHinzufuegen()
  {
		Vector<RegelAusfallAnderesZeichen> regeln = leseAktuelleEinstellungen().getParameter();
    _aktuelleParameter.setParameter(regeln);

		WVZInhalt inhalt = VerwaltungWvzInhalte.getInstance().getDefaultWvzInhalt(_anzeigeTyp);

		Aspect a = VerwaltungParaAusfallAnderesZeichen.getInstance().getSchaltgrundAllgemein();
		
		Vector<WVZInhalt> z = new Vector<WVZInhalt>();
		for (int i=0; i<_anzBilder; ++i)
			z.add(_dontCare);

		Vector<WVZInhalt> v = new Vector<WVZInhalt>();
		for (int i=0; i<_anzBilder; ++i)
			v.add(_dontCare);
		
		SystemObject wzgTyp = Konfig.getInstance().getWZGTypByPid(_anzeigeTyp.getPid());

		_aktuelleParameter.addRegel (ArbeitsRichtungVN.NACHFOLGER,
				                         inhalt,
				                         inhalt,
				                         SelektionAnzeige.AKTUELLE,
				                         false,
				                         wzgTyp,
				                         SelektionAnzeigeQuerschnitt.KEINE,
				                         z,
				                         wzgTyp,
				                         v,
				                         1,
				                         a);
				                         
		updateData(_aktuelleParameter);
  }

	/**
	 * Methode erzeugt das Composite mit der Tabelle
	 */
  private void erzeugeTabelle ()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    
    _compositeTabelle = new Composite(_groupRegeln, SWT.None);
    _compositeTabelle.setLayout(fillLayout1);
    
    _tvAnzeige = new TableViewer(_compositeTabelle, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());

    TableViewerColumn column0 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column0.getColumn().setWidth(60);
    column0.getColumn().setText("Lschen");
    column0.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column1.getColumn().setWidth(200);
    column1.getColumn().setText("Schaltgrund");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("Richtung");
    column2.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column3 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column3.getColumn().setWidth(50);
    column3.getColumn().setText("X");
    column3.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column4 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column4.getColumn().setWidth(130);
    column4.getColumn().setText("Regelteil anwenden");
    column4.setLabelProvider(new MyCellLabelProvider());
    
    TableViewerColumn column5 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column5.getColumn().setWidth(50);
    column5.getColumn().setText("Y");
    column5.setLabelProvider(new MyCellLabelProvider());
    
    TableViewerColumn column6 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column6.getColumn().setWidth(120);
    column6.getColumn().setText("Auswahl Anzeige");
    column6.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column7 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column7.getColumn().setWidth(150);
    column7.getColumn().setText("Auswahl A");
    column7.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column8 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column8.getColumn().setWidth(Tools.getMaxBreiteBild() * _anzBilder);
    column8.getColumn().setText("Inhalte A");
    column8.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column9 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column9.getColumn().setWidth(150);
    column9.getColumn().setText("Auswahl B");
    column9.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column10 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column10.getColumn().setWidth(Tools.getMaxBreiteBild() * _anzBilder);
    column10.getColumn().setText("Inhalte B");
    column10.setLabelProvider(new MyCellLabelProvider());

    // resize the row height using a MeasureItem listener
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
          	event.height = Tools.getZeilenHoehe(_anzeigeTyp, Tools.getZeilenHoehe(AnzeigeElementTyp.WZG_BAQ));
            break;

          default:
        }

      }
    };
    
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }

  /**
   * Klasse realsiert den TableContentProvider fr die Tabelle
   * @author ObertM
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
  	/**
  	 * Tabelviewer
  	 */
    private TableViewer tableViewer;

    /**
     * Methode liefert die Elemente der Tabelle
     */
    public Object[] getElements(Object inputElement)
    {
			Object[] re = null;

			if (inputElement instanceof TableModel [])
			{
        re = (TableModel[]) inputElement;
			}
			
			return re;
    }

    /**
     * Dispose
     */
    public void dispose()
    {
    }

    /**
     * Methode wird aufgerufen wenn sich der Inhalt der Tabelle gendert hat
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.deleteObserver(this);
        }
      }
      
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.addObserver(this);
        }
      }
    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();
      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel1 (Table table)
  {
    int tabLength = getAnzahlParameter();
    int elIndex   = 0;
    
    TableModel[] elements = new TableModel [tabLength];
    
    if (elements.length == 0)
    	return elements;

    for (int x=0; x<getAnzahlParameter(); ++x)
    {
	    elements[elIndex] = erzeugeZeileInTabelle(table);
	    elIndex++;
    }
    
    return elements;
  }

  /**
   * Methode erzeugt ein Tabellenmodell fr eine Zeile in der Tabelle
   * @param table Tabelle 
   * @return Tabellenmodell fr eine Zeile
   */
  private TableModel erzeugeZeileInTabelle (Table table)
  {
    /*
     * Layout festlegen
     */
  	
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.grabExcessVerticalSpace = true;

    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.verticalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;

    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.grabExcessVerticalSpace = true;

    GridData gd3 = new GridData();
    gd3.horizontalAlignment = GridData.CENTER;
    gd3.verticalAlignment = GridData.CENTER;
    gd3.grabExcessHorizontalSpace = true;
    gd3.grabExcessVerticalSpace = true;

    GridData gd4 = new GridData();
    gd4.horizontalAlignment = GridData.CENTER;
    gd4.verticalAlignment = GridData.CENTER;
    gd4.grabExcessHorizontalSpace = true;
    gd4.grabExcessVerticalSpace = true;

    GridData gd5 = new GridData();
    gd5.horizontalAlignment = GridData.CENTER;
    gd5.verticalAlignment = GridData.CENTER;
    gd5.grabExcessHorizontalSpace = true;
    gd5.grabExcessVerticalSpace = true;

    GridData gd6 = new GridData();
    gd6.horizontalAlignment = GridData.CENTER;
    gd6.verticalAlignment = GridData.CENTER;
    gd6.grabExcessHorizontalSpace = true;
    gd6.grabExcessVerticalSpace = true;

    GridData gd7 = new GridData();
    gd7.horizontalAlignment = GridData.CENTER;
    gd7.verticalAlignment = GridData.CENTER;
    gd7.grabExcessHorizontalSpace = true;
    gd7.grabExcessVerticalSpace = true;

    GridData gd8 = new GridData();
    gd8.horizontalAlignment = GridData.CENTER;
    gd8.verticalAlignment = GridData.CENTER;
    gd8.grabExcessHorizontalSpace = true;
    gd8.grabExcessVerticalSpace = true;

    GridData gd9 = new GridData();
    gd9.horizontalAlignment = GridData.CENTER;
    gd9.verticalAlignment = GridData.CENTER;
    gd9.grabExcessHorizontalSpace = true;
    gd9.grabExcessVerticalSpace = true;

    GridData gd10 = new GridData();
    gd10.horizontalAlignment = GridData.CENTER;
    gd10.verticalAlignment = GridData.CENTER;
    gd10.grabExcessHorizontalSpace = true;
    gd10.grabExcessVerticalSpace = true;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    /*
     * Button Lschen
     */
    Composite comp0 = new Composite( table, SWT.NONE );
    comp0.setBackground( table.getBackground() );
    comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp0.setLayoutData( gd0 );
    comp0.setLayout( new GridLayout() );
    Button b = new Button( comp0, SWT.CHECK );
    
    b.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				pruefeObLoeschenAngewaehlt();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
    
    /*
     * Combobox Schaltgrund    
     */
    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout() );
    
    Combo combo1 = new Combo( comp1, SWT.BORDER );
    combo1.setLayoutData( gdTableItem );
    
    Vector<SystemObject> aspekte = Tools.getAspekte();
    String [] sa = new String [aspekte.size()];
    for (int i=0; i<aspekte.size(); ++i)
      sa [i] = aspekte.get(i).getName();
    
    combo1.setItems(sa);
    combo1.select(0);

    /*
     * Combobox Richtung    
     */
    Composite comp2 = new Composite( table, SWT.NONE );
    comp2.setBackground( table.getBackground() );
    comp2.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp2.setLayoutData( gd2 );
    comp2.setLayout( new GridLayout() );
    
    Combo combo2 = new Combo( comp2, SWT.BORDER );
    combo2.setLayoutData( gdTableItem );
    
    ArbeitsRichtungVN[] arvn = Richtung.ArbeitsRichtungVN.values();
    String[] r = new String [arvn.length];
    
    for (int i=0; i<arvn.length; ++i)
    	r [i] = arvn [i].getName();
    
    combo2.setItems(r);
    combo2.select(0);

    /*
     * X
     */
    Composite comp3 = new Composite( table, SWT.NONE );
    comp3.setBackground( table.getBackground() );
    comp3.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp3.setLayoutData( gd3 );
    comp3.setLayout( new FillLayout() );
    
    Label lbImageX = new Label( comp3, SWT.CENTER );
    lbImageX.addMouseListener( new MouseAdapter()
    {
			@Override
      public void mouseDown( MouseEvent e )
      {
      	Tools.waehleZuPruefendesBild(e, _anzeigeTyp);
      }         
    });

    /*
     * Combobox Regelteil anwenden    
     */
    Composite comp4 = new Composite( table, SWT.NONE );
    comp4.setBackground( table.getBackground() );
    comp4.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp4.setLayoutData( gd4 );
    comp4.setLayout( new GridLayout() );
    
    Combo combo4 = new Combo( comp4, SWT.BORDER );
    combo4.setLayoutData( gdTableItem );
    
    String [] sa4 = new String [2];
    sa4 [0] = _anwenden;
    sa4 [1] = _nichtAnwenden;
    
    combo4.setItems(sa4);
    combo4.select(0);

    /*
     * Y
     */
    Composite comp5 = new Composite( table, SWT.NONE );
    comp5.setBackground( table.getBackground() );
    comp5.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp5.setLayoutData( gd5 );
    comp5.setLayout( new FillLayout() );
    
    Label lbImageY = new Label( comp5, SWT.CENTER );
    lbImageY.addMouseListener( new MouseAdapter()
    {
			@Override
      public void mouseDown( MouseEvent e )
      {
      	Tools.waehleBildInhaltAus(e, false, false, false, _anzeigeTyp);
      }         
    });

    /*
     * Combobox Auswahl Anzeige    
     */
    Composite comp6 = new Composite( table, SWT.NONE );
    comp6.setBackground( table.getBackground() );
    comp6.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp6.setLayoutData( gd6 );
    comp6.setLayout( new GridLayout() );
    
    Combo combo6 = new Combo( comp6, SWT.BORDER );
    combo6.setLayoutData( gdTableItem );
    
    String [] sa6 = new String [3];
    sa6 [0] = SelektionAnzeige.KEINE.getBezeichnung();
    sa6 [1] = SelektionAnzeige.AKTUELLE.getBezeichnung();
    sa6 [2] = SelektionAnzeige.ALLE.getBezeichnung();
    
    combo6.setItems(sa6);
    combo6.select(0);

    /*
     * Auswahl A ...
     */

    Composite comp7 = new Composite( table, SWT.NONE );
    comp7.setBackground( table.getBackground() );
    comp7.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp7.setLayoutData( gd7 );
    comp7.setLayout( new GridLayout() );

    /*
     * ... Combobox Auswahl Anzeigetyp    
     */
    final Combo combo7_1 = new Combo( comp7, SWT.BORDER );
    combo7_1.setLayoutData( gdTableItem );

    List<AnzeigeElementTyp> typen = AnzeigeElementTyp.getVersorgteTypen();
    String [] sa7_1 = new String [typen.size()];
    for (int i=0; i<typen.size(); ++i)
    	sa7_1 [i] = typen.get(i).getName();
    
    combo7_1.setItems(sa7_1);
    combo7_1.select(0);
    
    /*
     * ... Combobox Auswahl Querschnitte    
     */
    Combo combo7_2 = new Combo( comp7, SWT.BORDER );
    combo7_2.setLayoutData( gdTableItem );
    
    String [] sa7_2 = new String [3];
    sa7_2 [0] = SelektionAnzeigeQuerschnitt.KEINE.getBezeichnung();
    sa7_2 [1] = SelektionAnzeigeQuerschnitt.AKTUELL.getBezeichnung();
    sa7_2 [2] = SelektionAnzeigeQuerschnitt.LAENGSRICHTUNG.getBezeichnung();
    
    combo7_2.setItems(sa7_2);
    combo7_2.select(0);
    
    /*
     * Inhalte A
     */
    final Composite comp8 = new Composite( table, SWT.NONE );
    comp8.setBackground( table.getBackground() );
    comp8.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp8.setLayoutData( gd8 );
    comp8.setLayout( new FillLayout() );
    
    for (int i=0; i<_anzBilder; ++i)
    {
	    Label lbImageA = new Label( comp8, SWT.CENTER );
	    lbImageA.addMouseListener( new MouseAdapter()
	    {
				@Override
	      public void mouseDown( MouseEvent e )
	      {
	        AnzeigeElementTyp wzgTyp = bestimmeWZGTyp(combo7_1);
	      	Tools.waehleBildInhaltAus (e, true, false, false, wzgTyp);
	      }         
	    });
    }

    /*
     * Auf die Combox Anzeigetyp wird ein Listener gehngt (aber erst nach Anlagen des Composite mit den
     * WVZ Inhalten) der bewirkt, dass bei einer nderung des WZG Typs die WVZ Inhalte auf don't care
     * gesetzt werden.
     */
    combo7_1.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (comp8 != null)
				{
          AnzeigeElementTyp wzgTyp = bestimmeWZGTyp(combo7_1);
        
      	  Control[] children = comp8.getChildren();
      	  for (int i=0; i<children.length; ++i)
      	  {
      	  	Label l = (Label) children [i];
      	  	Tools.setzeBildInhalte(l, _dontCare, wzgTyp);
	        }
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});

    /*
     * Auswahl B ...
     */

    Composite comp9 = new Composite( table, SWT.NONE );
    comp9.setBackground( table.getBackground() );
    comp9.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp9.setLayoutData( gd9 );
    comp9.setLayout( new GridLayout() );

    /*
     * ... Combobox Auswahl Anzeigetyp    
     */
    final Combo combo9 = new Combo( comp9, SWT.BORDER );
    combo9.setLayoutData( gdTableItem );
    
    String [] sa9 = new String [typen.size()];
    for (int i=0; i<typen.size(); ++i)
    	sa9 [i] = typen.get(i).getName();

    combo9.setItems(sa9);
    combo9.select(0);

    /*
     * ... Spinner Ebene
     */
    Spinner spinner = new Spinner( comp9, SWT.BORDER );

    spinner.setMinimum(0);
    spinner.setMaximum(100);
    spinner.setDigits(0);
      
    String toolTip = "Ebene in Lngsrichtung. Zulssig von  " + spinner.getMinimum() 
                     + " bis " + spinner.getMaximum();
     
    spinner.setToolTipText(toolTip);
    spinner.setLayoutData( gdTableItem );

    /*
     * Inhalte B
     */
    final Composite comp10 = new Composite( table, SWT.NONE );
    comp10.setBackground( table.getBackground() );
    comp10.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp10.setLayoutData( gd10 );
    comp10.setLayout( new FillLayout() );
    
    for (int i=0; i<_anzBilder; ++i)
    {
	    Label lbImageB = new Label( comp10, SWT.CENTER );
	    lbImageB.addMouseListener( new MouseAdapter()
	    {
				@Override
	      public void mouseDown( MouseEvent e )
	      {
	        AnzeigeElementTyp wzgTyp = bestimmeWZGTyp(combo9);
	      	Tools.waehleBildInhaltAus(e, true, false, false, wzgTyp);
	      }         
	    });
    }
    
    /*
     * Auf die Combox Anzeigetyp wird ein Listener gehngt (aber erst nach Anlagen des Composite mit den
     * WVZ Inhalten) der bewirkt, dass bei einer nderung des WZG Typs die WVZ Inhalte auf don't care
     * gesetzt werden.
     */
    combo9.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (comp10 != null)
				{
          AnzeigeElementTyp wzgTyp = bestimmeWZGTyp(combo9);
        
      	  Control[] children = comp10.getChildren();
      	  for (int i=0; i<children.length; ++i)
      	  {
      	  	Label l = (Label) children [i];
      	  	Tools.setzeBildInhalte(l, _dontCare, wzgTyp);
	        }
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});

    return new TableModel( comp0, comp1, comp2, comp3, comp4, comp5, comp6, comp7, comp8, comp9, comp10 );
  }
  
  /**
   * Methode bestimmt den WZG Typ auf Basis des angewhlten WZG Typs in einer Combo Box
   * @param combo Combo Box
   * @return WZG Typ
   */
  private AnzeigeElementTyp bestimmeWZGTyp (Combo combo)
  {
    String selektiert = _anzeigeTyp.getPid(); 
    	
    try
    {
    	selektiert = combo.getItem( combo.getSelectionIndex() );
    }
    catch (Exception e1) 
    {
		}

    AnzeigeElementTyp wzgTyp = AnzeigeElementTyp.getTypByName(selektiert);
    
    return wzgTyp;
  }

  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public int getAnzahlParameter()
  {
    return _aktuelleParameter.getParameter().size();
  }
  
  /**
   * Methode liefert den Tabelviewer
   * @return
   */
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }

  /**
   * Renderer fr die Tabelle
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider 
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

//      if (item.getData("EDITOR") != null)
//      {
//        TableEditor editor = (TableEditor)item.getData("EDITOR");
//        editor.dispose();
//      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getLoeschen();
          break;
          
        case 1:
          comp = model.getSchaltgrund();
          break;
          
        case 2:
          comp = model.getRichtung();
          break;
          
        case 3:
          comp = model.getX();
          break;
          
        case 4:
          comp = model.getRegelteilAnwenden();
          break;

        case 5:
          comp = model.getY();
          break;

        case 6:
          comp = model.getSelektionAnzeige();
          break;

        case 7:
          comp = model.getAuswahlZusaetzlich();
          break;

        case 8:
          comp = model.getInhaltZusaetzlich();
          break;

        case 9:
          comp = model.getAuswahlVerschieben();
          break;

        case 10:
          comp = model.getInhaltVerschiebung();
          break;

        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }

  /**
   * Methode liefert die eingestellten Parameter
   * @return eingestellte Parameter
   */
	public ListeMitRegeln getParameter()
  {
 		aktionRegelLoeschen();
 		return leseAktuelleEinstellungen();
  }
		
	private ListeMitRegeln leseAktuelleEinstellungen ()
	{
		ListeMitRegeln regeln = new ListeMitRegeln();
		
		for (int j=0; j<_tableModel.length; ++j)
		{
      TableModel tm = _tableModel [j];
      
      RegelAusfallAnderesZeichen regel = bildeRegelAusTabellenModell(tm);
      regeln.addRegel(regel);
		}

		return regeln;
  }

	/**
	 * Methode bernimmt die Daten eines anderen Folders
	 * @param tvAnzeige Tableviewer der zu bernhmenden Tabelle
	 */
	public void setTvAnzeige (TableViewer tvAnzeige)
	{
    TableModel[] modelSet = (TableModel[]) tvAnzeige.getInput();

    Vector<RegelAusfallAnderesZeichen> v = new Vector<RegelAusfallAnderesZeichen>();

    for (int i=0; i<modelSet.length; ++i)
    {
    	RegelAusfallAnderesZeichen regel = bildeRegelAusTabellenModell(modelSet [i]);
  	  v.add (regel);
    }
    
    _aktuelleParameter.setParameter(v);
    updateData(_aktuelleParameter);
  }
	
	/**
	 * Methode bildet aus dem Tabellenmodell eine Regel
	 * @param tm Tabellenmodell
	 * @return Regel
	 */
	private RegelAusfallAnderesZeichen bildeRegelAusTabellenModell (TableModel tm)
	{
    /*
     * Schaltgrund
     */
    
    String nameSchaltgrund = tm.getSelectedSchaltgrund();
    Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);
    
    /*
     * Richtung 
     */
    String nameRichtung = tm.getSelectedRichtung();
    ArbeitsRichtungVN richtung = ArbeitsRichtungVN.getRichtung (nameRichtung);

    /*
     * X
     */
    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
    WVZInhalt x = verw.getWvzInhalt (_anzeigeTyp, tm.getSelectedX());

    /*
     * Anwenden
     */
    boolean anwenden = tm.getSelectedRegelTeilAnwenden().equals(_anwenden);

    /*
     * Y
     */
    WVZInhalt y = verw.getWvzInhalt (_anzeigeTyp, tm.getSelectedY());
    
    /*
     * Selektion Anzeige
     */
    String bezeichnung = tm.getSelectedSelektionAnzeige();
    SelektionAnzeige selektionAnzeige = SelektionAnzeige.getSelektion (bezeichnung);
    
    /*
     * Auswahl A - WZG Typ
     */
    bezeichnung = tm.getSelectedWZGTypZusaetzlich();
    AnzeigeElementTyp typA = AnzeigeElementTyp.getTypByName(bezeichnung);
    
    SystemObject wzgTypA = Konfig.getInstance().getWZGTypByPid (typA.getPid());
    
    /*
     * Auswahl A - Selektion Anzeigequerschnitt
     */
    bezeichnung = tm.getSelectedSelektionAQ();
    SelektionAnzeigeQuerschnitt selektionAnzeigeQuerschnitt = SelektionAnzeigeQuerschnitt.getSelektion (bezeichnung);
    
    /*
     * Auswahl A - WVZ Inhalte
     */
    
    List<String> tmp = new ArrayList<String>();
    
    for (String s : tm.getSelectedInhaltZusaetlich())
    {
    	if (!s.equals(AnzeigeInhaltTyp.DONT_CARE.getBildInhalt()))
    		if (!tmp.contains(s))
    			tmp.add(s);
    }

    Vector<WVZInhalt> inhalteA = new Vector<WVZInhalt>();
    for (String s : tmp)
    	inhalteA.add (verw.getWvzInhalt (typA, s));

    /*
     * Auswahl B - WZG Typ
     */
    bezeichnung = tm.getSelectedWZGTypVerschieben();
    AnzeigeElementTyp typB = AnzeigeElementTyp.getTypByName(bezeichnung);
    
    SystemObject wzgTypB = Konfig.getInstance().getWZGTypByPid(typB.getPid());

    /*
     * Auswahl B - Ebene
     */
    int ebene = tm.getSelectetEbene();

    /*
     * Auswahl B - WVZ Inhalte
     */
    tmp.clear();
    
    for (String s : tm.getSelectedInhaltVerschieben())
    {
    	if (!s.equals(AnzeigeInhaltTyp.DONT_CARE.getBildInhalt()))
    		if (!tmp.contains(s))
    			tmp.add(s);
    }

    Vector<WVZInhalt> inhalteB = new Vector<WVZInhalt>();
    for (String s : tmp)
    	inhalteB.add (verw.getWvzInhalt (typB, s));

    return new RegelAusfallAnderesZeichen(
                    richtung,
    		            x,
    		            y,
    		            selektionAnzeige,
    		            anwenden,
    		            wzgTypA,
    		            selektionAnzeigeQuerschnitt,
    		            inhalteA,
    		            wzgTypB,
    		            inhalteB,
    		            ebene,
    		            schaltgrund);
	}
}
