package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallandereszeichen;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Ausfallbehandlung", 
 * Regel "Ausfall Gleiches Zeichen".<br> 
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 */
public class RegelAusfallAnderesZeichen
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;
  
  /**
   * arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   */
  private ArbeitsRichtungVN m_richtung = ArbeitsRichtungVN.VORGAENGER;

  /**
   * Prfen ob Inhalt an keiner anderen Anzeige dieses AQ dargestellt wird
   */
  private boolean m_pruefeAndereZeichen;
  
  /**
   * An welchen Anzeigen soll Inhalt angezeigt werden
   */
  private SelektionAnzeige m_auswahlAnzeige;

  /**
   * Zustzlich zu schaltender Anzeigetyp
   */
  private SystemObject m_anzeigeTypZusaetzlich;
  
  /**
   * An welchem AQ soll der zustzliche Inhalt angezeigt werden
   */
  private SelektionAnzeigeQuerschnitt m_auswahlAnzeigeQuerschnitt;
  
  /**
   * Zustzlich zu schaltender Anzeigetyp in Lngsrichtung
   */
  private SystemObject m_anzeigeTypVerschiebung;
  
  /**
   * Ebene in der Baumstruktur der Vorgnger/Nachfolgerbeziehung bis zu der verschoben werden soll.
   */
  private int m_ebene;
  
  /**
   * Anzeigeninhalt (in der Regel: "X")
   */
  private WVZInhalt  m_anzeigenInhalt = new WVZInhalt();
  
  /**
   * angeforderter Anzeigeninhalt (in der Regel: "aus")
   */
  private WVZInhalt m_angeforderterAnzeigenInhalt = new WVZInhalt();

  /**
   * Reihe mit Zeichen von denen das erste darstellbare Zeichen angezeigt werden soll 
   * (in der Regel "X,Y,Z" im Zweig zustzlich)
   */
  private Vector<WVZInhalt> m_angeforderterAnzeigenInhaltZusaetzlich = new Vector<WVZInhalt>();

  /**
   * Reihe mit Zeichen von denen das erste darstellbare Zeichen angezeigt werden soll 
   * (in der Regel "X,Y,Z" im Zweig Verschiebung)
   */
  private Vector<WVZInhalt> m_angeforderterAnzeigenInhaltVerschiebung = new Vector<WVZInhalt>();
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelAusfallAnderesZeichen ()
  {
  }
  
  /**
   * Konstruktor der Klasse
   * @param richtung Richtung in die die Regel arbeitet
   * @param anzeigeInhalt zu prfender Anzeigeinhalt ("X")
   * @param angeforderterAnzeigeInhalt angeforderter Anzeigeinhalt ("aus")
   * @param auswahlAnzeige Welche Anzeige soll geschaltet werden
   * @param pruefeAndereZeichen Sollen die anderen Zeichen geprft werden
   * @param anzeigeTypZusaetzlich zustzlicher Anzeigetyp ("Typ A")
   * @param auswahlAnzeigeQuerschnitt Welche Anzeigequerschnitte sollen geschaltet werden
   * @param angeforderterAnzeigenInhaltZusaetzlich angeforderter Anzeigeninhalte ("X", "Y", "Z");
   * @param anzeigeTypVerschiebung Anzeigetyp Verschiebung ("Typ B")
   * @param angeforderterAnzeigenInhaltVerschiebung angeforderter Anzeigeninhalte ("X", "Y", "Z");
   * @param ebene Ebene bis zu der verschoben wird
   * @param schaltgrund Schaltgrund unter dem die Regel arbeitet
   */
  public RegelAusfallAnderesZeichen(ArbeitsRichtungVN richtung,
                        WVZInhalt anzeigeInhalt, 
                        WVZInhalt angeforderterAnzeigeInhalt,
                        SelektionAnzeige auswahlAnzeige,
                        boolean pruefeAndereZeichen,
                        SystemObject anzeigeTypZusaetzlich,
                        SelektionAnzeigeQuerschnitt auswahlAnzeigeQuerschnitt,
                        Vector<WVZInhalt> angeforderterAnzeigenInhaltZusaetzlich,
                        SystemObject anzeigeTypVerschiebung,
                        Vector<WVZInhalt> angeforderterAnzeigenInhaltVerschiebung,
                        int ebene,
                        Aspect schaltgrund )
  {
	  m_richtung = richtung;
	  m_anzeigenInhalt = anzeigeInhalt;
	  m_angeforderterAnzeigenInhalt = angeforderterAnzeigeInhalt;
	  m_auswahlAnzeige = auswahlAnzeige;
	  m_pruefeAndereZeichen = pruefeAndereZeichen;
	  m_anzeigeTypZusaetzlich = anzeigeTypZusaetzlich;
	  m_auswahlAnzeigeQuerschnitt = auswahlAnzeigeQuerschnitt;
	  m_angeforderterAnzeigenInhaltZusaetzlich = angeforderterAnzeigenInhaltZusaetzlich;
	  m_anzeigeTypVerschiebung = anzeigeTypVerschiebung;
	  m_angeforderterAnzeigenInhaltVerschiebung = angeforderterAnzeigenInhaltVerschiebung;
	  m_ebene = ebene;
	  m_schaltgrundRegel = schaltgrund;
  }

	/**
   * Methode liefert die Variable m_anzeigenInhalt zurck
   * @return Variable m_anzeigenInhalt
   */
  public WVZInhalt getAnzeigenInhalt()
  {
    return m_anzeigenInhalt;
  }

  /**
   * Methode setzt die Variable m_anzeigenInhalt
   * @param anzeigenInhalt Wert auf den die Variable m_anzeigenInhalt gesetzt werden soll
   */
  public void setAnzeigenInhalt( WVZInhalt anzeigenInhalt )
  {
    m_anzeigenInhalt = anzeigenInhalt;
  }

  /**
   * Methode liefert die Variable m_angeforderterAnzeigenInhalt zurck
   * @return Variable m_angeforderterAnzeigenInhalt
   */
  public WVZInhalt getAngeforderterAnzeigenInhalt()
  {
    return m_angeforderterAnzeigenInhalt;
  }

  /**
   * Methode setzt die Variable m_angeforderterAnzeigenInhalt
   * @param angeforderterAnzeigenInhalt Wert auf den die Variable m_angeforderterAnzeigenInhalt gesetzt werden soll
   */
  public void setAngeforderterAnzeigenInhalt( WVZInhalt angeforderterAnzeigenInhalt )
  {
    m_angeforderterAnzeigenInhalt = angeforderterAnzeigenInhalt;
  }
  
  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atg.sswSbaAusfallAnderesZeichen'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    // Schaltgrundregel
    
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    // VorgngerNachfolger
    
    String vorgaengerNachfolger = data.getItem( "VorgngerNachfolger" ).asTextValue().getText();
    
    if (vorgaengerNachfolger.equals( "Vorgnger" ))
      setRichtung (ArbeitsRichtungVN.VORGAENGER);
    else
      setRichtung (ArbeitsRichtungVN.NACHFOLGER);

    // PrfeAndereZeichen
    
    String jaNein = data.getItem( "PruefeAndereAnzeigen" ).asTextValue().getText();
    m_pruefeAndereZeichen = jaNein.equalsIgnoreCase( "Ja" );
    
    // AuswahlAnzeige
    
    int kodierung = data.getUnscaledValue( "AuswahlAnzeige" ).intValue();
    m_auswahlAnzeige = SelektionAnzeige.getValue( kodierung );
    
    // AnzeigeTypZustzlich
    
    m_anzeigeTypZusaetzlich = data.getReferenceValue( "AnzeigeTypZustzlich" ).getSystemObject();
    
    // AuswahlAnzeigeQuerschnitt
    
    kodierung = data.getUnscaledValue( "AuswahlAnzeigeQuerschnitt" ).intValue();
    m_auswahlAnzeigeQuerschnitt = SelektionAnzeigeQuerschnitt.getValue( kodierung );
    
    // AnzeigeTypVerschiebung

    m_anzeigeTypVerschiebung = data.getReferenceValue( "AnzeigeTypVerschiebung" ).getSystemObject();

    // Ebene
    
    m_ebene = (int) data.getUnscaledValue( "Ebene" ).longValue();
    
    // Anzeigeninhalt
    
    Data anzeigenInhalt = data.getItem( "Anzeigeinhalt" );
    m_anzeigenInhalt.auswerteAttributgruppe( anzeigenInhalt );
    
    // AngeforderterAnzeigenInhalt
    
    Data angeforderterAnzeigenInhalt = data.getItem( "AngeforderterAnzeigeInhalt" );
    m_angeforderterAnzeigenInhalt.auswerteAttributgruppe( angeforderterAnzeigenInhalt );

    // AngeforderterAnzeigenInhaltZustzlich
    
    Array a = data.getArray( "AngeforderterAnzeigeInhaltZustzlich" );
    int anzahl = a.getLength();
    m_angeforderterAnzeigenInhaltZusaetzlich.clear();
    
    for (int x = 0; x < anzahl; ++x)
    {
      WVZInhalt inhalt = new WVZInhalt();
      inhalt.auswerteAttributgruppe( a.getItem( x ) );
      
      m_angeforderterAnzeigenInhaltZusaetzlich.add( inhalt );
    }
    
    // AngeforderterAnzeigenInhaltVerschiebung

    a = data.getArray( "AngeforderterAnzeigeInhaltVerschiebung" );
    anzahl = a.getLength();
    m_angeforderterAnzeigenInhaltVerschiebung.clear();
    
    for (int x = 0; x < anzahl; ++x)
    {
      WVZInhalt inhalt = new WVZInhalt();
      inhalt.auswerteAttributgruppe( a.getItem( x ) );
      
      m_angeforderterAnzeigenInhaltVerschiebung.add( inhalt );
    }
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel ) 
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "Schaltgrundregel:          " + Tools.getObjektName( getSchaltgrundRegel() ) + "\n";
    s += "Richtung:                  " + getRichtung() + "\n";
    s += "PrfeAndereZeichen :       " + isPruefeAndereZeichen() + "\n";
    s += "AuswahlAnzeige:            " + getAuswahlAnzeige() + "\n";
    s += "AnzeigeTypZustzlich:      " + Tools.getObjektName( getAnzeigeTypZusaetzlich() ) + "\n";
    s += "AuswahlAnzeigeQuerschnitt: " + getAuswahlAnzeigeQuerschnitt() + "\n";
    s += "AnzeigeTypVerschiebung:    " + Tools.getObjektName( getAnzeigeTypVerschiebung() ) + "\n";
    s += "Ebene:                     " + getEbene() + "\n";
    s += "X = " + Tools.getWVZInhalt( m_anzeigenInhalt ) + "\n";
    s += "Y = " + Tools.getWVZInhalt( m_angeforderterAnzeigenInhalt ) + "\n";

    s += "AngeforderterAnzeigeInhaltZustzlich:" + "\n";
    for (WVZInhalt w : getAngeforderterAnzeigenInhaltZusaetzlich())
      s += Tools.getWVZInhalt( w ) + "\n";

    s += "AngeforderterAnzeigeInhaltVerschiebung:" + "\n";
    for (WVZInhalt w : getAngeforderterAnzeigenInhaltVerschiebung())
      s += Tools.getWVZInhalt( w ) + "\n";

    return s;
  }

  /**
   * Methode liefert die Variable m_pruefeAndereZeichen zurck
   * @return Variable m_pruefeAndereZeichen
   */
  public boolean isPruefeAndereZeichen()
  {
    return m_pruefeAndereZeichen;
  }

  /**
   * Methode setzt die Variable m_pruefeAndereZeichen
   * @param pruefeAndereZeichen Wert auf den die Variable m_pruefeAndereZeichen gesetzt werden soll
   */
  public void setPruefeAndereZeichen( boolean pruefeAndereZeichen )
  {
    m_pruefeAndereZeichen = pruefeAndereZeichen;
  }

  /**
   * Methode liefert die Variable m_richtung zurck
   * @return Variable m_richtung
   */
  public ArbeitsRichtungVN getRichtung()
  {
    return m_richtung;
  }

  /**
   * Methode setzt die Variable m_richtung
   * @param richtung Wert auf den die Variable m_richtung gesetzt werden soll
   */
  public void setRichtung( ArbeitsRichtungVN richtung )
  {
    m_richtung = richtung;
  }

  /**
   * Methode liefert die Variable m_auswahlAnzeige zurck
   * @return Variable m_auswahlAnzeige
   */
  public SelektionAnzeige getAuswahlAnzeige()
  {
    return m_auswahlAnzeige;
  }

  /**
   * Methode setzt die Variable m_auswahlAnzeige
   * @param auswahlAnzeige Wert auf den die Variable m_auswahlAnzeige gesetzt werden soll
   */
  public void setAuswahlAnzeige( SelektionAnzeige auswahlAnzeige )
  {
    m_auswahlAnzeige = auswahlAnzeige;
  }

  /**
   * Methode liefert die Variable m_anzeigeTypZusaetzlich zurck
   * @return Variable m_anzeigeTypZusaetzlich
   */
  public SystemObject getAnzeigeTypZusaetzlich()
  {
    return m_anzeigeTypZusaetzlich;
  }

  /**
   * Methode setzt die Variable m_anzeigeTypZusaetzlich
   * @param anzeigeTypZusaetzlich Wert auf den die Variable m_anzeigeTypZusaetzlich gesetzt werden soll
   */
  public void setAnzeigeTypZusaetzlich( SystemObject anzeigeTypZusaetzlich )
  {
    m_anzeigeTypZusaetzlich = anzeigeTypZusaetzlich;
  }

  /**
   * Methode liefert die Variable m_anzeigeTypVerschiebung zurck
   * @return Variable m_anzeigeTypVerschiebung
   */
  public SystemObject getAnzeigeTypVerschiebung()
  {
    return m_anzeigeTypVerschiebung;
  }

  /**
   * Methode setzt die Variable m_anzeigeTypVerschiebung
   * @param anzeigeTypVerschiebung Wert auf den die Variable m_anzeigeTypVerschiebung gesetzt werden soll
   */
  public void setAnzeigeTypVerschiebung( SystemObject anzeigeTypVerschiebung )
  {
    m_anzeigeTypVerschiebung = anzeigeTypVerschiebung;
  }

  /**
   * Methode liefert die Variable m_ebene zurck
   * @return Variable m_ebene
   */
  public int getEbene()
  {
    return m_ebene;
  }

  /**
   * Methode setzt die Variable m_ebene
   * @param ebene Wert auf den die Variable m_ebene gesetzt werden soll
   */
  public void setEbene( int ebene )
  {
    m_ebene = ebene;
  }

  /**
   * Methode liefert die Variable m_angeforderterAnzeigenInhaltZusaetzlich zurck
   * @return Variable m_angeforderterAnzeigenInhaltZusaetzlich
   */
  public Vector<WVZInhalt> getAngeforderterAnzeigenInhaltZusaetzlich()
  {
    return m_angeforderterAnzeigenInhaltZusaetzlich;
  }

  /**
   * Methode setzt die Variable m_angeforderterAnzeigenInhaltZusaetzlich
   * @param angeforderterAnzeigenInhaltZusaetzlich Wert auf den die Variable m_angeforderterAnzeigenInhaltZusaetzlich gesetzt werden soll
   */
  public void setAngeforderterAnzeigenInhaltZusaetzlich( Vector<WVZInhalt> angeforderterAnzeigenInhaltZusaetzlich )
  {
    m_angeforderterAnzeigenInhaltZusaetzlich = angeforderterAnzeigenInhaltZusaetzlich;
  }

  /**
   * Methode liefert die Variable m_angeforderterAnzeigenInhaltVerschiebung zurck
   * @return Variable m_angeforderterAnzeigenInhaltVerschiebung
   */
  public Vector<WVZInhalt> getAngeforderterAnzeigenInhaltVerschiebung()
  {
    return m_angeforderterAnzeigenInhaltVerschiebung;
  }

  /**
   * Methode setzt die Variable m_angeforderterAnzeigenInhaltVerschiebung
   * @param angeforderterAnzeigenInhaltVerschiebung Wert auf den die Variable m_angeforderterAnzeigenInhaltVerschiebung gesetzt werden soll
   */
  public void setAngeforderterAnzeigenInhaltVerschiebung( Vector<WVZInhalt> angeforderterAnzeigenInhaltVerschiebung )
  {
    m_angeforderterAnzeigenInhaltVerschiebung = angeforderterAnzeigenInhaltVerschiebung;
  }

  /**
   * Methode liefert die Variable m_auswahlAnzeigeQuerschnitt zurck
   * @return Variable m_auswahlAnzeigeQuerschnitt
   */
  public SelektionAnzeigeQuerschnitt getAuswahlAnzeigeQuerschnitt()
  {
    return m_auswahlAnzeigeQuerschnitt;
  }

  /**
   * Methode setzt die Variable m_auswahlAnzeigeQuerschnitt
   * @param auswahlAnzeigeQuerschnitt Wert auf den die Variable m_auswahlAnzeigeQuerschnitt gesetzt werden soll
   */
  public void setAuswahlAnzeigeQuerschnitt( SelektionAnzeigeQuerschnitt auswahlAnzeigeQuerschnitt )
  {
    m_auswahlAnzeigeQuerschnitt = auswahlAnzeigeQuerschnitt;
  }
}
