package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.bedienung;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.aufhebungen.aufhebung.VerwaltungParaAufhebung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallandereszeichen.VerwaltungParaAusfallAnderesZeichen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallgleicheszeichen.VerwaltungParaAusfallGleichesZeichen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausschaltregel.VerwaltungParaAusschaltRegel;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.lueckenfuellen.VerwaltungParaLueckenFuellen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonie.VerwaltungParaMonotonie;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonieKreuzPfeil.VerwaltungParaMonotonieKreuzPfeil;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.trichterung.VerwaltungParaTrichterung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.VerwaltungParaUniversal;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.verdopplung.VerwaltungParaVerdopplung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.zusammenfassen.VerwaltungParaZusammenfassen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.zusammenfassungEnde.VerwaltungParaZusammenfassungEnde;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.notwendigeschaltungen.absolutnotwendigeschaltung.VerwaltungParaAbsolutNotwendigeSchaltung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.notwendigeschaltungen.notwendigeschaltung.VerwaltungParaNotwendigeSchaltung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio.DefaultWerte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.VerwaltungParaAllgemeineVerriegelungsMatrix;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.spezielleVerriegelungsMatrix.VerwaltungParaSpezielleVerriegelungsMatrix;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verschiebungen.verschieben.VerwaltungParaVerschieben;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Composite zum Versorgen der Parameter Bedienung
 * @author ObertM
 *
 */
public class CompositeAspekte extends Composite
{
	/**
	 * Vektor mit allen mglichen Aspekten
	 */
	private List<SystemObject> _aspekte = new Vector<SystemObject>();

	/**
	 * Parameter Schaltgrnde
	 */
	private Vector<SystemObject> _parameter = new Vector<SystemObject>();

	/**
	 * Zuordnung der Rckmeldungen
	 */
	private CompositeZuordnung _zuordnung;
	
  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil
   * @param parameter aktuelle Parameter
   */
	CompositeAspekte(Composite parent, int style, ParameterBedienung parameter) 
  {
    super(parent, style);
    
    _parameter.clear();
    _parameter.addAll(parameter.getMoeglicheSchaltgruende());

    /*
     * mgliche Objekte bestimmen 
     */
    bestimmeAspekte ();

    initialize();
    
    aktualisiereEingabeElemente();
  }

	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;

    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);

		GridData gridDataCombo = new GridData();
		gridDataCombo.horizontalAlignment = GridData.FILL;
		gridDataCombo.grabExcessHorizontalSpace = true;

    /*
     * Composite mit der Zuordnung der WVZ-Anzeigen
     */
		GridData gridDataZuordnung = new GridData();
		gridDataZuordnung.horizontalAlignment = GridData.FILL;
		gridDataZuordnung.grabExcessHorizontalSpace = true;
		gridDataZuordnung.verticalAlignment = GridData.FILL;
		gridDataZuordnung.grabExcessVerticalSpace = true;

  	_zuordnung = new CompositeZuordnung(this, SWT.BORDER, "mgliche Aspekte (Schaltgrnde):", "zugeordnete Aspekte (Schaltgrnde):");
  	_zuordnung.setLayoutData(gridDataZuordnung);
    
  	/*
  	 * Composite mit dem Defaultbutton 
  	 */
  	createCompositeDefaultButton(this);
  }

  /**
   * Methode bestimmt die im DaV definierten Aspekte
   */
	private void bestimmeAspekte()
  {
		_aspekte = Tools.getSystemObjekte("typ.aspekt");
  }

	/**
   * Methode fgt die Aspekte hinzu
   */
  private void aktualisiereEingabeElemente ()
  {
  	_zuordnung.getMoeglicheObjekte().clear();
  	_zuordnung.getZugewieseneObjekte().clear();
  	
  	for (SystemObject so : _aspekte)
  		_zuordnung.getMoeglicheObjekte().add(so);
  	
  	for (SystemObject so : _parameter)
  	{
  		_zuordnung.getZugewieseneObjekte().add(so);
  		_zuordnung.getMoeglicheObjekte().remove(so);
  	}

  	_zuordnung.refresh ();
  }  

	/**
	 * Methode erzeugt das Composite mit dem Default Button
	 */
	private void createCompositeDefaultButton(Composite parent)
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		
		Composite composite = new Composite(parent, SWT.BORDER);
		composite.setLayout(new GridLayout(1, false));
		composite.setLayoutData(gridData1);

		Button buttonDefault = new Button(composite, SWT.NONE);
		
		buttonDefault.setText("Defaultwerte");
		buttonDefault.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseDown(MouseEvent e)
			{
				super.mouseDown(e);
				
				defaultWerte();
			}
		});
	}

	/**
	 * Methode definiert die Defaultwerte
	 */
	private void defaultWerte ()
	{
		java.util.List<String> pids = new ArrayList<String>();
		
		pids.add(VerwaltungParaTrichterung._pidDefaultAspekt);
		pids.add(VerwaltungParaVerdopplung._pidDefaultAspekt);
		pids.add(VerwaltungParaLueckenFuellen._pidDefaultAspekt);
		pids.add(VerwaltungParaMonotonie._pidDefaultAspekt);
		pids.add(VerwaltungParaZusammenfassen._pidDefaultAspekt);
		pids.add(VerwaltungParaUniversal._pidDefaultAspekt);
		pids.add(VerwaltungParaMonotonieKreuzPfeil._pidDefaultAspekt);
		pids.add(VerwaltungParaZusammenfassungEnde._pidDefaultAspekt);
		
		pids.add(VerwaltungParaAusfallGleichesZeichen._pidDefaultAspekt);
		pids.add(VerwaltungParaAusfallAnderesZeichen._pidDefaultAspekt);
		pids.add(VerwaltungParaAusschaltRegel._pidDefaultAspekt);
		
		pids.add(VerwaltungParaAbsolutNotwendigeSchaltung._pidDefaultAspekt);
		pids.add(VerwaltungParaNotwendigeSchaltung._pidDefaultAspekt);
		
		pids.add(VerwaltungParaVerschieben._pidDefaultAspekt);
		
		pids.add(VerwaltungParaAufhebung._pidDefaultAspekt);
		
		pids.add(VerwaltungParaAllgemeineVerriegelungsMatrix._pidDefaultAspekt);
//		pids.add(VerwaltungParaSpezielleVerriegelungsMatrix._pidDefaultAspekt);
		
		pids.add(DefaultWerte._pidHand);
		pids.add(DefaultWerte._pidNaesse);
		pids.add(DefaultWerte._pidStau);

		_parameter.clear();

		for (String pid : pids)
		{
			SystemObject so = Konfig.getInstance().getSystemObject(pid);
			if (so != null)
				_parameter.add(so);
		}

		aktualisiereEingabeElemente();
	}

  /**
   * Methode liefert die aktuell eingestellten Parameter
   * @return aktuell eingestellten Parameter
   */
	public Vector<SystemObject>  getParameter()
  {
    Vector<SystemObject> objekte = new Vector<SystemObject>();
    
    for (Object so : _zuordnung.getZugewieseneObjekte())
    	objekte.add((SystemObject) so);
    
  	return objekte;
  }

	/**
	 * Methode aktualisiert die View
	 */
	public void refresh()
  {
  }
}
