package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.lueckenfuellen;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Lngsabgleichs- und Querabgleichsregeln", Regel "Lckenfllen".<br>  Regel:<br>  Wenn Anzeigeinhalt = "X" und<br>  Inhalt des Vorgnger/Nachfolger in Lngsrichtung = "Y"<br>  Dann fr diesen Vorgnger/Nachfolger in Lngsrichtung "Z" anfordern.<br>   Fr die Regel ist einstellbar ob sie in Vorgnger- oder Nachfolgerrichtung arbeitet.
 * @author Dambach Werke GmbH
 */
public class RegelLueckenFuellen
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;
  
  /**
   * arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   */
  private ArbeitsRichtungVN m_richtung = ArbeitsRichtungVN.VORGAENGER;
  
  /**
  * Anzeigeninhalt (in der Regel "X")
  */
 private WVZInhalt m_anzeigenInhalt = new WVZInhalt();

 /**
  * Anzeigeninhalt Vorgnger/Nachfolger (in der Regel: ["X", "Y", "Z"])
  */
 private List<WVZInhalt> m_anzeigenInhaltVorgaengerNachfolger = new ArrayList<WVZInhalt>();

 /**
  * Anzeigeninhalt Vorgnger des Vorgnger / Nachfolger des Nachfolger (in der Regel: ["U", "V", "W"])
  */
 private List<WVZInhalt> m_anzeigenInhaltVorVorgaengerNachNachfolger = new ArrayList<WVZInhalt>();

 /**
  * Angeforderter Anzeigeninhalt (in der Regel: "A")
  */
 private WVZInhalt m_angeforderterAnzeigenInhalt = new WVZInhalt();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelLueckenFuellen ()
  {
  }
  
  /**
   * Konstruktor der Klasse
   * @param schaltgrundRegel Regelspezifischer Schaltgrund
   * @param richtung arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   * @param anzeigenInhalt Anzeigeninhalt (in der Regel "X")
   * @param anzeigenInhaltVorgaengerNachfolger Anzeigeninhalt Vorgnger/Nachfolger (in der Regel: ["X", "Y", "Z"])
   * @param anzeigenInhaltVorVorgaengerNachNachfolger Anzeigeninhalt Vorgnger des Vorgnger / Nachfolger des Nachfolger (in der Regel: ["U", "V", "W"])
   * @param angeforderterAnzeigenInhalt Angeforderter Anzeigeninhalt (in der Regel: "A")
   */
  public RegelLueckenFuellen (Aspect schaltgrundRegel, ArbeitsRichtungVN richtung, 
  		WVZInhalt anzeigenInhalt, List<WVZInhalt> anzeigenInhaltVorgaengerNachfolger,
  		List<WVZInhalt> anzeigenInhaltVorVorgaengerNachNachfolger,
  		WVZInhalt angeforderterAnzeigenInhalt)
  {
  	m_schaltgrundRegel = schaltgrundRegel;
  	m_richtung = richtung;
  	m_anzeigenInhalt = anzeigenInhalt;
  	m_anzeigenInhaltVorgaengerNachfolger = anzeigenInhaltVorgaengerNachfolger;
  	m_anzeigenInhaltVorVorgaengerNachNachfolger = anzeigenInhaltVorVorgaengerNachNachfolger;
  	m_angeforderterAnzeigenInhalt = angeforderterAnzeigenInhalt;
  }
  
  /**
   * Methode prft ob die Regel in Vorgngerrichtung arbeitet
   * @return Regel arbeitet in Vorgngerrichtung
   */
  public boolean isRichtungVorgaenger()
  {
    return m_richtung == ArbeitsRichtungVN.VORGAENGER;
  }

  /**
   * Methode setzt die Variable m_richtung
   * @param richtung Wert auf den die Variable m_richtung gesetzt werden soll
   */
  public void setRichtung ( ArbeitsRichtungVN richtung )
  {
    m_richtung = richtung;
  }


  /**
   * Methode liefert die Variable m_anzeigenInhalt zurck
   * @return Variable m_anzeigenInhalt
   */
  public WVZInhalt getAnzeigenInhalt()
  {
    return m_anzeigenInhalt;
  }


  /**
   * Methode setzt die Variable m_anzeigenInhalt
   * @param anzeigenInhalt Wert auf den die Variable m_anzeigenInhalt gesetzt werden soll
   */
  public void setAnzeigenInhalt( WVZInhalt anzeigenInhalt )
  {
    m_anzeigenInhalt = anzeigenInhalt;
  }


  /**
   * Methode liefert die Variable m_anzeigenInhaltVorgaengerNachfolger zurck
   * @return Variable m_anzeigenInhaltVorgaengerNachfolger
   */
  public List<WVZInhalt> getAnzeigenInhaltVorgaengerNachfolger()
  {
    return m_anzeigenInhaltVorgaengerNachfolger;
  }


  /**
   * Methode setzt die Variable m_anzeigenInhaltVorgaengerNachfolger
   * @param anzeigenInhaltVorgaengerNachfolger Wert auf den die Variable m_anzeigenInhaltVorgaengerNachfolger gesetzt werden soll
   */
  public void setAnzeigenInhaltVorgaengerNachfolger( List<WVZInhalt> anzeigenInhaltVorgaengerNachfolger )
  {
    m_anzeigenInhaltVorgaengerNachfolger = anzeigenInhaltVorgaengerNachfolger;
  }


  /**
   * Methode liefert die Variable m_angeforderterAnzeigenInhalt zurck
   * @return Variable m_angeforderterAnzeigenInhalt
   */
  public WVZInhalt getAngeforderterAnzeigenInhalt()
  {
    return m_angeforderterAnzeigenInhalt;
  }


  /**
   * Methode setzt die Variable m_angeforderterAnzeigenInhalt
   * @param angeforderterAnzeigenInhalt Wert auf den die Variable m_angeforderterAnzeigenInhalt gesetzt werden soll
   */
  public void setAngeforderterAnzeigenInhalt( WVZInhalt angeforderterAnzeigenInhalt )
  {
    m_angeforderterAnzeigenInhalt = angeforderterAnzeigenInhalt;
  }
  
  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaTrichterungProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    String vorgaengerNachfolger = data.getItem( "VorgngerNachfolger" ).asTextValue().getText();
    
    if (vorgaengerNachfolger.equals( "Vorgnger" ))
      setRichtung (ArbeitsRichtungVN.VORGAENGER);
    else
      setRichtung (ArbeitsRichtungVN.NACHFOLGER);
    
    // X bestimmen
    
    Data anzeigenInhalt = data.getItem( "Anzeigeinhalt" );
    m_anzeigenInhalt.auswerteAttributgruppe( anzeigenInhalt );
    
    // XYZ bestimmen
    
    m_anzeigenInhaltVorgaengerNachfolger.clear();
    Array vn = data.getArray( "AnzeigeinhaltVorgngerNachfolger" );
    int anzahl = vn.getLength();
    for (int x = 0; x < anzahl; ++x)
    {
      Data d = vn.getItem( x );
      WVZInhalt inhalt = new WVZInhalt();
      inhalt.auswerteAttributgruppe( d );
      
      m_anzeigenInhaltVorgaengerNachfolger.add( inhalt );
    }
    
    // UVW bestimmen
    
    m_anzeigenInhaltVorVorgaengerNachNachfolger.clear();
    vn = data.getArray( "AnzeigeinhaltVorVorgngerNachNachfolger" );
    anzahl = vn.getLength();
    for (int x = 0; x < anzahl; ++x)
    {
      Data d = vn.getItem( x );
      WVZInhalt inhalt = new WVZInhalt();
      inhalt.auswerteAttributgruppe( d );
      
      m_anzeigenInhaltVorVorgaengerNachNachfolger.add( inhalt );
    }
    
    // A bestimmen
    
    Data angeforderterInhalt = data.getItem( "AngeforderterAnzeigeInhalt" );
    m_angeforderterAnzeigenInhalt.auswerteAttributgruppe( angeforderterInhalt );
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel ) 
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "Richtung: " + m_richtung + "\n";
    s += "X = " + Tools.getWVZInhalt( m_anzeigenInhalt ) + "\n";
    s += "XYZ = \n";
    for (WVZInhalt inhalt : m_anzeigenInhaltVorgaengerNachfolger)
      s += "-" + Tools.getWVZInhalt( inhalt ) + "\n";
    s += "UVW = \n";
    for (WVZInhalt inhalt : m_anzeigenInhaltVorVorgaengerNachNachfolger)
      s += "-" + Tools.getWVZInhalt( inhalt ) + "\n";
    s += "A = " + Tools.getWVZInhalt( m_angeforderterAnzeigenInhalt ) + "\n";
    
    return s;
  }

  /**
   * Methode liefert die Richtung in die Regelklasse arbeitet.
   * @return Richtung
   */
  public ArbeitsRichtungVN getRichtung()
  {
    return m_richtung;
  }

  /**
   * Methode liefert die Variable m_anzeigenInhaltVorVorgaengerNachNachfolger zurck
   * @return Variable m_anzeigenInhaltVorVorgaengerNachNachfolger
   */
  public List<WVZInhalt> getAnzeigenInhaltVorVorgaengerNachNachfolger()
  {
    return m_anzeigenInhaltVorVorgaengerNachNachfolger;
  }

  /**
   * Methode setzt die Variable m_anzeigenInhaltVorVorgaengerNachNachfolger
   * @param anzeigenInhaltVorVorgaengerNachNachfolger Wert auf den die Variable m_anzeigenInhaltVorVorgaengerNachNachfolger gesetzt werden soll
   */
  public void setAnzeigenInhaltVorVorgaengerNachNachfolger( List<WVZInhalt> anzeigenInhaltVorVorgaengerNachNachfolger )
  {
    m_anzeigenInhaltVorVorgaengerNachNachfolger = anzeigenInhaltVorVorgaengerNachNachfolger;
  }
}
