package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.lueckenfuellen;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

/**
 * Tabellen Modell fr die Parameter "Lcken fllen"
 * @author ObertM
 *
 */
public class TableModel extends Observable implements Observer
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;
	
	/**
	 * Schaltgrund
	 */
  private Composite _schaltgrund;
  
  /**
   * Arbeitsrichtung
   */
  private Composite _richtung;

  /**
   * Parameter X
   */
  private Composite _x;

  /**
   * Parameter XYZ
   */
  private Composite _xyz;
  
  /**
   * Parameter UVW
   */
  private Composite _uvw;
  
  /**
   * Parameter A
   */
  private Composite _a;
  
  /**
   * Konstruktor der Klasse 
   * @param loeschen Lschen
   * @param schaltgrund Schaltgrund der Regel
   * @param richtung Arbeitsrichtung
   * @param x zu prfender Anzeigeinhalt
   * @param xyz Anzeigeinhalt des Vorgngers/Nachfolgers
   * @param uvw Anzeigeinhalt des Vorgngers des Vorgngers / Nachfolger des Nachfolgers
   * @param a zu schaltender Anzeigeinhalt
   */
  public TableModel(Composite loeschen, Composite schaltgrund, Composite richtung, Composite x, Composite xyz, Composite uvw, Composite a)
  {
  	this._loeschen = loeschen;
    this._schaltgrund = schaltgrund;
    this._richtung = richtung;   
    this._x = x;    
    this._xyz = xyz;    
    this._uvw = uvw;    
    this._a = a;    
  }

  /**
   * Methode liefert den eingestellten Wert des Composite Lschen
   * @return eingestellter Wert
   */
  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen);    
  }

  /**
   * Methode liefert den eingestellten Wert des Composite Schaltgrund
   * @return eingestellter Wert
   */
  public String getSelectedSchaltgrund ()
  {
    return getSelectedWert(_schaltgrund);    
  }

  /**
   * Methode liefert den eingestellten Wert des Composite Richtung
   * @return eingestellter Wert
   */
  public String getSelectedRichtung ()
  {
    return getSelectedWert(_richtung);    
  }

  /**
   * Methode liefert den eingestellten Wert des Composite X
   * @return eingestellter Wert
   */
  public String getSelectedX ()
  {
    return getSelectedWert(_x);    
  }

  /**
   * Methode liefert die eingestellten Werte des Composite XYZ
   * @return eingestellte Werte
   */
  public String[] getSelectedXYZ ()
  {
  	Control[] controls = _xyz.getChildren();
  	String [] selected = new String [controls.length];
  	
  	for (int i=0; i<controls.length; ++i)
  	  selected [i] = getSelectedWert(controls [i]);
  	
    return selected;    
  }

  /**
   * Methode liefert die eingestellten Werte des Composite UVW
   * @return eingestellte Werte
   */
  public String[] getSelectedUVW ()
  {
  	Control[] controls = _uvw.getChildren();
  	String [] selected = new String [controls.length];
  	
  	for (int i=0; i<controls.length; ++i)
  	  selected [i] = getSelectedWert(controls [i]);
  	
    return selected;    
  }
  
  /**
   * Methode liefert den eingestellten Wert des Composite A
   * @return eingestellter Wert
   */
  public String getSelectedA ()
  {
    return getSelectedWert(_a);    
  }

  /**
   * Methode setzt den Wert des Compiste Lschen
   * @param selWert zu setzender Wert
   */
  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen, selWert, null);
  }

  /**
   * Methode setzt den Wert des Compiste Lschen
   * @param selWert zu setzender Wert
   */
  public void setSelectedSchaltgrund(String selWert)
  {
    setSelectedWert(_schaltgrund, selWert, null);
  }

  /**
   * Methode setzt den Wert des Compiste Lschen
   * @param selWert zu setzender Wert
   */
  public void setSelectedRichtung(String selWert)
  {
    setSelectedWert(_richtung, selWert, null);
  }

  /**
   * Methode setzt den Wert des Compiste X
   * @param selWert zu setzender Wert
   */
  public void setSelectedX(Image selImage)
  {
    setSelectedWert(_x, null, selImage);
  }

  /**
   * Methode setzt die Werte des Compiste XYZ
   * @param selWert zu setzender Wert
   */
  public void setSelectedXYZ (Image[] selImage)
  {
  	Control[] controls = _xyz.getChildren();
  	
  	for (int i=0; i<controls.length; ++i)
  	  setSelectedWert (controls [i], null, selImage [i]);
  }

  /**
   * Methode setzt die Werte des Compiste UVW
   * @param selWert zu setzender Wert
   */
  public void setSelectedUVW (Image[] selImage)
  {
  	Control[] controls = _uvw.getChildren();
  	
  	for (int i=0; i<controls.length; ++i)
  	  setSelectedWert (controls [i], null, selImage [i]);
  }
  
  /**
   * Methode setzt den Wert des Compiste A
   * @param selWert zu setzender Wert
   */
  public void setSelectedA(Image selImage)
  {
    setSelectedWert(_a, null, selImage);
  }
  
  /**
   * Methode liefert den eingestellten Wert des ersten Control Elements eines Composite
   * als String zurck.
   * @param c Composite
   * @return Wert als String
   */
  public String getSelectedWert(Composite c)
  {
  	Control control = c.getChildren()[0];

  	return getSelectedWert(control);
  }

  /**
   * Methode liefert den eingestellten Wert eines Control Elements 
   * als String zurck.
   * @param control Control Element
   * @return Wert als String
   */
  public String getSelectedWert(Control control)
  {
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = cb.getItem( cb.getSelectionIndex() );
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  /**
   * Methode setzt den Wert des ersten Control Elements eines Composite
   * @param c Composite
   */
  public void setSelectedWert(Composite c, String selWert, Image selImage)
  {
    Control control = c.getChildren()[0];

    setSelectedWert(control, selWert, selImage);
  }

  /**
   * Methode setzt den Wert eines Control Elements eines Composite
   * @param c Composite
   */
  public void setSelectedWert(Control control, String selWert, Image selImage)
  {
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode setzt die Variable loeschen
   * @param loeschen zu bernehmender Wert fr loeschen
   */
  public void setLoeschen(Composite loeschen)
  {
  	_loeschen = loeschen;
  }

	/**
   * Methode liefert schaltgrund zurck
   * @return schaltgrund
   */
  public Composite getSchaltgrund()
  {
  	return _schaltgrund;
  }

	/**
   * Methode setzt die Variable schaltgrund
   * @param schaltgrund zu bernehmender Wert fr schaltgrund
   */
  public void setSchaltgrund(Composite schaltgrund)
  {
  	_schaltgrund = schaltgrund;
  }

	/**
   * Methode liefert richtung zurck
   * @return richtung
   */
  public Composite getRichtung()
  {
  	return _richtung;
  }

	/**
   * Methode setzt die Variable richtung
   * @param richtung zu bernehmender Wert fr richtung
   */
  public void setRichtung(Composite richtung)
  {
  	_richtung = richtung;
  }

	/**
   * Methode liefert x zurck
   * @return x
   */
  public Composite getX()
  {
  	return _x;
  }

	/**
   * Methode setzt die Variable x
   * @param x zu bernehmender Wert fr x
   */
  public void setX(Composite x)
  {
  	_x = x;
  }

	/**
   * Methode liefert xyz zurck
   * @return xyz
   */
  public Composite getXyz()
  {
  	return _xyz;
  }

	/**
   * Methode setzt die Variable xyz
   * @param xyz zu bernehmender Wert fr xyz
   */
  public void setXyz(Composite xyz)
  {
  	_xyz = xyz;
  }

	/**
   * Methode liefert uvw zurck
   * @return uvw
   */
  public Composite getUvw()
  {
  	return _uvw;
  }

	/**
   * Methode setzt die Variable uvw
   * @param uvw zu bernehmender Wert fr uvw
   */
  public void setUvw(Composite uvw)
  {
  	_uvw = uvw;
  }

	/**
   * Methode liefert a zurck
   * @return a
   */
  public Composite getA()
  {
  	return _a;
  }

	/**
   * Methode setzt die Variable a
   * @param a zu bernehmender Wert fr a
   */
  public void setA(Composite a)
  {
  	_a = a;
  }

}
