package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungQL;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaften;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Lngsabgleichs- und Querabgleichsregeln", Regel "Universal".<br>
 * Regel:<br>
 * Eine Reihe von Anzeigen, welche die folgenden Eigenschaften aufweist (whlbar je Anzeige und Anzeigenverknpfung):<br> 
 * Anzeigeinhalt ist "X" = ja / nein / beliebig<br>
 * Anzeigeinhalt des Vorintervalls ist "Y" = ja / nein / beliebig<br>
 * Hat keinen Vorgnger = ja / nein / beliebig<br>
 * Hat keinen Nachfolger = ja / nein / beliebig<br>
 * Ist vom WZGtyp "A" = ja / nein / beliebig<br>
 * Ist die Anzeige "4711" = ja / nein / beliebig<br>
 * Verweist auf Fahrstreifentyp "F" = ja / nein / beliebig<br>
 * Verweist auf die Fahrstreifen "4712" = ja / nein / beliebig<br<
 * Anzeige ausgefallen = ja / nein / beliebig<br>
 * WVZinhalt ["U", "V", "W"] ausgefallen = ja / nein / beliebig<br>
 * Anzahl Fahrstreifen AQ [ErsteAnzeigeDer Reihe] ist "n" = ja / nein / beliebig<br>
 * wird ersetzt durch die Reihe von Inhalten "1", "2", "3".<br> 
 * Dabei muss whlbar sein, dass der vorige Inhalt beibehalten wird, d.h. an einzelnen Anzeigen keine Ersetzung stattfindet. 
 * Alternativ dazu ist bei allen Anzeigen vom WZGTyp "B" auf allen beteiligten AQ der Inhalt "Y" anzufordern.
 * @author   Swarco Traffic Systems GmbH
 */
public class RegelUniversal
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;

  /**
   * arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   */
  private ArbeitsRichtungVN m_richtungVN = ArbeitsRichtungVN.VORGAENGER;

  /**
   * Arbeitet Regel in Quer- oder Lngsrichtung 
   */
  private ArbeitsRichtungQL m_richtungQL = ArbeitsRichtungQL.LAENGSRICHTUNG;

  /**
   * Reihe von Anzeigen 
   */
  private Vector<SystemObject> m_anzeigen = new Vector<SystemObject>();
  
  /**
   * Reihe von Eigenschaften
   */
  private Vector<UniversalEigenschaften> m_eigenschaften = new Vector<UniversalEigenschaften>();
  
  /**
   * angeforderte Inhalte der Anzeigen 
   */
  private Vector<WVZInhalt> m_angeforderteInhalte = new Vector<WVZInhalt>();
  
  /**
   * Anzeigentyp alternativ (in der Regel: WZGTyp "B"), wird kein Typ angegeben wird "normal"
   * geschaltet, bei Angabe eines alternativen Anzeigetyps die Alternative
   */
  private SystemObject m_alternativerAnzeigenTyp;
  
  /**
   * Alternativ zu schaltender Anzeigeninhalt (in der Regel: "Y")
   */
  private WVZInhalt m_alternativerAnzeigenInhalt = new WVZInhalt();

  /**
   * Methode prft ob die Regel in Vorgngerrichtung arbeitet
   * @return Regel arbeitet in Vorgngerrichtung
   */
  public boolean isRichtungVorgaenger()
  {
    return m_richtungVN == ArbeitsRichtungVN.VORGAENGER;
  }

  /**
   * Methode setzt die Variable m_richtungVN
   * @param richtungVN Wert auf den die Variable m_richtungVN gesetzt werden soll
   */
  public void setRichtungVN ( ArbeitsRichtungVN richtungVN )
  {
    m_richtungVN = richtungVN;
  }

  /**
   * Methode prft ob die Regel in Querrichtung arbeitet
   * @return Regel arbeitet in Querrichtung
   */
  public boolean isRichtungQuerrichtung()
  {
    return m_richtungQL == ArbeitsRichtungQL.QUERRICHTUNG;
  }

  /**
   * Methode setzt die Variable m_richtungQL
   * @param richtungQL Wert auf den die Variable m_richtungQL gesetzt werden soll
   */
  public void setRichtungQL ( ArbeitsRichtungQL richtungQL )
  {
    m_richtungQL = richtungQL;
  }

  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaRegelUniversal'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe( Data data, Aspect schaltgrundAllgemein )
  {
    // Schaltgrund der Regel
    
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    // Vorgnger-/Nachfolgerrichtung
    
    String vorgaengerNachfolger = data.getItem( "VorgngerNachfolger" ).asTextValue().getText();
    if (vorgaengerNachfolger.equals( "Vorgnger" ))
      setRichtungVN (ArbeitsRichtungVN.VORGAENGER);
    else
      setRichtungVN (ArbeitsRichtungVN.NACHFOLGER);

    // Quer-/Lngsrichtung
    
    String querLaengs = data.getItem( "QuerLngsRichtung" ).asTextValue().getText();
    if (querLaengs.equals( "Querrichtung" ))
      setRichtungQL (ArbeitsRichtungQL.QUERRICHTUNG);
    else
      setRichtungQL (ArbeitsRichtungQL.LAENGSRICHTUNG);
    
    // Reihe von Anzeigen
    
    m_anzeigen.clear();
    ReferenceArray ra = data.getReferenceArray( "Anzeigen" );
    int anzahl = ra.getLength();
    for (int x = 0; x < anzahl; ++x)
      m_anzeigen.add ( ra.getSystemObject( x ) );

    // Eigenschaften
    
    Array a1 = data.getArray( "Eigenschaften" );
    anzahl = a1.getLength();
    
    for (int x=0; x<anzahl; ++x)
    {
      UniversalEigenschaften ue = new UniversalEigenschaften();
      m_eigenschaften.add( ue );
      ue.auswerteAttributgruppe( a1.getItem( x ) );
    }
    
    // Reihe von Anzeigeinhalten
    
    m_angeforderteInhalte.clear();
    Array a = data.getArray( "AngeforderteInhalte" );
    anzahl = a.getLength();
    for (int x = 0; x < anzahl; ++x)
    {
      WVZInhalt inhalt = new WVZInhalt();
      inhalt.auswerteAttributgruppe( a.getItem( x ) );
      m_angeforderteInhalte.add( inhalt );
    }
    
    // Alternativer Anzeigetyp
    
    m_alternativerAnzeigenTyp = data.getReferenceValue( "AlternativerAnzeigeTyp" ).getSystemObject();
    
    // Alternativer Anzeigeinhalt
    
    m_alternativerAnzeigenInhalt = new WVZInhalt();
    m_alternativerAnzeigenInhalt.auswerteAttributgruppe( data.getItem( "AlternativerAnzeigenInhalt" ) );
  }

  /**
   * Methode liefert die Variable m_anzeigen zurck
   * @return Variable m_anzeigen
   */
  public Vector<SystemObject> getAnzeigen()
  {
    return m_anzeigen;
  }

  /**
   * Methode setzt die Variable m_anzeigen
   * @param anzeigen Wert auf den die Variable m_anzeigen gesetzt werden soll
   */
  public void setAnzeigen( Vector<SystemObject> anzeigen )
  {
    m_anzeigen = anzeigen;
  }

  /**
   * Methode liefert die Variable m_angeforderteInhalte zurck
   * @return Variable m_angeforderteInhalte
   */
  public Vector<WVZInhalt> getAngeforderteInhalte()
  {
    return m_angeforderteInhalte;
  }

  /**
   * Methode setzt die Variable m_angeforderteInhalte
   * @param angeforderteInhalte Wert auf den die Variable m_angeforderteInhalte gesetzt werden soll
   */
  public void setAngeforderteInhalte( Vector<WVZInhalt> angeforderteInhalte )
  {
    m_angeforderteInhalte = angeforderteInhalte;
  }

  /**
   * Methode liefert die Variable m_alternativerAnzeigenTyp zurck
   * @return Variable m_alternativerAnzeigenTyp
   */
  public SystemObject getAlternativerAnzeigenTyp()
  {
    return m_alternativerAnzeigenTyp;
  }

  /**
   * Methode setzt die Variable m_alternativerAnzeigenTyp
   * @param alternativerAnzeigenTyp Wert auf den die Variable m_alternativerAnzeigenTyp gesetzt werden soll
   */
  public void setAlternativerAnzeigenTyp( SystemObject alternativerAnzeigenTyp )
  {
    m_alternativerAnzeigenTyp = alternativerAnzeigenTyp;
  }

  /**
   * Methode liefert die Variable m_alternativerAnzeigenInhalt zurck
   * @return Variable m_alternativerAnzeigenInhalt
   */
  public WVZInhalt getAlternativerAnzeigenInhalt()
  {
    return m_alternativerAnzeigenInhalt;
  }

  /**
   * Methode setzt die Variable m_alternativerAnzeigenInhalt
   * @param alternativerAnzeigenInhalt Wert auf den die Variable m_alternativerAnzeigenInhalt gesetzt werden soll
   */
  public void setAlternativerAnzeigenInhalt( WVZInhalt alternativerAnzeigenInhalt )
  {
    m_alternativerAnzeigenInhalt = alternativerAnzeigenInhalt;
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel )
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }

  /**
   * Methode liefert die Variable m_eigenschaften zurck
   * @return Variable m_eigenschaften
   */
  public Vector<UniversalEigenschaften> getEigenschaften()
  {
    return m_eigenschaften;
  }

  /**
   * Methode setzt die Variable m_eigenschaften
   * @param eigenschaften Wert auf den die Variable m_eigenschaften gesetzt werden soll
   */
  public void setEigenschaften( Vector<UniversalEigenschaften> eigenschaften )
  {
    m_eigenschaften = eigenschaften;
  }
  
  /**
   * Methode gibt den Inhalt der Klasse als String aus
   */
  @Override
  public String toString()
  {
    String s = "";
    
    s += "Richtung V/N:" + m_richtungVN + "\n";
    s += "Richtung Q/L:" + m_richtungQL + "\n";
    s += "Anzeigen:\n";
    for (SystemObject so : m_anzeigen)
      s += Tools.getObjektName( so ) + "\n";
    s += "Eigenschaften:\n";
    s += m_eigenschaften.toString();
    s += "Angeforderte Inhalte:\n";
    for (WVZInhalt wi : m_angeforderteInhalte)
      s += "-" + Tools.getWVZInhalt( wi ) + "\n";
    s += "Alternativer Anzeigetyp: " + Tools.getObjektName( m_alternativerAnzeigenTyp ) + "\n";
    s += "Alternativer Anzeigeinghalt: " + Tools.getWVZInhalt( m_alternativerAnzeigenInhalt ) + "\n";
    
    return s;
  }

	/**
   * Methode liefert richtungVN zurck
   * @return richtungVN
   */
  public ArbeitsRichtungVN getRichtungVN()
  {
  	return m_richtungVN;
  }

	/**
   * Methode liefert richtungQL zurck
   * @return richtungQL
   */
  public ArbeitsRichtungQL getRichtungQL()
  {
  	return m_richtungQL;
  }

}
