package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeElementFigure;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltAuswahlDialog;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen.SBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Versorgung der Parameter
 */
public class CompositeParameterPriorisierung extends Composite
{
	private final String _default = " Defaultwert ";


	private final String _stau = " Stau ";
	private String _pidStau = "asp.sswSbaStau";
	private Aspect _schaltgrundStau;
	

	private final String _hand = " Hand ";
	private String _pidHand = "asp.sswSbaHandschaltung";
	private Aspect _schaltgrundHand;

	private final String _naesse = " Nsse ";
	private String _pidNaesse = "asp.sswSbaNssewarnung";
	private Aspect _schaltgrundNaesse;

	/**
	 * Aktuelles Composite
	 */
  private Composite _composite;

  /**
   * Composite der Tabelle
   */
  private Composite _compositeTabelle;

  /**
   * Gruppe mit den Regeln
   */
  private Group _groupRegeln;

  /**
   * Tabelviewer der Tabelle
   */
  private TableViewer _tvAnzeige;

  /**
   * Aktuelle Parameter
   */
  private ListeMitRegeln _aktuelleParameter;

  /**
   * Composite fr die Buttons
   */
	private Composite _compositeButtons;

	/**
	 * Button Hinzufgen
	 */
	private Button _buttonHinzufuegen;

	/**
	 * Button Lschen
	 */
	private Button _buttonLoeschen;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModel;
	
	/**
	 * Anzeigetyp fr den die Parameter versorgt werden
	 */
	private AnzeigeElementTyp _anzeigeTyp;


	private Button _buttonAutomatik;


	private Button _buttonDefault;

  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil des Fenstern
   * @param aktuell Aktuelle Parameter
   * @param pidAnzeigeTyp Pid des Anzeigetyps
   */
  CompositeParameterPriorisierung (Composite parent, int style, ListeMitRegeln aktuell, String pidAnzeigeTyp)
  {
    super (parent, style);

    this._aktuelleParameter = erzeugeKopie(aktuell);
    
    _anzeigeTyp = AnzeigeElementTyp.getTypByPid(pidAnzeigeTyp);
    
    _schaltgrundHand = Konfig.getInstance().getAspekt(_pidHand);

    _schaltgrundStau = Konfig.getInstance().getAspekt(_pidStau);

    _schaltgrundNaesse = Konfig.getInstance().getAspekt(_pidNaesse);

    initialize();
  }

  /**
   * Methode legt eine Arbeitskopie der akutellen Daten an
   * @param aktuell aktuelle Daten
   * @return Arbeitskopie
   */
  private ListeMitRegeln erzeugeKopie(ListeMitRegeln aktuell)
  {
	  ListeMitRegeln kopie = new ListeMitRegeln();
	  
	  Vector<RegelPriorisierungNeu> parameter = aktuell.getParameter();
	  for (RegelPriorisierungNeu regel : parameter)
	  	kopie.addRegel(regel.getWvzInhalt(),
	  			           regel.getPrioritaet(),
	  			           regel.getSchaltgrund());
	  
	  return kopie;
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
//    erzeugeKommentar();
    
    //Initialisierungen der Arrays
    
    List<TableModel[]> inputList = new ArrayList<TableModel[]>();
    
    erzeugeGruppeRegeln();
    
    Table table = _tvAnzeige.getTable();
    _tableModel = createModel1(table);
    
    if ( _tableModel != null )
      _tvAnzeige.setInput(_tableModel); 
    
    createButtons();
    
    updateData(_aktuelleParameter);
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Lngsabgleichsregel Verdopplung:\n\n" + 
  	                   "Wenn Anzeigeinhalt = X\n" +
  	                   "und Inhalt von keinem Vorgnger in Lngsrichtung = X\n" +
  	                   "und Inhalt von keinem Nachfolger in Lngsrichtung = X\n" +
  	                   "dann fr alle Vorgnger/Nachfolger in Lngsrichtung X anfordern.\n\n";
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Methode bernimmt die Daten und zeigt diese in der Tabelle an
   */
  private void updateData (ListeMitRegeln parameter)
  {
  	int anzahlParameter = parameter.getParameter().size();
  	
  	erzeugeTabellenModell (anzahlParameter);
  	
  	for (int j=0; j<anzahlParameter; ++j)
  	{
      TableModel tm = _tableModel [j];    	
    	RegelPriorisierungNeu aktuell = parameter.getParameter().get(j);
    	
    	/*
    	 * Lschen
    	 */
    	tm.setSelectedLoeschen("false");
    	
  	  /*
  	   * X
  	   */
  	  Label imX = (Label) tm.getX().getChildren()[0];
  	  WVZInhalt x = aktuell.getWvzInhalt();
  	  
  	  Tools.setzeBildInhalte(imX, x, _anzeigeTyp);
    	
    	/*
    	 * Schaltgrund
    	 */
    	
    	if (aktuell.getSchaltgrund() == null)
    	{
    	  tm.setSelectedSchaltgrund(_default);
    	}
    	else if (aktuell.getSchaltgrund() == _schaltgrundHand)
    	{
    	  tm.setSelectedSchaltgrund(_hand);
    	}
    	else if (aktuell.getSchaltgrund() == _schaltgrundStau)
    	{
    	  tm.setSelectedSchaltgrund(_stau);
    	}
    	else
    	{
  	    tm.setSelectedSchaltgrund(aktuell.getSchaltgrund().getName());
    	}
    	
    	/*
    	 * Prioritt
    	 */
    	tm.setSelectedPrioritaet("" + aktuell.getPrioritaet());
    }
  }

  /**
   * Methode erzeugt ein Tabellenmodell einer bestimmten Gre. Dabei werden bereits vorhandene
   * Composite in der Tabelle weiter verwendet
   * @param anzahlParameter Gre des erzeugten Tabellenmodells
   */
  private void erzeugeTabellenModell (int anzahlParameter)
  {
  	/*
  	 * Hat sich die Gre gendert
  	 */
  	if (_tableModel.length != anzahlParameter)
  	{
  		/*
  		 * Tabelle erweitern
  		 */
  		if (_tableModel.length < anzahlParameter)
  		{
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  
  		  for (int i=0; i<_tableModel.length; ++i)
  		  	v.add(_tableModel [i]);
  		  
  		  for (int i=_tableModel.length; i<anzahlParameter; ++i)
  		  	v.add (erzeugeZeileInTabelle(_tvAnzeige.getTable()));
  		  
  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		/*
  		 * Tabelle reduzieren
  		 */
  		else if (_tableModel.length > anzahlParameter)
  		{
  			/*
  			 * Nicht mehr bentigte Elemente lschen
  			 */
  			
  			for (int i = _tableModel.length - 1; i > anzahlParameter - 1; i--)
  			{
  				_tableModel [i].getPrioritaet().dispose();
  				_tableModel [i].getSchaltgrund().dispose();
  				_tableModel [i].getX().dispose();
  				_tableModel [i].getLoeschen().dispose();
  			}
  				
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  for (int i=0; i<anzahlParameter; ++i)
  		  	v.add(_tableModel [i]);

  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		
  	  _tvAnzeige.setInput(_tableModel);
  	  _tvAnzeige.getTable().layout(true);
  	}
  }
  
  /**
   * Methode erzeugt die Gruppe mit den Regeln
   */
  private void erzeugeGruppeRegeln()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupRegeln = new Group(_composite, SWT.NONE);
    _groupRegeln.setText("Regeln");
    
    _groupRegeln.setLayout(fillLayout1);
    _groupRegeln.setLayoutData(gridData2);

    erzeugeTabelle();
  }

  /**
   * Methode erzeugt das Composite mit den Buttons
   */
  private void createButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeButtons = new Composite( _composite, SWT.NONE );
    _compositeButtons.setLayoutData( gridData3 );
    _compositeButtons.setLayout( rowLayout );

    _buttonHinzufuegen = new Button( _compositeButtons, SWT.NONE );
    _buttonHinzufuegen.setText( "Regel hinzufgen" );
    _buttonHinzufuegen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelHinzufuegen();
      }
    } );

    _buttonLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonLoeschen.setText( "Selektierte Regeln loeschen" );
    _buttonLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelLoeschen();
      }

    } );
    _buttonLoeschen.setEnabled(false);

    _buttonDefault = new Button( _compositeButtons, SWT.NONE );
    _buttonDefault.setText( "Defaultwerte" );
    _buttonDefault.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionDefault();
      }

    } );

//    _buttonAutomatik = new Button( _compositeButtons, SWT.NONE );
//    _buttonAutomatik.setText( "Prioritt automatisch vergeben" );
//    _buttonAutomatik.addMouseListener( new MouseAdapter()
//    {
//      @Override
//      public void mouseDown( MouseEvent e )
//      {
//        aktionAutomatik ();
//      }
//
//    } );

  }

  /**
   * Methode vergibt die Prioritt automatisch in der Reihenfolge der Eintrge 
   */
  protected void aktionAutomatik()
  {
  	_aktuelleParameter = leseAktuelleEinstellungen();
  	Vector<RegelPriorisierungNeu> parameter = _aktuelleParameter.getParameter();
  	
  	int prio = parameter.size() * 10;
  	for (RegelPriorisierungNeu rpn : parameter)
  	{
  		rpn.setPrioritaet(prio);

  		System.out.println(rpn.getWvzInhalt().toString() + " " + prio);
  		
  		prio -= 10;
  		
  	}
  	
  	updateData(_aktuelleParameter);
  }

	/**
   * Methode prft ob Regeln zum Lschen ausgewhlt sind
   */
  private void pruefeObLoeschenAngewaehlt ()
  {
  	boolean angewaehlt = false;
  	
		for (TableModel tm : _tableModel)
			if (tm.getSelectedLoeschen().equals("true"))
				angewaehlt = true;
		
		_buttonLoeschen.setEnabled(angewaehlt);
  }
  
  /**
   * Methode zum Lschen der Regeln
   */
	private void aktionRegelLoeschen()
  {
		Vector<RegelPriorisierungNeu> regeln = leseAktuelleEinstellungen().getParameter();
		
    Vector<RegelPriorisierungNeu> loeschen = new Vector<RegelPriorisierungNeu>();
    
    for (int i=0; i<_tableModel.length; ++i)
    	if (!_tableModel [i].getSelectedLoeschen().equals("false"))
    		loeschen.add(regeln.get(i));

    for (RegelPriorisierungNeu r : loeschen)
    {
    	regeln.remove(r);
    }
    
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }

	/**
	 * Aktion Regel hinzufgen
	 */
	private void aktionRegelHinzufuegen()
  {
		Vector<RegelPriorisierungNeu> regeln = leseAktuelleEinstellungen().getParameter();
    _aktuelleParameter.setParameter(regeln);
		
		WVZInhalt dunkel = VerwaltungWvzInhalte.getInstance().getWvzInhalt(_anzeigeTyp, AnzeigeInhaltTyp.DUNKEL.getBildInhalt());
		_aktuelleParameter.addRegel (dunkel, 0, null);
		
		updateData(_aktuelleParameter);
  }

	/**
	 * Methode erzeugt das Composite mit der Tabelle
	 */
  private void erzeugeTabelle ()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    
    _compositeTabelle = new Composite(_groupRegeln, SWT.None);
    _compositeTabelle.setLayout(fillLayout1);
    
    _tvAnzeige = new TableViewer(_compositeTabelle, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());

    TableViewerColumn column0 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column0.getColumn().setWidth(70);
    column0.getColumn().setText("Lschen");
    column0.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column1.getColumn().setWidth(80);
    column1.getColumn().setText("WVZ-Inhalt");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column2.getColumn().setWidth(150);
    column2.getColumn().setText("Schaltgrund");
    column2.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column3 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column3.getColumn().setWidth(150);
    column3.getColumn().setText("Prioritt");
    column3.setLabelProvider(new MyCellLabelProvider());

    TableColumnSorter cSorter = new TableColumnSorter(_tvAnzeige, column3.getColumn()){

      @Override
      protected int doCompare( Viewer TableViewer, Object e1, Object e2 )
      {
        TableModel tm1 = (TableModel)e1;
        return tm1.compareTo( (TableModel)e2 );
      }
    };
    cSorter.setSorter( cSorter, TableColumnSorter.DESC );

    // resize the row height using a MeasureItem listener
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
          	event.height = Tools.getZeilenHoehe(_anzeigeTyp);
          	break;

          default:
        }

      }
    };
    
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }

  /**
   * Klasse realsiert den TableContentProvider fr die Tabelle
   * @author ObertM
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
  	/**
  	 * Tabelviewer
  	 */
    private TableViewer tableViewer;

    /**
     * Methode liefert die Elemente der Tabelle
     */
    public Object[] getElements(Object inputElement)
    {
			Object[] re = null;

			if (inputElement instanceof TableModel [])
			{
        re = (TableModel[]) inputElement;
			}
			
			return re;
    }

    /**
     * Dispose
     */
    public void dispose()
    {
    }

    /**
     * Methode wird aufgerufen wenn sich der Inhalt der Tabelle gendert hat
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.deleteObserver(this);
        }
      }
      
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.addObserver(this);
        }
      }
    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();
      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel1 (Table table)
  {
    int tabLength = getAnzahlParameter();
    int elIndex   = 0;
    
    TableModel[] elements = new TableModel [tabLength];
    
    if (elements.length == 0)
    	return elements;

    for (int x=0; x<getAnzahlParameter(); ++x)
    {
	    elements[elIndex] = erzeugeZeileInTabelle(table);
	    elIndex++;
    }
    
    return elements;
  }

  /**
   * Methode erzeugt ein Tabellenmodell fr eine Zeile in der Tabelle
   * @param table Tabelle 
   * @return Tabellenmodell fr eine Zeile
   */
  private TableModel erzeugeZeileInTabelle (Table table)
  {
    /*
     * Layout festlegen
     */
  	
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.grabExcessVerticalSpace = true;

    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.verticalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;

    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.grabExcessVerticalSpace = true;
    
    GridData gd3 = new GridData();
    gd3.horizontalAlignment = GridData.CENTER;
    gd3.verticalAlignment = GridData.CENTER;
    gd3.grabExcessHorizontalSpace = true;
    gd3.grabExcessVerticalSpace = true;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    /*
     * Button Lschen
     */
    Composite comp0 = new Composite( table, SWT.NONE );
    comp0.setBackground( table.getBackground() );
    comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp0.setLayoutData( gd1 );
    comp0.setLayout( new GridLayout() );
    Button b = new Button( comp0, SWT.CHECK );
    
    b.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				pruefeObLoeschenAngewaehlt();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
    
    /*
     * WVZ Inhalt
     */
    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new FillLayout() );
    
    Label lbImageX = new Label( comp1, SWT.CENTER );
    lbImageX.addMouseListener( new MouseAdapter()
    {
			@Override
      public void mouseDown( MouseEvent e )
      {
				Tools.waehleZuSchaltendesBild(e, _anzeigeTyp);
      }         
    });

    /*
     * Combobox Schaltgrund    
     */
    Composite comp2 = new Composite( table, SWT.NONE );
    comp2.setBackground( table.getBackground() );
    comp2.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp2.setLayoutData( gd2 );
    comp2.setLayout( new GridLayout() );
    
    Combo combo1 = new Combo( comp2, SWT.BORDER );
    combo1.setLayoutData( gdTableItem );
    
    Vector<SystemObject> aspekte = Tools.getAspekte();
    String [] sa = new String [aspekte.size() + 3];
    
    sa [0] = _default;
    sa [1] = _hand;
    sa [2] = _stau;
    
    for (int i=0; i<aspekte.size(); ++i)
      sa [i + 3] = aspekte.get(i).getName();
    
    combo1.setItems(sa);
    combo1.select(0);

    /*
     * Spinner Prioritt
     */
    Composite comp3 = new Composite( table, SWT.NONE );
    comp3.setBackground( table.getBackground() );
    comp3.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp3.setLayoutData( gd3 );
    comp3.setLayout( new GridLayout() );
    
    Spinner spinner = new Spinner( comp3, SWT.BORDER );

    spinner.setMinimum(0);
    spinner.setMaximum(Integer.MAX_VALUE);
    spinner.setDigits(0);
      
    String toolTip = "Prioritt des Anzeigeinhalts unter dem Aspekt. Zulssig von  " + spinner.getMinimum() 
                     + " bis " + spinner.getMaximum();
     
    comp3.setToolTipText(toolTip);
    spinner.setToolTipText(toolTip);
    spinner.setLayoutData( gdTableItem );
    
    return new TableModel( comp0, comp1, comp2, comp3 );
  }
  
  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public int getAnzahlParameter()
  {
    return _aktuelleParameter.getParameter().size();
  }
  
  /**
   * Methode liefert den Tabelviewer
   * @return
   */
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }

  /**
   * Renderer fr die Tabelle
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider 
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

//      if (item.getData("EDITOR") != null)
//      {
//        TableEditor editor = (TableEditor)item.getData("EDITOR");
//        editor.dispose();
//      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getLoeschen();
          break;
          
        case 1:
          comp = model.getX();
          break;
          
        case 2:
          comp = model.getSchaltgrund();
          break;

        case 3:
          comp = model.getPrioritaet();
          break;

        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }

  /**
   * Methode liefert die eingestellten Parameter
   * @return eingestellte Parameter
   */
	public ListeMitRegeln getParameter()
  {
 		aktionRegelLoeschen();
 		return leseAktuelleEinstellungen();
  }
		
	private ListeMitRegeln leseAktuelleEinstellungen ()
	{
		ListeMitRegeln regeln = new ListeMitRegeln();
		
		for (int j=0; j<_tableModel.length; ++j)
		{
      TableModel tm = _tableModel [j];

      /*
       * X
       */
      VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
      WVZInhalt x = verw.getWvzInhalt(_anzeigeTyp, tm.getSelectedX());

      /*
       * Schaltgrund
       */
      String nameSchaltgrund = tm.getSelectedSchaltgrund();
      
      Aspect schaltgrund = null;
      
      if (nameSchaltgrund.equals(_hand))
      	schaltgrund = _schaltgrundHand;
      else if (nameSchaltgrund.equals(_stau))
      	schaltgrund = _schaltgrundStau;
      else if (nameSchaltgrund.equals(_default))
      	schaltgrund = null;
      else
        schaltgrund = Tools.getAspekt(nameSchaltgrund);
      
      /*
       * Prioritt
       */
      int prio = tm.getSelectedPrioritaet();
      
      regeln.addRegel (x, prio, schaltgrund);
		}

		return regeln;
  }

	/**
	 * Methode bernimmt die Daten eines anderen Folders
	 * @param tvAnzeige Tableviewer der zu bernhmenden Tabelle
	 */
	public void setTvAnzeige (TableViewer tvAnzeige)
	{
    TableModel[] modelSet = (TableModel[]) tvAnzeige.getInput();

    Vector<RegelPriorisierungNeu> v = new Vector<RegelPriorisierungNeu>();

    for (int i=0; i<modelSet.length; ++i)
    {
      VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
      WVZInhalt x = verw.getWvzInhalt(_anzeigeTyp, modelSet [i].getSelectedX());

      String nameSchaltgrund = modelSet [i].getSelectedSchaltgrund();
      Aspect a = Tools.getAspekt(nameSchaltgrund);

      int prio = modelSet [i].getSelectedPrioritaet();

  	  v.add (new RegelPriorisierungNeu (x, prio, a));
    }
    
    _aktuelleParameter.setParameter(v);
    updateData(_aktuelleParameter);
  }
	
  private static abstract class TableColumnSorter extends ViewerComparator
  {
    public static final int ASC = 1;
    public static final int NONE = 0;
    public static final int DESC = -1;
    
    private int _direction = -1;
    private TableColumn _column;
    private TableViewer _tableViewer;
    
    public TableColumnSorter(TableViewer tv, TableColumn col)
    {
      this._column = col;
      this._tableViewer = tv;
      
      this._column.addSelectionListener( new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent e)
        {
          if ( TableColumnSorter.this._tableViewer.getComparator() != null )
          {
            if ( TableColumnSorter.this._tableViewer.getComparator() == TableColumnSorter.this )
            {
              int tdirection = TableColumnSorter.this._direction;
              if ( tdirection == ASC )
                setSorter( TableColumnSorter.this, DESC );
//              else if ( tdirection == DESC )
//                setSorter( TableColumnSorter.this, NONE );
              else
                setSorter( TableColumnSorter.this, ASC );
            }
          }
        }
      });
      
    }
  
    public void setSorter(TableColumnSorter sorter, int direction)
    {
      if ( direction == NONE )
      {
        _column.getParent().setSortColumn( null );
        _column.getParent().setSortDirection( SWT.NONE );
        _tableViewer.setComparator( null );
      }
      else
      {
        _column.getParent().setSortColumn( _column );
        sorter._direction = direction;
        if ( direction == ASC )
          _column.getParent().setSortDirection( SWT.DOWN );
        else
          _column.getParent().setSortDirection( SWT.UP ); 
        if ( _tableViewer.getComparator() == sorter )
          _tableViewer.refresh();
        else
          _tableViewer.setComparator( sorter );   
      }
    }
    
    public int compare(Viewer viewer, Object e1, Object e2)
    {
      return _direction * doCompare(viewer, e1, e2);
    }
    
    protected abstract int doCompare(Viewer TableViewer, Object e1, Object e2);
  
  }

  /**
   * Methode setzt die Parameter auf Defaultwerte
   */
	private void aktionDefault()
  {
		Prio [] prio = null;

		Vector<RegelPriorisierungNeu> regeln = new Vector<RegelPriorisierungNeu>();
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);

  	DefaultWerte dw = new DefaultWerte ();
  	prio = dw.getDefaultWerte(_anzeigeTyp);

//    if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A14)
//    {
//    	DefaultWerteA14 dw = new DefaultWerteA14();
//    	prio = dw.getDefaultWerte(_anzeigeTyp);
//    }
//
//    if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.B27)
//    {
//			DefaultWerteB27 dw = new DefaultWerteB27();
//			prio = dw.getDefaultWerte(_anzeigeTyp);
//    }
//
//    if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A8)
//    {
//    	DefaultWerteA8 dw = new DefaultWerteA8();
//    	prio = dw.getDefaultWerte(_anzeigeTyp);
//    }

		if (prio != null)
		{
	    int aktPrio = prio.length * 10;
	    
	    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
	    
	    for (int i=0; i<prio.length; ++i)
	    {
	  		WVZInhalt inhalt = verw.getWvzInhalt(_anzeigeTyp, prio [i].getInhalt().getBildInhalt());
		  	_aktuelleParameter.addRegel (inhalt, aktPrio, prio[i].getSchaltgrund());
		  	
		  	aktPrio -= 10;
	    }
		}
		
		updateData(_aktuelleParameter);

  }
}
