package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.steuerungskern;

import java.util.HashMap;
import java.util.Vector;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * Darstellung eines Composite mit unterschiedlichen Foldern fr die einzelnen
 * Anzeigetypen
 */
public class CompositeParameterAuswahl extends Composite
{ 
	/**
	 * Kennung Iterationen
	 */
	private final String _iterationen = "Iterationen/Zeitraster";

	/**
	 * Kennung Aspekte
	 */
	private final String _aspekte = "Aspekte";

	/**
	 * Kennung Anzeigen
	 */
	private final String _anzeigen = "Anzeigen";

  /**
   * Instanz mit den Foldern
   */
  private CTabFolder _tabFolder = null;

  /**
   * Composite
   */
  private Composite _composite;

  /**
   * Pid des auslsenden Objekts
   */
	private String _instanzSteuerungsKern;

	/**
	 * Aktuelle Parameter des Steuerungskerns
	 */
	private ParameterSteuerungsKern _parameter;

	/**
	 * Konstuktor der Klasse
	 * @param parent ausfrufendes Composite
	 * @param style Stil
	 * @param para aktuelle Parameter
	 * @param aktuell aktuelle Parameter
	 * @param instanz Instanz des Steuerungskerns
	 */
  public CompositeParameterAuswahl (Composite parent, int style, ParameterSteuerungsKern para, String instanz)
  {
    super (parent, style);
    
    _parameter = para;
    
    _instanzSteuerungsKern = instanz;
    
    initialize();
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);

    /*
     * Tabs anlegen fr die unterschiedlichen Parameterarten
     */
    
    Vector<String> parameterArten = new Vector<String>();
    
    parameterArten.add(_iterationen);
    parameterArten.add(_aspekte);
    parameterArten.add(_anzeigen);
//    parameterArten.add(_zeitraster);

    createCompositeTab();
    erzeugeTabItems (parameterArten);
    
    /*
     * Daten einblenden
     */
    
    for (String s : parameterArten)
      verarbeiteDaten(s);
  }
    
  /**
   * Methode legt das Composite mit den Foldern an
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder( _composite, SWT.NONE );
    _tabFolder.setLayout( new GridLayout() );
    _tabFolder.setLayoutData( gridData3 );

    _tabFolder.addSelectionListener( new SelectionListener()
    {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder) e.getSource();
        CTabItem itemSel = folder.getSelection();

        aktualisiereFolder(itemSel);
      }
      
      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }
    });
  }

  /**
   * Methode zum Aktualisieren aller Folder
   */
  public void aktualisiereAlleFolder ()
  {
    CTabItem[] items = _tabFolder.getItems();
    for (CTabItem itemSel : items)
    	aktualisiereFolder(itemSel);
  }

  /**
   * Methode zum Aktualisieren eines Folder
   * @param itemSel selektierter Folder
   */
  public void aktualisiereFolder (CTabItem itemSel)
  {
    Composite cSel = (Composite) itemSel.getControl();
    
    if ( cSel.getChildren() != null && cSel.getChildren().length > 0 )
    {
      String pid = (String)itemSel.getData(itemSel.getText());
      
    	if (pid.equals(_iterationen))
    	{
    		CompositeIterationen compParSel = (CompositeIterationen) cSel.getChildren()[0];
    		int anzahl = compParSel.getAnzahlTvAnzeige();
    		for (int i=0; i<anzahl; ++i)
    		{
  	      TableViewer tvParameter = compParSel.getTvAnzeige(i);          
	        tvParameter.refresh();
    		}
    	}
    		      
    	if (pid.equals(_aspekte))
    	{
    		CompositeAspekte compParSel = (CompositeAspekte) cSel.getChildren()[0];
    		compParSel.refresh();
    	}

    	if (pid.equals(_anzeigen))
    	{
    		CompositeAnzeigen compParSel = (CompositeAnzeigen) cSel.getChildren()[0];
    		compParSel.refresh();
    	}

    }
  }
  
  /**
   * Methode erzeugt die Tabs fr die einzelnen Parameterarten
   * @param parameterArten Parameterarten
   */
  public void erzeugeTabItems( Vector<String> parameterArten )
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    
    for (String s : parameterArten)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );
      tabItem.setText( s );
      tabItem.setData( s, s );

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }
  
  /**
   * Methode blendet die aktuellen Parameter in einem Foldern ein
   * @param parameterArt Parameterart
   */
  private void verarbeiteDaten (String parameterArt)
  {
    CTabItem[] items = _tabFolder.getItems();
    
    // Tab zum Systemobject ermitteln
    
    CTabItem item = null;
    for (CTabItem ti : items)
    {
      String pid = (String)ti.getData(ti.getText());

      if (pid.equals(parameterArt))
      {
        item = ti;
        break;
      }
    }

    // Tab gefunden
    
    if (item != null)
    {
      Composite c = (Composite)item.getControl();
      Control[] children = c.getChildren();

      // Tab neu anlegen, wenn noch leer
      
      if (children.length == 0)
      {
      	if (parameterArt.equals(_iterationen))
      	{
          CompositeIterationen compPar = new CompositeIterationen ( c, SWT.None, _parameter );
          compPar.layout( true );
          c.layout( true );
      	}
      	
      	if (parameterArt.equals(_aspekte))
      	{
          CompositeAspekte compPar = new CompositeAspekte( c, SWT.NONE, _parameter );
          compPar.layout( true );
          c.layout( true );
      	}

      	if (parameterArt.equals(_anzeigen))
      	{
          CompositeAnzeigen compPar = new CompositeAnzeigen( c, SWT.NONE, _parameter );
          compPar.layout( true );
          c.layout( true );
      	}
      }
    }
  }

  /**
   * Methode liefert die aktuell eingestellten Parameter
   * @return aktuell eingestellte Parameter
   */
	public ParameterSteuerungsKern getParameter()
  {
		ParameterSteuerungsKern para = new ParameterSteuerungsKern();
		
	  CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
      String pid = (String)item.getData(item.getText());
      
      /*
       * Anzahl Iterationen
       */
      if (pid.equals(_iterationen))
      {
	      Composite c = (Composite)item.getControl();
	      CompositeIterationen compPar = (CompositeIterationen) c.getChildren()[0];
	      
	      ParameterSteuerungsKern p = compPar.getParameter();
	      
	      para.setAnzahlIterationen(p.getAnzahlIterationen());
	      para.setParaZeitraster(p.getParaZeitraster());
      }
      
      /*
       * Aspekte
       */
      if (pid.equals(_aspekte))
      {
	      Composite c = (Composite)item.getControl();
	      CompositeAspekte compPar = (CompositeAspekte) c.getChildren()[0];
	      
	      ParaAspekte p = compPar.getParameter();
	      
	      para.setParaAspekte(p);
      }

      /*
       * Anzeigen
       */
      if (pid.equals(_anzeigen))
      {
	      Composite c = (Composite)item.getControl();
	      CompositeAnzeigen compPar = (CompositeAnzeigen) c.getChildren()[0];
	      
	      ParaAnzeigen p = compPar.getParameter();
	      
	      
	      
	      para.setParaAnzeigen (p);
      }

	  }
	  
	  return para;
  }
	
  /**
   * Methode liefert die Tabellenviewer der einzlnen Tabellen	
   * @return Tabellenviewer der einzlnen Tabellen
   */
	public HashMap<String, TableViewer> getTableViewer ()
	{
		HashMap<String, TableViewer> tableViewer = new HashMap<String, TableViewer>();

		CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
//	    Composite c = (Composite)item.getControl();
//	    CompositeParameterZusammenfassen compPar = (CompositeParameterZusammenfassen) c.getChildren()[0];
//	    
//	    TableViewer viewer = compPar.getTvAnzeige();
//      String pid = (String)item.getData(item.getText());
//      
//      tableViewer.put(pid, viewer);
	  }
		
		return tableViewer;
	}

	/**
	 * Methode zum bernehmen der Tabellenviewer der einzelnen Tabellen
	 * @param tableViewer zu bernehmende Viewer
	 */
	public void setTableViewer (HashMap<String, TableViewer> tableViewer)
	{
		CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
//	    Composite c = (Composite)item.getControl();
//	    CompositeParameterZusammenfassen compPar = (CompositeParameterZusammenfassen) c.getChildren()[0];
//	    
//      String pid = (String)item.getData(item.getText());
//	    TableViewer viewer = tableViewer.get(pid);
//	    
//	    if (viewer != null)
//	    	compPar.setTvAnzeige(viewer);
	  }
	}

}
