package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix;

import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemein.Design;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemein.EVA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemein.Spalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemein.TableModelVerriegelung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Versorgung der Parameter
 */
public class CompositeAVM extends Composite implements ICompositeParameterAllgemeineVerriegelungsMatrix
{
	/**
	 * Aktuelles Composite
	 */
  private Composite _composite;

  /**
   * Composite der Tabelle
   */
  private Composite _compositeTabelle;

  /**
   * Gruppe mit den Regeln
   */
  private Group _groupRegeln;

  /**
   * Tabelviewer der Tabelle
   */
  private TableViewer _tvAnzeige;

  /**
   * Aktuelle Parameter
   */
  private ListeMitRegeln _aktuelleParameter;

  /**
   * Composite fr die Buttons
   */
	private Composite _compositeButtons;

	/**
	 * Button Hinzufgen
	 */
	private Button _buttonHinzufuegen;

	/**
	 * Button Lschen
	 */
	private Button _buttonLoeschen;

	/**
	 * Tabellenmodell
	 */
	private TableModelVerriegelung[] _tableModel;
	
	/**
	 * Breite der Spalten mit den WVZ Inhalten
	 */
	private int _breiteSpalte = 60;

	private Button _buttonAlleLoeschen;

	private Design _design;

  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil des Fenstern
   * @param aktuell Aktuelle Parameter
   * @param typ Typ des Anzeigequerschnitts
   */
  public CompositeAVM (Composite parent, int style, ListeMitRegeln aktuell, Design design)
  {
    super (parent, style);
    
    this._aktuelleParameter = erzeugeKopie(aktuell);
    
    _design = design;
    
    initialize();
  }

  /**
   * Methode legt eine Arbeitskopie der akutellen Daten an
   * @param aktuell aktuelle Daten
   * @return Arbeitskopie
   */
  private ListeMitRegeln erzeugeKopie(ListeMitRegeln aktuell)
  {
	  ListeMitRegeln kopie = new ListeMitRegeln();
	  
	  Vector<RegelAllgemeineVerriegelungsMatrix> parameter = aktuell.getParameter();
	  for (RegelAllgemeineVerriegelungsMatrix regel : parameter)
	  	kopie.addRegel(regel.getZuPruefendeZeichenFolge(),
	  			           regel.getRichtung(),
	  			           regel.getSchaltgrundRegel());
	  
	  return kopie;
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
    //Initialisierungen der Arrays
    
    erzeugeGruppeRegeln();
    
    Table table = _tvAnzeige.getTable();
    _tableModel = createModel1(table);
    
    if ( _tableModel != null )
      _tvAnzeige.setInput(_tableModel); 
    
    createButtons();
    
    updateData(_aktuelleParameter);
  }
  
  /**
   * Methode bernimmt die Daten und zeigt diese in der Tabelle an
   */
  private void updateData (ListeMitRegeln parameter)
  {
  	int anzahlParameter = parameter.getParameter().size();
  	
  	erzeugeTabellenModell (anzahlParameter);
  	
  	for (int j=0; j<anzahlParameter; ++j)
  	{
      TableModelVerriegelung tm = _tableModel [j];    	
    	RegelAllgemeineVerriegelungsMatrix aktuell = parameter.getParameter().get(j);
    	
    	/*
    	 * Lschen
    	 */
    	tm.setSelectedLoeschen("false");
    	
    	/*
    	 * Schaltgrund
    	 */
    	
    	if (aktuell.getSchaltgrundRegel() == null)
    	{
    	  tm.setSelectedSchaltgrund("");
    	}
    	else
    	{
  	    tm.setSelectedSchaltgrund(aktuell.getSchaltgrundRegel().getName());
    	}
    	
    	/*
    	 * Richtung
    	 */
  	  tm.setSelectedRichtung(aktuell.getRichtung().getName());

  	  /*
  	   * Definition der Reihenfolge der Anzeigen (muss bereinstimmen mit der
  	   * Vorgnger/Nachfolger Beziehung in Querrichtung
  	   */
  	  Vector<VersorgungsElement> elemente = new Vector<VersorgungsElement>();

  	  for (Spalte s : _design.getSpalten())
  	  {
  	  	if (s.getEva() == EVA.A)
  	  	{
  	      elemente.add (new VersorgungsElement((Label) tm.getA (s.getIndex()).getA(), AnzeigeElementTyp.WZG_A));
  	  	}

  	  	if (s.getEva() == EVA.AD)
  	  	{
  	      elemente.add (new VersorgungsElement((Label) tm.getAD (s.getIndex()).getA(), AnzeigeElementTyp.WZG_A));
  	      elemente.add (new VersorgungsElement((Label) tm.getAD (s.getIndex()).getD(), AnzeigeElementTyp.WZG_D));
  	  	}

  	  	if (s.getEva() == EVA.BC)
  	  	{
  	  	  elemente.add (new VersorgungsElement((Label) tm.getBC(s.getIndex()).getB(),  AnzeigeElementTyp.WZG_B));
  	  	  elemente.add (new VersorgungsElement((Label) tm.getBC(s.getIndex()).getC(),  AnzeigeElementTyp.WZG_C));
  	  	}
  	  	
  	  	if (s.getEva() == EVA.BLINKER_BC)
  	  	{
	  	    elemente.add (new VersorgungsElement((Label) tm.getBlinkerBC(s.getIndex()).getBlinker(),  AnzeigeElementTyp.BLINKER));
	  	    elemente.add (new VersorgungsElement((Label) tm.getBlinkerBC(s.getIndex()).getB(),        AnzeigeElementTyp.WZG_B));
	  	    elemente.add (new VersorgungsElement((Label) tm.getBlinkerBC(s.getIndex()).getC(),        AnzeigeElementTyp.WZG_C));
  	  	}
  	  	
  	  	if (s.getEva() == EVA.A_BLINKER_BC)
  	  	{
  	  	  elemente.add (new VersorgungsElement((Label) tm.getABlinkerBC(s.getIndex()).getA(),        AnzeigeElementTyp.WZG_A));
  	  	  elemente.add (new VersorgungsElement((Label) tm.getABlinkerBC(s.getIndex()).getBlinker(),  AnzeigeElementTyp.BLINKER));
  	  	  elemente.add (new VersorgungsElement((Label) tm.getABlinkerBC(s.getIndex()).getB(),        AnzeigeElementTyp.WZG_B));
  	  	  elemente.add (new VersorgungsElement((Label) tm.getABlinkerBC(s.getIndex()).getC(),        AnzeigeElementTyp.WZG_C));
  	  	}

  	  	if (s.getEva() == EVA.T)
  	  	{
  	      elemente.add (new VersorgungsElement((Label) tm.getTAQ (s.getIndex()).getTAQ(), AnzeigeElementTyp.WZG_TAQ));
  	  	}

  	  	if (s.getEva() == EVA.BLINKER_T)
  	  	{
  	      elemente.add (new VersorgungsElement((Label) tm.getBlinkerTAQ(s.getIndex()).getBlinker(), AnzeigeElementTyp.BLINKER));
   	      elemente.add (new VersorgungsElement((Label) tm.getBlinkerTAQ(s.getIndex()).getTAQ(),     AnzeigeElementTyp.WZG_TAQ));
   	    }
  	  }

    	int index = 0;

  	  for (VersorgungsElement e : elemente)
  	  {
    	  WVZInhalt inhalt = VerwaltungWvzInhalte.getInstance().getBeliebig();
    	  
    	  /*
    	   * Sonderfall Blinker: in der Erstversorgung haben die Blinker gefehlt um ein Einlesen
    	   * dieser fehlerhafte Daten zu ermglichen wurde dieser Spezialfall eingebaut, kann
    	   * spter entfallen
    	   */
    	  if (e.getTyp() == AnzeigeElementTyp.BLINKER)
    	  {
          if (aktuell.getZuPruefendeZeichenFolge().size() == elemente.size())
    	  	  inhalt = aktuell.getZuPruefendeZeichenFolge().get(index++);
    	  }
    	  else
    	  	inhalt = aktuell.getZuPruefendeZeichenFolge().get(index++);
    	  
    	  Tools.setzeBildInhalte(e.getLabel(), inhalt, e.getTyp(), true);
  	  }
    }
  	
  	_buttonAlleLoeschen.setEnabled (anzahlParameter != 0);
  	pruefeObLoeschenAngewaehlt();
  }

  /**
   * Methode erzeugt ein Tabellenmodell einer bestimmten Gre. Dabei werden bereits vorhandene
   * Composite in der Tabelle weiter verwendet
   * @param anzahlParameter Gre des erzeugten Tabellenmodells
   */
  private void erzeugeTabellenModell (int anzahlParameter)
  {
  	/*
  	 * Hat sich die Gre gendert
  	 */
  	if (_tableModel.length != anzahlParameter)
  	{
  		/*
  		 * Tabelle erweitern
  		 */
  		if (_tableModel.length < anzahlParameter)
  		{
  		  Vector<TableModelVerriegelung> v = new Vector<TableModelVerriegelung>();
  		  
  		  for (int i=0; i<_tableModel.length; ++i)
  		  	v.add(_tableModel [i]);
  		  
  		  for (int i=_tableModel.length; i<anzahlParameter; ++i)
  		  	v.add (erzeugeZeileInTabelle(_tvAnzeige.getTable()));
  		  
  		  _tableModel = new TableModelVerriegelung [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		/*
  		 * Tabelle reduzieren
  		 */
  		else if (_tableModel.length > anzahlParameter)
  		{
  			/*
  			 * Nicht mehr bentigte Elemente lschen
  			 */
  			
  			for (int i = _tableModel.length - 1; i > anzahlParameter - 1; i--)
  			{
  				for (int j=_design.getSpalten().length - 1; j >= 0; j--)
  				{
  					Spalte s = _design.getSpalten()[j];
  					
  					if (s.getEva() == EVA.A)
  	  				_tableModel [i].getA(s.getIndex()).dispose();

  					if (s.getEva() == EVA.AD)
  	  				_tableModel [i].getAD(s.getIndex()).dispose();

  					if (s.getEva() == EVA.BC)
  						_tableModel [i].getBC(s.getIndex()).dispose();

  					if (s.getEva() == EVA.BLINKER_BC)
  						_tableModel [i].getBlinkerBC(s.getIndex()).dispose();

  					if (s.getEva() == EVA.A_BLINKER_BC)
  						_tableModel [i].getABlinkerBC(s.getIndex()).dispose();
  					
  					if (s.getEva() == EVA.T)
  	  				_tableModel [i].getTAQ(s.getIndex()).dispose();

  					if (s.getEva() == EVA.BLINKER_T)
  	  				_tableModel [i].getBlinkerTAQ(s.getIndex()).dispose();

  				}
  				
  				_tableModel [i].getRichtung().dispose();
  				_tableModel [i].getSchaltgrund().dispose();
  				_tableModel [i].getLoeschen().dispose();
  			}
  				
  		  Vector<TableModelVerriegelung> v = new Vector<TableModelVerriegelung>();
  		  for (int i=0; i<anzahlParameter; ++i)
  		  	v.add(_tableModel [i]);

  		  _tableModel = new TableModelVerriegelung [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		
  	  _tvAnzeige.setInput(_tableModel);
  	  _tvAnzeige.getTable().layout(true);
  	}
  }
  
  /**
   * Methode erzeugt die Gruppe mit den Regeln
   */
  private void erzeugeGruppeRegeln()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupRegeln = new Group(_composite, SWT.NONE);
    _groupRegeln.setText("Regeln");
    
    _groupRegeln.setLayout(fillLayout1);
    _groupRegeln.setLayoutData(gridData2);

    erzeugeTabelle();
  }

  /**
   * Methode erzeugt das Composite mit den Buttons
   */
  private void createButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeButtons = new Composite( _composite, SWT.NONE );
    _compositeButtons.setLayoutData( gridData3 );
    _compositeButtons.setLayout( rowLayout );

    _buttonHinzufuegen = new Button( _compositeButtons, SWT.NONE );
    _buttonHinzufuegen.setText( "Regel hinzufgen" );
    _buttonHinzufuegen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelHinzufuegen();
      }
    } );

    _buttonLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonLoeschen.setText( "Selektierte Regeln loeschen" );
    _buttonLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelLoeschen();
      }

    } );
    _buttonLoeschen.setEnabled(false);

    _buttonAlleLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonAlleLoeschen.setText( "Alle Regeln loeschen" );
    _buttonAlleLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        aktionAlleRegelLoeschen();
      }
    } );

  }

	/**
   * Methode zum Lschen aller Meldungen
   */
  protected void aktionAlleRegelLoeschen()
  {
		Vector<RegelAllgemeineVerriegelungsMatrix> regeln = new Vector<RegelAllgemeineVerriegelungsMatrix>();
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }

  /**
   * Methode prft ob Regeln zum Lschen ausgewhlt sind
   */
  private void pruefeObLoeschenAngewaehlt ()
  {
  	boolean angewaehlt = false;
  	
		for (TableModelVerriegelung tm : _tableModel)
			if (tm.getSelectedLoeschen().equals("true"))
				angewaehlt = true;
		
		_buttonLoeschen.setEnabled(angewaehlt);
  }
  
  /**
   * Methode zum Lschen der Regeln
   */
	private void aktionRegelLoeschen()
  {
		Vector<RegelAllgemeineVerriegelungsMatrix> regeln = leseAktuelleEinstellungen().getParameter();
    Vector<RegelAllgemeineVerriegelungsMatrix> loeschen = new Vector<RegelAllgemeineVerriegelungsMatrix>();
    
    for (int i=0; i<_tableModel.length; ++i)
    	if (!_tableModel [i].getSelectedLoeschen().equals("false"))
    		loeschen.add(regeln.get(i));

    for (RegelAllgemeineVerriegelungsMatrix r : loeschen)
    {
    	regeln.remove(r);
    }
    
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }

	/**
	 * Aktion Regel hinzufgen
	 */
	private void aktionRegelHinzufuegen()
  {
		Vector<RegelAllgemeineVerriegelungsMatrix> regeln = leseAktuelleEinstellungen().getParameter();
    _aktuelleParameter.setParameter(regeln);

		WVZInhalt beliebig = VerwaltungWvzInhalte.getInstance().getBeliebig();
		Aspect a = VerwaltungParaAllgemeineVerriegelungsMatrix.getInstance().getSchaltgrundAllgemein();
		
		Vector<WVZInhalt> v = new Vector<WVZInhalt>();
		
		for (Spalte s : _design.getSpalten())
		{ 
			for (int i=0; i<s.getEva().getAnzahlAnzeigen(); ++i)
				v.add(beliebig);
		}
		
		_aktuelleParameter.addRegel(v, ArbeitsRichtungVN.NACHFOLGER, a);
		
		updateData(_aktuelleParameter);
  }

	/**
	 * Methode erzeugt das Composite mit der Tabelle
	 */
  private void erzeugeTabelle ()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    
    _compositeTabelle = new Composite(_groupRegeln, SWT.None);
    _compositeTabelle.setLayout(fillLayout1);
    
    _tvAnzeige = new TableViewer(_compositeTabelle, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());

    TableViewerColumn column0 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column0.getColumn().setWidth(70);
    column0.getColumn().setText("Lschen");
    column0.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column1.getColumn().setWidth(150);
    column1.getColumn().setText("Schaltgrund");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column2.getColumn().setWidth(150);
    column2.getColumn().setText("Richtung");
    column2.setLabelProvider(new MyCellLabelProvider());

    
    for (int i=0; i<_design.getSpalten().length; ++i)
    {
    	Spalte s = _design.getSpalten()[i];
    	
	    TableViewerColumn column3 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
	    column3.getColumn().setWidth(_breiteSpalte);
	    column3.getColumn().setText(s.getText());
	    column3.setLabelProvider(new MyCellLabelProvider());
    }

    // resize the row height using a MeasureItem listener
    
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
          	
          	int hoehe = 90;  // Default
          	
          	for (Spalte s : _design.getSpalten())
          		if (s.getEva() == EVA.A_BLINKER_BC)
          			hoehe = 140;
          	
            event.height = hoehe;
            break;

          default:
        }

      }
    };
    
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }

  /**
   * Klasse realsiert den TableContentProvider fr die Tabelle
   * @author ObertM
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
  	/**
  	 * Tabelviewer
  	 */
    private TableViewer tableViewer;

    /**
     * Methode liefert die Elemente der Tabelle
     */
    public Object[] getElements(Object inputElement)
    {
			Object[] re = null;

			if (inputElement instanceof TableModelVerriegelung [])
			{
        re = (TableModelVerriegelung[]) inputElement;
			}
			
			return re;
    }

    /**
     * Dispose
     */
    public void dispose()
    {
    }

    /**
     * Methode wird aufgerufen wenn sich der Inhalt der Tabelle gendert hat
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModelVerriegelung[] tms = (TableModelVerriegelung[])newInput;

        if (tms != null)
        {
          for (TableModelVerriegelung tm : tms)
            tm.deleteObserver(this);
        }
      }
      
      if (newInput != null)
      {
        TableModelVerriegelung[] tms = (TableModelVerriegelung[])newInput;
        if (tms != null)
        {
          for (TableModelVerriegelung tm : tms)
            tm.addObserver(this);
        }
      }
    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();
      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * @param table
   * @return TabellenModell
   */
  private TableModelVerriegelung[] createModel1 (Table table)
  {
    int tabLength = getAnzahlParameter();
    int elIndex   = 0;
    
    TableModelVerriegelung[] elements = new TableModelVerriegelung [tabLength];
    
    if (elements.length == 0)
    	return elements;

    for (int x=0; x<getAnzahlParameter(); ++x)
    {
	    elements[elIndex] = erzeugeZeileInTabelle(table);
	    elIndex++;
    }
    
    return elements;
  }

  /**
   * Methode erzeugt ein Tabellenmodell fr eine Zeile in der Tabelle
   * @param table Tabelle 
   * @return Tabellenmodell fr eine Zeile
   */
  private TableModelVerriegelung erzeugeZeileInTabelle (Table table)
  {
    /*
     * Layout festlegen
     */
  	
    GridData gdL = new GridData();
    gdL.horizontalAlignment = GridData.CENTER;
    gdL.verticalAlignment = GridData.CENTER;
    gdL.grabExcessHorizontalSpace = true;
    gdL.grabExcessVerticalSpace = true;

    GridData gdSG = new GridData();
    gdSG.horizontalAlignment = GridData.CENTER;
    gdSG.verticalAlignment = GridData.CENTER;
    gdSG.grabExcessHorizontalSpace = true;
    gdSG.grabExcessVerticalSpace = true;

    GridData gdR = new GridData();
    gdR.horizontalAlignment = GridData.CENTER;
    gdR.verticalAlignment = GridData.CENTER;
    gdR.grabExcessHorizontalSpace = true;
    gdR.grabExcessVerticalSpace = true;
    
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    /*
     * Button Lschen
     */
    Composite compL = new Composite( table, SWT.NONE );
    compL.setBackground( table.getBackground() );
    compL.setBackgroundMode( SWT.INHERIT_DEFAULT );
    compL.setLayoutData( gdL );
    compL.setLayout( new GridLayout() );
    Button b = new Button( compL, SWT.CHECK );
    
    b.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				pruefeObLoeschenAngewaehlt();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
    
    /*
     * Combobox Schaltgrund    
     */
    Composite compSG = new Composite( table, SWT.NONE );
    compSG.setBackground( table.getBackground() );
    compSG.setBackgroundMode( SWT.INHERIT_DEFAULT );
    compSG.setLayoutData( gdSG );
    compSG.setLayout( new GridLayout() );
    
    Combo combo1 = new Combo( compSG, SWT.BORDER );
    combo1.setLayoutData( gdTableItem );
    
    Vector<SystemObject> aspekte = Tools.getAspekte();
    String [] sa = new String [aspekte.size()];
    for (int i=0; i<aspekte.size(); ++i)
      sa [i] = aspekte.get(i).getName();
    
    combo1.setItems(sa);
    combo1.select(0);

    /*
     * Combobox Richtung    
     */
    Composite compR = new Composite( table, SWT.NONE );
    compR.setBackground( table.getBackground() );
    compR.setBackgroundMode( SWT.INHERIT_DEFAULT );
    compR.setLayoutData( gdR );
    compR.setLayout( new GridLayout() );
    
    Combo combo2 = new Combo( compR, SWT.BORDER );
    combo2.setLayoutData( gdTableItem );
    
    ArbeitsRichtungVN[] arvn = Richtung.ArbeitsRichtungVN.values();
    String[] r = new String [arvn.length];
    
    for (int i=0; i<arvn.length; ++i)
    	r [i] = arvn [i].getName();
    
    combo2.setItems(r);
    combo2.select(0);

    Vector<Composite> composites = new Vector<Composite>();
    
    for (Spalte s : _design.getSpalten())
    {
    	if (s.getEva() == EVA.A)
        composites.add(new CompositeA          (table, SWT.NONE));

    	if (s.getEva() == EVA.AD)
        composites.add(new CompositeAD         (table, SWT.NONE));

    	if (s.getEva() == EVA.BC)
        composites.add(new CompositeBC         (table, SWT.NONE));

    	if (s.getEva() == EVA.BLINKER_BC)
        composites.add(new CompositeBlinkerBC  (table, SWT.NONE));

    	if (s.getEva() == EVA.A_BLINKER_BC)
        composites.add(new CompositeABlinkerBC (table, SWT.NONE));
    	
    	if (s.getEva() == EVA.T)
        composites.add(new CompositeTAQ        (table, SWT.NONE));

    	if (s.getEva() == EVA.BLINKER_T)
        composites.add(new CompositeBlinkerTAQ (table, SWT.NONE));
    }

    return new TableModelVerriegelung ( compL, compSG, compR, composites );
  }
  
  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public int getAnzahlParameter()
  {
    return _aktuelleParameter.getParameter().size();
  }
  
  /**
   * Methode liefert den Tabelviewer
   * @return
   */
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }

  /**
   * Renderer fr die Tabelle
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider 
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

      int cnt = cell.getColumnIndex();

      TableModelVerriegelung model = (TableModelVerriegelung)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getLoeschen();
          break;
          
        case 1:
          comp = model.getSchaltgrund();
          break;
          
        case 2:
          comp = model.getRichtung();
          break;
          
        default:
        	if (cnt >= 3 && cnt < 3 + _design.getSpalten().length)
        	{
        		int index = cnt - 3;
        		
        		Spalte s = _design.getSpalten()[index];
        		
        		if (s.getEva() == EVA.A)
        	    comp = model.getA (s.getIndex());		

        		if (s.getEva() == EVA.AD)
        	    comp = model.getAD (s.getIndex());		

        		if (s.getEva() == EVA.BC)
        	    comp = model.getBC (s.getIndex());

        		if (s.getEva() == EVA.BLINKER_BC)
        	    comp = model.getBlinkerBC (s.getIndex());
        		
        		if (s.getEva() == EVA.A_BLINKER_BC)
        	    comp = model.getABlinkerBC (s.getIndex());		

        		if (s.getEva() == EVA.T)
        	    comp = model.getTAQ (s.getIndex());		

        		if (s.getEva() == EVA.BLINKER_T)
        	    comp = model.getBlinkerTAQ (s.getIndex());		
        	}
        	
        	break;
        	
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);
    }
  }

  /**
   * Methode liefert die eingestellten Parameter
   * @return eingestellte Parameter
   */
	public ListeMitRegeln getParameter()
  {
 		aktionRegelLoeschen();
 		return leseAktuelleEinstellungen();
  }
		
	/**
	 * Methode liest die aktuell eingestellten Parameter
	 * @return aktuell eingestellte Parameter
	 */
	private ListeMitRegeln leseAktuelleEinstellungen ()
	{
		ListeMitRegeln regeln = new ListeMitRegeln();
		
		for (int j=0; j<_tableModel.length; ++j)
		{
      TableModelVerriegelung tm = _tableModel [j];
      
      RegelAllgemeineVerriegelungsMatrix regel = bildeRegelAusTabellenModell(tm);
      regeln.addRegel(regel);
		}

		return regeln;
  }

	/**
	 * Methode bildet aus dem Tabellenmodell eine Regel
	 * @param tm Tabellenmodell
	 * @return Regel
	 */
	private RegelAllgemeineVerriegelungsMatrix bildeRegelAusTabellenModell (TableModelVerriegelung tm)
	{
		System.out.println("CompositeAVMB27Typ1.bildeRegelAusTabellenModell()");
		
    /*
     * Schaltgrund
     */
    
    String nameSchaltgrund = tm.getSelectedSchaltgrund();
    Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);
    
    /*
     * Richtung 
     */
    String nameRichtung = tm.getSelectedRichtung();
    ArbeitsRichtungVN richtung = ArbeitsRichtungVN.getRichtung (nameRichtung);

    Vector<InhaltMitTyp> inhalte = new Vector<InhaltMitTyp>();
    
    for (Spalte s : _design.getSpalten())
    {
    	if (s.getEva() == EVA.A)
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_A,   tm.getSelectedA (s.getIndex())));

    	if (s.getEva() == EVA.AD)
    	{
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_A,   tm.getSelectedAD_A (s.getIndex())));
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_D,   tm.getSelectedAD_D (s.getIndex())));
    	}

    	if (s.getEva() == EVA.BC)
    	{
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_B,   tm.getSelectedBC_B  (s.getIndex())));
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_C,   tm.getSelectedBC_C  (s.getIndex())));
    	}

    	if (s.getEva() == EVA.BLINKER_BC)
    	{
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.BLINKER, tm.getSelectedBlinkerBC_Blinker (s.getIndex())));
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_B,   tm.getSelectedBlinkerBC_B       (s.getIndex())));
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_C,   tm.getSelectedBlinkerBC_C       (s.getIndex())));
    	}
    	
    	if (s.getEva() == EVA.A_BLINKER_BC)
    	{
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_A,   tm.getSelectedABlinkerBC_A       (s.getIndex())));
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.BLINKER, tm.getSelectedABlinkerBC_Blinker (s.getIndex())));
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_B,   tm.getSelectedABlinkerBC_B       (s.getIndex())));
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_C,   tm.getSelectedABlinkerBC_C       (s.getIndex())));
    	}

    	if (s.getEva() == EVA.T)
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_TAQ, tm.getSelectedTAQ (s.getIndex())));

    	if (s.getEva() == EVA.BLINKER_T)
    	{
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.BLINKER, tm.getSelectedBlinkerTAQ_Blinker (s.getIndex())));
    		inhalte.add(new InhaltMitTyp(AnzeigeElementTyp.WZG_TAQ, tm.getSelectedBlinkerTAQ_TAQ     (s.getIndex())));
    	}

    }
    
    System.out.println(inhalte.size());
    for (InhaltMitTyp imt : inhalte)
    	System.out.println(imt.getTyp().getName() + " - " + imt.getInhalt());
    
		Vector<WVZInhalt> wvzInhalte = VerwaltungWvzInhalte.getInstance().getWvzInhalte(inhalte);

		for (WVZInhalt x : wvzInhalte)
			System.out.println(x.toString());
		
    return new RegelAllgemeineVerriegelungsMatrix (schaltgrund, richtung, wvzInhalte);
	}
	
	/**
	 * Methode bernimmt die Daten eines anderen Folders
	 * @param tvAnzeige Tableviewer der zu bernhmenden Tabelle
	 */
	public void setTvAnzeige (TableViewer tvAnzeige)
	{
		System.out.println("CompositeAVM.setTvAnzeige()");
		
    TableModelVerriegelung[] modelSet = (TableModelVerriegelung[]) tvAnzeige.getInput();

    Vector<RegelAllgemeineVerriegelungsMatrix> v = new Vector<RegelAllgemeineVerriegelungsMatrix>();

    for (int i=0; i<modelSet.length; ++i)
    {
    	v.add(bildeRegelAusTabellenModell (modelSet [i]));
    }
    
    _aktuelleParameter.setParameter(v);
    updateData(_aktuelleParameter);
  }
	
  /**
   * Methode zum Aktualisieren des Folders
   */
  public void aktualisiereFolder ()
  {
    TableViewer tvParameter = _tvAnzeige;
    tvParameter.refresh();
  }
}
