package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.EAnzeigeQuerschnittTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen.SBA;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Verriegelungsmatrix", Regel "Allgemeine Verriegelungsmatrix".<br>  
 * @author Dambach Werke GmbH
 */
public class RegelAllgemeineVerriegelungsMatrix
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;
  
  /**
   * arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   */
  private ArbeitsRichtungVN m_richtung = ArbeitsRichtungVN.VORGAENGER;
  
  /**
   * Kennung fr die Regel
   */
  private String m_kennung = "";
  
  /**
   * Liste mit der zu prfenden Zeichenfolge (in der Regel: "X", "Y", "Z",  ), die
   * Lnge der Sequenz ergibt sich aus der Anzahl der Elemente
   */
  private Vector<WVZInhalt> m_zuPrfendeZeichenFolge = new Vector<WVZInhalt>();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelAllgemeineVerriegelungsMatrix ()
  {
  }
  
  /**
   * Konstruktor der Klasse
   * @param schaltgrundRegel Regelspezifischer Schaltgrund
   * @param richtung arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   * @param inhalte zu prfende Zeichen
   */
  public RegelAllgemeineVerriegelungsMatrix(Aspect schaltgrundRegel, ArbeitsRichtungVN richtung,
      Vector<WVZInhalt> inhalte)
  {
  	m_schaltgrundRegel = schaltgrundRegel;
  	
  	m_richtung = richtung;
  	
  	m_zuPrfendeZeichenFolge = inhalte;
  }

	/**
   * Methode prft ob die Regel in Vorgngerrichtung arbeitet
   * @return Regel arbeitet in Vorgngerrichtung
   */
  public boolean isRichtungVorgaenger()
  {
    return m_richtung == ArbeitsRichtungVN.VORGAENGER;
  }

  /**
   * Methode setzt die Variable m_richtung
   * @param richtung Wert auf den die Variable m_richtung gesetzt werden soll
   */
  public void setRichtung ( ArbeitsRichtungVN richtung )
  {
    m_richtung = richtung;
  }
  
  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaRegelAllgemeineVerriegelungsMatrix'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    String vorgaengerNachfolger = data.getItem( "VorgngerNachfolger" ).asTextValue().getText();
    
    if (vorgaengerNachfolger.equals( "Vorgnger" ))
      setRichtung (ArbeitsRichtungVN.VORGAENGER);
    else
      setRichtung (ArbeitsRichtungVN.NACHFOLGER);
    
    m_zuPrfendeZeichenFolge.clear();
    
    try
    {
      setKennung(data.getTextValue("Kennung").getText());
    }
    catch (Exception e) 
    {
    	if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A14)
        setKennung(EAnzeigeQuerschnittTyp.A14_TYP1.getName());
		}
    
    Array a = data.getArray( "Zeichenfolge" );
    int anzahl = a.getLength();
    
    for (int x = 0; x < anzahl; ++x)
    {
      WVZInhalt wi = new WVZInhalt();
      wi.auswerteAttributgruppe( a.getItem( x ) );
      
      m_zuPrfendeZeichenFolge.add( wi );
      
    }
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck 
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel ) 
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "Richtung: " + m_richtung + "\n";
    s += "Zu prfende Zeichen:\n";
    for (WVZInhalt wi : m_zuPrfendeZeichenFolge)
      s += "-" +wi.toString();

    return s;
  }

  /**
   * Methode liefert die Richtung in die Regelklasse arbeitet.
   * @return Richtung
   */
  public ArbeitsRichtungVN getRichtung()
  {
    return m_richtung;
  }

  /**
   * Methode liefert die Variable m_zuPrfendeZeichenFolge zurck
   * @return Variable m_zuPrfendeZeichenFolge
   */
  public Vector<WVZInhalt> getZuPruefendeZeichenFolge()
  {
    return m_zuPrfendeZeichenFolge;
  }

  /**
   * Methode setzt die Variable m_zuPrfendeZeichenFolge
   * @param zuPrfendeZeichenFolge Wert auf den die Variable m_zuPrfendeZeichenFolge gesetzt werden soll
   */
  public void setZuPrfendeZeichenFolge( Vector<WVZInhalt> zuPrfendeZeichenFolge )
  {
    m_zuPrfendeZeichenFolge = zuPrfendeZeichenFolge;
  }

	/**
   * Methode liefert kennung zurck
   * @return kennung
   */
  public String getKennung()
  {
  	return m_kennung;
  }

	/**
   * Methode setzt die Variable kennung
   * @param kennung zu bernehmender Wert fr kennung
   */
  public void setKennung(String kennung)
  {
  	m_kennung = kennung;
  }
}
