package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt die Parameter pro Anzeigetyp (Typ A, Typ B, etc.)
 * @author       Dambach Werke GmbH
 */
class RegelnProAnzeigeTyp
{
  /**
   * Parameter in einer Hashmap. Als Key dient der Anzeigetyp, als Value eine Liste mit
   * Parameterstzen
   */
  private HashMap<SystemObject, ListeMitRegeln> m_parameter = new HashMap<SystemObject, ListeMitRegeln> ();    
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelnProAnzeigeTyp ()
  {
  }

  /**
   * Methode fgt einen Regelsatz fr einen Anzeigetyp hinzu
   * @param anzeigeTyp Anzeigetyp
   * @param regeln Regelsatz
   */
  public void addRegel (SystemObject anzeigeTyp, ListeMitRegeln regeln)
  {
  	m_parameter.put(anzeigeTyp, regeln);
  }

	/**
   * Methode liefert die Parameter fr einen Anzeigetyp
   * @param anzeigeTyp Anzeigetyp
   * @return Parameter fr diesen Anzeigetyp
   */
  public ListeMitRegeln getParameter (SystemObject anzeigeTyp)
  {
    synchronized (m_parameter)
    {
      if (!m_parameter.containsKey( anzeigeTyp ))
      {
        ListeMitRegeln liste = new ListeMitRegeln();
        m_parameter.put( anzeigeTyp, liste );
      }

      if (m_parameter.get(anzeigeTyp ) != null)
        return m_parameter.get(anzeigeTyp ); 
      
      return null;
    }
  }

  /**
   * Methode liefert die Variable m_parameter zurck
   * @return Variable m_parameter
   */
  public HashMap<SystemObject, ListeMitRegeln> getParameter()
  {
    return m_parameter;
  }

  /**
   * Methode zum Auswerten der Attributliste 'atl.sswSbaAllgemeineVerriegelungsMatrixProAnzeigeTyp'
   * @param array empfangene Daten vom DaV
   */
  public void auswerteAttributgruppe (Array array, Aspect schaltgrundAllgemein)
  {
    int anzahl = array.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeigeTyp = d1.getReferenceValue( "AnzeigeTyp" ).getSystemObject();
      
        ListeMitRegeln liste = new ListeMitRegeln();
        liste.auswerteAttributgruppe( d1, schaltgrundAllgemein );

        m_parameter.put (anzeigeTyp, liste);
      }
    }
  }
  
  /**
   * Methode lscht die Parameter
   */
  public void clear ()
  {
    synchronized (m_parameter)
    {
      m_parameter.clear();
    }
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      Iterator<Entry<SystemObject, ListeMitRegeln>> it = m_parameter.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry<SystemObject, ListeMitRegeln> entry = it.next();
        
        SystemObject wvzTyp = (SystemObject) entry.getKey();
        ListeMitRegeln lmr = (ListeMitRegeln) entry.getValue();
        
        s += "WVZTyp: " + Tools.getObjektName(  wvzTyp ) + "\n";
        s += lmr.toString() + "\n";
      }
    }

    return s;
  }

  /**
   * Methode bereinigt die Parameter, dabei werden die Anzeigetypen aus dem Parametersatz entfernt,
   * fr die keine Parameter definiert sind. Wird dies nicht gemacht, hat die Steuerung Probleme die
   * richtigen Parameter zu finden.
   */
	public void bereinigeParameter ()
  {
		synchronized (m_parameter)
    {
	  	List<SystemObject> zuLoeschen = new ArrayList<SystemObject>();
	  	
		  Iterator<Entry<SystemObject, ListeMitRegeln>> it = m_parameter.entrySet().iterator();
		  while (it.hasNext())
		  {
		  	Entry<SystemObject, ListeMitRegeln> entry = it.next();
	  	
		  	if (entry.getValue().getParameter().size() == 0)
		  		zuLoeschen.add(entry.getKey());
		  }
	  	
		  for (SystemObject so : zuLoeschen)
		  	m_parameter.remove(so);
    }
  }
}
