package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verschiebungen.verschieben;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Verschiebungen", Regel "Verschieben".<br>  
 * @author Dambach Werke GmbH
 */
public class RegelVerschieben
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;
  
  /**
   * arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   */
  private ArbeitsRichtungVN m_richtung = ArbeitsRichtungVN.VORGAENGER;
  
  /**
   * zu bercksichtigende Schaltgrnde
   */
  private List<Aspect> m_schaltgruende = new ArrayList<Aspect>();
  
  /**
   * Ebende in der Baumstruktur der Vorgnger/Nachfolgerbeziehung bis zu der verschoben werden soll.
   */
  private int m_ebene;
  
  /**
   * WZGTyp des Vorgngers/Nachfolgers in Lngsrichtung .
   */
  private SystemObject m_anzeigeTyp;
  
  /**
   * Sollen abhngige Anzeigen mit verschoben werden .
   */
  private boolean m_abhaengigeAnzeigenVerschieben;
  
  /**
   * Anzeigeninhalt (in der Regel: "X")
   */
  private WVZInhalt  m_anzeigenInhalt = new WVZInhalt();
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelVerschieben ()
  {
  }
  
  /**
   * Konstruktor der Klasse
   * @param richtung arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   * @param schaltgruende zu bercksichtigende Schaltgrnde
   * @param ebene Ebende in der Baumstruktur der Vorgnger/Nachfolgerbeziehung bis zu der verschoben werden soll
   * @param anzeigeTyp WZGTyp des Vorgngers/Nachfolgers in Lngsrichtung 
   * @param abhaengigeAnzeigen Sollen abhngige Anzeigen mit verschoben werden 
   * @param x Anzeigeninhalt (in der Regel: "X")
   * @param schaltgrund Regelspezifischer Schaltgrund
   */
  public RegelVerschieben(ArbeitsRichtungVN richtung,
      List<Aspect> schaltgruende, int ebene, SystemObject anzeigeTyp,
      boolean abhaengigeAnzeigen, WVZInhalt x, Aspect schaltgrund)
  {
	  m_richtung = richtung;
	  m_schaltgruende = schaltgruende;
	  m_ebene = ebene;
	  m_anzeigeTyp = anzeigeTyp;
	  m_abhaengigeAnzeigenVerschieben = abhaengigeAnzeigen;
	  m_anzeigenInhalt = x;
	  m_schaltgrundRegel = schaltgrund;
  }

	/**
   * Methode prft ob die Regel in Vorgngerrichtung arbeitet
   * @return Regel arbeitet in Vorgngerrichtung
   */
  public boolean isRichtungVorgaenger()
  {
    return m_richtung == ArbeitsRichtungVN.VORGAENGER;
  }

  /**
   * Methode setzt die Variable m_richtung
   * @param richtung Wert auf den die Variable m_richtung gesetzt werden soll
   */
  public void setRichtung ( ArbeitsRichtungVN richtung )
  {
    m_richtung = richtung;
  }


  /**
   * Methode liefert die Variable m_anzeigenInhalt zurck
   * @return Variable m_anzeigenInhalt
   */
  public WVZInhalt getAnzeigenInhalt()
  {
    return m_anzeigenInhalt;
  }


  /**
   * Methode setzt die Variable m_anzeigenInhalt
   * @param anzeigenInhalt Wert auf den die Variable m_anzeigenInhalt gesetzt werden soll
   */
  public void setAnzeigenInhalt( WVZInhalt anzeigenInhalt )
  {
    m_anzeigenInhalt = anzeigenInhalt;
  }

  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaRegelVerschieben'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    // Schaltgrundregel
    
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    // VorgngerNachfolger
    
    String vorgaengerNachfolger = data.getItem( "VorgngerNachfolger" ).asTextValue().getText();
    
    if (vorgaengerNachfolger.equals( "Vorgnger" ))
      setRichtung (ArbeitsRichtungVN.VORGAENGER);
    else
      setRichtung (ArbeitsRichtungVN.NACHFOLGER);
    
    // Schaltgrund
    
    ReferenceArray a = data.getReferenceArray( "Schaltgrund" );
    int anzahl = a.getLength();
    
    m_schaltgruende.clear();
    for (int x = 0; x < anzahl; ++x)
      m_schaltgruende.add ( (Aspect) a.getReferenceValue( x ).getSystemObject() );

    // Ebene
    
    m_ebene = (int) data.getUnscaledValue( "Ebene" ).longValue();
    
    // Anzeigetyp
    
    m_anzeigeTyp = data.getReferenceValue( "AnzeigeTyp" ).getSystemObject();
    
    // AbhngigeAnzeigenVerschieben
    
    String jaNein = data.getItem( "AbhngigeAnzeigenVerschieben" ).asTextValue().getText();
    m_abhaengigeAnzeigenVerschieben = jaNein.equalsIgnoreCase( "Ja" );

    // Anzeigeninhalt
    
    Data anzeigenInhalt = data.getItem( "Anzeigeinhalt" );
    m_anzeigenInhalt.auswerteAttributgruppe( anzeigenInhalt );
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel ) 
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "Richtung: " + m_richtung + "\n";
    s += "X = " + Tools.getWVZInhalt( m_anzeigenInhalt ) + "\n";
    s += "Schaltgrnde:\n";
    for (Aspect a : m_schaltgruende)
      System.out.println("- " + Tools.getObjektName( a ) + "\n");
    s += "Ebene: " + m_ebene + "\n";
    s += "Anzeigetyp: " + Tools.getObjektName( m_anzeigeTyp ) + "\n";
    s += "Abhngige Anzeigen verschieben: " + m_abhaengigeAnzeigenVerschieben + "\n";
    
    return s;
  }

  /**
   * Methode liefert die Richtung in die Regelklasse arbeitet.
   * @return Richtung
   */
  public ArbeitsRichtungVN getRichtung()
  {
    return m_richtung;
  }

  /**
   * Methode liefert die Variable m_schaltgruende zurck
   * @return Variable m_schaltgruende
   */
  public List<Aspect> getSchaltgruende()
  {
    return m_schaltgruende;
  }

  /**
   * Methode setzt die Variable m_schaltgruende
   * @param schaltgruende Wert auf den die Variable m_schaltgruende gesetzt werden soll
   */
  public void setSchaltgruende( List<Aspect> schaltgruende )
  {
    m_schaltgruende = schaltgruende;
  }

  /**
   * Methode liefert die Variable m_ebene zurck
   * @return Variable m_ebene
   */
  public int getEbene()
  {
    return m_ebene;
  }

  /**
   * Methode setzt die Variable m_ebene
   * @param ebene Wert auf den die Variable m_ebene gesetzt werden soll
   */
  public void setEbene( int ebene )
  {
    m_ebene = ebene;
  }

  /**
   * Methode liefert die Variable m_anzeigeTyp zurck
   * @return Variable m_anzeigeTyp
   */
  public SystemObject getAnzeigeTyp()
  {
    return m_anzeigeTyp;
  }

  /**
   * Methode setzt die Variable m_anzeigeTyp
   * @param anzeigeTyp Wert auf den die Variable m_anzeigeTyp gesetzt werden soll
   */
  public void setAnzeigeTyp( SystemObject anzeigeTyp )
  {
    m_anzeigeTyp = anzeigeTyp;
  }

  /**
   * Methode liefert die Variable m_abhaengigeAnzeigenVerschieben zurck
   * @return Variable m_abhaengigeAnzeigenVerschieben
   */
  public boolean isAbhaengigeAnzeigenVerschieben()
  {
    return m_abhaengigeAnzeigenVerschieben;
  }

  /**
   * Methode setzt die Variable m_abhaengigeAnzeigenVerschieben
   * @param abhaengigeAnzeigenVerschieben Wert auf den die Variable m_abhaengigeAnzeigenVerschieben gesetzt werden soll
   */
  public void setAbhaengigeAnzeigenVerschieben( boolean abhaengigeAnzeigenVerschieben )
  {
    m_abhaengigeAnzeigenVerschieben = abhaengigeAnzeigenVerschieben;
  }
}
