package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.vorgaengerNachfolger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse mit statischen Hilfsmethode
 * @author ObertM
 *
 */
public class ToolsVN
{
	/**
	 * Kennung "undefiniert"
	 */
	final static String _undefiniert = "undefiniert";
	
	/**
	 * Methode bestimmt die Anzeigen zu einem Anzeigequerschnitt
	 * @param aq Anzeigequerschnitt
	 * @return Liste mit Anzeigen
	 */
  public static List<SystemObject> bestimmeAnzeigenZuAq (SystemObject aq)
  {
		List<SystemObject> anzeigenDesQuerschnitts = new ArrayList<SystemObject>();
		
		for (SystemObject so : Tools.getSystemObjekteAnzeigen(aq))
  		anzeigenDesQuerschnitts.add (so);
		
		return anzeigenDesQuerschnitts;
  }
  
  /**
   * Methode bestimmt aus einer Liste (org.eclipse.swt.widgets.List) mit Namen, die entsprechenden
   * Systemobjekte
   * @param liste Liste mit Namen
   * @param moeglicheObjekte Objekte
   * @return Liste mit Objekten
   */
	public static SystemObject[] bestimmeSystemObjekteAusListe (org.eclipse.swt.widgets.List liste, List<SystemObject> moeglicheObjekte)
	{
		int anzahl = liste.getItemCount();
	
		SystemObject[] objekte = new SystemObject [anzahl];
	
		for (int i=0; i<liste.getItemCount(); ++i)
		{
	   	String name = liste.getItem(i);
	   	objekte [i] = bestimmeObjekt(name, moeglicheObjekte);
		}
		
		return objekte;
	}
	
	/**
	 * Methode bestimmt den AQ zu einem Text der ber die Methode bestimmeName erzeugt wurde 
	 * @param name zu vergleichender Text
	 * @return Systemobjekt
	 */
	public static SystemObject bestimmeObjekt(String name, List<SystemObject> objekte)
  {
		for (SystemObject so : objekte)
			if (bestimmeBezeichnung(so).equals(name))
				return so;
		
		return null;
  }

  /**
   * Methode bestimmt den Text der als Bezeichnung fr einen Anzeigequerschnitt bzw. eine Anzeige
   * angezeigt werden.
   * @param objekt zu betrachtendes Objekt 
   * @return Bezeichnung fr das Objekt
   */
  public static String bestimmeBezeichnung (SystemObject objekt)
  {
  	if (objekt == null)
  		return null;
  	
  	String name = objekt.getNameOrPidOrId();
  	
  	if (objekt.getType().getPid().equals("typ.anzeige"))
  	{
		  SystemObject typ = Konfig.getInstance().getWzgTypVonAnzeige(objekt.getPid());
		  String nameTyp = "";
		  
		  if (typ != null)
		  	nameTyp = typ.getName();
		  
		  String praeFix = "SBA AnzeigeTyp Typ";
		  if (nameTyp.contains(praeFix))
		  	nameTyp = nameTyp.replace(praeFix, "");

		  /*
		   * Sonderfall Blinker
		   */
		  praeFix = "SBA AnzeigeTyp ";
		  if (nameTyp.contains(praeFix))
		  	nameTyp = nameTyp.replace(praeFix, "");

		  name += " (" + nameTyp + ")"; 
  	}
  	
  	return name;
  }

  public static List<String> bestimmeObjektNamen (List<SystemObject> objekte, boolean mitUndefiniert)
  {
    return bestimmeObjektNamen(objekte.toArray(new SystemObject[objekte.size()]), mitUndefiniert);
  }

  public static List<String> bestimmeObjektNamen (SystemObject[] objekte, boolean mitUndefiniert)
  {
  	List<String> l  = new ArrayList<String>();
  	
  	if (objekte != null)
  	{
	    for (SystemObject so: objekte)
	    	l.add( ToolsVN.bestimmeBezeichnung(so) );
  	}
  	
    Collections.sort( l );
    if (mitUndefiniert)
      l.add(0, _undefiniert);
    
    return l;
  }

//  /**
//   * Methode liefert eine Liste mit den Systemobjekten vom Typ "typ.anzeigeQuerschnitt"
//   * @return Liste mit Systemobjekten
//   */
//	public static List<SystemObject> bestimmeAnzeigeQuerschnitte()
//  {
//		return bestimmeSystemObjekte("typ.anzeigeQuerschnitt");
//  }
//	
//  /**
//   * Methode liefert eine Liste mit den Systemobjekten vom Typ "typ.anzeige"
//   * @return Liste mit Systemobjekten
//   */
//	public static List<SystemObject> bestimmeAnzeigen()
//  {
//		return bestimmeSystemObjekte("typ.anzeige");
//  }
	
  /**
   * Methode liefert eine Liste mit den Systemobjekten zu einem bestimmten Typ. Dabei werden
   * Objetke die zur PUA gehren nicht bercksichtigt (Pid beginnt mit "swarco.pua")
   * @return Liste mit Systemobjekten
   */
	public static List<SystemObject> bestimmeSystemObjekte(String typ)
  {
		ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		
		List<SystemObject> l = new ArrayList<SystemObject>();
		List<SystemObject> vorhandeneObjekte = connection.getDataModel().getType( typ ).getElements();
		
		for (SystemObject so : vorhandeneObjekte)
			if (!so.getPid().toLowerCase().startsWith("swarco.pua"))
				l.add(so);
				
		return l;
  }
}
