package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungMoeglicheAnzeigeQuerschnitte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.abhaengigkeiten.Abhaengigkeiten;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;


/**
 * Klasse definiert die Defaulteinstellungen fr die B27. Diese Einstellungen werden nur
 * verwendet, wenn ber den Datenverteiler keine anderen Parameter versorgt sind
 * (atg.sswSbaAnzeigeTypMitWVZInhalt).
 * 
 * @author ObertM
 * 
 */
public class DefaultEinstellungenB27 implements IProjektEinstellungen
{
	/**
	 * Prfix mit der die Pid der AnzeigeTypen (typ.sswSbaAnzeigeTyp) beginnen
	 */
	private static final String _praefixAnzeigeTypen = "sba.anzeigeTyp.B27";

	/**
	 * Prfix mit der die Pid der WvzInhalte (typ.wvzInhalt) beginnen
	 */
	private static final String _praefixWvzInhalte = "wvzInhalt.uz.B27";

	/**
	 * Prfix mit der die Pid der WvzInhalte typ.anzeigeQuerschnitt) beginnen
	 */
	private static final String _praefixAnzeigeQuerschnitt = "aq.B27";

	/**
	 * Methode liefert eine Vector mit den im Projekt zu verwendenden Anzeigetypen
	 * @return Liste mit Anzeigetypen
	 */
	public Vector<AnzeigeTypEigenschaften> getDefaultAnzeigeTypen()
	{
		Vector<AnzeigeTypEigenschaften> l = new Vector<AnzeigeTypEigenschaften>();
		
		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_A,
		    _praefixAnzeigeTypen + ".TypA", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_WZG_A)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_B,
		    _praefixAnzeigeTypen + ".TypB", _praefixWvzInhalte + ".Dunkel",
        getWvzInhalte(EAnzeigeTypen.TYP_WZG_B)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_C,
		    _praefixAnzeigeTypen + ".TypC", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_WZG_C)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_BLINKER,
		    _praefixAnzeigeTypen + ".Blinker", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_BLINKER)));

		return l;
	};

	/**
	 * Methode liefert die Pids der zu verwendenden WvzInhalte (typ.wvzInhalt) in Abhngigkeit 
	 * des Anzeigetyps.
	 * @param typ Anzeigetyp
	 * @return Liste mit Pids
	 */
	private List<String> getWvzInhalte (EAnzeigeTypen typ) 
  {
		String praefix = _praefixWvzInhalte;
		
		List<String> list = new ArrayList<String>();
		
		if (typ == EAnzeigeTypen.TYP_WZG_A) 
		{
			// normale Inhalte
			
			list.add(praefix + ".Dunkel");
			
			list.add(praefix + ".tlsZulaessigeHoechst120");
			list.add(praefix + ".tlsZulaessigeHoechst100");
			list.add(praefix + ".tlsZulaessigeHoechst80");
			list.add(praefix + ".tlsZulaessigeHoechst60");
			list.add(praefix + ".tlsZulaessigeHoechst50");
			list.add(praefix + ".tlsZulaessigeHoechst40");

			list.add(praefix + ".tlsEndeZulaessigeHoechst120");
			list.add(praefix + ".tlsEndeZulaessigeHoechst100");
			list.add(praefix + ".tlsEndeZulaessigeHoechst80");
			list.add(praefix + ".tlsEndeZulaessigeHoechst60");
			list.add(praefix + ".tlsEndeZulaessigeHoechst50");
			list.add(praefix + ".tlsEndeZulaessigeHoechst40");

			list.add(praefix + ".tlsEndeAllerStreckenverbote");
			list.add(praefix + ".tlsStau");
			list.add(praefix + ".tlsFahrstreifenGesperrt");
			list.add(praefix + ".tlsFahrstreifenWechselnNachLinks");
			list.add(praefix + ".tlsFahrstreifenWechselnNachRechts");
			
			// negierte Inhalte
			
			list.add(praefix + ".nichtDunkel");
			
			list.add(praefix + ".nichtTlsZulaessigeHoechst120");
			list.add(praefix + ".nichtTlsZulaessigeHoechst100");
			list.add(praefix + ".nichtTlsZulaessigeHoechst80");
			list.add(praefix + ".nichtTlsZulaessigeHoechst60");
			list.add(praefix + ".nichtTlsZulaessigeHoechst50");
			list.add(praefix + ".nichtTlsZulaessigeHoechst40");

			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst120");
			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst100");
			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst80");
			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst60");
			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst50");
			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst40");

			list.add(praefix + ".nichtTlsEndeAllerStreckenverbote");
			list.add(praefix + ".nichtTlsStau");
			list.add(praefix + ".nichtTlsFahrstreifenGesperrt");
			list.add(praefix + ".nichtTlsFahrstreifenWechselnNachLinks");
			list.add(praefix + ".nichtTlsFahrstreifenWechselnNachRechts");

		} 
		else if (typ == EAnzeigeTypen.TYP_WZG_B)  
		{
			// normale Inhalte
			
			list.add(praefix + ".Dunkel");

			list.add(praefix + ".tlsUeberholverbotAllgemein");
			list.add(praefix + ".tlsEndeUeberholverbotAllgemein");
			list.add(praefix + ".tlsUeberholverbotLkw");
			list.add(praefix + ".tlsEndeUeberholverbotLkw");
			list.add(praefix + ".tlsGefahrenstelle");
			list.add(praefix + ".tlsBaustelle");
			list.add(praefix + ".tlsStau");
			list.add(praefix + ".tlsGlaettegefahr");
			
			// negierte Inhalte
			
			list.add(praefix + ".nichtDunkel");

			list.add(praefix + ".nichtTlsUeberholverbotAllgemein");
			list.add(praefix + ".nichtTlsEndeUeberholverbotAllgemein");
			list.add(praefix + ".nichtTlsUeberholverbotLkw");
			list.add(praefix + ".nichtTlsEndeUeberholverbotLkw");
			list.add(praefix + ".nichtTlsGefahrenstelle");
			list.add(praefix + ".nichtTlsBaustelle");
			list.add(praefix + ".nichtTlsStau");
			list.add(praefix + ".nichtTlsGlaettegefahr");

		} 
		else if (typ == EAnzeigeTypen.TYP_WZG_C)  
		{
			// normale Inhalte 

			list.add(praefix + ".Dunkel");

			list.add(praefix + ".tls7Komma5Tonnen");
			list.add(praefix + ".tlsAuf1000Meter");
			list.add(praefix + ".tlsAuf1500Meter");
			list.add(praefix + ".tlsAuf2000Meter");
			list.add(praefix + ".tls1000Meter");
			list.add(praefix + ".tls1500Meter");
			list.add(praefix + ".tls2000Meter");
			list.add(praefix + ".tlsNebel");
			list.add(praefix + ".tlsNaesse");
			list.add(praefix + ".tlsUnfall");
			list.add(praefix + ".tlsSmog");

			// negierte Inhalte 

			list.add(praefix + ".nichtDunkel");

			list.add(praefix + ".nichtTls7Komma5Tonnen");
			list.add(praefix + ".nichtTlsAuf1000Meter");
			list.add(praefix + ".nichtTlsAuf1500Meter");
			list.add(praefix + ".nichtTlsAuf2000Meter");
			list.add(praefix + ".nichtTls1000Meter");
			list.add(praefix + ".nichtTls1500Meter");
			list.add(praefix + ".nichtTls2000Meter");
			list.add(praefix + ".nichtTlsNebel");
			list.add(praefix + ".nichtTlsNaesse");
			list.add(praefix + ".nichtTlsUnfall");
			list.add(praefix + ".nichtTlsSmog");
		} 

		else if (typ == EAnzeigeTypen.TYP_BLINKER)  
		{
			// normale Inhalte
			
			list.add(praefix + ".Dunkel");

			list.add(praefix + ".tlsGelbesBlinklicht");
			
			// negierte Inhalte
			
			list.add(praefix + ".nichtDunkel");

			list.add(praefix + ".nichtTlsGelbesBlinklicht");

		} 
		
		return list;
  }
	
	@Override
  public String getPidBeibehalten()
  {
	  return _praefixWvzInhalte + ".Beibehalten";
  }

	@Override
  public String getPidBeliebig()
  {
	  return _praefixWvzInhalte + ".Beliebig";
  }

	@Override
  public String getPidDunkel()
  {
	  return _praefixWvzInhalte + ".Dunkel";
  }

	@Override
  public String getPidDontCare()
  {
		return _praefixWvzInhalte + ".DontCare";
  }

	@Override
  public HashMap<String, EAnzeigeQuerschnittTyp> getDefaultAnzeigeQuerschnitte()
  {
	  HashMap<String, EAnzeigeQuerschnittTyp> zuordnungen = new HashMap<String, EAnzeigeQuerschnittTyp>();
	  	
  	// Liste aller Anzeigequerschnitte
  	
  	List<SystemObject> objekte = Tools.getSystemObjekte("typ.anzeigeQuerschnitt");
  	for (SystemObject so : objekte)
  		if (so.getPid().startsWith(_praefixAnzeigeQuerschnitt))
  			zuordnungen.put(so.getPid(), EAnzeigeQuerschnittTyp.UNDEFINIERT);
  		
    // Zuordnen der Anzeigetypen berschreiben

  	String [] typ1 = new String []
    {
  			"aq.B27.4221.HFB.N",
  			"aq.B27.4223.HFB.N",
  			"aq.B27.4224.HFB.N",
  			
  			"aq.B27.4229.HFB.N",
  			
  			"aq.B27.4235.HFB.N",
  			"aq.B27.4236.HFB.N",
  			"aq.B27.4238.HFB.N",
  			"aq.B27.4239.HFB.N",
  			
  			"aq.B27.4395.HFB.S",
  			"aq.B27.4392.HFB.S",
  			"aq.B27.4390.HFB.S",
  			"aq.B27.4389.HFB.S",
  			"aq.B27.4387.HFB.S",
  			
  			"aq.B27.4377.HFB.S",
  			"aq.B27.4375.HFB.S",
    };

  	/**
  	 * Pids der Anzeigen vom Typ2
  	 */
  	String [] typ2 = new String []
    {
  			"aq.B27.4225.HFB.N",
  			"aq.B27.4230.HFB.N",
  			
  			"aq.B27.4384.HFB.S",
    };

  	/**
  	 * Pids der Anzeigen vom Typ3
  	 */
  	String [] typ3 = new String []
    {
  	  	"aq.B27.4201.HFB.N",
  	  	"aq.B27.4203.HFB.N",
  	  	"aq.B27.4206.HFB.N",
  	  	"aq.B27.4210.HFB.N",
  	  	"aq.B27.4213.HFB.N",
  	  	"aq.B27.4218.HFB.N",
  	  	
  	  	"aq.B27.4372.HFB.S",
  	  	"aq.B27.4371.HFB.S",
  	  	"aq.B27.4366.HFB.S",
  	  	"aq.B27.4363.HFB.S",
  	  	"aq.B27.4365.HFB.S",
    };

  	/**
  	 * Pids der Anzeigen vom Typ4
  	 */
  	String [] typ4 = new String []
    {
  	  	"aq.B27.4230.Ein1.N",
  	  	"aq.B27.4230.Ein2.N",
  	  	    
  	  	"aq.B27.4239.Ein.N",
  	  	
  	  	"aq.B27.4390.Ein.S",
  	  	"aq.B27.4387.Ein.S",
  	  	"aq.B27.4384.Ein.S",
    };

  	/**
  	 * Pids der Anzeigen vom Typ5
  	 */
  	String [] typ5 = new String []
    {
  	  	"aq.B27.4233.HFB.N",
    };

  	/**
  	 * Pids der Anzeigen vom Typ6
  	 */
  	String [] typ6 = new String []
    {
  	  	"aq.B27.4242.HFB.N",
    };

  	/**
  	 * Pids der Anzeigen vom Typ7
  	 */
  	String [] typ7 = new String []
    {
  			"aq.B27.4398.HFB.S",
    };

  	/**
  	 * Pids der Anzeigen vom Typ8
  	 */
  	String [] typ8 = new String []
    {
  			"aq.B27.4380.HFB.S",
    };

   	for (String pid : typ1)
   		zuordnungen.put(pid, EAnzeigeQuerschnittTyp.B27_TYP1);

   	for (String pid : typ2)
   		zuordnungen.put(pid, EAnzeigeQuerschnittTyp.B27_TYP2);
	  	
   	for (String pid : typ3)
   		zuordnungen.put(pid, EAnzeigeQuerschnittTyp.B27_TYP3);

   	for (String pid : typ4)
   		zuordnungen.put(pid, EAnzeigeQuerschnittTyp.B27_TYP4);

   	for (String pid : typ5)
   		zuordnungen.put(pid, EAnzeigeQuerschnittTyp.B27_TYP5);

   	for (String pid : typ6)
   		zuordnungen.put(pid, EAnzeigeQuerschnittTyp.B27_TYP6);

   	for (String pid : typ7)
   		zuordnungen.put(pid, EAnzeigeQuerschnittTyp.B27_TYP7);

   	for (String pid : typ8)
   		zuordnungen.put(pid, EAnzeigeQuerschnittTyp.B27_TYP8);

   	// prfen ob alle Querschniite versorgt wurden
   	
   	Iterator<Entry<String, EAnzeigeQuerschnittTyp>> it = zuordnungen.entrySet().iterator();
   	while (it.hasNext())
   	{
   		Entry<String, EAnzeigeQuerschnittTyp> entry = it.next();
   		
   		if (entry.getValue() == EAnzeigeQuerschnittTyp.UNDEFINIERT)
   			System.err.println("Fehler: fr Anzeigequerschnitt " + entry.getKey() + 
   					" wurde kein Anzeigequerschnitttyp definiert !");
   	}
   	
	  return zuordnungen;
  }

	@Override
  public SystemObject[] getDefaultQuerBeziehung(SystemObject soAQ)
  {
  	EAnzeigeQuerschnittTyp typ = VerwaltungMoeglicheAnzeigeQuerschnitte.getAnzeigeQuerschnittTyp(soAQ);
  	
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);

  	Vector<SystemObject> reihenfolge = new Vector<SystemObject>(); 
  	                                                
  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP1)
  	{
  		reihenfolge.add (anzeigen [1]); // A links
  		reihenfolge.add (anzeigen [4]); // Blinker
  		reihenfolge.add (anzeigen [2]); // B
  		reihenfolge.add (anzeigen [3]); // C
  		reihenfolge.add (anzeigen [0]); // A rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP2)
  	{
  		reihenfolge.add (anzeigen [2]); // A links
  		reihenfolge.add (anzeigen [8]); // Blinker links
  		reihenfolge.add (anzeigen [4]); // B links
  		reihenfolge.add (anzeigen [6]); // C links
  		reihenfolge.add (anzeigen [1]); // A mitte
  		reihenfolge.add (anzeigen [7]); // Blinker rechts
  		reihenfolge.add (anzeigen [3]); // B rechts
  		reihenfolge.add (anzeigen [5]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP3)
  	{
  		reihenfolge.add (anzeigen [7]); // Blinker links
  		reihenfolge.add (anzeigen [1]); // A links
  		reihenfolge.add (anzeigen [3]); // B links
  		reihenfolge.add (anzeigen [5]); // C links
  		
  		reihenfolge.add (anzeigen [6]); // Blinker rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
  		reihenfolge.add (anzeigen [2]); // B rechts
  		reihenfolge.add (anzeigen [4]); // C rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP4)
  	{
  		reihenfolge.add (anzeigen [3]); // Blinker 
  		reihenfolge.add (anzeigen [0]); // A 
  		reihenfolge.add (anzeigen [1]); // B 
  		reihenfolge.add (anzeigen [2]); // C 
    };

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP5)
  	{
  		reihenfolge.add (anzeigen [2]); // A links
  		reihenfolge.add (anzeigen [1]); // A mitte
  		reihenfolge.add (anzeigen [5]); // Blinker
  		reihenfolge.add (anzeigen [3]); // B
  		reihenfolge.add (anzeigen [4]); // C
  		reihenfolge.add (anzeigen [0]); // A rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP6)
  	{
  		reihenfolge.add (anzeigen [1]); // A links
  		reihenfolge.add (anzeigen [5]); // Blinker
  		reihenfolge.add (anzeigen [3]); // B
  		reihenfolge.add (anzeigen [4]); // C
  		reihenfolge.add (anzeigen [0]); // A mitte
  		reihenfolge.add (anzeigen [2]); // A rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP7)
  	{
  		reihenfolge.add (anzeigen [1]); // A links
  		reihenfolge.add (anzeigen [4]); // Blinker links
  		reihenfolge.add (anzeigen [2]); // B links
  		reihenfolge.add (anzeigen [3]); // C links
  		reihenfolge.add (anzeigen [0]); // A mitte
  		reihenfolge.add (anzeigen [5]); // A rechts
  		reihenfolge.add (anzeigen [8]); // Blinker rechts
  		reihenfolge.add (anzeigen [6]); // B rechts
  		reihenfolge.add (anzeigen [7]); // C rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP8)
  	{
  		reihenfolge.add (anzeigen [3]); // A links
  		reihenfolge.add (anzeigen [9]); // Blinker links
  		reihenfolge.add (anzeigen [5]); // B links
  		reihenfolge.add (anzeigen [7]); // C links
  		reihenfolge.add (anzeigen [2]); // A mitte links
  		
  		reihenfolge.add (anzeigen [1]); // A mitte rechts
  		reihenfolge.add (anzeigen [8]); // Blinker rechts
  		reihenfolge.add (anzeigen [4]); // B rechts
  		reihenfolge.add (anzeigen [6]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
    };

  	return reihenfolge.toArray(new SystemObject[anzeigen.length]);
  }

	@Override
  public Vector<Abhaengigkeiten> getDefaultAbhaengigkeiten(SystemObject soAQ)
  {
  	EAnzeigeQuerschnittTyp typ = VerwaltungMoeglicheAnzeigeQuerschnitte.getAnzeigeQuerschnittTyp(soAQ);
  	
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);
  	
		Vector<Abhaengigkeiten> v = new Vector<Abhaengigkeiten>();

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP1)
  	{
  		v.add (new Abhaengigkeiten(anzeigen [2], anzeigen [3]));
  	}

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP2)
  	{
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [6]));
  		v.add (new Abhaengigkeiten(anzeigen [3], anzeigen [5]));
  	}

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP3)
  	{
  		v.add (new Abhaengigkeiten(anzeigen [3], anzeigen [5]));
  		v.add (new Abhaengigkeiten(anzeigen [2], anzeigen [4]));
  	}

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP4)
  	{
  		v.add (new Abhaengigkeiten(anzeigen [1], anzeigen [2]));
  	}

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP5)
  	{
  		v.add (new Abhaengigkeiten(anzeigen [3], anzeigen [4]));
  	}

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP6)
  	{
  		v.add (new Abhaengigkeiten(anzeigen [3], anzeigen [4]));
  	}

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP7)
  	{
  		v.add (new Abhaengigkeiten(anzeigen [2], anzeigen [3]));
  		v.add (new Abhaengigkeiten(anzeigen [6], anzeigen [7]));
  	}

  	if (typ == EAnzeigeQuerschnittTyp.B27_TYP8)
  	{
  		v.add (new Abhaengigkeiten(anzeigen [5], anzeigen [7]));
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [6]));
  	}

	  return v;
  }

}
