/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemein.Design;

/**
 * Klasse definiert ber ein Enum die unterschiedlichen AnzeigeTypen
 * @author SWARCO Traffic Systems GmbH
 * @author Matthias Obert
 */
public enum EAnzeigeQuerschnittTyp
{
	UNDEFINIERT (0, "Undefiniert"),
	
	/*------------------------------------------------------------------------------
	 * A14
	 *------------------------------------------------------------------------------*/ 
	
	/* A14 Typ 1
	 * 
	 *           +-----+
	 *           |  B  |
	 *           |  3  |
	 * +-----+   +-----+  +-----+
	 * |  A  |   +-----+  |  A  |
	 * |  2  |   | C 4 |  |  1  |
	 * +-----+   +-----+  +-----+
	 */
  A14_TYP1  ( 1, "A14 Typ 1", SchilderTypenVM._A14_Typ1),
  
	/*------------------------------------------------------------------------------
	 * B27
	 *------------------------------------------------------------------------------*/
  
  /* B27 Typ 1
   * 
   *              BL
   *           +-----+
   *           |  B  |
   *           |  3  |
   * +-----+   +-----+  +-----+
   * |  A  |   +-----+  |  A  |
   * |  2  |   | C 4 |  |  1  |
   * +-----+   +-----+  +-----+
   */
  
  B27_TYP1  ( 6, "B27 Typ 1", SchilderTypenVM._B27_Typ1),
  
  /* B27 Typ 2
   * 
   *              BL                BL
   *           +-----+           +-----+
   *           |  B  |           |  B  |
   *           |  2  |           |  2  |
   * +-----+   +-----+  +-----+  +-----+  +-----+
   * | A3l |   +-----+  | A3  |  +-----+  | A3r |
   * |     |   | C 1 |  |     |  | C 1 |  |     |
   * +-----+   +-----+  +-----+  +-----+  +-----+
   */
  
  B27_TYP2  ( 7, "B27 Typ 2", SchilderTypenVM._B27_Typ2),

  /* B27 Typ 3
   * 
   *    BL                BL
   * +------+          +------+
   * |  A2  |          |  A2  |
   * |      |          |      |
   * +------+          +------+
   * +------+          +------+
   * |  B1  |          |  B1  |
   * |      |          |      |
   * +------+          +------+
   * +------+          +------+
   * |  C2  |          |  C2  |
   * +------+          +------+
   */
  
  B27_TYP3  ( 8, "B27 Typ 3", SchilderTypenVM._B27_Typ3),
  
  /* B27 Typ 4
   * 
   *    BL    
	 * +------+ 
	 * |  A4  | 
	 * |      | 
	 * +------+ 
	 * +------+ 
	 * |  B2  | 
	 * |      | 
	 * +------+ 
	 * +------+ 
	 * |  C1  | 
	 * +------+ 
	 */
  
  B27_TYP4  ( 9, "B27 Typ 4", SchilderTypenVM._B27_Typ4),
  
 /* B27 Typ 5
  * 
  *                          BL
  *                       +-----+
  *                       |  B  |
  *                       |  1  |
  * +-----+      +-----+  +-----+  +-----+
  * | A5  |      | A1l |  +-----+  | A1r |
  * |     |      |     |  | C 1 |  |     |
  * +-----+      +-----+  +-----+  +-----+
  */
  
  B27_TYP5  (10, "B27 Typ 5", SchilderTypenVM._B27_Typ5),
  
 /* B27 Typ 6
  * 
  *            BL
  *          +-----+
  *          |  B  |
  *          |  1  |
  * +-----+  +-----+  +-----+    +-----+
  * | A1l |  +-----+  | A1  |    | A1r |
  * |     |  | C 1 |  |     |    |     |
  * +-----+  +-----+  +-----+    +-----+
  */
  
  B27_TYP6  (11, "B27 Typ 6", SchilderTypenVM._B27_Typ6),
  
 /* B27 Typ 7
  * 
  *              BL                         BL
  *           +-----+                     +-----+
  *           |  B  |                     |  B  |
  *           |  1  |                     |  1  |
  * +-----+   +-----+  +-----+   +-----+  +-----+
  * | A1l |   +-----+  | A1r |   | A2  |  +-----+
  * |     |   | C 1 |  |     |   |     |  | C 1 |
  * +-----+   +-----+  +-----+   +-----+  +-----+
  */
  
  B27_TYP7  (12, "B27 Typ 7", SchilderTypenVM._B27_Typ7),
  
 /* B27 Typ 8
  * 
  *              BL                           BL
  *           +-----+                      +-----+
  *           |  B  |                      |  B  |
  *           |  2  |                      |  2  |
  * +-----+   +-----+  +-----+   +-----+   +-----+  +-----+
  * | A3l |   +-----+  | A3r |   | A3l |   +-----+  | A3r |
  * |     |   | C 1 |  |     |   |     |   | C 1 |  |     |
  * +-----+   +-----+  +-----+   +-----+   +-----+  +-----+
  */
  
  B27_TYP8  (13, "B27 Typ 8", SchilderTypenVM._B27_Typ8),
  
	/*------------------------------------------------------------------------------
	 * A8
	 *------------------------------------------------------------------------------*/
  
 /* A8 Typ 1
  * 
  *           +-----+           +-----+
  *           |  B  |           |  B  |
  *           |  5  |           |  4  |
  * +-----+   +-----+  +-----+  +-----+  +-----+
  * | A3  |   +-----+  | A2  |  +-----+  | A1  |
  * |     |   | C 7 |  |     |  | C 6 |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+
  */
  
  A8_TYP1   (18, "A8 Typ 1",   SchilderTypenVM._A8_Typ1),
  
 /* A8 Typ 2
  * 
  *           +-----+           +-----+            +-----+
  *           |  B  |           |  B  |            |  B  |
  *           |  7  |           |  6  |            |  5  |
  * +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+
  * | A4  |   +-----+  | A3  |  +-----+  | A2  |   +-----+  | A1  |
  * |     |   | C10 |  |     |  | C 9 |  |     |   | C 8 |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+
  */
  
  A8_TYP2   (19, "A8 Typ 2",   SchilderTypenVM._A8_Typ2),
  
 /* A8 Typ 3
  * 
  * +-----+
  * | A1  |
  * |     |
  * +-----+
  */
  
  A8_TYP3   (20, "A8 Typ 3",   null                    ),
  
 /* A8 Typ 4
  * 
  *           +-----+                      +-----+
  *           |  B  |                      |  B  |
  *           |  6  |                      |  5  |
  * +-----+   +-----+  +-----+   +-----+   +-----+  +-----+
  * | A4  |   +-----+  | A3  |   | A2  |   +-----+  | A1  |
  * |     |   | C 8 |  |     |   |     |   | C 7 |  |     |
  * +-----+   +-----+  +-----+   +-----+   +-----+  +-----+
  */
  
  A8_TYP4   (21, "A8 Typ 4",   SchilderTypenVM._A8_Typ4),
  
 /* A8 Typ 5
  * 
  *           +-----+           +-----+           +-----+           +-----+
  *           |  B  |           |  B  |           |  B  |           |  B  |
  *           |  9  |           |  8  |           |  7  |           |  6  |
  * +-----+   +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+
  * | A5  |   +-----+  | A4  |  +-----+  | A3  |  +-----+  | A2  |  +-----+  | A1  |
  * |     |   | C13 |  |     |  | C12 |  |     |  | C11 |  |     |  | C10 |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+
  */
  
  A8_TYP5   (22, "A8 Typ 5",   SchilderTypenVM._A8_Typ5),
  
  /* A8 Typ 6
   * 
   *           +-----+           +-----+
   *           |  B  |           |  B  |
   *           |  5  |           |  4  |
   * +-----+   +-----+  +-----+  +-----+  +-----+
   * | A3  |   +-----+  | A2  |  +-----+  | A1  |
   * |     |   | C 7 |  |     |  | C 6 |  |     |
   * +-----+   +-----+  +-----+  +-----+  +-----+
   */
  
  A8_TYP6   (23, "A8 Typ 6",   SchilderTypenVM._A8_Typ6),
  
 /* A8 Typ 6d
  * 
  *           +-----+           +-----+
  *           |  B  |           |  B  |
  *           |  5  |           |  4  |
  * +-----+   +-----+  +-----+  +-----+  +-----+
  * | A3  |   +-----+  | A2  |  +-----+  | A1  |
  * |     |   | C 7 |  |     |  | C 6 |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+
  * +------------------------------------------+
  * |                    D8                    |
  * +------------------------------------------+
  */
  
  A8_TYP6d  (24, "A8 Typ 6d",  SchilderTypenVM._A8_Typ6),
  
 /* A8 Typ 7
  * 
  *           +-----+           +-----+            +-----+
  *           |  B  |           |  B  |            |  B  |
  *           |  7  |           |  6  |            |  5  |
  * +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+
  * | A4  |   +-----+  | A3  |  +-----+  | A2  |   +-----+  | A1  |
  * |     |   | C10 |  |     |  | C 9 |  |     |   | C 8 |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+
  */

  A8_TYP7   (25, "A8 Typ 7",   SchilderTypenVM._A8_Typ7),
  
 /* A8 Typ 7d
  * 
  *           +-----+           +-----+            +-----+
  *           |  B  |           |  B  |            |  B  |
  *           |  7  |           |  6  |            |  5  |
  * +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+
  * | A4  |   +-----+  | A3  |  +-----+  | A2  |   +-----+  | A1  |
  * |     |   | C10 |  |     |  | C 9 |  |     |   | C 8 |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+
  * +-------------------------------------------------------------+
  * |                             D11                             |
  * +-------------------------------------------------------------+
  */
  
  A8_TYP7d  (26, "A8 Typ 7d",  SchilderTypenVM._A8_Typ7),
  
 /* A8 Typ 8
  * 
  * +-----+
  * | A1  |
  * |     |
  * +-----+
  */

  A8_TYP8   (27, "A8 Typ 8",   null                    ),
  
 /* A8 Typ 9
  * 
  *           +-----+                      +-----+
  *           |  B  |                      |  B  |
  *           |  6  |                      |  5  |
  * +-----+   +-----+  +-----+   +-----+   +-----+  +-----+
  * | A4  |   +-----+  | A3  |   | A2  |   +-----+  | A1  |
  * |     |   | C 8 |  |     |   |     |   | C 7 |  |     |
  * +-----+   +-----+  +-----+   +-----+   +-----+  +-----+
  */
  
  A8_TYP9   (28, "A8 Typ 9",   SchilderTypenVM._A8_Typ9),
  
 /* A8 Typ 10
  * 
  *           +-----+           +-----+           +-----+           +-----+
  *           |  B  |           |  B  |           |  B  |           |  B  |
  *           |  9  |           |  8  |           |  7  |           |  6  |
  * +-----+   +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+
  * | A5  |   +-----+  | A4  |  +-----+  | A3  |  +-----+  | A2  |  +-----+  | A1  |
  * |     |   | C13 |  |     |  | C12 |  |     |  | C11 |  |     |  | C10 |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+
  */
  
  A8_TYP10  (29, "A8 Typ 10",  SchilderTypenVM._A8_Typ10),
  
 /* A8 Typ 10d
  * 
  *           +-----+           +-----+           +-----+           +-----+
  *           |  B  |           |  B  |           |  B  |           |  B  |
  *           |  9  |           |  8  |           |  7  |           |  6  |
  * +-----+   +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+
  * | A5  |   +-----+  | A4  |  +-----+  | A3  |  +-----+  | A2  |  +-----+  | A1  |
  * |     |   | C13 |  |     |  | C12 |  |     |  | C11 |  |     |  | C10 |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+  +-----+
  * +------------------------------------------------------------------------------+
  * |                                       D14                                    |
  * +------------------------------------------------------------------------------+
  */

  A8_TYP10d (30, "A8 Typ 10d", SchilderTypenVM._A8_Typ10),
  
 /* A8 Typ 11
  * 
  *           +-----+           +-----+           
  *           |  B  |           |  B  |           
  *           |  6  |           |  5  |           
  * +-----+   +-----+  +-----+  +-----+  +-----+    +-----+
  * | A4  |   +-----+  | A3  |  +-----+  | A2  |    | A1  |
  * |     |   | C8 |   |     |  | C 7 |  |     |    |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+    +-----+
  */
  
  A8_TYP11  (31, "A8 Typ 11",  SchilderTypenVM._A8_Typ11),
  
 /* A8 Typ 11d
  * 
  *           +-----+           +-----+           
  *           |  B  |           |  B  |           
  *           |  6  |           |  5  |           
  * +-----+   +-----+  +-----+  +-----+  +-----+    +-----+
  * | A4  |   +-----+  | A3  |  +-----+  | A2  |    | A1  |
  * |     |   | C8 |   |     |  | C 7 |  |     |    |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+    +-----+
  * +------------------------------------------+
  * |                    D9                    |
  * +------------------------------------------+
  */
  
  A8_TYP11d (32, "A8 Typ 11d", SchilderTypenVM._A8_Typ11),
  
 /* A8 Typ 12
  * 
  *           +-----+           +-----+                             +-----+
  *           |  B  |           |  B  |                             |  B  |
  *           |  8  |           |  7  |                             |  6  |
  * +-----+   +-----+  +-----+  +-----+  +-----+           +-----+  +-----+  +-----+
  * | A5  |   +-----+  | A4  |  +-----+  | A3  |           | A2  |  +-----+  | A1  |
  * |     |   | C11 |  |     |  | C10 |  |     |           |     |  | C9  |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+           +-----+  +-----+  +-----+
  */
  
  A8_TYP12  (33, "A8 Typ 12",  SchilderTypenVM._A8_Typ12),
  
 /* A8 Typ 12d
  * 
  *           +-----+           +-----+                             +-----+
  *           |  B  |           |  B  |                             |  B  |
  *           |  8  |           |  7  |                             |  6  |
  * +-----+   +-----+  +-----+  +-----+  +-----+           +-----+  +-----+  +-----+
  * | A5  |   +-----+  | A4  |  +-----+  | A3  |           | A2  |  +-----+  | A1  |
  * |     |   | C11 |  |     |  | C10 |  |     |           |     |  | C9  |  |     |
  * +-----+   +-----+  +-----+  +-----+  +-----+           +-----+  +-----+  +-----+
  * +------------------------------------------+
  * |                    D12                   |
  * +------------------------------------------+
  */
  
  A8_TYP12d (34, "A8 Typ 12d", SchilderTypenVM._A8_Typ12),
  
 /*           +-----+           
  *           |  B  |           
  *           |  3  |           
  * +-----+   +-----+  +-----+  
  * | A2  |   +-----+  | A1  |  
  * |     |   | C 4 |  |     |  
  * +-----+   +-----+  +-----+  
  */

  A8_TYP13  (35, "A8 Typ 13",  SchilderTypenVM._A8_Typ13),
  
 /* A8 Typ 14
  * 
  *                     +-----+           +-----+            +-----+
  *                     |  B  |           |  B  |            |  B  |
  *                     |  7  |           |  6  |            |  5  |
  * +-----+   +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+  +-----+
  * | T13 |   | A4  |   +-----+  | A3  |  +-----+  | A2  |   +-----+  | A1  |  | T12 |
  * |     |   |     |   | C10 |  |     |  | C 9 |  |     |   | C 8 |  |     |  |     |
  * +-----+   +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+  +-----+
  *                                                                   | d11 |
  *                                                                   |     |
  *                                                                   +-----+
  */
  
  A8_TYP14  (36, "A8 Typ 14",  SchilderTypenVM._A8_Typ14),
  
 /* A8 Typ 14d
  * 
  *                     +-----+           +-----+            +-----+
  *                     |  B  |           |  B  |            |  B  |
  *                     |  7  |           |  6  |            |  5  |
  * +-----+   +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+  +-----+
  * | T13 |   | A4  |   +-----+  | A3  |  +-----+  | A2  |   +-----+  | A1  |  | T12 |
  * |     |   |     |   | C10 |  |     |  | C 9 |  |     |   | C 8 |  |     |  |     |
  * +-----+   +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+  +-----+
  *                                                                   | d11 |
  *                                                                   |     |
  *                                                                   +-----+  
  * +-----------------------------------------------------------------------+
  * |                                       D14                             |
  * +-----------------------------------------------------------------------+
  */

  A8_TYP14d (37, "A8 Typ 14d", SchilderTypenVM._A8_Typ14),
  
 /* A8 Typ 15
  * 
  * +-----+             +-----+           +-----+            +-----+  +-----+
  * |BL15 |             |  B  |           |  B  |            |  B  |  |BL14 |
  * +-----+             |  7  |           |  6  |            |  5  |  +-----+
  * +-----+   +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+  +-----+
  * | T13 |   | A4  |   +-----+  | A3  |  +-----+  | A2  |   +-----+  | A1  |  | T12 |
  * |     |   |     |   | C10 |  |     |  | C 9 |  |     |   | C 8 |  |     |  |     |
  * +-----+   +-----+   +-----+  +-----+  +-----+  +-----+   +-----+  +-----+  +-----+
  *                                                                   | d11 |
  *                                                                   |     |
  *                                                                   +-----+
  */
  
  A8_TYP15  (38, "A8 Typ 15",  SchilderTypenVM._A8_Typ15),
  
 /* A8 Typ 16
  * 
  * +-----+   +-----+   +-----+
  * | BL3 |   | BL2 |   | BL1 |
  * +-----+   +-----+   +-----+
  * +-----+   +-----+   +-----+
  * |  T5 |   |  W6 |   |  T4 |
  * +-----+   +-----+   +-----+
  */

  A8_TYP16  (39, "A8 Typ 16",  null                     ),
  
 /* A8 Typ 17
  * 
  * +-----+ 
  * | BL1 | 
  * +-----+ 
  * +-----+ 
  * |  W2 | 
  * +-----+ 
  */

  A8_TYP17  (40, "A8 Typ 17",  null                     ),
  
 /* A8 Typ 18
  * 
  * +-----+   +-----+
  * | BL2 |   | BL1 |
  * +-----+   +-----+
  * +-----+   +-----+
  * |  T4 |   |  T3 |
  * +-----+   +-----+
  */

  A8_TYP18  (41, "A8 Typ 18",  null                     ),
  
 /* A8 Typ 19
  * 
  * +-----+   +-----+
  * |BAQ2 |   |BAQ1 |
  * +-----+   +-----+
  */

  A8_TYP19  (42, "A8 Typ 19",  null                     ),

	/*------------------------------------------------------------------------------
	 * A2
	 *------------------------------------------------------------------------------*/

  /* A2 Typ BAG
   * 
   * +-----+   +-----+
   * |BAG2 |   |BAG1 |
   * +-----+   +-----+
   */

   A2_TYPBAG  (43, "A2 Typ BAG",  null),

   /* A2 Typ A/B/C
    * 
  	* +------+ 
		* |  A1  | 
		* |      | 
		* +------+ 
		* +------+ 
		* |  B1  | 
		* |      | 
		* +------+ 
	  * +------+
	  * |  C1  |
	  * +------+
	  */

    A2_TYP_ABC (44, "A2 Typ A/B/C",  SchilderTypenVM._A2_TypABC),

   /* A2 Typ A/B
    * 
	 * +------+ 
	 * |  A1  | 
	 * |      | 
	 * +------+ 
	 * +------+ 
	 * |  B1  | 
	 * |      | 
	 * +------+ 
   */

    A2_TYP_AB (45, "A2 Typ A/B",  SchilderTypenVM._A2_TypAB),

  ;

  /**
   * Name fr den Anzeigetyp (wird bei der Auswahl der Anzeigetypen verwendet)
   */
  private String _name;

	/**
	 * Kodierung im DaV
	 */
	private int _kodierung;
	
	/**
	 * Schildertyp fr die Verriegelungsmatrix
	 */
	private Design _designVM;
	
	/**
	 * Konstruktor
	 * @param kodierung Kodierung im DaV
	 */
  private EAnzeigeQuerschnittTyp (int kodierung, String name)
  {
  	_kodierung = kodierung;
  	
  	_name = name;
  }

	/**
	 * Konstruktor des Enums
	 * @param kodierung Kodierung im DaV
	 * @param name Name des Typs (wird in den Parameterdialogen verwendet)
	 * @param klasseAllgemeineVerriegelung Klasse mit dem Composite zur Versorgung der 'Allgemeinen Verriegelungsmatrix'
	 * @param klasseSpezielleVerriegelung Klasse mit dem Composite zur Versorgung der 'Speziellen Verriegelungsmatrix'
	 */
  private EAnzeigeQuerschnittTyp (int kodierung, String name, Design typVM)
  {
  	_kodierung = kodierung;
  	
  	_name = name;
  	
  	_designVM = typVM;
  }

	/**
   * Methode liefert name zurck
   * @return name
   */
  public String getName()
  {
  	return _name;
  }

  /**
   * Methode liefert den Typ zu einem bestimmten Namen
   * @param name Name
   * @return Typ
   */
	public static EAnzeigeQuerschnittTyp getTyp (String name)
  {
		for (EAnzeigeQuerschnittTyp aqt : values())
			if (aqt.getName().equals(name))
				return aqt;
		
		return null;
  }

  /**
   * Methode liefert den Typ zu einer Kodierung
   * @param name Name
   * @return Typ
   */
	public static EAnzeigeQuerschnittTyp getTyp (int kodierung)
  {
		for (EAnzeigeQuerschnittTyp aqt : values())
			if (aqt.getKodierung() == kodierung)
				return aqt;
		
		return UNDEFINIERT;
  }
	
	/**
   * Methode liefert kodierung zurck
   * @return kodierung
   */
  public int getKodierung()
  {
  	return _kodierung;
  }

	/**
   * Methode liefert designVM zurck
   * @return designVM
   */
  public Design getDesignVM()
  {
  	return _designVM;
  }

	/**
   * Methode setzt die Variable designVM
   * @param designVM zu bernehmender Wert fr designVM
   */
  public void setDesignVM(Design designVM)
  {
  	_designVM = designVM;
  }
}
