package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.GridData;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class ProjektSpezifischeEinstellungen
{
	/**
	 * Enum zum Kodieren der unterschiedlichen SBA
	 * @author ObertM
	 *
	 */
	public enum SBA
	{
		/**
		 * bisher nicht bercksichtigte SBA
		 */
	  UNBEKANNT,
	  
	  /**
	   * SBA A14
	   */
	  A14,
	  
	  /**
	   * SBA B27
	   */
	  B27,
	  
	  /**
	   * SBA A8
	   */
	  A8,

	  /**
	   * SBA A81
	   */
	  A81,

	  /**
	   * SBA A2
	   */
	  A2,

	};
	
	/**
	 * Pid der SBA B27
	 */
	private static String _pidSBAB27 = "bw.uz.B27.streckenBeeinflussungsAnlage.SBA";

	/**
	 * Pid der SBA A14
	 */
	private static String _pidSBAA14 = "st.sba.uz.a14.streckenBeeinflussungsAnlage.SBA";

	/**
	 * Pid der SBA A8
	 */
	private static String _pidSBAA8 = "bw.uz.A8.sba_stgt.streckenBeeinflussungsAnlage.SBA";

	/**
	 * Pid der SBA A81
	 */
	private static String _pidSBAA81 = "bw.uz.A81.sba_stgt.streckenBeeinflussungsAnlage.SBA";

	/**
	 * Pid der SBA A2 Sachsen-Anhalt
	 */
	private static String _pidSBAA2 = "st.sba.uz.a2.streckenBeeinflussungsAnlage.SBA";

	/**
	 * Aktuelle Streckenbeeinflussungsanlage
	 */
	private static SBA _sba;
	
	/**
	 * Pid der SBA fr die das Plugin arbeitet
	 */
	private static String _pidSba;
	
	/**
	 * Einzige Instanz der Klasse
	 */
	private static ProjektSpezifischeEinstellungen _instance = null;
	
	/**
	 * Privater Konstruktor der Klasse
	 */
	private ProjektSpezifischeEinstellungen()
  {
		bestimmeSBA ();
  }
	
	/**
	 * Methode zum Bestimmen der SBA
	 */
	private static void bestimmeSBA()
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    List<SystemObject> imDaVVorhandeneSBA = new ArrayList<SystemObject>();
    List<SystemObject> tmp = connection.getDataModel().getType( "typ.sswSbaStreckenBeeinflussungsAnlage" ).getElements();
    for (SystemObject so : tmp)
    	if (!so.getPid().startsWith("swarco.pua"))
    		imDaVVorhandeneSBA.add(so);
    
//    System.out.println("ProjektSpezifischeEinstellungen.bestimmeSBA() " + imDaVVorhandeneSBA.size());
    
    /*
     * Prfen ob mehrere SBA im DaV versorgt sind
     */
    if (imDaVVorhandeneSBA.size() == 1)
    	_pidSba = imDaVVorhandeneSBA.get(0).getPid();
    else
    {
    	/*
    	 * Falls mehrere vorhanden sind, prfen ob eine der vordefinierten SBA
    	 * vorhanden ist, falls ja wird die erste bekannte SBA verwendet.
    	 */

      for (SystemObject so : imDaVVorhandeneSBA)
      {
        if ( so.getPid().equals(_pidSBAA14) ||
        		 so.getPid().equals(_pidSBAB27) ||
        		 so.getPid().equals(_pidSBAA8 ) ||
        		 so.getPid().equals(_pidSBAA81) ||
        		 so.getPid().equals(_pidSBAA2) )
        {
        	_pidSba = so.getPid();
        	break;
        }
      }
    }
    
    _sba = SBA.UNBEKANNT;
    
    if (_pidSba.equals(_pidSBAA14))
     	_sba = SBA.A14;

    if (_pidSba.equals(_pidSBAB27))
    	_sba = SBA.B27;

    if (_pidSba.equals(_pidSBAA8))
    	_sba = SBA.A8;

    if (_pidSba.equals(_pidSBAA81))
    	_sba = SBA.A81;

    if (_pidSba.equals(_pidSBAA2))
    	_sba = SBA.A2;

    System.out.println("ProjektSpezifischeEinstellungen.bestimmeSBA() Aktuelle SBA: " + _sba);
  }

	/**
	 * Methode liefert die einzige Instanz der Klasse
	 * @return
	 */
	public static ProjektSpezifischeEinstellungen getInstance ()
	{
  	if (_instance == null)
  	{
  		synchronized (ProjektSpezifischeEinstellungen.class)
      {
  			if (_instance == null)
  			{
  				_instance = new ProjektSpezifischeEinstellungen();
  			}
      }
  	}
    return _instance;
	}
	
	/**
   * Methode liefert pidSba zurck
   * @return pidSba
   */
  public static String getPidSba()
  {
  	if (_pidSba == null)
  		bestimmeSBA();
  	
  	return _pidSba;
  }

	/**
   * Methode liefert sba zurck
   * @return sba
   */
  public SBA getSba()
  {
  	return _sba;
  }

  /**
   * Methode liefert die vertikale Ausrichtung der Anzeigen bei den Versorgungsdialogen (oben oder unten)
   * @return vertikale Ausrichtung
   */
  public int getAusrichtung ()
  {
  	int ausrichtung = GridData.END;
  	
  	if (_sba == SBA.A8)
  		ausrichtung = GridData.BEGINNING;
  	
  	return ausrichtung;
  }
}
