package de.bsvrz.sba.plugin.parameter.steuerungskern.tools;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;
import java.util.Vector;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeElementFigure;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltAuswahlDialog;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungMoeglicheSchaltgruende;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen.SBA;

/**
 * Klasse enthlt allgemeine Funktionen die von den Regelklassen benutzt werden knnen
 * @author Dambach Werke GmbH
 * @author Matthias Obert
 */
public class Tools
{
	/**
	 * Hashmap mit der Zuordnung der Anzeigen zu den Anzeigequerschnitten
	 */
	private static ConcurrentHashMap<SystemObject, SystemObject[]> _anzeigenZuAq = new ConcurrentHashMap<SystemObject, SystemObject[]>();
	
	/**
   * Parameterloser Konstruktor der Klasse
   */
  public Tools ()
  {
  }

  /**
   * Methode bestimmt die Anzeigetypen die versorgt werden knnen. Dies ist abhngig davon, ob
   * die anlagenspezifischen Parameter oder die Parameter eines Anzeigequerschnitts oder einer
   * Anzeige versorgt werden sollen
   * @param pidObjekt Pid des Objekts fr das die Anzeigetypen bestimmt werden sollen
   * @return Anzeigetypen
   */
  public static Vector<SystemObject> bestimmeAnzeigeTypen (String pidObjekt)
  {
  	Vector<SystemObject> v = new Vector<SystemObject>();
  	
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    
    if (connection != null)
    {
	  	SystemObject so = connection.getDataModel().getObject(pidObjekt);
	  	if (so != null)
	  	{
				/*
	    	 * Anlagenspezifische Paramerer: alle Typen versorgbar
	    	 */
	  		if (so.getType().getPid().equals("typ.sswSbaStreckenBeeinflussungsAnlage"))
	    		v = bestimmeAnzeigeTypenZuAnlage (pidObjekt);
	    	
	  		/*
	  		 * Parameter Anzeigequerschnitt: nur die Typen die an diesem Anzeigequerschnitt
	  		 * mglich sind
	  		 */
	  		if (so.getType().getPid().equals("typ.anzeigeQuerschnitt"))
	  			v = bestimmeAnzeigeTypenZuAnzeigeQuerschnitt (pidObjekt);
	
	  		/*
	  		 * Parameter Anzeige: nur die Typen die an dieser Anzeige
	  		 * mglich sind
	  		 */
	  		if (so.getType().getPid().equals("typ.anzeige"))
	  			v = bestimmeAnzeigeTypenZuAnzeige (pidObjekt);
	  		
	  		/*
	  		 * Objekte noch alphabetisch sortieren
	  		 */
	  		
	  		TreeMap<String, SystemObject> sort = new TreeMap<String, SystemObject>();
	  		for (SystemObject so1 : v)
	  			if (isVerwenden (so1))
	  			  sort.put(so1.getName(), so1);
	  		
	  		v.clear();
	  		Iterator<Entry<String, SystemObject>> it = sort.entrySet().iterator();
	  		while (it.hasNext())
	  		{ 
	  			Entry<String, SystemObject> entry = it.next();
	  			v.add(entry.getValue());
	  		}
	  	}
    }

  	return v;
  }

  /**
   * Methode prft ob der Anzeigetyp verwendet werden soll
   * @param so Systemobjekt des Anzeigetyps
   * @return Info ob Anzeigetyp verwendet werden soll
   */
	private static boolean isVerwenden(SystemObject so)
  {
		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A8)
		{
			/*
			 * Typ D (Freitext) wird nicht bercksichtigt
			 */
		  if (so.getPid().equals("sba.anzeigeTyp.A8.sba_stgt.TypD2"))
		  	return false;
		}
		
    return true;
  }

	/**
   * Methode bestimmt die bentigten Anzeigetypen einer SBA 
   * @param pidAnzeige Pid eines Objekts vom Typ "typ.sswSbaStreckenBeeinflussungsAnlage"
   * @return Anzeigetyp
   */
  private static Vector<SystemObject> bestimmeAnzeigeTypenZuAnlage(String pidObjekt)
  {
		ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		DataModel model = connection.getDataModel();
		
  	Vector<SystemObject> v = new Vector<SystemObject>();

  	for (AnzeigeElementTyp aet : AnzeigeElementTyp.getVersorgteTypen())
  	{
  		SystemObject so = model.getObject(aet.getPid()); 
  		v.add (so);
  	}
  	
    return v;
  }

	/**
   * Methode bestimmt den Anzeigetyp einer Anzeige. 
   * @param pidAnzeige Pid eines Objekts vom Typ "typ.anzeige"
   * @return Anzeigetyp
   */
  private static Vector<SystemObject> bestimmeAnzeigeTypenZuAnzeige ( String pidAnzeige )
  {
  	Vector<SystemObject> v = new Vector<SystemObject>();
  	
  	SystemObject typ = Konfig.getInstance().getWzgTypVonAnzeige(pidAnzeige);
    if (typ != null)
    	v.add(typ);
    
    return v;
  }

	private static Vector<SystemObject> bestimmeAnzeigeTypenZuAnzeigeQuerschnitt(
      String pidAnzeigeQuerschnitt)
  {
		return Konfig.getInstance().getWzgTypenVonAnzeigeQuerschnitt(pidAnzeigeQuerschnitt);
  }

	public static SystemObject getAnzeigeTyp(String pid)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    return connection.getDataModel().getObject(pid);
  }

  /**
   * Methode liefert die im DaV definierten Aspekte sortiert nach Name
   * @return die im DaV definierten Aspekte
   */
  public static Vector<SystemObject> getAspekte ()
  { 	
		List<SystemObject> typen = VerwaltungMoeglicheSchaltgruende.getInstance().getSchaltgruende();

    TreeMap<String, SystemObject> sort = new TreeMap<String, SystemObject>();
    for (SystemObject so : typen)
      sort.put(so.getName(), so);
      
    Vector<SystemObject> v = new Vector<SystemObject>();
    for (String s : sort.keySet())
      v.add (sort.get(s));
      
    return v;
  }

  /**
   * Methode liefert eine Liste mit den Namen der Aspekte
   * @return Liste mit den Namen der Aspekte
   */
  public static String [] getNamenDerAspekte ()
  {
    Vector<SystemObject> aspekte = getAspekte();
    String [] namen = new String [aspekte.size()];
    for (int i=0; i<aspekte.size(); ++i)
      namen [i] = aspekte.get(i).getName();
    
    return namen;
  }
  
  /**
   * Methode liefert den Aspekt zu einem bestimmten Namen
   * @param name Name des gesuchten Aspekts
   * @return Aspekt
   */
	public static Aspect getAspekt (String name)
  {
	  Vector<SystemObject> schaltgruende = getAspekte();
	  for (SystemObject so : schaltgruende)
	  {
	  	if (so.getName().equals(name))
	  		return (Aspect) so;
	  }
	  
	  return null;
  }

  /**
   * Methode liefert den Namen des Anzeigeinhalts eines WVZ-Inhalts. geliefert.
   * @param wvzInhalt WVZInhalt dess Namen ausgegeben werden soll
   * @return Name des WVZInhalts oder der String 'null'
   */
  public static String getWVZInhalt (WVZInhalt wvzInhalt)
  {
    if (wvzInhalt == null)
      return "null";
    else if (wvzInhalt.getAnzeigenInhalt() == null)
      return "null";
    else if (wvzInhalt.getAnzeigenInhalt().getName() == null)
      return null;
    
    return wvzInhalt.getAnzeigenInhalt().getName();
  }

  /**
   * Methode liefert den Namen eines Systemobjekt. Ist das Systemobjekt null
   * oder der Name des Systemobjekts null wird der Text 'null' zurckgeliefert.
   * @param objekt Objekt dess Namen ausgegeben werden soll
   * @return Name des Objekts oder der String 'null'
   */
  public static String getObjektName (SystemObject objekt)
  {
    if (objekt == null)
      return "null";
    else
      if (objekt.getName() == null)
        return "null";
      else
        return objekt.getName();
  }
  
	/**
	 * Text fr Zustand "undefiniert"
	 */
	public static String _undefiniert = "undefiniert";
	
	/**
	 * Methode liefert eine Liste mit allen Systemobjekttypen zu einem bergebenen Basistyp.
	 * Dabei werden automatisch die mglichen Subtypen bestimmt.
	 * @param pidTyp Pid des Basistyps
	 * @return Liste mit Typen (Basistyp + Subtyp)
	 */
  public static List<SystemObjectType> getSystemObjektTypen (String pidTyp)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    
    List<SystemObjectType> liste = new ArrayList<SystemObjectType>();
    
    SystemObjectType sot = connection.getDataModel().getType(pidTyp);
    bestimmeObjektTypen(sot, liste);
    
//    for (SystemObjectType s : liste)
//    	System.out.println("Tools.getSystemObjektTypen() " + s.getPid());
    
    return liste;
  }
  
  /**
   * Methode bestimmt rekursiv die vorhandenen Objekttypen
   * @param sot ausgehendes Objekt
   * @param liste Liste in die die Objekte eingetragen werden
   */
  private static void bestimmeObjektTypen(SystemObjectType sot, List<SystemObjectType> liste)
  {
    if ( sot.getSubTypes().size() == 0 )
    {
    	liste.add(sot);
    }
    else
    {
    	liste.add(sot);
    	
      for (SystemObjectType subTyp : sot.getSubTypes())
      	bestimmeObjektTypen(subTyp, liste);
    }
  }
  
  /**
   * Methode liefert alle Objekte die zu einem Systemobjekttyp gehren. Dabei werden auch
   * eventuelle Subtypen des bergebenen Typs bercksichtigt.
   * @param pidTyp Pid des Objekttyps
   * @return Liste mit Objekten
   */
  public static List<SystemObject> getSystemObjekte (String pidTyp)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    
    List<SystemObjectType> typen = getSystemObjektTypen(pidTyp);
    
    List<SystemObject> objekte = new ArrayList<SystemObject>();
    
    for (SystemObjectType sot : typen)
    {
  	  List<SystemObject> elemente = connection.getDataModel().getType( sot.getPid() ).getElements();
  	  for (SystemObject so : elemente)
  	  {
  	  	if (!so.getPid().startsWith("swarco.pua"))
	  	    if (!objekte.contains(so))
	  	    	objekte.add(so);
  	  }
    }
    
    return objekte;
  }
  
  /**
   * Methode bestimmt eine Liste mit den alphabetisch sortieren Namen der bergebenen Objekte.
   * Zustzlich kann der Text "undefiniert" mit einsortiert werden (bei optionalen Referenzen)
   * @param objekte Liste mit Systemobjekten
   * @param mitUndefiniert Merker ob der Text undefiniert mit ausgegeben werden soll
   * @return Liste mit alphabetisch sortierten Namen
   */
  public static List<String> bestimmeObjektNamen (List<SystemObject> objekte, boolean mitUndefiniert)
  {
  	List<String> l  = new ArrayList<String>();
  	
    for (SystemObject so: objekte)
    	l.add( getName(so) );

    Collections.sort( l );
    if (mitUndefiniert)
      l.add(0, _undefiniert);
    
    return l;
  }

  /**
   * Methode bestimmt eine Liste mit den alphabetisch sortieren Namen der bergebenen Objekte.
   * @param objekte Liste mit Systemobjekten
   * @return Liste mit alphabetisch sortierten Namen
   */
  public static List<String> bestimmeObjektNamen (List<SystemObject> objekte)
  {
  	return bestimmeObjektNamen(objekte, false);
  }
  
  /**
   * Methode liefert die Bezeichnung fr ein Systemobjekt. Hierber kann einheitlich erreicht werden,
   * ob nur der Namen oder der Name oder die Pid oder die Id ausgegeben werden
   * @param systemObject SystemObjekt
   * @return Bezeichnung
   */
  public static String getName (SystemObject systemObject)
  {
  	if (systemObject == null)
  		return null;
  	
  	return systemObject.getNameOrPidOrId();
  }

  /**
   * Methode bestimmt die Hoehe der Zeile fr die Parameter in Abhngigkeit vom Anzeigeelementtyp
   * @param anzeigeTyp Anzeigeelementtyp
   * @return Hhe
   */
  public static int getZeilenHoehe (AnzeigeElementTyp anzeigeTyp)
  {
  	int hoehe = 40;
  	
  	if (anzeigeTyp == AnzeigeElementTyp.WZG_BAQ)
  		hoehe = 90;
  	else if (anzeigeTyp == AnzeigeElementTyp.WZG_TAQ)
  		hoehe = 50;
  	else if (anzeigeTyp == AnzeigeElementTyp.WZG_WAQ)
  		hoehe = 50;
  	else if (anzeigeTyp == AnzeigeElementTyp.WZG_BAG)
  		hoehe = 90;
  	
  	return hoehe;
  }

  /**
   * Methode bestimmt die Hoehe der Zeile fr die Parameter in Abhngigkeit vom Anzeigeelementtyp
   * @param anzeigeTyp Anzeigeelementtyp
   * @return Hhe
   */
  public static int getZeilenHoehe (AnzeigeElementTyp anzeigeTyp, int max)
  {
  	int hoehe = getZeilenHoehe(anzeigeTyp);
  	if (hoehe < max)
  		hoehe = max;
  	
  	return hoehe;
  }

  /**
   * Methode bestimmt den Text der als bezeichnugn fr die Anzeigetypen verwendet werden soll. 
   * U.U. sind die Bezeichnungen aus dem DaV zu lang.
   * @param so System Objekt
   * @return Bezeichnung
   */
  public static String getBezeichnungAnzeigeTyp (SystemObject so)
  {
  	return so.getName().replace("SBA AnzeigeTyp ", "");
  }

  /**
   * Methode liefert die maximale Breite der grafischen Darstellung eines Bildes
   * @return maximale Breite
   */
  public static int getMaxBreiteBild ()
  { 
  	return 50;
  }

	public static void setzeBildInhalte (Label label, WVZInhalt inhalt, AnzeigeElementTyp anzeigeTyp)
	{
		try
		{
		  setzeBildInhalte(label, inhalt, anzeigeTyp, false);
		}
		catch (Exception e) 
		{
			e.printStackTrace();
		}
	}

	/**
	 * Methode setzt einen WVZ Inhalt an einem Label
	 * @param label Label
	 * @param inhalt dazustellender WVZ-Inhalt
	 */
	public static void setzeBildInhalte (Label label, WVZInhalt inhalt, AnzeigeElementTyp anzeigeTyp, boolean packen)
	{
	  if (inhalt != null && inhalt.getAnzeigenInhalt() != null)
	  {
	  	String pidWvzInhalt = inhalt.getPid();

	  	AnzeigeInhaltTyp ait = AnzeigeInhaltTyp.getTyp(pidWvzInhalt);
	  	
	  	if (ait != null)
	  	{
	  	  label.setData(ait.getBildInhalt());
	  	
	      Image imageX = AnzeigeElementFigure.getImage( anzeigeTyp, ait );
	      label.setImage( imageX );
	
	      if (packen)
	        label.getParent().pack(true);
	  	}
	  }
	}
	
	public static void waehleZuPruefendesBild (MouseEvent e, AnzeigeElementTyp anzeigeTyp)
	{
	  boolean mitDontCare          = false;
		boolean mitBeliebig          = true;
		boolean mitNegiertenInhalten = true;
		
		waehleBildInhaltAus(e, mitDontCare, mitBeliebig, mitNegiertenInhalten, anzeigeTyp);	
	}

	public static void waehleZuPruefendesBildMitDontCare (MouseEvent e, AnzeigeElementTyp anzeigeTyp)
	{
	  boolean mitDontCare          = true;
		boolean mitBeliebig          = true;
		boolean mitNegiertenInhalten = true;
		
		waehleBildInhaltAus(e, mitDontCare, mitBeliebig, mitNegiertenInhalten, anzeigeTyp);	
	}

	public static void waehleZuSchaltendesBild (MouseEvent e, AnzeigeElementTyp anzeigeTyp)
	{
	  boolean mitDontCare          = false;
		boolean mitBeliebig          = false;
		boolean mitNegiertenInhalten = false;
		
		waehleBildInhaltAus(e, mitDontCare, mitBeliebig, mitNegiertenInhalten, anzeigeTyp);	
	}
	
	
  /**
   * Method zum Auswhlen eines Bildinhaltes, ausgelst ber einen MouseEvent. Anschlieend wird
   * das ausgewhlte Bild an dem Label gesetzt
   * @param e MousEvent
   * @param anzeigeTyp  AnzeigeTyp
   */
  public static void waehleBildInhaltAus (MouseEvent e, boolean mitDontCare, boolean mitBeliebig, boolean mitNegiertenInhalten, AnzeigeElementTyp anzeigeTyp)
  {
    AnzeigeInhaltAuswahlDialog dialog = new AnzeigeInhaltAuswahlDialog(PlatformUI.getWorkbench().getDisplay()
        .getActiveShell());
    
    AnzeigeInhaltTyp inhalt = dialog.oeffne (anzeigeTyp, mitDontCare, mitBeliebig, mitNegiertenInhalten );
    
    if (inhalt != null)
    {
      Label im = (Label)e.getSource();
      im.setData( inhalt.getBildInhalt() );
      Image image = AnzeigeElementFigure.getImage( anzeigeTyp, inhalt );
      im.setImage( image );
    }           
  }

  /**
   * Methode liefert eine Liste mit den Systemobjekten die zu einem Anzeigequerschnitt
   * gehren. Die Reihenfolge ist dabei Anzeige 1, Anzeige 2, ...
   * @param soAQ Systemobjekt des AQ
   * @return Liste mit Anzeigen
   */
  public static SystemObject[] _getSystemObjekteAnzeigen (SystemObject soAQ)
  {
  	System.out.println("Tools.getSystemObjekteAnzeigen() " + soAQ.getPid());
  	
    if (_anzeigenZuAq.contains(soAQ))
    	return _anzeigenZuAq.get(soAQ);
    
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
	  if (connection == null)
	  	return new SystemObject [0];

  	String pidAQ = soAQ.getPid();
  	
  	String tmp = pidAQ.replace("aq.", "");
  	
  	Vector<SystemObject> soAnzeigen = new Vector<SystemObject>();
  	
    if (connection != null)
    {
      DataModel datenModell = connection.getDataModel();

      int i = 0;
      SystemObject so = null;
      
      do
      {
        String pidAnzeige = "anzeige." + tmp + ".anzeige" + (i+1);
        i++;
        
        System.out.println("Tools.getSystemObjekteAnzeigen() " + pidAnzeige);
        
        so = datenModell.getObject( pidAnzeige );
        
        if (so != null)
        	soAnzeigen.add (so);
        else
        	System.err.println("Tools.getSystemObjekteAnzeigen() Kein Systemobjekt zu PID " + pidAnzeige + " gefunden !");
      }
      while (so != null);
    }

    SystemObject[] alsArray = soAnzeigen.toArray(new SystemObject [soAnzeigen.size()]);
    
    _anzeigenZuAq.put (soAQ, alsArray);
    
    return soAnzeigen.toArray(alsArray);
  }
  
  /**
   * Methode liefert eine Liste mit den Systemobjekten die zu einem Anzeigequerschnitt
   * gehren. Die Reihenfolge ist dabei Anzeige 1, Anzeige 2, ...
   * @param soAQ Systemobjekt des AQ
   * @return Liste mit Anzeigen
   */
  public static SystemObject[] getSystemObjekteAnzeigen (SystemObject soAQ)
  {
//  	System.out.println("Tools.getSystemObjekteAnzeigen() " + soAQ.getPid());
  	
    if (_anzeigenZuAq.contains(soAQ))
    	return _anzeigenZuAq.get(soAQ);
    
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
	  if (connection == null)
	  	return new SystemObject [0];

	  TreeMap<String, SystemObject> sorter = new TreeMap<String, SystemObject>();
	  ConfigurationObject co = (ConfigurationObject)soAQ;
	  NonMutableSet set = co.getNonMutableSet("Anzeigen");
	  if (set != null)
	  {
	    List<SystemObject> anzeigen = set.getElements();
	    for (SystemObject so : anzeigen)
      	sorter.put(so.getPid(), so);
	  }

	  SystemObject a[] = new SystemObject [sorter.size()];
	  int index = 0;
	  Iterator<Entry<String, SystemObject>> it = sorter.entrySet().iterator();
	  while (it.hasNext())
	  {
	  	Entry<String, SystemObject> entry = it.next();
	  	a [index++] = entry.getValue();
	  	
//	  	System.out.println("Tools.getSystemObjekteAnzeigen() " + entry.getValue().getPid());
	  }

    _anzeigenZuAq.put (soAQ, a);

	  return a; 
  }
}
