/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.editor;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sba.plugin.parametereditor.Activator;

/**
 * Hilfsklasse zum ffnen des Parametereditors.
 * 
 * @author HofmannJ
 * 
 */
public class OpenViewParameterEditor
{
	/**
	 * ID der zu ffnenden View
	 */
	private static String _viewID = ViewParameterEditor.ID;

	/**
	 * Array fr die HEX-Konvertierung der MD5-Prfsumme
	 */
	private static final char[] _digits = { '0', '1', '2', '3', '4', '5', '6',
			'7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

	/**
	 * ffnet eine {@link ViewParameterEditor}. An Hand der bergebenen Liste von
	 * {@link ParameterEditorKapsel}n wird fr die View eine eindeutige Secondary
	 * ID bestimmt. Eine View mit den gleichen {@link ParameterEditorKapsel}n
	 * kann nur 1x offen sein.
	 * 
	 * @param kapselListe
	 *          Liste von {@link ParameterEditorKapsel}n, an Hand derer die View
	 *          initialisiert wird.
	 * @param instanziieren
	 *          Wenn <code>true</code>, dann wird die View mit den Daten der
	 *          {@link ParameterEditorKapsel}n initialisiert und die
	 *          entsprechenden Daten werden vom DaV geladen. Bei
	 *          <code>false</code> ist dies nicht der Fall.
	 * @return Liefert die Instanz der Klasse {@link ViewParameterEditor} zurck,
	 *         welche geffnet wurde.
	 */
	public static ViewParameterEditor openView(
			LinkedList<ParameterEditorKapsel> kapselListe, boolean instanziieren, boolean editable)
	{
		ViewParameterEditor re = null;

		String secId = "";

		if (kapselListe.size() > 0)
		{
			Iterator<ParameterEditorKapsel> it = kapselListe.iterator();
			while (it.hasNext())
			{
				ParameterEditorKapsel kapsel = it.next();
				secId = secId + kapsel.getId() + "; ";
			}
		}

		/*
		 * Letztes Semikolon und Leerzeichen entfernen
		 */
		if (secId.length() > 2)
		{
			secId = secId.substring(0, secId.length() - 2);
		}

		/*
		 * MD5-Prfsumme der ID
		 */
		String secIdHex = md5sum(secId);

		/*
		 * ffnen der View
		 */
		IWorkbenchWindow window = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow();

		try
		{
			window.getActivePage().showView(_viewID, secIdHex,
					IWorkbenchPage.VIEW_VISIBLE);

			IViewReference reference = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage().findViewReference(
							_viewID, secIdHex);
			IViewPart view = reference.getView(true);
			if (view instanceof ViewParameterEditor)
			{
				re = (ViewParameterEditor) view;
				window.getActivePage().activate(re);
			}

		}
		catch (PartInitException e)
		{
			String msg = "Die ViewParameterEditor (ID: " + _viewID + ", SecId: "
					+ secIdHex + ") konnte nicht geffnet werden!.";

			Activator.getDefault().getLog().log(
					new Status(Status.WARNING, Activator.PLUGIN_ID, msg, e));
		}

		if ((instanziieren) && (re != null))
		{
			re.setEditable(editable);
//			re.bearbeiteKapseln(kapselListe);
			re.bearbeiteKapselnJob(kapselListe);
		}

		return re;
	}

	private static String hexEncode(byte[] bytes)
	{
		StringBuffer s = new StringBuffer(bytes.length * 2);
		for (int i = 0; i < bytes.length; i++)
		{
			byte b = bytes[i];
			s.append(_digits[(b & 0xf0) >> 4]);
			s.append(_digits[b & 0x0f]);
		}
		return s.toString();
	}

	/**
	 * Liefwert den MD5-Hashwert eines {@link String}s.
	 * 
	 * @param wert
	 *          {@link String}
	 * @return MD5-Hashwert des {@link String}s.
	 */
	private static String md5sum(String wert)
	{
		String re = "";

		try
		{
			MessageDigest md = MessageDigest.getInstance("MD5");
			byte[] digest = md.digest(wert.getBytes());
			re = hexEncode(digest);

		}
		catch (NoSuchAlgorithmException e)
		{
			e.printStackTrace();
		}

		return re;
	}
}
