package de.bsvrz.sba.plugin.zlt.dav;

import java.util.Date;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Umsetzung der ATG <code>atg.tlsZltProgrammaktivierung</code>.
 * 
 * @author Swarco Traffic Systems GmbH
 * 
 */
public class ZltProgrammaktivierung extends AbstractAtgData
{
	private Date _zeitstempel;
	private String _status;
	private Long _programmNummer;
	private String _kuerzel;

	public ZltProgrammaktivierung() {
		super();
	}

	public ZltProgrammaktivierung(ResultData result) {
		super();
		initFromResult(result);
	}

	public ZltProgrammaktivierung(SystemObject so) {
		super(so);
	}

	public ZltProgrammaktivierung(SystemObject so, Date zeitstempel,
			String status, Long programmNummer, String kuerzel)
	{
		super(so);
		_zeitstempel = zeitstempel;
		_status = status;
		_programmNummer = programmNummer;
		_kuerzel = kuerzel;
	}

	@Override
	public void initFromResult(ResultData result)
	{
		if (result != null)
		{
			setSo(result.getObject());

			// Zeitstempel
			setZeitstempel(new Date(result.getDataTime()));

			if (result.hasData())
			{
				Data data = result.getData();

				// // Zeitstempel
				// long zeitstempel = data.getTimeValue("Zeitstempel").getMillis();
				// setZeitstempel(new Date(zeitstempel));

				// Status
				String status = data.getScaledValue("Status").getText();
				setStatus(status);

				// Programm-Nr
				long programmNr = data.getUnscaledValue("Programm-Nr").longValue();
				setProgrammNummer(programmNr);

				// Krzel
				String kuerzel = data.getTextValue("Krzel").getText();
				setKuerzel(kuerzel);

			} else
			{
				// setZeitstempel(null);
				setStatus(null);
				setProgrammNummer(null);
				setKuerzel(null);
			}
		}
	}

	public Date getZeitstempel()
	{
		return _zeitstempel;
	}
	public void setZeitstempel(Date zeitstempel)
	{
		_zeitstempel = zeitstempel;
	}
	public String getStatus()
	{
		return _status;
	}
	public void setStatus(String status)
	{
		_status = status;
	}
	public Long getProgrammNummer()
	{
		return _programmNummer;
	}
	public void setProgrammNummer(Long programmNummer)
	{
		_programmNummer = programmNummer;
	}
	public String getKuerzel()
	{
		return _kuerzel;
	}
	public void setKuerzel(String kuerzel)
	{
		_kuerzel = kuerzel;
	}

	@Override
	public String toString()
	{
		return "ZltProgrammaktivierung [_kuerzel=" + _kuerzel
				+ ", _programmNummer=" + _programmNummer + ", _status=" + _status
				+ ", _zeitstempel=" + _zeitstempel + ", toString()=" + super.toString()
				+ "]";
	}

	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + ((_kuerzel == null) ? 0 : _kuerzel.hashCode());
		result = prime * result
				+ ((_programmNummer == null) ? 0 : _programmNummer.hashCode());
		result = prime * result + ((_status == null) ? 0 : _status.hashCode());
		result = prime * result
				+ ((_zeitstempel == null) ? 0 : _zeitstempel.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj)
	{
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		ZltProgrammaktivierung other = (ZltProgrammaktivierung) obj;
		if (_kuerzel == null)
		{
			if (other._kuerzel != null)
				return false;
		} else if (!_kuerzel.equals(other._kuerzel))
			return false;
		if (_programmNummer == null)
		{
			if (other._programmNummer != null)
				return false;
		} else if (!_programmNummer.equals(other._programmNummer))
			return false;
		if (_status == null)
		{
			if (other._status != null)
				return false;
		} else if (!_status.equals(other._status))
			return false;
		if (_zeitstempel == null)
		{
			if (other._zeitstempel != null)
				return false;
		} else if (!_zeitstempel.equals(other._zeitstempel))
			return false;
		return true;
	}

}
