/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;

/**
 * Initialisiert die Prferenzen dieses Plug-Ins.
 * 
 * @author HofmannJ
 * 
 */
public class PreferenceInit extends AbstractPreferenceInitializer
{
	/**
	 * Name des Properies mit dem Delay des Workflow-Weiterschalten Dialoges fr
	 * die Wiedervorlage.
	 */
	public static final String PROP_DELAY_WIEDERVORLAGE_WF_WEITERSCHALTEN = "DELAY_WIEDERVORLAGE_WF_WEITERSCHALTEN";

	/**
	 * Name des Propertiers mit dem Delay des Schaltempfehlungs-Dialoges fr die
	 * Wiedervorlage.
	 */
	public static final String PROP_DELAY_WIEDERVORLAGE_SCHALTEMPFEHLUNG = "DELAY_WIEDERVORLAGE_SCHALTEMPFEHLUNG";

	public static final String PROP_PATH_SND_WF = "PATH_SND_WF";
	public static final String PROP_DELAY_SND_WF = "DELAY_SND_WF";
	public static final String PROP_PATH_SND_WEITERSCHALTEN = "PATH_SND_WEITERSCHALTEN";
	public static final String PROP_DELAY_SND_WEITERSCHALTEN = "DELAY_SND_WEITERSCHALTEN";
	public static final String PROP_SND_WF_AKTIV = "SND_WF_AKTIV";
	public static final String PROP_SND_WEITERSCHALTEN_AKTIV = "SND_WEITERSCHALTEN_AKTIV";
	public static final String PROP_SCHALTEMPFEHLUNG_INAKTIV = "PROP_SCHALTEMPFEHLUNG_INAKTIV";

	public PreferenceInit()
	{
		super();
	}

	@Override
	public void initializeDefaultPreferences()
	{
		int delay = 1 * 60;
		Activator.getDefault().getPreferenceStore().setDefault(
				PROP_DELAY_WIEDERVORLAGE_WF_WEITERSCHALTEN, delay);
		Activator.getDefault().getPreferenceStore().setDefault(
				PROP_DELAY_WIEDERVORLAGE_SCHALTEMPFEHLUNG, delay);

		Activator.getDefault().getPreferenceStore().setDefault(PROP_DELAY_SND_WF,
				10);
		Activator.getDefault().getPreferenceStore().setDefault(
				PROP_DELAY_SND_WEITERSCHALTEN, 10);

		Activator.getDefault().getPreferenceStore().setDefault(PROP_SND_WF_AKTIV,
				true);
		Activator.getDefault().getPreferenceStore().setDefault(
				PROP_SND_WEITERSCHALTEN_AKTIV, true);
	}

	/**
	 * Liefert das Delay in Sekunden fr die Wiedervorlage des
	 * Workflow-Weiterschalten Dialoges.
	 * 
	 * @return
	 */
	public static int getDelayWiedervorlageWfWeiterschalten()
	{
		int re = 0;

		re = Activator.getDefault().getPreferenceStore().getInt(
				PROP_DELAY_WIEDERVORLAGE_WF_WEITERSCHALTEN);

		return re;
	}

	/**
	 * Liefert das Delay in Sekunden fr die Wiedervorlage des Schaltempfehlungs
	 * Dialoges.
	 * 
	 * @return
	 */
	public static int getDelayWiedervorlageSchaltempfehlung()
	{
		int re = 0;

		re = Activator.getDefault().getPreferenceStore().getInt(
				PROP_DELAY_WIEDERVORLAGE_SCHALTEMPFEHLUNG);

		return re;
	}

	/**
	 * Liefert das Delay in Sekunden fr den Sound des Workflow-Dialoges.
	 * 
	 * @return
	 */
	public static int getDelaySndWf()
	{
		int re = 0;

		re = Activator.getDefault().getPreferenceStore().getInt(PROP_DELAY_SND_WF);

		return re;
	}

	/**
	 * Liefert den Pfad zum Sound des Workflows.
	 * 
	 * @return
	 */
	public static String getPathSndWf()
	{
		String re = null;

		re = Activator.getDefault().getPreferenceStore()
				.getString(PROP_PATH_SND_WF);

		return re;
	}

	/**
	 * Liefert das Delay in Sekunden fr den Sound des Weiterschalten-Dialoges.
	 * 
	 * @return
	 */
	public static int getDelaySndWeiterschalten()
	{
		int re = 0;

		re = Activator.getDefault().getPreferenceStore().getInt(
				PROP_DELAY_SND_WEITERSCHALTEN);

		return re;
	}

	/**
	 * Liefert den Pfad zum Sound des Weiterschalten-Dialoges.
	 * 
	 * @return
	 */
	public static String getPathSndWeiterschalten()
	{
		String re = null;

		re = Activator.getDefault().getPreferenceStore().getString(
				PROP_PATH_SND_WEITERSCHALTEN);

		return re;
	}

	public static boolean isSndWfAktiv()
	{
		return Activator.getDefault().getPreferenceStore().getBoolean(
				PROP_SND_WF_AKTIV);
	}

	public static boolean isSndWeiterschaltenAktiv()
	{
		return Activator.getDefault().getPreferenceStore().getBoolean(
				PROP_SND_WEITERSCHALTEN_AKTIV);
	}
	
	public static boolean isSchaltempfehlungInaktiv()
	{
	  return Activator.getDefault().getPreferenceStore().getBoolean(
	      PROP_SCHALTEMPFEHLUNG_INAKTIV);
	}

}
