/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Kommunikation bzgl. ATG <code>atg.tsfParaAblaufTSF</code> mit dem DaV.<br>
 * Z.Zt. kann nur die Anzahl der Stufen eines Programms bestimmt werden.
 * 
 * @author HofmannJ
 * 
 */
public class DaVKomTSFParaAblaufTSF
{
	/**
	 * Speichert die Singleton-Instanz dieser Klasse.
	 */
	private static DaVKomTSFParaAblaufTSF _instance = null;

	/**
	 * Privater Konstruktor.
	 */
	private DaVKomTSFParaAblaufTSF()
	{
	}

	/**
	 * Liefert die Singleton-Instanz dieser Klasse.
	 * 
	 * @return Singleton-Instanz dieser Klasse.
	 */
	public static DaVKomTSFParaAblaufTSF getInstance()
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFParaAblaufTSF.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFParaAblaufTSF();
				}
			}
		}

		return _instance;
	}

	/**
	 * Liefert die Verbindung zum DaV.
	 * 
	 * @return Verbindung zum DaV oder <code>null</code>, falls nicht verfgbar.
	 */
	private ClientDavInterface getCon()
	{
		return DaVVerbindung.getInstanz().getDaVVerbindung();
	}

	/**
	 * Bestimmt zu einem Abschnitt die Anzahl der Stufen fr ein bestimmtes
	 * Programm.
	 * 
	 * @param soAbschnitt
	 *          SystemObjekt des Abschnitts vom Typ <code>typ.tsfAbschnitt</code>.
	 * @param programm
	 *          Das Programm.
	 * @return Liefert einen Wert >-1, wenn die Anzahl der Stufen bestimmt werden
	 *         konnte.
	 */
	public int bestimmeAnzahlStufen(SystemObject soAbschnitt, E_TSFBefehl programm)
	{
		int re = -1;

		if ((soAbschnitt != null) && (programm != null) && (getCon() != null))
		{
			DataModel dataModel = getCon().getDataModel();
			AttributeGroup atg = dataModel
					.getAttributeGroup(Defs.ATG_TSF_PARA_ABLAUF_TSF);
			Aspect asp = dataModel.getAspect(Defs.ASP_PARAMETER_SOLL);
			DataDescription desc = new DataDescription(atg, asp);

			ResultData resultData = getCon().getData(soAbschnitt, desc, 50l);
			if ((resultData != null) && (resultData.hasData()))
			{
				Data data = resultData.getData();

				Array aPrg = data.getArray("Programme");
				for (int i = 0; i < aPrg.getLength(); i++)
				{
					Data prg = aPrg.getItem(i);
					if (prg.getUnscaledValue("Programm").intValue() == programm.getWert())
					{
						re = prg.getArray("Stufen").getLength();
						break;
					}
				}
			}
		}

		return re;
	}

}
