/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.Date;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung.ZustandSammlung;

/**
 * Enthlt verschiedene Definitionen fr den Betriebs dieses Plug-Ins.
 * 
 * @author HofmannJ
 * 
 */
public class Defs
{
	public static final SimpleDateFormat SDF = new SimpleDateFormat(
			"dd.MM.yyyy HH:mm:ss");

	/*
	 * Abschnitte
	 */
	
//	Abschnitt 1 AS Stuttgart-Mhringen bis AK Stuttgart FR Karlsruhe (bw.uz.A8.sba_stgt.tsf.abschnitt1.W)
	public static final String PID_BER1_ABS1 = "bw.uz.A8.sba_stgt.tsf.abschnitt1.W";
//	Abschnitt 2 AK Stuttgart bis As Stuttgart-Mhringen FR Mnchen (bw.uz.A8.sba_stgt.tsf.abschnitt2.O)
	public static final String PID_BER1_ABS2 = "bw.uz.A8.sba_stgt.tsf.abschnitt2.O";
//	public static final String PID_BER2_ABS3 = "tsf.mainzerring.bereich2.abschnitt3.FRMainz";
//	public static final String PID_BER2_ABS4 = "tsf.mainzerring.bereich2.abschnitt4.FRMainz";

	/**
	 * Dialoge nur ffnen, wenn die eigene ID bermittelt wurde.
	 */
	public static final boolean AUF_GENAUE_ID_ACHTEN = true;

	/**
	 * Typ <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 */
//	protected static final String TYP_TEMP_SEITENSTREIFEN_FREIGABE = "typ.temporaereSeitenstreifenFreigabe";
	protected static final String TYP_TEMP_SEITENSTREIFEN_FREIGABE = "typ.sswSbaUeSeitenstreifenFreigabe";

	/**
	 * Attributgruppe <code>atg.tsfWorkflow</code>.
	 */
	protected static final String ATG_TSF_WORKFLOW = "atg.sswSbaTsfWorkflow";

	/**
	 * Aspekt <code>asp.tsfZustand</code>.
	 */
	public static final String ASP_TSF_ZUSTAND = "asp.sswSbaTsfZustand";
	
	/**
	 * Aspekt <code>asp.tsfZustand</code>.
	 */
	public static final String ASP_ANALYSE = "asp.analyse";

	/**
	 * Typ <code>typ.tsfAbschnitt</code>.
	 */
	public static final String TYP_TSF_ABSCHNITT = "typ.sswSbaTsfAbschnitt";

	/**
	 * Atg <code>atg.tsfAbschnittBefehl</code>.
	 */
	public static final String ATG_TSF_ABSCHNITT_BEFEHL = "atg.sswSbaTsfAbschnittBefehl";

	/**
	 * Asp <code>asp.parameterSoll</code>.
	 */
	protected static final String ASP_PARAMETER_SOLL = "asp.parameterSoll";
	/**
	 * Asp <code>asp.tsfVorgabe</code>.
	 */
	public static final String ASP_TSF_VORGABE = "asp.sswSbaTsfVorgabe";

	/**
	 * Asp <code>asp.vorgabe</code>.
	 */
	protected static final String ASP_VORGABE = "asp.vorgabe";

	/**
	 * Asp <code>asp.tsfVorgabe</code>.
	 */
	protected static final String ASP_TSF_VORGABE_ZUSTAND = "asp.sswSbaTsfVorgabeZustand";

	/**
	 * Asp <code>asp.eigenschaften</code>.
	 */
	protected static final String ASP_EIGENSCHAFTEN = "asp.eigenschaften";

	/**
	 * ATG <code>atg.tsfWeiterSchalten</code>.
	 */
	protected static final String ATG_TSF_WEITER_SCHALTEN = "atg.sswSbaTsfWeiterSchalten";

	/**
	 * ATG <code>atg.tsfWeiterSchaltenAntwort</code>
	 */
	protected static final String ATG_TSF_WEITER_SCHALTEN_ANTWORT = "atg.sswSbaTsfWeiterSchaltenAntwort";

	/**
	 * ATG <code>atg.tsfZustandSteuerung</code>
	 */
	protected static final String ATG_TSF_ZUSTAND_STEUERUNG = "atg.sswSbaTsfZustandSteuerung";
	
	/**
	 * ATG <code>atg.sswSbaTsfUeInputAnalyse</code>
	 */
	protected static final String ATG_TSF_URSACHENEINHEIT_INPUT_ANALYSE = "atg.sswSbaTsfUeInputAnalyse";

	/**
	 * ATG <code>atg.tsfParaAblaufTSF</code>
	 */
	protected static final String ATG_TSF_PARA_ABLAUF_TSF = "atg.sswSbaTsfParaAblaufTSF";

	/**
	 * ATG <code>atg.tsfAbschnitte</code>
	 */
	protected static final String ATG_TFS_ABSCHNITTE = "atg.sswSbaTsfAbschnitte";

	/**
	 * ATG <code>atg.tsfAbschnittDiagnoseDaten</code>
	 */
	protected static final String ATG_TSF_ABSCHNITT_DIAGNOSE_DATEN = "atg.sswSbaTsfAbschnittDiagnoseDaten";
	
	/**
	 * ATG <code>atg.tsfZustandArgoscan</code>.
	 */
	public static final String ATG_TSF_ZUSTAND_ARGOSCAN = "atg.sswSbaTsfZustandArgoscan";

	/**
	 * Liefert die ID dieses Clients.
	 * 
	 * @return
	 */
	public static String getClientId()
	{
		String re = null;
		if (DaVVerbindung.getInstanz().getDaVVerbindung() != null)
		{
			re = DaVVerbindung.getInstanz().getDaVVerbindung()
					.getClientDavParameters().getApplicationName();
		} else
		{
			try
			{
				re = InetAddress.getLocalHost().getHostAddress();
			} catch (UnknownHostException e)
			{
				e.printStackTrace();
			}
		}

		if (re == null)
		{
			re = "IP NICHT BESTIMMBAR";
		}
		
//		re += "#" + SDF.format(new Date());

		return re;
	}

	/**
	 * Flag das anzeigt, dass Berechtigungen ignoriert werden sollen.
	 */
	public static final boolean BERECHTIGUNGEN_IGNORIEREN = true;

	/**
	 * Prft ob die Steuerung einen Abschnitt gerade in Bearbeitung hat.
	 * 
	 * @param tsfWorkflowAbschnitt
	 *          Zu prfender Abschnitt.
	 * @return Liefert <code>true</code> wenn der entsprechende
	 *         {@link M_TSFWorkflowAbschnitt} sich gerade in Bearbeitung von der
	 *         Steuerung befindet, sonst <code>false</code>.
	 */
	public static boolean pruefeObSteuerungFuerAbschnittInBearbeitung(
			M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt)
	{
		boolean inBearbeitung = false;

		if ((tsfWorkflowAbschnitt != null)
				&& (tsfWorkflowAbschnitt.getAbschnitt() != null))
		{
			String pidAbs = tsfWorkflowAbschnitt.getAbschnitt().getPid();
			inBearbeitung = pruefeObSteuerungFuerAbschnittInBearbeitung(pidAbs);
		}

		return inBearbeitung;
	}

	/**
	 * Prft ob die Steuerung einen Abschnitt gerade in Bearbeitung hat.
	 * 
	 * @param pidAbs
	 *          Zu prfender Abschnitt.
	 * @return Liefert <code>true</code> wenn der entsprechende
	 *         {@link M_TSFWorkflowAbschnitt} sich gerade in Bearbeitung von der
	 *         Steuerung befindet, sonst <code>false</code>.
	 */
	public static boolean pruefeObSteuerungFuerAbschnittInBearbeitung(
			String pidAbs)
	{
		boolean inBearbeitung = false;

		if (pidAbs != null)
		{
			GUIModelTSFSteuerung modelZustand = DaVKomTSFSteuerung.getInstance()
			.getModelZustand();
			if(modelZustand != null)
			{
				ZustandSammlung sammlungZustand = modelZustand
				.getZustandSteuerungSammlung(pidAbs);
			
				if(sammlungZustand != null)
				{
					if (sammlungZustand.getProgrammZustand() == E_TSFProgrammZustand.In_Bearbeitung)
					{
						inBearbeitung = true;
					}
				}
			}
		}

//		System.out.println("Defs.pruefeObSteuerungFuerAbschnittInBearbeitung() => "
//				+ inBearbeitung + " fr Abschitt " + pidAbs);

		return inBearbeitung;
	}

	public static boolean istBestimmtesProgrammFuerAbschnittErreicht(
			M_TSFWorkflowAbschnitt abs)
	{
		boolean programmErreicht = false;

		if (abs != null)
		{
			String pidAbs = abs.getAbschnitt().getPid();
			E_TSFBefehl programm = abs.getAktion();

			if (pidAbs != null)
			{
				programmErreicht = istBestimmtesProgrammFuerAbschnittErreicht(pidAbs,
						programm);
			}
		}

		return programmErreicht;
	}

	public static boolean istBestimmtesProgrammFuerAbschnittErreicht(
			String pidAbs, E_TSFBefehl programm)
	{
		boolean programmErreicht = false;

		if (pidAbs != null)
		{
			GUIModelTSFSteuerung modelZustand = DaVKomTSFSteuerung.getInstance()
					.getModelZustand();

			ZustandSammlung zustandSteuerungSammlung = modelZustand
					.getZustandSteuerungSammlung(pidAbs);

			if (zustandSteuerungSammlung != null)
			{
				if ((zustandSteuerungSammlung.getProgrammZustand() == E_TSFProgrammZustand.Geschaltet)
						&& (zustandSteuerungSammlung.getZielProgramm() == programm))
				{
					programmErreicht = true;
				}
			}

//			System.out
//					.println("Defs.istBestimmtesProgrammFuerAbschnittErreicht() => "
//							+ programmErreicht + " Pid=" + pidAbs + " Prg=" + programm);
		}
		return programmErreicht;
	}
	
	public static boolean istZielProgrammFuerAbschnittInBearbeitung(String pidAbs, E_TSFBefehl programm)
	{
		boolean zielProgrammInBearbeitung = false;
		
		if (pidAbs != null)
		{
			GUIModelTSFSteuerung modelZustand = DaVKomTSFSteuerung.getInstance()
					.getModelZustand();

			ZustandSammlung zustandSteuerungSammlung = modelZustand
					.getZustandSteuerungSammlung(pidAbs);

			if (zustandSteuerungSammlung != null)
			{
				if ((zustandSteuerungSammlung.getProgrammZustand() == E_TSFProgrammZustand.In_Bearbeitung)
						&& (zustandSteuerungSammlung.getZielProgramm() == programm))
				{
					zielProgrammInBearbeitung = true;
				}
			}

		}
		
		return zielProgrammInBearbeitung;
	}
}
