/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.dlg.DialogWeiterschalten;
import de.sts.buv.plugin.workflow.tsf.dlg.ViewSchnellsperrung;
import de.sts.buv.plugin.workflow.tsf.dlg.ViewWeiterschalten;

/**
 * Informiert Dialoge/Views ber die von {@link DaVKomTSFWeiterSchalten}
 * gelieferten Informationen.
 * 
 * @author HofmannJ
 * 
 */
public class GUIManager_TSFWeiterschalten implements PropertyChangeListener
{
	private GUIModelTSFWeiterSchalten _guiModelTSFWeiterSchalten = null;

	private static final boolean USE_VIEWS = true;

	/**
	 * Map mit dem geffneten Dialogen, zugeordnet zum SystemObjekt vom Typ
	 * <code>...</code> als Key.
	 */
	private HashMap<String, DialogWeiterschalten> _mapDlg = new HashMap<String, DialogWeiterschalten>();

	public GUIManager_TSFWeiterschalten(
			GUIModelTSFWeiterSchalten guiModelTSFWeiterSchalten)
	{
		if (guiModelTSFWeiterSchalten != null)
		{
			_guiModelTSFWeiterSchalten = guiModelTSFWeiterSchalten;
			_guiModelTSFWeiterSchalten.addListener(this);
		}
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{
		if ((evt != null) && (evt.getPropertyName() != null))
		{
			String propName = evt.getPropertyName();
			if (propName.equals(GUIModelTSFWorkflow.PROP_INIT))
			{
				// Initialisierung
				if (evt.getNewValue() != null)
				{
					if (evt.getNewValue() instanceof M_TSFWeiterSchalten[])
					{
						M_TSFWeiterSchalten[] aTSFW = (M_TSFWeiterSchalten[]) evt
								.getNewValue();
						for (M_TSFWeiterSchalten mTSFW : aTSFW)
						{
							genFillDlg(mTSFW);
						}
					}
				}
			} else if (propName.equals(GUIModelTSFWorkflow.PROP_ADDED))
			{
				// ADDED
				M_TSFWeiterSchalten tsfwsOld = null;
				M_TSFWeiterSchalten tsfwsNew = null;

				if ((evt.getOldValue() != null)
						&& (evt.getOldValue() instanceof M_TSFWeiterSchalten))
				{
					tsfwsOld = (M_TSFWeiterSchalten) evt.getOldValue();
				}

				if ((evt.getNewValue() != null)
						&& (evt.getNewValue() instanceof M_TSFWeiterSchalten))
				{
					tsfwsNew = (M_TSFWeiterSchalten) evt.getNewValue();
				}

				if (tsfwsNew != null)
				{
					genFillDlg(tsfwsNew);
				}

			} else if (propName.equals(GUIModelTSFWorkflow.PROP_REMOVED))
			{
				// REMOVED
				M_TSFWeiterSchalten tsfwsRemoved = null;
				if ((evt.getOldValue() != null)
						&& (evt.getOldValue() instanceof M_TSFWeiterSchalten))
				{
					tsfwsRemoved = (M_TSFWeiterSchalten) evt.getOldValue();

					if ((tsfwsRemoved != null) && (tsfwsRemoved.getSo() != null))
					{
						if (!USE_VIEWS)
						{
							DialogWeiterschalten dlgRemoved = null;

							synchronized (_mapDlg)
							{
								dlgRemoved = _mapDlg.get(tsfwsRemoved.getSo().getPid());
							}

							if (dlgRemoved != null)
							{
								final DialogWeiterschalten fDlgRemoved = dlgRemoved;
								Display display = PlatformUI.getWorkbench().getDisplay();

								Runnable run = new Runnable()
								{

									@Override
									public void run()
									{
										fDlgRemoved.dispose();
									}
								};

								display.syncExec(run);
							}
						}

						if (USE_VIEWS)
						{
							final String pidAbschnittRemoved = tsfwsRemoved.getSo().getPid();

							Runnable run = new Runnable()
							{

								@Override
								public void run()
								{
									// System.out.println("-> Schliee ViewWeiterschalten "
									// + pidAbschnittRemoved);
									ViewWeiterschalten
											.schliesseViewWeiterschalten(pidAbschnittRemoved);
								}
							};
							PlatformUI.getWorkbench().getDisplay().asyncExec(run);
						}
					}
				}

			}

		}

	}

	public void removeDialog(DialogWeiterschalten dialog)
	{
		if ((dialog != null) && (dialog.getData() != null)
				&& (dialog.getData().getSo() != null))
		{
			SystemObject so = dialog.getData().getSo();
			String pid = so.getPid();
			synchronized (_mapDlg)
			{
				_mapDlg.remove(pid);
			}
		}
	}

	protected void genFillDlg(final M_TSFWeiterSchalten tsfWeiterSchalten)
	{
		if ((tsfWeiterSchalten != null) && (tsfWeiterSchalten.getSo() != null))
		{
			/*
			 * Die Aufforderung zum Weiterschalten erfolgt an einen konkreten Client,
			 * definiert durch die Applikationskennung am DAV.
			 */
//			System.out.println("ID bermittelt   => \""
//					+ tsfWeiterSchalten.getClientId() + "\"");
//			System.out.println("ID sollte lauten => \"" + Defs.getClientId() + "\"");

			boolean dialog_oeffnen = true;
			if (Defs.AUF_GENAUE_ID_ACHTEN)
			{
				if (!tsfWeiterSchalten.getClientId().equals(Defs.getClientId()))
				{
					dialog_oeffnen = false;
				}
			}

			if (dialog_oeffnen)
			{

				DialogWeiterschalten dlg = null;

				final String pidAbschnitt = tsfWeiterSchalten.getSo().getPid();

				if (!USE_VIEWS)
				{
					synchronized (_mapDlg)
					{
						if (_mapDlg.containsKey(pidAbschnitt))
						{
							// Key + Dlg gefunden
							dlg = _mapDlg.get(pidAbschnitt);
						}
					}

					if (dlg != null)
					{
						/*
						 * Mit vorhandenem Dialog weiterarbeiten.
						 */
						final DialogWeiterschalten fDlg = dlg;
						Runnable run = new Runnable()
						{

							@Override
							public void run()
							{
								fDlg.setData(tsfWeiterSchalten);
								// Dialog nach vorne bringen
								fDlg.getParent().setActive();
							}
						};
						Display display = PlatformUI.getWorkbench().getDisplay();
						display.asyncExec(run);
					} else
					{
						/*
						 * Neuen Dialog erzeugen und in der Map sichern. Durch
						 * Synchronisation am UI-Thread wird mehrfachinitialisierung
						 * verhindert.
						 */
						final DialogWeiterschalten newDialog = createNewDialogWeiterschalten();
						if (newDialog != null)
						{
							synchronized (_mapDlg)
							{
								_mapDlg.put(tsfWeiterSchalten.getSo().getPid(), newDialog);
							}
						} else
						{
							System.err
									.println(this.getClass().getName()
											+ " => Neuer DialogWeiterschalten konnten nicht erzeugt werden.");
						}

						/*
						 * Neuen Dialog mit Daten fllen.
						 */
						Display display = PlatformUI.getWorkbench().getDisplay();
						Runnable runSetzeDialogDaten = new Runnable()
						{

							@Override
							public void run()
							{
								newDialog.oeffne(tsfWeiterSchalten);
							}
						};
						display.asyncExec(runSetzeDialogDaten);

					}
				}

				if (USE_VIEWS)
				{
					Runnable run = new Runnable()
					{
						@Override
						public void run()
						{
						  if (!tsfWeiterSchalten.getProgramm().equals(E_TSFBefehl.Schnellsperrung)){
						    ViewWeiterschalten viewWeiterschalten = ViewWeiterschalten
						    .zeigeViewWeiterschalten(pidAbschnitt);
						    viewWeiterschalten.setData(tsfWeiterSchalten);						    
						  }else{
						    ViewSchnellsperrung viewWeiterschalten = ViewSchnellsperrung
						    .zeigeViewWeiterschalten(pidAbschnitt);
						    viewWeiterschalten.setData(tsfWeiterSchalten);						    						    
						  }
						}
					};
					PlatformUI.getWorkbench().getDisplay().asyncExec(run);
				}
			} else
			{
				// Dialog schlieen, wenn nur noch fr andere Clients bestimmt.
				if (tsfWeiterSchalten.getSo() != null)
				{
					final SystemObject so = tsfWeiterSchalten.getSo();

					if (!USE_VIEWS)
					{
						DialogWeiterschalten dlgToClose = null;

						if ((so != null) && (so.getPid() != null))
						{
							synchronized (_mapDlg)
							{
								dlgToClose = _mapDlg.get(so.getPid());
							}
						}

						if (dlgToClose != null)
						{
							final DialogWeiterschalten fDlgToClose = dlgToClose;

							Runnable run = new Runnable()
							{

								@Override
								public void run()
								{
									fDlgToClose.dispose();

								}
							};
							PlatformUI.getWorkbench().getDisplay().asyncExec(run);
						}
					}

					if (USE_VIEWS)
					{
						if ((so != null) && (so.getPid() != null))
						{
							final String pidAbschnitt = so.getPid();

							Runnable run = new Runnable()
							{

								@Override
								public void run()
								{
									if (ViewWeiterschalten
											.existiertViewWeiterschalten(pidAbschnitt))
									{
										ViewWeiterschalten
												.schliesseViewWeiterschalten(pidAbschnitt);
									}
								}
							};

							PlatformUI.getWorkbench().getDisplay().asyncExec(run);
						}
					}
				}
			}
		}
	}

	/**
	 * Erzeugt einen neuen Dialog. Synchronisiert an Hand des UI-Threads. Kann von
	 * jedem anderen Thread aus aufgerufen werden.
	 * 
	 * @return
	 */
	private DialogWeiterschalten createNewDialogWeiterschalten()
	{
		final WrapperDlg wrapperDlg = new WrapperDlg();

		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				// Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				// .getShell();

				Shell shell = new Shell(SWT.SHELL_TRIM);

				DialogWeiterschalten dlgNeu = new DialogWeiterschalten(shell,
						GUIManager_TSFWeiterschalten.this);

				wrapperDlg.setDlg(dlgNeu);
			}
		};

		PlatformUI.getWorkbench().getDisplay().syncExec(run);

		return wrapperDlg.getDlg();
	}

	/**
	 * Wrapper fr {@link DialogWeiterschalten}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class WrapperDlg
	{
		private DialogWeiterschalten _dlg = null;

		public DialogWeiterschalten getDlg()
		{
			return _dlg;
		}

		public void setDlg(DialogWeiterschalten dlg)
		{
			_dlg = dlg;
		}

	}

	/**
	 * Prft ob fr ein bestimmtes Objekt bereits ein Eintrag in der Map der
	 * Dialoge vorhanden ist.
	 * 
	 * @param so
	 * @return
	 */
	protected boolean containsDialogForObject(SystemObject so)
	{
		boolean re = false;

		if (so != null)
		{
			re = containsDialogForObject(so.getPid());
		}

		return re;
	}

	/**
	 * Prft ob fr ein bestimmtes Objekt bereits ein Eintrag in der Map der
	 * Dialoge vorhanden ist.
	 * 
	 * @param pid
	 * @return
	 */
	protected boolean containsDialogForObject(String pid)
	{
		boolean re = false;

		synchronized (_mapDlg)
		{
			if (_mapDlg.containsKey(pid))
			{
				re = true;
			}
		}

		return re;
	}

}
