/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Informiert Dialoge/Views ber die von {@link DaVKomTSFWeiterSchalten}
 * gelieferten Informationen.
 * 
 * @author HofmannJ
 * 
 */
public class GUIModelTSFWeiterSchalten
{
	private TreeMap<SystemObject, M_TSFWeiterSchalten> _tm = new TreeMap<SystemObject, M_TSFWeiterSchalten>();

	/**
	 * Listenersupport
	 */
	private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);

	/**
	 * Wird gesendet mit einem Array aller registrierten
	 * {@link M_TSFWeiterSchalten}s als newValue, sobald sich ein Listener
	 * anmeldet.
	 */
	public static final String PROP_INIT = "PROP_INIT";

	/**
	 * Wird gesendet, sobald ein {@link M_TSFWeiterSchalten} hinzugefgt. Falls
	 * vorher ein alter verfgbar war wird dieser als oldValue bermittelt, der
	 * neue als newValue.
	 */
	public static final String PROP_ADDED = "PROP_ADDED";

	/**
	 * Wird gesendet, sobald ein {@link M_TSFWeiterSchalten} entfernt wird. Der
	 * oldValue entspricht dem entfernten {@link M_TSFWeiterSchalten}, der
	 * newValue ist <code>null</code>.
	 */
	public static final String PROP_REMOVED = "PROP_REMOVED";

	public void putTSFWeiterSchalten(M_TSFWeiterSchalten tsfWeiterSchalten)
	{
		if ((tsfWeiterSchalten != null) && (tsfWeiterSchalten.getSo() != null))
		{
			M_TSFWeiterSchalten oldVal = null;
			M_TSFWeiterSchalten newVal = tsfWeiterSchalten;

			synchronized (_tm)
			{
				if (_tm.containsKey(tsfWeiterSchalten.getSo()))
				{
					oldVal = _tm.get(tsfWeiterSchalten.getSo());
				}
				_tm.put(tsfWeiterSchalten.getSo(), tsfWeiterSchalten);
			}

			_pcs.firePropertyChange(PROP_ADDED, oldVal, newVal);

		} else
		{
			System.err
					.println("TSFWeiterSchalten oder dessen SystemObject ist null.");
		}
	}

	protected M_TSFWeiterSchalten removeWeiterSchalten(SystemObject so)
	{
		M_TSFWeiterSchalten removedTSFWeiterSchalten = null;

		synchronized (_tm)
		{
			if (_tm.containsKey(so))
			{
				removedTSFWeiterSchalten = _tm.remove(so);
			}
		}

		if (removedTSFWeiterSchalten != null)
		{
			_pcs.firePropertyChange(PROP_REMOVED, removedTSFWeiterSchalten, null);
		}

		return removedTSFWeiterSchalten;
	}

	protected M_TSFWeiterSchalten[] getWorkflows()
	{
		LinkedList<M_TSFWeiterSchalten> lWF = new LinkedList<M_TSFWeiterSchalten>();

		synchronized (_tm)
		{
			Iterator<Entry<SystemObject, M_TSFWeiterSchalten>> it = _tm.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<SystemObject, M_TSFWeiterSchalten> entry = it.next();
				if (entry.getValue() != null)
				{
					lWF.add(entry.getValue());
				}
			}
		}

		return lWF.toArray(new M_TSFWeiterSchalten[lWF.size()]);
	}

	public M_TSFWeiterSchalten getWorkflow(SystemObject so)
	{
		M_TSFWeiterSchalten re = null;

		if (so != null)
		{
			synchronized (_tm)
			{
				re = _tm.get(so);
			}
		}

		return re;
	}

	public void addListener(PropertyChangeListener pcl)
	{
		_pcs.addPropertyChangeListener(pcl);

		PropertyChangeEvent pce = new PropertyChangeEvent(this, PROP_INIT, null,
				getWorkflows());
		pcl.propertyChange(pce);
	}

	public void removeListener(PropertyChangeListener pcl)
	{
		_pcs.removePropertyChangeListener(pcl);
	}
}
