/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.Date;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung.ZustandSammlung;

/**
 * Definiert die Eigenschaften des Workflow pro Abschnitt. Reprsentiert einen
 * Eintrag von <code>atl.tsfWorkflowAbschnitt</code>.
 * 
 * @author HofmannJ
 * 
 */
public class M_TSFWorkflowAbschnitt
{
	/**
	 * TSFAbschnittReferenz, Referenz auf ein Objekt des Typs
	 * <code>typ.tsfAbschnitt</code>.
	 */
	private SystemObject _abschnitt = null;

	/**
	 * Befehl der TSF.
	 */
	private E_TSFBefehl _aktion = null;

	/**
	 * Ursache fr eine TSF Empfehlung
	 */
	private E_TSFUrsache _ursache = null;

	/**
	 * {@link M_TSFWorkflow} zu dem dieser Abschnitt gehrt.
	 */
	private M_TSFWorkflow _tsfWorkflow = null;

	/**
	 * Zeitstempel
	 */
	private Long _timeStamp = null;

	public M_TSFWorkflowAbschnitt()
	{
	}

	/**
	 * 
	 * @param abschnitt
	 * @param aktion
	 * @param ursache
	 * @param timeStamp
	 *          Zeitstempel des Datensatzes vom DaV.
	 */
	public M_TSFWorkflowAbschnitt(SystemObject abschnitt, E_TSFBefehl aktion,
			E_TSFUrsache ursache, long timeStamp)
	{
		setAbschnitt(abschnitt);
		setAktion(aktion);
		setUrsache(ursache);
		this._timeStamp = timeStamp;
	}

	public SystemObject getAbschnitt()
	{
		return _abschnitt;
	}

	public void setAbschnitt(SystemObject abschnitt)
	{
		_abschnitt = abschnitt;
	}

	public E_TSFBefehl getAktion()
	{
		return _aktion;
	}

	public void setAktion(E_TSFBefehl aktion)
	{
		_aktion = aktion;
	}

	public E_TSFUrsache getUrsache()
	{
		return _ursache;
	}

	public void setUrsache(E_TSFUrsache ursache)
	{
		_ursache = ursache;
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("Abschnitt: ");
		if (getAbschnitt() != null)
		{
			sb.append(getAbschnitt());
		} else
		{
			sb.append("-");
		}
		sb.append(", ");

		sb.append("Aktion: ");
		if (getAktion() != null)
		{
			sb.append(getAktion());
		} else
		{
			sb.append("-");
		}
		sb.append(", ");

		sb.append("Ursache: ");
		if (getUrsache() != null)
		{
			sb.append(getUrsache());
		} else
		{
			sb.append("-");
		}

		return sb.toString();
	}

	public M_TSFWorkflowAbschnitt duplicate()
	{
		M_TSFWorkflowAbschnitt neu = new M_TSFWorkflowAbschnitt();

		neu._timeStamp = new Long(this._timeStamp.longValue());
		neu.setAbschnitt(this.getAbschnitt());
		neu.setAktion(this.getAktion());
		neu.setUrsache(this.getUrsache());

		return neu;
	}

	/**
	 * Prft ob fr diesen Abschnitt der gewnschte Zustand aktuell erreicht ist.
	 * 
	 * @return Liefert <code>true</code> wenn der Zustand erreicht ist, sonst
	 *         <code>false</code>.
	 */
	public boolean isZustandErreicht()
	{
		boolean re = false;

		if (getAktion() != null)
		{
			GUIModelTSFSteuerung model = DaVKomTSFSteuerung.getInstance()
					.getModelZustand(); // TODO JH VON VORGABE NACH ZUSTAND
			ZustandSammlung zustandSteuerungSammlung = model
					.getZustandSteuerungSammlung(getAbschnitt());

			if (zustandSteuerungSammlung != null)
			{
				if ((zustandSteuerungSammlung.getZielProgramm() != null)
						&& (getAktion() != null))
				{
					if (zustandSteuerungSammlung.getZielProgramm() == getAktion())
					{
						E_TSFProgrammZustand programmZustand = zustandSteuerungSammlung
								.getProgrammZustand();
						if ((programmZustand != null)
								&& (programmZustand == E_TSFProgrammZustand.Geschaltet))
						{
							re = true;
						}
					} else
					{
						// Nix tun
					}
				} else
				{
					/*
					 * Aktuelles Zielprogramm oder geforderte Aktion null.
					 */
				}
			} else
			{
				System.err.println("Zustand Steuerung Sammlung = null!!!!");
			}
		}

		return re;
	}
	
	/**
	 * Prft ob fr diesen Abschnitt der gewnschte Zustand aktuell erreicht ist.
	 * 
	 * @return Liefert <code>true</code> wenn der Zustand erreicht ist, sonst
	 *         <code>false</code>.
	 */
	public boolean isZustandInBearbeitung()
	{
	  boolean re = false;
	  
	  if (getAktion() != null)
	  {
	    GUIModelTSFSteuerung model = DaVKomTSFSteuerung.getInstance()
	    .getModelZustand(); // TODO JH VON VORGABE NACH ZUSTAND
	    ZustandSammlung zustandSteuerungSammlung = model
	    .getZustandSteuerungSammlung(getAbschnitt());
	    
	    if (zustandSteuerungSammlung != null)
	    {
	      if ((zustandSteuerungSammlung.getZielProgramm() != null)
	          && (getAktion() != null))
	      {
	        if (zustandSteuerungSammlung.getZielProgramm() == getAktion())
	        {
	          E_TSFProgrammZustand programmZustand = zustandSteuerungSammlung
	          .getProgrammZustand();
	          if ((programmZustand != null)
	              && (programmZustand == E_TSFProgrammZustand.In_Bearbeitung))
	          {
	            re = true;
	          }
	        } else
	        {
	          // Nix tun
	        }
	      } else
	      {
	        /*
	         * Aktuelles Zielprogramm oder geforderte Aktion null.
	         */
	      }
	    } else
	    {
	      System.err.println("Zustand Steuerung Sammlung = null!!!!");
	    }
	  }
	  
	  return re;
	}

	/**
	 * Liefert den {@link M_TSFWorkflow}, zu dem dieser Abschnitt gehrt.
	 * 
	 * @return
	 */
	public M_TSFWorkflow getTsfWorkflow()
	{
		return _tsfWorkflow;
	}

	/**
	 * Setzt den {@link M_TSFWorkflow}, zu dem dieser Abschnitt gehrt.
	 * 
	 * @param tsfWorkflow
	 */
	public void setTsfWorkflow(M_TSFWorkflow tsfWorkflow)
	{
		_tsfWorkflow = tsfWorkflow;
	}

	/**
	 * Liefert den Zeitstempel des Datensatzes vom DaV.
	 * 
	 * @return Zeitstempel.
	 */
	public Date getTimeStamp()
	{
		Date timeStamp = null;

		if (_timeStamp != null)
		{
			timeStamp = new Date(_timeStamp);
		}

		return timeStamp;
	}

	protected boolean isGleich(M_TSFWorkflowAbschnitt o)
	{
		boolean gleich = false;

		if (o != null)
		{
			if (o._abschnitt == this.getAbschnitt())
			{
				if (o.getAktion() == this.getAktion())
				{
					if (o.getUrsache() == this.getUrsache())
					{
						if (o.getTimeStamp().equals(this.getTimeStamp()))
						{
							gleich = true;
						}
					}
				}
			}
		}

		return gleich;
	}

}
