/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.util.HashMap;

import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.sts.buv.plugin.workflow.tsf.dav.Defs;

/**
 * Manager-Klasse zum Verwalten der Dialoge {@link DialogSchalten} zum Schalten
 * eines Abschnitts.
 * 
 * @author HofmannJ
 * 
 */
public class DialogSchaltenGuiManager
{
	private static DialogSchaltenGuiManager _instance = null;

	/**
	 * Key = Pid des zu schaltenden Abschnitts, Typ <code>typ.tsfAbschnitt</code>,
	 * Value=Der entsprechende {@link DialogSchalten} zum jeweiligen Abschnitt.
	 */
	private HashMap<String, DialogSchalten> _map = new HashMap<String, DialogSchalten>();

	private DialogSchaltenGuiManager()
	{
	}

	public static DialogSchaltenGuiManager getInstance()
	{
		if (_instance == null)
		{
			synchronized (DialogSchaltenGuiManager.class)
			{
				if (_instance == null)
				{
					_instance = new DialogSchaltenGuiManager();
				}
			}
		}

		return _instance;
	}

	public DialogSchalten removeDialogSchalten(String pid)
	{
		DialogSchalten removed = null;

		if (pid != null)
		{
			synchronized (_map)
			{
				removed = _map.remove(pid);
			}
		}

		return removed;
	}

	/**
	 * ffnet einen Dialog zum Schalten des jeweiligen Abschnitts. Bevor der
	 * Dialog jedoch geffnet wird wird geprft, ob sich der entsprechende
	 * Abschnitt gerade in einer Schaltung befindet. Ist dies der Fall, erscheint
	 * statts des Dialogs eine entsprechende Warn-Meldung und der Hinweis, dass
	 * sich diese Schaltung NUR noch Abbrechen lsst.
	 * 
	 * @param pidAbs PID des Abschnitts
	 */
	public void openDialogSchalten(final String pidAbs)
	{
		if (pidAbs != null)
		{
			DialogSchalten dialogSchalten = null;

			synchronized (_map)
			{
				if (_map.containsKey(pidAbs))
				{
					dialogSchalten = _map.get(pidAbs);
				}
			}

			if (dialogSchalten == null)
			{
				dialogSchalten = createNewDialogWeiterschalten();
				synchronized (_map)
				{
					_map.put(pidAbs, dialogSchalten);
				}

				
				/*
				 * Prfen ob schaltbar
				 */
				boolean schaltbar = !Defs.pruefeObSteuerungFuerAbschnittInBearbeitung(pidAbs);
				

//				/*
//				 * Jetzt muss das noch gegen Bereiche in Bearbeitung geprft werden.
//				 * Dies ist auf einem Remote-System mit der jetzigen Struktur NICHT
//				 * mglich.
//				 */
//				if (schaltbar)
//				{
//					boolean abschnittInAktiverBereichsschaltung = TSFBereichSchaltenVerwaltung
//							.getInstance().isAbschnittInAktiverBereichsschaltung(pidAbs);
//					if (abschnittInAktiverBereichsschaltung)
//					{
//						schaltbar = false;
//					}
//				}

				final boolean fNurAbbrechenMoeglich = !schaltbar;

				final DialogSchalten fDialogSchalten = dialogSchalten;
				Runnable runNeuenDlgOeffnen = new Runnable()
				{

					@Override
					public void run()
					{
						fDialogSchalten.oeffne(pidAbs, fNurAbbrechenMoeglich);
					}
				};
				PlatformUI.getWorkbench().getDisplay().asyncExec(runNeuenDlgOeffnen);

			} else
			{
				final DialogSchalten fDialogSchalten = dialogSchalten;
				Runnable runExistierendenDlgInVordergrundBringen = new Runnable()
				{
					@Override
					public void run()
					{
						fDialogSchalten.getParent().forceActive();
					}
				};
				PlatformUI.getWorkbench().getDisplay().asyncExec(
						runExistierendenDlgInVordergrundBringen);
			}

		}
	}

	/**
	 * Erzeugt einen neuen Dialog. Synchronisiert an Hand des UI-Threads. Kann von
	 * jedem anderen Thread aus aufgerufen werden.
	 * 
	 * @return
	 */
	private DialogSchalten createNewDialogWeiterschalten()
	{
		final WrapperDlg wrapperDlg = new WrapperDlg();

		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
						.getShell();

				DialogSchalten dlgNeu = new DialogSchalten(shell,
						DialogSchaltenGuiManager.this);

				wrapperDlg.setDlg(dlgNeu);
			}
		};

		PlatformUI.getWorkbench().getDisplay().syncExec(run);

		return wrapperDlg.getDlg();
	}

	/**
	 * Wrapper fr {@link DialogSchalten}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class WrapperDlg
	{
		private DialogSchalten _dlg = null;

		public DialogSchalten getDlg()
		{
			return _dlg;
		}

		public void setDlg(DialogSchalten dlg)
		{
			_dlg = dlg;
		}

	}
}
