/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.awt.Toolkit;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.events.ShellListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.DefsFonts;
import de.sts.buv.plugin.workflow.tsf.PreferenceInit;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFAbschnittBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.Defs;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFProgrammZustand;
import de.sts.buv.plugin.workflow.tsf.dav.GUIManager_WorkflowDialog;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFAbschnittBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFWorkflow;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFWorkflowAbschnitt;
import de.sts.buv.plugin.workflow.tsf.dav.WiedervorlageTSFWorkflow;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung.ZustandSammlung;
import de.sts.buv.plugin.workflow.tsf.snd.AePlayWave;

/**
 * Dialog mit dem die Programmvorschlge angezeigt werden, die via ATG
 * <code>atg.tsfWorkflow</code>, ASP <code>asp.tsfZustand</code> fr Objekte des
 * Typs <code>typ.temporaereSeitenstreifenFreigabe</code> reinkommen.
 */
public class DialogWFAbfrage extends Dialog implements Runnable
{
	private static final String HELP_ID = Activator.PLUGIN_ID
			+ ".dialogworkflowabfrage";

	/**
	 * Shell
	 */
	private Shell shell;

	private Object _re = null;

	/**
	 * Tabelle mit den zu schaltenden Strategien
	 */
	private Table _tabelle;

	/**
	 * Tableviewer fr diese Tabelle
	 */
	private TableViewer _tvGruppen;

	private M_TSFWorkflow _data = null;

	/**
	 * Dieses Flag wird von der Main-Methode auf <code>true</code> gesetzt, wenn
	 * der Dialog zum Testen losgelst von der RCP-Umgebung laufen soll.
	 */
	private static boolean _startWithoutEclipseRcp = false;

	private static Shell parentShell;

	private GUIManager_WorkflowDialog _guiwfDialogManager;

	private Label _lFahrtrichtung;

	private ButtonSelectionListener _buttonSelectionListener;

	/**
	 * Liste der Buttons, die vor einer Aktualisierung der Tabelle gecleared
	 * werden muss.
	 */
	private LinkedList<Button> _lButtons = new LinkedList<Button>();

	private HashMap<Button, ButtonPropertyChangeListener> _mapButtonPcl = new HashMap<Button, ButtonPropertyChangeListener>();

	private Timer _sndTimer;

	private Button _buttonTonAusschalten;

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 *          aufrufendes Fenster
	 */
	public DialogWFAbfrage(Shell parent,
			GUIManager_WorkflowDialog guiwfDialogManager)
	{
		super(parent);

		this._guiwfDialogManager = guiwfDialogManager;

		if (!_startWithoutEclipseRcp)
		{
			PlatformUI.getWorkbench().addWorkbenchListener(new IWorkbenchListener()
			{
				public void postShutdown(IWorkbench workbench)
				{
				}

				public boolean preShutdown(IWorkbench workbench, boolean forced)
				{
					dispose();
					return true;
				}
			});
		}
	}

	protected GUIManager_WorkflowDialog getGuiwfDialogManager()
	{
		return _guiwfDialogManager;
	}

	/**
	 * ffnet den Dialog {@link DialogWFAbfrage} zum Annehmen oder Ablehnen eines
	 * Schaltvorschlags
	 * 
	 * @return
	 */
	public Object oeffne(M_TSFWorkflow data)
	{
    System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>> DialogWFAbfrage.oeffne() <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");

		/*
		 * Shell auf SystemEbene, kann verschoben werden da auf hherer Ebene wie
		 * der UrlasserDialog.
		 */
		// shell = new Shell(getParent(), SWT.TITLE | SWT.BORDER | SWT.CLOSE
		// | SWT.APPLICATION_MODAL);
		// shell = new Shell(getParent(), SWT.TITLE | SWT.BORDER | SWT.CLOSE
		// | SWT.ON_TOP);
	  
		shell = new Shell(getParent(), SWT.TITLE | SWT.BORDER);
		
		 //Fr die F1 Taste
	    PlatformUI.getWorkbench().getHelpSystem().setHelp( shell, HELP_ID );

		RowLayout rowLayout = new RowLayout();
		rowLayout.type = org.eclipse.swt.SWT.VERTICAL;

		rowLayout.wrap = true;

		shell.setLayout(new FillLayout());
		shell.addShellListener(new ShellListener()
		{

			@Override
			public void shellActivated(ShellEvent e)
			{
			}

			@Override
			public void shellClosed(ShellEvent e)
			{
				// e.doit = false;
				//
				// if (!_startWithoutEclipseRcp)
				// {
				// UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench()
				// .getDisplay().getActiveShell());
				// // UrlasserDialogInfo info = uld.anzeigen(
				// // BenutzerGruppenFunktion.SCHALTEN, true);
				// UrlasserDialogInfo info = uld.anzeigen(null);
				// if (info.getEingabeOk())
				// {
				// // _antwort.setAntwortNichtSchalten();
				// // _antwort.setUrlasserInfo(info);
				// dispose();
				// WorkflowMeldungenHandler.DIALOG = null;
				// }
				// }
				dispose();
			}

			@Override
			public void shellDeactivated(ShellEvent e)
			{
			}

			@Override
			public void shellDeiconified(ShellEvent e)
			{
			}

			@Override
			public void shellIconified(ShellEvent e)
			{
			}
		});

		shell.setText("Empfehlung 1");
		createCompositeImage();
		Display display = getParent().getDisplay();

		/*
		 * Intial mit Daten fllen.
		 */
		setData(data);

		// Zentrieren des Dialogs

		int w = 750; // 791;
		int b = 450;

		// Grsse Bildschirm

		Toolkit test = java.awt.Toolkit.getDefaultToolkit();

		/*
		 * Auflsung durch 2 subtrahiert durch die Hlfte der in der Shell
		 * angegebenen Horizontalen, damit der Dialog mittig am Bildschirm erscheint
		 */

		int x = test.getScreenSize().width / 2 - w / 2;

		/*
		 * Auflsung durch 2 subtrahiert durch die Hlfte der in der Shell
		 * angegebenen Verzikalen
		 */

		int y = test.getScreenSize().height / 2 - b / 2;

		shell.setLocation(x, y);
		shell.setSize(w, b);

		// shell.pack();

		shell.open();

		startSoundTimer();

		while (!shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		return _re;
	}

	public void setData(M_TSFWorkflow data)
	{
		this._data = data;
		final M_TSFWorkflow fData = data;

		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{

				if (fData != null)
				{
					SystemObject soFahrtrichtung = fData.getSo();
					if (soFahrtrichtung != null)
					{
						String text = "Abschnitt " + soFahrtrichtung.getNameOrPidOrId();
						if (!_lFahrtrichtung.getText().equals(text))
						{
							_lFahrtrichtung.setText(text);
						}
						_lFahrtrichtung.getParent().layout(true);

					} else
					{
						_lFahrtrichtung.setText("Fahrtrichtung -");
					}

					setInput(fData);

					if (fData.getAbschnitte().length > 0)
					{
						TableItem[] items = _tabelle.getItems();

						_tvGruppen.getTable().addListener(SWT.MeasureItem, new Listener()
						{
							public void handleEvent(Event event)
							{
								// height cannot be per row so simply set
								event.height = 30;
							}
						});

						for (int i = 0; i < items.length; i += 1)
						{
							TableEditor editor = new TableEditor(_tabelle);
							Button button = new Button(_tabelle, SWT.PUSH);
							synchronized (_lButtons)
							{
								_lButtons.add(button);
							}
							button.setText("SCHALTEN");
							button.pack();

							editor.grabVertical = true;
							editor.minimumWidth = button.getSize().x;
							editor.horizontalAlignment = SWT.CENTER;
							editor.setEditor(button, items[i], 3);

							// button.setData(_modelClass[i]);
							// button.addSelectionListener(new ButtonSelectionListener());

							if ((items[i] != null) && (items[i].getData() != null)
									&& (items[i].getData() instanceof M_TSFWorkflowAbschnitt))
							{
								M_TSFWorkflowAbschnitt workflowAbschnitt = (M_TSFWorkflowAbschnitt) items[i]
										.getData();
								button.setData(workflowAbschnitt);

								button.addSelectionListener(getButtonSelectionListener());

								ButtonPropertyChangeListener buttonPropertyChangeListener = new ButtonPropertyChangeListener(
										button, workflowAbschnitt);
								synchronized (_mapButtonPcl)
								{
									_mapButtonPcl.put(button, buttonPropertyChangeListener);
								}
							}
						}
					}

					if (fData.getAbschnitte().length == 0)
					{
						dispose();
					}

				}

			}
		};
		this.getParent().getDisplay().asyncExec(run);

	}

	private void setInput(Object input)
	{
		synchronized (_lButtons)
		{
			Iterator<Button> it = _lButtons.iterator();
			while (it.hasNext())
			{
				Button button = it.next();
				disposeButton(button);
			}
			_lButtons.clear();
		}

		if (_tvGruppen != null)
		{
			_tvGruppen.setInput(input);
		}
	}

	/**
	 * Disposed einen Button:
	 * <ul>
	 * <li>ButtonSelectionListener entfernen</li>
	 * <li>ButtonPropertyChangeListener disposen</li>
	 * <li>Button Widget Disposen.</li>
	 * </ul>
	 * 
	 * @param button
	 */
	private void disposeButton(Button button)
	{
		if (button != null)
		{
			button.removeSelectionListener(getButtonSelectionListener());

			// PCL auf Button disposen
			ButtonPropertyChangeListener bPCL = null;
			synchronized (_mapButtonPcl)
			{
				if (_mapButtonPcl.containsKey(button))
				{
					bPCL = _mapButtonPcl.get(button);
					_mapButtonPcl.remove(button);
				}
			}

			if (bPCL != null)
			{
				bPCL.dispose();
			}

			// Button disposen
			button.dispose();
		}
	}

	private ButtonSelectionListener getButtonSelectionListener()
	{
		if (_buttonSelectionListener == null)
		{
			_buttonSelectionListener = new ButtonSelectionListener();
		}

		return _buttonSelectionListener;
	}

	private class ButtonSelectionListener implements SelectionListener
	{

		@Override
		public void widgetDefaultSelected(SelectionEvent e)
		{
		}

		@Override
		public void widgetSelected(SelectionEvent e)
		{
			Object data = e.widget.getData();
			// if (data != null)
			// {
			// System.out.println("ButtonSelListener Data => "
			// + data.getClass().getName());
			// } else
			// {
			// System.out.println("ButtonSelListener Data => NULL!!!");
			// }
			if ((data != null) && (data instanceof M_TSFWorkflowAbschnitt))
			{
				M_TSFWorkflowAbschnitt workflowAbschnitt = (M_TSFWorkflowAbschnitt) data;

				// M_TSFAbschnittBefehl befehl = new M_TSFAbschnittBefehl(Defs
				// .getClientId(), workflowAbschnitt.getAbschnitt(),
				// E_TSFBefehl.Freigabe, E_TSFProgrammZustand.Undefiniert);
				// DaVKomTSFAbschnittBefehl.getInstance().sendeAbschnittBefehl(befehl);
				E_TSFBefehl aktion = workflowAbschnitt.getAktion();
				M_TSFAbschnittBefehl befehl = new M_TSFAbschnittBefehl(Defs
						.getClientId(), workflowAbschnitt.getAbschnitt(), aktion,
						E_TSFProgrammZustand.Undefiniert);
				DaVKomTSFAbschnittBefehl.getInstance().sendeAbschnittBefehl(befehl);
			}
		}

	}

	public M_TSFWorkflow getData()
	{
		return _data;
	}

	/**
	 * This method initializes compositeImage
	 * 
	 */
	private void createCompositeImage()
	{
		Composite top = new Composite(shell, SWT.NONE);
		top.setLayout(new GridLayout(1, true));

		// Zeile 0 - Fahrrichtung darstellen
		GridData gdZeile0 = new GridData();
		gdZeile0.grabExcessHorizontalSpace = true;
		gdZeile0.horizontalAlignment = GridData.FILL;
		Composite zeile0 = new Composite(top, SWT.BORDER);
		zeile0.setLayout(new GridLayout(1, false));
		zeile0.setLayoutData(gdZeile0);

		// Zeile 1

		GridData gdZeile1 = new GridData();
		gdZeile1.horizontalAlignment = GridData.FILL;
		gdZeile1.verticalAlignment = GridData.FILL;
		gdZeile1.grabExcessHorizontalSpace = true;
		gdZeile1.grabExcessVerticalSpace = true;

		Composite zeile1 = new Composite(top, SWT.BORDER);
		zeile1.setLayout(new GridLayout(1, false));
		zeile1.setLayoutData(gdZeile1);

		// Zeile 2

		GridData gdZeile2 = new GridData();
		gdZeile2.horizontalAlignment = GridData.FILL;
		gdZeile2.grabExcessHorizontalSpace = true;

		Composite zeile2 = new Composite(top, SWT.BORDER);
		zeile2.setLayout(new GridLayout(4, false));
		zeile2.setLayoutData(gdZeile2);

		addFahrtrichtung(zeile0);
		addProgrammVorschlag(zeile1);
		addProgrammButtons(zeile2);
	}

	private void addFahrtrichtung(Composite parent)
	{

		_lFahrtrichtung = new Label(parent, SWT.None);
//		Font f1 = new Font(null, "Arial", 12, SWT.BOLD);
		_lFahrtrichtung.setFont(DefsFonts.FONT_ARIAL_12_PKT_BOLD);
		_lFahrtrichtung.setText("Fahrtrichtung: ");

		GridData gdLFahrtrichtung = new GridData();
		gdLFahrtrichtung.grabExcessHorizontalSpace = true;
		gdLFahrtrichtung.horizontalAlignment = GridData.CENTER;
		_lFahrtrichtung.setLayoutData(gdLFahrtrichtung);
	}

	/**
	 * Methode realisiert das Composite mit dem Programmvorschlag
	 * 
	 * @param parent
	 */
	private void addProgrammVorschlag(Composite parent)
	{
		// Composite

		// GridData gridDataComposite = new GridData();
		// gridDataComposite.horizontalAlignment = GridData.FILL;
		// gridDataComposite.verticalAlignment = GridData.FILL;
		// gridDataComposite.grabExcessHorizontalSpace = true;
		// gridDataComposite.grabExcessVerticalSpace = true;
		//
		// Composite composite = new Composite(parent, SWT.BORDER);
		// composite.setLayout(new GridLayout(1, true));
		// composite.setLayoutData(gridDataComposite);

		Composite neu = new Composite(parent, SWT.NONE);
		neu.setLayout(new GridLayout(3, false));
		GridData gdInfoText = new GridData();
		gdInfoText.horizontalAlignment = GridData.FILL;
		gdInfoText.grabExcessHorizontalSpace = true;
		neu.setLayoutData(gdInfoText);

		// Achtung Symbol links

		Label achtungLinks = new Label(neu, SWT.NONE);
		achtungLinks.setImage(getIcon("achtung.png"));

		GridData gridDataAchtungLinks = new GridData();
		gridDataAchtungLinks.horizontalAlignment = GridData.BEGINNING;
		achtungLinks.setLayoutData(gridDataAchtungLinks);

		// Textfeld

		GridData gridDataText = new GridData();
		gridDataText.grabExcessHorizontalSpace = true;
		gridDataText.horizontalAlignment = GridData.CENTER;
		gridDataText.verticalAlignment = GridData.BEGINNING;

		Label labelProgrammVorschlag = new Label(neu, SWT.NONE);
		labelProgrammVorschlag.setLayoutData(gridDataText);

//		Font f1 = new Font(null, "Arial", 16, SWT.BOLD);
		labelProgrammVorschlag.setFont(DefsFonts.FONT_ARIAL_16_PKT_BOLD);

		// String v = "\n" + "Aufgrund genderter Indikatoren werden vom \n"
		// + "Strategiemanager folgende Schaltungen empfohlen:\n\n";

		String v = "\n" + "Aufgrund genderter Indikatoren werden vom \n"
				+ "Steuermodell folgende Schaltungen empfohlen:\n\n";

		labelProgrammVorschlag.setText(v);
		labelProgrammVorschlag.setAlignment(SWT.CENTER);

		// Achtung Symbol rechts

		Label achtungRechts = new Label(neu, SWT.NONE);
		achtungRechts.setText("Achtung");
		achtungRechts.setImage(getIcon("achtung.png"));

		GridData gridDataAchtungRechts = new GridData();
		gridDataAchtungRechts.horizontalAlignment = GridData.BEGINNING;
		achtungRechts.setLayoutData(gridDataAchtungLinks);

		// Tabelle mit nderungen

		createTabelle(parent);

		// addVorschauButtons(composite);
	}

	/**
	 * Erzeugt die Liste der Parameter, sofern nicht <code>null</code>.
	 * 
	 * @param parent
	 *          Eltern-Composite
	 * @return Liste der Benutzer
	 */
	private Table createTabelle(Composite parent)
	{
		if (_tabelle == null)
		{
			_tabelle = new Table(parent, SWT.FULL_SELECTION | SWT.MULTI
					| SWT.H_SCROLL | SWT.V_SCROLL);
			_tabelle.setHeaderVisible(true);
			_tabelle.setLinesVisible(true);

			TableColumn tcStatus = new TableColumn(_tabelle, SWT.LEFT);
			tcStatus.setText("Abschnitt");
			tcStatus.setWidth(300);

			TableColumn tcStrategie = new TableColumn(_tabelle, SWT.LEFT);
			tcStrategie.setText("Aktion");
			tcStrategie.setWidth(125);

			TableColumn tcAktivierungsArt = new TableColumn(_tabelle, SWT.LEFT);
			tcAktivierungsArt.setText("Ursache");
			tcAktivierungsArt.setWidth(125);

			TableColumn tcZeitpunktClosingLoop = new TableColumn(_tabelle, SWT.LEFT);
			tcZeitpunktClosingLoop.setText("Schalten");
			tcZeitpunktClosingLoop.setWidth(150);

			_tvGruppen = new TableViewer(_tabelle);
			_tvGruppen.setContentProvider(new CP());
			_tvGruppen.setLabelProvider(new LP());
			setInput(_data);

			/*
			 * ToolTips auf der Spalte der Strategien.
			 */
			final Listener labelListener = new Listener()
			{
				public void handleEvent(Event event)
				{
					Label label = (Label) event.widget;
					Shell shell = label.getShell();
					switch (event.type)
					{
					case SWT.MouseDown:
						Event e = new Event();
						e.item = (TableItem) label.getData("_TABLEITEM");
						// Assuming table is single select, set the selection as if
						// the mouse down event went through to the table
						_tvGruppen.getTable().setSelection(
								new TableItem[] { (TableItem) e.item });
						_tvGruppen.getTable().notifyListeners(SWT.Selection, e);
						// fall through
					case SWT.MouseExit:
						shell.dispose();
						break;
					}
				}
			};

			Listener tableListener = new Listener()
			{
				Shell tip = null;

				Label label = null;

				public void handleEvent(Event event)
				{
					switch (event.type)
					{
					case SWT.Dispose:
					case SWT.KeyDown:
					case SWT.MouseMove:
					{
						if (tip == null)
							break;
						tip.dispose();
						tip = null;
						label = null;
						break;
					}
					case SWT.MouseHover:
					{
						ViewerCell cell = _tvGruppen.getCell(new Point(event.x, event.y));

						if (cell != null)
						{
							if (tip != null && !tip.isDisposed())
								tip.dispose();
							tip = ShellOnTop.getShell(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), SWT.TOOL); 
							  
							tip.setLayout(new FillLayout());
							label = new Label(tip, SWT.NONE);
							label.setForeground(PlatformUI.getWorkbench()
									.getActiveWorkbenchWindow().getShell().getDisplay()
									.getSystemColor(SWT.COLOR_INFO_FOREGROUND));
							label.setBackground(PlatformUI.getWorkbench()
									.getActiveWorkbenchWindow().getShell().getDisplay()
									.getSystemColor(SWT.COLOR_INFO_BACKGROUND));
							label.setData("_TABLEITEM", cell);
							if (cell.getColumnIndex() == 1)
							{
								// String[] strategieInfos = Tools.getStrategieInfos(cell
								// .getText());
								//
								// if (strategieInfos != null)
								// {
								// label.setText(Tools.buildToolTipTextSwt(strategieInfos));
								// } else
								{
									label.setText(cell.getText());
								}
								label.addListener(SWT.MouseExit, labelListener);
								label.addListener(SWT.MouseDown, labelListener);
								Point size = tip.computeSize(SWT.DEFAULT, SWT.DEFAULT);

								Point pt = _tvGruppen.getTable().toDisplay(event.x, event.y);

								tip.setBounds(pt.x + 12, pt.y + 12, size.x, size.y);
								tip.setVisible(true);
							} else
							{
								// Keinen Tooltip darstellen.
							}
						}
					}
					}
				}
			};
			_tvGruppen.getTable().addListener(SWT.Dispose, tableListener);
			_tvGruppen.getTable().addListener(SWT.KeyDown, tableListener);
			_tvGruppen.getTable().addListener(SWT.MouseMove, tableListener);
			_tvGruppen.getTable().addListener(SWT.MouseHover, tableListener);

			/*
			 * Layout
			 */
			GridData gdTabelleStrategien = new GridData();
			gdTabelleStrategien.grabExcessHorizontalSpace = true;
			gdTabelleStrategien.horizontalAlignment = GridData.FILL;
			gdTabelleStrategien.grabExcessVerticalSpace = true;
			gdTabelleStrategien.verticalAlignment = GridData.FILL;
			gdTabelleStrategien.minimumHeight = 180;

			_tabelle.setLayoutData(gdTabelleStrategien);
		}

		return _tabelle;
	}

	private void addProgrammButtons(Composite parent)
	{
		GridData gridDataButton = new GridData();
		gridDataButton.horizontalAlignment = GridData.CENTER;
		gridDataButton.grabExcessHorizontalSpace = true;
		gridDataButton.grabExcessVerticalSpace = true;
		gridDataButton.verticalAlignment = GridData.CENTER;
		gridDataButton.minimumWidth = 150;

		_buttonTonAusschalten = new Button(parent, SWT.CENTER);
		_buttonTonAusschalten.setText("&Ton ausschalten");
		_buttonTonAusschalten.setLayoutData(gridDataButton);
		// buttonTonAusschalten.setEnabled(false);
		_buttonTonAusschalten.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}

			public void widgetSelected(SelectionEvent e)
			{
				if (_sndTimer != null)
				{
					schalteTonAus();
					_buttonTonAusschalten.setText("&Ton einschalten");
				} else
				{
					startSoundTimer();
					if (_sndTimer != null)
					{
						_buttonTonAusschalten.setText("&Ton ausschalten");
					}
				}
			}
		});
		_buttonTonAusschalten.setEnabled(PreferenceInit.isSndWfAktiv());

		// Wiedervorlage
		Button buttonWiedervorlage = new Button(parent, SWT.PUSH);
		buttonWiedervorlage.setText("&Wiedervorlage");
		buttonWiedervorlage.setLayoutData(gridDataButton);
		buttonWiedervorlage.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionWiedervorlage();
			}
		});

		// Schalten

		Button buttonSchliessen = new Button(parent, SWT.CENTER);
		buttonSchliessen.setText("&Ablehnen");
		buttonSchliessen.setLayoutData(gridDataButton);

		buttonSchliessen.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}

			public void widgetSelected(SelectionEvent e)
			{
				dispose();
			}
		});

		Button bHilfe = new Button(parent, SWT.PUSH);
		bHilfe.setText("&Hilfe");
		bHilfe.setLayoutData(gridDataButton);
		bHilfe.addSelectionListener(new SelectionListener()
		{

			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}

		});

		parent.pack();
	}

	private void addVorschauButtons(Composite parent)
	{
		GridData gridDataComposite = new GridData();
		gridDataComposite.horizontalAlignment = GridData.FILL;
		gridDataComposite.verticalAlignment = GridData.FILL;
		gridDataComposite.grabExcessHorizontalSpace = true;
		gridDataComposite.grabExcessVerticalSpace = true;

		Composite comp = new Composite(parent, SWT.BORDER);
		comp.setLayout(new GridLayout(2, true));
		comp.setLayoutData(gridDataComposite);

		GridData gridDataButton = new GridData();
		gridDataButton.horizontalAlignment = GridData.CENTER;
		gridDataButton.grabExcessHorizontalSpace = true;
		gridDataButton.grabExcessVerticalSpace = true;
		gridDataButton.verticalAlignment = GridData.CENTER;
		gridDataButton.minimumWidth = 200;

		// Vorschau Rendsburg aktivieren

		Button buttonVorschauRendsburg = new Button(comp, SWT.CENTER);
		buttonVorschauRendsburg.setText("Schaltbildvorschau Rendsburg");
		buttonVorschauRendsburg.setLayoutData(gridDataButton);
		// buttonVorschauRendsburg.setEnabled(false);
		buttonVorschauRendsburg.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}

			public void widgetSelected(SelectionEvent e)
			{
				// Opener.openVorschauSchaltungWwaRd();
			}
		});

		// Vorschau Kiel aktivieren

		Button buttonVorschauAkKiel = new Button(comp, SWT.CENTER);
		buttonVorschauAkKiel.setText("Schaltbildvorschau AK Kiel");
		buttonVorschauAkKiel.setLayoutData(gridDataButton);
		// buttonVorschauAkKiel.setEnabled(false);
		buttonVorschauAkKiel.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}

			public void widgetSelected(SelectionEvent e)
			{
				// Opener.openVorschauSchaltungAkKielWestRd();
			}
		});

		parent.pack();
	}

	private void aktionWiedervorlage()
	{
		if (_data != null)
		{
			new WiedervorlageTSFWorkflow(_data);
			this.dispose();
		}
	}

	/**
	 * Aufrumarbeiten am Ende der Lebensphase dieses DLG:
	 * <ul>
	 * <li>Aus dem {@link GUIManager_WorkflowDialog} entfernen,</li>
	 * <li>Dispose auf der verwendeten Shell aufrufen.</li>
	 * </ul>
	 */
	public void dispose()
	{
		// Vom GUI-Manager entfernen
		if (getGuiwfDialogManager() != null)
		{
			getGuiwfDialogManager().removeDialog(this);
		}

		// Buttons Disposen
		synchronized (_lButtons)
		{
			Iterator<Button> it = _lButtons.iterator();
			while (it.hasNext())
			{
				Button button = it.next();
				disposeButton(button);
			}
			_lButtons.clear();
		}

		// Sound deaktivieren
		schalteTonAus();

		// Shell disposen
		if (!shell.isDisposed())
			shell.dispose();
	}

	/**
	 * ContentProvider fr die Tabelle der Benutzer.
	 * 
	 * @author HofmannJ
	 */
	private class CP implements IStructuredContentProvider
	{
		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof M_TSFWorkflow)
			{
				M_TSFWorkflow mTSFWorkflow = (M_TSFWorkflow) inputElement;
				M_TSFWorkflowAbschnitt[] abschnitte = mTSFWorkflow.getAbschnitte();
				re = abschnitte;
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}
	}

	/**
	 * Labelprovider fr die Tabelle der Benutzer.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			Image re = null;

			// if (element instanceof WorkflowStrategieStatusKapsel)
			// {
			// WorkflowStrategieStatusKapsel kapsel = (WorkflowStrategieStatusKapsel)
			// element;
			//
			// switch (columnIndex)
			// {
			// case 0:
			// {
			// if (kapsel.getStatus().equals(WorkflowStategieStatus.AKTIVIEREN))
			// re = getIcon("add_16x16.png");
			//
			// if (kapsel.getStatus().equals(WorkflowStategieStatus.DEAKTIVIEREN))
			// re = getIcon("remove_16x16.png");
			//
			// break;
			// }
			// }
			// }

			if (element instanceof M_TSFWorkflowAbschnitt)
			{
				// Nix zu tun
			}

			return re;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if ((element != null) && (element instanceof M_TSFWorkflowAbschnitt))
			{
				M_TSFWorkflowAbschnitt abschnitt = (M_TSFWorkflowAbschnitt) element;

				switch (columnIndex)
				{
				case 0:
					if (abschnitt.getAbschnitt() != null)
					{
						re = abschnitt.getAbschnitt().getName();
					}
					break;
				case 1:
					if (abschnitt.getAktion() != null)
					{
						re = abschnitt.getAktion().toString();
					}
					break;
				case 2:
					if (abschnitt.getUrsache() != null)
					{
						re = abschnitt.getUrsache().toString();
					}
					break;
				default:
					break;
				}
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	private Image getIcon(String name)
	{
		Image re = null;

		if (name != null)
		{
			if (!_startWithoutEclipseRcp)
			{
				re = Activator.getImageDescriptor("icons/" + name).createImage();
			}
			// {
			// // String pfad =
			// "C:/Dokumente und Einstellungen/Pittner/Eigene Dateien/workspace_sba_mainzer_ring_35/de.sts.vba.rendsburg.plugin.workflow/icons/";
			// // re = new Image(parentShell.getDisplay(), pfad);
			// }
		}
		return re;
	}

	// public static void main(String[] args)
	// {
	// DialogWFAbfrage._startWithoutEclipseRcp = true;
	// parentShell = createSShell();
	//
	// Date d = new Date();
	// DialogWFAbfrage mdlg = new DialogWFAbfrage(parentShell, null);
	//
	// WorkflowDaten daten = new WorkflowDaten();
	//
	// Strategie s1 = new Strategie("S1");
	// Strategie s2 = new Strategie("S2");
	// Strategie s3 = new Strategie("S3");
	// Strategie s4 = new Strategie("S4");
	// Strategie s5 = new Strategie("S5");
	// Strategie s6 = new Strategie("S6");
	// Strategie s7 = new Strategie("S7");
	// Strategie s8 = new Strategie("S8");
	// Strategie s9 = new Strategie("S9");
	// Strategie s10 = new Strategie("S10");
	//
	// // daten.getAktiveStrategien().add(s1);
	// // daten.getAktiveStrategien().add(s2);
	// // daten.getAktiveStrategien().add(s3);
	// //
	// daten.getZuAktivierendeStrategien().add(new StrategieMitWorkflow(s4));
	// // daten.getZuAktivierendeStrategien().add(
	// // new StrategieMitWorkflow(s5, AktivierungsArt.CLOSING_LOOP,
	// // d.getTime() + 60000));
	// // daten.getZuAktivierendeStrategien().add(new StrategieMitWorkflow(s6));
	// // daten.getZuAktivierendeStrategien().add(new StrategieMitWorkflow(s7));
	// // daten.getZuAktivierendeStrategien().add(
	// // new StrategieMitWorkflow(s8, AktivierungsArt.CLOSING_LOOP,
	// // d.getTime() + 120000));
	// // daten.getZuAktivierendeStrategien().add(new StrategieMitWorkflow(s9));
	// // daten.getZuAktivierendeStrategien().add(new StrategieMitWorkflow(s10));
	// //
	// // daten.getZuDeaktivierendeStrategien().add(new StrategieMitWorkflow(s2));
	//
	// daten.setId("");
	// daten.setZusatz("");
	//
	// // mdlg.oeffne(daten);
	// mdlg.oeffne(null);
	// }

	/**
	 * This method initializes sShell
	 */
	public static Shell createSShell()
	{
		Shell shell = new Shell();
		shell.setText("Shell");
		shell.setSize(new Point(300, 200));
		shell.setLayout(new GridLayout());
		return shell;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run()
	{
		int verzoegerung = 10;
		while (verzoegerung > 0)
		{
			verzoegerung--;

			try
			{
				Thread.sleep(1000);
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}
	}

	private void aktionHilfe()
	{
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(HELP_ID);
	}

	/**
	 * Ordnet einen Butten mit der Pid eines Abschnitts einen
	 * {@link PropertyChangeListener} zu.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class ButtonPropertyChangeListener implements PropertyChangeListener
	{
		private Button _button = null;
		private M_TSFWorkflowAbschnitt _tsfWorkflowAbschnitt;

		private ButtonPropertyChangeListener(Button button,
				M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt)
		{
			this._button = button;
			this._tsfWorkflowAbschnitt = tsfWorkflowAbschnitt;
			listenerAnmelden();
		}

		public SystemObject getSoTfsAbschnitt()
		{
			SystemObject soTfsAbschnitt = null;

			if (getTsfWorkflowAbschnitt() != null)
			{
				soTfsAbschnitt = getTsfWorkflowAbschnitt().getAbschnitt();
			}

			return soTfsAbschnitt;
		}

		public M_TSFWorkflowAbschnitt getTsfWorkflowAbschnitt()
		{
			return _tsfWorkflowAbschnitt;
		}

		public Button getButton()
		{
			return _button;
		}

		/**
		 * Liefert das lt. Workflow geforderte Zielprogramm
		 * 
		 * @return
		 */
		public E_TSFBefehl getZielProgrammWf()
		{
			E_TSFBefehl zielProgrammWf = null;

			if (getTsfWorkflowAbschnitt() != null)
			{
				M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt = getTsfWorkflowAbschnitt();
				zielProgrammWf = tsfWorkflowAbschnitt.getAktion();

			}

			return zielProgrammWf;
		}

		private void listenerAnmelden()
		{
			if (getSoTfsAbschnitt() != null)
			{
				DaVKomTSFSteuerung.getInstance().getModelVorgabe().addListener(
						getSoTfsAbschnitt(), this);
			}
		}

		private void listenerAbmelden()
		{
			if (getSoTfsAbschnitt() != null)
			{
				DaVKomTSFSteuerung.getInstance().getModelVorgabe().removeListener(
						getSoTfsAbschnitt(), this);
			}
		}

		@Override
		public void propertyChange(PropertyChangeEvent evt)
		{
			if ((evt != null)
					&& (evt.getNewValue() instanceof ZustandSammlung))
			{
				E_TSFBefehl zielProgrammWf = getZielProgrammWf();

				if (zielProgrammWf != null)
				{
					ZustandSammlung sammlung = (ZustandSammlung) evt
							.getNewValue();

					if (sammlung.getSoTfsAbschnitt() == getSoTfsAbschnitt())
					{
						/*
						 * ZielProgramm lt. Steuerung muss dem geforderten Zielprogramm des
						 * Workflows entsprechen.
						 */
						boolean istZielPrgGeschaltet = false;
						boolean istSchaltvorgangImGange = false;
						if ((sammlung.getZielProgramm() != null)
								&& (sammlung.getZielProgramm() == zielProgrammWf))
						{
							if ((sammlung.getProgrammZustand() != null)
									&& (sammlung.getProgrammZustand() == E_TSFProgrammZustand.Geschaltet))
							{
								istZielPrgGeschaltet = true;
							}

							if (sammlung.getProgrammZustand() == E_TSFProgrammZustand.In_Bearbeitung)
							{
								istSchaltvorgangImGange = true;
							}
						}

						boolean buttonAktivieren = ((!istZielPrgGeschaltet) && (!istSchaltvorgangImGange));

						if (getButton() != null)
						{
							final boolean enableButton = !istZielPrgGeschaltet;
							final boolean fButtonAktivieren = buttonAktivieren;

							Runnable run = new Runnable()
							{
								@Override
								public void run()
								{
									// getButton().setEnabled(fButtonAktivieren);

									/*
									 * Button aktivieren/deaktivieren nur wenn ntig.
									 */
									if (getButton().isEnabled() != fButtonAktivieren)
									{
										getButton().setEnabled(fButtonAktivieren);
										System.out
												.println(">>> Setze Button auf " + fButtonAktivieren
														+ " fr Abs " + getSoTfsAbschnitt());
									}

								}
							};
							PlatformUI.getWorkbench().getDisplay().asyncExec(run);

						}
					}
				}
			}
		}

		public void dispose()
		{
			listenerAbmelden();
		}
	}

	private Timer startSoundTimer()
	{
		if (PreferenceInit.isSndWfAktiv())
		{
			if (_sndTimer == null)
			{
				int delaySndWf = PreferenceInit.getDelaySndWf();
				final String pathSndWf = PreferenceInit.getPathSndWf();

				TimerTask sndTask = new TimerTask()
				{
					@Override
					public void run()
					{
						new AePlayWave(pathSndWf).start();
					}
				};

				_sndTimer = new Timer("WF Timer");
				_sndTimer.schedule(sndTask, 0l, delaySndWf * 1000l);
			}
		}

		return _sndTimer;
	}

	private void schalteTonAus()
	{
		if (_sndTimer != null)
		{
			_sndTimer.cancel();
			_sndTimer = null;
		}
	}

}
