/*  Copyright (C) 2004 by Friederich Kupzog Elektronik & Software
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
    Author: Friederich Kupzog  
    fkmk@kupzog.de
    www.kupzog.de/fkmk
*/
 
 
package de.kupzog.ktools.kprint.gui;

import org.eclipse.swt.custom.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;
import org.eclipse.swt.printing.*;

import de.kupzog.ktools.kprint.boxes.*;


/**
 * A GUI dialog that layouts and displays a PDocument.
 * It also allows to print the document.
 * @author Friederich Kupzog
 */
public class PrintPreview 
extends KDialog {
	private final static int SPACE_ARROUND_PAGE = 10;

	protected PDocument document;
	
	protected CLabel guiImageLabel;
	protected Text guiPageText;
	protected Combo guiZoom;
	protected ScrolledComposite guiScrollArea;
	
	protected boolean layoutNeccessary;
	
	protected int percent;
	
	protected int page;
	
	private ToolItem firstPageToolItem;
	private ToolItem lastPageToolItem;
	private ToolItem nextPageToolItem;
	private ToolItem prevPageToolItem;
	
	/**
	 * @param parent
	 * @param title
	 * @param icon
	 */
	public PrintPreview(Shell parent, String title, Image icon, PDocument doc) {
		super(parent, title, icon);
		createContents();
		document = doc;
		page = 1;
		percent = 100;
		layoutNeccessary = true;

		addToolItem("print", Messages.getString("PrintPreview.1"), IconSource.getImage("print")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		firstPageToolItem = addToolItem("first", Messages.getString("PrintPreview.4"), IconSource.getImage("FirstRec")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		prevPageToolItem = addToolItem("prev", Messages.getString("PrintPreview.7"), IconSource.getImage("PrevRec")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		nextPageToolItem = addToolItem("next", Messages.getString("PrintPreview.10"), IconSource.getImage("NextRec")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		lastPageToolItem = addToolItem("last", Messages.getString("PrintPreview.13"), IconSource.getImage("LastRec")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		Button close = addButtonRight(Messages.getString("PrintPreview.butClose"),""); //$NON-NLS-1$ //$NON-NLS-2$
		//addButtonRight("Seite &einrichten","");
		close.setFocus();
		
		guiPageText = new Text(guiToolBarArea, SWT.BORDER);
		guiPageText.setText(guiPageText.getText()+"        "); //$NON-NLS-1$
		guiPageText.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		adjustToToolBar(guiPageText);
		guiPageText.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent arg0) {
				guiPageText.setText(""); //$NON-NLS-1$
			}
			public void focusLost(FocusEvent arg0) {
				setPageText();
			}
		});
		guiPageText.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent arg0) {
				if (arg0.keyCode == SWT.CR)
				{
					onEnterInPageText();
				}
			}
		});
		guiPageText.setToolTipText(Messages.getString("PrintPreview.19")); //$NON-NLS-1$
		
		guiZoom = new Combo(guiToolBarArea, SWT.BORDER | SWT.READ_ONLY);
		guiZoom.add("500%"); //$NON-NLS-1$
		guiZoom.add("200%"); //$NON-NLS-1$
		guiZoom.add("100%"); //$NON-NLS-1$
		guiZoom.add("80%"); //$NON-NLS-1$
		guiZoom.add("50%"); //$NON-NLS-1$
		guiZoom.add("20%"); //$NON-NLS-1$
		guiZoom.add(Messages.getString("PrintPreview.26")); //$NON-NLS-1$
		adjustToToolBar(guiZoom);
		guiZoom.setToolTipText(Messages.getString("PrintPreview.27")); //$NON-NLS-1$
		guiZoom.select(2);
		guiZoom.addSelectionListener(
			new SelectionAdapter() {
				public void widgetSelected(SelectionEvent arg0) {
					onCombo(((Combo)arg0.widget).getText());
				}
			}
		);
		guiMainArea.setLayout(new FillLayout());
		guiScrollArea = new ScrolledComposite(guiMainArea,SWT.H_SCROLL | SWT.V_SCROLL);
		guiScrollArea.getVerticalBar().setIncrement(12);
		guiImageLabel = new CLabel(guiScrollArea, SWT.DRAW_TRANSPARENT);
		guiScrollArea.setContent(guiImageLabel);
		showPage();
		guiImageLabel.setSize(guiImageLabel.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		
	}
	
	private void onEnterInPageText()
	{
		try {
			int mypage = Integer.parseInt(guiPageText.getText());
			if (mypage > 0 && mypage <= document.getNumOfPages())
			{
				page = mypage;
				showPage();
				guiZoom.setFocus();
			}
			else
			{
				MsgBox.show(Messages.getString("PrintPreview.28")+document.getNumOfPages() +Messages.getString("PrintPreview.29")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		} catch (Exception e) {
			MsgBox.show(Messages.getString("PrintPreview.30")); //$NON-NLS-1$
		}
	}

	
	private void setPageText() {
		guiPageText.setText(Messages.getString("PrintPreview.31")+page+Messages.getString("PrintPreview.32")+ document.getNumOfPages()+ "       "); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	public int getShellStyle() {
		return super.getShellStyle() | SWT.MAX | SWT.MIN;
	}

	
	protected void doLayout() {
		int x = Display.getCurrent().getBounds().width - 100;
		int y = Display.getCurrent().getBounds().height - 10;
		guiShell.setSize(x,y);
		guiShell.setMaximized(true);
	}
	


	/**
	 * Renders a preview Image
	 * @param document
	 * The PDocument to render
	 * @param page
	 * The page to render
	 * @param layoutNeccessary
	 * If true, the document will be layouted before rendering
	 * @param percent
	 * The scaling factor for the image
	 * @return
	 */
	public static Image getPageImage(
			PDocument document, 
			int page, 
			boolean localLayoutNeccessary, 
			int percent)
	{
		Shell cursorShell = Display.getCurrent().getActiveShell();
		if (cursorShell != null) cursorShell.setCursor(Display.getCurrent().getSystemCursor(SWT.CURSOR_WAIT));
		Point dpi = Display.getCurrent().getDPI();
		int shadowSize = (int) (5.0*percent/100.0);
		try {
			int h = (int)Math.round(document.getPageHeight()/2.54*dpi.y*percent/100 + shadowSize);
			int w = (int)Math.round(document.getPageWidth()/2.54*dpi.x*percent/100 + shadowSize);
			Image newImage = new Image(Display.getCurrent(), w, h);
			GC gc = new GC(newImage);
			
			PBox.setParameters(gc, Display.getCurrent(), dpi, percent);
			if (localLayoutNeccessary) document.layout();
			document.draw(page);

			// Schatten
			
			gc.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GRAY));
			gc.fillRectangle(0, newImage.getBounds().height-shadowSize, 
							newImage.getBounds().width, newImage.getBounds().height);
			gc.fillRectangle(newImage.getBounds().width-shadowSize, 0, 
							newImage.getBounds().width-shadowSize, newImage.getBounds().height);

			gc.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
			gc.fillRectangle(0, newImage.getBounds().height-shadowSize, 
					shadowSize, newImage.getBounds().height);
			gc.fillRectangle(newImage.getBounds().width-shadowSize, 0, 
							 newImage.getBounds().width, shadowSize);

			
			gc.dispose(); 
			if ( SPACE_ARROUND_PAGE > 0 ) {
				Image newImage2 = new Image(Display.getCurrent(),w + 2 * SPACE_ARROUND_PAGE ,h + 2 * SPACE_ARROUND_PAGE );
				gc = new GC(newImage2);
				gc.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
				gc.fillRectangle(0, 0, 
						newImage2.getBounds().width, newImage2.getBounds().height);
				gc.drawImage(newImage,  SPACE_ARROUND_PAGE , SPACE_ARROUND_PAGE );
				gc.dispose();
				newImage.dispose();
				return newImage2;
			} else {
				return newImage;
			}
		}
		catch (Exception e)
		{
			System.out.println(e.getMessage());
			return null;
		}	finally {
			if (cursorShell != null)
				cursorShell.setCursor(null);
		}			
	}
	
	public void open() {
		guiScrollArea.setFocus();
		super.open();
	}
	
	protected void onCombo(String text)
	{
		if (text.startsWith(Messages.getString("PrintPreview.34"))) //$NON-NLS-1$
		{
			long ypixel = Math.round(document.getPageHeight()/2.54*Display.getCurrent().getDPI().y);
			long xpixel = Math.round(document.getPageWidth()/2.54*Display.getCurrent().getDPI().x);
			
			ypixel += 2* SPACE_ARROUND_PAGE + 5 + 7; // 7 extra for rounding errors
			xpixel += 2* SPACE_ARROUND_PAGE + 5 + 7;
			
			int yscale = (int)(100*(guiScrollArea.getBounds().height)/ypixel);
			int xscale = (int)(100*(guiScrollArea.getBounds().width)/xpixel);
			
			percent = Math.min(yscale, xscale);
		}
		else
		{
			text = text.substring(0,text.length()-1);
			percent = 100;
			try {
				percent = Integer.parseInt(text);	
			} catch (Exception e1) {
				MsgBox.show("'"+text+Messages.getString("PrintPreview.36")); //$NON-NLS-1$ //$NON-NLS-2$
				guiZoom.select(3);
			}
		}
		if (guiImageLabel.getImage()!= null) guiImageLabel.getImage().dispose();
		guiImageLabel.setImage(getPageImage(
				document,
				page,
				true,
				percent));
		layoutNeccessary = false;
		guiImageLabel.setSize(guiImageLabel.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		guiScrollArea.setFocus();
		
	}
	
	protected void onToolItem(ToolItem toolitem, String name) {
		if (name.equals("next")) //$NON-NLS-1$
		{
			if (page < document.getNumOfPages())
			{
				page++;
				showPage();
			}
		}
		else if (name.equals("prev")) //$NON-NLS-1$
		{
			if (page > 1)
			{
				page--;
				showPage();
			}
		}
		else if (name.equals("first")) //$NON-NLS-1$
		{
			page = 1;
			showPage();
		}
		else if (name.equals("last")) //$NON-NLS-1$
		{
			page = document.getNumOfPages();
			showPage();
		}
		else if (name.equals("print")) //$NON-NLS-1$
		{
			onPrint();
		}
	}
	
	private void showPage() {
		Image oldImg = guiImageLabel.getImage();
		guiImageLabel.setImage(getPageImage(
				document,
				page,
				layoutNeccessary,
				percent));
		layoutNeccessary = false;
		setPageText();
		firstPageToolItem.setEnabled( page > 1 );
		prevPageToolItem.setEnabled( page > 1 );
		lastPageToolItem.setEnabled( page < document.getNumOfPages() );
		nextPageToolItem.setEnabled ( page < document.getNumOfPages() );
		if (oldImg!= null) oldImg.dispose();
	}

	protected void onButton(Button button, String buttonText) {
		if (buttonText.startsWith(Messages.getString("PrintPreview.butClose"))) onClose(); //$NON-NLS-1$
		else if (buttonText.startsWith(Messages.getString("PrintPreview.43"))) onPageSetup(); //$NON-NLS-1$
	}
	
	protected void onClose()
	{
		if (guiImageLabel.getImage()!= null) guiImageLabel.getImage().dispose();
		PTextStyle.disposeFonts();
		close();
	}
	
	protected void onPageSetup()
	{
		/* funktioniert nicht:
		 * - Abgeschnittene Tabellen bleiben abgeschnitten
		 * - Skalierung geht nicht
		 */
		PageSetup ps = new PageSetup(guiShell);
		ps.open();
		document.readMeasuresFromPageSetup();
		if (guiImageLabel.getImage()!= null) guiImageLabel.getImage().dispose();
		guiImageLabel.setImage(getPageImage(
				document,
				page,
				true,
				percent));
		layoutNeccessary = false;
		guiImageLabel.setSize(guiImageLabel.computeSize(SWT.DEFAULT, SWT.DEFAULT));

	}
	
	
	protected void onPrint()
	{
		PrintDialog dialog = new PrintDialog(guiShell, SWT.BORDER);
		PrinterData data = dialog.open();
		if (data == null) return;
		if (data.printToFile) {
			FileDialog d = new FileDialog (guiShell, SWT.SAVE);
			d.setFilterNames (new String [] {"All Files (*.*)"}); //$NON-NLS-1$
			d.setFilterExtensions (new String [] {"*.*"}); //Windows wild cards //$NON-NLS-1$
			d.setFilterPath ("c:\\"); //Windows path //$NON-NLS-1$
			d.setFileName (""); //$NON-NLS-1$
			d.open ();
			data.fileName = d.getFilterPath() + "\\" + d.getFileName(); //$NON-NLS-1$
			
		}
	
		Printer printer = new Printer(data);
		GC gc = new GC(printer);
	
		PBox.setParameters(gc, printer, printer.getDPI(), (int)(document.getScale()*100));
		document.layout();
	
		if (printer.startJob(document.getTitle())) 
		{
			if (data.scope == PrinterData.ALL_PAGES)
			{
				data.startPage = 1;
				data.endPage = document.getNumOfPages();
			}
			else if (data.scope == PrinterData.SELECTION)
			{
				data.startPage = page;
				data.endPage = page;
			}
			else if (data.scope == PrinterData.PAGE_RANGE)
			{
				if (data.startPage > document.getNumOfPages()) 
					data.startPage = document.getNumOfPages();
				if (data.endPage > document.getNumOfPages()) 
					data.endPage = document.getNumOfPages();
			}
	
			for (int page = data.startPage; page <= data.endPage; page++)
			{
				printer.startPage();
				document.draw(page);
				printer.endPage();
			}
			printer.endJob();
		}
		gc.dispose();
		printer.dispose();
		layoutNeccessary = true;
		
	}


	
	
	

}
