/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.Formatter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LightweightSystem;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.parts.ScrollableThumbnail;
import org.eclipse.draw2d.parts.Thumbnail;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.help.internal.toc.HrefUtil;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ListDialog;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.internal.ActionDescriptor;
import org.eclipse.ui.internal.PluginAction;
import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorLVE;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.FahrStreifenModelAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.VerhaltenDoTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.DoMitVerbLinieEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.editor.GraphicalEditorToView;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaAnsichtProvider;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Anzeige;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Bild;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.CompositeSperrungBereiche;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.CompositeSperrungDarstellung;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.DynamischerWechselText;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Schaltbild;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgramm;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgrammKopierenDialog;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Sperrung;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Anzeige.EnumAnzeigeProgrammStatus;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.DynamischerWechselText.EnumDynamischerWechseltextTextAusrichtung;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgramm.EnumSonderProgrammAktion;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgramm.EnumSonderProgrammRichtung;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgramm.EnumSonderProgrammZustand;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.SituationUtil;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.DarstellungEditPartFactory;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit.Eingabetyp;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegende;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.dynobj.DynamischeObjekte;

/**
 * View-Klasse, welche die Sicht der SonderProgramme darstellt
 * 
 * @author Pittner
 * 
 */
public class ZustandsDarstellungSonderProgramm extends GraphicalEditorToView implements IDaVDataUpdateListener
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.zustandsdarstellungSonderProgramm";

  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection; // @jve:decl-index=0:

  /**
   * Die Sonderprogramm-Datenstruktur
   */
  private SonderProgramm _sonderProgrammDatensatz;

  /**
   * Die Liste mit den DaV-Sonderprogramm-Daten
   */
  private Map<String, ResultData> _sonderProgrammMap = new HashMap<String, ResultData>();

  /**
   * Die Liste der Programmtypen
   */
  private List<SswSbaProgrammTyp> _listProgrammTypen;

  /**
   * Die Schaltflaechen
   */
  private Button _radioButtonAbSofort;

  private Button _radioButtonVorlaeufigNicht;

  private Button _radioButtonVon;

  private Button _radioButtonSimulation = null;

  private Button _radioButtonSchaltung = null;

  private Button _buttonVerlassen = null;

  private Button _buttonOk = null;

  private Button _buttonReset = null;

  private Button _buttonLoeschen = null;

  private Button _radioButtonBisJetzt;

  private Button _radioButtonBisAufWeiteres;

  private Button _radioButtonBis;

  private Button _radioButtonQuerLaengsabgleich;

  private Button _radioButtonAusfallbehandlung;

  private Button _radioButtonVerschiebung;

  private Button _radioButtonAufhebung;

  private Button _radioButtonHandschaltungAktiv;

  private Button _radioButtonPrioritaet;

  /**
   * Die Gruppen
   */
  private Group groupFilterDefinitionen12;

  private Group _groupFilterDefinitionenBetriebsart;

  private Group _groupFilterDefinitionenHandschaltung;

  private Group groupFilterDefinitionen11;

  /**
   * Die Textfelder
   */
  private Text textStartKm = null;

  private Text textStopKm = null;

  private Text _textName;

  /**
   * Stundenauswahl Combobox
   */
  private Combo _comboStundenauswahl = null;

  /**
   * MouseAdapter fr die Auswahl der Stunden manuell per Maus.
   */
  private MouseAdapter _mouseAdapterStundenAuswahl = null;

  private Combo _comboProgrammTyp;

  /**
   * Combo fr die Wochentagsauswahl
   */
  private Combo _comboWochentagsauswahl;

  /**
   * MouseAdapter fr die Auswahl der Wochentage manuell per Maus.
   */
  private MouseAdapter _mouseAdapterWochentagsAuswahl = null;

  /**
   * Combo fr die Monatsauswahl.
   */
  private Combo _comboMonatsauswahl;

  /**
   * MouseAdapter fr die Auswahl der Monate manuell per Maus.
   */
  private MouseAdapter _mouseAdapterMonatsAuswahl = null;

  private Combo _comboPrioritaet;

  private ComboViewer _comboViewerProgrammTyp;

  private ComboViewer _cvComboBoxPrioritaet;

  /**
   * Die Labels
   */
  private Label labelTyp;

  private Label labelName;

  private Label label21;

  private Label label22;

  private Label labelStopKm;

  private Label labelStartKm;

  /**
   * Die Shash
   */
  private SashForm _sf1;

  /**
   * Die Composites
   */
  private Composite _parent;

  private Composite _composite;

  private Composite composite = null;

  private Composite composite2 = null;

  private Composite composite3 = null;

  private Composite _compositeUntenLinks = null;

  private Composite _compositeUntenRechts = null;

  private Composite _compositeUntenLinksUnten = null;

  private Composite _compositeZeitwaehlerStunden;

  private Composite _compositeZeitwaehlerWochentage;

  private Composite _compositeZeitwaehlerMonate;

  private CompositeSperrungBereiche _compositeFahrstreifenSperrungen;

  /**
   * Der Menuemanager fr das Kontextmenue der Strecken
   */
  private MenuManager _kontextMenuMgr;

  // /**
  // * Der GraphicalViewer
  // */
  // private GraphicalViewer graphicalViewer;

  /**
   * Die bersicht
   */
  private Thumbnail _thumbnail;

  /**
   * Die Zeitspezifikationselemente
   */
  private DatumZeit zeitVon;

  private DatumZeit zeitBis;

  /**
   * Die Tabellen
   */
  private TableViewer _tvEreignisse;

  private TableViewer _tvSystemKalenderEintraege;

  /**
   * Die Secondary-ViewId
   */
  private String _secondaryId;

  private Map<String, List<ResultData>> _mapWanderBaustellen = new HashMap<String, List<ResultData>>();

  private Map<String, List<ResultData>> _mapProgrammHierarchie = new HashMap<String, List<ResultData>>();

  private Button _buttonKopieren;
  
  private ZustandsDarstellungSonderProgramm _instance;
  
  private DynamischeObjekte _verwaltungDynamischeObjekte;
  
  /**
   * Vorsatz zum Bilden der Pid fr Ursacheneinheiten zum Schalten von Sonderprogramme.
   */
  private final String _praefixPid = "sba.sp";
  /**
   * Anhang zum Bilden der Pid fr Ursacheneinheiten zum Schalten von Sonderprogramme.
   */
  private String _postfixPid;

  private Composite _compositePicker;
  
//  private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);
//  
//  public final static String PROP_SONDER_PROGRAMM_INIT_SW = "SONDER_PROGRAMM_INIT_SW";
//  public final static String PROP_SONDER_PROGRAMM_INIT_NO = "SONDER_PROGRAMM_INIT_NO";
//
//  public final static String PROP_SONDER_PROGRAMM_DATA_SW = "SONDER_PROGRAMM_DATA_SW";
//  public final static String PROP_SONDER_PROGRAMM_DATA_NO = "SONDER_PROGRAMM_DATA_NO";


  @Override
  public void init(IViewSite site) throws PartInitException
  {
    _secondaryId = site.getSecondaryId();
    _instance = this;
    if (_secondaryId != null && _secondaryId.length() > 0)
    {
      setPartName(getPartName() + " FR " + _secondaryId);
    }
    super.init(site);
  }

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;
    if (DaVKommunikationDobj.getInstanz().holeDaVVerbindung() != null)
    {
      _connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      _verwaltungDynamischeObjekte = DynamischeObjekte.getInstanz(_connection);
      IPreferenceStore store = Activator.getDefault().getPreferenceStore();
      _postfixPid = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    }

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.CENTER;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.grabExcessHorizontalSpace = false;
    gridData11.verticalAlignment = GridData.CENTER;
    gridData11.horizontalAlignment = GridData.CENTER;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.CENTER;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.BEGINNING;
    gridData.grabExcessHorizontalSpace = false;
    gridData.verticalAlignment = GridData.CENTER;
    GridData gridData3 = new GridData();
    gridData3.verticalIndent = 0;
    _sf1 = new SashForm(parent, SWT.HORIZONTAL | SWT.BORDER);
    // _sc = new ScrolledComposite(_sf1, SWT.H_SCROLL | SWT.V_SCROLL);
    // _composite = new Composite(_sc, SWT.NONE);
    _composite = new Composite(_sf1, SWT.NONE);

    GridLayout gl = new GridLayout();
    gl.marginHeight = 0;
    gl.marginTop = 0;
    gl.marginBottom = 0;
    gl.verticalSpacing = 0;

    _composite.setLayout(gl);
    _composite.setLayoutData(gridData3);

    // Ansicht in einen extra Viewer packen.
    graphicalViewer = new ScrollingGraphicalViewer();
    graphicalViewer.createControl(_sf1);
    /*
     * Wichtig - Damit wei der Viewer welchen EditPart er fr ein Modell Objekt benutzen soll.
     */
    graphicalViewer.setEditDomain(new EditDomain());
    graphicalViewer.setRootEditPart(new SbaScalableRootEditPart());
    graphicalViewer.setEditPartFactory(new DarstellungEditPartFactory());

    getCommandStack().setUndoLimit(1);
    
    String bezeichnungAnsicht = null;
    if (_secondaryId.equals("SW"))
    {
      bezeichnungAnsicht = DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_SW;
      // name = store.getString(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_SW);
      // Fr die F1 Taste
      PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".sonderSW");
    }
    else if (_secondaryId.equals("NO"))
    {
      bezeichnungAnsicht = DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_NO;
      // name = store.getString(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_NO);
      // Fr die F1 Taste
      PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".sonderNO");
    }
   
    // if (name != null && name.length() > 0)
    // graphicalViewer.setContents(EcoreUtil.copy(SbaEinstellungen.INSTANCE.getAnsicht(name)));

    Ansicht ansicht = SbaAnsichtProvider.getInstanz().getAnsichtKopie(bezeichnungAnsicht);
    if (ansicht != null)
      graphicalViewer.setContents(ansicht);

    restoreFromPrefs();

    initialize();

    createDropDownAction();

    _sf1.setWeights(new int[] { 120, 200 });
    _composite.pack();
    // _sc.setContent(_composite);

    if (_connection != null)
    {
      // AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
      // Ansicht ansicht = contents.getModel();

      StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
      darstellung.setSpaltenBorder(true);

      List<SystemObject> list = new ArrayList<SystemObject>();

      EList<Ebene> ebenen = darstellung.getEbenen();
      for (Ebene ebene : ebenen)
      {
        EList<DoModel> dos = ebene.getDoObjekte();
        for (DoModel dom : dos)
        {
          if (dom == null)
            System.out.println("dom ist null: " + dom);

          if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
          {
            AnzeigeQuerschnittModelTypAbstract model = (AnzeigeQuerschnittModelTypAbstract)dom;
            list.add(model.getSystemObject());

            EList<AnzeigeModelTypAbstract> komponenten = model.getKomponenten();
            for (AnzeigeModelTypAbstract komp : komponenten)
            {
              list.add(komp.getSystemObject());
            }
          }
          if (dom instanceof FahrStreifenModelAbstract)
          {
            FahrStreifenModelAbstract model = (FahrStreifenModelAbstract)dom;
            list.add(dom.getSystemObject());

            EList<DetektorLVE> komponenten = model.getKomponenten();
            for (DetektorLVE komp : komponenten)
            {
              list.add(komp.getSystemObject());
            }
          }
        }
      }

      DaVKommunikationDobj.getInstanz().anmeldenAufDaten(list);
      DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);
      DaVKommunikationDobj.getInstanz().anmeldenAufDatenSonder();
      
      anmeldenSituationen(_connection);

    }

    erzeugeKontextMenue();
    registriereActions();

    graphicalViewer.getControl().addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDoubleClick(MouseEvent e)
      {
        ISelection selection = graphicalViewer.getSelection();
        IStructuredSelection ss = (IStructuredSelection)selection;
        if (ss != null && !ss.isEmpty())
        {
          Object obj = ss.getFirstElement();
          if (obj instanceof DoModelEditPart)
          {
            DoModelEditPart part = (DoModelEditPart)obj;

            DoModel model = (DoModel)part.getModel();
            DoTyp doTyp = model.getDoTyp();

//            System.out.println("DoubleClick!!!: " + model.getSystemObject());

            if (doTyp instanceof VerhaltenDoTyp)
            {
              VerhaltenDoTyp coDoTyp = (VerhaltenDoTyp)doTyp;

              int kontext = -1;

              if (model instanceof AnzeigeQuerschnittModelTypAbstract)
              {
                AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)model;
                kontext = aq.getKontext();
              }
              else if (model instanceof AnzeigeModelTypAbstract)
              {
                AnzeigeModelTypAbstract a = (AnzeigeModelTypAbstract)model;
                EObject parent = a.eContainer();
                if (parent != null)
                {
                  if (parent instanceof AnzeigeQuerschnittModelTypAbstract)
                  {
                    AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parent;
                    kontext = aq.getKontext();
                  }
                }
              }

              String id = null;

              switch (kontext)
              {
                case 0:
                  id = coDoTyp.getDoppelKlickIdIstZustand();
                  break;
                case 1:
                  id = coDoTyp.getDoppelKlickIdSollZustand();
                  break;
                case 2:
                  id = coDoTyp.getDoppelKlickIdHandSchaltung();
                  break;
                case 3:
                  id = coDoTyp.getDoppelKlickIdSimulation();
                  break;
                case 4:
                  id = coDoTyp.getDoppelKlickIdTrichter();
                  break;

                default:
                  break;
              }

              if (id != null)
                leseActions(id);
            }
          }
        }
      }
    });
    getSite().setSelectionProvider(this.getGraphicalViewer());
    aktualisiereSichtKeineSelektion();
  }

  private void createDropDownAction()
  {
    IActionBars actionBars = getViewSite().getActionBars();
    IMenuManager menuManager = actionBars.getMenuManager();
    if (menuManager.isEmpty())
    {
      AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
      Ansicht ansicht = contents.getModel();
      Darstellung darstellung = ansicht.getDarstellung();
      EList<Double> stufen = darstellung.getZoomStufen();

      final ScalableRootEditPart rootEditPart = (ScalableRootEditPart)graphicalViewer.getRootEditPart();
      final ZoomManager zoomManager = rootEditPart.getZoomManager();

      double[] dd = new double[stufen.size()];

      for (int i = 0; i < dd.length; i++)
      {
        dd[i] = stufen.get(i);
      }
      zoomManager.setZoomLevels(dd);
      for (final Double stufe : stufen)
      {
        Action zoomAction = new Action() {
          public void run()
          {
            zoomManager.setZoom(stufe);
          }

        };
        zoomAction.setText("Zoomstufe " + stufe);
        menuManager.add(zoomAction);
      }
    }
  }

  private void initialize()
  {
    GridData gdParent = new GridData();
    gdParent.horizontalAlignment = GridData.FILL;
    gdParent.grabExcessHorizontalSpace = true;
    gdParent.grabExcessVerticalSpace = true;
    gdParent.verticalAlignment = GridData.FILL;
    Composite parent4Form = new Composite(_composite, SWT.NONE);
    parent4Form.setLayout(new GridLayout());
    parent4Form.setLayoutData(gdParent);
    createForm(parent4Form);
  }

  private void createForm(Composite parent)
  {
    // Sets up the toolkit.
    FormToolkit toolkit = new FormToolkit(parent.getDisplay());

    // Creates a form instance.
    // Form form = toolkit.createForm(composite);
    ScrolledForm form = toolkit.createScrolledForm(parent);
    form.setLayoutData(new GridData(GridData.FILL_BOTH));

    // Sets title.
    // form.setText("Definition Sonder- und Handprogramm");

    // Adds body contents.
    form.getBody().setLayout(new GridLayout(1, false));

    /*
     * Section Programmtyp-/Namensauswahl
     */
    Section sectionProgrammtypNamensauswahl = toolkit.createSection(form.getBody(), Section.TITLE_BAR
        | Section.EXPANDED);
    sectionProgrammtypNamensauswahl.setText("Programm-Typ und Name");
    Composite clientProgrammtypNamensauswahl = toolkit.createComposite(sectionProgrammtypNamensauswahl, SWT.WRAP);
    clientProgrammtypNamensauswahl.setLayout(new GridLayout());
    sectionProgrammtypNamensauswahl.setClient(clientProgrammtypNamensauswahl);
    sectionProgrammtypNamensauswahl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createCompositeProgrammTypName(clientProgrammtypNamensauswahl);

    /*
     * Streckenbereich + Fahrstreifensperrung.
     */
    Section sectionStrecke = toolkit.createSection(form.getBody(), Section.TITLE_BAR | Section.TREE_NODE
        | Section.EXPANDED);
    sectionStrecke.setText("Streckenbereich");
    Composite clientStrecke = toolkit.createComposite(sectionStrecke, SWT.WRAP);
    clientStrecke.setLayout(new GridLayout());
    sectionStrecke.setClient(clientStrecke);
    sectionStrecke.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    // createAuswahlComposite(clientStrecke);
    createCompositeStreckenbereich(clientStrecke);

    /*
     * Fahrstreifensperrung
     */
    Section sectionFahrstreifensperrung = toolkit.createSection(form.getBody(), Section.TITLE_BAR | Section.TREE_NODE
        | Section.COMPACT);
    sectionFahrstreifensperrung.setText("Fahrstreifensperrung");
    Composite clientFahrstreifensperrung = toolkit.createComposite(sectionFahrstreifensperrung, SWT.WRAP);
    clientFahrstreifensperrung.setLayout(new GridLayout());
    sectionFahrstreifensperrung.setClient(clientFahrstreifensperrung);
    sectionFahrstreifensperrung.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    // createAuswahlComposite(clientFahrstreifensperrung);
    createCompositeSperrungFahrstreifen(clientFahrstreifensperrung);

    /*
     * Section Handschaltung & Betriebsart
     */
    Section sectionHandschaltungBetriebsart = toolkit.createSection(form.getBody(), Section.TITLE_BAR
        | Section.EXPANDED);
    sectionHandschaltungBetriebsart.setText("Handschaltung und Betriebsart festlegen");
    Composite clientHandschaltungBetriebsart = toolkit.createComposite(sectionHandschaltungBetriebsart, SWT.WRAP);
    clientHandschaltungBetriebsart.setLayout(new GridLayout(2, true));
    sectionHandschaltungBetriebsart.setClient(clientHandschaltungBetriebsart);
    sectionHandschaltungBetriebsart.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createGroupFilterDefinitionenHandschaltung(clientHandschaltungBetriebsart);
    createGroupFilterDefinitionenBetriebsart(clientHandschaltungBetriebsart);

    /*
     * Section Ctrl
     */
    Section sectionCtrl = toolkit.createSection(form.getBody(), Section.TITLE_BAR | Section.EXPANDED);
    sectionCtrl.setText("Aktionen");
    Composite clientCtrl = toolkit.createComposite(sectionCtrl, SWT.WRAP);
    clientCtrl.setLayout(new GridLayout(5, true));
    sectionCtrl.setClient(clientCtrl);
    sectionCtrl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createCompositeCtrl(clientCtrl);

    /*
     * Section Zusatzprfung
     */
    Section sectionZusatzpruefung = toolkit.createSection(form.getBody(), Section.TITLE_BAR | Section.TREE_NODE
        | Section.COMPACT);
    sectionZusatzpruefung.setText("Zusatzprfung");
    Composite clientZusatzpruefung = toolkit.createComposite(sectionZusatzpruefung, SWT.WRAP);
    clientZusatzpruefung.setLayout(new GridLayout());
    sectionZusatzpruefung.setClient(clientZusatzpruefung);
    sectionZusatzpruefung.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createGroupFilterDefinitionenZusatzpruefung(clientZusatzpruefung);

    /*
     * Section Kalenderauswahl
     */
    Section sectionKalenderauswahl = toolkit.createSection(form.getBody(), Section.TITLE_BAR | Section.TREE_NODE
        | Section.COMPACT);
    sectionKalenderauswahl.setText("Kalenderauswahl");
    Composite clientKalenderauswahl = toolkit.createComposite(sectionKalenderauswahl, SWT.WRAP);
    sectionKalenderauswahl.setClient(clientKalenderauswahl);
    sectionKalenderauswahl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createTimeComposite(clientKalenderauswahl);

    /*
     * Section Zeitbereich
     */
    Section sectionZeitbereich = toolkit.createSection(form.getBody(), Section.TITLE_BAR | Section.TREE_NODE
        | Section.COMPACT);
    sectionZeitbereich.setText("Zeitbereich");
    Composite clientZeitbereich = toolkit.createComposite(sectionZeitbereich, SWT.WRAP);
    clientZeitbereich.setLayout(new GridLayout(2, true));
    sectionZeitbereich.setClient(clientZeitbereich);
    sectionZeitbereich.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

    // Zeitbereich Links
    Composite compZeitbereichLinks = new Composite(clientZeitbereich, SWT.NONE);
    compZeitbereichLinks.setLayout(new GridLayout(1, false));
    GridData gdCompZeitbereichLinks = new GridData();
    gdCompZeitbereichLinks.grabExcessHorizontalSpace = true;
    gdCompZeitbereichLinks.grabExcessVerticalSpace = true;
    gdCompZeitbereichLinks.horizontalAlignment = GridData.FILL;
    gdCompZeitbereichLinks.verticalAlignment = GridData.FILL;
    compZeitbereichLinks.setLayoutData(gdCompZeitbereichLinks);
    createCompositeZeitbereichVon(compZeitbereichLinks);
    createCompositeZeitbereichBis(compZeitbereichLinks);

    // Zeitbereich Rechts
    Composite compZeitbereichRechts = new Composite(clientZeitbereich, SWT.NONE);
    compZeitbereichRechts.setLayout(new GridLayout(1, false));
    GridData gdCompZeitbereichRechts = new GridData();
    gdCompZeitbereichRechts.grabExcessHorizontalSpace = true;
    gdCompZeitbereichRechts.horizontalAlignment = GridData.FILL;
    compZeitbereichRechts.setLayoutData(gdCompZeitbereichRechts);

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    gridLayout.marginWidth = 0;
    gridLayout.verticalSpacing = 5;
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.horizontalSpacing = 0;
    gridLayout.marginHeight = 5;

    Group grpStundenAuswahl = new Group(compZeitbereichRechts, SWT.NONE);
    grpStundenAuswahl.setText("Stundenauswahl");
    GridData gdGrpStundenAuswahl = new GridData();
    gdGrpStundenAuswahl.grabExcessHorizontalSpace = true;
    gdGrpStundenAuswahl.horizontalAlignment = GridData.FILL;
    grpStundenAuswahl.setLayout(gridLayout);
    grpStundenAuswahl.setLayoutData(gdGrpStundenAuswahl);
    createComboStundenauswahl(grpStundenAuswahl);
    createZeitwaehlerStunden(grpStundenAuswahl);

    Group grpWochentagsAuswahl = new Group(compZeitbereichRechts, SWT.NONE);
    grpWochentagsAuswahl.setText("Wochentagsauswahl");
    GridData gdGrpWochentagsAuswahl = new GridData();
    gdGrpWochentagsAuswahl.grabExcessHorizontalSpace = true;
    gdGrpWochentagsAuswahl.horizontalAlignment = GridData.FILL;
    grpWochentagsAuswahl.setLayout(gridLayout);
    grpWochentagsAuswahl.setLayoutData(gdGrpWochentagsAuswahl);
    createComboWochentagsauswahl(grpWochentagsAuswahl);
    createZeitwaehlerWochentage(grpWochentagsAuswahl);

    Group grpMonatsAuswahl = new Group(compZeitbereichRechts, SWT.NONE);
    grpMonatsAuswahl.setText("Monatsauswahl");
    GridData gdGrpMonatsauswahl = new GridData();
    gdGrpMonatsauswahl.grabExcessHorizontalSpace = true;
    gdGrpMonatsauswahl.horizontalAlignment = GridData.FILL;
    grpMonatsAuswahl.setLayout(gridLayout);
    grpMonatsAuswahl.setLayoutData(gdGrpMonatsauswahl);
    createComboMonatsauswahl(grpMonatsAuswahl);
    createZeitwaehlerMonate(grpMonatsAuswahl);

    /*
     * Section Picker
     */
    Section sectionPicker = toolkit.createSection(form.getBody(), Section.TITLE_BAR | Section.TREE_NODE
        | Section.COMPACT);
    sectionPicker.setText("Picker");
    final Composite clientPicker = toolkit.createComposite(sectionPicker, SWT.WRAP);
    clientPicker.setLayout(new GridLayout());
    sectionPicker.setClient(clientPicker);
    sectionPicker.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    _compositePicker = createCompositePickerParent(clientPicker);
    sectionPicker.addExpansionListener(new IExpansionListener() {
      
      @Override
      public void expansionStateChanging(ExpansionEvent e)
      {
        // TODO Auto-generated method stub
        
        Object data = e.data;
        if (data instanceof Boolean)
        {
          Boolean state = (Boolean)data;
          if (state){
            if (_compositePicker == null){
//              System.out.println("Thumbnail: " + _thumbnail);
              _compositePicker = createCompositePickerParent(clientPicker);
            }
            createCompositePicker(_compositePicker);
          }
        }        
      }
      
      @Override
      public void expansionStateChanged(ExpansionEvent e)
      {
        // TODO Auto-generated method stub
        Object data = e.data;
        if (data instanceof Boolean)
        {
          Boolean state = (Boolean)data;
          if (!state){           
            _thumbnail.deactivate();
            _thumbnail = null;
            _compositePicker.dispose();
            _compositePicker = null;
          }
        }
      }
    });
  }

  private Composite createCompositePickerParent(Composite clientPicker)
  {
//    System.out.println("ZustandsDarstellungSonderProgramm.createCompositePickerParent()");
    
    final Composite compositePicker = new Composite(clientPicker, SWT.NONE);
    compositePicker.setLayout(new GridLayout());
    GridData gdCompositePicker = new GridData();
    gdCompositePicker.widthHint = 200;
    gdCompositePicker.heightHint = 300;
    gdCompositePicker.minimumWidth = 200;
    gdCompositePicker.minimumHeight = 300;
    gdCompositePicker.grabExcessHorizontalSpace = true;
    gdCompositePicker.grabExcessVerticalSpace = true;
    gdCompositePicker.horizontalAlignment = GridData.FILL;
    gdCompositePicker.verticalAlignment = GridData.FILL;
    compositePicker.setLayoutData(gdCompositePicker);
    return compositePicker;
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

    IWorkbenchPage page = getSite().getPage();

    if (page != null)
    {
      IViewReference fr = page.findViewReference(AuswahlEbenenView.ID);

      if (fr != null)
      {
        IViewPart view = fr.getView(true);
        AuswahlEbenenView auswahl = (AuswahlEbenenView)view;
        auswahl.setContent((AnsichtEditPart)graphicalViewer.getContents());
      }

      fr = page.findViewReference(AuswahlSpaltenView.ID);

      if (fr != null)
      {
        IViewPart view = fr.getView(true);
        AuswahlSpaltenView auswahl = (AuswahlSpaltenView)view;
        auswahl.setContent((AnsichtEditPart)graphicalViewer.getContents());
      }

      fr = page.findViewReference(AuswahlSonderProgrammView.ID);

      if (fr != null)
      {
        IViewPart view = fr.getView(true);
        AuswahlSonderProgrammView auswahl = (AuswahlSonderProgrammView)view;
        auswahl.setContent(this, false);
      }
    }
  }

  private void createTimeComposite(Composite parent)
  {
    if (parent == null)
    {
      GridData gdParent = new GridData();
      gdParent.horizontalAlignment = GridData.FILL;
      gdParent.grabExcessHorizontalSpace = true;
      gdParent.grabExcessVerticalSpace = true;
      gdParent.verticalAlignment = GridData.FILL;
      parent = new Composite(_composite, SWT.NONE);
      parent.setLayout(new GridLayout());
      parent.setLayoutData(gdParent);
    }
    else
    {
      parent.setLayout(new GridLayout());
    }

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;

    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;
    gd.verticalAlignment = GridData.FILL;

    Composite timeComposite = new Composite(parent, SWT.NONE);
    timeComposite.setLayoutData(gd);
    timeComposite.setLayout(gridLayout);

    createEventList(timeComposite);
    createCalendarList(timeComposite);
  }

  private void createCompositeSperrungFahrstreifen(Composite auswahlComposite)
  {
    _compositeFahrstreifenSperrungen = new CompositeSperrungBereiche(auswahlComposite, SWT.NONE, null, 0, _secondaryId);
  }

  private void createEventList(Composite parent)
  {

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;

    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;
    gd.verticalAlignment = GridData.FILL;

    Group group = new Group(parent, SWT.NONE);
    group.setLayoutData(gd);
    group.setLayout(gridLayout);
    group.setText("Ereignisse");

    GridData gdTable = new GridData();
    gdTable.horizontalAlignment = GridData.FILL;
    gdTable.grabExcessHorizontalSpace = true;
    gdTable.grabExcessVerticalSpace = true;
    gdTable.verticalAlignment = GridData.FILL;

    _tvEreignisse = new TableViewer(group, SWT.HIDE_SELECTION);

    TableViewerColumn column = new TableViewerColumn(_tvEreignisse, SWT.NONE);
    column.getColumn().setWidth(135);

    _tvEreignisse.setContentProvider(new ArrayContentProvider());
    _tvEreignisse.setLabelProvider(new TableLabelProvider());
    _tvEreignisse.getTable().setLayoutData(gdTable);
    _tvEreignisse.getTable().setLinesVisible(true);
    _tvEreignisse.getTable().setHeaderVisible(false);

    GridData gdButton = new GridData();
    gdButton.verticalAlignment = GridData.BEGINNING;

    Button button = new Button(group, SWT.NONE);
    button.setLayoutData(gdButton);
    button.setText(" ... ");

    button.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        // ffne Listdialog
        ListDialog listDialog = new ListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell()) {
          protected int getTableStyle()
          {
            return SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER;
          }

          @Override
          protected void configureShell(Shell shell)
          {
            // Fr die F1 Taste
            PlatformUI.getWorkbench().getHelpSystem().setHelp(shell, Activator.PLUGIN_ID + ".auswahlEreignisse");

            super.configureShell(shell);
          }
        };

        listDialog.setTitle("Auswahl Ereignisse");
        listDialog.setMessage("Folgende Ereigniss stehen zur Auswahl:");
        listDialog.setContentProvider(new ArrayContentProvider());
        listDialog.setLabelProvider(new TableLabelProvider());

        List<SystemObject> items = new ArrayList<SystemObject>();
        ClientDavInterface connection = DaVKommunikation.getInstanz().getConnection();
        if (connection != null)
        {
          DataModel dataModel = connection.getDataModel();
          for (SystemObject so : dataModel.getType("typ.ereignis").getElements())
          {
            if (so.isValid())
            {
              items.add(so);
            }
          }
        }
        listDialog.setInput(items);
        if (listDialog.open() == ListDialog.OK)
        {
          List model = new ArrayList();
          Object[] results = listDialog.getResult();
          if (results != null)
          {
            model.addAll(Arrays.asList(results));
          }
          _tvEreignisse.setInput(model);
        }
      }
    });
  }

  private void createCalendarList(Composite parent)
  {

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;

    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;
    gd.verticalAlignment = GridData.FILL;

    Group group = new Group(parent, SWT.NONE);
    group.setLayoutData(gd);
    group.setLayout(gridLayout);
    group.setText("Systemkalendereintrge");

    GridData gdTable = new GridData();
    gdTable.horizontalAlignment = GridData.FILL;
    gdTable.grabExcessHorizontalSpace = true;
    gdTable.grabExcessVerticalSpace = true;
    gdTable.verticalAlignment = GridData.FILL;

    _tvSystemKalenderEintraege = new TableViewer(group, SWT.HIDE_SELECTION);

    TableViewerColumn column = new TableViewerColumn(_tvSystemKalenderEintraege, SWT.NONE);
    column.getColumn().setWidth(135);

    _tvSystemKalenderEintraege.setContentProvider(new ArrayContentProvider());
    _tvSystemKalenderEintraege.setLabelProvider(new TableLabelProvider());
    _tvSystemKalenderEintraege.getTable().setLayoutData(gdTable);
    _tvSystemKalenderEintraege.getTable().setLinesVisible(true);
    _tvSystemKalenderEintraege.getTable().setHeaderVisible(false);

    GridData gdButton = new GridData();
    gdButton.verticalAlignment = GridData.BEGINNING;

    Button button = new Button(group, SWT.NONE);
    button.setLayoutData(gdButton);
    button.setText(" ... ");

    button.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        // ffne Listdialog
        ListDialog listDialog = new ListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell()) {
          protected int getTableStyle()
          {
            return SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER;
          }

          @Override
          protected void configureShell(Shell shell)
          {
            // Fr die F1 Taste
            PlatformUI.getWorkbench().getHelpSystem().setHelp(shell, Activator.PLUGIN_ID + ".auswahlSystemkalender");

            super.configureShell(shell);
          }
        };
        listDialog.setTitle("Auswahl Systemkalendereintrge");
        listDialog.setMessage("Folgende Systemkalendereintrge stehen zur Auswahl:");
        listDialog.setContentProvider(new ArrayContentProvider());
        listDialog.setLabelProvider(new TableLabelProvider());

        List<SystemObject> items = new ArrayList<SystemObject>();
        ClientDavInterface connection = DaVKommunikation.getInstanz().getConnection();
        if (connection != null)
        {
          DataModel dataModel = connection.getDataModel();
          for (SystemObject so : dataModel.getType("typ.systemKalenderEintrag").getElements())
          {
            if (so.isValid())
            {
              items.add(so);
            }
          }
        }
        listDialog.setInput(items);
        if (listDialog.open() == ListDialog.OK)
        {
          List model = new ArrayList();
          Object[] results = listDialog.getResult();
          if (results != null)
          {
            model.addAll(Arrays.asList(results));
          }
          _tvSystemKalenderEintraege.setInput(model);
        }
      }
    });
  }

  public class TableLabelProvider implements ILabelProvider
  {

    @Override
    public Image getImage(Object element)
    {
      return null;
    }

    @Override
    public String getText(Object element)
    {
      return ((SystemObject)element).getName();
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }
  }

  /**
   * This method initializes composite01
   * 
   */
  private void createCompositeProgrammTypName(Composite parent)
  {
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.FILL;
    gridData21.grabExcessHorizontalSpace = false;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.verticalSpan = 3;
    gridData11.verticalAlignment = GridData.FILL;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    Composite compPrgName = new Composite(parent, SWT.NONE);
    compPrgName.setLayoutData(gridData);
    compPrgName.setLayout(gridLayout);

    labelTyp = new Label(compPrgName, SWT.NONE);
    labelTyp.setText("Programm-Typ");
    _comboProgrammTyp = new Combo(compPrgName, SWT.BORDER | SWT.READ_ONLY);
    _comboProgrammTyp.setLayoutData(gridData21);
    _comboProgrammTyp.addModifyListener(new ModifyListener() {

      @Override
      public void modifyText(ModifyEvent e)
      {

      }
    });
    _comboProgrammTyp.pack();

    _comboViewerProgrammTyp = new ComboViewer(_comboProgrammTyp);
    _comboViewerProgrammTyp.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection ss = (IStructuredSelection)event.getSelection();
        if (ss != null && !ss.isEmpty())
        {
          Object fe = ss.getFirstElement();
          if (fe != null)
          {
            if (fe instanceof String)
            {
              /*
               * Alter Mechanismus, ungenau.
               */

              String s = (String)fe;

              // Bestimmung Hand und Prioritt aus ProgrammTyp und im folgenden setzen

              if (s.equals("HandLeer"))
              {
                _radioButtonHandschaltungAktiv.setSelection(true);
                _radioButtonPrioritaet.setSelection(false);
                _comboPrioritaet.select(0);
              }
              else
              {
                _radioButtonHandschaltungAktiv.setSelection(false);
                _radioButtonPrioritaet.setSelection(false);
                _comboPrioritaet.select(0);
              }

              CompositeSperrungDarstellung comp = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung()[0];
              if (comp != null && !comp.isDisposed())
              {
                boolean b = DaVHelper.isProgrammTypSpurSperrung(s);
                comp.setVisible(b);
                if (comp.isVisible())
                {
                  comp.initialisiereControls();
                }
              }
            }
            else if (fe instanceof SswSbaProgrammTyp)
            {
              /*
               * Genauer Mechanismus.
               */
              SswSbaProgrammTyp programmTyp = (SswSbaProgrammTyp)fe;

              aktionSelektionComboProgrammTyp(programmTyp);
            }
          }
        }
      }
    });
    _comboViewerProgrammTyp.setLabelProvider(new ILabelProvider() {
      @Override
      public Image getImage(Object element)
      {
        return null;
      }

      @Override
      public String getText(Object element)
      {
        String returnString = "";

        if (element instanceof String)
        {
          returnString = (String)element;
        }
        else if (element instanceof SswSbaProgrammTyp)
        {
          SswSbaProgrammTyp programmTyp = (SswSbaProgrammTyp)element;
          if (programmTyp.getBezeichnung() != null)
          {
            returnString = programmTyp.getBezeichnung();
          }
        }

        return returnString;

      }

      @Override
      public void addListener(ILabelProviderListener listener)
      {
      }

      @Override
      public void dispose()
      {
      }

      @Override
      public boolean isLabelProperty(Object element, String property)
      {
        return false;
      }

      @Override
      public void removeListener(ILabelProviderListener listener)
      {
      }

    });
    _comboViewerProgrammTyp.setContentProvider(new ArrayContentProvider());

    _listProgrammTypen = bestimmeProgrammTypen(_connection);
    // Combo fuellen
    _comboViewerProgrammTyp.setInput(_listProgrammTypen);

    labelName = new Label(compPrgName, SWT.NONE);
    labelName.setText("Name");
    _textName = new Text(compPrgName, SWT.BORDER);
    _textName.setLayoutData(gridData1);
  }

  /**
   * Fhrt die Aktion hinter der Selektion auf der Combo-Box Programm-Typ durch.
   * 
   * @param programmTyp
   *          Konkreter {@link SswSbaProgrammTyp} oder <code>null</code>, falls dieser aus der aktuellen Selektion der
   *          ComboBox bestimmt werden soll.
   * @return Liefert <code>true</code> wenn erfolgreich, sonst <code>false</code>.
   */
  private boolean aktionSelektionComboProgrammTyp(SswSbaProgrammTyp programmTyp)
  {
    boolean erfolgreich = false;

    if (programmTyp == null)
    {
      if (_comboViewerProgrammTyp != null)
      {
        ISelection selection = _comboViewerProgrammTyp.getSelection();
        if ((selection != null) && (selection instanceof StructuredSelection))
        {
          StructuredSelection ss = (StructuredSelection)selection;
          Object firstElement = ss.getFirstElement();
          if ((firstElement != null) && (firstElement instanceof SswSbaProgrammTyp))
          {
            programmTyp = (SswSbaProgrammTyp)firstElement;
          }
        }
      }
    }

    if (programmTyp != null)
    {
      erfolgreich = true;

      // Radiobutton Handschaltung
      _radioButtonHandschaltungAktiv.setSelection(programmTyp.isHandprogramm());

      if (programmTyp.getPrioritaet() != null)
      {
        // Radiobutton Prioritt
        if (programmTyp.getPrioritaet() < 0)
        {
          _radioButtonPrioritaet.setSelection(false);
          _comboPrioritaet.select(0);
          _comboPrioritaet.setEnabled(false);
        }
        else
        {
          if (programmTyp.getPrioritaet() >= 0)
          {
            _radioButtonPrioritaet.setSelection(true);
            _comboPrioritaet.setText("" + programmTyp.getPrioritaet());
            _comboPrioritaet.setEnabled(true);
          }
        }
      }
      else
      {
        _radioButtonPrioritaet.setSelection(false);
        _comboPrioritaet.select(0);
        _comboPrioritaet.setEnabled(false);
      }

      CompositeSperrungDarstellung comp = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung()[0];
      if (comp != null && !comp.isDisposed())
      {
        comp.setVisible(programmTyp.isSpursperrung());
        if (comp.isVisible())
        {
          comp.initialisiereControls();
        }
      }
    }

    return erfolgreich;
  }

  public static List<SswSbaProgrammTyp> bestimmeProgrammTypen(ClientDavInterface connection)
  {
    ArrayList<SswSbaProgrammTyp> re = new ArrayList<SswSbaProgrammTyp>();
    if (connection != null)
    {
      AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaProgrammTyp");

      Aspect asp = connection.getDataModel().getAspect("asp.parameterSoll");

      IPreferenceStore store = Activator.getDefault().getPreferenceStore();
      String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
      if (pidSba != null && pidSba.length() > 0)
      {
        SystemObject object = connection.getDataModel().getObject(pidSba);
        if (object != null)
        {
          ResultData result = connection.getData(object, new DataDescription(atg, asp), 1500);

          if (result != null)
          {
            Data dta = result.getData();
            if (dta != null)
            {
              Array array = dta.getItem("ProgrammTypen").asArray();

              for (int i = 0; i < array.getLength(); i++)
              {
                SswSbaProgrammTyp sswSbaProgrammTyp = new SswSbaProgrammTyp(array.getItem(i));
                re.add(sswSbaProgrammTyp);
              }
            }
          }
        }
      }
    }
    return re;
  }

  /**
   * Umsetzung von <code>atg.sswSbaProgrammTyp</code>.
   * 
   * @author HofmannJ
   * 
   */
  public static class SswSbaProgrammTyp
  {
    /**
     * Bezeichnung
     */
    private String _bezeichnung = null;

    /**
     * Handprogramm
     */
    private boolean _handprogramm = false;

    /**
     * Prioritt.
     */
    private Integer _prioritaet = null;

    /**
     * Spursperrung.
     */
    private boolean _spursperrung = false;

    public SswSbaProgrammTyp(Data data)
    {
      if (data != null)
      {
        /*
         * Bezeichnung
         */
        setBezeichnung(data.getTextValue("Bezeichnung").getText());

        /*
         * Spursperrung
         */
        boolean spursperrung = false;
        int iValSpursperrung = data.getUnscaledValue("Spursperrung").intValue();
        if (iValSpursperrung == 1)
        {
          spursperrung = true;
        }
        setSpursperrung(spursperrung);

        /*
         * Handschaltung
         */
        Data itemHandschaltung = data.getItem("Handschaltung");

        boolean handprogramm = false;
        int ivalHandprogramm = itemHandschaltung.getUnscaledValue("Handprogramm").intValue();
        if (ivalHandprogramm == 1)
        {
          handprogramm = true;
        }
        setHandprogramm(handprogramm);

        /*
         * Prioritt
         */
        int prioritaet = itemHandschaltung.getUnscaledValue("Prioritt").intValue();
        setPrioritaet(prioritaet);

      }
    }

    public String getBezeichnung()
    {
      return _bezeichnung;
    }

    public void setBezeichnung(String bezeichnung)
    {
      _bezeichnung = bezeichnung;
    }

    public boolean isSpursperrung()
    {
      return _spursperrung;
    }

    public void setSpursperrung(boolean spursperrung)
    {
      _spursperrung = spursperrung;
    }

    public boolean isHandprogramm()
    {
      return _handprogramm;
    }

    public void setHandprogramm(boolean handprogramm)
    {
      _handprogramm = handprogramm;
    }

    public Integer getPrioritaet()
    {
      return _prioritaet;
    }

    public void setPrioritaet(Integer prioritaet)
    {
      _prioritaet = prioritaet;
    }

    @Override
    public String toString()
    {
      return getBezeichnung();
    }

    public String toStringLong()
    {
      return "SswSbaProgrammTyp [_bezeichnung=" + _bezeichnung + ", _handprogramm=" + _handprogramm + ", _prioritaet="
          + _prioritaet + "]";
    }

  }

  private void createCompositeStreckenbereich(Composite parent)
  {

    GridData gridData11 = new GridData();
    gridData11.verticalAlignment = GridData.CENTER;
    gridData11.horizontalAlignment = GridData.FILL;
    GridData gridData12 = new GridData();
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.verticalAlignment = GridData.CENTER;
    gridData12.horizontalAlignment = GridData.FILL;
    GridData gridData13 = new GridData();
    gridData13.grabExcessHorizontalSpace = true;
    gridData13.verticalAlignment = GridData.CENTER;
    gridData13.horizontalAlignment = GridData.FILL;

    GridData gridData15 = new GridData();
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.horizontalAlignment = GridData.FILL;
    GridData gridData16 = new GridData();
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.horizontalAlignment = GridData.FILL;

    GridData gridData18 = new GridData();
    gridData18.horizontalAlignment = GridData.FILL;
    gridData18.grabExcessHorizontalSpace = true;
    gridData18.grabExcessVerticalSpace = false;
    gridData18.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout13 = new GridLayout();
    gridLayout13.numColumns = 4;
    gridLayout13.makeColumnsEqualWidth = false;

    Composite groupStreckenbereich = new Composite(parent, SWT.NONE);

    groupStreckenbereich.setLayoutData(gridData18);
    groupStreckenbereich.setLayout(gridLayout13);
    label21 = new Label(groupStreckenbereich, SWT.NONE);
    label21.setLayoutData(gridData11);
    label21.setText("startKm");
    textStartKm = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
    textStartKm.setLayoutData(gridData15);
    textStartKm.setTextLimit(7);
    textStartKm.setText("000,000");
    textStartKm.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent event)
      {
        switch (event.keyCode)
        {
          case SWT.CR:
            DaVHelper.validiereTextInput(_secondaryId, textStartKm.getText(), textStopKm.getText());
            break;
          case SWT.ESC:
            // System.out.println(SWT.ESC);
            break;
        }
      }
    });

    Button btnBereichVon = new Button(groupStreckenbereich, SWT.NONE);
    btnBereichVon.setText("ber AQ");
    btnBereichVon.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionStartKm();
      }
    });

    labelStartKm = new Label(groupStreckenbereich, SWT.NONE);
    labelStartKm.setLayoutData(gridData12);
    labelStartKm.setText("keine Ausw.");

    label22 = new Label(groupStreckenbereich, SWT.NONE);
    label22.setLayoutData(gridData11);
    label22.setText("endeKm ");

    textStopKm = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
    textStopKm.setLayoutData(gridData16);
    textStopKm.setTextLimit(7);
    textStopKm.setText("000,000");
    textStopKm.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent event)
      {
        switch (event.keyCode)
        {
          case SWT.CR:
            DaVHelper.validiereTextInput(_secondaryId, textStartKm.getText(), textStopKm.getText());
            break;
          case SWT.ESC:
            // System.out.println(SWT.ESC);
            break;
        }
      }
    });

    Button btnBereichBis = new Button(groupStreckenbereich, SWT.NONE);
    btnBereichBis.setText("ber AQ");
    btnBereichBis.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionStopKm();
      }
    });

    labelStopKm = new Label(groupStreckenbereich, SWT.NONE);
    labelStopKm.setLayoutData(gridData12);
    labelStopKm.setText("keine Ausw.");
  }

  /**
   * This method initializes groupFilterDefinitionen71
   * 
   */
  private void createCompositeZeitbereichVon(Composite parent)
  {
    int height = 17;

    GridData gridData33 = new GridData();
    gridData33.horizontalAlignment = GridData.CENTER;
    gridData33.verticalAlignment = GridData.CENTER;
    GridData gridData32 = new GridData();
    gridData32.horizontalAlignment = GridData.CENTER;
    gridData32.verticalAlignment = GridData.END;
    GridData gridData31 = new GridData();
    gridData31.horizontalAlignment = GridData.BEGINNING;
    gridData31.grabExcessVerticalSpace = false;
    gridData31.verticalAlignment = GridData.CENTER;
    GridData gridData30 = new GridData();
    gridData30.horizontalAlignment = GridData.BEGINNING;
    gridData30.verticalAlignment = GridData.FILL;
    gridData30.heightHint = height;
    gridData30.horizontalSpan = 2;
    GridData gridData29 = new GridData();
    gridData29.horizontalAlignment = GridData.BEGINNING;
    gridData29.verticalAlignment = GridData.FILL;
    gridData29.heightHint = height;
    gridData29.horizontalSpan = 2;
    GridData gridData28 = new GridData();
    gridData28.horizontalAlignment = GridData.BEGINNING;
    gridData28.verticalAlignment = GridData.FILL;
    GridLayout gridLayout4 = new GridLayout(2, false);
    gridLayout4.verticalSpacing = 5;
    gridLayout4.marginHeight = 0;
    GridData gridDataGrpZeitbereichVon = new GridData();
    gridDataGrpZeitbereichVon.horizontalAlignment = GridData.FILL;
    gridDataGrpZeitbereichVon.grabExcessHorizontalSpace = true;
    gridDataGrpZeitbereichVon.grabExcessVerticalSpace = false;
    gridDataGrpZeitbereichVon.verticalAlignment = GridData.BEGINNING;
    GridData gridData211 = new GridData();
    gridData211.horizontalAlignment = GridData.BEGINNING;
    gridData211.verticalAlignment = GridData.CENTER;

    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 2;

    groupFilterDefinitionen11 = new Group(parent, SWT.NONE);
    groupFilterDefinitionen11.setText("Zeitbereich Von");
    groupFilterDefinitionen11.setLayoutData(gridDataGrpZeitbereichVon);
    groupFilterDefinitionen11.setLayout(gridLayout1);

    _radioButtonAbSofort = new Button(groupFilterDefinitionen11, SWT.RADIO);
    _radioButtonAbSofort.setText("ab sofort");
    _radioButtonAbSofort.setLayoutData(gridData30);

    _radioButtonVorlaeufigNicht = new Button(groupFilterDefinitionen11, SWT.RADIO);
    _radioButtonVorlaeufigNicht.setText("vorlufig nicht");
    _radioButtonVorlaeufigNicht.setLayoutData(gridData29);

    _radioButtonVon = new Button(groupFilterDefinitionen11, SWT.RADIO);
    _radioButtonVon.setText("von");
    GridData gridData10 = new GridData();
    gridData10.horizontalAlignment = GridData.FILL;
    gridData10.grabExcessHorizontalSpace = false;
    gridData10.verticalAlignment = GridData.CENTER;
    gridData10.heightHint = height;

    zeitVon = new DatumZeit(groupFilterDefinitionen11, SWT.BORDER, Eingabetyp.datumuhrEinfach, false, false);
    zeitVon.setDatum(new Date());
    zeitVon.setLayoutData(gridData10);
    zeitVon.pack();

    _radioButtonAbSofort.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        zeitVon.setEnabled(false);
      }
    });
    _radioButtonVorlaeufigNicht.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        zeitVon.setEnabled(false);
      }
    });
    _radioButtonVon.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        zeitVon.setEnabled(true);
      }
    });
    _radioButtonAbSofort.setSelection(true);
    zeitVon.setEnabled(false);
  }

  /**
   * This method initializes groupFilterDefinitionen72
   * 
   */
  private void createCompositeZeitbereichBis(Composite parent)
  {
    int height = 17;

    GridData gridData33 = new GridData();
    gridData33.horizontalAlignment = GridData.CENTER;
    gridData33.verticalAlignment = GridData.CENTER;
    GridData gridData32 = new GridData();
    gridData32.horizontalAlignment = GridData.CENTER;
    gridData32.verticalAlignment = GridData.END;
    GridData gridData31 = new GridData();
    gridData31.horizontalAlignment = GridData.BEGINNING;
    gridData31.grabExcessVerticalSpace = false;
    gridData31.verticalAlignment = GridData.CENTER;
    GridData gridData30 = new GridData();
    gridData30.horizontalAlignment = GridData.BEGINNING;
    gridData30.verticalAlignment = GridData.FILL;
    gridData30.heightHint = height;
    gridData30.horizontalSpan = 2;
    GridData gridData29 = new GridData();
    gridData29.horizontalAlignment = GridData.BEGINNING;
    gridData29.verticalAlignment = GridData.FILL;
    gridData29.heightHint = height;
    gridData29.horizontalSpan = 2;
    GridData gridData28 = new GridData();
    gridData28.horizontalAlignment = GridData.BEGINNING;
    gridData28.verticalAlignment = GridData.FILL;
    GridLayout gridLayout4 = new GridLayout(2, false);
    gridLayout4.verticalSpacing = 5;
    gridLayout4.marginHeight = 0;
    GridData gridDataGroupZeitbereichBis = new GridData();
    gridDataGroupZeitbereichBis.grabExcessHorizontalSpace = true;
    gridDataGroupZeitbereichBis.horizontalAlignment = GridData.FILL;
    gridDataGroupZeitbereichBis.grabExcessVerticalSpace = false;
    gridDataGroupZeitbereichBis.verticalAlignment = GridData.BEGINNING;
    GridData gridData211 = new GridData();
    gridData211.horizontalAlignment = GridData.BEGINNING;
    gridData211.verticalAlignment = GridData.CENTER;

    GridLayout gridLayoutGrpZeitbereichBis = new GridLayout();
    gridLayoutGrpZeitbereichBis.numColumns = 2;

    groupFilterDefinitionen12 = new Group(parent, SWT.NONE);
    groupFilterDefinitionen12.setText("Zeitbereich Bis");
    groupFilterDefinitionen12.setLayoutData(gridDataGroupZeitbereichBis);
    groupFilterDefinitionen12.setLayout(gridLayoutGrpZeitbereichBis);

    _radioButtonBisJetzt = new Button(groupFilterDefinitionen12, SWT.RADIO);
    _radioButtonBisJetzt.setText("bis jetzt");
    _radioButtonBisJetzt.setLayoutData(gridData30);

    _radioButtonBisAufWeiteres = new Button(groupFilterDefinitionen12, SWT.RADIO);
    _radioButtonBisAufWeiteres.setText("bis auf weiteres");
    _radioButtonBisAufWeiteres.setLayoutData(gridData29);

    _radioButtonBis = new Button(groupFilterDefinitionen12, SWT.RADIO);
    _radioButtonBis.setText("bis");
    GridData gridData10 = new GridData();
    gridData10.horizontalAlignment = GridData.FILL;
    gridData10.grabExcessHorizontalSpace = false;
    gridData10.verticalAlignment = GridData.CENTER;
    gridData10.heightHint = height;

    zeitBis = new DatumZeit(groupFilterDefinitionen12, SWT.BORDER, Eingabetyp.datumuhrEinfach, false, false);
    zeitBis.setDatum(new Date());
    zeitBis.setLayoutData(gridData10);
    zeitBis.pack();

    _radioButtonBisJetzt.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        zeitBis.setEnabled(false);
      }
    });
    _radioButtonBisAufWeiteres.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        zeitBis.setEnabled(false);
      }
    });
    _radioButtonBis.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        zeitBis.setEnabled(true);
      }
    });
    _radioButtonBisJetzt.setSelection(true);
    zeitBis.setEnabled(false);
  }

  /**
   * This method initializes groupFilterDefinitionen178
   * 
   */
  private void createGroupFilterDefinitionenHandschaltung(Composite parent)
  {
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.BEGINNING;
    gridData21.grabExcessHorizontalSpace = true;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData18 = new GridData();
    gridData18.horizontalAlignment = GridData.FILL;
    gridData18.grabExcessHorizontalSpace = true;
    gridData18.verticalAlignment = GridData.FILL;
    GridData gridData19 = new GridData();
    gridData19.horizontalAlignment = GridData.FILL;
    gridData19.grabExcessHorizontalSpace = true;
    gridData19.horizontalSpan = 2;
    GridLayout gridLayout13 = new GridLayout();
    gridLayout13.numColumns = 2;
    _groupFilterDefinitionenHandschaltung = new Group(parent, SWT.NONE);
    _groupFilterDefinitionenHandschaltung.setText("Handschaltung");
    _groupFilterDefinitionenHandschaltung.setLayoutData(gridData18);
    _groupFilterDefinitionenHandschaltung.setLayout(gridLayout13);
    _radioButtonHandschaltungAktiv = new Button(_groupFilterDefinitionenHandschaltung, SWT.CHECK);
    _radioButtonHandschaltungAktiv.setLayoutData(gridData19);
    _radioButtonHandschaltungAktiv.setText("Aktiv");

    _radioButtonPrioritaet = new Button(_groupFilterDefinitionenHandschaltung, SWT.CHECK);
    _radioButtonPrioritaet.setText("Prioritt");
    _radioButtonPrioritaet.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        Button btn = (Button)e.getSource();
        boolean sel = btn.getSelection();
        _comboPrioritaet.setEnabled(sel);
      }
    });
    _radioButtonPrioritaet.setSelection(false);

    _comboPrioritaet = new Combo(_groupFilterDefinitionenHandschaltung, SWT.BORDER | SWT.NONE);
    _comboPrioritaet.setLayoutData(gridData21);
    _comboPrioritaet.setEnabled(false);
    _cvComboBoxPrioritaet = new ComboViewer(_comboPrioritaet);
    _cvComboBoxPrioritaet.add("undefiniert");
    _cvComboBoxPrioritaet.add("0");
    _cvComboBoxPrioritaet.add("60000");

    _cvComboBoxPrioritaet.setSelection(new StructuredSelection("undefiniert"));
  }

  /**
   * This method initializes groupFilterDefinitionen180
   * 
   */
  private void createGroupFilterDefinitionenZusatzpruefung(Composite parent)
  {
    GridData gridDataGroupZusatzpruefung = new GridData();
    gridDataGroupZusatzpruefung.horizontalAlignment = GridData.FILL;
    gridDataGroupZusatzpruefung.grabExcessHorizontalSpace = true;
    gridDataGroupZusatzpruefung.verticalAlignment = GridData.FILL;
    GridLayout gridLayoutGroupZusatzpruefung = new GridLayout(2, true);

    Composite _groupFilterDefinitionenZusatzpruefung = new Composite(parent, SWT.NONE);
    _groupFilterDefinitionenZusatzpruefung.setLayoutData(gridDataGroupZusatzpruefung);
    _groupFilterDefinitionenZusatzpruefung.setLayout(gridLayoutGroupZusatzpruefung);

    _radioButtonQuerLaengsabgleich = new Button(_groupFilterDefinitionenZusatzpruefung, SWT.CHECK);
    _radioButtonQuerLaengsabgleich.setText("QuerLngsabgleich");

    _radioButtonAusfallbehandlung = new Button(_groupFilterDefinitionenZusatzpruefung, SWT.CHECK);
    _radioButtonAusfallbehandlung.setText("Ausfallbehandlung");

    _radioButtonVerschiebung = new Button(_groupFilterDefinitionenZusatzpruefung, SWT.CHECK);
    _radioButtonVerschiebung.setText("Verschiebung");

    _radioButtonAufhebung = new Button(_groupFilterDefinitionenZusatzpruefung, SWT.CHECK);
    _radioButtonAufhebung.setText("Aufhebung");

    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    boolean b = store.getBoolean(DefsPreference.P_USE_ZUSATZ_PRUEFUNG_SONDER_PRG);
    _groupFilterDefinitionenZusatzpruefung.setEnabled(b);
  }

  /**
   * This method initializes groupFilterDefinitionen191
   * 
   */
  private void createGroupFilterDefinitionenBetriebsart(Composite parent)
  {
    GridData gridDataGrpFilterDefBetriebsart = new GridData();
    gridDataGrpFilterDefBetriebsart.horizontalAlignment = GridData.FILL;
    gridDataGrpFilterDefBetriebsart.grabExcessHorizontalSpace = true;
    gridDataGrpFilterDefBetriebsart.verticalAlignment = GridData.FILL;
    GridLayout gridLayoutGroupFilterDefBetriebsart = new GridLayout();
    _groupFilterDefinitionenBetriebsart = new Group(parent, SWT.NONE);
    _groupFilterDefinitionenBetriebsart.setText("Betriebsart");
    _groupFilterDefinitionenBetriebsart.setLayoutData(gridDataGrpFilterDefBetriebsart);
    _groupFilterDefinitionenBetriebsart.setLayout(gridLayoutGroupFilterDefBetriebsart);

    _radioButtonSimulation = new Button(_groupFilterDefinitionenBetriebsart, SWT.CHECK);
    _radioButtonSimulation.setText("Simulation");

    _radioButtonSchaltung = new Button(_groupFilterDefinitionenBetriebsart, SWT.CHECK);
    _radioButtonSchaltung.setText("Schaltung");
  }

  /**
   * This method initializes composite92
   */
  private void createCompositePicker(Composite parent)
  {
//    System.out.println("------------> ZustandsDarstellungSonderProgramm.createCompositePicker()");
    
    GridData gridDateCompositePicker = new GridData();
    gridDateCompositePicker.horizontalAlignment = GridData.FILL;
    gridDateCompositePicker.grabExcessHorizontalSpace = true;
    gridDateCompositePicker.grabExcessVerticalSpace = true;
    gridDateCompositePicker.verticalAlignment = GridData.FILL;

    FigureCanvas overview = new FigureCanvas(parent, SWT.BORDER);
    overview.setLayoutData(gridDateCompositePicker);

    LightweightSystem lws = new LightweightSystem(overview);
    if (graphicalViewer != null)
    {
      RootEditPart rep = graphicalViewer.getRootEditPart();
      if (rep instanceof ScalableRootEditPart)
      {
        ScalableRootEditPart root = (ScalableRootEditPart)rep;

        _thumbnail = new ScrollableThumbnail((Viewport)root.getFigure());
        _thumbnail.setBorder(new MarginBorder(3));
        _thumbnail.setSource(root.getLayer(LayerConstants.PRINTABLE_LAYERS));
        lws.setContents(this._thumbnail);
      }
    }
  }

  /**
   * This method initializes composite10
   * 
   */
  private void createCompositeCtrl(Composite parent)
  {
    GridData gdButtonVerlassen = new GridData();
    gdButtonVerlassen.horizontalAlignment = GridData.FILL;
    gdButtonVerlassen.verticalAlignment = GridData.CENTER;
    GridData gdButtonLoeschen = new GridData();
    gdButtonLoeschen.horizontalAlignment = GridData.FILL;
    gdButtonLoeschen.verticalAlignment = GridData.CENTER;
    GridData gdButtonKopieren = new GridData();
    gdButtonKopieren.horizontalAlignment = GridData.FILL;
    gdButtonKopieren.verticalAlignment = GridData.CENTER;
    GridData gdButtonReset = new GridData();
    gdButtonReset.horizontalAlignment = GridData.FILL;
    gdButtonReset.verticalAlignment = GridData.CENTER;
    GridData gdButtonOk = new GridData();
    gdButtonOk.horizontalAlignment = GridData.FILL;
    gdButtonOk.verticalAlignment = GridData.CENTER;

    _buttonVerlassen = new Button(parent, SWT.NONE);
    _buttonVerlassen.setText("Neu");
    _buttonVerlassen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        // dispose();
        aktionNeu();
      }
    });
    _buttonVerlassen.setLayoutData(gdButtonVerlassen);

    _buttonOk = new Button(parent, SWT.NONE);
    _buttonOk.setText("OK");
    _buttonOk.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        SonderProgramm sonderProgramm = ueberpruefeErfassteDaten();
        if (sonderProgramm != null)
        {
          versendeDaten(_connection, sonderProgramm, false, false);
        }
      }
    });
    _buttonOk.setLayoutData(gdButtonOk);

    _buttonReset = new Button(parent, SWT.NONE);
    _buttonReset.setText("Reset");
    _buttonReset.setLayoutData(gdButtonReset);
    _buttonReset.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        setzteAlleAnzeigenMitKontext((AnsichtEditPart)graphicalViewer.getContents(), AnzeigeInhaltTyp.BELIEBIG_BI, 4);
      }
    });

    _buttonKopieren = new Button(parent, SWT.NONE);
    _buttonKopieren.setText("Kopieren");
    _buttonKopieren.setLayoutData(gdButtonKopieren);
    _buttonKopieren.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        aktionSonderProgrammKopieren();       
      }
    });
 
    _buttonLoeschen = new Button(parent, SWT.NONE);
    _buttonLoeschen.setText("Lschen");
    _buttonLoeschen.setLayoutData(gdButtonLoeschen);
    _buttonLoeschen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        aktionLoeschen();
      }
    });
  }
  
 
  /**
   *  Hilfsmethode, erzeugt die Pid eines Sonderprogramms nach der Methode des Sonderprogramm-Generators 
   * 
   **/
  public static String erzeugePidUrsachenEinheitSonderProgramm(String bezeichnung, String praefix,  String postfix, String fahrRichtung){ 
    
    if (bezeichnung == null)
     throw new IllegalArgumentException("Bezeichnung ist null!");
    
    String re = "";
    
    re = praefix + "." + fahrRichtung.toLowerCase() + "." + bezeichnung.replace(" ", "")  + "." + postfix;   
    
    return re;    
  }

  private void aktionNeu()
  {
    /*
     * Name zurcksetzen
     */
    _textName.setText("");

    /*
     * Zusatzprfungen zurcksetzen
     */
    _radioButtonQuerLaengsabgleich.setSelection(false);
    _radioButtonAusfallbehandlung.setSelection(false);
    _radioButtonVerschiebung.setSelection(false);
    _radioButtonAufhebung.setSelection(false);

    /*
     * Kalenderauswahl leeren
     */
    _tvEreignisse.setInput(Arrays.asList(new Object[0]));
    _tvSystemKalenderEintraege.setInput(Arrays.asList(new Object[0]));

    /*
     * Zeitbereich von zurcksetzen
     */
    _radioButtonVorlaeufigNicht.setSelection(false);
    _radioButtonVon.setSelection(false);
    _radioButtonAbSofort.setSelection(true);

    /*
     * Zeitbereich bis zurcksetzen
     */
    _radioButtonBisJetzt.setSelection(false);
    _radioButtonBis.setSelection(false);
    _radioButtonBisAufWeiteres.setSelection(true);

    /*
     * Combo Stundenauswahl zurksezen
     */
    setzeZurueckComboStundenauswahl();
    /*
     * Combo Wochentagsauswahl zurcksetzen
     */
    setzeZurueckComboWochentagsauswahl();

    /*
     * Combo Monatsauswahl zurcksetzen
     */
    setzeZurueckComboMonatsauswahl();

    /*
     * TabFolder mit den Sperrungen zuruecksetzen
     */
    if (!aktionSelektionComboProgrammTyp(null))
    {
      /*
       * Handschaltung und Betriebsart leeren gem nicht vorhandener Selektion ComboBox ProgrammTyp.
       */
      _radioButtonHandschaltungAktiv.setSelection(false);
      _radioButtonPrioritaet.setSelection(false);
      _comboPrioritaet.select(0);
      _comboPrioritaet.setEnabled(false);
    }
    _radioButtonSimulation.setSelection(false);
    _radioButtonSchaltung.setSelection(false);

  }

  /**
   * ComboBox fr die Stundenauswahl
   * 
   */
  private void createComboStundenauswahl(Composite parent)
  {
    GridData gridData26 = new GridData();
    gridData26.horizontalAlignment = GridData.CENTER;
    gridData26.verticalAlignment = GridData.CENTER;
    _comboStundenauswahl = new Combo(parent, SWT.BORDER | SWT.READ_ONLY);
    _comboStundenauswahl.setLayoutData(gridData26);
    String[] ss = new String[] { "ganzer Tag", "07:00 - 09:00", "15:00 - 19:00", "06:00 - 22:00", "22:00 - 06:00",
        "leere Auswahl", "eigene Auswahl" };
    _comboStundenauswahl.setItems(ss);
    _comboStundenauswahl.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        if ((e.getSource() != null) && (e.getSource() instanceof Combo))
        {
          Combo comboStundenauswahl = (Combo)e.getSource();
          aktionComboStundenauswahlSelektion(comboStundenauswahl);
        }

      }
    });
    _comboStundenauswahl.select(0);
  }

  /**
   * Fhrt die Aktion aus, die hinter der Stundenselektion ComboBox steht.
   * 
   * @param comboStundenAuswahl
   */
  private void aktionComboStundenauswahlSelektion(Combo comboStundenAuswahl)
  {
    if (comboStundenAuswahl != null)
    {
      Control[] children = _compositeZeitwaehlerStunden.getChildren();

      Label[] labels = new Label[children.length];

      for (int i = 0; i < children.length; i++)
      {
        Control c = children[i];
        if (c instanceof Label)
        {
          Label l = (Label)c;
          labels[i] = l;
        }
      }

      for (int i = 0; i < labels.length; i++)
      {
        Label l = labels[i];
        l.setBackground(ColorConstants.white);
        if (_mouseAdapterStundenAuswahl != null)
          l.removeMouseListener(_mouseAdapterStundenAuswahl);
      }
      switch (comboStundenAuswahl.getSelectionIndex())
      {
        case 0:
          for (int i = 0; i < labels.length; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 1:
          for (int i = 6; i < 9; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 2:
          for (int i = 14; i < 19; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 3:
          for (int i = 5; i < 21; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 4:
          for (int i = 0; i < 6; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          for (int i = 22; i < 24; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 5:
          break;
        case 6:
          _mouseAdapterStundenAuswahl = new MouseAdapter() {
            @Override
            public void mouseDown(MouseEvent e)
            {
              Label l = (Label)e.getSource();
              Integer x = (Integer)l.getData();
              if (l.getBackground().equals(ColorConstants.blue))
                l.setBackground(ColorConstants.white);
              else if (l.getBackground().equals(ColorConstants.white))
                l.setBackground(ColorConstants.blue);
            }
          };

          for (int i = 0; i < labels.length; i++)
          {
            Label l = labels[i];

            l.addMouseListener(_mouseAdapterStundenAuswahl);
          }
          break;
        default:
      }
    }
  }

  private void setzeZurueckComboStundenauswahl()
  {
    _comboStundenauswahl.select(0);
    aktionComboStundenauswahlSelektion(_comboStundenauswahl);

  }

  /**
   * This method initializes combo22
   * 
   */
  private void createComboWochentagsauswahl(Composite parent)
  {
    GridData gridData26 = new GridData();
    gridData26.horizontalAlignment = GridData.CENTER;
    gridData26.verticalAlignment = GridData.CENTER;
    _comboWochentagsauswahl = new Combo(parent, SWT.BORDER | SWT.READ_ONLY);
    _comboWochentagsauswahl.setLayoutData(gridData26);
    String[] ss = new String[] { "ganze Woche", "Mo - Fr", "Di - Do", "Mo - Sa", "leere Auswahl", "eigene Auswahl" };
    _comboWochentagsauswahl.setItems(ss);
    _comboWochentagsauswahl.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        Combo comboWochentagsauswahl = (Combo)e.getSource();
        aktionComboWochentagsauswahlSelektion(comboWochentagsauswahl);
      }
    });

    _comboWochentagsauswahl.select(0);
  }

  private void aktionComboWochentagsauswahlSelektion(Combo comboWontagsauswahl)
  {
    if (comboWontagsauswahl != null)
    {
      Control[] children = _compositeZeitwaehlerWochentage.getChildren();

      Label[] labels = new Label[children.length];

      for (int i = 0; i < children.length; i++)
      {
        Control c = children[i];
        if (c instanceof Label)
        {
          Label l = (Label)c;
          labels[i] = l;
        }
      }

      for (int i = 0; i < labels.length; i++)
      {
        Label l = labels[i];
        l.setBackground(ColorConstants.white);
        if (_mouseAdapterWochentagsAuswahl != null)
          l.removeMouseListener(_mouseAdapterWochentagsAuswahl);
      }
      switch (comboWontagsauswahl.getSelectionIndex())
      {
        case 0:
          for (int i = 0; i < labels.length; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 1:
          for (int i = 0; i < 5; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 2:
          for (int i = 1; i < 4; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 3:
          for (int i = 0; i < 6; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 4:
          break;
        case 5:
          _mouseAdapterWochentagsAuswahl = new MouseAdapter() {
            @Override
            public void mouseDown(MouseEvent e)
            {
              Label l = (Label)e.getSource();
              Integer x = (Integer)l.getData();
              if (l.getBackground().equals(ColorConstants.blue))
                l.setBackground(ColorConstants.white);
              else if (l.getBackground().equals(ColorConstants.white))
                l.setBackground(ColorConstants.blue);
            }
          };

          for (int i = 0; i < labels.length; i++)
          {
            Label l = labels[i];

            l.addMouseListener(_mouseAdapterWochentagsAuswahl);
          }
          break;

        default:
      }
    }

  }

  private void setzeZurueckComboWochentagsauswahl()
  {
    _comboWochentagsauswahl.select(0);
    aktionComboWochentagsauswahlSelektion(_comboWochentagsauswahl);
  }

  /**
   * This method initializes combo23
   * 
   */
  private void createComboMonatsauswahl(Composite parent)
  {
    GridData gridData26 = new GridData();
    gridData26.horizontalAlignment = GridData.CENTER;
    gridData26.verticalAlignment = GridData.CENTER;
    _comboMonatsauswahl = new Combo(parent, SWT.BORDER | SWT.READ_ONLY);
    _comboMonatsauswahl.setLayoutData(gridData26);
    String[] ss = new String[] { "ganzes Jahr", "Januar - Juni", "April - September", "Oktober - Mrz",
        "leere Auswahl", "eigene Auswahl" };
    _comboMonatsauswahl.setItems(ss);
    _comboMonatsauswahl.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        Combo comboMonatsAuswahl = (Combo)e.getSource();
        aktionComboMonatsauswahl(comboMonatsAuswahl);
      }
    });

    _comboMonatsauswahl.select(0);
  }

  private void aktionComboMonatsauswahl(Combo comboMonatsAuswahl)
  {
    if (comboMonatsAuswahl != null)
    {
      Control[] children = _compositeZeitwaehlerMonate.getChildren();

      Label[] labels = new Label[children.length];

      for (int i = 0; i < children.length; i++)
      {
        Control c = children[i];
        if (c instanceof Label)
        {
          Label l = (Label)c;
          labels[i] = l;
        }
      }

      for (int i = 0; i < labels.length; i++)
      {
        Label l = labels[i];
        l.setBackground(ColorConstants.white);
        if (_mouseAdapterMonatsAuswahl != null)
          l.removeMouseListener(_mouseAdapterMonatsAuswahl);
      }
      switch (comboMonatsAuswahl.getSelectionIndex())
      {
        case 0:
          for (int i = 0; i < labels.length; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 1:
          for (int i = 0; i < 6; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 2:
          for (int i = 3; i < 9; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 3:
          for (int i = 9; i < 12; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          for (int i = 0; i < 3; i++)
          {
            Label l = labels[i];
            l.setBackground(ColorConstants.blue);
          }
          break;
        case 4:
          break;
        case 5:
          _mouseAdapterMonatsAuswahl = new MouseAdapter() {
            @Override
            public void mouseDown(MouseEvent e)
            {
              Label l = (Label)e.getSource();
              Integer x = (Integer)l.getData();
              if (l.getBackground().equals(ColorConstants.blue))
                l.setBackground(ColorConstants.white);
              else if (l.getBackground().equals(ColorConstants.white))
                l.setBackground(ColorConstants.blue);
            }
          };

          for (int i = 0; i < labels.length; i++)
          {
            Label l = labels[i];

            l.addMouseListener(_mouseAdapterMonatsAuswahl);
          }
          break;
        default:
      }
    }
  }

  private void setzeZurueckComboMonatsauswahl()
  {
    _comboMonatsauswahl.select(0);
    aktionComboMonatsauswahl(_comboMonatsauswahl);
  }

  /**
   * This method initializes combo21
   * 
   */
  private void createZeitwaehlerStunden(Composite parent)
  {
    RowLayout rl = new RowLayout(SWT.HORIZONTAL);
    rl.spacing = 0;
    rl.marginLeft = 0;
    rl.marginRight = 0;
    rl.justify = false;

    GridData gridData27 = new GridData();
    gridData27.horizontalAlignment = GridData.FILL;
    gridData27.grabExcessHorizontalSpace = true;
    gridData27.horizontalIndent = 0;
    gridData27.verticalAlignment = GridData.CENTER;
    _compositeZeitwaehlerStunden = new Composite(parent, SWT.BORDER);
    _compositeZeitwaehlerStunden.setLayoutData(gridData27);
    _compositeZeitwaehlerStunden.setLayout(rl);

    for (int i = 0; i < 24; i++)
    {
      Label l = new Label(_compositeZeitwaehlerStunden, SWT.BORDER);
      l.setLayoutData(new RowData(7, 13));
      l.setData(i);
      l.setBackground(ColorConstants.blue);
      l.setToolTipText("Stunde " + i);
    }
    createZeitwhlerStundenText(parent);
  }

  private void createZeitwhlerStundenText(Composite parent)
  {
    Composite compositeStundenText = new Composite(parent, SWT.NONE);
    compositeStundenText.setLayout(new FormLayout());

    int offset = 0;

    for (int j = 0; j < 12; j++)
    {
      Label l = new Label(compositeStundenText, SWT.NONE);
      int index = (j * 2);
      l.setText("" + index);

      switch (j)
      {
        case 0:
          offset += 4;
          break;
        case 5:
          offset += 15;
          break;
        default:
          offset += 18;
      }

      FormData formData = new FormData();
      formData.left = new FormAttachment(compositeStundenText, offset, SWT.LEFT);
      formData.top = new FormAttachment(compositeStundenText, 0, SWT.BOTTOM);
      l.setLayoutData(formData);
    }
  }

  /**
   * This method initializes combo21
   * 
   */
  private void createZeitwaehlerWochentage(Composite parent)
  {
    String[] ss = new String[] { "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag", "Sonntag" };

    FillLayout fillLayout = new FillLayout(SWT.HORIZONTAL);
    fillLayout.spacing = 0;
    fillLayout.marginWidth = 0;
    fillLayout.marginHeight = 2;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.horizontalIndent = 0;
    gridData.verticalAlignment = GridData.CENTER;
    _compositeZeitwaehlerWochentage = new Composite(parent, SWT.BORDER);
    _compositeZeitwaehlerWochentage.setLayoutData(gridData);
    _compositeZeitwaehlerWochentage.setLayout(fillLayout);

    for (int i = 0; i < 7; i++)
    {
      Label l = new Label(_compositeZeitwaehlerWochentage, SWT.BORDER);
      l.setData(i);
      l.setBackground(ColorConstants.blue);
      l.setToolTipText(ss[i]);
    }
    createZeitwhlerWochentageText(parent);
  }

  private void createZeitwhlerWochentageText(Composite parent)
  {
    String[] ss = new String[] { "Mo", "Di", "Mi", "Do", "Fr", "Sa", "So" };

    Composite compositeWochentageText = new Composite(parent, SWT.NONE);
    compositeWochentageText.setLayout(new FormLayout());

    int offset = 0;

    for (int j = 0; j < 7; j++)
    {
      Label l = new Label(compositeWochentageText, SWT.NONE);
      l.setText(ss[j]);

      switch (j)
      {
        case 0:
          offset += 10;
          break;
        case 1:
          offset += 35;
          break;
        default:
          offset += 30;
      }

      FormData formData = new FormData();
      formData.left = new FormAttachment(compositeWochentageText, offset, SWT.LEFT);
      formData.top = new FormAttachment(compositeWochentageText, 0, SWT.BOTTOM);
      l.setLayoutData(formData);
    }
  }

  /**
   * This method initializes combo21
   * 
   */
  private void createZeitwaehlerMonate(Composite parent)
  {
    String[] ss = new String[] { "Januar", "Februar", "Mrz", "April", "Mai", "Juni", "Juli", "August", "September",
        "Oktober", "November", "Dezember" };

    FillLayout fillLayout = new FillLayout(SWT.HORIZONTAL);
    fillLayout.spacing = 0;
    fillLayout.marginWidth = 0;
    fillLayout.marginHeight = 2;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.horizontalIndent = 0;
    gridData.verticalAlignment = GridData.CENTER;
    _compositeZeitwaehlerMonate = new Composite(parent, SWT.BORDER);
    _compositeZeitwaehlerMonate.setLayoutData(gridData);
    _compositeZeitwaehlerMonate.setLayout(fillLayout);

    for (int i = 0; i < 12; i++)
    {
      Label l = new Label(_compositeZeitwaehlerMonate, SWT.BORDER);
      l.setData(i);
      l.setBackground(ColorConstants.blue);
      l.setToolTipText(ss[i]);
    }
    createZeitwhlerMonateText(parent);
  }

  private void createZeitwhlerMonateText(Composite parent)
  {
    String[] ss = new String[] { "Jan", "Mr", "Mai", "Jul", "Sep", "Nov" };
    Composite compositeMonateText = new Composite(parent, SWT.NONE);
    compositeMonateText.setLayout(new FormLayout());

    int offset = 0;

    for (int j = 0; j < 6; j++)
    {
      Label l = new Label(compositeMonateText, SWT.NONE);
      l.setText(ss[j]);

      switch (j)
      {
        case 0:
          offset += 4;
          break;
        case 1:
          offset += 35;
          break;
        default:
          offset += 36;
      }

      FormData formData = new FormData();
      formData.left = new FormAttachment(compositeMonateText, offset, SWT.LEFT);
      formData.top = new FormAttachment(compositeMonateText, 0, SWT.BOTTOM);
      l.setLayoutData(formData);
    }
  }

  @Override
  public void dispose()
  {
    IWorkbenchPage page = getSite().getPage();

    if (page != null)
    {
      IViewReference fr = page.findViewReference(AuswahlSonderProgrammView.ID);
      if (fr != null)
      {
        IViewPart view = fr.getView(true);
        if (view != null)
        {
          AuswahlSonderProgrammView auswahl = (AuswahlSonderProgrammView)view;
          auswahl.setContent(null, false);
        }

      }

      fr = page.findViewReference(AuswahlSpaltenView.ID);
      if (fr != null)
      {
        IViewPart view = fr.getView(true);
        if (view != null)
        {
          AuswahlSpaltenView auswahl = (AuswahlSpaltenView)view;
          if (auswahl != null)
          {
            auswahl.setContent(null);
          }
        }
      }

    }

    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    if (page != null)
    {
      page.hideView(this);
    }
  }

  /**
   * @return
   */
  private SonderProgramm ueberpruefeErfassteDaten()
  {
    boolean bBezeichnung = false;
    boolean bDefinition = false;
    boolean bAktion = false;
    boolean bZustand = true;
    boolean bRichtung = false;
    boolean bKilometer = false;
    boolean bKilometer2 = false;
    boolean bAnzeigeQuerschnitt = false;
    boolean bHandschaltung = false;
    boolean bPrioHandschaltung = false;
    boolean bQuerLaengsabgleich = false;
    boolean bZusatzpruefung = false;
    boolean bSchaltOption = false;
    boolean bBetriebsArt = false;
    boolean bVorgabenSchaltungAq = false;
    boolean bZeitBereichVon = false;
    boolean bZeitBereichBis = false;
    boolean bAuswahlStunden = false;
    boolean bAuswahlTage = false;
    boolean bAuswahlMonate = false;

    // SonderProgramm sonderProgramm = _sonderProgrammDatensatz;
    SonderProgramm sonderProgramm = new SonderProgramm();
    
    if (_sonderProgrammDatensatz!= null)
      sonderProgramm.setPid(_sonderProgrammDatensatz.getPid());

    /*
     * Name der Engstelle
     */
    if ((_textName.getText() != null) && (_textName.getText().length() > 0))
    {
      sonderProgramm.setBezeichnung(_textName.getText());

      bBezeichnung = true;
    }

    /*
     * Definition des Grundprogramms
     */
    ISelection selection = _comboViewerProgrammTyp.getSelection();
    if (selection != null)
    {
      if (selection instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)selection;
        Object element = ss.getFirstElement();
        if (element != null)
        {
          if (element instanceof String)
          {
            String s = (String)element;
            if (s != null)
            {
              sonderProgramm.setTypBezeichnung(s);
              bDefinition = true;
            }
          }
          else if (element instanceof SswSbaProgrammTyp)
          {
            SswSbaProgrammTyp programmTyp = (SswSbaProgrammTyp)element;
            String bezeichnung = programmTyp.getBezeichnung();
            if (bezeichnung != null)
            {
              sonderProgramm.setTypBezeichnung(bezeichnung);
              bDefinition = true;
            }
          }

        }
      }
    }

    /*
     * Aktion SonderProgramm
     */
    {
      if (sonderProgramm.getGewaehlteSonderProgrammAktion() == null)
      {
        bAktion = false;
      }
    }

    /*
     * Zustand SonderProgramm
     */
    {
      if (sonderProgramm.getGewaehlterSonderProgrammZustand() == null)
      {
        if (_sonderProgrammDatensatz != null)
          sonderProgramm.setGewaehlterSonderProgrammZustand(_sonderProgrammDatensatz
              .getGewaehlterSonderProgrammZustand());
        else
          sonderProgramm.setGewaehlterSonderProgrammZustand(EnumSonderProgrammZustand.inaktiv);

        bZustand = true;
      }
    }

    /*
     * Richtung SonderProgramm
     */
    {
      if (sonderProgramm.getGewaehlteSonderProgrammRichtung() == null)
      {
        if (_sonderProgrammDatensatz != null)
          sonderProgramm.setGewaehlteSonderProgrammRichtung(_sonderProgrammDatensatz
              .getGewaehlteSonderProgrammRichtung());
        else
        {
          if (_secondaryId != null)
          {
            EnumSonderProgrammRichtung richtung = EnumSonderProgrammRichtung.valueOf(_secondaryId);
            if (richtung != null)
            {
              sonderProgramm.setGewaehlteSonderProgrammRichtung(richtung);
            }
          }
        }

        bRichtung = true;
      }
    }

    /*
     * Kilometrierung
     */
    if (textStartKm != null && textStopKm != null)
    {
      try
      {
        String t1 = textStartKm.getText();
        t1 = t1.replace(",", ".");
        Double d1 = new Double(t1);
        sonderProgramm.setAuswahlVonKm(d1 * 1000.0);

        String t2 = textStopKm.getText();
        t2 = t2.replace(",", ".");
        Double d2 = new Double(t2);
        sonderProgramm.setAuswahlBisKm(d2 * 1000.0);

        EnumSonderProgrammRichtung richtung = sonderProgramm.getGewaehlteSonderProgrammRichtung();
        if (richtung.equals(EnumSonderProgrammRichtung.NO))
        {
          // if (d2 >= d1)
          bKilometer = true;
        }
        else if (richtung.equals(EnumSonderProgrammRichtung.SW))
        {
          // if (d1 >= d2)
          bKilometer = true;
        }
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }

    /*
     * Kilometrierung Spursperrung
     */
    Boolean b2 = false;
    if (_sonderProgrammDatensatz != null)
    {
      b2 = _sonderProgrammDatensatz.isSpurSperrung();
    }
    else
    {
      b2 = ermittleSpurSperrungAusKonfig(_connection, sonderProgramm.getTypBezeichnung());
    }
    sonderProgramm.setSpurSperrung(b2);

    int length = _compositeFahrstreifenSperrungen.bestimmeAnzahlStufen();

    Sperrung[] sperrungen = new Sperrung[length];

    for (int i = 0; i < length; i++)
    {
      CompositeSperrungDarstellung comp = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung()[i];

      if (comp.getTextStartKmSperrung() != null && comp.getTextStopKmSperrung() != null
          && comp.getGroupSperrung() != null)
      {
        String t1 = comp.getTextStartKmSperrung().getText();
        t1 = t1.replace(",", ".");
        Double d1 = new Double(t1);

        String t2 = comp.getTextStopKmSperrung().getText();
        t2 = t2.replace(",", ".");
        Double d2 = new Double(t2);

        EnumSonderProgrammRichtung richtung = sonderProgramm.getGewaehlteSonderProgrammRichtung();

        if (richtung.equals(EnumSonderProgrammRichtung.NO))
        {
          bKilometer2 = true;
        }
        else if (richtung.equals(EnumSonderProgrammRichtung.SW))
        {
          bKilometer2 = true;
        }
        else
        {

        }

        Control[] children = comp.getGroupSperrung().getChildren();

        Boolean[] bb = new Boolean[8];

        if (children != null && children.length > 0)
          for (int j = 0; j < children.length; j++)
          {
            Button btn = (Button)children[j];
            bb[j] = btn.getSelection();
          }
        else
        {
          bKilometer2 = false;
        }

        sperrungen[i] = new Sperrung(d1 * 1000.0, d2 * 1000.0, bb);

      }
    }

    sonderProgramm.setSperrungen(sperrungen);

    if (b2 != null && b2)
    {
      // keine Aktion
    }
    else
    {
      bKilometer2 = true;
    }

    /*
     * Handprogramm
     */
    if (_radioButtonHandschaltungAktiv != null)
    {
      sonderProgramm.setHandSchaltung(_radioButtonHandschaltungAktiv.getSelection());

      bHandschaltung = true;
    }

    /*
     * Prioritt Handprogramm
     */
    if (_cvComboBoxPrioritaet != null)
    {

      String text = null;

      // ISelection sel = _cvComboBox.getSelection();
      // if (selection instanceof StructuredSelection)
      // {
      // StructuredSelection ss = (StructuredSelection)sel;
      //
      // if (ss.getFirstElement() != null)
      // {
      // text = ss.getFirstElement().toString();
      // }
      // }

      if (text == null)
      {
        text = _cvComboBoxPrioritaet.getCombo().getText();
      }

      try
      {
        if (!text.equals("undefiniert"))
        {
          Integer i = new Integer(text);
          if (i <= 60000 && i > -1)
          {
            // Wert liegt im Bereich
            bPrioHandschaltung = true;
          }
          else
          {
            // Wert liegt ausserhalb.
          }
        }
        else
        {
          // Text ist "undefiniert", also Ok.
          bPrioHandschaltung = true;
        }
      }
      catch (Exception ex)
      {
        // Text kann in keine Zahl gewandelt werden.
      }

      if (bPrioHandschaltung)
      {
        sonderProgramm.setPrioritaetHandschaltung(text);
      }
    }
    /*
     * QuerLaengsabgleich
     */
    if (_radioButtonQuerLaengsabgleich != null)
    {
      sonderProgramm.setQuerLaengsabgleich(_radioButtonQuerLaengsabgleich.getSelection());

      bQuerLaengsabgleich = true;
    }

    /*
     * Zusatzpruefung
     */
    if (_radioButtonAusfallbehandlung != null && _radioButtonVerschiebung != null && _radioButtonAufhebung != null)
    {
      sonderProgramm.getZusatzPruefung()[0] = _radioButtonAusfallbehandlung.getSelection();
      sonderProgramm.getZusatzPruefung()[1] = _radioButtonVerschiebung.getSelection();
      sonderProgramm.getZusatzPruefung()[2] = _radioButtonAufhebung.getSelection();

      bZusatzpruefung = true;
    }

    /*
     * Betriebsart
     */
    if (_radioButtonSimulation != null && _radioButtonSchaltung != null)
    {
      sonderProgramm.getBetriebsArt()[0] = _radioButtonSimulation.getSelection();
      sonderProgramm.getBetriebsArt()[1] = _radioButtonSchaltung.getSelection();

      bBetriebsArt = true;
    }

    /*
     * Zeitbereich Von
     */
    if (_radioButtonAbSofort != null && _radioButtonVorlaeufigNicht != null && zeitVon != null)
    {
      sonderProgramm.getZeitBereichVon()[0] = _radioButtonAbSofort.getSelection();
      sonderProgramm.getZeitBereichVon()[1] = _radioButtonVorlaeufigNicht.getSelection();
      sonderProgramm.getZeitBereichVon()[2] = zeitVon.getDatum().getTime();

      // Vorbelegung ab sofort
      if (!_radioButtonAbSofort.getSelection() && !_radioButtonVorlaeufigNicht.getSelection()
          && !_radioButtonVon.getSelection())
        sonderProgramm.getZeitBereichVon()[0] = true;

      bZeitBereichVon = true;
    }

    /*
     * Zeitbereich Bis
     */
    if (_radioButtonBisJetzt != null && _radioButtonBisAufWeiteres != null && zeitBis != null)
    {
      sonderProgramm.getZeitBereichBis()[0] = _radioButtonBisJetzt.getSelection();
      sonderProgramm.getZeitBereichBis()[1] = _radioButtonBisAufWeiteres.getSelection();
      sonderProgramm.getZeitBereichBis()[2] = zeitBis.getDatum().getTime();

      // Vorbelegung bis auf weiteres
      if (!_radioButtonBisJetzt.getSelection() && !_radioButtonBisAufWeiteres.getSelection()
          && !_radioButtonBis.getSelection())
        sonderProgramm.getZeitBereichBis()[1] = true;

      bZeitBereichBis = true;
    }

    /*
     * Auswahl Stunden
     */
    if (_comboStundenauswahl != null && _compositeZeitwaehlerStunden != null)
    {
      String s = _comboStundenauswahl.getItem(_comboStundenauswahl.getSelectionIndex());
      sonderProgramm.setStundenAuswahlBereiche(s);
      Control[] children = _compositeZeitwaehlerStunden.getChildren();

      Boolean[] bools = new Boolean[24];

      for (int i = 0; i < children.length; i++)
      {
        Label l = (Label)children[i];
        if (l.getBackground().equals(ColorConstants.blue))
          bools[i] = true;
        else
          bools[i] = false;

      }

      sonderProgramm.setStundenAuswahlStunden(bools);
      bAuswahlStunden = true;
    }

    /*
     * Auswahl Wochentage
     */
    if (_comboWochentagsauswahl != null && _compositeZeitwaehlerWochentage != null)
    {
      String s = _comboWochentagsauswahl.getItem(_comboWochentagsauswahl.getSelectionIndex());
      sonderProgramm.setWochenTagAuswahlBereiche(s);
      Control[] children = _compositeZeitwaehlerWochentage.getChildren();

      Boolean[] bools = new Boolean[7];

      for (int i = 0; i < children.length; i++)
      {
        Label l = (Label)children[i];
        if (l.getBackground().equals(ColorConstants.blue))
          bools[i] = true;
        else
          bools[i] = false;
      }

      sonderProgramm.setWochenTagAuswahlTage(bools);
      bAuswahlTage = true;
    }
    
    
    /*
     * Auswahl Ereignisse
     */
    if (_tvEreignisse != null && _tvEreignisse.getInput() != null)
    {
      Object input = _tvEreignisse.getInput();
      if (input instanceof List)
      {
        List ee = (List)input;
        
        int size = ee.size();
        SystemObject[] objekte = new SystemObject[size];
        
        for (int j = 0; j < size; j++)
        {
          objekte[j] = (SystemObject)ee.get(j);
          System.out.println("ZustandsDarstellungSonderProgramm.ueberpruefeErfassteDaten(Ereignisse)" + objekte[j]);
        }
        sonderProgramm.setEreignisse(objekte);
      }
    }
    
    /*
     * Auswahl Systemkalender
     */
    if (_tvSystemKalenderEintraege != null && _tvSystemKalenderEintraege.getInput() != null)
    {
      Object input = _tvSystemKalenderEintraege.getInput();
      if (input instanceof List)
      {
        List sk = (List)input;
        
        int size = sk.size();
        SystemObject[] objekte = new SystemObject[size];
        
        for (int j = 0; j < size; j++)
        {
          objekte[j] = (SystemObject)sk.get(j);
          System.out.println("ZustandsDarstellungSonderProgramm.ueberpruefeErfassteDaten(SKs)" + objekte[j]);
        }
        sonderProgramm.setSystemKalenderEintraege(objekte);
      }
    }

    /*
     * Auswahl Monate
     */
    if (_comboMonatsauswahl != null && _compositeZeitwaehlerMonate != null)
    {
      String s = _comboMonatsauswahl.getItem(_comboMonatsauswahl.getSelectionIndex());
      sonderProgramm.setMonatsAuswahlBereiche(s);
      Control[] children = _compositeZeitwaehlerMonate.getChildren();

      Boolean[] bools = new Boolean[12];

      for (int i = 0; i < children.length; i++)
      {

        Label l = (Label)children[i];
        if (l.getBackground().equals(ColorConstants.blue))
          bools[i] = true;
        else
          bools[i] = false;
      }

      sonderProgramm.setMonatsAuswahlMonate(bools);
      bAuswahlMonate = true;
    }

    List<AnzeigeQuerschnittModelTypAbstract> aqAuswahl = holeAqModelObjekteVonAqAuswahl();

    int length2 = aqAuswahl.size();

    Schaltbild[] ss = new Schaltbild[length2];

    for (int i = 0; i < aqAuswahl.size(); i++)
    {
      AnzeigeQuerschnittModelTypAbstract aq = aqAuswahl.get(i);
      Schaltbild sss = new Schaltbild();
      SystemObject so = aq.getSystemObject();

      if (so != null)
        sss.setAnzeigeQuerschnitt(so);

      List<AnzeigeModelTypAbstract> aes = aq.getKomponenten();

      Anzeige[] aa = new Anzeige[aes.size()];

      for (int j = 0; j < aes.size(); j++)
      {
        Anzeige aaa = new Anzeige();
        AnzeigeModelTypAbstract ae = (AnzeigeModelTypAbstract)aes.get(j);

        SystemObject anzeige = ae.getSystemObject();
        aaa.setAnzeige(anzeige);
        aaa.setPrioritaet(ae.getPrioritaet());
        aaa.setBlinken(ae.getBlinken() > 0 ? true : false);
        aaa.setBlinktakt(ae.getBlinktakt());
        aaa.setNotwendig(ae.getNotwendig() > 0 ? true : false);
        aaa.setAbsolutNotwendig(ae.getAbsolutNotwendig() > 0 ? true : false);
        aaa.setVerschiebbarerVorgaenger(ae.getVerschiebungVorgaenger());
        aaa.setVerschiebbarerNachfolger(ae.getVerschiebungNachfolger());
        aaa.setAbsolutNotwendig(ae.getAbsolutNotwendig() > 0 ? true : false);
        aaa.setGewaehlterAnzeigeProgrammStatus(EnumAnzeigeProgrammStatus.Sonderprogramm);
        aaa.setQuelle(null);

        Bild bild = new Bild();

        // Vorbelegen der Datenstruktur fuer den Wechseltext
        DynamischerWechselText dynamischerWechselText = new DynamischerWechselText();
        dynamischerWechselText.setFont(null);
        dynamischerWechselText.setGewaehlteTextAusrichtung(EnumDynamischerWechseltextTextAusrichtung.Linksbndig);
        dynamischerWechselText.setTextDefinition(null);
        aaa.setDynamischerWechselText(dynamischerWechselText);

        Object feature = ae.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);

        if (feature instanceof ResultData)
        {
          ResultData result = (ResultData)feature;
          AttributeGroup atg = result.getDataDescription().getAttributeGroup();

          if (atg.getPid().equals("atg.anzeigeEigenschaftIst") || atg.getPid().equals("atg.anzeigeEigenschaftSoll"))
          {
            Data data = result.getData();
            if (data != null)
            {

              SystemObject anzeigeInhalt = data.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue()
                  .getSystemObject();
              bild.setAnzeigeInhalt(anzeigeInhalt);
            }
          }

        }
        else if (feature instanceof AnzeigeInhaltTyp)
        {
          AnzeigeInhaltTyp inhalt = (AnzeigeInhaltTyp)feature;
          SystemObject anzeigeInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, anzeige, inhalt.getType());
          bild.setAnzeigeInhalt(anzeigeInhalt);
        }
        else if (feature instanceof VisuAnzeigeKapsel)
        {
          VisuAnzeigeKapsel kapsel = (VisuAnzeigeKapsel)feature;
          String ausrichtung = kapsel.getAusrichtung();
          if (ausrichtung != null)
          {
            EnumDynamischerWechseltextTextAusrichtung value = EnumDynamischerWechseltextTextAusrichtung
                .valueOf(ausrichtung);
            if (value != null)
              dynamischerWechselText.setGewaehlteTextAusrichtung(value);
          }

          String pidFont = kapsel.getSchriftart();
          SystemObject font = _connection.getDataModel().getObject(pidFont);
          if (font != null)
            dynamischerWechselText.setFont(font);
          String[] pidsZeichen = kapsel.getPidsZeichen();

          List<SystemObject> list = new ArrayList<SystemObject>();
          for (String pidZeichen : pidsZeichen)
          {
            SystemObject zeichen = _connection.getDataModel().getObject(pidZeichen);
            if (zeichen != null)
              list.add(zeichen);
          }

          SystemObject[] array = list.toArray(new SystemObject[list.size()]);
          dynamischerWechselText.setTextDefinition(array);
          aaa.setDynamischerWechselText(dynamischerWechselText);

        }
        bild.setGrafikDarstellungen(null);
        aaa.setBild(bild);
        aa[j] = aaa;
      }

      sss.setAnzeigen(aa);
      ss[i] = sss;
    }

    if (ss != null && ss.length > 0)
      bAnzeigeQuerschnitt = true;

    sonderProgramm.setSchaltbilder(ss);

    boolean returnValue = true;

    if (!bBezeichnung)
    {
      fehlerDialog("Name ungltig", "Es muss ein Name eingetragen werden !");
      returnValue = false;
    }
    if (!bDefinition)
    {
      fehlerDialog("Programm ungltig", "Es muss ein Programm ausgewhlt werden !");
      returnValue = false;
    }
    // Werden nicht ausgewertet
    if (!bAktion)
    {
    }
    if (!bRichtung)
    {
    }
    if (!bZustand)
    {
    }
    if (!bAnzeigeQuerschnitt)
    {
      fehlerDialog("Auswahl ungltig", "Es muss ein Anzeigequerschnitt ausgewhlt werden!");
      returnValue = false;
    }

    if (!bHandschaltung)
    {
      fehlerDialog("Handschaltung ungltig", "Auswahl ist ungltig");
      returnValue = false;
    }
    if (!bPrioHandschaltung)
    {
      fehlerDialog("Prioritt Handschaltung ungltig", "Auswahl ist ungltig");
      returnValue = false;
    }
    if (!bQuerLaengsabgleich)
    {
      fehlerDialog("QuerLngsabgleich ungltig", "Auswahl ist ungltig");
      returnValue = false;
    }
    if (!bZusatzpruefung)
    {
      fehlerDialog("Zusatzprfung ungltig", "Auswahl ist ungltig");
      returnValue = false;
    }
    if (!bBetriebsArt)
    {
      fehlerDialog("Betriebsart ungltig", "Auswahl ist ungltig");
      returnValue = false;
    }
    // Wird nicht ausgewertet
    if (!bVorgabenSchaltungAq)
    {
    }
    if (!bZeitBereichVon)
    {
      fehlerDialog("Zeitbereich Von ungltig", "Auswahl ist ungltig");
      returnValue = false;
    }
    if (!bZeitBereichBis)
    {
      fehlerDialog("Zeitbereich Bis ungltig", "Auswahl ist ungltig");
      returnValue = false;
    }
    if (!bAuswahlStunden)
    {
      fehlerDialog("Auswahl Stunden ungltig", "Auswahl ist ungltig");
      returnValue = false;
    }
    if (!bAuswahlTage)
    {
      fehlerDialog("Auswahl Wochentag ungltig", "Auswahl ist ungltig");
      returnValue = false;
    }
    if (!bAuswahlMonate)
    {
      fehlerDialog("Auswahl Monate ungltig", "Auswahl ist ungltig");
      returnValue = false;

    }

    boolean bilden = false;

    if (_sonderProgrammDatensatz != null)
    {

      if (sonderProgramm.getTypBezeichnung() != null)
      {
        if (!sonderProgramm.getTypBezeichnung().equals(_sonderProgrammDatensatz.getTypBezeichnung()))
        {
          bilden = true;
        }
      }

      if (sonderProgramm.getBezeichnung() != null)
      {
        if (!sonderProgramm.getBezeichnung().equals(_sonderProgrammDatensatz.getBezeichnung()))
        {
          bilden = true;
        }
      }

      Double d1 = sonderProgramm.getAuswahlVonKm();
      Double d2 = _sonderProgrammDatensatz.getAuswahlVonKm();

      if (!d1.equals(d2))
      {
        bilden = true;
      }

      d1 = sonderProgramm.getAuswahlBisKm();
      d2 = _sonderProgrammDatensatz.getAuswahlBisKm();

      if (!d1.equals(d2))
      {
        bilden = true;
      }

      Sperrung[] sp1 = sonderProgramm.getSperrungen();
      Sperrung[] sp2 = _sonderProgrammDatensatz.getSperrungen();

      if (sp1 != null && sp2 != null)
      {
        if (sp1.length != sp2.length)
        {
          bilden = true;
        }
        else
        {
          for (int i = 0; i < sp1.length; i++)
          {
            Sperrung s1 = sp1[i];
            Sperrung s2 = sp2[i];

            if (s1 != null && s2 != null)
            {
              Double d3 = s1.getSperrungVonKm();
              Double d4 = s2.getSperrungVonKm();

              if (d3 != null && d4 != null)
              {
                if (!d3.equals(d4))
                {
                  bilden = true;
                }
              }
              Double d5 = s1.getSperrungBisKm();
              Double d6 = s2.getSperrungBisKm();

              if (d5 != null && d6 != null)
              {
                if (!d5.equals(d6))
                {
                  bilden = true;
                }
              }

              Boolean[] sperrungFahrspur = s1.getSperrungFahrspur();
              for (int j = 0; j < sperrungFahrspur.length; j++)
              {
                Boolean bb1 = sperrungFahrspur[j];
                Boolean bb2 = s2.getSperrungFahrspur()[j];

                if (bb1 != null && bb2 != null && !bb1.equals(bb2))
                {
                  bilden = true;
                }
              }
            }
            else
            {
              bilden = true;
            }
          }
        }
      }
      else
      {
        bilden = true;
      }

      Boolean[] schaltOptionen = sonderProgramm.getSchaltOptionen();
      if (schaltOptionen != null)
      {
        for (int i = 0; i < schaltOptionen.length; i++)
        {
          Boolean b = schaltOptionen[i];
          if (b != null && !b.equals(_sonderProgrammDatensatz.getSchaltOptionen()[i]))
          {
            bilden = true;
          }
        }
      }
    }
    else
      bilden = true;

    if (bilden)
    {
      sonderProgramm.setGewaehlteSonderProgrammAktion(EnumSonderProgrammAktion.Bilden);
    }
    else
    {
      sonderProgramm.setGewaehlteSonderProgrammAktion(EnumSonderProgrammAktion.ndern);
    }

    if (returnValue)
      return sonderProgramm;
    else
      return null;
  }

  private void fehlerDialog(String titel, String meldung)
  {
    MessageBox mb = new MessageBox(Display.getDefault().getActiveShell(), SWT.ICON_ERROR | SWT.OK);
    mb.setText(titel);
    mb.setMessage(meldung);
    mb.open();
  }

  public void aktualisiereSicht(SonderProgramm sonderProgramm)
  {
    if (sonderProgramm != null)
    {
      // Grundprogramm in Auswahl setzen
      if (_listProgrammTypen != null)
      {
        if (_comboViewerProgrammTyp.getContentProvider() != null)
          _comboViewerProgrammTyp.setInput(_listProgrammTypen);
      }

      /*
       * Umsetzung des Sonderprogrammstrings nach der passenden SswSbaProgrammTyp-Instanz.
       */
      String s2 = sonderProgramm.getTypBezeichnung();
      if ((s2 != null) && (_listProgrammTypen != null))
      {
        Iterator<SswSbaProgrammTyp> it = _listProgrammTypen.iterator();
        while (it.hasNext())
        {
          SswSbaProgrammTyp sswSbaProgrammTyp = it.next();
          if ((sswSbaProgrammTyp != null) && (sswSbaProgrammTyp.getBezeichnung().equals(s2)))
          {
            _comboViewerProgrammTyp.setSelection(new StructuredSelection(sswSbaProgrammTyp));
            break;
          }
        }
      }

      // Name
      if (!_textName.isDisposed())
      {
        _textName.setText(sonderProgramm.getBezeichnung());
      }
      // AuswahlStartKm
      Double d1 = sonderProgramm.getAuswahlVonKm();
      String x1 = "";
      if (d1 != null)
      {

        Double copy = new Double(d1);
        if (copy > 0.0)
        {
          copy /= 1000.0;
        }
        x1 = new Formatter().format("%1$7.3f", copy).toString();
      }
      if (!textStartKm.isDisposed())
      {
        textStartKm.setText(x1);
        markiereLabelAq(_connection, _secondaryId, textStartKm, labelStartKm);
      }

      // AuswahlEndeKm
      Double d2 = sonderProgramm.getAuswahlBisKm();
      String x2 = "";
      if (d2 != null)
      {
        Double copy = new Double(d2);
        if (copy > 0.0)
        {
          copy /= 1000.0;
        }
        x2 = new Formatter().format("%1$7.3f", copy).toString();
      }
      if (!textStopKm.isDisposed())
      {
        textStopKm.setText(x2);
        markiereLabelAq(_connection, _secondaryId, textStopKm, labelStopKm);
      }

      // Gltigkeit
      if (_tvEreignisse.getContentProvider() != null)
      {
        SystemObject[] ee = sonderProgramm.getEreignisse();
        if (ee != null)
        {
          _tvEreignisse.setInput(Arrays.asList(ee));
        }
      }

      if (_tvSystemKalenderEintraege.getContentProvider() != null)
      {
        SystemObject[] ss = sonderProgramm.getSystemKalenderEintraege();
        if (ss != null)
        {
          _tvSystemKalenderEintraege.setInput(Arrays.asList(ss));
        }
      }

      // if (_compositeParameterStufen.isDisposed())
      {
        _compositeFahrstreifenSperrungen.setSonderProgrammDatensatz(sonderProgramm);

        // TabFolder mit den Sperrungen zuruecksetzen
        _compositeFahrstreifenSperrungen.erzeugeBenoetigteReiter(1);
        CTabItem item = _compositeFahrstreifenSperrungen.getTabFolder().getItems()[0];
        if (item != null && !item.isDisposed())
        {
          Composite cSel = (Composite)item.getControl();
          if (((Composite)cSel).getChildren().length > 0)
          {
            CompositeSperrungDarstellung comp = (CompositeSperrungDarstellung)cSel.getChildren()[0];
            comp.initialisiereControls();            
          }
        }

        Sperrung[] sperrungen = sonderProgramm.getSperrungen();
        if (sperrungen != null && sperrungen.length > 0)
        {
          _compositeFahrstreifenSperrungen.erzeugeBenoetigteReiter(sperrungen.length);
          for (int i = 0; i < sperrungen.length; i++)
          {
            Sperrung sperrung = sperrungen[i];

            // SperrungStartKm
            Double d3 = sperrung.getSperrungVonKm();
            String x3 = "";
            if (d3 != null)
            {

              Double copy = new Double(d3);
              if (copy > 0.0)
              {
                copy /= 1000.0;
              }
              x3 = new Formatter().format("%1$7.3f", copy).toString();
            }
            Text text2 = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung()[i]
                .getTextStartKmSperrung();
            if (text2 != null
            // && !text2.isDisposed()
            )
            {
              if (x3.length() > 0)
                text2.setText(x3);
              else
                text2.setText("  0,000");

              Label label34 = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung()[i]
                  .getLabelStartKmSperrung();
              markiereLabelAq(_connection, _secondaryId, text2, label34);
            }

            // SperrungEndeKm
            Double d4 = sperrung.getSperrungBisKm();
            String x4 = "";
            if (d4 != null)
            {
              Double copy = new Double(d4);
              if (copy > 0.0)
              {
                copy /= 1000.0;
              }
              x4 = new Formatter().format("%1$7.3f", copy).toString();
            }
            Text text3 = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung()[i].getTextStopKmSperrung();
            if (text3 != null
            // && !text3.isDisposed()
            )
            {
              if (x4.length() > 0)
                text3.setText(x4);
              else
                text3.setText("  0,000");

              Label label33 = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung()[i]
                  .getLabelStopKmSperrung();
              markiereLabelAq(_connection, _secondaryId, text3, label33);
            }

            CompositeSperrungDarstellung[] comps = _compositeFahrstreifenSperrungen.getCompositeParameterDarstellung();
            if (comps != null && comps.length > 0)
            {
              CompositeSperrungDarstellung comp = comps[i];
              if (comp != null
              // && !comp.isDisposed()
              )
              {
                Group group = comp.getGroupSperrung();
                Control[] children = group.getChildren();
                if (children != null)
                {
                  for (int j = 0; j < children.length; j++)
                  {
                    Button btn = (Button)children[j];
                    if (btn != null && !btn.isDisposed())
                    {
                      Boolean b = sperrung.getSperrungFahrspur()[j];
                      if (btn != null && b != null)
                        btn.setSelection(b);
                    }
                  }
                }
              }
            }
          }
        }
        else
        {
          _compositeFahrstreifenSperrungen.erzeugeBenoetigteReiter(1);
          item = _compositeFahrstreifenSperrungen.getTabFolder().getItems()[0];
          if (item != null && !item.isDisposed())
          {
            Composite cSel = (Composite)item.getControl();
            if (((Composite)cSel).getChildren().length > 0)
            {
              CompositeSperrungDarstellung comp = (CompositeSperrungDarstellung)cSel.getChildren()[0];
              comp.setVisible(false);
            }
          }
        }
      }

      Boolean bo = sonderProgramm.isSpurSperrung();
      if (bo != null && !bo)
      {
        // keine Aktion
      }
      // Handschaltung
      Boolean obj0 = sonderProgramm.getHandSchaltung();
      if (obj0 != null)
      {
        if (_radioButtonHandschaltungAktiv != null && !_radioButtonHandschaltungAktiv.isDisposed())
        {
          _radioButtonHandschaltungAktiv.setSelection(obj0);
        }
      }

      if (_radioButtonPrioritaet != null && !_radioButtonPrioritaet.isDisposed())
      {
        _radioButtonPrioritaet.setSelection(false);
        _comboPrioritaet.setEnabled(false);
      }

      String p0 = sonderProgramm.getPrioritaetHandschaltung();
      if (p0 != null)
      {
        if (_cvComboBoxPrioritaet != null && !_cvComboBoxPrioritaet.getCombo().isDisposed())
        {
          String[] items = _cvComboBoxPrioritaet.getCombo().getItems();

          int index = -1;
          for (int i = 0; i < items.length; i++)
          {
            String s = items[i];
            if (s.equals(p0))
            {
              index = i;
              break;
            }
          }

          if (index < 0)
          {
            _cvComboBoxPrioritaet.getCombo().setText(p0);

          }
          _cvComboBoxPrioritaet.getCombo().select(index);

          // Wenn keine Handschaltung, dann Prio auf undefiniert
          if (obj0 != null)
          {
            if (!obj0)
            {
              _cvComboBoxPrioritaet.setSelection(new StructuredSelection("undefiniert"));
            }
            else
            {
              if (_radioButtonPrioritaet != null && !_radioButtonPrioritaet.isDisposed())
              {

                if (!p0.equals("undefiniert"))
                {
                  _radioButtonPrioritaet.setSelection(obj0);
                  _comboPrioritaet.setEnabled(obj0);
                }
                else
                {
                  _radioButtonPrioritaet.setSelection(false);
                }
              }
            }
          }

        }
      }
      // QuerLaengsabgleich
      Boolean obj1 = sonderProgramm.getQuerLaengsabgleich();
      if (obj1 != null)
      {
        if (_radioButtonQuerLaengsabgleich != null && !_radioButtonQuerLaengsabgleich.isDisposed())
        {
          _radioButtonQuerLaengsabgleich.setSelection(obj1);
        }
      }
      // Zusatzpruefung
      Boolean[] objs2 = sonderProgramm.getZusatzPruefung();
      if (objs2 != null)
      {
        if (_radioButtonAusfallbehandlung != null && _radioButtonVerschiebung != null && _radioButtonAufhebung != null
            && !_radioButtonAusfallbehandlung.isDisposed() && !_radioButtonVerschiebung.isDisposed()
            && !_radioButtonAufhebung.isDisposed())
        {
          if (objs2[0] != null && objs2[1] != null && objs2[2] != null)
          {
            _radioButtonAusfallbehandlung.setSelection(objs2[0]);
            _radioButtonVerschiebung.setSelection(objs2[1]);
            _radioButtonAufhebung.setSelection(objs2[2]);
          }
        }
      }
      // Betriebsart
      Boolean[] objs4 = sonderProgramm.getBetriebsArt();
      if (objs4 != null)
      {
        if (_radioButtonSimulation != null && _radioButtonSchaltung != null && !_radioButtonSimulation.isDisposed()
            && !_radioButtonSchaltung.isDisposed())
        {
          if (objs4[0] != null && objs4[1] != null)
          {
            _radioButtonSimulation.setSelection(objs4[0]);
            _radioButtonSchaltung.setSelection(objs4[1]);
          }
        }
      }
      // Zeitbereich Von
      Object[] objs1 = sonderProgramm.getZeitBereichVon();
      if (objs1 != null)
      {
        if (objs1[0] != null)
        {
          if (_radioButtonAbSofort != null && !_radioButtonAbSofort.isDisposed())
            _radioButtonAbSofort.setSelection((Boolean)objs1[0]);
        }
        if (objs1[1] != null)
        {
          if (_radioButtonVorlaeufigNicht != null && !_radioButtonVorlaeufigNicht.isDisposed())
            _radioButtonVorlaeufigNicht.setSelection((Boolean)objs1[1]);
        }

        if (!_radioButtonAbSofort.isDisposed() && !_radioButtonVorlaeufigNicht.isDisposed()
            && !_radioButtonVon.isDisposed())
        {
          if (_radioButtonAbSofort.getSelection() || _radioButtonVorlaeufigNicht.getSelection())
          {
            _radioButtonVon.setSelection(false);
          }
          else
          {
            _radioButtonVon.setSelection(true);
            if (objs1[2] != null)
            {
              Date d = new Date((Long)objs1[2]);
              if (!zeitVon.isDisposed())
                zeitVon.setDatum(d);
            }
          }
        }
      }
      // Zeitbereich Bis
      Object[] objs11 = sonderProgramm.getZeitBereichBis();
      if (objs1 != null)
      {

        if (objs11[0] != null)
        {
          if (_radioButtonBisJetzt != null && !_radioButtonBisJetzt.isDisposed())
            _radioButtonBisJetzt.setSelection((Boolean)objs11[0]);
        }
        if (objs11[1] != null)
        {
          if (_radioButtonBisAufWeiteres != null && !_radioButtonBisAufWeiteres.isDisposed())
            _radioButtonBisAufWeiteres.setSelection((Boolean)objs11[1]);
        }

        if (!_radioButtonBisJetzt.isDisposed() && !_radioButtonBisAufWeiteres.isDisposed()
            && !_radioButtonBis.isDisposed())
        {
          if (_radioButtonBisJetzt.getSelection() || _radioButtonBisAufWeiteres.getSelection())
          {
            _radioButtonBis.setSelection(false);
          }
          else
          {
            _radioButtonBis.setSelection(true);
            if (objs11[2] != null)
            {
              Date d = new Date((Long)objs11[2]);
              if (!zeitVon.isDisposed())
                zeitBis.setDatum(d);
            }
          }
        }
      }
      // Stunden
      String s12 = sonderProgramm.getStundenAuswahlBereiche();
      if (s12 != null)
      {
        if (!_comboStundenauswahl.isDisposed())
        {
          for (int i = 0; i < _comboStundenauswahl.getItemCount(); i++)
          {
            if (s12.equals(_comboStundenauswahl.getItem(i)))
            {
              _comboStundenauswahl.select(i);
            }
          }
        }
      }
      if (!_compositeZeitwaehlerStunden.isDisposed())
      {
        Control[] children1 = _compositeZeitwaehlerStunden.getChildren();
        Boolean[] stunden = sonderProgramm.getStundenAuswahlStunden();
        if (stunden != null)
        {
          for (int i = 0; i < stunden.length; i++)
          {
            Label l = (Label)children1[i];

            if (stunden[i] != null)
            {
              if (stunden[i])
                l.setBackground(ColorConstants.blue);
              else
                l.setBackground(ColorConstants.white);
            }
          }
        }
      }
      // Wochentage
      String s13 = sonderProgramm.getWochenTagAuswahlBereiche();
      if (s13 != null)
      {
        if (!_comboWochentagsauswahl.isDisposed())
        {
          for (int i = 0; i < _comboWochentagsauswahl.getItemCount(); i++)
          {
            if (s13.equals(_comboWochentagsauswahl.getItem(i)))
            {
              _comboWochentagsauswahl.select(i);
            }
          }
        }
      }
      if (!_compositeZeitwaehlerWochentage.isDisposed())
      {
        Control[] children2 = _compositeZeitwaehlerWochentage.getChildren();
        Boolean[] wochentage = sonderProgramm.getWochenTagAuswahlTage();
        if (wochentage != null)
        {
          for (int i = 0; i < wochentage.length; i++)
          {
            Label l = (Label)children2[i];

            if (wochentage[i] != null)
            {
              if (wochentage[i])
                l.setBackground(ColorConstants.blue);
              else
                l.setBackground(ColorConstants.white);
            }

          }
        }
      }
      // Monate
      String s14 = sonderProgramm.getMonatsAuswahlBereiche();
      if (s14 != null)
      {
        if (!_comboWochentagsauswahl.isDisposed())
        {
          for (int i = 0; i < _comboMonatsauswahl.getItemCount(); i++)
          {
            if (s14.equals(_comboMonatsauswahl.getItem(i)))
            {
              _comboMonatsauswahl.select(i);
            }
          }
        }
      }
      if (!_compositeZeitwaehlerMonate.isDisposed())
      {
        Control[] children3 = _compositeZeitwaehlerMonate.getChildren();
        Boolean[] monate = sonderProgramm.getMonatsAuswahlMonate();
        if (monate != null)
        {
          for (int i = 0; i < monate.length; i++)
          {
            Label l = (Label)children3[i];

            if (monate[i] != null)
            {
              if (monate[i])
                l.setBackground(ColorConstants.blue);
              else
                l.setBackground(ColorConstants.white);
            }

          }
        }
      }

      // Schaltbild zuruecksetzen
      setzteAlleAnzeigenMitKontext((AnsichtEditPart)graphicalViewer.getContents(), AnzeigeInhaltTyp.BELIEBIG_BI, 4);

      // Schaltbild
      Schaltbild[] schaltbilder = sonderProgramm.getSchaltbilder();

      if (schaltbilder != null)
      {
        for (int i = 0; i < schaltbilder.length; i++)
        {
          Schaltbild schaltbild = schaltbilder[i];

          Anzeige[] anzeigen = schaltbild.getAnzeigen();
          for (Anzeige anzeige : anzeigen)
          {

            SystemObject so = anzeige.getAnzeige();
            DoModel doModel = DaVHelper.holeAnzeigeModelTypAbstractVonSystemObject((AnsichtEditPart)graphicalViewer
                .getContents(), so, 4);
            Bild bild = anzeige.getBild();
            if (bild != null)
            {
              SystemObject anzeigeInhalt = bild.getAnzeigeInhalt();

              if (anzeigeInhalt != null)
              {
                AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");
                Data cd = anzeigeInhalt.getConfigurationData(atg);
                String bi = cd.getItem("Bildinhalt").asTextValue().getValueText();

                AnzeigeInhaltTyp inhaltTyp = AnzeigeInhaltTyp.setBildInhalt(bi, sonderProgramm.getDatenZeit());

                if (doModel instanceof AnzeigeModelTypAbstract)
                {
                  AnzeigeModelTypAbstract castedModel = (AnzeigeModelTypAbstract)doModel;
                  anzeige.setNotwendig(anzeige.isNotwendig());
                  anzeige.setAbsolutNotwendig(anzeige.isAbsolutNotwendig());
                  anzeige.setPrioritaet(anzeige.getPrioritaet());
                  anzeige.setBlinken(anzeige.isBlinken());
                  String takt = anzeige.getBlinktakt();
                  if (takt.contains(","))
                  {
                    String taktNeu = takt.replace(",", ".");
                    anzeige.setBlinktakt(taktNeu);
                  }
                  else
                    anzeige.setBlinktakt(takt);

                  if (inhaltTyp != null)
                  {
                    EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                    final SetCommand cmd = new SetCommand(castedModel, feature, inhaltTyp);
                    getCommandStack().execute(cmd);
                  }

                }
              }
            }

            DynamischerWechselText dynamischerWechselText = anzeige.getDynamischerWechselText();
            if (dynamischerWechselText != null)
            {
              SystemObject font = dynamischerWechselText.getFont();

              if (font != null)
              {
                EnumDynamischerWechseltextTextAusrichtung ausrichtung = dynamischerWechselText
                    .getGewaehlteTextAusrichtung();
                SystemObject[] textDefinition = dynamischerWechselText.getTextDefinition();
                List<String> pidsZeichen = new ArrayList<String>();
                for (int j = 0; j < textDefinition.length; j++)
                {
                  SystemObject zeichen = textDefinition[j];
                  pidsZeichen.add(zeichen.getPid());
                }

                int abstand = 0;
                if (_connection != null)
                {
                  AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.font");
                  Data cd = font.getConfigurationData(atg);
                  double dv = cd.getItem("ZeichenAbstandStandard").asScaledValue().doubleValue();
                  if (dv > 0.0)
                    abstand = (int)(1 / dv);
                }

                VisuAnzeigeKapsel kapsel = new VisuAnzeigeKapsel(pidsZeichen.toArray(new String[pidsZeichen.size()]),
                    font.getPid(), ausrichtung.name(), abstand);
                if (kapsel != null)
                {

                  AnzeigeModelTypAbstract a = (AnzeigeModelTypAbstract)doModel;
                  EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                  final SetCommand cmd = new SetCommand(doModel, feature, kapsel);
                  getCommandStack().execute(cmd);
                }
              }
            }

            AnzeigeModelTypAbstract ae = (AnzeigeModelTypAbstract)doModel;

            // System.out.println(ae + " : " + anzeige);

            if (anzeige != null && ae != null)
            {
              ae.setPrioritaet(anzeige.getPrioritaet());
              ae.setBlinken(anzeige.isBlinken() ? 1 : 0);
              String blinktakt = anzeige.getBlinktakt();
              ae.setBlinktakt(blinktakt);
              ae.setNotwendig(anzeige.isNotwendig() ? 1 : 0);
              ae.setAbsolutNotwendig(anzeige.isAbsolutNotwendig() ? 1 : 0);
              ae.setVerschiebungVorgaenger(anzeige.getVerschiebbarerVorgaenger());
              ae.setVerschiebungNachfolger(anzeige.getVerschiebbarerNachfolger());
            }
          }
        }
      }
      
//      boolean absteigend = DaVHelper.ermittleKilometrierungAufsteigendAbsteigend(_secondaryId);      
//        if (d1 > 0)
//          d1 /= 1000.0;
//        if (d2 > 0)
//          d2 /= 1000.0;
//      SortedMap<String, SystemObject> listeKilometrierungAq = DaVHelper.holeListeKilometrierungAq(_secondaryId,
//          false, d1, d2);

      // Die AQ werden nicht anhand der Kilometrierung ausgewaehlt sondern dananch ob Inhalte gepixelt wurden
      SortedMap<Integer, DoMitVerbLinieEditPart> listeKilometrierungAq = DaVHelper.holeAnzeigeQuerschnittModelTypAbstractVonKontext(getGraphicalViewer(), 4, false);
      
      if (listeKilometrierungAq.size() > 0)
      {         
        // der letzte Eintrag der Liste ist relevant, da dieser die Scroll-Position liefert
        DoMitVerbLinieEditPart epart = listeKilometrierungAq.get(listeKilometrierungAq.lastKey());
        if (epart != null)
        {           
          IFigure target = ((GraphicalEditPart)epart).getFigure();
          FigureCanvas figureCanvas = (FigureCanvas)getGraphicalViewer().getControl();
          // y-Ausdehnung sichbarer Ausschnitts im Viewer 
          int yViewPort = figureCanvas.getViewport().getSize().height;              
          // y-Ausdehnung der Figure
          int heightFigure = target.getBounds().height;
          // y-Position der Figuren der Darstellung
          int yFigure = target.getBounds().y;
          
          int  yScroll = 0;  
          // es wird nur gescrollt, wenn die y-Position der Figure grsser ist 
          // als die y-Ausdehnung des sichbare Ausschnitts 
          if (yFigure > yViewPort){
           yScroll = (yFigure - yViewPort) + heightFigure;
          }
          
          //System.out.println("yScroll :" + yViewPort + " -> " + yFigure + " -> " + target.getBounds() + " ->>>> " + yScroll);
          
          figureCanvas.scrollTo(0 , yScroll);                                     
        }          
      }          
      _sonderProgrammDatensatz = sonderProgramm;
    }
  }

  public void aktualisiereSichtKeineSelektion()
  {
    // Grundprogramm in Auswahl setzen
    initialisiereControls();
  }

  /**
   * Hier wird die initiale Ausprgung der Controls (z.B. selektiert oder nicht selektiert) durchgefhrt.
   */
  private void initialisiereControls()
  {
    if (_comboViewerProgrammTyp.getContentProvider() != null)
    {

    }
    // Name
    if (!_textName.isDisposed())
      _textName.setText("");
    // AuswahlStartKm
    Double von = new Double(0);

    IPreferenceStore store = Activator.getDefault().getPreferenceStore();

    String pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO;
    String pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO;

    if (_secondaryId.equals("SW"))
    {
      pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW;
      pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW;
    }

    double d3 = store.getDouble(pref1);
    double d4 = store.getDouble(pref2);

    boolean absteigend = DaVHelper.ermittleKilometrierungAufsteigendAbsteigend(_secondaryId);

    SortedMap<String, SystemObject> listeKilometrierungAq = DaVHelper.holeListeKilometrierungAq(_secondaryId,
        absteigend, d3, d4);
    if (listeKilometrierungAq != null && !listeKilometrierungAq.isEmpty())
    {
      String fkey = listeKilometrierungAq.firstKey();
      String[] split = fkey.split("_");
      if (split.length > 0)
        von = new Double(split[0]);

    }

    String s = new Formatter().format("%1$7.3f", von.doubleValue()).toString();

    if (!textStartKm.isDisposed())
    {
      textStartKm.setText(s);
    }
    // AuswahlEndeKm
    Double bis = new Double(0);
    if (listeKilometrierungAq != null && !listeKilometrierungAq.isEmpty())
    {
      String lkey = listeKilometrierungAq.lastKey();
      String[] split = lkey.split("_");
      if (split.length > 0)
        bis = new Double(split[0]);
    }

    s = new Formatter().format("%1$7.3f", bis.doubleValue()).toString();

    if (!textStartKm.isDisposed())
    {
      textStopKm.setText(s);
    }

    // Labels mit AQ-Bezeichnung setzen
    if (_connection != null && !textStartKm.isDisposed() && !textStopKm.isDisposed())
    {
      ZustandsDarstellungSonderProgramm.markiereLabelAq(_connection, _secondaryId, textStartKm, labelStartKm);
      ZustandsDarstellungSonderProgramm.markiereLabelAq(_connection, _secondaryId, textStopKm, labelStopKm);
    }

    // TabFolder mit den Sperrungen zuruecksetzen
    if (!_compositeFahrstreifenSperrungen.isDisposed())
    {
      _compositeFahrstreifenSperrungen.erzeugeBenoetigteReiter(1);
      CTabItem item = _compositeFahrstreifenSperrungen.getTabFolder().getItems()[0];
      Composite cSel = (Composite)item.getControl();
      if (((Composite)cSel).getChildren().length > 0)
      {
        CompositeSperrungDarstellung comp = (CompositeSperrungDarstellung)cSel.getChildren()[0];
        comp.initialisiereControls();
        comp.setVisible(false);
      }
    }

    // Handschaltung
    if (!_radioButtonHandschaltungAktiv.isDisposed() && !_radioButtonPrioritaet.isDisposed()
        && !_comboPrioritaet.isDisposed())
    {
      _radioButtonHandschaltungAktiv.setSelection(false);
      _radioButtonPrioritaet.setSelection(false);
      _comboPrioritaet.setEnabled(false);
    }
    // QuerLaengsabgleich
    if (!_radioButtonQuerLaengsabgleich.isDisposed())
    {
      _radioButtonQuerLaengsabgleich.setSelection(false);
    }
    // Zusatzpruefung
    if (!_radioButtonAusfallbehandlung.isDisposed() && !_radioButtonVerschiebung.isDisposed()
        && !_radioButtonAufhebung.isDisposed())
    {
      _radioButtonAusfallbehandlung.setSelection(false);
      _radioButtonVerschiebung.setSelection(false);
      _radioButtonAufhebung.setSelection(false);
    }
    // // Schaltoptionen
    // if (!radioButton281.isDisposed() && ! radioButton381.isDisposed()){
    // radioButton281.setSelection(false);
    // radioButton381.setSelection(false);
    // }
    // Betriebsart
    if (!_radioButtonSimulation.isDisposed() && !_radioButtonSchaltung.isDisposed())
    {
      _radioButtonSimulation.setSelection(false);
      _radioButtonSchaltung.setSelection(false);
    }
    // Zeitbereich Von
    if (!_radioButtonAbSofort.isDisposed() && !_radioButtonVorlaeufigNicht.isDisposed()
        && !_radioButtonVon.isDisposed())
    {
      _radioButtonAbSofort.setSelection(true);
      _radioButtonVorlaeufigNicht.setSelection(false);
      _radioButtonVon.setSelection(false);
    }
    // Zeitbereich Bis
    if (!_radioButtonBisJetzt.isDisposed() && !_radioButtonBisAufWeiteres.isDisposed() && !_radioButtonBis.isDisposed())
    {
      _radioButtonBisJetzt.setSelection(false);
      _radioButtonBisAufWeiteres.setSelection(true);
      _radioButtonBis.setSelection(false);
    }
    // Stunden
    if (!_comboStundenauswahl.isDisposed())
    {
      _comboStundenauswahl.select(0);
    }
    // Wochentage
    if (!_comboWochentagsauswahl.isDisposed())
    {
      _comboWochentagsauswahl.select(0);
    }
    // Monate
    if (!_comboMonatsauswahl.isDisposed())
    {
      _comboMonatsauswahl.select(0);
    }

    setzteAlleAnzeigenMitKontext((AnsichtEditPart)graphicalViewer.getContents(), AnzeigeInhaltTyp.BELIEBIG_BI, 4);
  
  }

  /**
   * @param result
   * @return
   */
  public static SonderProgramm fuelleDatenStruktur(ClientDavInterface connection, ResultData result)
  {

    if (result != null && result.hasData())
    {
      SonderProgramm sonderProgrammDatensatz = new SonderProgramm();
      sonderProgrammDatensatz.setPid(result.getObject().getPidOrId());
      sonderProgrammDatensatz.setDatenZeit(result.getDataTime());

      Data data = result.getData();
      // if (data != null)
      // {
      // Name
      String s1 = data.getItem("nameAnwender").asTextValue().getValueText();
      sonderProgrammDatensatz.setBezeichnung(s1);

      // Grundprogramm
      String text = data.getItem("typBezeichnung").asTextValue().getText();
      if (text != null)
        sonderProgrammDatensatz.setTypBezeichnung(text);

      // Zustand
      String zustand = data.getItem("Zustand").asTextValue().getValueText();
      EnumSonderProgrammZustand enumZustand = EnumSonderProgrammZustand.valueOf(zustand);
      sonderProgrammDatensatz.setGewaehlterSonderProgrammZustand(enumZustand);

      // Richtung
      String richtung = data.getItem("Fahrtrichtung").asTextValue().getValueText();
      EnumSonderProgrammRichtung enumRichtung = EnumSonderProgrammRichtung.valueOf(richtung);
      sonderProgrammDatensatz.setGewaehlteSonderProgrammRichtung(enumRichtung);

      // Streckenabschnitt
      Array array1 = data.getItem("Streckenabschnitt").asArray();
      if (array1.getLength() > 0)
      {
        Data item = array1.getItem(0);
        // StartKm
        Double d1 = item.getItem("startKm").asUnscaledValue().doubleValue();
        // EndeKm
        Double d2 = item.getItem("endeKm").asUnscaledValue().doubleValue();
        sonderProgrammDatensatz.setAuswahlVonKm(d1);
        sonderProgrammDatensatz.setAuswahlBisKm(d2);
        Array array2 = item.getItem("FahrstreifenSperrung").asArray();

        Sperrung[] sperrungen = new Sperrung[array2.getLength()];
        for (int i = 0; i < array2.getLength(); i++)
        {
          Data item2 = array2.getItem(i);
          // StartKm
          Double d3 = item2.getItem("startKm").asUnscaledValue().doubleValue();
          // EndeKm
          Double d4 = item2.getItem("endeKm").asUnscaledValue().doubleValue();
          // FahrStreifen
          Array array3 = item2.getItem("gesperrteFahrstreifen").asArray();

          Boolean[] bb = new Boolean[8];
          for (int j = 0; j < bb.length; j++)
          {
            bb[j] = false;
          }

          for (int j = 0; j < array3.getLength(); j++)
          {
            String s = array3.getItem(j).asTextValue().getValueText();
            if (s.equals("HFS"))
              bb[0] = true;
            else if (s.equals("1FS"))
              bb[1] = true;
            else if (s.equals("2FS"))
              bb[2] = true;
            else if (s.equals("3FS"))
              bb[3] = true;
            else if (s.equals("4FS"))
              bb[4] = true;
            else if (s.equals("5FS"))
              bb[5] = true;
            else if (s.equals("6FS"))
              bb[6] = true;
            else if (s.equals("STNDS"))
              bb[7] = true;
          }
          sperrungen[i] = new Sperrung(d3, d4, bb);
        }

        sonderProgrammDatensatz.setSperrungen(sperrungen);
      }

      Array array = data.getItem("Zeitbereich").asArray();
      for (int loc = 0; loc < array.getLength(); loc++)
      {
        Data item2 = array.getItem(loc);
        // Zeitbereich Von
        Object[] os1 = new Object[3];
        os1[0] = item2.getItem("ZeitbereichVon").getItem("abSofort").asTextValue().getValueText().equals("Ja");
        os1[1] = item2.getItem("ZeitbereichVon").getItem("vorlufigNicht").asTextValue().getValueText().equals("Ja");
        os1[2] = item2.getItem("ZeitbereichVon").getItem("von").asTimeValue().getMillis();
        sonderProgrammDatensatz.setZeitBereichVon(os1);
        // Zeitbereich Bis
        Object[] os2 = new Object[3];
        os2[0] = item2.getItem("ZeitbereichBis").getItem("bisJetzt").asTextValue().getValueText().equals("Ja");
        os2[1] = item2.getItem("ZeitbereichBis").getItem("bisAufWeiteres").asTextValue().getValueText().equals("Ja");
        os2[2] = item2.getItem("ZeitbereichBis").getItem("bis").asTimeValue().getMillis();
        sonderProgrammDatensatz.setZeitBereichBis(os2);
        // Stunden
        String s12 = item2.getItem("Stundenauswahl").getItem("Bereiche").asTextValue().getValueText();
        sonderProgrammDatensatz.setStundenAuswahlBereiche(s12);
        Boolean[] bo3 = new Boolean[24];
        Array stunden = item2.getItem("Stundenauswahl").getItem("Stunden").asArray();
        for (int i = 0; i < stunden.getLength(); i++)
        {
          bo3[i] = stunden.getItem(i).asTextValue().getValueText().equals("Ja");
        }
        sonderProgrammDatensatz.setStundenAuswahlStunden(bo3);
        // Wochentage
        String s13 = item2.getItem("Wochentagsauswahl").getItem("Bereiche").asTextValue().getValueText();
        sonderProgrammDatensatz.setWochenTagAuswahlBereiche(s13);
        Boolean[] bo4 = new Boolean[7];
        Array wochentage = item2.getItem("Wochentagsauswahl").getItem("Wochentage").asArray();
        for (int i = 0; i < wochentage.getLength(); i++)
        {
          bo4[i] = wochentage.getItem(i).asTextValue().getText().equals("Ja");
        }
        sonderProgrammDatensatz.setWochenTagAuswahlTage(bo4);
        // Monate
        String s14 = item2.getItem("Monatsauswahl").getItem("Bereiche").asTextValue().getValueText();
        sonderProgrammDatensatz.setMonatsAuswahlBereiche(s14);
        Boolean[] bo5 = new Boolean[12];
        Array monate = item2.getItem("Monatsauswahl").getItem("Monate").asArray();
        for (int i = 0; i < monate.getLength(); i++)
        {
          bo5[i] = monate.getItem(i).asTextValue().getValueText().equals("Ja");
        }
        sonderProgrammDatensatz.setMonatsAuswahlMonate(bo5);

        // Gltigkeit
        Array array4 = data.getItem("Gltigkeit").getItem("Ereignis").asArray();
        SystemObject[] ee = new SystemObject[array4.getLength()];
        for (int i = 0; i < array4.getLength(); i++)
        {
          ee[i] = array4.getItem(i).asReferenceValue().getSystemObject();
        }
        sonderProgrammDatensatz.setEreignisse(ee);
        
        Array array5 = data.getItem("Gltigkeit").getItem("SystemKalenderEintrag").asArray();
        SystemObject[] sk = new SystemObject[array5.getLength()];
        for (int i = 0; i < array5.getLength(); i++)
        {
          sk[i] = array5.getItem(i).asReferenceValue().getSystemObject();
        }
        sonderProgrammDatensatz.setSystemKalenderEintraege(sk);

        // Handschaltung
        Boolean bo = data.getItem("handschaltung").asTextValue().getValueText().equals("Ja");

        sonderProgrammDatensatz.setHandSchaltung(bo);
        // Spursperrung
        Boolean bo03 = ermittleSpurSperrungAusKonfig(connection, text);
        sonderProgrammDatensatz.setSpurSperrung(bo03);
        // Prio Hand
        String p = data.getItem("handschaltungPrioritt").asTextValue().getValueText();
        sonderProgrammDatensatz.setPrioritaetHandschaltung(p);
        // QuerLngasgleich
        Boolean bo00 = data.getItem("QuerLngsabgleich").asTextValue().getValueText().equals("Ja");
        sonderProgrammDatensatz.setQuerLaengsabgleich(bo00);
        // Zusatzpruefung
        Boolean[] bo0 = new Boolean[3];
        bo0[0] = data.getItem("Zusatzprfung").getItem("Ausfallbehandlung").asTextValue().getValueText().equals("Ja");
        bo0[1] = data.getItem("Zusatzprfung").getItem("Verschiebung").asTextValue().getValueText().equals("Ja");
        bo0[2] = data.getItem("Zusatzprfung").getItem("Aufhebung").asTextValue().getValueText().equals("Ja");
        sonderProgrammDatensatz.setZusatzPruefung(bo0);
        // Schaltoptionen
        Boolean[] bo1 = new Boolean[2];
        bo1[0] = data.getItem("Schaltoptionen").getItem("Aufhebung").asTextValue().getValueText().equals("Ja");
        bo1[1] = data.getItem("Schaltoptionen").getItem("Trichter").asTextValue().getValueText().equals("Ja");
        sonderProgrammDatensatz.setSchaltOptionen(bo1);
        // Betriebsart
        Boolean[] bo04 = new Boolean[2];
        bo04[0] = data.getItem("Betriebsart").getItem("Simulation").asTextValue().getValueText().equals("Ja");
        bo04[1] = data.getItem("Betriebsart").getItem("Schaltung").asTextValue().getValueText().equals("Ja");
        sonderProgrammDatensatz.setBetriebsArt(bo04);

        // Schaltbild erzeugen
        if (connection != null)
        {
          AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaEinzelMassnahme");
          Aspect asp = connection.getDataModel().getAspect("asp.parameterSoll");
          ResultData result2 = connection.getData(result.getObject(), new DataDescription(atg, asp), 1500);
          Data data2 = result2.getData();
          if (data2 != null)
          {
            Array aktiv = data2.getItem("Aktiv").asArray();

            for (int cnt = 0; cnt < aktiv.getLength(); cnt++)
            {
              Schaltbild[] ss = new Schaltbild[1];
              Schaltbild sss = new Schaltbild();
              Data data3 = aktiv.getItem(cnt);
              Array array3 = data3.getItem("AnzeigenDatenstze").asArray();
              Anzeige[] aa = new Anzeige[array3.getLength()];
              for (int i = 0; i < array3.getLength(); i++)
              {
                Data data4 = array3.getItem(i);
                Anzeige aaa = new Anzeige();

                SystemObject anzeige = data4.getItem("Anzeige").asReferenceValue().getSystemObject();
                aaa.setAnzeige(anzeige);
                boolean notwendig = data4.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue()
                    .getValueText().equals("Ja");
                aaa.setNotwendig(notwendig);
                boolean absolutNotwendig = data4.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue()
                    .getValueText().equals("Ja");
                aaa.setAbsolutNotwendig(absolutNotwendig);

                // System.out.println("ZustandsDarstellungSonderProgramm.fuelleDatenStruktur(): " + notwendig + " " +
                // absolutNotwendig);

                String s2 = data4.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().getValueText();
                try
                {
                  if (s2.contains(","))
                    s2 = s2.replace(",", ".");

                  Integer prio = new Integer(s2);
                  aaa.setPrioritaet(prio.toString());
                }
                catch (Exception e)
                {
                  aaa.setPrioritaet("undefiniert");
                }
                int vorgaenger = (int)data4.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger")
                    .asUnscaledValue().longValue();
                int nachfolger = (int)data4.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger")
                    .asUnscaledValue().longValue();

                aaa.setVerschiebbarerVorgaenger(vorgaenger);
                aaa.setVerschiebbarerNachfolger(nachfolger);

                Bild bild = new Bild();

                SystemObject anzeigeInhalt = data4.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue()
                    .getSystemObject();
                bild.setAnzeigeInhalt(anzeigeInhalt);

                boolean blinken = data4.getItem("WVZInhalt").getItem("Blinken").asTextValue().getValueText().equals(
                    "Ja");
                aaa.setBlinken(blinken);

                String s3 = data4.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().getValueText();
                try
                {
                  if (s3.contains(","))
                    s3 = s3.replace(",", ".");
                  Double takt = new Double(s3);

                  aaa.setBlinktakt(takt.toString());

                }
                catch (Exception e)
                {
                  aaa.setBlinktakt("Undefiniert");
                }

                SystemObject[] grafikDarstellungen = data4.getItem("WVZInhalt").getItem("GrafikDarstellungen")
                    .asReferenceArray().getSystemObjectArray();
                bild.setGrafikDarstellungen(grafikDarstellungen);

                aaa.setBild(bild);

                DynamischerWechselText dynamischerWechselText = new DynamischerWechselText();

                Data item3 = data4.getItem("WVZInhalt").getItem("DynamischerWechseltext");
                SystemObject font = item3.getItem("Font").asReferenceValue().getSystemObject();
                dynamischerWechselText.setFont(font);

                String strAusrichtung = item3.getItem("TextAusrichtung").asTextValue().getValueText();
                EnumDynamischerWechseltextTextAusrichtung ausrichtung = EnumDynamischerWechseltextTextAusrichtung
                    .valueOf(strAusrichtung);
                dynamischerWechselText.setGewaehlteTextAusrichtung(ausrichtung);

                SystemObject[] textDefinition = item3.getItem("Textdefinition").asReferenceArray()
                    .getSystemObjectArray();
                dynamischerWechselText.setTextDefinition(textDefinition);

                // if (font != null && ausrichtung != null && textDefinition != null)
                if (ausrichtung != null && textDefinition != null)
                  aaa.setDynamischerWechselText(dynamischerWechselText);
                else
                {
//                  System.out.println("Font: " + font);
//                  System.out.println("Ausrichtung: " + ausrichtung);
//                  System.out.println("Text: " + textDefinition);
                  aaa.setDynamischerWechselText(dynamischerWechselText);
                }

                aa[i] = aaa;
              }
              sss.setAnzeigen(aa);
              ss[0] = sss;
              sonderProgrammDatensatz.setSchaltbilder(ss);
            }
          }
        }
      }
      return sonderProgrammDatensatz;
    }
    else
    {
      System.out.println("resultData ist null: " + result.hasData());
    }
    return null;
  }

  private static Boolean ermittleSpurSperrungAusKonfig(ClientDavInterface connection, String text)
  {
    Boolean bo3 = false;
    if (connection != null)
    {
      AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaProgrammTyp");
      Aspect asp = connection.getDataModel().getAspect("asp.parameterSoll");
      IPreferenceStore store = Activator.getDefault().getPreferenceStore();
      String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
      if (pidSba != null && pidSba.length() > 0)
      {
        SystemObject object = connection.getDataModel().getObject(pidSba);
        if (object != null)
        {
          ResultData result1 = connection.getData(object, new DataDescription(atg, asp), 1500);
          if (result1 != null && result1.hasData())
          {
            Data dta = result1.getData();
            Array array2 = dta.getItem("ProgrammTypen").asArray();
            for (int i = 0; i < array2.getLength(); i++)
            {
              String text1 = array2.getItem(i).getItem("Bezeichnung").asTextValue().getText();
              if (text1 != null)
              {
                if (text1.equals(text))
                {
                  String s = array2.getItem(i).getItem("Spursperrung").asTextValue().getText();
                  if (s.equals("Ja"))
                    bo3 = true;
                  break;
                }
              }
            }
          }
        }
      }
    }
    return bo3;
  }

  private boolean versendeLoeschen()
  {
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaManahmeVorschlagAnforderung");

    Data data = _connection.createData(atg);

    SystemObject app = (SystemObject)_connection.getLocalApplicationObject();

    data.getItem("absenderZeichen").asTextValue().setText(app.getName());

    data.getItem("Bezeichnung").asTextValue().setText(_sonderProgrammDatensatz.getBezeichnung());

    String ptyp = "";
    String typBezeichnung = _sonderProgrammDatensatz.getTypBezeichnung();
    if (typBezeichnung != null)
    {
      typBezeichnung = ptyp;
    }
    data.getItem("ProgrammTyp").asTextValue().setText(typBezeichnung);
    data.getItem("Aktion").asTextValue().setText(EnumSonderProgrammAktion.Lschen.toString());

    data.getItem("Fahrtrichtung").asTextValue().setText(EnumSonderProgrammRichtung.NO.toString());
    data.getItem("QuerLngsabgleich").asTextValue().setText("Nein");

    String s1 = "Nein";
    data.getItem("Zusatzprfung").getItem("Ausfallbehandlung").asTextValue().setText(s1);
    data.getItem("Zusatzprfung").getItem("Verschiebung").asTextValue().setText(s1);
    data.getItem("Zusatzprfung").getItem("Aufhebung").asTextValue().setText(s1);

    String s2 = "Nein";
    data.getItem("Schaltoptionen").getItem("Aufhebung").asTextValue().setText(s2);
    data.getItem("Schaltoptionen").getItem("Trichter").asTextValue().setText(s2);

    String s3 = "Nein";
    data.getItem("Betriebsart").getItem("Simulation").asTextValue().setText(s3);
    data.getItem("Betriebsart").getItem("Schaltung").asTextValue().setText(s3);

    // Es wird erst einmal kein Handprogramm definiert.
    String s31 = "Nein";
    data.getItem("Handprogramm").getItem("Handprogramm").asTextValue().setText(s31);
    data.getItem("Handprogramm").getItem("Prioritt").asTextValue().setText("0");

    Aspect asp = _connection.getDataModel().getAspect("asp.sswSbaAnforderung");

    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = _connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        List<ResultData> list = new ArrayList<ResultData>();

        ResultData resultData = new ResultData(object, new DataDescription(atg, asp), new Date().getTime(), data);
        list.add(resultData);

        UrlasserDialog urlasserDialog = new UrlasserDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
        Boolean schlieenOk = (urlasserDialog.anzeigen(list)).getEingabeOk();
      }
    }
    return false;
  }

  public boolean versendeDaten(ClientDavInterface connection, SonderProgramm sonderProgramm, boolean loeschen, boolean kopieren)
  {
    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaManahmeVorschlagAnforderung");
    Aspect asp = connection.getDataModel().getAspect("asp.sswSbaAnforderung");
    Data data = connection.createData(atg);

    schreibeData(connection, sonderProgramm, loeschen, data);
    int len = getCompositeParameterStufen().bestimmeAnzahlStufen();
    schreibeDataStreckenAbschnitt(sonderProgramm, data, len);
    schreibeDataSchaltbild(connection, sonderProgramm, data);

    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        boolean urlasserAnzeigen = false;

        if (sonderProgramm.getGewaehlteSonderProgrammAktion().equals(EnumSonderProgrammAktion.Bilden))
        {
          if (_sonderProgrammDatensatz != null)
          {
            if (_sonderProgrammDatensatz.getBezeichnung().equals(_textName.getText()))
            {
              if (!kopieren){
                MessageBox mb = new MessageBox(Display.getCurrent().getActiveShell(), SWT.ICON_QUESTION | SWT.YES
                    | SWT.NO);
                mb.setText("Sonderprogramm-Bilden");
                mb.setMessage("Soll ein neues Sonderprogramm gebildet werden\n (alte Daten gehen somit verloren)?");
                if (mb.open() == SWT.YES)
                {
                  urlasserAnzeigen = true;
                }
                else
                {

                }
                
              }else{
                urlasserAnzeigen = true;
              }
            }
            else
            {
              urlasserAnzeigen = true;
            }
          }
          else
          {
            urlasserAnzeigen = true;
          }
        }
        else
        {
          urlasserAnzeigen = true;
        }

        if (urlasserAnzeigen)
        {
          List<ResultData> list = new ArrayList<ResultData>();
          ResultData resultData = new ResultData(object, new DataDescription(atg, asp), new Date().getTime(), data);
          list.add(resultData);

          UrlasserDialog urlasserDialog = new UrlasserDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
          Boolean schlieenOk = (urlasserDialog.anzeigen(list)).getEingabeOk();
        }

      }
    }
    return true;

  }

  public static void schreibeData(ClientDavInterface connection, SonderProgramm sonderProgramm, boolean loeschen,
      Data data)
  {
    SystemObject app = (SystemObject)connection.getLocalApplicationObject();

    data.getItem("absenderId").asReferenceValue().setSystemObject(app);
    data.getItem("absenderZeichen").asTextValue().setText(app.getName());
    data.getItem("Bezeichnung").asTextValue().setText(sonderProgramm.getBezeichnung());

    String typBezeichnung = sonderProgramm.getTypBezeichnung();
    data.getItem("ProgrammTyp").asTextValue().setText(typBezeichnung);

    // Gltigkeit Ereignisse
    SystemObject[] ereignisse = sonderProgramm.getEreignisse();
    if (ereignisse != null)
    {
      ReferenceArray refs1 = data.getItem("Gltigkeit").getItem("Ereignis").asReferenceArray();
      refs1.setLength(ereignisse.length);
      for (int i = 0; i < ereignisse.length; i++)
      {
        refs1.getReferenceValue(i).setSystemObject(ereignisse[i]);
      }
    }

    // Gltigkeit Syskals
    SystemObject[] syskals = sonderProgramm.getSystemKalenderEintraege();
    if (syskals != null)
    {
      ReferenceArray refs2 = data.getItem("Gltigkeit").getItem("SystemKalenderEintrag").asReferenceArray();
      refs2.setLength(syskals.length);
      for (int i = 0; i < syskals.length; i++)
      {
        refs2.getReferenceValue(i).setSystemObject(syskals[i]);
      }
    }

    // Streckenabschnitt
    data.getItem("Fahrtrichtung").asTextValue().setText(sonderProgramm.getGewaehlteSonderProgrammRichtung().toString());

    // schreibeDataStreckenAbschnitt(sonderProgramm, data);

    // EnumSonderProgrammAktion enumAktion = sonderProgramm.getGewaehlteSonderProgrammAktion();
    if (!loeschen)
    {
      data.getItem("Aktion").asTextValue().setText(sonderProgramm.getGewaehlteSonderProgrammAktion().toString());
      // data.getItem("Aktion").asTextValue().setText(enumAktion.toString());
    }
    else
    {
      data.getItem("Aktion").asTextValue().setText(EnumSonderProgrammAktion.Lschen.toString());
    }

    Boolean b00 = sonderProgramm.getHandSchaltung();
    String s00 = "Nein";
    if (b00)
      s00 = "Ja";
    data.getItem("Handprogramm").getItem("Handprogramm").asTextValue().setText(s00);

    String p0 = "";
    String p = sonderProgramm.getPrioritaetHandschaltung();
    if (p != null)
      p0 = p;

    data.getItem("Handprogramm").getItem("Prioritt").asTextValue().setText(p0);

    Boolean b0 = sonderProgramm.getQuerLaengsabgleich();
    String s0 = "Nein";
    if (b0)
      s0 = "Ja";
    data.getItem("QuerLngsabgleich").asTextValue().setText(s0);

    Boolean b1 = sonderProgramm.getZusatzPruefung()[0];
    String s1 = "Nein";
    if (b1)
      s1 = "Ja";
    data.getItem("Zusatzprfung").getItem("Ausfallbehandlung").asTextValue().setText(s1);
    b1 = sonderProgramm.getZusatzPruefung()[1];
    s1 = "Nein";
    if (b1)
      s1 = "Ja";
    data.getItem("Zusatzprfung").getItem("Verschiebung").asTextValue().setText(s1);
    b1 = sonderProgramm.getZusatzPruefung()[2];
    s1 = "Nein";
    if (b1)
      s1 = "Ja";
    data.getItem("Zusatzprfung").getItem("Aufhebung").asTextValue().setText(s1);

    data.getItem("Schaltoptionen").getItem("Aufhebung").asTextValue().setText("Nein");
    data.getItem("Schaltoptionen").getItem("Trichter").asTextValue().setText("Nein");

    Boolean b3 = sonderProgramm.getBetriebsArt()[0];
    String s3 = "Nein";
    if (b3)
      s3 = "Ja";
    data.getItem("Betriebsart").getItem("Simulation").asTextValue().setText(s3);
    b3 = sonderProgramm.getBetriebsArt()[1];
    s3 = "Nein";
    if (b3)
      s3 = "Ja";
    data.getItem("Betriebsart").getItem("Schaltung").asTextValue().setText(s3);

    // Das Array wird hier erst einmal mit einem Feldelement voreingestellt.
    Array array = data.getItem("Zeitbereich").asArray();
    array.setLength(1);
    for (int i = 0; i < array.getLength(); i++)
    {
      Data item = array.getItem(i);
      boolean b4 = (Boolean)sonderProgramm.getZeitBereichVon()[0];
      String s4 = "Nein";
      if (b4)
        s4 = "Ja";
      item.getItem("ZeitbereichVon").getItem("abSofort").asTextValue().setText(s4);
      b4 = (Boolean)sonderProgramm.getZeitBereichVon()[1];
      s4 = "Nein";
      if (b4)
        s4 = "Ja";
      item.getItem("ZeitbereichVon").getItem("vorlufigNicht").asTextValue().setText(s4);
      item.getItem("ZeitbereichVon").getItem("von").asTimeValue()
          .setMillis((Long)sonderProgramm.getZeitBereichVon()[2]);

      boolean b5 = (Boolean)sonderProgramm.getZeitBereichBis()[0];
      String s5 = "Nein";
      if (b5)
        s5 = "Ja";
      item.getItem("ZeitbereichBis").getItem("bisJetzt").asTextValue().setText(s5);
      b5 = (Boolean)sonderProgramm.getZeitBereichBis()[1];
      s5 = "Nein";
      if (b5)
        s5 = "Ja";
      item.getItem("ZeitbereichBis").getItem("bisAufWeiteres").asTextValue().setText(s5);
      item.getItem("ZeitbereichBis").getItem("bis").asTimeValue()
          .setMillis((Long)sonderProgramm.getZeitBereichBis()[2]);

      item.getItem("Stundenauswahl").getItem("Bereiche").asTextValue().setText(
          sonderProgramm.getStundenAuswahlBereiche());
      Boolean[] bools1 = sonderProgramm.getStundenAuswahlStunden();
      Array array1 = item.getItem("Stundenauswahl").getItem("Stunden").asArray();
      array1.setLength(24);
      for (int j = 0; j < array1.getLength(); j++)
      {
        if (bools1[j])
          array1.getItem(j).asTextValue().setText("Ja");
        else
          array1.getItem(j).asTextValue().setText("Nein");
      }

      item.getItem("Wochentagsauswahl").getItem("Bereiche").asTextValue().setText(
          sonderProgramm.getWochenTagAuswahlBereiche());
      Boolean[] bools2 = sonderProgramm.getWochenTagAuswahlTage();
      Array array2 = item.getItem("Wochentagsauswahl").getItem("Wochentage").asArray();
      array2.setLength(7);
      for (int j = 0; j < bools2.length; j++)
      {
        if (bools2[j])
          array2.getItem(j).asTextValue().setText("Ja");
        else
          array2.getItem(j).asTextValue().setText("Nein");
      }

      item.getItem("Monatsauswahl").getItem("Bereiche").asTextValue()
          .setText(sonderProgramm.getMonatsAuswahlBereiche());
      Boolean[] bools3 = sonderProgramm.getMonatsAuswahlMonate();
      Array array3 = item.getItem("Monatsauswahl").getItem("Monate").asArray();
      array3.setLength(12);
      for (int j = 0; j < bools3.length; j++)
      {
        if (bools3[j])
          array3.getItem(j).asTextValue().setText("Ja");
        else
          array3.getItem(j).asTextValue().setText("Nein");
      }
    }

    // if (sonderProgramm.getGewaehlteSonderProgrammAktion().equals(EnumSonderProgrammAktion.ndern))
    // {
    // schreibeDataSchaltbild(connection, sonderProgramm, data);
    // }
    // else
    {
      Array array4 = data.getItem("AnzeigenDatenstze").asArray();
      array4.setLength(0);
    }
  }

  public void schreibeDataSchaltbild(ClientDavInterface connection, SonderProgramm sonderProgramm, Data data)
  {
    holeSchaltbildAusDarstellung(connection, sonderProgramm);
    // Schaltbilder
    schreibeDataSchaltbild(sonderProgramm, data);
  }

  public void holeSchaltbildAusDarstellung(ClientDavInterface connection, SonderProgramm sonderProgramm)
  {
    List<AnzeigeQuerschnittModelTypAbstract> aqAuswahl = holeAqModelObjekteVonAqAuswahl();

    Schaltbild[] ss = new Schaltbild[aqAuswahl.size()];

    for (int i = 0; i < aqAuswahl.size(); i++)
    {
      AnzeigeQuerschnittModelTypAbstract aq = aqAuswahl.get(i);
      SystemObject so = aq.getSystemObject();
      Schaltbild sss = new Schaltbild();

      EList<AnzeigeModelTypAbstract> elemente = aq.getKomponenten();

      ArrayList<AnzeigeModelTypAbstract> aes1 = new ArrayList<AnzeigeModelTypAbstract>();
      for (int j = 0; j < elemente.size(); j++)
      {
        AnzeigeModelTypAbstract ae = (AnzeigeModelTypAbstract)elemente.get(j);
        Object feature = ae.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);

        if (feature instanceof AnzeigeInhaltTyp)
        {
          AnzeigeInhaltTyp inhalt = (AnzeigeInhaltTyp)feature;
          if (!inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
          {
            aes1.add(ae);
          }
        }
        else if (feature instanceof VisuAnzeigeKapsel)
        {
          VisuAnzeigeKapsel inhalt = (VisuAnzeigeKapsel)feature;
          // if (!inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
          {
            aes1.add(ae);
          }
        }
      }

      Anzeige[] aa = new Anzeige[aes1.size()];
      for (int j = 0; j < aes1.size(); j++)
      {
        Anzeige aaa = new Anzeige();
        AnzeigeModelTypAbstract ae = (AnzeigeModelTypAbstract)aes1.get(j);

        SystemObject anzeige = ae.getSystemObject();
        aaa.setAnzeige(anzeige);
        aaa.setNotwendig(ae.getNotwendig() > 0 ? true : false);
        aaa.setAbsolutNotwendig(ae.getAbsolutNotwendig() > 0 ? true : false);
        aaa.setPrioritaet(ae.getPrioritaet());
        aaa.setGewaehlterAnzeigeProgrammStatus(EnumAnzeigeProgrammStatus.Sonderprogramm);
        aaa.setQuelle(null);

        Bild bild = new Bild();

        Object feature = ae.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);

        DynamischerWechselText dynamischerWechselText = new DynamischerWechselText();
        dynamischerWechselText.setFont(null);
        dynamischerWechselText.setGewaehlteTextAusrichtung(EnumDynamischerWechseltextTextAusrichtung.Linksbndig);
        dynamischerWechselText.setTextDefinition(null);

        if (feature instanceof ResultData)
        {
          ResultData result = (ResultData)feature;
          AttributeGroup atg2 = result.getDataDescription().getAttributeGroup();

          if (atg2.getPid().equals("atg.anzeigeEigenschaftIst") || atg2.getPid().equals("atg.anzeigeEigenschaftSoll"))
          {
            Data data2 = result.getData();
            if (data2 != null)
            {

              SystemObject anzeigeInhalt = data2.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue()
                  .getSystemObject();
              bild.setAnzeigeInhalt(anzeigeInhalt);
            }
          }

        }
        else if (feature instanceof AnzeigeInhaltTyp)
        {
          AnzeigeInhaltTyp inhalt = (AnzeigeInhaltTyp)feature;
          SystemObject anzeigeInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(connection, anzeige, inhalt.getType());
          bild.setAnzeigeInhalt(anzeigeInhalt);
        }
        else if (feature instanceof VisuAnzeigeKapsel)
        {
          VisuAnzeigeKapsel kapsel = (VisuAnzeigeKapsel)feature;
          String pidFont = kapsel.getSchriftart();
          SystemObject font = connection.getDataModel().getObject(pidFont);

          String ausrichtung = kapsel.getAusrichtung();

          String[] pidsZeichen = kapsel.getPidsZeichen();

          ArrayList<SystemObject> list = new ArrayList<SystemObject>();

          for (String pidZeichen : pidsZeichen)
          {
            SystemObject zeichen = connection.getDataModel().getObject(pidZeichen);
            list.add(zeichen);
          }

          dynamischerWechselText.setFont(font);

          EnumDynamischerWechseltextTextAusrichtung enumWta = EnumDynamischerWechseltextTextAusrichtung
              .valueOf(ausrichtung);

          dynamischerWechselText.setGewaehlteTextAusrichtung(enumWta);

          dynamischerWechselText.setTextDefinition(list.toArray(new SystemObject[list.size()]));
        }

        String prio = ae.getPrioritaet();
        aaa.setPrioritaet(prio);
        aaa.setNotwendig(ae.getNotwendig() > 0 ? true : false);
        aaa.setAbsolutNotwendig(ae.getAbsolutNotwendig() > 0 ? true : false);
        aaa.setBlinken(ae.getBlinken() > 0 ? true : false);
        aaa.setBlinktakt(ae.getBlinktakt());
        aaa.setVerschiebbarerVorgaenger(ae.getVerschiebungVorgaenger());
        aaa.setVerschiebbarerNachfolger(ae.getVerschiebungNachfolger());

        bild.setGrafikDarstellungen(null);
        aaa.setBild(bild);

        aaa.setDynamischerWechselText(dynamischerWechselText);

        aa[j] = aaa;
      }

      if (aes1.size() > 0)
      {
        sss.setAnzeigeQuerschnitt(so);
        sss.setAnzeigen(aa);
        ss[i] = sss;
      }
    }

    List<Schaltbild> ss1 = new ArrayList<Schaltbild>();

    for (int i = 0; i < ss.length; i++)
    {
      Schaltbild schaltbild = ss[i];
      if (schaltbild != null)
        ss1.add(schaltbild);

    }

    Schaltbild[] ss2 = new Schaltbild[ss1.size()];
    Schaltbild[] ss3 = ss1.toArray(ss2);
    sonderProgramm.setSchaltbilder(ss3);
  }

  public static void schreibeDataSchaltbild(SonderProgramm sonderProgramm, Data data)
  {
    Schaltbild[] schaltbilder = sonderProgramm.getSchaltbilder();

    if (schaltbilder == null)
      return;
    
    int cntAnzeigen = 0;
    for (int i = 0; i < schaltbilder.length; i++)
    {
      Schaltbild schaltbild = schaltbilder[i];
      Anzeige[] anzeigen = schaltbild.getAnzeigen();
      for (int k = 0; k < anzeigen.length; k++)
      {
        Anzeige anzeige = anzeigen[k];
        cntAnzeigen++;
      }
    }

    Array array4 = data.getItem("AnzeigenDatenstze").asArray();
    array4.setLength(cntAnzeigen);

    int loci = 0;
    for (int i = 0; i < schaltbilder.length; i++)
    {
      Schaltbild schaltbild = schaltbilder[i];

      // if (schaltbild == null)
      // continue;

      Data item = array4.getItem(i);
      Anzeige[] anzeigen = schaltbild.getAnzeigen();
      for (int j = 0; j < anzeigen.length; j++)
      {
        Anzeige anzeige = anzeigen[j];

        Data item2 = array4.getItem(loci);
        item2.getItem("Anzeige").asReferenceValue().setSystemObject(anzeige.getAnzeige());

        SystemObject anzeigeInhalt = anzeige.getBild().getAnzeigeInhalt();

        item2.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(anzeigeInhalt);
        item2.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().setText(anzeige.getPrioritaet());
        item2.getItem("WVZInhalt").getItem("Blinken").asTextValue().setText(anzeige.isBlinken() ? "Ja" : "Nein");

        String blinktakt = anzeige.getBlinktakt();

        item2.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText(blinktakt);
        item2.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().setText(
            anzeige.isNotwendig() ? "Ja" : "Nein");
        item2.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText(
            anzeige.isAbsolutNotwendig() ? "Ja" : "Nein");
        item2.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asUnscaledValue().set(
            anzeige.getVerschiebbarerVorgaenger());
        item2.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asUnscaledValue().set(
            anzeige.getVerschiebbarerNachfolger());

        DynamischerWechselText dwt = anzeige.getDynamischerWechselText();

        if (dwt != null)
        {
          EnumDynamischerWechseltextTextAusrichtung enumDWT = dwt.getGewaehlteTextAusrichtung();
          if (enumDWT != null)
          {
            item2.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung").asTextValue()
                .setText(enumDWT.toString());
          }
          else
          {
            item2.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung").asTextValue()
                .setText(EnumDynamischerWechseltextTextAusrichtung.Linksbndig.toString());
          }
          SystemObject font = dwt.getFont();
          if (font != null)
            item2.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("Font").asReferenceValue()
                .setSystemObject(font);

          SystemObject[] textDefinition = dwt.getTextDefinition();
          if (textDefinition != null)
          {
            ReferenceArray refs = item2.getItem("WVZInhalt").getItem("DynamischerWechseltext")
                .getItem("Textdefinition").asReferenceArray();
            refs.setLength(dwt.getTextDefinition().length);
            for (int k = 0; k < refs.getLength(); k++)
            {
              ReferenceValue ref = refs.getReferenceValue(k);
              ref.setSystemObject(textDefinition[k]);
            }
          }
        }
        loci++;
      }
    }
  }

  /**
   * @param sonderProgramm
   * @param data
   * @param len
   */
  public static void schreibeDataStreckenAbschnitt(SonderProgramm sonderProgramm, Data data, int len)
  {
    Double d1 = sonderProgramm.getAuswahlVonKm();
    long l1 = d1.longValue();
    Double d2 = sonderProgramm.getAuswahlBisKm();
    long l2 = d2.longValue();

    Array array0 = data.getItem("Streckenabschnitt").asArray();
    array0.setLength(1);

    array0.getItem(0).getItem("startKm").asUnscaledValue().set(l1);
    array0.getItem(0).getItem("endeKm").asUnscaledValue().set(l2);

    Array array01 = array0.getItem(0).getItem("FahrstreifenSperrung").asArray();
    Boolean bx = sonderProgramm.isSpurSperrung();

    Sperrung[] sperrungen = sonderProgramm.getSperrungen();

    array01.setLength(len);

    for (int i = 0; i < len; i++)
    {
      Sperrung sperrung = sperrungen[i];
      Double d3 = sperrung.getSperrungVonKm();
      long l3 = d3.longValue();
      Double d4 = sperrung.getSperrungBisKm();
      long l4 = d4.longValue();

      array01.getItem(i).getItem("startKm").asUnscaledValue().set(l3);
      array01.getItem(i).getItem("endeKm").asUnscaledValue().set(l4);

      Boolean[] bb = sperrung.getSperrungFahrspur();

      Array array02 = array01.getItem(i).getItem("gesperrteFahrstreifen").asArray();
      if (bb != null && bb.length > 0)
      {
        List<String> list = new ArrayList<String>();

        for (int j = 0; j < bb.length; j++)
        {
          if (bb[j])
          {
            switch (j)
            {
              case 0:
                list.add("HFS");
                break;
              case 1:
              case 2:
              case 3:
              case 4:
              case 5:
              case 6:
                list.add(j + "FS");
                break;
              case 7:
                list.add("STNDS");
                break;
              default:
            }
          }

        }
        array02.setLength(list.size());
        for (int j = 0; j < list.size(); j++)
        {
          array02.getItem(j).asTextValue().setText(list.get(j));
        }
      }
      else
      {
        array02.setLength(0);
      }

    }
    if (bx != null && bx)
    {

    }
    else
    {
      // array01.setLength(0);
    }
  }

  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    ResultData result = e.getResult();
    if (result != null)
      verarbeiteDaVDaten(result);
  }

  private void verarbeiteDaVDaten(ResultData result)
  {

    AttributeGroup atg = result.getDataDescription().getAttributeGroup();
    Aspect asp = result.getDataDescription().getAspect();
//    String prop = "";
//    if (_secondaryId.equals("NO"))
//      prop = PROP_SONDER_PROGRAMM_DATA_NO;
//    else if (_secondaryId.equals("SW"))
//      prop = PROP_SONDER_PROGRAMM_DATA_SW;

    if (atg.getPid().equals("atg.sswSbaSonderprogramm"))
    {
      Data data = result.getData();
      if (data != null)
      {
        String fr = data.getItem("Fahrtrichtung").asTextValue().getText();

        String frRef = "";

        if (_secondaryId.equals("SW"))
          frRef = "SW";
        else if (_secondaryId.equals("NO"))
          frRef = "NO";

        if (fr.equals(frRef))
        {
          if (_sonderProgrammMap.containsKey(result.getObject().getPid()))
          {
            _sonderProgrammMap.remove(result.getObject().getPid());
          }
          
          _sonderProgrammMap.put(result.getObject().getPid(), result);

          ArrayList<ResultData> list = new ArrayList<ResultData>();
          Collection<ResultData> values = _sonderProgrammMap.values();

          for (ResultData resultData : values)
          {
            Data dta = resultData.getData();
            if (dta != null)
            {
              String name = dta.getItem("nameAnwender").asTextValue().getText();            
              if (name != null && name.length() > 0)
                list.add(resultData);
            }
          }
          IWorkbenchPage page = getSite().getPage();
          if (page != null)
          {
            IViewReference ref = page.findViewReference(AuswahlSonderProgrammView.ID);

            if (ref != null)
            {
              IViewPart view = ref.getView(true);
              AuswahlSonderProgrammView auswahl = (AuswahlSonderProgrammView)view;
              auswahl.setContent(this, true, result.getObject().getPid());
            }
          }
          
//          benachrichtigeListener(prop, null, result.getObject());
        }
      }
    }
    else if (atg.getPid().equals("atg.sswSbaManahmeVorschlagAntwort"))
    {
      Data data = result.getData();
      
      
//      System.out.println("verarbeiteDaVDaten(atg.sswSbaManahmeVorschlagAntwort): " + result);

      if (data != null)
      {
        SystemObject so = data.getItem("Sonderprogramm").asReferenceValue().getSystemObject();
         
        String pid = null;
        if (so != null)
        { 
            pid = so.getPid();
        }
        String status = data.getItem("Status").asTextValue().getText();
        final String fehlerArt = data.getItem("Fehler").getItem("Art").asTextValue().getText();
        String fehlerBeschreibung = data.getItem("Fehler").getItem("Beschreibung").asTextValue().getText();

        if (status.equals("gelscht") && pid != null)
        {
          if (_sonderProgrammMap.containsKey(pid))
          {
            ResultData rd = _sonderProgrammMap.get(pid);
            SystemObject refObj = rd.getObject();
            if (refObj.getId() == so.getId()){
              _sonderProgrammMap.remove(pid);
                            
//              System.out.println("!!!gelscht: " + pid);
              
              IWorkbenchPage page = getSite().getPage();
              if (page != null)
              {
                IViewReference ref = page.findViewReference(AuswahlSonderProgrammView.ID);
                
                if (ref != null)
                {
                  IViewPart view = ref.getView(true);
                  AuswahlSonderProgrammView auswahl = (AuswahlSonderProgrammView)view;
                  aktualisiereSichtKeineSelektion();
                  auswahl.setContent(this, true, result.getObject().getPid());
                }
              }            
//              benachrichtigeListener(prop, null, so);
//              aktualisiereSichtKeineSelektion();              
            }
            
            
          }else{
//            System.out.println("------------> nicht vorhanden!!!: " + pid);
          }
        }
      }
    }
    else if (atg.getPid().equals("atg.sswSbaSchaltbildAnzeige"))
    {
      if (asp.getPid().equals("asp.sswSbaBefehle"))
      {
        DaVHelper.verarbeiteDaVDatenSswSbaSchaltbild(_connection, getGraphicalViewer(), result, 1);
      }
      else if (asp.getPid().equals("asp.sswSbaSonderprogrammSimu"))
      {
        DaVHelper.verarbeiteDaVDatenSswSbaSchaltbild(_connection, getGraphicalViewer(), result, 3);
      }
    }
    else if (atg.getPid().equals("atg.strfallZustand"))
    {
      SystemObject so = result.getObject();
      FahrStreifenModelAbstract dom = DaVHelper.holeFahrStreifenModelTypVonSystemObject(
          (AnsichtEditPart)graphicalViewer.getContents(), so);

      if (dom != null)
      {
        EStructuralFeature feature = AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__DATA_UPDATE;
        final SetCommand cmd = new SetCommand(dom, feature, result);
        getCommandStack().execute(cmd);
      }
    }
    else if (atg.getPid().equals("atg.tlsGloDeFehler"))
    {
      SystemObject so = result.getObject();

      DetektorModelTypAbstract dom = DaVHelper.holeDetektorModelTypAbstractVonSystemObject(
          (AnsichtEditPart)graphicalViewer.getContents(), so);

      if (dom != null)
      {
        EStructuralFeature feature = AnlagendarstellungPackage.Literals.DETEKTOR_MODEL_TYP_ABSTRACT__DATA_UPDATE;
        final SetCommand cmd = new SetCommand(dom, feature, result);
        getCommandStack().execute(cmd);
      }
    }
    else if (result.getObject().isOfType("typ.sswSbaAnzeigeQuerschnitt"))
    {
      String strAtgHandeingriffBetriebsart = "atg.sswSbaHandeingriffBetriebsart";
      String strAtgHelligkeitProzent = "atg.sswSbaHelligkeitProzentErgebnis";

      AnsichtEditPart contents = (AnsichtEditPart)getGraphicalViewer().getContents();

      SystemObject so = result.getObject();
//      AttributeGroup atg2 = _connection.getDataModel().getAttributeGroup("atg.sswSbaAnzeigeQuerschnitt");
//      Data cd = so.getConfigurationData(atg2);
//      if (cd != null)
//      {
//        SystemObject so2 = cd.getItem("AnzeigeQuerschnitt").asReferenceValue().getSystemObject();
      
        SystemObject so2 = DaVHelper.holeSystemObjectAqVonSswSbaAq(_connection, so);

        if (so2 != null){
          AnzeigeQuerschnittModelTypAbstract dom = DaVHelper.holeAnzeigeQuerschnittModelTypAbstractVonSystemObject(
              contents, so2, 1);
          
          if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
          {
            AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)dom;
            if (aq.getKontext() == 1)
            {
              if (result.getDataDescription().getAttributeGroup().getPid().equals(strAtgHandeingriffBetriebsart)
                  || result.getDataDescription().getAttributeGroup().getPid().equals(strAtgHelligkeitProzent))
              {
                DaVHelper.verarbeiteDaVDatenSswSbaAq(_connection, getGraphicalViewer(), result, 1);
              }
            }
          }          
        }
//      }
    }

    if (atg.getPid().equals("atg.sswSbaWanderbaustelle"))
    {
      SystemObject so = result.getObject();

      atg = so.getDataModel().getAttributeGroup("atg.sswSbaStreckenBeeinflussungsAnlageRichtung");
      Data cd = so.getConfigurationData(atg);
      String fr = cd.getItem("Fahrtrichtung").asTextValue().getValueText();

      if (fr.equals(_secondaryId))
      {
        if (result.hasData())
        {
          Data data = result.getData();
          Array array = data.getItem("Wanderbaustelle").asArray();

          for (int i = 0; i < array.getLength(); i++)
          {
            String name = array.getItem(i).getItem("Name").asTextValue().getValueText();
            SystemObject[] sps = array.getItem(i).getItem("Sonderprogramme").asReferenceArray().getSystemObjectArray();

            List<ResultData> list = new ArrayList<ResultData>();

            for (SystemObject sp : sps)
            {
              if (_sonderProgrammMap.containsKey(sp.getPid()))
              {
                ResultData resultSp = _sonderProgrammMap.get(sp.getPid());
                if (resultSp != null && !list.contains(resultSp))
                {
                  list.add(resultSp);
                }
              }
            }

            if (_mapWanderBaustellen.containsKey(name))
            {
              _mapWanderBaustellen.remove(name);
            }

            _mapWanderBaustellen.put(name, list);

            IWorkbenchPage page = getSite().getPage();
            if (page != null)
            {
              IViewReference ref = page.findViewReference(AuswahlSonderProgrammView.ID);

              if (ref != null)
              {
                IViewPart view = ref.getView(true);
                AuswahlSonderProgrammView auswahl = (AuswahlSonderProgrammView)view;
                auswahl.setContent(this, true, result.getObject().getPid());
              }
            }
          }
        }
      }
    }

    if (atg.getPid().equals("atg.sswSbaSonderprogrammHierarchie"))
    {
      SystemObject so = result.getObject();

      atg = so.getDataModel().getAttributeGroup("atg.sswSbaStreckenBeeinflussungsAnlageRichtung");
      Data cd = so.getConfigurationData(atg);
      String fr = cd.getItem("Fahrtrichtung").asTextValue().getValueText();

      if (fr.equals(_secondaryId))
      {
        if (result.hasData())
        {
          Data data = result.getData();
          Array array = data.getItem("Men").asArray();

          for (int i = 0; i < array.getLength(); i++)
          {
            String name = array.getItem(i).getItem("Name").asTextValue().getValueText();
            SystemObject[] sps = array.getItem(i).getItem("Sonderprogramme").asReferenceArray().getSystemObjectArray();

            List<ResultData> list1 = new ArrayList<ResultData>();

            for (SystemObject sp : sps)
            {
              if (_sonderProgrammMap.containsKey(sp.getPid()))
              {
                ResultData resultSp = _sonderProgrammMap.get(sp.getPid());
                if (resultSp != null && !list1.contains(resultSp))
                {
                  list1.add(resultSp);
                }
              }
            }

            if (_mapProgrammHierarchie.containsKey(name))
            {
              _mapProgrammHierarchie.remove(name);
            }

            _mapProgrammHierarchie.put(name, list1);

            IWorkbenchPage page = getSite().getPage();
            if (page != null)
            {
              IViewReference ref = page.findViewReference(AuswahlSonderProgrammView.ID);

              if (ref != null)
              {
                IViewPart view = ref.getView(true);
                AuswahlSonderProgrammView auswahl = (AuswahlSonderProgrammView)view;
                auswahl.setContent(this, true, result.getObject().getPid());
              }
            }
          }
        }
      }
    }
    else if (atg.getPid().equals("atg.situationsEigenschaften"))
    {
//      System.out.println("SituationsUpdate: " + result);
      
      if (result != null){
        SituationUtil.verarbeiteSituationUpdate(getGraphicalViewer(), result);
      }
        
       
    }
    // else
    // {
    // DaVHelper.verarbeiteDaVDaten(_connection, getGraphicalViewer(), result, 1);
    // }
  }

  private List<AnzeigeQuerschnittModelTypAbstract> holeAqModelObjekteVonAqAuswahl()
  {
    List<AnzeigeQuerschnittModelTypAbstract> list = new ArrayList<AnzeigeQuerschnittModelTypAbstract>();

    AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    darstellung.setSpaltenBorder(true);

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {

      EList<DoModel> doObjekte = ebene.getDoObjekte();
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {

          AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)dom;
          int aqKontext = aq.getKontext();
          if (aqKontext == 4)
          {
            list.add(aq);
          }
        }
      }
    }
    return list;
  }

  public SonderProgramm getSonderProgrammDatensatz()
  {
    return _sonderProgrammDatensatz;
  }

  public void setSonderProgrammDatensatz(SonderProgramm sonderProgrammDatensatz)
  {
    _sonderProgrammDatensatz = sonderProgrammDatensatz;
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = graphicalViewer.getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen bercksichtigt werden. Dies ist fr ein
   * Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  public void leseActionSets(String id)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg.getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String contributor = confEle.getContributor().getName();
      if (Activator.PLUGIN_ID.equals(contributor))
      {
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {
          String att = element.getAttribute("id");
          if (att.equals(id))
          {
            boolean pullDownStyle = false;
            String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
            if (style != null)
            {
              pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
            }
            else
            {
              String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
              pullDownStyle = pulldown != null && pulldown.equals("true");
            }

            ActionDescriptor desc = null;
            if (pullDownStyle)
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
            }
            else
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
            }
            PluginAction action = desc.getAction();
//            Event event = new Event();
////            event.data = holeDataAusTreeSelection();            
//            event.data = ueberpruefeErfassteDaten();            
//            action.runWithEvent(event);
            _kontextMenuMgr.add(action);
          }
        }
      }
    }
  }

  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen bercksichtigt werden. Dies ist fr ein
   * Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  public void leseActions(String id)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg.getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String contributor = confEle.getContributor().getName();
      if (Activator.PLUGIN_ID.equals(contributor))
      {
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {
          String att = element.getAttribute("id");
          if (att.equals(id))
          {
            boolean pullDownStyle = false;
            String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
            if (style != null)
            {
              pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
            }
            else
            {
              String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
              pullDownStyle = pulldown != null && pulldown.equals("true");
            }

            ActionDescriptor desc = null;
            if (pullDownStyle)
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
            }
            else
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
            }
            PluginAction action = desc.getAction();
            action.run();
          }
        }
      }
    }
  }

  /**
   * Liefert das KontextMenue zum Navigator
   */
  private void erzeugeKontextMenue()
  {
    _kontextMenuMgr = new MenuManager("#SonderProgrammMenu");
    _kontextMenuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager)
      {
        IStructuredSelection ss = (IStructuredSelection)graphicalViewer.getSelection();

        if (!ss.isEmpty())
        {
          final Object obj = ss.getFirstElement();

          _kontextMenuMgr.removeAll();
          aktionKontextMenue(obj);
//          _kontextMenuMgr.add(new Action("Einstellung Anzeigen"){
//            
//            public void run() {
//              
//              aktionKontextMenue(obj);
//              
//            };
//          });
        }
      }
    });

    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
    Menu menu = _kontextMenuMgr.createContextMenu(graphicalViewer.getControl());
    // Das Men dem Baum bekannt machen
    graphicalViewer.getControl().setMenu(menu);
    getSite().registerContextMenu(_kontextMenuMgr, graphicalViewer);
  }

  /**
   * Adaptable implementation for Editor
   */
  public Object getAdapter(Class adapter)
  {
    // we need to handle common GEF elements we created
    if (adapter == GraphicalViewer.class || adapter == EditPartViewer.class)
      return getGraphicalViewer();
    else if (adapter == CommandStack.class)
      return getCommandStack();
    else if (adapter == EditDomain.class)
      return getEditDomain();
    else if (adapter == ActionRegistry.class)
      return getActionRegistry();
    if (adapter == ILegende.class) {
      return new Legende();
    }

    // the super implementation handles the rest
    return super.getAdapter(adapter);
  }

  @Override
  protected void initializeGraphicalViewer()
  {
    // TODO Auto-generated method stub

  }

  public GraphicalViewer getGraphicalViewer()
  {
    return graphicalViewer;
  }

  public void setGraphicalViewer(GraphicalViewer graphicalViewer)
  {
    this.graphicalViewer = graphicalViewer;
  }

  public Map<String, ResultData> getSonderProgrammMap()
  {
    return _sonderProgrammMap;
  }

  public void setzteAlleAnzeigenMitKontext(AnsichtEditPart contents, AnzeigeInhaltTyp inhalt, int kontext)
  {
    if (contents == null || inhalt == null)
      throw new IllegalArgumentException("Parameter ist null: " + contents + " - " + inhalt);

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    darstellung.setSpaltenBorder(true);

    List<SystemObject> list = new ArrayList<SystemObject>();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {

      EList<DoModel> doObjekte = ebene.getDoObjekte();
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {

          AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)dom;
          int aqKontext = aq.getKontext();
          if (kontext == aqKontext)
          {

            EList<AnzeigeModelTypAbstract> anzeigen = aq.getKomponenten();
            for (AnzeigeModelTypAbstract a : anzeigen)
            {
              a.setPrioritaet("undefiniert");
              a.setNotwendig(0);
              a.setAbsolutNotwendig(0);
              a.setBlinken(0);
              a.setBlinktakt("Undefiniert");
              a.setVerschiebungNachfolger(0);
              a.setVerschiebungVorgaenger(0);

              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              final SetCommand cmd = new SetCommand(a, feature, inhalt);
              getCommandStack().execute(cmd);
            }
          }
        }
      }
    }
  }

  public static void markiereLabelAq(ClientDavInterface connection, String secondaryId, Text text, Label label)
  {
    if (connection != null)
    {
      AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");

      IPreferenceStore store = Activator.getDefault().getPreferenceStore();

      String pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO;
      String pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO;

      if (secondaryId.equals("SW"))
      {
        pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW;
        pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW;
      }

      double d3 = store.getDouble(pref1);
      double d4 = store.getDouble(pref2);

      boolean absteigend = DaVHelper.ermittleKilometrierungAufsteigendAbsteigend(secondaryId);

      Collection<SystemObject> values = DaVHelper.holeListeKilometrierungAq(secondaryId, absteigend, d3, d4).values();

      String name = null;

      for (SystemObject so : values)
      {
        Data cd = so.getConfigurationData(atg);
        double dv = cd.getItem("Wert").asUnscaledValue().doubleValue();
        if (dv > 0.0)
          dv /= 1000.0;

        String s = new Formatter().format("%1$7.3f", dv).toString();
        if (text.getText().equals(s))
        {
          name = so.getName();
          break;
        }

      }
      if (name != null)
      {
        label.setText(name);
        label.setToolTipText(name);
      }
      else
      {
        label.setText("keine Ausw.");
        label.setToolTipText("");
      }
    }
  }

  public static class ListLabelProvider implements ILabelProvider
  {
    private static ListLabelProvider instance;

    public static ListLabelProvider getInstance()
    {
      synchronized (ListLabelProvider.class)
      {
        if (instance == null)
        {
          instance = new ListLabelProvider();
        }
        return instance;
      }
    }

    @Override
    public Image getImage(Object element)
    {
      return null;
    }

    @Override
    public String getText(Object element)
    {
      return ((SystemObject)element).getName();
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }
  }

  /**
   * Registriert die Actions zum Aufruf der Outline-View
   */
  private void registriereActions()
  {
    IMenuManager menuManager = this.getViewSite().getActionBars().getMenuManager();
    IToolBarManager toolBarManager = this.getViewSite().getActionBars().getToolBarManager();

    Action oeffneAction = new Action() {
      public void run()
      {
        oeffneAuswahlSonderProgrammView();
      }

    };
    oeffneAction.setText("ffne SonderProgramm-Auswahl");

    ImageDescriptor id = ImageDescriptor.createFromImage(ImageFactory.getImage("icons/prop_ps.png"));

    oeffneAction.setImageDescriptor(id);
    menuManager.add(oeffneAction);
    toolBarManager.add(oeffneAction);

    Action oeffneAction2 = new Action() {
      public void run()
      {
        oeffneAuswahlSpaltenView();
      }

    };
    oeffneAction2.setText("ffne Spalten-Auswahl");

    ImageDescriptor id2 = ImageDescriptor.createFromImage(ImageFactory.getImage("icons/th_horizontal.png"));
    oeffneAction2.setImageDescriptor(id2);
    menuManager.add(oeffneAction2);
    toolBarManager.add(oeffneAction2);
    
    IAction aktionLegende = holeAction("de.bsvrz.buv.rw.bitctrl.eclipse.legende.oeffneLegende");
    if (aktionLegende != null){
      menuManager.add(aktionLegende);
      toolBarManager.add(aktionLegende);            
    }

  }

  private void oeffneAuswahlSpaltenView()
  {
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    try
    {
      page.showView(AuswahlSpaltenView.ID);
      IViewReference ref = page.findViewReference(AuswahlSpaltenView.ID);
      if (ref != null)
      {
        AuswahlSpaltenView view = (AuswahlSpaltenView)ref.getView(true);
        if (view != null)
        {
          view.setContent((AnsichtEditPart)graphicalViewer.getContents());
        }
      }
    }
    catch (PartInitException e)
    {
      e.printStackTrace();
    }
  }

  private void oeffneAuswahlSonderProgrammView()
  {
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    try
    {
      page.showView(AuswahlSonderProgrammView.ID);
      IViewReference ref = page.findViewReference(AuswahlSonderProgrammView.ID);
      if (ref != null)
      {
        AuswahlSonderProgrammView view = (AuswahlSonderProgrammView)ref.getView(true);
        if (view != null)
        {
          view.setContent(this, false);
        }
      }
    }
    catch (PartInitException e)
    {
      e.printStackTrace();
    }
  }

  private void aktionStartKm()
  {
    SystemObject so = DaVHelper.aktionListeAuswahlAq(_connection, _secondaryId);
    if (_connection != null && so != null)
    {

      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
      Data cd = so.getConfigurationData(atg);
      double dv = cd.getItem("Wert").asUnscaledValue().doubleValue();
      if (dv > 0.0)
        dv /= 1000.0;

      if (so.getPid().equals("sba.aq.A8.6130.HFB.W"))
      {
        dv = 208.43;
      }

      String s = new Formatter().format("%1$7.3f", dv).toString();

      if (DaVHelper.validiereTextInput(_secondaryId, s, textStopKm.getText()))
      {
        textStartKm.setText(s);
        labelStartKm.setText(so.getName());
        labelStartKm.setToolTipText(so.getName());
      }
    }
  }

  private void aktionStopKm()
  {
    SystemObject so = DaVHelper.aktionListeAuswahlAq(_connection, _secondaryId);
    if (_connection != null && so != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
      Data cd = so.getConfigurationData(atg);
      double dv = cd.getItem("Wert").asUnscaledValue().doubleValue();
      if (dv > 0.0)
        dv /= 1000.0;

      if (so.getPid().equals("sba.aq.A8.6130.HFB.W"))
      {
        dv = 208.43;
      }

      String s = new Formatter().format("%1$7.3f", dv).toString();

      if (DaVHelper.validiereTextInput(_secondaryId, textStartKm.getText(), s))
      {
        textStopKm.setText(s);
        labelStopKm.setText(so.getName());
        labelStopKm.setToolTipText(so.getName());
      }
    }
  }

  public Text getTextStopKm()
  {
    return textStopKm;
  }

  public Text getTextStartKm()
  {
    return textStartKm;
  }

  public Map<String, List<ResultData>> getWanderBaustellen()
  {
    return _mapWanderBaustellen;
  }

  public Map<String, List<ResultData>> getProgrammHierarchie()
  {
    return _mapProgrammHierarchie;
  }

  public CompositeSperrungBereiche getCompositeParameterStufen()
  {
    return _compositeFahrstreifenSperrungen;
  }

  public void setCompositeParameterStufen(CompositeSperrungBereiche compositeParameterStufen)
  {
    _compositeFahrstreifenSperrungen = compositeParameterStufen;
  }

  public void aktionProgrammTypAendern()
  {
    SonderProgramm quelle = ueberpruefeErfassteDaten();
    if (quelle != null)
    {
      SonderProgrammKopierenDialog dialog = new SonderProgrammKopierenDialog(PlatformUI.getWorkbench()
          .getActiveWorkbenchWindow().getShell(), _secondaryId);
      
      SonderProgramm kopie = dialog.oeffne(quelle.kopie(), true);
      if (kopie != null)
      {
        AttributeGroup atg1 = _connection.getDataModel().getAttributeGroup("atg.sswSbaSonderprogramm");
        Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
        Aspect asp2 = _connection.getDataModel().getAspect("asp.parameterVorgabe");
        
        SystemObject quellObj = _connection.getDataModel().getObject(quelle.getPid());
        ResultData rd1 = _connection.getData(quellObj, new DataDescription(atg1, asp), 1000);        
        DynamicObject dynObj = (DynamicObject)_connection.getDataModel().getObject(kopie.getPid());         
        if (dynObj != null){
          Data data1 = rd1.getData().createModifiableCopy();
          data1.getItem("typBezeichnung").asTextValue().setText(kopie.getTypBezeichnung());
          List<ResultData> list = new ArrayList<ResultData>();
          ResultData resultData1 = new ResultData(dynObj, new DataDescription(atg1, asp2), new Date().getTime(), data1);
          list.add(resultData1); 
          UrlasserDialog urlasserDialog = new UrlasserDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
          boolean eingabeOk = (urlasserDialog.anzeigen(list)).getEingabeOk();          
        }                       
      }
    }
  }
  public void aktionSonderProgrammKopieren()
  {
    SonderProgramm quelle = ueberpruefeErfassteDaten();
    if (quelle != null)
    {
      SonderProgrammKopierenDialog dialog = new SonderProgrammKopierenDialog(PlatformUI.getWorkbench()
          .getActiveWorkbenchWindow().getShell(), _secondaryId);
       
      SonderProgramm kopie = dialog.oeffne(quelle.kopie(), false);
      
      //System.out.println("Kopie: " + kopie.getBezeichnung() + " -> " + kopie.getTypBezeichnung() + " : " + kopie.getGewaehlteSonderProgrammAktion().toString());
      if (kopie != null)
      {
        AttributeGroup atg1 = _connection.getDataModel().getAttributeGroup("atg.sswSbaSonderprogramm");
        AttributeGroup atg2 = _connection.getDataModel().getAttributeGroup("atg.sswSbaEinzelMassnahme");
        AttributeGroup atg3 = _connection.getDataModel().getAttributeGroup("atg.sswSbaUrsachenEinheit");
        Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
        Aspect asp2 = _connection.getDataModel().getAspect("asp.parameterVorgabe");
        
        SystemObject quellObj = _connection.getDataModel().getObject(quelle.getPid());
        ResultData rd1 = _connection.getData(quellObj, new DataDescription(atg1, asp), 1000);
        ResultData rd2 = _connection.getData(quellObj, new DataDescription(atg2, asp), 1000);
        ResultData rd3 = _connection.getData(quellObj, new DataDescription(atg3, asp), 1000);
        
        SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaUeSonderprogramm");
        
        String bezeichnung = kopie.getBezeichnung();
        String pidUe = erzeugePidUrsachenEinheitSonderProgramm(bezeichnung, _praefixPid, _postfixPid, _secondaryId);
        
        try
        { 
          DynamicObject dynObj = _verwaltungDynamischeObjekte.erzeugeObjekt((DynamicObjectType)type, bezeichnung, pidUe);
          if (dynObj != null){
            Data data1 = rd1.getData().createModifiableCopy();
            data1.getItem("nameAnwender").asTextValue().setText(bezeichnung);
            data1.getItem("typBezeichnung").asTextValue().setText(kopie.getTypBezeichnung());
            Data data2 = rd2.getData().createModifiableCopy();
            Data data3 = rd3.getData().createModifiableCopy();                 
            List<ResultData> list = new ArrayList<ResultData>();
            ResultData resultData1 = new ResultData(dynObj, new DataDescription(atg1, asp2), new Date().getTime(), data1);
            list.add(resultData1);
            ResultData resultData2 = new ResultData(dynObj, new DataDescription(atg2, asp2), new Date().getTime(), data2);
            list.add(resultData2);
            ResultData resultData3 = new ResultData(dynObj, new DataDescription(atg3, asp2), new Date().getTime(), data3);
            list.add(resultData3);
            UrlasserDialog urlasserDialog = new UrlasserDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
            Boolean schlieenOk = (urlasserDialog.anzeigen(list)).getEingabeOk();
            if (schlieenOk){
              IPreferenceStore store = Activator.getDefault().getPreferenceStore();
              String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
              ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(pidSba);
              MutableSet set = co.getMutableSet("UrsachenEinheiten");
              if (set != null){
                _verwaltungDynamischeObjekte.fuegeObjektInMengeEin(dynObj, set);
              }
            }else{
              _verwaltungDynamischeObjekte.loescheObjekt(dynObj);
            }
            
            
            //parametriereDynObjUrsachenEinheit(dynObj, rd1.getData(), rd2.getData(), rd3.getData());
          }else{
            MessageBox mb = new MessageBox(Display.getCurrent().getActiveShell(), SWT.ICON_ERROR | SWT.OK);
            mb.setText("Sonderprogramm Kopieren");
            mb.setMessage("Es is ein Fehler beim Erstellen des dynamischen Objektes aufgetreten!\n" 
                + "Der Vorgang wird abgebrochen.");
          }
          
        }
        catch (Exception e1)
        {
          // TODO Auto-generated catch block
          MessageBox mb = new MessageBox(Display.getCurrent().getActiveShell(), SWT.ICON_ERROR | SWT.OK);
          mb.setText("Sonderprogramm Kopieren");
          mb.setMessage("Es is ein Fehler beim Erstellen des dynamischen Objektes aufgetreten!\n" 
              + "Der Vorgang wird abgebrochen.");
        }
      }
    }
  }

  public void aktionLoeschen()
  {
    SonderProgramm sonderProgramm = ueberpruefeErfassteDaten();
    if (sonderProgramm != null)
    {
      versendeDaten(_connection, sonderProgramm, true, false);
    }
    else
    {
      if (_sonderProgrammDatensatz != null)
        versendeLoeschen();
    }
  }

  private void aktionKontextMenue(Object obj)
  {
    if (obj instanceof DoModelEditPart)
    {
      DoModelEditPart part = (DoModelEditPart)obj;
      DoModel model = (DoModel)part.getModel();
      DoTyp doTyp = model.getDoTyp();
      if (doTyp instanceof VerhaltenDoTyp)
      {
        VerhaltenDoTyp coDoTyp = (VerhaltenDoTyp)doTyp;

        int kontext = -1;

        if (model instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)model;
          kontext = aq.getKontext();
        }
        else if (model instanceof AnzeigeModelTypAbstract)
        {
          AnzeigeModelTypAbstract a = (AnzeigeModelTypAbstract)model;
          EObject parent = a.eContainer();
          if (parent != null)
          {
            if (parent instanceof AnzeigeQuerschnittModelTypAbstract)
            {
              AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parent;
              kontext = aq.getKontext();
            }
          }
        }

        String id = null;

        switch (kontext)
        {
          case 0:
            id = coDoTyp.getKontextMenueIdIstZustand();
            break;
          case 1:
            id = coDoTyp.getKontextMenueIdSollZustand();
            break;
          case 2:
            id = coDoTyp.getKontextMenueIdHandSchaltung();
            break;
          case 3:
            id = coDoTyp.getKontextMenueIdSimulation();
            break;
          case 4:
            id = coDoTyp.getKontextMenueIdTrichter();
            break;

          default:
            break;
        }
        if (id != null)
          leseActionSets(id);
      }
    }
  }
  
  /**
   * Umrechnungsmethode, die Abhngig von der Position in der Darstellung den dazugehrigen Autobhankilometer berechnet.
   * 
   * @param y
   *          Position in Pixel
   * @return Autobahnkilometer
   */
  public static Double berechneAutoBahnKmVonPixel(int y)
  {

    Double faktor = 29.0 / 1140.0;
    Double anfangKM = 393.0;
    Double endeKM = 422.0;

    return endeKM - faktor * y;
  }

  /**
   * Umrechnungsmethode, die Abhngig von der Kilometrierung in der Darstellung die Hhe in Pixel berechnet.
   * 
   * @param d
   *          Autobahnkilometer
   * @return Hhe in Pixel
   */
  public static Integer berechnePixelVonAutoBahnKm(Double d)
  {

    Double schritte = 1140.0 / 29.0;
    Double anfangKM = 393.0;
    Double endeKM = 422.0;

    Double wert = (endeKM - d) * schritte;

    return new Integer(wert.intValue());
  }
  
  
//  /**
//   * Methode fgt einen Listener hinzu
//   * @param l Listener
//   * @param funktion Funktion auf die der Listener reagiert
//   */
//  public void addPropertyChangeListener(PropertyChangeListener l)
//  {
//    _pcs.addPropertyChangeListener(l);
//    if (_secondaryId.equals("NO"))
//      benachrichtigeListener(PROP_SONDER_PROGRAMM_INIT_NO, null, this);
//    else if (_secondaryId.equals("SW"))
//      benachrichtigeListener(PROP_SONDER_PROGRAMM_INIT_SW, null, this);
//  }
//
//  /**
//   * Methode lscht einen Listener
//   * @param l Listener
//   * @param funktion Funktion auf die der Listener reagiert
//   */
//  public void removePropertyChangeListener(PropertyChangeListener l)
//  {
//    _pcs.removePropertyChangeListener(l);
//  }
//
//  /**
//   * Methode benachrichtigt die Listener ber die nderung einer Funktion
//   * @param funktion Funktion auf die der Listener reagiert
//   */
//  public void benachrichtigeListener(String prop, Object old, Object newValue)
//  {
//    _pcs.firePropertyChange(prop, old, newValue);
//  }
  
}
